Package: Epi
Version: 2.65
Date: 2026-04-20
Title: Statistical Analysis in Epidemiology
Authors@R: c(person("Bendix", "Carstensen", role = c("aut", "cre"),
                    email = "b@bxc.dk"),
             person("Martyn", "Plummer", role = "aut",
                    email = "Martyn.Plummer@warwick.ac.uk"),
             person("Esa", "Laara", role = "ctb"),
             person("Michael", "Hills", role = "ctb"))
Depends: R (>= 3.5.0), utils
Imports: cmprsk, etm, splines, MASS, survival, plyr, dplyr, Matrix,
        numDeriv, data.table, zoo, mgcv, magrittr
Suggests: mstate, nlme, lme4, demography, popEpi, tidyr
Description: Functions for demographic and epidemiological analysis in
  the Lexis diagram, i.e. register and cohort follow-up data. In
  particular representation, manipulation, rate estimation and
  simulation for multistate data - the Lexis suite of functions, which
  includes interfaces to 'mstate', 'etm' and 'cmprsk' packages.
  Contains functions for Age-Period-Cohort and Lee-Carter modeling and
  a function for interval censored data. Has functions for extracting
  and manipulating parameter estimates and predicted values (ci.lin
  and its cousins), as well as a number of epidemiological data sets.
License: GPL-2
URL: http://bendixcarstensen.com/Epi/
NeedsCompilation: yes
Packaged: 2026-04-14 14:32:55 UTC; BCAR0029
Author: Bendix Carstensen [aut, cre],
  Martyn Plummer [aut],
  Esa Laara [ctb],
  Michael Hills [ctb]
Maintainer: Bendix Carstensen <b@bxc.dk>
Repository: CRAN
Date/Publication: 2026-04-15 08:40:14 UTC
Built: R 4.4.3; x86_64-apple-darwin20; 2026-04-15 12:28:01 UTC; unix
Archs: Epi.so.dSYM

useDynLib(Epi, .registration=TRUE)
export(
  apc.frame,
  apc.fit,
  apc.plot,
  apc.lines,
 plot.apc,
lines.apc,
   pc.points,
   pc.lines,
   pc.matpoints,
   pc.matlines,
   pc.matshade,
   cp.points,
   cp.lines,
   cp.matpoints,
   cp.matlines,
   cp.matshade,
          cal.yr,
  as.Date.cal.yr,
  ccwc,
  ci.pd,
  ci.lin,
  ci.exp,
  ci.cum,
  ci.surv,
  ci.pred,
  ci.ratio,
  ci.mat,
  ci.Crisk,
  matshade,
  lls,
  clear,
  contr.orth,
  contr.2nd,
  contr.cum,
  contr.diff,
  detrend,
  decurve,
  dur,
  erl, surv1, surv2, erl1, yll,
  effx,
  effx.match,
  float,
  print.floated,
  gen.exp,
  mat2pol,
          Icens,
    print.Icens,
  summary.Icens,
  plotevent,
  fit.add,
  fit.mult,
  ftrend,
  fgrep,
  ngrep,
  lgrep,
  legendbox,
      plotCIF,
   stackedCIF,
 show.apc.LCa,
      apc.LCa,
          LCa.fit,
    print.LCa,
  summary.LCa,
  predict.LCa,
     plot.LCa,
  Lexis.diagram,
  Lexis.lines,
   Life.lines,
                Lexis,
              unLexis,
          merge.Lexis,
           plot.Lexis,
         points.Lexis,
          lines.Lexis,
         PY.ann.Lexis,
         subset.Lexis,
             "[.Lexis",
          cbind.Lexis,
          rbind.Lexis,
          order.Lexis,
           orderLexis,
            sortLexis,
          print.Lexis,
        summary.Lexis,
  print.summary.Lexis,
           splitLexis,
      transform.Lexis,
         levels.Lexis,
            nid.Lexis,
            bootLexis,
        Relevel.Lexis,
      factorize.Lexis,
             cutLexis,
            mcutLexis,
            rcutLexis,
           countLexis,
          stack.Lexis,
           tmat.Lexis,
          boxes.Lexis,
          boxes.matrix,
          boxes.MS,
         msdata.Lexis,
            etm.Lexis,
            crr.Lexis,
            glm.Lexis,
            gam.Lexis,
          coxph.Lexis,
            AaJ.Lexis,
             simLexis,
         addCov.Lexis,
        addDrug.Lexis,
         coarse.Lexis,
         subset.stacked.Lexis,
      transform.stacked.Lexis,
           plot.pState,
          lines.pState,
  Lexis2msm,
  entry,
  exit,
  status,
  timeBand,
  timeScales,
  timeSince,
  tsNA20,
  breaks,
  absorbing,
  transient,
  preceding,
  before,
  succeeding,
  after,
  tbox,
  dbox,
  fillarr,
  boxarr,
  boxes,
  factorize,
  rm.tr,
  PY.ann,
  N2Y,
  tmat,
  nState,
  pState,
  msdata,
        mh,
  print.mh,
  ncut,
  nice,
  NArray,
  ZArray,
  Ns,
  Termplot,
  pctab,
  plotEst,
  pointsEst,
  projection.ip,
  in.span,
  inSpan,
  id.span,
  idSpan,
  thinCol,
  linesEst,
  rateplot,
  Aplot,
  Pplot,
  Cplot,
  nid,
  Relevel,
  Relevel.factor,
  ROC,
  twoby2,
  Wald,
  stat.table,
  clogistic,
  poisreg,
  harm)
# Import generic methods
importFrom( utils, stack )
importFrom( splines, ns, bs )
importFrom( plyr, rbind.fill )
# importFrom( purrr, "%>%" )
importFrom( magrittr, "%>%" )
importFrom( dplyr, summarize, group_by, arrange, left_join, inner_join,
            select, rename, mutate, ungroup, filter, near)
importFrom( cmprsk, crr)
importFrom( etm, etm )
# importFrom( mstate, msdata )
importFrom( MASS, mvrnorm, ginv )
importFrom( survival, clogit, coxph, Surv, survfit )
importFrom( mgcv, gam )
importFrom( numDeriv, hessian )
importFrom( Matrix, nearPD )
importFrom( zoo, na.locf )
importFrom("grDevices", "gray", "rainbow", "adjustcolor", "dev.cur")
importFrom("graphics", "abline", "arrows", "axis", "box", "layout",
           "lines", "locator", "matlines", "matplot", "matpoints",
           "mtext", "par", "plot", "plot.new", "plot.window", "points",
           "polygon", "rect", "rug", "segments", "strheight",
           "strwidth", "text")
importFrom("stats", ".getXlevels", "AIC", "addmargins", "anova", "approx",
           "ave", "as.formula", "binomial", "coef", "complete.cases",
           "contr.sum", "dpois", "fisher.test", "fitted", "formula",
           "gaussian", "glm", "is.empty.model", "lm", "make.link",
           "median", "model.extract", "model.matrix", "model.offset",
           "model.response", "nlm", "pchisq", "pnorm", "poisson",
           "predict", "predict.lm", "qnorm", "qt", "quantile",
           "rpois", "runif", "termplot", "update", "vcov",
           "weighted.mean", "xtabs")
importFrom("utils", "flush.console", "str")

# register S3 methods
S3method(     print, Icens)
S3method(   summary, Icens)
S3method(     print, floated)
S3method(      plot, apc)
S3method(     lines, apc)
S3method(      plot, Lexis)
S3method(      plot, pState)
S3method(     lines, pState)
S3method(    points, Lexis)
S3method(     lines, Lexis)
S3method(    PY.ann, Lexis)
S3method(     merge, Lexis)
S3method(    subset, Lexis)
S3method(    subset, stacked.Lexis)
S3method(       "[", Lexis)
S3method(     cbind, Lexis)
S3method(     rbind, Lexis)
S3method(     print, Lexis)
S3method(   summary, Lexis)
S3method(     print, summary.Lexis)
S3method( transform, Lexis)
S3method( transform, stacked.Lexis)
S3method(    levels, Lexis)
S3method(       nid, Lexis)
S3method(       nid, default)
S3method(   Relevel, Lexis)
S3method(   Relevel, factor)
S3method(   Relevel, default)
S3method( factorize, Lexis)
S3method( factorize, default)
S3method(    addCov, Lexis)
S3method(   addDrug, Lexis)
S3method(     stack, Lexis)
S3method(      tmat, Lexis)
S3method(     boxes, Lexis)
S3method(     boxes, matrix)
S3method(     boxes, MS)
S3method(    msdata, Lexis)
S3method(       etm, Lexis)
S3method(       AaJ, Lexis)
S3method(     print, mh)
S3method(     print, LCa)
S3method(   summary, LCa)
S3method(   predict, LCa)
S3method(      plot, LCa)
S3method(     print, stat.table)
S3method(     print, clogistic)
S3method(      coef, clogistic)
S3method(      vcov, clogistic)
S3method(   as.Date, cal.yr)

summary.Lexis <-
function( object, simplify=TRUE, scale=1, by=NULL,
          Rates=FALSE, timeScales=FALSE, ... )
{
# If we have a by argument find out what to do
    if (!is.null(by)) {
        if (is.character(by)) {
            if (!all(by %in% names(object)))
                stop("Wrong 'by' argument: '", paste(by,collapse="','"),
                     "' - must be name(s) of variable(s) in the Lexis object")
            else res <- lapply(split(object, object[, by]), summary.Lexis,
                               by = NULL, simplify = simplify, scale = scale,
                               Rates = Rates, timeScales=timeScales, ...)
        }
        else {
            if (length(by) != nrow(object))
                stop("Wrong length of 'by' argument:", length(by),
                     "must be same length as rows of the Lexis object:", nrow(object) )
            else res <- lapply(split(object, by), summary.Lexis,
                               by = NULL, simplify = simplify, scale = scale,
                               Rates = Rates, timeScales=timeScales, ...)
        }
        # to avoid printing the time scale information repeatedly
        for( i in 1:(length(res)-1) ) res[[i]]$timeScales <- NULL
        return( res )
    }

# Table(s) of all transitions (no. records)
tr <- trans <- with( object, table(lex.Cst,lex.Xst) )

# Remove diagonal, i.e. records with no transition
for( i in intersect(rownames(trans),colnames(trans)) ) tr[i,i] <- 0

# Margins added
trans <- addmargins(trans)
tr    <- addmargins(tr)    # Sum omitting the diagonal
trm   <- tr[,ncol(tr)]

# Compute person-years in each Cst-state
pyrs  <- with( object,
               addmargins( tapply( lex.dur,lex.Cst,sum,na.rm=TRUE),
                                   FUN=function(x) sum(x,na.rm=TRUE) ) )/scale
pers  <- with( object,
               c( tapply( lex.id, lex.Cst, function(x) length(unique(x)) ),
                  length(unique(lex.id)) ) )

# Amend the table of records with column of events and person-years
trans <- cbind( trans, trm, pyrs, pers )

# Annotate the table nicely
colnames( trans )[ncol(trans)-2:0] <-
    c(" Events:","Risk time:"," Persons:" )
colnames( trans )[ncol(tr)] <- " Records:"
names( dimnames( trans ) ) <- c("From","\nTransitions:\n     To")

# Make the rates and annotate the table nicely
rates <- sweep( tr, 1, pyrs, "/" )
colnames( rates )[ncol(rates)] <- "Total"
names( dimnames( rates ) ) <-
     c("From",
       paste("\nRates",
             if( scale != 1 ) paste(" (per ",scale,")",sep=""),
             ":\n     To", sep="") )
if( simplify )
  {
  trans <- trans[!is.na(pyrs),]
  rates <- rates[!is.na(pyrs),]
  }
if( nrow(trans)==2 )
  trans <- trans[1,,drop = FALSE]
res <- list( Transitions = trans,
                   Rates = rates[-nrow(rates),,drop=FALSE],
              timeScales = timeSince(object) )

if( !timeScales ) res <- res[-3]
if( !Rates      ) res <- res[-2]
class( res ) <- "summary.Lexis"
res
}

print.summary.Lexis <-
function( x, ..., digits=2 )
{
print( round( x$Transitions, digits ) )
if( "Rates" %in% names(x) )
print( round( x$Rates      , digits ) )
# if( "timeScales" %in% names(x) )
if( !is.null(x$timeScales) )
  {  
cat("\nTimescales:\n")
print( x$timeScales )
  }
}

# The factorize method
factorize <- function (x, ...) UseMethod("factorize")

# Default method is just the Relevel method
factorize.default <- Relevel.default

# The Lexis version of this
  Relevel.Lexis <-
factorize.Lexis <-
function (x, states=NULL, print=TRUE, ... )
{
# Is this really a Lexis object
if( !inherits(x,"Lexis") ) stop( "First argument must be a Lexis object" )

# If lex.Cst and lex.Xst are not factors, make them:
if( !is.factor(x$lex.Cst) | !is.factor(x$lex.Xst) )
  {
Cst <- factor(x$lex.Cst)
Xst <- factor(x$lex.Xst)
  }
else # just use the factors as they are
  {
Cst <- x$lex.Cst
Xst <- x$lex.Xst
  }
# - but amend them to have the same set of levels
all.levels = union(levels(Cst), levels(Xst))
Cst <- factor(Cst, levels = all.levels)
Xst <- factor(Xst, levels = all.levels)

# A table of actually occurring levels and their names
tCX <- table(Cst) + table(Xst)
all.levels <- names( tCX[tCX>0] )

# If states are not given, just return factors with reduced levels
if( is.null(states) )
  {
x$lex.Cst <- factor( Cst, levels = all.levels )
x$lex.Xst <- factor( Xst, levels = all.levels )
  }

# If new state names are given as a list it implies merging of them
if( !is.null( states ) & is.list( states ) )
  {
  x$lex.Cst <- Relevel( Cst, states, ... )
  x$lex.Xst <- Relevel( Xst, states, ... )
  if( print )
    {
    # Construct translation table between old and grouped states to print
    tC <- table( Cst, x$lex.Cst )
    tX <- table( Xst, x$lex.Xst )
    cC <- matrix( colnames(tC), nrow(tC), ncol(tC), byrow=T )
    cX <- matrix( colnames(tX), nrow(tX), ncol(tX), byrow=T )
    cC[tC==0] <- ""
    cX[tX==0] <- ""
    print( data.frame( type=rep( c("lex.Cst","lex.Xst"),
                                 c(nrow(tC),nrow(tX)) ),
                       old=c(rownames(tC),rownames(tX)),
                       new=c( apply( cC, 1, paste, collapse="" ),
                              apply( cX, 1, paste, collapse="" ) ) ) )
    }
  }

# If states is a character vector we assume that it's just new names
if( !is.null( states ) & is.character( states ) )
  {
  if( length( states ) != nlevels(Cst) )
    stop( "Second argument is a vector of length ", length(states),
          ", but it should be the joint no. of states, ",
         length(all.levels),
          "\ncorresponding to ", all.levels )
  levels( Cst ) <- levels( Xst ) <- states
  x$lex.Cst <- Cst
  x$lex.Xst <- Xst
  if( print )
    {
    cat( "New levels for lex.Xst and lex.Cst generated:\n" )
    print( data.frame( old=all.levels, new=levels(x$lex.Cst) ) )
    }
  }

# If states is a numeric vector we assume that it's just reordering
if( !is.null( states ) & is.numeric( states ) )
  {
  x$lex.Cst <- Relevel( Cst, states )
  x$lex.Xst <- Relevel( Xst, states )
  }

return( x )
}

LexisScarf <-
function(nw = 4,   # fields around
         nl = 20,  # fields along
         nm = 23,  # no. stitches per field
         np = 30,  # no. persons
      scale = 1.5, # maximal FU in units of nw
        clr = c("red", "blue", "limegreen")
         )
{
# everything is measured in units of stitches
wd <- nm * nw
lg <- nm * nl + 1
mm <- matrix(0, wd, lg)
x <- col(mm)
y <- row(mm)
d <- mm

# here is the the grid
d[, 0:nl * nm + 1] <- 1
d[0:(nw - 1) * nm + 1, ] <- 1

# pdf("lexis-scarf.pdf", height = 10, width = 50)
# plot the grid
par(mar = rep(0, 4))
plot(as.vector(x),
     as.vector(y), pch = 16,
     col = gray((c(8, 4) / 10)[as.vector(d) + 1])
     )
# entry points
ew <- floor(runif(np) * wd)
el <- floor(runif(np) * lg)
# life lengths
ll <- floor(runif(np) * wd * scale)
cl <- clr[sample(1:length(clr), np, replace = TRUE)]
# function to generate life lines
LL <- function(xx, # x (length) at start
               yy, # y (width) at start
               ll) # length of FU
      {
      # folding the life lines
      tt <- list(y = ifelse((yy - 0:ll) <= 0,
                       wd + (yy - 0:ll),
                             yy - 0:ll),
                 x = xx + 0:ll)
      # cut the parts outside the scarf
      del <- tt$y < 0  |
             tt$y > wd |
             tt$x < 0  |
             tt$x > lg
      tt$x <- tt$x[!del]
      tt$y <- tt$y[!del]
      tt
      }
# points as life lines for the persons
for(i in 1:np)
    points(LL(el[i],
              ew[i],
              ll[i]),
        col = cl[i], pch = 16)
NULL
# dev.off()
}

apc.fit <-
function( data,
             A,
             P,
             D,
             Y,
         ref.c,
         ref.p,
           dist = c("poisson","binomial"),
          model = c("ns","bs","ls","factor"),
        dr.extr = "Y",
           parm = c("ACP","APC","AdCP","AdPC","Ad-P-C","Ad-C-P","AC-P","AP-C"),
           npar = c( A=5, P=5, C=5 ),
          scale = 1,
          alpha = 0.05,
      print.AOV = TRUE )
{
 dist <- match.arg(dist)
model <- match.arg(model)
drtyp <- deparse(substitute(dr.extr))
 parm <- toupper(match.arg(parm))

has.data <- !missing( data )
has.pref <- !missing( ref.p )
has.cref <- !missing( ref.c )

if(has.data)
  {
  if (length(match(c("A", "P", "D", "Y"), names(data))) != 4)
  stop("Data frame ", deparse(substitute(data)),
       " has columns:\n", names(data),
       "\nmust have variables:\n", "A (age), P (period), D (cases) and Y (person-time)")
  data <- data[,c("A","P","D","Y")]
  data <- data[complete.cases(data),]
  A <- data$A
  P <- data$P
  D <- data$D
  Y <- data$Y
  }
else
  {
  nm <- logical(4)
  nm[1] <- missing(A)
  nm[2] <- missing(P)
  nm[3] <- missing(D)
  nm[4] <- missing(Y)
  if (any(nm))
      stop("Variable", if (sum(nm) > 1)
          "s", paste(c(" A", " P", " D", " Y")[nm], collapse = ","),
          " missing from input")
  if( diff(range( lv <-
                  sapply( list(A = A, P = P, D = D, Y = Y),
                          length) ) ) != 0 )
      stop( "\nLengths of variables (", paste(paste(names(lv),
            lv, sep = ":"), collapse = ", "), ") are not the same." )
  }

# Utility to compute where the median amount y is on the x scale
med <-
function(x, y)
{
o <- order(x)
a <- y[o]
names(a) <- x[o]
return( as.numeric(names(a[cumsum(a)/sum(a) > 0.5][1])) )
}
# Set the reference points on the period and cohort scales
p0 <- ifelse( has.pref, ref.p, med(P  , D) )
c0 <- ifelse( has.cref, ref.c, med(P-A, D) )
 
# Number of parameters in the spline modeling 
if( is.list(npar) & length(npar)<3 )
  stop("npar given as a list - should have length 3! \n")
if( !is.list(npar) & length(npar)!=3 )
  {
  npar <- rep(npar, 3)[1:3]
  names(npar) = c("A","P","C")
  cat("NOTE: npar is specified as:\n") ; print( npar )
  }

if( is.null(names(npar)) ) names(npar) <- c("A", "P", "C")
# Labeling of confidence intervals
lu <- paste(formatC( c(alpha/2, 1 - alpha/2) * 100,
                     format = "f", digits = 1), "%", sep = "")

if( is.list(model) ) {
    if (!all(sapply(model, is.function)))
        stop("'model' is a list, but not all elements are functions as they should be.")
    if ((lmod <- length(model)) < 3)
        stop("'model' is a list, with", lmod, "elements, it should have three.")
    if (is.null(names(model)))
        names(model) <- c("A", "P", "C")
    MA <- model[["A"]](A)
    MP <- model[["P"]](P)
    MC <- model[["C"]](P-A)
    Rp <- model[["P"]](p0)
    Rc <- model[["C"]](c0)
}
else {
    if (model == "factor") {
        MA <- model.matrix(~factor(A) - 1)
        MP <- model.matrix(~factor(P) - 1)
        MC <- model.matrix(~factor(P - A) - 1)
        Rp <- MP[abs(P - p0) == min(abs(P - p0)), , drop = FALSE][1, ]
        Rc <- MC[abs(P - A - c0) == min(abs(P - A - c0)), , drop = FALSE][1, ]
    }
    if (model == "ns") {
        # is npar a list
        knl <- is.list( npar )
        # if scalar expand
        if( !knl & length(npar)==1 ) npar <- rep( npar, 3 )
        # if no names, provide them
        if( is.null(names(npar)) ) names(npar) <- c("A","P","C")
        # if names too long or wrong case, rectify
        names( npar ) <- toupper( substr(names(npar),1,1) )
        # if not a list make it one with the correct knots
        if( !knl ){
          nkn <- npar
          eqp <- function(n) (1:n-0.5)/n
          npar <- list( A = quantile( rep(  A,D), probs=eqp(nkn["A"]) ),
                        P = quantile( rep(P  ,D), probs=eqp(nkn["P"]) ),
                        C = quantile( rep(P-A,D), probs=eqp(nkn["C"]) ) )                       
          }
        MA <- Ns(  A, knots = npar[["A"]] )
        MP <- Ns(P  , knots = npar[["P"]] )
        MC <- Ns(P-A, knots = npar[["C"]] )
        Rp <- ns(p0, knots = attr(MP,"knots"),
            Boundary.knots = attr(MP,"Boundary.knots"))
        Rc <- ns(c0, knots = attr(MC,"knots"),
            Boundary.knots = attr(MC,"Boundary.knots"))
        Knots <- list( Age = sort(c(attr(MA,"knots"),
                                    attr(MA,"Boundary.knots"))),
                       Per = sort(c(attr(MP,"knots"),
                                    attr(MP,"Boundary.knots"))),
                       Coh = sort(c(attr(MC,"knots"),
                                    attr(MC,"Boundary.knots"))))
    }
    if (model %in% c("bs", "ls")) {
        deg <- switch(model, ls = 1, bs = 3)
        knl <- is.list(npar)
        if (knl) nk <- sapply(npar, length)
        MA <- if (knl) bs(A, knots = npar[["A"]][-c(1,nk[1])],
                    Boundary.knots = npar[["A"]][ c(1,nk[1])], degree = deg)
              else     bs(A, df = npar[["A"]], degree = deg)
        MP <- if (knl) bs(P, knots = npar[["P"]][-c(1,nk[2])],
                    Boundary.knots = npar[["P"]][ c(1,nk[2])], degree = deg)
              else     bs(P, df = npar[["P"]], degree = deg)
        MC <- if (knl) bs(P - A, knots = npar[["C"]][-c(1, nk[3])],
                        Boundary.knots = npar[["C"]][ c(1, nk[3])], degree = deg)
              else     bs(P - A, df = npar[["C"]], degree = deg)
        Rp <- bs(p0, knots = attr(MP,"knots"),
            Boundary.knots = attr(MP,"Boundary.knots"),
                    degree = attr(MP,"degree"))
        Rc <- bs(c0, knots = attr(MC,"knots"),
            Boundary.knots = attr(MC,"Boundary.knots"),
                    degree = attr(MC,"degree"))
        Knots <- list(Age = sort(c(attr(MA,"knots"),attr(MA,"Boundary.knots"))),
                      Per = sort(c(attr(MP,"knots"),attr(MP,"Boundary.knots"))),
                      Coh = sort(c(attr(MC,"knots"),attr(MC,"Boundary.knots"))))
    }
}
if (tolower(substr(dist, 1, 2)) == "po") {
    m.APC <- glm(D ~ MA + I(P - p0) + MP + MC,
                 offset = log(Y), family = poisson)
    Dist <- "Poisson with log(Y) offset"
}
is.bin <- FALSE
if (is.bin <- tolower(substr(dist, 1, 3)) %in% c("bin")) {
    m.APC <- glm(cbind(D, Y - D) ~ MA + I(P - p0) + MP + MC,
                 family = binomial)
    Dist <- "Binomial regression (logistic) of D/Y"
}
m.AP <- update(m.APC, . ~ . - MC)
m.AC <- update(m.APC, . ~ . - MP)
m.Ad <- update(m.AP , . ~ . - MP)
m.A  <- update(m.Ad , . ~ . - I(P - p0))
m.0  <- update(m.A  , . ~ . - MA)
AOV  <- anova(m.A, m.Ad, m.AC, m.APC, m.AP, m.Ad, test = "Chisq")
colnames(AOV)[1:4] <- c("Mod. df.","Mod. dev.",
                        "Test df.","Test dev.")
AOV <- abs(AOV)
AOV <- cbind(Model = c("Age",
                       "Age-drift",
                       "Age-Cohort",
                       "Age-Period-Cohort",
                       "Age-Period",
                       "Age-drift"),
               AIC = c(AIC(m.A),
                       AIC(m.Ad),
                       AIC(m.AC),
                       AIC(m.APC),
                       AIC(m.AP),
                       AIC(m.Ad)), 
               AOV,
     'Test dev/df' = AOV[,"Test dev."]/AOV[,"Test df."],
        'H0      ' = c("","zero drift ",  
                          "Coh eff|dr.",   
                          "Per eff|Coh",  
                          "Coh eff|Per",  
                          "Per eff|dr."))
A.pt  <- unique(A)
A.pos <- match(A.pt, A)
P.pt  <- unique(P)
P.pos <- match(P.pt, P)
C.pt  <- unique(P - A)
C.pos <- match(C.pt, P - A)
MA <- cbind(1, MA)

# Determine the inner product (diagonal) for projection
if (!mode(dr.extr) %in% c("character", "numeric"))
    stop("\"dr.extr\" must be of mode \"character\" or \"numeric\".\n")

if (is.character(dr.extr))
   { 
   wt <- rep(1, length(D) )
   drtyp <- "1-weights"
   if( toupper(substr(dr.extr, 1, 1)) %in% c("T","D") )
     { wt <- D
       drtyp <- "D-weights" } else
   if( toupper(substr(dr.extr, 1, 1)) %in% c("L","R") )
     { wt <- (Y^2)/D
       drtyp <- "Y^2/D-weights" } else
   if( toupper(substr(dr.extr, 1, 1)) %in% c("Y") )
     { wt <- Y
       drtyp <- "Y-weights" }
   }
if ( is.numeric(dr.extr) )
   {
   if( length(dr.extr)==1 )
     { wt <- D + dr.extr*Y
       drtyp <- paste("D+",dr.extr,"*Y weights",sep="") }
   if( length(dr.extr)==nrow(data) )
     { wt <- dr.extr
       if( any(wt<0) ) stop("dr.extr must be non-negative")
       drtyp <- "extn-weights" }
   }
Rp <- matrix(Rp, nrow = 1)
Rc <- matrix(Rc, nrow = 1)
xP <- Epi::detrend(rbind(Rp, MP), c(p0, P  ), weight = c(0, wt))
xC <- Epi::detrend(rbind(Rc, MC), c(c0, P-A), weight = c(0, wt))
 
MPr <- xP[-1,,drop=FALSE] - has.pref * xP[rep(1, nrow(MP)),,drop=FALSE]
MCr <- xC[-1,,drop=FALSE] - has.cref * xC[rep(1, nrow(MC)),,drop=FALSE]
if (length(grep("-", parm)) == 0) {
    if (parm %in% c("ADPC", "ADCP", "APC", "ACP"))
        m.APC <- update(m.0, . ~ . - 1 + MA + I(P - p0) + MPr + MCr)
    drift <- rbind( ci.exp(m.APC, subset = "I\\(", alpha = alpha),
                    ci.exp(m.Ad , subset = "I\\(", alpha = alpha) )
    rownames(drift) <- c(paste("APC (",drtyp,")",sep=""), "A-d")
    if (parm == "ADCP")
        m.APC <- update(m.0, . ~ . - 1 + MA + I(P - A - c0) + MPr + MCr)
    if (parm == "APC") {
        MPr <- cbind(P - p0, MPr)
        m.APC <- update(m.0, . ~ . - 1 + MA + MPr + MCr)
    }
    if (parm == "ACP") {
        MCr <- cbind(P - A - c0, MCr)
        m.APC <- update(m.0, . ~ . - 1 + MA + MPr + MCr)
    }
    Age <- cbind(Age = A.pt, ci.exp(m.APC, subset = "MA",
        ctr.mat = MA[A.pos,,drop=FALSE], alpha = alpha))[order(A.pt),]
    Per <- cbind(Per = P.pt, ci.exp(m.APC, subset = "MPr",
        ctr.mat = MPr[P.pos,,drop=FALSE], alpha = alpha))[order(P.pt),]
    Coh <- cbind(Coh = C.pt, ci.exp(m.APC, subset = "MCr",
        ctr.mat = MCr[C.pos,,drop=FALSE], alpha = alpha))[order(C.pt),]
    colnames(Age)[-1] <- c("Rate", lu)
    colnames(Per)[-1] <- c("P-RR", lu)
    colnames(Coh)[-1] <- c("C-RR", lu)
    Type <- paste("ML of APC-model", Dist, ": (", parm, "):\n")
    Model <- m.APC
}
else {
    adc <- update(m.0, . ~ . - 1 + MA + I(P - A - c0))
    adp <- update(m.0, . ~ . - 1 + MA + I(P - p0))
    drift <- ci.exp(adc, subset = "I\\(")
    rownames(drift) <- "A-d"
    xP <- cbind(1, P - p0, MPr)
    xC <- cbind(1, P - A - c0, MCr)
    lP <- cbind(P - p0, MPr)
    lC <- cbind(P - A - c0, MCr)
    if (parm == "AD-C-P") {
        rc <- update(m.0, . ~ . - 1 + xC, offset = predict(adc, type = "link"))
        rp <- update(m.0, . ~ . - 1 + xP, offset = predict(adc, type = "link"))
        A.eff <- ci.exp(adc, subset = "MA", ctr.mat = MA[A.pos,], alpha = alpha)
        C.eff <- ci.exp( rc, subset = "xC", ctr.mat = xC[C.pos,], alpha = alpha)
        P.eff <- ci.exp( rp, subset = "xP", ctr.mat = xP[P.pos,], alpha = alpha)
        Model <- list( adc, rc, rp )
    }
    else if (parm == "AD-P-C") {
        rp <- update(m.0, . ~ . - 1 + xP, offset = predict(adp,type = "link"))
        rc <- update(m.0, . ~ . - 1 + xC, offset = predict(rp,type = "link"))
        A.eff <- ci.exp(adp, subset = "MA", ctr.mat = MA[A.pos,], alpha = alpha)
        P.eff <- ci.exp(rp, subset = "xP", ctr.mat = xP[P.pos,], alpha = alpha)
        C.eff <- ci.exp(rc, subset = "xC", ctr.mat = xC[C.pos,], alpha = alpha)
        Model <- list( adp, rp, rc )
    }
    else if (parm == "AC-P") {
        ac <- update(m.0, . ~ . - 1 + MA + lC)
        rp <- update(m.0, . ~ . - 1 + xP, offset = predict(ac,type = "link"))
        A.eff <- ci.exp(ac, subset = "MA", ctr.mat = MA[A.pos,], alpha = alpha)
        C.eff <- ci.exp(ac, subset = "lC", ctr.mat = lC[C.pos,], alpha = alpha)
        P.eff <- ci.exp(rp, subset = "xP", ctr.mat = xP[P.pos,], alpha = alpha)
        Model <- list( ac, rp )
    }
    else if (parm == "AP-C") {
        ap <- update(m.0, . ~ . - 1 + MA + lP)
        rc <- update(m.0, . ~ . - 1 + xC, offset = predict(ap,type = "link"))
        A.eff <- ci.exp(ap, subset = "MA", ctr.mat = MA[A.pos,], alpha = alpha)
        P.eff <- ci.exp(ap, subset = "lP", ctr.mat = lP[P.pos,], alpha = alpha)
        C.eff <- ci.exp(rc, subset = "xC", ctr.mat = xC[C.pos,], alpha = alpha)
        Model <- list( ap, rc )
    }
    Age <- cbind(Age = A.pt, A.eff)[order(A.pt),]
    Per <- cbind(Per = P.pt, P.eff)[order(P.pt),]
    Coh <- cbind(Cph = C.pt, C.eff)[order(C.pt),]
    colnames(Age)[-1] <- c("A.eff", lu)
    colnames(Per)[-1] <- c("P.eff", lu)
    colnames(Coh)[-1] <- c("C.eff", lu)
    Type <- paste("Sequential modelling", Dist, ": (", parm, "):\n")
}
# If the model was binomial we convert to probabilities
o2p <- function(o) o/(1+o)
if( is.bin ) Age[,-1] <- o2p(Age[,-1]) 

res <- list(Type = Type,
           Model = Model,
             Age = Age,
             Per = Per,
             Coh = Coh,
           Drift = drift,
             Ref = c(Per = if ( parm %in%
                                c("APC","ADPC","Ad-P-C","AP-C") ) p0 else NA,
                     Coh = if ( parm %in%
                                c("ACP","ADCP","Ad-C-P","AC-P") ) c0 else NA ),
           Anova = AOV)
# If a spline model is used, add a "Knots" component to the apc-object
if (model %in% c("ns", "bs"))
    res <- c(res, list(Knots = Knots))
res$Age[, -1] <- res$Age[, -1] * scale
if (print.AOV) {
    print(res$Type)
    print(res$Anova)
}
# Print warnings about reference points:
if( !has.pref & parm %in% c("APC","ADPC") )
    cat( "No reference period given; ",
         "reference period for age-effects is chosen as\n",
         "the median date of event: ", p0, ".\n" )
if( !has.cref & parm %in% c("ACP","ADCP") )
    cat( "No reference cohort given; ",
         "reference cohort for age-effects is chosen as\n",
         "the median date of birth for persons  with event: ", c0, ".\n" )
class(res) <- "apc"
invisible(res)
}

# First the utility functions


######################################################################
lint <-
function( ci, tt, u )
{
# Makes a linear interpolation, but does not crash if all ci values are
# identical, but requires that both ci and tt are non-decreasing.
# ci plays the role of cumulative intensity, tt of time
if( any( diff(ci)<0 ) | any( diff(tt)<0 ) ) stop("Non-increasing arguments")
c.u <- min( c( ci[ci>u], max(ci) ) )
c.l <- max( c( ci[ci<u], min(ci) ) )
t.u <- min( c( tt[ci>u], max(tt) ) )
t.l <- max( c( tt[ci<u], min(tt) ) )
# c.u==c.l if u is outside the range of ci
ifelse( c.u==c.l, t.l, t.l + (u-c.l)/(c.u-c.l)*(t.u-t.l) )
}


######################################################################
sim1 <-
function( rt, time.pts )
{
# Simulates a single transition time and state based on the data frame
# rt with columns lex.id and timescales. It is assumed that the coumns
# in in rt are the id, followed by the set of estimated transition
# rates to the different states reachable from the current one.
ci <- apply( rbind(0,rt[,-1,drop=FALSE]), 2, cumsum )[1:nrow(rt),,drop=FALSE]
tt <- uu <- -log( runif(ncol(ci)) )
for( i in 1:ncol(ci) ) tt[i] <- lint( ci[,i], time.pts, uu[i] )
# Note this resulting data frame has 1 row and is NOT a Lexis object
data.frame( lex.id  = rt[1,1],
            lex.dur = min(tt,na.rm=TRUE),
            lex.Xst = factor( if( min(tt) < max(time.pts) )
                                  colnames(ci)[tt==min(tt)]
                              else NA ) )
}


######################################################################
simX <-
function( nd, Tr, time.pts, tS )
{
# Simulation is done from the data frame nd, in chunks of starting
# state, lex.Cst. This is necessary because different states have
# different (sets of) exit rates. Therefore, this function simulates
# for a set of persons from the same starting state.
np <- length( time.pts )
nr <- nrow( nd )
if( nr==0 ) return( NULL )

# The 'as.character' below is necessary because indexing by a factor
# by default is by the number of the level, and we are not indexing by
# this, but by components of Tr which just happens to have names that
# are a subset of the levels of lex.Cst.
cst <- as.character( unique(nd$lex.Cst) )
if( length(cst)>1 ) stop( "More than one lex.Cst present:\n", cst, "\n" )

# Expand each person by the time points
prfrm <- nd[rep(1:nr,each=np),]
prfrm[,tS] <- prfrm[,tS] + rep(time.pts,nr)
prfrm$lex.dur <- il <- min( diff(time.pts) )
# Poisson-models should use the estimated rate at the midpoint of the
# intervals, and have risk time equal to 1 in order to accommodate
# both poisson and poisreg families - they only produce identical
# predictions if lex.dur is 1 (i.e. offset is 0), scaling is after prediction
prfrp <- prfrm
prfrp[,"lex.dur"] <- 1
prfrp[,tS] <- prfrp[,tS]+il/2

# Make a data frame with predicted rates for each of the transitions
# out of this state for these times
rt <- data.frame( lex.id = prfrm$lex.id )
for( i in 1:length(Tr[[cst]]) )
   {
   if( inherits( Tr[[cst]][[i]], "glm" ) )
   rt <- cbind( rt, predict( Tr[[cst]][[i]],
                             type="response",
                             newdata=prfrp ) * il ) # scaled to interval
   else
   if( inherits( Tr[[cst]][[i]], "coxph" ) )
   rt <- cbind( rt, predict( Tr[[cst]][[i]],
                             type="expected",
                             newdata=prfrm ) )
   else
   if( is.function( Tr[[cst]][[i]] ) )
   rt <- cbind( rt, Tr[[cst]][[i]](prfrm) )
   else
   stop( "Invalid object supplied as transition, elements of the list must be either:\n",
         "- a glm(poisson) object fitted to a Lexis object\n",
         "- a coxph object fitted to a Lexis object\n",
         "- a function that takes a Lexis object as argument and returns\n",
         "  average rates for each record in the same units as lex.dur.")
   }
names( rt )[-1] <- names( Tr[[cst]] )

# Then find the transition time and exit state for each person:
xx <- match( c("lex.dur","lex.Xst"), names(nd) )
if( any(!is.na(xx)) ) nd <- nd[,-xx[!is.na(xx)]]
merge( nd,
       do.call( rbind,
                lapply( split( rt,
                               rt$lex.id ),
                        sim1,
                        time.pts ) ),
       by="lex.id" )
}


######################################################################
get.next <-
function( sf, tr.st, tS, tF )
{
# Produces an initial Lexis object for the next simulation for those
# who have ended up in a transient state.
# Note that this exploits the existence of the "time.since" attribute
# for Lexis objects and assumes that a character vector naming the
# transient states is supplied as argument.
if( nrow(sf)==0 ) return( sf )
nxt <- sf[sf$lex.Xst %in% tr.st,]
if( nrow(nxt) == 0 ) return( nxt )
nxt[,tS] <- nxt[,tS] + nxt$lex.dur
wh <- tF
for( i in 1:length(wh) )
   if( wh[i] != "" ) nxt[nxt$lex.Xst==wh[i],tS[i]] <- 0
nxt$lex.Cst <- nxt$lex.Xst
return( nxt )
}


######################################################################
chop.lex <-
function( obj, tS, cens )
{
# A function that chops off all follow-up beyond cens since entry for
# each individual
# Entry times on 1st timescale
zz <- entry( obj, 1, by.id=TRUE )
# Merge with the revised exit times on this timescale
ww <- merge( obj, data.frame( lex.id = as.numeric(names(zz)),
                                cens = zz+cens ) )
# Only retain records with an entry time prior to the revised exit time
ww <- ww[ww[,tS[1]] < ww$cens,]
# Revise the duration according the the revised exit time
x.dur <- pmin( ww$lex.dur, ww[,"cens"]-ww[,tS[1]] )
# Change lex.Xst to lex.Cst for those with shortened follow-up
ww$lex.Xst[x.dur<ww$lex.dur] <- ww$lex.Cst[x.dur<ww$lex.dur]
# Insert the updated follow-yp time
ww$lex.dur <- pmin( x.dur, ww$lex.dur )
ww
}


######################################################################
simLexis <-
function( Tr, # List of lists of transition objects
        init, # Lexis object of persons to simulate.
           N = 1, # No. persons simulated per line in init
      lex.id,
     t.range = 20, # Range for rate computation in the simulation
       n.int = 101, # length of time intervals
    time.pts = seq(0,t.range,length.out=n.int)
         )
{
# Expand the input data frame using N and put in lex.id
if( time.pts[1] !=0 )
    stop( "First time point must be 0, time.pts[1:3]= ",
          time.pts[1:3] )

# Expand init
if( !missing(N) )
  {
  if( length(N) == 1 )
       init <- init[rep(1:nrow(init),each=N),]
  else init <- init[rep(1:nrow(init),     N),]
  }
# and update lex.id if necessary
if( !missing(lex.id) )
  {
  if( length(lex.id)==nrow(init) )
       init$lex.id <- lex.id
  else init$lex.id <- 1:nrow(init)
  }
else   init$lex.id <- 1:nrow(init)

# Check/fix attributes
if( is.null( tS <- attr(init,"time.scales") ) )
  stop( "No time.scales attribute for init" )
if( is.null( tF <- attr(init,"time.since") ) )
  {
  attr(init,"time.since") <- tF <- rep( "", tS )
  warning( "'time.since' attribute set to blanks" )
  }

# Convenience constants
np <- length( time.pts )
tr.st <- names( Tr )

# Set up a NULL object to hold the follow-up records
sf <- NULL

# Take as initiators only those who start in a transient state
nxt <- init[init$lex.Cst %in% tr.st,]

# If some are not in a transient state then say so
if( nrow(nxt) < nrow(init) )
  {
  tt <- table(init$lex.Cst)
  tt <- tt[tt>0]
  nt <- length(tt)
  warning("\nSome initiators start in a absorbing state\n",
          "Initiator states represented are: ",
          paste( rbind( names(tt), rep(":",nt),
                        paste(tt), rep(" ",nt) ), collapse="" ), "\n",
          "Transient states are: ", paste( names( Tr ), coll=" " ) )
  if( nrow(nxt)==0 ) stop( "\nNo initiators in transient states!" )
  }

# Then we update those who are in a transient states and keep on doing
# that till all are in absorbing states or censored
while( nrow(nxt) > 0 )
{
nx <- do.call( rbind.data.frame,
               lapply( split( nxt,
                              nxt$lex.Cst ),
                       simX,
                       Tr, time.pts, tS ) )
sf <- rbind.data.frame( sf, nx )
nxt <- get.next( nx, tr.st, tS, tF )
}

# Doctor lex.Xst levels, fix values for the censored
sf$lex.Xst <- factor( sf$lex.Xst, levels=levels(sf$lex.Cst) )
sf$lex.Xst[is.na(sf$lex.Xst)] <- sf$lex.Cst[is.na(sf$lex.Xst)]

# Nicely order the output by persons, then times and states
nord <- match( c( "lex.id", tS,
                  "lex.dur",
                  "lex.Cst",
                  "lex.Xst" ), names(sf) )
noth <- setdiff( 1:ncol(sf), nord )
sf <- sf[order(sf$lex.id,sf[,tS[1]]),c(nord,noth)]
rownames(sf) <- NULL
# Finally, supply attributes - note we do not supply the "breaks"
# attribute as this is irrelevant for simulated objects
attr( sf, "time.scales" ) <- tS
attr( sf, "time.since"  ) <- tF
chop.lex( sf, tS, max(time.pts) )
}


######################################################################
nState <-
function ( obj,
            at,
          from,
     time.scale = 1 )
{
# Counts the number of persons in each state of the Lexis object 'obj'
# at the times 'at' from the time 'from' in the time scale
# 'time.scale'

# Determine timescales and absorbing and transient states
tS <- check.time.scale(obj,time.scale)
TT <- tmat(obj)
absorb <- rownames(TT)[apply(!is.na(TT),1,sum)==0]
transient <- setdiff( rownames(TT), absorb )

# Expand each record length(at) times
tab.frm <- obj[rep(1:nrow(obj),each=length(at)),
               c(tS,"lex.dur","lex.Cst","lex.Xst")]

# Stick in the corresponding times on the chosen time scale
tab.frm$when <- rep( at, nrow(obj) ) + from

# For transient states keep records that includes these points in time
tab.tr <- tab.frm[tab.frm[,tS]                 <= tab.frm$when &
                  tab.frm[,tS]+tab.frm$lex.dur >  tab.frm$when,]
tab.tr$State <- tab.tr$lex.Cst

# For absorbing states keep records where follow-up ended before
tab.ab <- tab.frm[tab.frm[,tS]+tab.frm$lex.dur <= tab.frm$when &
                  tab.frm$lex.Xst %in% absorb,]
tab.ab$State <- tab.ab$lex.Xst

# Make a table using the combination of those in transient and
# absorbing states.
with( rbind( tab.ab, tab.tr ), table( when, State ) )
}


######################################################################
pState <-
function( nSt, perm=1:ncol(nSt) )
{
# Compute cumulative proportions of persons across states in order
# designate by 'perm'
tt <- t( apply( nSt[,perm], 1, cumsum ) )
tt <- sweep( tt, 1, tt[,ncol(tt)], "/" )
class( tt ) <- c("pState","matrix")
tt
}


######################################################################
plot.pState <-
function( x,
        col = rainbow(ncol(x)),
     border = "transparent",
       xlab = "Time",
       ylim = 0:1,
       ylab = "Probability", ... )
{
# Function to plot cumulative probabilities along the time scale.
matplot( as.numeric(rownames(x)), x, type="n",
         ylim=ylim, yaxs="i", xaxs="i",
         xlab=xlab, ylab=ylab, ... )
lines.pState( x,
        col = col,
     border = border, ... )
}


######################################################################
lines.pState <-
function( x,
        col = rainbow(ncol(x)),
     border = "transparent", ... )
{
# Function to plot cumulative probabilities along the time scale.

# Fixing the colors:
nc <- ncol(x)
col    <- rep( col   , nc )[1:nc]
border <- rep( border, nc )[1:nc]

# Just for coding convenience when plotting polygons
pSt <- cbind( 0, x )
for( i in 2:ncol(pSt) )
   {
   polygon( c(    as.numeric(rownames(pSt)) ,
              rev(as.numeric(rownames(pSt))) ),
            c(    pSt[,i  ],
              rev(pSt[,i-1]) ),
            col=col[i-1], border=border[i-1], ... )
   }
}

# The paths method
paths <- function (Lx, ...) UseMethod("paths")

paths.default <-
paths.Lexis <-
function(Lx, dfr = FALSE, ...)
{
# sort the Lx object
Lx <- sortLexis(Lx)
# indicator of last record for each person
last <- !duplicated(Lx$lex.id, fromLast = TRUE)
# records with transitions
tran <- with(Lx, lex.Cst != lex.Xst)
# either of the two - not as Lexis
base <- unLexis(Lx[tran | last, c("lex.id", "lex.Cst","lex.Xst")])
# convert to character
for (i in 2:3) base[,i] <- as.character(base[,i])
# split by persons
lbas <- split(base, base$lex.id)
# construct character vector of visited states, weed out duplicates
visits <- function(x) # visited states for one person
          {
          zz <- c(x$lex.Cst[1], x$lex.Xst)
          zz[c(TRUE, zz[-1] != zz[-length(zz)])]
          }
# create list of character vectors
paths <- lapply(lbas, visits)
# paste them together
ff <- factor(sapply(paths, paste, collapse = "->"))

# if required, render as data frame else just return factor ff
if (!dfr) return(ff)
   else {
        df <- data.frame(lex.id = names(ff), path = ff)
        if (is.numeric(Lx$lex.id)) df$lex.id <- as.numeric(df$lex.id)
        return(df)
        }
}

NArray <-
function( x, cells=NA )
{
if( !is.list(x) ) stop("Argument must be a (named) list." )
array( cells, dimnames=x, dim=sapply( x, length ) )
}

ZArray <- function( x, cells=0 ) NArray( x, cells=cells )

larray <-
function( data=NA, dim, dimnames )
{
if( is.list(data) )
  return( array(data=NA,dim=sapply(data,length),
                        dimnames=data) )
else
if( is.list(dim) )
  return( array(data=data,dim=sapply(dim,length),
                          dimnames=dim) )
else
if( is.list(dimnames) )
  return( array(data=data,dim=sapply(dimnames,length),
                          dimnames=dimnames) )
else
if( !missing(dimnames) )
  return( array(data=data,dim=length(data),
                          dimnames=dimnames) )
else
  return( array(data=data,dim=length(data)) )
}

summary.Lexis <-
function(object,
       simplify = TRUE,
          scale = 1,
             by = NULL, # mat = TRUE,
          Rates = FALSE,
     timeScales = FALSE, ... )
{
    # If we have a by argument find out what to do
    if (!is.null(by)) {
        if (is.character(by)) {
            if (!all(by %in% names(object)))
                stop("Wrong 'by' argument: '", paste(by,collapse="','"),
                     "' - must be name(s) of variable(s) in the Lexis object")
            else res <- lapply(split(object, object[, by]),
                               summary.Lexis,
                               by = NULL, # mat = FALSE,
                               simplify = simplify, scale = scale,
                               Rates = Rates, timeScales = timeScales, ...)
            } else {
            if (length(by) != nrow(object))
                stop("Wrong length of 'by' argument:", length(by),
                     "must be same length as rows of the Lexis object:", nrow(object) )
            else res <- lapply(split(object, by),
                               summary.Lexis,
                               by = NULL, # mat = FALSE,
                               simplify = simplify, scale = scale,
                               Rates = Rates, timeScales = timeScales, ...)
        }
        # to avoid printing the time scale information repeatedly
        for( i in 1:(length(res)-1) ) res[[i]]$timeScales <- NULL
        return( res )
    }

# Table(s) of all transitions (no. records)
tr <- trans <- with( object, table(lex.Cst,lex.Xst) )

# Remove diagonal, i.e. records with no transition
for( i in intersect(rownames(trans), colnames(trans)) ) tr[i,i] <- 0

# Margins added
trans <- addmargins(trans)
tr    <- addmargins(tr)    # Sum omitting the diagonal
trm   <- tr[,ncol(tr)]

# Compute person-years in each Cst-state
pyrs  <- with(object,
              addmargins(tapply(lex.dur, lex.Cst, sum, na.rm = TRUE),
                         FUN = function(x) sum(x, na.rm=TRUE)))/scale
# Enumarate persons in each Cst-state
pers  <- with(object,
              c(tapply(lex.id, lex.Cst, function(x) length(unique(x))),
                length(unique(lex.id))))

# Amend the table of records with columns of events, person-years and persons
trans <- cbind(trans, trm, pyrs, pers)

# Annotate the table nicely
colnames(trans)[ncol(trans) - 2:0] <- c("Events:","Risk time:","Persons:" )
colnames(trans)[ncol(tr)] <- "Records:"
names(dimnames(trans)) <- c("From", "\nTransitions:\n     To")

# Make the rates and annotate the table nicely
rates <- sweep( tr, 1, pyrs, "/" )
colnames( rates )[ncol(rates)] <- "Total"
names( dimnames( rates ) ) <-
     c("From",
       paste("\nRates",
             if( scale != 1 ) paste0(" (per ",scale,")"),
             ":\n     To", sep="") )
if( simplify )
  {
  trans <- trans[!is.na(pyrs), ]
  rates <- rates[!is.na(pyrs), ]
  }
if( nrow(trans) == 2 )
  trans <- trans[1, , drop = FALSE]
res <- list(Transitions = trans,
                  Rates = rates[-nrow(rates),,drop=FALSE],
             timeScales = timeSince(object) )

if (!timeScales) res <- res[-3]
if (!Rates     ) res <- res[-2]
# return as a matrix if feasible
# if (length(res) == 1 & mat) res <- res[[1]]
class( res ) <- "summary.Lexis"
res
}

print.summary.Lexis <-
function( x, ..., digits=2 )
{
if (!is.list(x)) {
   print( round(unclass(x), digits) )
                 } else {
print( round( x$Transitions, digits ) )
if( "Rates" %in% names(x) )
print( round( x$Rates      , digits ) )
# if( "timeScales" %in% names(x) )
if( !is.null(x$timeScales) )
  {
cat("\nTimescales:\n")
print( x$timeScales )
  }                     }
}

Cplot <-
function( rates,
          age = as.numeric( rownames( rates ) ),
          per = as.numeric( colnames( rates ) ),
         grid = FALSE,
       c.grid = grid,
        ygrid = grid,
     col.grid = gray( 0.9 ),
        c.lim = NULL,
         ylim = range( rates[rates>0], na.rm=TRUE ),
           at = NULL,
       labels = paste( at ),
        c.lab = names( dimnames( rates ) )[2],
         ylab = deparse( substitute( rates ) ),
         type = "l",
          lwd = 2,
          lty = 1,
          col = par( "fg" ),
       log.ax = "y",
          las = 1,
        xannx = 1/20,
          ann = FALSE,
      cex.ann = 0.8,
       a.thin = seq( 1, length( age ), 2 ),
          ...
          )
{
# Convert the age-period table to an age-cohort table
rt <- as.table( rates )
dimnames( rt ) <- list( age = age, per = per )
rtf <- data.frame( rt )
rtf$age <- as.numeric( as.character( rtf$age ) )
rtf$per <- as.numeric( as.character( rtf$per ) )

# Check that age and period classe have equal lengths:
wa <- diff( ag <- sort( unique(rtf$age) ) )
wp <- diff( pg <- sort( unique(rtf$per) ) )
if( unique(wa) != unique(wp) )
  stop("Age and period intervals must have same width:\n",
       "Ages:", ag, "\n", "Periods:", pg, "\n",
       "No plot with cohort produced.\n" )

# Table by age and cohort
ac <- tapply( rtf$Freq, list( rtf$age, rtf$per-rtf$age ), mean )
coh <- as.numeric( colnames( ac ) )
if( is.null( c.lim ) )
  c.lim <- range( coh, na.rm=TRUE ) + c(0,diff( range( coh ) )/30) * ann

# Plot the frame
if( ann ) c.lim <- c.lim - c(diff( range( coh ) ) * xannx,0)
matplot( coh, t(ac), type="n",
         xlim=c.lim, ylim=ylim, xlab=c.lab, ylab=ylab,
         log=log.ax, las=las, yaxt=if( !is.null( at ) ) "n" else "s" )
if( !is.null( at ) ) axis( side=2, at=at, labels=labels, yaxt="s", las=las ) 
# and the grid if required
if( !missing( c.grid ) | !missing( grid ) )
  {
  if( is.logical( c.grid ) & c.grid[1] ) c.grid <- nice( coh, log=par("xlog") )
  abline( v=c.grid, col=col.grid )
  }
if( !missing( ygrid ) | !missing( grid ) )
  {
  if( is.logical( ygrid ) & ygrid[1] ) ygrid <- nice( rates[!is.na(rates)], log=par("ylog") )
  abline( h=ygrid, col=col.grid )
  }
box()
# then the curves
matlines( coh, t(ac), lwd=lwd, lty=lty, col=col, type=type, ... )
# annotate them if required (every second by default )
if( ann )
  {
   nr <- nrow( ac )
   nc <- ncol( ac )
   # Find the cohorts for the last rates in each age-class
   c.end <- rev( per )[1] - age
   text( c.end[a.thin], rates[,ncol(rates)][a.thin],
         paste( "", age[a.thin] ), adj=c(0,0.5),
         cex=cex.ann, col=if( length(col)==1 ) col else col[a.thin] ) 
  }
}

ci.cum <-
function(obj,
     ctr.mat = NULL,
      subset = NULL,
        intl = NULL, 
       alpha = 0.05,
         Exp = TRUE,
      ci.Exp = FALSE,
      sample = FALSE)
{
qzwpr <- NULL
# First extract all the coefficients and the variance-covariance matrix
cf  <- COEF(obj)
vcv <- VCOV(obj)

# use first column of ctr.mat to derive 
if (is.null(intl))
   {
   cnam <- deparse(substitute(ctr.mat))
   # if called from ci.surv the nane is in qzwpr
   if (exists("qzwpr")) cnam <- qzwpr
   tnam <- colnames(ctr.mat)[1]
   if (is.null(tnam)) tnam <- paste0(cnam, "[,1]")
   intl <- diff(ctr.mat[,1])[1]
   cat("NOTE: interval length chosen from ", cnam, " as ",
       tnam, "[2] - ", tnam, "[1]", "\n", sep = "")
   }
# Check if the intervals matches ctr.mat
if( length( intl ) == 1 ) intl <- rep( intl, nrow( ctr.mat ) )
if( length( intl ) != nrow( ctr.mat ) ) stop( "intl must match ctr.mat" )

if( inherits( ctr.mat, "data.frame" ) )
  {
  ctr.mat <- df2ctr( obj, ctr.mat )
  } else
  {
if(is.character(subset)) {
  sb <- numeric(0)
  for(i in 1:length(subset)) sb <- c(sb, grep(subset[i],
                                              names(cf)))
  subset <- sb # unique( sb )
  }
# If subset is not given, make it the entire set
if( is.null( subset ) ) subset <- 1:length( cf )
# Exclude units where aliasing has produced NAs.
# Not needed after replacement with 0s
# subset <- subset[!is.na( cf[subset] )]
 cf <-  cf[subset]
vcv <- vcv[subset,subset]
if( is.null( ctr.mat ) )
  {
  ctr.mat <- diag( length( cf ) )
  rownames( ctr.mat ) <- names( cf )
  }
if( dim( ctr.mat )[2] != length(cf) )
    stop( paste("\n Dimension of ", deparse(substitute(ctr.mat)),
          ": ", paste(dim(ctr.mat), collapse = "x"),
          ", not compatible with no of parameters in ",
          deparse(substitute(obj)), ": ", length(cf), sep = ""))
}
    
# Finally, here is the actual computation of the estimates
    ct <- ctr.mat %*% cf
    vc <- ctr.mat %*% vcv %*% t( ctr.mat )
# If a sample is requested replace the estimate by a sample
    if( sample ) ct <- t( mvrnorm( sample, ct, vc ) )
# If Exp was requested, we take the exponential of the estimates
# before we cumulate the sum
if( Exp )
  {
  ct <- exp( ct )
  vc <- ( ct[,1] %*% t(ct[,1]) ) * vc
  }
# Here is the cumulation matrix
  cum.mat <- 1 - upper.tri( diag(ct[,1]) )
  # Multiply columns of the matrix with interval lengths
  cum.mat <- t( intl * t( cum.mat ) )
# This is then multiplied to the coefficients
    ct <- cum.mat %*% ct
    if( sample ) return( ct )
    else
      {
      vc <- cum.mat %*% vc %*% t( cum.mat )
      se <- sqrt( diag( vc ) )
      if( !ci.Exp )
        {
        cum <- cbind( ct, se ) %*% ci.mat( alpha=alpha )
        return( cbind( cum, "StdErr"=se ) )
        }       
      else
        {  
        cum <- exp( cbind( log(ct), se/ct ) %*% ci.mat( alpha=alpha ) )
        return( cbind( cum, "Erf"=exp( qnorm(1-alpha/2)*se/as.vector(ct) ) ) )
        } 
      }
}

ci.surv <-
function( obj,
      ctr.mat = NULL,
       subset = NULL,
         intl = NULL,
        alpha = 0.05,
          Exp = TRUE,
       sample = FALSE )
{
qzwpr <- NULL
# carry the name across to ci.cum
if (is.null(intl)) qzwpr <<- deparse(substitute(ctr.mat))
CH <- ci.cum( obj,
      ctr.mat = ctr.mat,
       subset = subset,
         intl = intl,
        alpha = alpha,
          Exp = Exp,
       ci.Exp = TRUE,
       sample = sample )
exp(-rbind(0, CH[1:(nrow(CH) - 1), -4]))        
}

contr.orth <-
  function( n )
  {
  if( is.numeric( n ) && length( n )==1 )
  levs <- 1:n
  else {
  levs <- n
  n <- length( n )
  }
  Z <- contr.sum( n )
  L <- 1:n - mean(1:n)
  contr <- Z - L%*%( ( t(L) %*% L )^(-1) ) %*% ( t(L) %*% Z )
  contr[,1:(n-2)]
  }

ci.pd <-
  function( aa, bb=NULL, cc=NULL, dd=NULL,
            method = "Nc",
             alpha = 0.05,
        conf.level = 0.95,
            digits = 3,
             print = TRUE,
       detail.labs = FALSE )
  {
# Computes the approximate c.i. for the probability difference
# Optional methods:
# -- "AC", Agresti and Caffo, Am Statistician (2000),
# -- "Nc", method 10 from Newcombe, Stat.Med. 17, (1998), pp.873 ff.

if ( !(method %in% c("AC", "Nc") ) )
   stop( paste('Method', method, 'unsupported: Only "Nc" and "AC" supported') )

# Fix the confidence level
if( missing( alpha ) )           alpha <- 1 - conf.level
if( missing( conf.level ) ) conf.level <- 1 - alpha

# Allow various forms of vector and matrix input
  prefix <- ""
  if( is.vector( aa ) & length( aa ) > 1 ) prefix <- names( aa )
  if( length( dim( aa ) ) == 2 )
    {
    bb <- aa[1,2]
    cc <- aa[2,1]
    dd <- aa[2,2]
    aa <- aa[1,1]
    }
  if( length( dim( aa ) ) == 3 )
    {
    prefix <- paste( if( is.null( dimnames( aa ) ) ) 1:dim(aa)[3]
                     else dimnames( aa )[[3]], ": ", sep="" )
    bb <- aa[1,2,]
    cc <- aa[2,1,]
    dd <- aa[2,2,]
    aa <- aa[1,1,]
    }
  if( length( dim( aa ) ) > 3 )
    stop( "Maximal array dimension (3) exceeded!" )

# Function to give roots in a 2nd degree polynomial
# (Polyroot does not work on vectors of coefficients)
  pol2 <-
  function( Aye, Bee, Sea )
  {
  Dee <- Bee^2 - 4 * Aye * Sea
  lo <- ifelse( Dee >= 0, ( -Bee - sqrt( Dee ) ) / ( 2 * Aye ), NA )
  hi <- ifelse( Dee >= 0, ( -Bee + sqrt( Dee ) ) / ( 2 * Aye ), NA )
  cbind( lo, hi )
  }
# Put the data in the right form
  x1 <- aa
  n1 <- aa+cc
  p1 <- x1/n1
  x2 <- bb
  n2 <- bb+dd
  p2 <- x2/n2
  pd <- x1/n1 - x2/n2
  z <- qnorm( 1-alpha/2 )
  zz <- z^2
  if ( method == "AC" )
     { x1.1 <- x1+1
       n1.2 <- n1+2
       x2.1 <- x2+1
       n2.2 <- n2+2
       p1.1 <- x1.1/n1.2
       p2.1 <- x2.1/n2.2
       pd.1 <- p1.1 - p2.1
       SE.4 <- sqrt( p1.1 * ( 1-p1.1) /n1.2 +  p2.1 * ( 1-p2.1) /n2.2 )
       res <- cbind( n1, p1, n2, p2, pd, pd.1 - z*SE.4, pd.1 + z*SE.4 )
      }
  else
  if ( method == "Nc" )
     { A1 <-        1 + zz / n1
       B1 <- -2*x1/n1 - zz / n1
       C1 <-          ( x1 / n1 )^2
       r1 <- pol2( A1, B1, C1 )
       A2 <-        1 + zz / n2
       B2 <- -2*x2/n2 - zz / n2
       C2 <-          ( x2 / n2 )^2
       r2 <- pol2( A2, B2, C2 )
       dlt <- sqrt( ( x1/n1 - r1[,1] )^2 +
                    ( x2/n2 - r2[,2] )^2 )
       eps <- sqrt( ( x1/n1 - r1[,2] )^2 +
                    ( x2/n2 - r2[,1] )^2 )
       res <- cbind(n1, p1, n2, p2, pd, pd-dlt, pd+eps )
      }
  colnames( res ) <- c("n1","p1","n2","p2",
                       "diff",paste(   alpha/2 *100,"%",sep=""),
                              paste((1-alpha/2)*100,"%",sep="") )
  rownames( res ) <- prefix
  if( detail.labs ) rownames( res ) <-
                     paste( prefix, ": ",
                            aa, "/(", aa, "+", cc, ") - ",
                            bb, "/(", bb, "+", dd, ")", sep="" )
  if( print ) print( round( res, digits ) )
  invisible( res )
  }

steplines <-
function( x,
          y,
       left = TRUE,
      right = !left,
      order = TRUE,
        ... )
{
# A function to plot step-functions
#
  # Get the logic right if right is supplied...
  left <- !right   # ... right!
  n    <- length( x )
  if( any( order ) ) ord <- order(x) else ord <- 1:n
  dbl <- rep( 1:n, rep( 2, n) )
  xv  <- c( !left, rep( T, 2*(n-1) ),  left)
  yv  <- c(  left, rep( T, 2*(n-1) ), !left)
  lines( x[ord[dbl[xv]]],
         y[ord[dbl[yv]]], ... )
}

interp <-
function ( target, fv, res )
{
# Linear interpolaton of the values in the N by 2 matrix res,
# to the target value target on the N-vector fv.
# Used for placing tickmarks on the ROC-curves.
#
where <- which( fv>target )[1] - 1:0
  int <- fv[where]
   wt <- ( int[2] - target ) / diff( int )
wt[2] <- 1-wt
t( res[where,] ) %*% wt
}

ROC.tic <-
function ( tt,
          txt = formatC(tt,digits=2,format="f"),
         dist = 0.02,
        angle = +135,
          col = "black",
          cex = 1.0,
           fv,
          res )
{
# Function for drawing tickmarks on a ROC-curve
#
for (i in 1:length(tt))
    {
    pnt <- interp ( tt[i], fv, res )
    x <- 1-pnt[2]
    y <- pnt[1]
    lines( c( x, x+dist*cos(pi*angle/180) ),
           c( y, y+dist*sin(pi*angle/180) ), col=col )
    text( x+dist*cos(pi*angle/180),
          y+dist*sin(pi*angle/180), txt[i], col=col,
          adj=c( as.numeric(abs(angle)>=90),
                 as.numeric(    angle <= 0)), cex=cex )
    }
}

ROC <-
function ( test = NULL,
           stat = NULL,
           form = NULL,
           plot = c( "sp", "ROC" ),
             PS = is.null(test),      # Curves on probability scale
             PV = TRUE,               # sn, sp, PV printed at "optimality" point
             MX = TRUE,               # tick at "optimality" point
             MI = TRUE,               # Model fit printed
            AUC = TRUE,               # Area under the curve printed
           grid = seq(0,100,10),      # Background grid (%)
       col.grid = gray( 0.9 ),
           cuts = NULL,
            lwd = 2,
           data = parent.frame(),
            ... )
{
# First all the computations
#
# Name of the response
  rnam <- if ( !missing( test ) )
             deparse( substitute( test ) ) else
             "lr.eta"
# Fit the model and get the info for the two possible types of input
  if( is.null( form ) )
    {
    if( is.null( stat ) | is.null( test ) )
      stop( "Either 'test' AND 'stat' OR 'formula' must be supplied!" )
    lr <- glm( stat ~ test, family=binomial )#, data=data )
    resp <- stat
    Model.inf <- paste("Model: ", deparse( substitute( stat ) ), "~",
                                  deparse( substitute( test ) ) )
    }
  else
    {
    lr <- glm(form, family = binomial, data = data)
    resp <- eval( parse(text = deparse(form[[2]])), envir=lr$model )
    Model.inf <- paste("Model: ",paste(paste(form)[c(2,1,3)], collapse=" "))
    }
# From the empirical distribution function for test for each of
# the two categories of resp.

# First a table of the test (continuous variable) vs. the response and
# adding a row of 0s so that we have all points for the ROC curve
  m  <- as.matrix( base::table( switch( PS+1, test, lr$fit ), resp ) )
  m  <- addmargins( rbind( 0, m ), 2 )
# What values of test/eta do the rows refer to
  fv <- c( -Inf, sort( unique( switch( PS+1, test, lr$fit ) ) ) )
# How many rows in this matrix
  nr <- nrow(m)
# Calculate the empirical distribution functions (well, cumulative numbers):
  m <- apply( m, 2, cumsum )
# Then the relevant measures are computed.
  sns <- (m[nr,2]-m[,2]) /   m[nr,2]
  spc <-          m[,1]  /   m[nr,1]
  pvp <-          m[,2]  /           m[,3]
  pvn <- (m[nr,1]-m[,1]) / ( m[nr,3]-m[,3] )
  res <- data.frame( cbind( sns, spc, pvp, pvn, fv ) )
  names( res ) <- c( "sens", "spec", "pvp", "pvn", rnam )
  # AUC by triangulation
  auc <- sum( (res[-1,"sens"]+res[-nr,"sens"])/2 * abs(diff(1-res[,"spec"])) )

# Plot of sens, spec, PV+, PV-:
if ( any( !is.na( match( c( "SP", "SNSP", "SPV" ), toupper( plot ) ) ) ) )
{
# First for probability scale
if ( PS ) {
       plot( 0:1, 0:1,
             xlim=0:1, xlab="Cutpoint for predicted probability",
             ylim=0:1, ylab=" ",
             type="n" )
       if( is.numeric( grid ) ) abline( h=grid/100, v=grid/100, col=col.grid )
       box()
       for ( j in 4:1 ){
       steplines( fv, res[,j], lty=1, lwd=lwd, col=gray((j+1)/7)) }
       text( 0, 1.01, "Sensitivity", cex=0.7, adj=c(0,0), font=2 )
       text( 1, 1.01, "Specificity", cex=0.7, adj=c(1,0), font=2 )
       text( 0,  m[nr,2]/m[nr,3]-0.01, "PV+", cex=0.7, adj=c(0,1), font=2 )
       text( 0 + strwidth( "PV+", cex=0.7 ),  m[nr,2]/m[nr,3]-0.01,
             paste( " (= ", m[nr,2],"/", m[nr,3], " =",
                    formatC( 100*m[nr,2]/m[nr,3], digits=3 ),
                    "%)", sep=""),
             adj=c(0,1), cex=0.7 )
       text( 1, 1-m[nr,2]/m[nr,3]-0.01, "PV-", cex=0.7, adj=c(1,1), font=2 )
            }
# then for test-variable scale
else {
       xl <- range( test )
       plot( xl, 0:1,
             xlim=xl,
             xlab=paste( deparse( substitute( test ) ), "(quantiles)" ),
             ylim=0:1,         ylab=" ",
             type="n" )
       if( is.numeric( grid ) )
         abline( h=grid/100, v=quantile( test, grid/100 ), col=col.grid )
       box()
       for ( j in 4:1 ){
       steplines( fv, res[,j], lty=1, lwd=lwd, col=gray((j+1)/7))}
       text( xl[1], 1.01, "Sensitivity", cex=0.7, adj=c(0,0), font=2 )
       text( xl[2], 1.01, "Specificity", cex=0.7, adj=c(1,0), font=2 )
       text( xl[1],  m[nr,2]/m[nr,3]-0.01, "PV+", cex=0.7, adj=c(0,1), font=2 )
       text( xl[1] + strwidth( "PV+", cex=0.7 ),  m[nr,2]/m[nr,3]-0.01,
             paste( " (= ", m[nr,2],"/", m[nr,3], " =",
                    formatC( 100*m[nr,2]/m[nr,3], digits=3 ),
                    "%)", sep=""),
             adj=c(0,1), cex=0.7 )
       text( xl[2], 1-m[nr,2]/m[nr,3]-0.01, "PV-", cex=0.7, adj=c(1,1), font=2 )
       }
}

# Plot of ROC-curve:
if ( any( !is.na( match( "ROC", toupper( plot ) ) ) ) )
{
       plot( 1-res[,2], res[,1],
             xlim=0:1, xlab="1-Specificity",
             ylim=0:1, ylab=  "Sensitivity",
             type="n", ...)
       if( is.numeric( grid ) ) abline( h=grid/100, v=grid/100, col=gray( 0.9 ) )
       abline( 0, 1, col=gray( 0.4 ) )
       box()
       lines( 1-res[,"spec"], res[,"sens"], lwd=lwd )

  # Tickmarks on the ROC-curve
       if ( !is.null(cuts) )
       {
       ROC.tic( cuts,
                txt=formatC( cuts, digits=2, format="f" ),
                fv=fv, res=res, dist=0.03, cex=0.7)
       }

  # Plot of optimality point
       if (MX)
       {
       mx <- max( res[,1]+res[,2] )
       mhv <- which( (res[,1]+res[,2])==mx )[1]
       mxf <- fv[mhv]
       abline( mx-1, 1, col=gray(0.4) )
       ROC.tic( mxf,
                txt=paste( rnam, "=", formatC( mxf, format="f", digits=3 ) ),
                fv=fv, res=res, dist=0.03, cex=0.7, angle=135 )
       }

  # Model information
       if (MI)
       {
       crn <- par()$usr
       text(0.95*crn[2]+0.05*crn[1], 0.07, Model.inf,
            adj=c(1,0.5),cex=0.7)
       cf <- summary(lr)$coef[,1:2]
       nf <- dimnames(cf)[[1]]

       text(0.95*crn[2]+0.05*crn[1], 0.10,
            paste("Variable\ \ \ \ \ \ est.\ \ \ \ \ (s.e.) \ \ \n",
                  paste(rbind(nf,
                              rep("\ \ \ \ ",length(nf)),
                              formatC(cf[,1],digits=3,format="f"),
                              rep("\ \ \ (",length(nf)),
                              formatC(cf[,2],digits=3,format="f"),
                              rep(")",length(nf)),
                              rep("\n",length(nf))),
                        collapse=""),
                  collapse=""),
            adj=c(1,0), cex=0.7 )
       }

  # Print the area under the curve
       if (AUC)
       {
       crn <- par()$usr
       text( 0.95*crn[2]+0.05*crn[1], 0.00,
             paste( "Area under the curve:",
                    formatC( auc, format="f", digits=3, width=5 ) ),
             adj=c(1,0), cex=0.7 )
       }

  # Predictive values at maximum
       if (PV)
       {
       if(!MX) { mx <- max(res[,1]+res[,2])
                 mhv <- which((res[,1]+res[,2])==mx)
                 mxf <- fv[mhv]
               }
       ROC.tic(mxf, fv=fv, res=res,
                    txt= paste( "Sens: ",
                                formatC(100*res[mhv,1],digits=1,format="f"),
                         "%\n", "Spec: ",
                                formatC(100*res[mhv,2],digits=1,format="f"),
                         "%\n", "PV+: ",
                                formatC(100*res[mhv,3],digits=1,format="f"),
                         "%\n", "PV-: ",
                                formatC(100*res[mhv,4],digits=1,format="f"),
                         "%", sep="" ),
                    dist=0.1, cex=0.7, angle=-45 )
       }
}
invisible( list( res=res, AUC=auc, lr=lr ) )
}

contr.2nd <-
function( n )
{
if( is.numeric(n) && length(n) == 1 )
  levs<- 1:n
else {
  levs <- n
  n <- length(n)
}
if( n<3 ) stop( "Contrasts not defined for ", n-2, " degrees of freedom" )
outer( 1:n, 3:n-1, FUN=function(x,y) pmax( x-y, 0 ) )
}

Aplot <-
function( rates,
            age = as.numeric( dimnames( rates )[[1]] ),
            per = as.numeric( dimnames( rates )[[2]] ),
           grid = FALSE,
         a.grid = grid,
          ygrid = grid,
       col.grid = gray( 0.9 ),
          a.lim = range( age, na.rm=TRUE ),
           ylim = range( rates[rates>0], na.rm=TRUE ),
             at = NULL,
         labels = paste( at ),
          a.lab = names( dimnames( rates ) )[1],
           ylab = deparse( substitute( rates ) ),
           type = "l",
            lwd = 2,
            lty = 1,
            col = par( "fg" ),
         log.ax = "y",
            las = 1,
          c.col = col,
          p.col = col,
          c.ann = FALSE,
          p.ann = FALSE,
          xannx = 1/20,
        cex.ann = 0.8,
         c.thin = seq( 2, length( age ) + length( per ) - 1, 2 ),
         p.thin = seq( 1, length( per ), 2 ),
        p.lines = TRUE,
        c.lines = !p.lines,
            ...   # arguments passed on to matlines()
            )
{

# Plot the frame
if( p.ann ) a.lim <- a.lim + c(0,diff( range( age ) ) * xannx)
if( c.ann ) a.lim <- a.lim - c(  diff( range( age ) ) * xannx,0)
matplot( age, rates, type="n",
         xlim=a.lim, ylim=ylim, xlab=a.lab, ylab=ylab,
         log=log.ax, las=las, yaxt=if( !is.null( at ) ) "n" else "s" )
if( !is.null( at ) ) axis( side=2, at=at, labels=labels, yaxt="s", las=las )
# and the grid if required:
if( !missing( a.grid ) | !missing( grid ) )
  {
  if( is.logical( a.grid ) & a.grid[1] )
    a.grid <- nice( age, log=par("xlog") )
  abline( v=a.grid, col=col.grid )
  }
if( !missing( ygrid ) | !missing( ygrid ) )
    {
  if( is.logical( ygrid ) & ygrid[1] ) ygrid <- nice( rates[!is.na(rates)], log=par("ylog") )
  abline( h=ygrid, col=col.grid )
  }
box()
# What lines were required?
if( !missing( c.lines ) & missing( p.lines ) ) p.lines <- !c.lines

# Period curves:
if( p.lines ){
matlines( age, rates, type=type, lwd=lwd, lty=lty, col=p.col, ... )
# annotate them if required (every second by default):
if( p.ann )
  {
   nr <- nrow( rates )
   nc <- ncol( rates )
   text( rep( age[nr], nc )[p.thin], rates[nr,][p.thin],
         paste( "", per[p.thin] ), adj=c(0,0.5), cex=cex.ann,
         col=if( length(p.col)==1 ) p.col else p.col[p.thin] )  
  }
}

# Cohort curves:
if( c.lines ){
# First convert the age-period table to an age-cohort frame
rt <- as.table( rates )
dimnames( rt ) <- list( age = age, per = per )
rtf <- data.frame( rt )
rtf$age <- as.numeric( as.character( rtf$age ) )
rtf$per <- as.numeric( as.character( rtf$per ) )
ac <- tapply( rtf$Freq, list( rtf$age, rtf$per-rtf$age ), mean )
matlines( age, ac, type=type, lwd=lwd, lty=lty, col=c.col, ... )
# annotate them if required (every other by default):
if( c.ann )
  {
   nr <- nrow( rt )
   nc <- ncol( rt )
   # Find the ages, cohorts and rates where the cohort curves starts
   a.min <- c( rev( age ), rep( age[1], nc-1 ) )
   p.min <- c( rep( per[1], nr-1 ), per )
   c.min <- p.min - a.min           
   r.min <- c(rt[nr:1,1],rt[1,2:nc])
   text( a.min[c.thin], r.min[c.thin], paste( "", c.min[c.thin] ),
         adj=c(1,0.5), cex=cex.ann,
         col=if( length(c.col)==1 ) c.col else c.col[c.thin] )
  }
}

}

N2Y <-
function( A, P, N,
          data=NULL,
          return.dfr=TRUE )
{
# Make local versions of variables if a dataframe is supplied
if( !is.null(data) )
  {
  A <- if( !missing(A) ) eval( substitute(A), data, parent.frame() )
       else data$A
  P <- if( !missing(P) ) eval( substitute(P), data, parent.frame() )
       else data$P
  N <- if( !missing(N) ) eval( substitute(N), data, parent.frame() )
       else data$N
  }
# Derive the interval lengths from supplied data
A.int <- unique( diff(sort(unique(A))) )
P.int <- unique( diff(sort(unique(P))) )
# Check if something is fishy
if( length(A.int)!=1 ) stop( "Non-uniform age interval lengths:\n", A.int )
if( length(P.int)!=1 ) stop( "Non-uniform period interval lengths:\n", P.int )
if( A.int!=P.int ) stop( "Unequal age and period interval lengths:\n",
                         "age: ", A.int, ", period: ", P.int )
# Put population prevalence data in a table
Ntab <- xtabs( N ~ A + P )
# Devise a table for the risk times
Ydim <- c(dimnames(Ntab),list(wh=c("lo","up")))
# Note: one less period category
Ytab <- NArray( Ydim )[,-dim(Ntab)[2],]    
# How many age and period classes
na <- nrow(Ytab)
np <- ncol(Ytab)
for(a in 1:na) for(p in 1:np)
   {
if( a < na ) Ytab[a,p,"up"] <- Ntab[a  ,p]/3 + Ntab[a+1,p+1]/6
if( a >  1 ) Ytab[a,p,"lo"] <- Ntab[a-1,p]/6 + Ntab[a  ,p+1]/3
if( a == 1 ) Ytab[a,p,"lo"] <- Ntab[a  ,p]/2 + Ntab[a  ,p+1]/2 - Ytab[a,p,"up"]
if( a ==na ) Ytab[a,p,"up"] <- Ntab[a  ,p]/2 + Ntab[a  ,p+1]/2 - Ytab[a,p,"lo"]
   }
# Remember to multiply to get the follow-up time
Ytab <- Ytab * A.int
# Convert to a data frame if required (the default)
if( return.dfr )
  {
      ## If a dataframe is required as return value
      Ytab <- data.frame(expand.grid(dimnames(Ytab)), Y=c(Ytab))
      ## Retrieve the numerical values of left endpoints of intervals
      Ytab$A <- as.numeric(as.character(Ytab$A))
      Ytab$P <- as.numeric(as.character(Ytab$P))
      ## Compute the correct midpoints from the supplied data
      Ytab$A <- Ytab$A + A.int * (1 + (Ytab$wh == "up"))/3
      Ytab$P <- Ytab$P + P.int * (1 + (Ytab$wh == "lo"))/3
      Ytab <- Ytab[, c("A","P","Y")]
  }
Ytab
}

###################################################################################
### apc-LCa comparison
apc.LCa <-
function( data,               # cohort reference for the interactions
   keep.models = FALSE, ... )
{
models <- c("APa","ACa","APaC","APCa","APaCa" )
LCa.list <- list()
length( LCa.list ) <- 5
names( LCa.list ) <- models
for( mod in models ){ cat( mod, ":\n" )
   LCa.list[[mod]] <- LCa.fit( data = data,
                              model = mod, ... ) }
APC <- apc.fit( data, npar = list( A=LCa.list[[2]]$knots$a.kn,
                                   P=LCa.list[[1]]$knots$p.kn,
                                   C=LCa.list[[1]]$knots$c.kn ) )
dev <- c( APC$Anova[c(2,5,3,4),"Mod. dev."],
          sapply( LCa.list, function(x) x$deviance ) )
df  <- c( APC$Anova[c(2,5,3,4),"Mod. df."],
          sapply( LCa.list, function(x) x$df ) )
names(dev)[1:4] <- names(df)[1:4] <-
gsub( "rift","",
gsub("eriod","",
gsub("ohort","",
gsub(    "-","",
gsub(  "ge", "", APC$Anova[c(2,5,3,4),"Model"])))))
if( keep.models ) return( list( dev = cbind( dev, df ),
                                apc = APC,
                                LCa = LCa.list ) )
             else return( cbind( dev, df ) )
}

show.apc.LCa <-
function( x, dev.scale=TRUE, top="Ad", ... )
{
if( is.list(x) ) x <- x[[1]]
print(x)
TM <- matrix(NA,9,9)
rownames( TM ) <- colnames( TM ) <-
paste( rownames(x), "\n", formatC(x[,1],format="f",digits=1) )
TM[1,2:3] <-
TM[2,c(4,5)] <-
TM[3,c(4,6)] <-
TM[4,c(7,8)] <-
TM[5,7] <-
TM[6,8] <-
TM[c(7,8),9] <- 1
bp <- list( x=c(50,30,70,50,10,90,30,70,50),
            y=c(90,70,70,50,50,50,30,30,10) )
if( !dev.scale )
  boxes.matrix( TM, boxpos=bp, hm=1.5, wm=1.5, ... )
else {
    print( top )
    print( str(x) )
    print( rownames(x) )
  bp$y <- 5+90*(pmin(x[top,1],x[,1])-x[9,1])/(x[top,1]-x[9,1])
  boxes.matrix( TM, boxpos=bp, hm=1.5, wm=1.5, ... )
  }
}

# The method
nid <- function (Lx, ...) UseMethod("nid")

nid.default <-
nid.Lexis <-
function(Lx, by = NULL, ...)
{
if (!is.null(by))
  {
  if (!(by %in% names(Lx))) stop( "'by' must be the name of a variable in Lx" )
  if (!is.factor(Lx[,by])) Lx[,by] <- factor(Lx[,by])
  }
if(is.null(by))
  length(unique(Lx$lex.id))
else
  sapply(split(Lx, Lx[,by]), nid.Lexis )
}

# Make a boostrap sample of a Lexis object:
# Sample the *persons* with replacement, possibly sampling within levels of by=
bootLexis <-
function(Lx,
       size = NULL,
         by = NULL,
    replace = TRUE )
{
if(!inherits(Lx, "Lexis")) stop("Only meaningful for Lexis objects.")

isDT <- inherits(Lx, "data.table")
if (isDT) class(Lx) <- c("Lexis", "data.frame")

# determine size of the bootstrap samples if not given
if (is.null(size)) size <- nid.Lexis(Lx, by = by)

# allowing for a length 1 x-vector
REsample <- function(x, sz) x[sample.int(length(x),
                                         size = sz,
                                         replace = replace)]

if (is.null(by)) { # Simple bootstrap
  bLx <- subid.Lexis(Lx, REsample(unique(Lx$lex.id), size))
} else { # Bootstrap by groups
  bLx <- NULL
  spL <- split(Lx, Lx[,by])
  for( i in 1:length(spL))
     {
     bLx <- rbind(bLx,
                  cbind(bootLexis(spL[[i]], size = size[i]),
                        bgr = paste(i)))
     }
  bLx$lex.id <- as.integer(interaction(bLx$lex.id, bLx$bgr))
  bLx <- bLx[, -grep("bgr", names(bLx))]
  }
# return the result after converting to data.table if needed
if (isDT) class(bLx) <- c("Lexis", "data.table", "data.frame")
bLx
}

# A utility function that returns a Lexis object subsetted to a set of
# lex.ids, allowing for repeat values of the original lex.id, new
# (numerical) lex.ids generated
subid.Lexis <-
function(Lx, ids)
{
tt <- table(ids)
bLx <- NULL
max.id <- 0
for (j in 1:max(tt))
    {
    # avoid note about no visible binding
        wh <- NULL
    lex.id <- NULL
    # who appears at least j times in the sample ?
    wh <<- names(tt[tt>=j])
    sb <- subset(Lx, lex.id %in% wh)
    # remember their original id
    sb$old.id <- sb$lex.id
    # assure that different samples of the same person has different lex.id (numeric)
    sb$lex.id <- as.integer(factor(sb$lex.id)) + max.id
    max.id <- max(sb$lex.id)
    bLx <- rbind(bLx, sb)
    }
# Generate new lex.ids in the order 1:N
bLx$lex.id <- as.integer(factor(bLx$lex.id))
bLx
}

mcutLexis <-
function( L0, # A Lexis object
       timescale = 1,    # the time scale referred to by L0[,wh]
       wh, # indices/names of columns witha dates of state entries (events)
      new.states = NULL, # Names of the event types (states)
precursor.states = transient(L0), # NULL,
      seq.states = TRUE, # Should state names reflect ordering of events
      new.scales = NULL, # Time-scales referring to time since
    ties.resolve = FALSE # Are tied event times accepted?
        )
{
### we rely on referring to the timescale and event time variables by name
if( is.numeric(timescale) ) timescale <- timeScales(L0)[timescale]    
if( is.numeric(wh) ) wh <- names(L0)[wh]    

### don't be silly
if( length(wh)==1 )
# return( docut( L0, osv ) ) # old cutLexis should be absorbed here
  stop( "mcutLexis not needed for one type of event - use cutLexis\n" )

### states    
if( is.null(new.states) )
  {
  new.states <- wh
  cat( "NOTE: Name of new states set to\n", new.states )
  }
if( length(wh) != length(new.states) ) 
  stop( "wh and new.states must have same length, but lengths are",
        "wh:", length(wh), "and new.states:", length(new.states), "\n" )

### precursor state
if(missing(precursor.states))
  cat("NOTE: Precursor states set to", precursor.states, "\n")
    
### timescales
# either all or none    
if( is.logical(new.scales) )
  if( any( new.scales ) )
  {  
  new.scales <- paste( "tf", new.states, sep="" )
  cat( "NOTE: new.scales set to: ", new.scales, "\n" )
  }
if( is.character(new.scales) & length(new.scales) != length(wh) )
  {
  new.scales <- paste( "tf", new.states, sep="" )
  warning( "new.scales not of same length as wh. Set to: ",
           new.scales, "\n" )
  }
if( is.character(new.scales) & length(intersect(new.scales,timeScales(L0))) )
  stop( "Names of new time scales must be different from names of existing timescales:\n",
        timeScales(L0) )  
    
### Tied transition times untied    
has.ties <- any( wh.tied <- apply( L0[,wh], 1,
                            function(x) any(diff(sort(x[!is.na(x)]))==0) ) )
if( has.ties & is.logical(ties.resolve) & !ties.resolve )
  stop( "Tied event times not allowed with ties.resolve=FALSE:\n",
        "there were", length(wh.tied), "records with tied event times.")
if( has.ties & is.logical(ties.resolve) & ties.resolve ) ties.resolve <- 1/100
if( has.ties & is.numeric(ties.resolve) )
  {
  rnd <- L0[wh.tied,wh]*0
  rnd[,] <- runif(rnd,-1,1) * ties.resolve
  L0[wh.tied,wh] <- L0[wh.tied,wh] + rnd
  cat("NOTE:", sum(wh.tied),
 "records with tied events times resolved (adding", ties.resolve,
 "random uniform),\n",
 "     so results are only",
 "reproducible if the random number seed was set.\n")
  } 
# End of checks

# The object to return initiated as NULL
Lcut <- NULL

# Utility function returning sequences of ocurrences as paste of numbers
NAorder <- 
function (x) 
    {
    oo <- order(x, na.last = T)
    on <- (1:length(oo))[oo]
    on[is.na(x[oo])] <- NA
    paste(on[!is.na(on)], collapse = "-")
    }
    
# where do the different sequences of events actually occur in data
L0$whseq <- apply( L0[,wh], 1, NAorder )

# Loop through the actually occurring orders of event occurrences
for( sq in unique(L0$whseq) )
{ 
# Persons with none of the events occurring transferred to result
if( sq=="" ) Lcut <- rbind( Lcut, L0[L0$whseq=="",] )
else {
    
# Extract the subset of persons with a given sequence of events
Ltmp <- L0[L0$whseq==sq,]

# The numerical sequence of states (refer to the elements of wh)
ost <- as.numeric( strsplit( sq, "-" )[[1]] )
nxst <- ""
prst <- precursor.states
for( cs in ost )
   {
 nxst <- ifelse( cs==ost[1],  new.states[cs],
                 paste( nxst, new.states[cs], sep="-" ) )
 Ltmp <- cutLexis( Ltmp, cut = Ltmp[,wh[cs]],
                   timescale = timescale,
                   new.state = nxst,
            precursor.states = prst )
 # include the created state among the precursor states for next cut
 prst <- c(prst,nxst)
   } # end of for loop through events in this sequence (cs)
    
# Attach it to the end of the Lexis object
Lcut <- rbind( Lcut, Ltmp )
     } # end of the else clause

} # end of for loop through sequences (sq)

# Do we want the sequences, the unordered set of previous events or
# just the current one:
old.seq <- seq.states
if( is.logical(seq.states) ) seq.states <- ifelse( seq.states, "s", "u" )
if( is.character(seq.states) ) seq.states <- tolower( substr(seq.states,1,1) )
if( !(seq.states %in% c("s","o","u","l","c")) )
    stop( "What do you mean by seq.states=", old.seq,
          "? - it should abbreviate to one of s, o, u, l or c \n")
# Unordered or last (current) states    
if( seq.states %in% c("u","l","c") ) 
  {
  # Each list element is a vector of states visited
  slvl <- strsplit( levels( Lcut ), "-" )
  # merge those that have the same elements or take the last
  rlvl <- if( seq.states=="u" ) { sapply( lapply( slvl, sort ), paste, collapse="+" )
                           } else sapply( slvl, function(x) x[length(x)] )
  # Relevel the states    
  levels( Lcut$lex.Cst ) <-
  levels( Lcut$lex.Xst ) <- rlvl
  }

# Did we ask for timescales as time since events?
if( !is.null(new.scales) )
  {
  # insert columns for the new time scales
  Lcut <- Lcut[,c(rep("whseq",length(new.scales)),names(Lcut))]
  names( Lcut )[1:length(new.scales)] <- new.scales
  for( i in 1:length(wh) )
     Lcut[,i] <- ifelse( Lcut[,timescale] - Lcut[,wh[i]] < 0,
                         NA,
                         Lcut[,timescale] - Lcut[,wh[i]] )
  # set attributes
  attr( Lcut, "time.scales" ) <- c( attr( Lcut, "time.scales" ), new.scales )
  attr( Lcut, "time.since"  ) <- c( attr( Lcut, "time.since"  ), new.states )
  }
    
# return the cut object without the auxilary variable
rmcol <- grep( "whseq", names(Lcut) )
Lcut[,-rmcol]  
  }

ci.mat <-
function( alpha=0.05, df=Inf )
{
ciM <- rbind( c(1,1,1), qt(1-alpha/2,df)*c(0,-1,1) )
colnames( ciM ) <- c("Estimate", 
   paste( formatC( 100*   alpha/2 , format="f", digits=1 ), "%", sep="" ),
   paste( formatC( 100*(1-alpha/2), format="f", digits=1 ), "%", sep="" ) )
ciM
}

contr.cum <-
function(n) 
{
    if (is.numeric(n) && length(n) == 1) 
        levs <- 1:n
    else {
        levs <- n
        n <- length(n)
    }
    contr <- array(0, c(n, n), list(levs, levs))
    contr[col(contr) <= row(contr)] <- 1
    if (n < 2) stop(paste("Contrasts not defined for", n - 1, "degrees of freedom"))
    contr <- contr[, -1, drop = FALSE]
    contr
}

cal.yr <-
function( x,
     format = "%Y-%m-%d",
         wh = NULL )
{
cl.typ <- c("Date","POSIXct","POSIXlt","date","dates","chron")
# Check if the input is a data frame and convert column bu column
if (inherits(x, "data.frame"))
   { 
   if (is.null(wh) & missing(format))
      {
      # Indicator of where a date-type variable is
      wh <- which(sapply(x, inherits, cl.typ))
      }
   if (is.null(wh) & !missing(format))
      {
      # Indicator of where the character variables are
      wh <- which(sapply(x, is.character))
      }
   if (is.character(wh)) wh <- match(wh, names(x))
   # Convert the dates or the character variables one at a time
   for (i in wh)
       {
       if (is.character(x[,i]))
          x[,i] <- cal.yr(x[,i], format = format)
       else
          x[,i] <- cal.yr(x[,i])
       }
   return(x)
   } # end of dataframe doings
# now converting a single vector to decimal years:
# Check if the input is some kind of date or time object
if (any(inherits(x, cl.typ))) x <- as.Date(as.POSIXct(x))
else if( is.character(x))     x <- as.Date(x,
                                           format = format)
else if( is.factor(x))        x <- as.Date(as.character(x),
                                           format = format)
else stop("\nInput should be a data frame, a character vector",
          "a factor or some kind of date or time object:\n",
          "Date, POSIXct, POSIXlt, date, dates or chron")
res <- as.numeric(x) / 365.25 + 1970
class(res) <- c("cal.yr", "numeric")
return(res)
}

# Clear the workspace of all objects whose names don't start with a
# full stop, and remove objects attached immediately after the global 
clear <-
function () 
{
    env <- as.environment(1)
    to.go <- ls(env, all.names = FALSE)
    continue <- TRUE
    while (continue) {
        nxt <- search()[[2]]
        if (substr(nxt, 1, 8) != "package:") 
            detach()
        else continue <- FALSE
    }
    remove(list = to.go, envir = env)
}

contr.diff <-
function(n)
{
    if (is.numeric(n) && length(n) == 1) 
        levs <- 1:n
    else {
        levs <- n
        n <- length(n)
    }
    contr <- array(0, c(n, n), list(levs, levs))
    contr[col(contr) == row(contr)] <- 1
    contr[row(contr) - col(contr) == 1] <- -1
    if (n < 2) stop(paste("Contrasts not defined for", n - 1, "degrees of freedom"))
    contr
}

# The coef() methods in nlme and lme4 do something different,
# other objects do not even have coef or vcov methods defined,
# so we make a workaround by specifying our own generic methods:
COEF          <- function(x, ...) UseMethod("COEF")
COEF.default  <- function(x, ...) coef(x, ...)
VCOV          <- function(x, ...) UseMethod("VCOV")
VCOV.default  <- function(x, ...) vcov(x, complete = TRUE, ...)

# Then we can get from these methods what we want from lme, mer etc.
COEF.lme      <- function(x, ...) nlme::fixed.effects(x)
COEF.mer      <- function(x, ...) lme4::fixef(x)
COEF.lmerMod  <- function(x, ...) lme4::fixef(x)
COEF.glmerMod <- function(x, ...) lme4::fixef(x)

# The vcov returns a matrix with the wrong class so we strip that:
VCOV.lme      <- function(x, ...) as.matrix(vcov(x))
VCOV.mer      <- function(x, ...) as.matrix(vcov(x))
VCOV.lmerMod  <- function(x, ...) as.matrix(vcov(x))
VCOV.glmerMod <- function(x, ...) as.matrix(vcov(x))

# For the rest of the non-conforming classes we then just need the
# methods not defined
# VCOV.coxph    <- function(x, ...)
#                  survival::vcov.coxph(x, complete=FALSE, ...)
COEF.crr      <- function(x, ...) x$coef
VCOV.crr      <- function(x, ...) x$var
COEF.MIresult <- function(x, ...) x$coefficients
VCOV.MIresult <- function(x, ...) x$variance
COEF.mipo     <- function(x, ...) x$qbar
VCOV.mipo     <- function(x, ...) x$t
VCOV.gnlm     <- function(x, ...) x$cov
VCOV.rq       <- function(x, ...) summary(x, cov=TRUE)$cov

df2ctr <-
function(obj, nd)
    {
if (!(inherits(obj, "lm") | inherits(obj, "coxph")))
  stop("data frame facility not inplemented for ", class(obj), " objects")

# Factors in the prediction frame must have more than one level, which
# they typically do not have in the specification of a prediction
# frame, so we find the factors in the prediction frame and expand
# levels to the complete set of levels which we get from the model
# object This should secure the working of model.matrix(); note that
# the machinery differs between gam and non-gam glms

dcl <- attr(obj$terms, "dataClasses")[-1] # the first is the reponse
whf <- (dcl == "factor")
if (any(whf))
   {
for (fn in names(dcl)[which(whf)])
    {
    nd[,fn] <- factor(nd[,fn],
                      levels = if (inherits(obj, "gam"))
                                  levels(obj$var.summary[[fn]])
                             else        obj$xlevels    [[fn]])
    }
   }

# The folowing is needed to keep NA rows from the data frame supplied
org.op <- options( na.action='na.pass' )
on.exit( options( org.op ) )

# The contrast matrix from the model - differs a between (g)lm, gam and coxph
if (inherits(obj,"coxph")) MM <- model.matrix(         obj     ,    data=nd)
if (inherits(obj,"gam"  )) MM <- model.matrix(         obj     , newdata=nd)
   else
if (inherits(obj,"lm"   )) MM <- model.matrix( formula(obj)[-2],    data=nd)
return(MM)
    }

ci.dfr <-
function(obj, ndx, ndr,
       xvars = NULL,
        vcov = FALSE,
       alpha = 0.05,
         Exp = FALSE,
      sample = FALSE )
{
if( nrow(ndr)==1 ) ndr <- ndr[rep(1,nrow(ndx)),,drop=FALSE]
if(         ( nrow(ndx)  !=       nrow(ndr)) |
    any(sort(names(ndx)) != sort(names(ndr))) )
    stop("\nThe two prediction frames must have same dimensions",
         "and column names, but dimensions are (",
         paste(dim(ndx), collapse=","), ") and (",
         paste(dim(ndr), collapse=","), ")\n",
         "and the column names are:\n",
         "exp: ", paste(names(ndx), collapse=", "), "\n",
         "ref: ", paste(names(ndr), collapse=", "), "\n")
# Now supply and fix those variables that are needed in order to get
# model.matrix working:

# Supplied variable names:
 cols <- names( ndx )
# Factors in model; which are supplied; derive names of omitted
# factors (called ofacs)
 facs <- names( obj$xlevels )
ofacs <- setdiff( facs, cols )
# omitted *variables* must be supplied
ovars <- setdiff( xvars, facs )
# Construct the extra columns with bogus data (their contribution will
# be null)
xcols <- ndx[,NULL]
if (length(ofacs) > 0 ) for (fn in ofacs) xcols[,fn] <- obj$xlevels[[fn]][1]
if (length(ovars) > 0 ) for (vn in ovars) xcols[,vn] <- 1
if (dim(xcols)[2] > 0)
  {
  ndx <- cbind(ndx, xcols)
  ndr <- cbind(ndr, xcols)
  }
ci.lin(obj,
   ctr.mat = df2ctr(obj, ndx) - df2ctr(obj, ndr),
      vcov = vcov,
     alpha = alpha,
       Exp = Exp,
    sample = sample)
}

ci.dfr2 <-
function( obj, nd1, nd2, nd3, nd4,
         xvars = NULL,
          vcov = FALSE,
         alpha = 0.05,
           Exp = FALSE,
        sample = FALSE )
{
if( nrow(nd2)==1 ) nd2 <- nd2[rep(1,nrow(nd1)),,drop=FALSE]
if( nrow(nd3)==1 ) nd3 <- nd3[rep(1,nrow(nd1)),,drop=FALSE]
if( nrow(nd4)==1 ) nd4 <- nd4[rep(1,nrow(nd1)),,drop=FALSE]
if( ( nrow(nd2) !=  nrow(nd1)) |
    ( nrow(nd3) !=  nrow(nd1)) |
    ( nrow(nd4) !=  nrow(nd1)) |
 any(names(nd2) != names(nd1)) |
 any(names(nd3) != names(nd1)) |
 any(names(nd4) != names(nd1)) )
    stop("\nThe prediction frames must have same dimensions and column names:",
         "but dimensions are: (",
         paste( dim(nd1),collapse=","), ") and (",
         paste( dim(nd2),collapse=","), ") and (",
         paste( dim(nd3),collapse=","), ") and (",
         paste( dim(nd4),collapse=","), ")\n",
         "and column names are:\n",
         "1: ", paste( names(nd1), collapse=", " ), "\n",
         "2: ", paste( names(nd2), collapse=", " ), "\n",
         "3: ", paste( names(nd3), collapse=", " ), "\n",
         "4: ", paste( names(nd4), collapse=", " ), "\n")
# Now supply and fix those variables that are needed in order to get
# model.matrix working:
# Supplied variable names:
 cols <- names( nd1 )
# Factors in model; which are supplied; derive names of omitted
# factors (ofacs)
 facs <- names( obj$xlevels )
ofacs <- setdiff( facs, cols )
# omitted *variables* must be supplied
ovars <- setdiff( xvars, facs )
# Construct the extra columns with bogus data (their contribution will
# be null)
xcols <- nd1[,NULL]
if( length(ofacs) > 0 ) for( fn in ofacs ) xcols[,fn] <- obj$xlevels[[fn]][1]
if( length(ovars) > 0 ) for( vn in ovars ) xcols[,vn] <- 1
if( dim(xcols)[2]>0 )
  {
  nd1 <- cbind( nd1, xcols )
  nd2 <- cbind( nd2, xcols )
  nd3 <- cbind( nd3, xcols )
  nd4 <- cbind( nd4, xcols )
  }
ci.lin( obj,
    ctr.mat = df2ctr( obj, nd1 )
             -df2ctr( obj, nd2 )
             -df2ctr( obj, nd3 )
             +df2ctr( obj, nd4 ),
       vcov = vcov,
      alpha = alpha,
        Exp = Exp,
     sample = sample )
}

ci.lin <-
function( obj,
      ctr.mat = NULL,
       subset = NULL,
       subint = NULL,
        xvars = NULL,
        diffs = FALSE,
         fnam = !diffs,
         vcov = FALSE,
        alpha = 0.05,
           df = Inf,
          Exp = FALSE,
       sample = FALSE )
{
# If ctr.mat is a data frame, call df2ctr
if( inherits( ctr.mat, "data.frame" ) ) ctr.mat <- df2ctr( obj, ctr.mat )

# If ctr.mat is a list of two dataframes then call ci.dfr
if( inherits( ctr.mat, "list" ) )
  {
  if( length(ctr.mat)==2 )
    {
  if( !inherits( ctr.mat[[1]], "data.frame" ) |
      !inherits( ctr.mat[[2]], "data.frame" ) )
      stop("If ctr.mat is a list it must be a list of data frames")
  return( ci.dfr( obj, ctr.mat[[1]], ctr.mat[[2]],
                xvars = xvars,
                 vcov = vcov,
                alpha = alpha,
                  Exp = Exp,
               sample = sample ) )
    }
# If ctr.mat is a list of two dataframes then call ci.dfr2 for 2nd
# order differences
  if( length(ctr.mat)==4 )
    {
  if( !inherits( ctr.mat[[1]], "data.frame" ) |
      !inherits( ctr.mat[[2]], "data.frame" ) |
      !inherits( ctr.mat[[3]], "data.frame" ) |
      !inherits( ctr.mat[[4]], "data.frame" ) )
      stop("If ctr.mat is a list it must be a list of data frames")
  return( ci.dfr2( obj, ctr.mat[[1]], ctr.mat[[2]], ctr.mat[[3]], ctr.mat[[4]],
                 xvars = xvars,
                  vcov = vcov,
                 alpha = alpha,
                   Exp = Exp,
                sample = sample ) )
    }
  }

# First extract all the coefficients and the variance-covariance matrix
cf  <- COEF( obj )
vcv <- VCOV( obj )
# Alised parameters are set to 0
if( any( is.na(  cf ) ) )  cf[is.na(cf )] <- 0
if( any( is.na( vcv ) ) ) vcv[is.na(vcv)] <- 0

# Function for computing a contrast matrix for all possible
# differences between a set of parameters.
all.dif <-
function( cf, pre=FALSE )
{
nn <- length( cf )
nr <- nn * ( nn - 1 ) / 2
nam <- names( cf )

# Work out the indexes of parameter pairs to compare
#
xx <- numeric( 0 )
for( j in 2:nn ) xx <- c(xx, j:nn )
ctr <- cbind( rep( 1:(nn-1), (nn-1):1 ), xx )

# Now for the annotation:
# Find out how large a proportion of rownames are identical
i <- 1
while( all( substr( nam, 1, i ) == substr( nam[1], 1, i ) ) ) i <- i+1

# If a factor name is given, then use this, otherwise the identical part
# of the parameter names
if( is.character( pre ) ) {
prefix <- pre
  pre <- TRUE
} else {
prefix <- substr( nam[1], 1, i-1 )
}
rn <- paste( if( pre ) prefix else "",
             substring( nam[ctr[,1]], i ), "vs.",
             substring( nam[ctr[,2]], i ) )

# Finally, construct the contrast matrix and attach the rownames
cm <- matrix( 0, nr, nn )
cm[cbind(1:nr,ctr[,1])] <- 1
cm[cbind(1:nr,ctr[,2])] <- -1
rownames( cm ) <- rn
cm
}
# end of the function all.dif for all differences

# Were all differences requested?
if( diffs )
  {
  if( is.character( subset ) )
    {
    if ( inherits( obj, "lm" ) &
         length( grep( subset, names( obj$xlevels ) ) )>0 )
       { # The case of factor level differences we find the relevant
         # subset of parameters by reconstructing names of parameters
         wf <- grep( subset, af <- names( obj$xlevels ) )
         # All factor levels
         fn <- obj$xlevels[[af[wf]]]
         # Reconstruct names of relevant parameter names
         pnam <- paste( af[wf], fn, sep="" )
         # Find them in the parameter vector
         wh <- match( pnam, names( coef( obj ) ) )
         # Get the relevant subset, and stick in 0s for NAs
         cf <- coef( obj )[wh]
         cf[is.na( cf )] <- 0
         vcv <- vcov( obj, complete=FALSE )[wh,wh]
         vcv[is.na( vcv )] <- 0
         names( cf ) <- rownames( vcv ) <- colnames( vcv ) <-
             paste( subset, ": ", fn, sep="" )
       } else {
         subset <- grep( subset, names( cf ) )
             cf <-  cf[subset]
            vcv <- vcv[subset,subset]
       }
    } else {
     cf <-  cf[subset]
    vcv <- vcv[subset,subset]
   }
   ctr.mat <- all.dif( cf, pre=fnam )
  }

if( !diffs )
  {
  if( is.character( subset ) ) {
    sb <- numeric(0)
    for( i in 1:length( subset ) ) sb <- c(sb,grep( subset[i], names( cf )  ))
    subset <- sb # unique( sb )
    }
  if( is.character( subint ) ) {
    sb <- 1:length(cf)
    for( i in 1:length(subint) ) sb <- intersect( sb, grep(subint[i],names(cf)) )
    subset <- sb # unique( sb )
    }
  if( is.null( subset ) & is.null( subint ) ) subset <- 1:length( cf )
  # Exclude units where aliasing has produced NAs.
  # Not needed after replacement with 0s
  # subset <- subset[!is.na( cf[subset] )]
   cf <-  cf[subset]
  vcv <- vcv[subset,subset]
  if( is.null( ctr.mat ) )
    {
    ctr.mat <- diag( length( cf ) )
    rownames( ctr.mat ) <- names( cf )
    }
  if( dim( ctr.mat )[2] != length(cf) )
      stop( paste("\n Dimension of ", deparse(substitute(ctr.mat)),
            ": ", paste(dim(ctr.mat), collapse = "x"),
            ", not compatible with no of parameters in ",
            deparse(substitute(obj)), ": ", length(cf), sep = ""))
  }
# Finally, here is the actual computation
  if( sample )
    {
    # mvrnorm() returns a vector if sample=1, otherwise a sample by
    # length(cf) matrix - hence the rbind so we always get a row
    # matrix and res then becomes an nrow(ctr.mat) by sample matrix
    res <- ctr.mat %*% t( rbind(mvrnorm( sample, cf, vcv )) )
    }
  else
    {
    ct <- ctr.mat %*% cf
    vc <- ctr.mat %*% vcv %*% t( ctr.mat )
    se <- sqrt( diag( vc ) )
    ci <- cbind( ct, se ) %*% ci.mat( alpha=alpha, df=df )
    t0 <- cbind( se, ct/se, 2 * ( pnorm( -abs( ct / se ) ) ) )
    colnames(t0) <- c("StdErr", "z", "P")
    res <- cbind(ci, t0)[, c(1, 4:6, 2:3), drop=FALSE]
    if( Exp )
      {
      res <- cbind(      res[,1:4     ,drop=FALSE],
                    exp( res[,c(1,5,6),drop=FALSE] ) )
      colnames( res )[5] <- "exp(Est.)"
      }
  # class( res ) <- c("ci.lin","matrix")
    }
# Return the requested structure
if( sample ) invisible( res )
else
if( vcov ) invisible( list( coef=ct[,1], vcov=vc ) ) else res
}

# print.ci.lin <-
# function( x, ..., digits=3 )
# {
# print( round( unclass(x), digits ) )
# }

# Handy wrapper
ci.exp <-
function( ..., Exp=TRUE, pval=FALSE )
{
res <- if( Exp )
         {
ci.lin( ..., Exp=TRUE  )[,if(pval) c(5:7,4)   else 5:7     ,drop=FALSE]
         } else {
ci.lin( ..., Exp=FALSE )[,if(pval) c(1,5,6,4) else c(1,5,6),drop=FALSE]
                }
# class( res ) <- c( "ci.lin", "matrix" )
res
}

# Wrapper for predict.glm to give estimates and confidence intervals
ci.pred <-
function( obj, newdata,
         Exp = NULL,
       alpha = 0.05 )
{
if( !inherits( obj, "lm" ) ) stop("Not usable for non-(g)lm objects")
if( !inherits( obj, "glm" ) & inherits( obj, "lm" ) )
    return( predict.lm( obj, newdata=newdata, interval="confidence" ) )
# get the prediction and se on the link scale
else {
zz <- predict( obj, newdata=newdata, se.fit=TRUE, type="link" )
# compute ci on link scale
zz <- cbind( zz$fit, zz$se.fit ) %*% ci.mat( alpha=alpha )
# transform as requested
if( missing(Exp) ) {   return( obj$family$linkinv(zz) )
} else {  if(  Exp ) { return(                exp(zz) )
   } else if( !Exp )   return(                    zz  )
       }
     }
}

# Function to calculate RR with CIs from independent rates with CIs;
# r1 and r2 are assumed to be vectors or 2 or 3-column matrices with
# rate, lower and upper confidence limits repectively.
ci.ratio <-
function( r1, r2,
         se1 = NULL, # standard error of rt1
         se2 = NULL, # standard error of rt2
      log.tr = !is.null(se1) & !is.null(se2), # is this log-rates?
       alpha = 0.05,
        pval = FALSE )
{
if( is.data.frame(r1) ) r1 <- as.matrix( r1 )
if( is.data.frame(r2) ) r2 <- as.matrix( r2 )
if( is.matrix(r1) & !is.null(se1) ) warning("r1 is matrix, se1 is ignored")
if( is.matrix(r2) & !is.null(se2) ) warning("r2 is matrix, se2 is ignored")

# if supplied as 1-column matrix change to vector
if( is.matrix(r1) ) if( ncol(r1)==1 ) r1 <- as.vector( r1 )
if( is.matrix(r2) ) if( ncol(r2)==1 ) r2 <- as.vector( r2 )

# move to log scale
if( !log.tr ) {
  r1 <- log( r1 )
  r2 <- log( r2 )
  }

# how wide are the condidence intervals
if( is.matrix(r1) ) if( ncol(r1)>1 ) rg1 <- t( apply(r1,1,range) )
if( is.matrix(r2) ) if( ncol(r2)>1 ) rg2 <- t( apply(r2,1,range) )

# get the estimates on the log-scale
R1 <- if( is.matrix(r1) ) apply( rg1, 1, mean ) else r1
R2 <- if( is.matrix(r2) ) apply( rg2, 1, mean ) else r2
if( is.null(se1) ) se1 <- apply( rg1, 1, diff ) / (2*qnorm(1-alpha/2))
if( is.null(se2) ) se2 <- apply( rg2, 1, diff ) / (2*qnorm(1-alpha/2))

# compute the RR and the c.i. and optionally the p-value
 lrr <- R1 - R2
slrr <- sqrt( se1^2 + se2^2 )
  rr <- cbind(lrr,slrr) %*% ci.mat(alpha=alpha)
if( !log.tr ) rr <- exp( rr )
if( pval ) return( cbind( rr, 1-pchisq( (lrr/slrr)^2, 1 ) ) )
      else return(        rr                                )
}

Termplot <-
function( obj,
         plot = TRUE,
         xlab = NULL,
         ylab = NULL,
          xeq = TRUE,
         yshr = 1.0,
        alpha = 0.05,
        terms = NULL,
       max.pt = NULL )
{
# max.pt suppiled
no.max <- missing( max.pt )
# Extract the curves to plot
zz <- termplot( obj, se=TRUE, plot=FALSE, terms=terms )
nt <- length( zz )
for( i in 1:nt )
{ 
# Thin the number of points in each returned term
if( no.max ) max.pt <- nrow( zz[[i]] )
if( is.numeric(max.pt) & (nrow(zz[[i]]) > max.pt) )
  zz[[i]] <- zz[[i]][round(seq(1,nrow(zz[[i]]),,max.pt)),]
# Compute the estimate and the c.i. on log-scale
zz[[i]] <- cbind( zz[[i]][,1],
                  exp(as.matrix(zz[[i]][,2:3])%*%ci.mat(alpha=alpha)) )
}

# Labels 
if( is.null(xlab) ) xlab <- names( zz )
if( is.null(ylab) ) ylab <- rep("",nt)

## Compute ranges of y and x
xw <- sapply( zz, function(x) diff(range(x[,1  ])) )
yl <- sapply( zz, function(x)      range(x[,2:4])  )
mr <- max( apply( yl, 2, function(x) exp(diff(log(x)))) )
yl <-      apply( yl, 2, function(x) exp(mean(log(x))+c(-1,1)*log(mr)/2*yshr))

if( plot )
  {
## Plot the effects side by side
  par( mfrow=c(1,nt), mar=c(3,3,1,1), mgp=c(3,1,0)/1.6 )#, bty="n", las=1 )
  if( xeq )
  ## Plot the terms so that x- and y-axes have the same extent
    {    
# Margins and total width in inches
    mw <- sum(par("mai")[c(2,4)])
    tw <- par("pin")[1]
# Widths of each plot, approx. at least
    pw <- (tw-nt*mw)*xw/sum(xw)+mw
    layout( rbind(1:nt), widths=pw )
    }
  for( i in 1:nt )
  matplot( zz[[i]][,1], zz[[i]][,2:4],
           xlab=xlab[i], xaxs="i", xlim=range(zz[[i]][,1]),
           ylab=ylab[i], yaxs="i", ylim=yl[,i],       
           log="y", type="l", lty=1, lwd=c(5,2,2), col="black" )
  }
## Return the extracted terms
invisible( zz )
}

lls <-
# A function that expands the functionality of ls()
function( pos = 1, pat = "", all=FALSE, print=TRUE )
{
# First a function that returns length/dim when you ask for it
dimx <- function(dd) if (is.null(dim(dd))) length(dd) else dim(dd)
# A vector of object names
lll <- ls( pos=pos, pattern=pat, all.names=all )
# Are there any objects at all?
if( length(lll) > 0 )
{
obj.mode <-
obj.clas <-
obj.dimx <-
obj.size <- character(0)
# Then find mode, class, name and dimension of them and return it
for(i in 1:length(lll))
{
obj.mode[i] <-        eval( parse(text = paste(       "mode(`", lll[i], "`)",sep="")))
obj.clas[i] <- paste( eval( parse(text = paste(      "class(`", lll[i], "`)",sep=""))), collapse=" " )
obj.dimx[i] <- paste( eval( parse(text = paste(       "dimx(`", lll[i], "`)",sep=""))), collapse=" " )
obj.size[i] <- formatC( eval( parse(text = paste("unclass(object.size(`", lll[i], "`))",sep="")))/2^10,
                        format="f", digits=1, big.mark=",", width=14, flag=" " )
}
dfr <-
data.frame( name = lll,
            mode = obj.mode,
           class = obj.clas,
             dim = obj.dimx,
      sizeKbytes = obj.size,
stringsAsFactors=FALSE )
names( dfr )[5] <- "      size(Kb)"
print( invisible( dfr ), right=FALSE )
}
}

tbox <-
function( txt, x, y, wd, ht,
          font=2, lwd=2,
          col.txt=par("fg"),
          col.border=par("fg"),
          col.bg="transparent" )
{
rect( x-wd/2, y-ht/2, x+wd/2, y+ht/2,
      lwd=lwd, border=col.border, col=col.bg )
text( x, y, txt, font=font, col=col.txt )
invisible( c( x, y, wd, ht ) )
}

dbox <-
function( x, y, wd, ht=wd,
          font=2, lwd=2, cwd=5,
          col.cross=par("fg"),
          col.border=par("fg"),
          col.bg="transparent" )
{
rect( x-wd/2, y-ht/2, x+wd/2, y+ht/2, lwd=lwd, border=col.border, col=col.bg )
ch <- ht*2/3
segments( c(x     , x-ch/3),
          c(y+ch/2, y+ch/6),
          c(x     , x+ch/3),
          c(y-ch/2, y+ch/6), lwd=cwd, col=col.cross )
invisible( c( x, y, wd, ht ) )
}

fillarr <-
function( x1, y1, x2, y2, gap=2, fr=0.8,
          angle=17, lwd=2, length=par("pin")[1]/30, ... )
{
fr <- 1-gap/sqrt((x1-x2)^2+(y1-y2)^2)
if( !missing(fr)  ) if( fr > 1 ) fr <- fr/100
for( a in 1:angle )
arrows( x1 + (x2-x1)*(1-fr)/2,
        y1 + (y2-y1)*(1-fr)/2,
        x2 - (x2-x1)*(1-fr)/2,
        y2 - (y2-y1)*(1-fr)/2, angle=a, lwd=lwd, ... )
}

std.vec <-
function( a, b )
{
  l <- sqrt(a^2+b^2)
  if( l==0 )
  return( c(0,0) )
  else
  return( c(a/l,b/l) )
}

boxarr <-
function (b1, b2, offset = FALSE, pos = 0.45, ...)
{
    d <- std.vec(b2[1] - b1[1], b2[2] - b1[2])
    dd <- d * offset
    x1 <- b1[1] - dd[2]
    y1 <- b1[2] + dd[1]
    w1 <- b1[3]
    h1 <- b1[4]
    x2 <- b2[1] - dd[2]
    y2 <- b2[2] + dd[1]
    w2 <- b2[3]
    h2 <- b2[4]
    hx1 <- x1 + ifelse((y2-y1) != 0, (x2-x1) * ((h1/2)/abs(y2-y1)), sign(x2-x1) * w1/2)
    vx1 <- x1 + ifelse((x2-x1) != 0, (x2-x1) * ((w1/2)/abs(x2-x1)), 0)
    hx2 <- x2 + ifelse((y1-y2) != 0, (x1-x2) * ((h2/2)/abs(y1-y2)), sign(x1-x2) * w2/2)
    vx2 <- x2 + ifelse((x1-x2) != 0, (x1-x2) * ((w2/2)/abs(x1-x2)), 0)
    hy1 <- y1 + ifelse((y2-y1) != 0, (y2-y1) * ((h1/2)/abs(y2-y1)), 0)
    vy1 <- y1 + ifelse((x2-x1) != 0, (y2-y1) * ((w1/2)/abs(x2-x1)), sign(y2-y1) * h1/2)
    hy2 <- y2 + ifelse((y1-y2) != 0, (y1-y2) * ((h2/2)/abs(y1-y2)), 0)
    vy2 <- y2 + ifelse((x1-x2) != 0, (y1-y2) * ((w2/2)/abs(x1-x2)), sign(y1-y2) * h2/2)
if( abs(vy1-y1) < h1/2 ) { bx1 <- vx1
                           by1 <- vy1 }
                    else { bx1 <- hx1
                           by1 <- hy1 }
if( abs(vy2-y2) < h2/2 ) { bx2 <- vx2
                           by2 <- vy2 }
                    else { bx2 <- hx2
                           by2 <- hy2 }
fillarr( bx1, by1, bx2, by2, ... )
invisible( list( x = bx1*(1-pos)+bx2*pos,
                 y = by1*(1-pos)+by2*pos,
                 d = d ) )
}

wh.no <-
function(tt, i, j)
{
## Utility to count the number of non-NA off diagonal elements with
## row<i or ( col=i & col<=j )
diag(tt) <- NA
sum(!is.na(tt[1:i,])) - sum(!is.na(tt[i,])) + sum(!is.na(tt[i,1:j]))
}

boxes <- function (obj, ...) UseMethod("boxes")

boxes.matrix <-
function( obj, ... )
  {
  Epi::boxes.Lexis( obj, ... )
  }

boxes.Lexis <-
function( obj, boxpos = FALSE,
                wmult = 1.20,
                hmult = 1.20 + 0.85*(!show.Y),
                  cex = 1.40,
               show   = inherits( obj, "Lexis" ),
               show.Y = show,
              scale.Y = 1,
             digits.Y = 1,
              show.BE = FALSE,
               BE.sep = c("","","          ",""),
               show.D = show,
              scale.D = FALSE,
             digits.D = as.numeric(as.logical(scale.D)),
               show.R = show & is.numeric(scale.R),
              scale.R = 1,
             digits.R = as.numeric(as.logical(scale.R)),
               DR.sep = if( show.D ) c("\n(",")") else c("",""),
                eq.wd = TRUE,
                eq.ht = TRUE,
                   wd,
                   ht,
               subset = NULL,
              exclude = NULL,
                 font = 1,
                  lwd = 2,
           col.txt    = par("fg"),
           col.border = col.txt,
           col.bg     = "transparent",
           col.arr    = par("fg"),
              lwd.arr = lwd,
             font.arr = font,
              pos.arr = 0.45,
              txt.arr = NULL,
          col.txt.arr = col.arr,
           offset.arr = 2, ... )
{
if( inherits(obj,"Lexis") )
  {
  if( !is.factor(obj$lex.Cst) | !is.factor(obj$lex.Xst) ) obj <- factorize( obj )
  tm <- tmat( obj, Y=TRUE )
  tt <- tmat( obj, Y=FALSE )
  # If show.BE is a character, check if zeros should be printed
  noz <- FALSE
  if( is.character( show.BE ) )
    {
    noz <- ( tolower(show.BE) %in% c("nz","noz","nozero","nozeros") )
    show.BE <- TRUE
    }
  if( show.BE )
    {
    # Derive the persons at start and at end of study in each state
    Beg <- as.vector( table( status( obj, at="entry", by.id=TRUE ) ) )
    End <- as.vector( table( status( obj, at="exit" , by.id=TRUE ) ) )
    BE.line <- paste( ifelse( noz & Beg==0,
                              rep("  ",nchar(BE.sep[1])+2),
                              paste(BE.sep[1],
                                    formatC( Beg,
                                             format="f",
                                             digits=0,
                                             big.mark="," ),
                                    BE.sep[2],sep="") ),
                      ifelse( noz & End==0,
                              rep("  ",nchar(BE.sep[2])+nchar(BE.sep[3])+2),
                              paste(BE.sep[3],
                                    formatC( End,
                                             format="f",
                                             digits=0,
                                             big.mark="," ),
                                    BE.sep[4],sep="") ),
                      sep="" )
    }
  }
else if( is.matrix(obj) & diff(dim(obj))==0 )
  {
  tm <- tt <- obj
  }
else stop( "First argument must be a Lexis object or a square matrix.\n" )

# Put the transitions into D and the diagonal into Y.
D <- tm
diag( D ) <- NA
Y <- diag( tm ) / scale.Y

# Explicitly given numbers to be put in boxes ?
if( is.numeric(show.Y) )
  {
  Y <- show.Y
  show.Y <- TRUE
  }

# Compute the rates - vectors are automatically expanded to matrices columnwise
R <- D / Y * ifelse(scale.R,scale.R,1)

# If no person-years available anywhere, neither they nor rates can be shown
if( all(is.na(Y)) ) show.Y <- show.R <- FALSE

# Derive state names, no. states and no. transitions
                      st.nam <- colnames( tm )
if( is.null(st.nam) ) st.nam <- paste(1:ncol(tm))
            pl.nam <- st.nam
      n.st <- length( st.nam )
      n.tr <- sum( !is.na(tm) ) - sum( !is.na(diag(tm)) )

# No extra line with person-years when they are NA, put in no at
# beginnning / end of study
if( show.Y ) pl.nam <- gsub( "\\\nNA", "",
                             paste( st.nam,
                                    formatC( Y,
                                             format="f",
                                             digits=digits.Y,
                                             big.mark="," ),
                                    sep="\n" ) )
if( show.BE ) pl.nam <- paste( pl.nam, BE.line, sep="\n" )

# Any subsetting:
sbst <- 1:nrow(tm)
if( !is.null(exclude) )
  {
  if( is.character(exclude) )
    exclude <- match( exclude, rownames(tm) )
  sbst <- sbst[-exclude]
  }
if( !is.null(subset) )
  {
  if( is.character(subset) )
      sbst <- match( subset, rownames(tm) )
  else sbst <- subset
  }
subset <- sbst

# Recycling of box-arguments
if( !missing(ht) )
if(   length(ht       )<n.st ) ht         <- rep(ht        ,n.st)[1:n.st]
if( !missing(wd) )
if(   length(wd       )<n.st ) wd         <- rep(wd        ,n.st)[1:n.st]
if( length(font       )<n.st ) font       <- rep(font      ,n.st)[1:n.st]
if( length(lwd        )<n.st ) lwd        <- rep(lwd       ,n.st)[1:n.st]
if( length(col.border )<n.st ) col.border <- rep(col.border,n.st)[1:n.st]
if( length(col.bg     )<n.st ) col.bg     <- rep(col.bg    ,n.st)[1:n.st]
if( length(col.txt    )<n.st ) col.txt    <- rep(col.txt   ,n.st)[1:n.st]
# Recycling of arrow-arguments
if( length(col.arr    )<n.tr ) col.arr    <- rep(col.arr    ,n.tr)[1:n.tr]
if( length(col.txt.arr)<n.tr ) col.txt.arr<- rep(col.txt.arr,n.tr)[1:n.tr]
if( length(lwd.arr    )<n.tr ) lwd.arr    <- rep(lwd.arr    ,n.tr)[1:n.tr]
if( length(font.arr   )<n.tr ) font.arr   <- rep(font.arr   ,n.tr)[1:n.tr]
if( length(pos.arr    )<n.tr ) pos.arr    <- rep(pos.arr    ,n.tr)[1:n.tr]

if( length(col.arr    )>n.tr ) col.arr    <- col.arr    [1:n.tr]
if( length(col.txt.arr)>n.tr ) col.txt.arr<- col.txt.arr[1:n.tr]
if( length(lwd.arr    )>n.tr ) lwd.arr    <- lwd.arr    [1:n.tr]
if( length(font.arr   )>n.tr ) font.arr   <- font.arr   [1:n.tr]
if( length(pos.arr    )>n.tr ) pos.arr    <- pos.arr    [1:n.tr]

# Here comes the plot
# First setting up the plot area, and restoring the plot parameters later
opar <- par( mar=c(0,0,0,0), cex=cex )
on.exit( par(opar) )
plot( NA, bty="n",
      xlim=0:1*100, ylim=0:1*100, xaxt="n", yaxt="n", xlab="", ylab="" )
# String height and width only meaningful after a plot has been called
if( missing(ht) )
  {
  ht <- strheight( pl.nam ) * hmult
  if( eq.ht ) ht <- rep( max(ht), length(ht) )
  }
if( missing(wd) )
  {
  wd <- strwidth(  pl.nam ) * wmult
  if( eq.wd ) wd <- rep( max(wd), length(wd) )
  }

# If not supplied, ask for positions of boxes
if( is.list(boxpos) )
  {
  names(boxpos) <- tolower( names(boxpos) )
  if( length(intersect(names(boxpos),c("x","y")))<2 )
    stop( "The list given in 'boxpos=' must have components 'x' and 'y'" )
  if( length(boxpos$x) != n.st | length(boxpos$y) != n.st )
    stop( "The elements 'x' and 'y' of boxpos must both have length equal to no. states", n.st )
  xx <- boxpos$x
  yy <- boxpos$y
  }
if( is.logical(boxpos) )
  if( boxpos )
  {
  ang <- pi - 2*pi*((1:n.st-0.5)/n.st)
  xx <- cos( ang ) * 35 + 50
  yy <- sin( ang ) * 35 + 50
  }
  else
  {
  xx <- yy <- numeric(n.st)
  for( i in subset )
     {
     cat( "\nClick for level ", st.nam[i] )
     flush.console()
     pt <- locator(1)
     xx[i] <- pt$x
     yy[i] <- pt$y
     tbox( pl.nam[i], xx[i], yy[i], wd[i], ht[i], ... )
     }
  cat( "\n" )
  }

# Plot the boxes and record position and size
b <- list()
for( i in subset ) b[[i]] <- tbox( pl.nam[i], xx[i], yy[i], wd[i], ht[i],
                                font=font[i],
                                  lwd=lwd[i],
                          col.txt=col.txt[i],
                    col.border=col.border[i],
                            col.bg=col.bg[i] )

# Arrows and text on them
arrowtext <- character(0)
for( i in subset ) for( j in subset )
  {
  if( !is.na(tt[i,j]) & i!=j )
    {
    # Which number of arrow is currently processed?
    a <- wh.no( tt, i, j )
    arr <- boxarr( b[[i]], b[[j]],
                   offset=(!is.na(tt[j,i]))*offset.arr,
                   lwd=lwd.arr[a], col=col.arr[a], pos=pos.arr[a], ... )
    if( show.D | show.R )
      {
    if( show.D & D[i,j]>0 )
      arrowtext[a] <- formatC( D[i,j], format="f", digits=0, big.mark="," )
    else arrowtext[a] <- ""
    if( show.R & R[i,j]>0 )
      arrowtext[a] <- paste( if( !is.null(arrowtext[a]) )
                                   paste( arrowtext[a], DR.sep[1], sep="" ),
                           formatC( R[i,j], format="f", digits=digits.R, big.mark="," ),
                          if( length(DR.sep) > 1 ) DR.sep[2], sep="" )
      }
    else
    if( !is.null(txt.arr) )
      arrowtext[a] <- txt.arr[a]
    if( !is.null(arrowtext[a]) )
    text( arr$x-arr$d[2], arr$y+arr$d[1],
          arrowtext[a],
          adj=as.numeric(c(arr$d[2]>0,arr$d[1]<0)),
          font=font.arr[a], col=col.txt.arr[a] )
    }
  }
# Redraw the boxes with white background to remove any arrows crossing
for( i in subset ) tbox( pl.nam[i], xx[i], yy[i], wd[i], ht[i],
                         font=font[i], lwd=lwd[i], col.bg=par("bg") )
# Then redraw the boxes again
for( i in subset ) tbox( pl.nam[i], xx[i], yy[i], wd[i], ht[i],
                                font=font[i],
                                  lwd=lwd[i],
                          col.txt=col.txt[i],
                    col.border=col.border[i],
                            col.bg=col.bg[i] )

# Finally create an object with all information to re-draw the display
MSboxes <- list( Boxes = data.frame( xx = xx,
                                     yy = yy,
                                     wd = wd,
                                     ht = ht,
                                   font = font,
                                    lwd = lwd,
                                col.txt = col.txt,
                             col.border = col.border,
                                 col.bg = col.bg, stringsAsFactors=FALSE ),
           State.names = pl.nam,
                  Tmat = tt,
                Arrows = data.frame( lwd.arr = lwd.arr,
                                     col.arr = col.arr,
                                     pos.arr = pos.arr,
                                 col.txt.arr = col.txt.arr,
                                    font.arr = font.arr,
                                  offset.arr = offset.arr, stringsAsFactors=FALSE ),
             Arrowtext = arrowtext )
class( MSboxes ) <- "MS"
invisible( MSboxes )
}

boxes.MS <-
function( obj, sub.st, sub.tr, cex=1.5, ... )
{
if( !inherits(obj,"MS") )
  stop( "You must supply an object of class 'MSboxes'" )

n.st <- nrow( obj$Boxes )
n.tr <- nrow( obj$Arrows )
if( missing(sub.st) ) sub.st <- 1:n.st
if( missing(sub.tr) ) sub.tr <- 1:n.tr

# First setting up the plot area, and restoring the plot parameters later
opar <- par( mar=c(0,0,0,0), cex=cex )
on.exit( par(opar) )
plot( NA, bty="n",
      xlim=0:1*100, ylim=0:1*100, xaxt="n", yaxt="n", xlab="", ylab="" )

# Exercise the subsets by putting the relevant colors to "transparent"
obj$Boxes[-sub.st,c("col.txt",
                    "col.border",
                    "col.bg")] <- "transparent"
obj$Arrows[-sub.tr,c("col.arr",
                     "col.txt.arr")] <- "transparent"
# Then draw the boxes
b <- list()
for( i in 1:n.st ) b[[i]] <- with( obj$Boxes,
                    tbox( obj$State.names[i], xx[i], yy[i], wd[i], ht[i],
                                font=font[i],
                                  lwd=lwd[i],
                          col.txt=col.txt[i],
                    col.border=col.border[i],
                            col.bg=col.bg[i] ) )

# and the arrows
for (i in 1:n.st ) for( j in 1:n.st )
    {
 if (!is.na(obj$Tmat[i,j]) & i!=j )
    {
    a <- wh.no( obj$Tmat, i, j )
    arr <- with( obj$Arrows,
           boxarr( b[[i]], b[[j]],
                   offset=(!is.na(obj$Tmat[i,j]))*offset.arr[a],
                   lwd=lwd.arr[a],
                   col=col.arr[a],
                   pos=pos.arr[a], ... ) )
    with( obj$Arrows,
    text( arr$x-arr$d[2], arr$y+arr$d[1], obj$Arrowtext[a],
          adj=as.numeric(c(arr$d[2]>0,arr$d[1]<0)),
         font=font.arr[a],
          col=col.txt.arr[a] ) )
    }
   }
# Redraw the boxes with "bg" background to remove any arrows crossing
for( i in sub.st ) with( obj$Boxes,
           tbox( obj$State.names[i], xx[i], yy[i], wd[i], ht[i],
                       font=font[i],
                         lwd=lwd[i],
                     col.txt=par("bg"),
                  col.border=par("bg"),
                      col.bg=par("bg") ) )
# Then redraw the boxes again
for( i in sub.st ) with( obj$Boxes,
                   tbox( obj$State.names[i], xx[i], yy[i], wd[i], ht[i],
                               font=font[i],
                                 lwd=lwd[i],
                         col.txt=col.txt[i],
                   col.border=col.border[i],
                           col.bg=col.bg[i] ) )
# Done!
invisible( NULL )
}

 doCutLexis <- function(data, cut, timescale, new.scale=FALSE ) {

    ## new.scale is a character constant with the name of the new timescale

    ## Code each new interval using new variable lex.cut:
    ## 0 = unchanged interval (cut occurs after exit)
    ## 1 = first part of split interval
    ## 2 = second part of split interval (or cut occurs before interval)

    cut[is.na(cut)] <- Inf #If a cut time is missing, it never happens

    ## First intervals (before the cut)
    in.1 <-          entry(data, timescale)
    ex.1 <- pmin(cut, exit(data, timescale))

    ## Create Lexis object for first intervals
    lx.1 <- data
    lx.1$lex.dur <- ex.1 - in.1
    lx.1$lex.cut <- ifelse(cut < exit(data, timescale), 1, 0)
    if( new.scale ) lx.1[,"lex.new.scale"] <- NA

    ## Second intervals (after the cut)
    in.2 <- pmax(cut, entry(data, timescale))
    ex.2 <-            exit(data, timescale)

    ## Create Lexis object for second intervals
    lx.2 <- data
    lx.2$lex.dur <- ex.2 - in.2
    lx.2$lex.cut <- 2
    if( new.scale ) lx.2[,"lex.new.scale"] <- in.2 - cut

    ## Update entry times
    lx.2[, timeScales(data)] <- exit(data) - lx.2$lex.dur

    return(rbind(lx.1, lx.2))
}

setStatus.default <- function(data, new.state)
{
    data$lex.Xst[data$lex.cut == 1] <- new.state[data$lex.cut == 1]
    data$lex.Cst[data$lex.cut == 2] <- new.state

    return(data)
}

setStatus.numeric <- function(data, new.state, precursor.states=NULL,
                              progressive=TRUE) {

    if (!is.numeric(new.state)) {
        stop("If lex.Cst, lex.Xst are numeric, new.state must be numeric too")
    }

    data$lex.Xst[data$lex.cut == 1] <- new.state[data$lex.cut == 1]
    data$lex.Cst[data$lex.cut == 2] <- new.state

    exit.state <- data$lex.Xst[data$lex.cut == 2]
    is.precursor <- exit.state %in% precursor.states
    if (progressive) {
        is.precursor <- is.precursor | (exit.state < new.state)
    }
    data$lex.Xst[data$lex.cut == 2][is.precursor] <- new.state[is.precursor]

    return(data)
}

setStatus.factor <-
function( data,
          new.state,
          precursor.states=NULL,
          progressive=TRUE)
{
    if (!is.character(new.state)) {
        stop("new.state must be a character vector, but it is ",str(new.state))
    }

    current.states <- levels(data$lex.Cst)
    new.states <- setdiff(new.state,current.states)
    new.states <- new.states[!is.na(new.states)]

    ## Modify factor levels if necessary
    if (length(new.states) > 0) {
       all.states <- c(current.states, sort(new.states))
       new.order <- match( c(intersect(precursor.states,current.states),
                             new.states,
                             setdiff(current.states,precursor.states)),
                           all.states )
       levels(data$lex.Cst) <- all.states
       levels(data$lex.Xst) <- all.states
       }

    data$lex.Xst[data$lex.cut == 1] <- new.state[data$lex.cut == 1]
    data$lex.Cst[data$lex.cut == 2] <- new.state

    exit.state <- data$lex.Xst[data$lex.cut==2]
    is.precursor <-  exit.state %in% precursor.states
    if (progressive) {
        if (is.ordered(data$lex.Xst)) {
            is.precursor <- is.precursor | (exit.state < new.state)
        }
        else {
            warning("progressive=TRUE argument ignored for unordered factor")
        }
    }
    data$lex.Xst[data$lex.cut==2][is.precursor] <- new.state[is.precursor]

    # Reorder factor levels sensibly
    if (!progressive & length(new.states)>0){
       data$lex.Cst <- Relevel( data$lex.Cst, new.order )
       data$lex.Xst <- Relevel( data$lex.Xst, new.order )
       }

    return(data)
}

# Added by BxC
match.cut <-
function(data, cut, ts)
{
if(sum(!is.na(match(c("lex.id", "cut", "new.state"), names(cut)))) < 3 )
   stop("The dataframe supplied in the cut= argument must have columns",
        "'lex.id', 'cut', 'new.state', but the columns are:\n", names(cut))
else
   {
   if( length(unique(cut$lex.id)) < nrow(cut) )
     stop("Values of 'lex.id' must be unique in the 'cut' dataframe\n",
          "- maybe you are looking for rcutLexis?")
   else
     # added April 2021 if cut$cut exactly matches a time in data move
     # the cut date a bit earlier in time
     # first, exact mathes are put in wh
     wh <- merge(cut,
                 data[, c("lex.id", ts)],
                 by.x = c("lex.id", "cut"),
                 by.y = c("lex.id", ts),
                  all = FALSE) # ask for an inner join (the default)
     if(nrow(wh) > 0) cut[cut$lex.id %in% wh$lex.id, "cut"] <-
                      cut[cut$lex.id %in% wh$lex.id, "cut"] -
                      min(data$lex.dur) / 100
     # end added stuff
     zz <- merge(data[, "lex.id", drop=FALSE], cut, all.x=TRUE)
     if (is.factor(data$lex.Cst))
        zz$new.state <- as.character(zz$new.state)
     if (is.numeric(data$lex.Cst))
        zz$new.state <- as.numeric(zz$new.state)
     return(zz)
   }
}
# End of addition / change

cutLexis <- function(data,
                     cut,
                     timescale = 1,
                     new.state = nlevels(data$lex.Cst)+1,
                     new.scale = FALSE,
                     split.states = FALSE,
                     progressive = FALSE,
                     precursor.states = transient(data),
                     count = FALSE)
{
    if (!inherits(data, "Lexis"))
      stop("data must be a Lexis object")

    if( count )
      return( countLexis( data=data, cut=cut, timescale=timescale ) )

    # Added April 2021, BxC
    if (inherits(cut, "data.frame")){
    # The merge in match.cut returns data frame ordered by id, so
    # the input must be sorted too (added March 2024)
      data <- sortLexis(data)

      zz <- match.cut(data,
                      cut,
                      check.time.scale(data, timescale))
      cut <- zz$cut
      new.state <- zz$new.state
      }
    else if (length(cut) == 1) {
      cut <- rep(cut, nrow(data))
      }
    else if (length(cut) != nrow(data)) {
        stop("'cut' must have length 1 or nrow(data) (=", nrow(data),
             "),\n --- but it has length ", length(cut),".")
      }

    timescale <- check.time.scale(data, timescale)
    if (length(timescale) > 1) {
        stop("Multiple time scales not allowed")
    }

    ## If we want to add a new timescale, construct the name
    if( is.logical(new.scale) ) {
      if( new.scale ) scale.name <- paste( "tf", new.state[1], sep="" )
      }
    else {
      scale.name <- new.scale
      new.scale <- TRUE
         }

    if (missing(new.state)) {
        new.state <- data$lex.Cst       #Carry forward last state
       }
    else if (length(new.state) == 1) {
        new.state <- rep(new.state, nrow(data))
       }
    else if (length(new.state) != nrow(data)) {
        stop("'new.state' must have length 1 or nrow(data) (=", nrow(data),
             "),\n --- but it has length ", length(new.state))
       }

    if (progressive) {
        if (is.factor(data$lex.Cst) && !is.ordered(data$lex.Cst)) {
            stop("progressive=TRUE invalid for unordered factors")
        }
        if (any(data$lex.Xst < data$lex.Cst)) {
            stop("Lexis object is not progressive before splitting")
        }
    }

    lx <- doCutLexis( data, cut, timescale, new.scale=TRUE )
    if (is.factor(data$lex.Cst)) {
        lx <- setStatus.factor(lx, new.state, precursor.states, progressive)
      }
    else if (is.numeric(data$lex.Cst)) {
        lx <- setStatus.numeric(lx, new.state, precursor.states, progressive)
      }
    else {
        lx <- setStatus.default(lx, new.state)
      }

    ## Remove redundant intervals
    lx <- lx[lx$lex.dur > 0,]
    ## Remove the lex.cut column
    lx <- lx[,-match("lex.cut",names(lx))]
    ## Update the states visited after the cut
    if( split.states & is.factor( data$lex.Cst ) )
      {
      post.states <- setdiff( levels(data$lex.Cst), precursor.states )
      tmp.Cst <- as.character( lx$lex.Cst )
      tmp.Cst <- ifelse( !is.na(lx$lex.new.scale)  &
                                lx$lex.new.scale>0 &
                         tmp.Cst %in% post.states,
                         paste( tmp.Cst,"(",new.state,")",sep="" ),
                         tmp.Cst )
      tmp.Xst <- as.character( lx$lex.Xst )
      tmp.Xst <- ifelse( !is.na(lx$lex.new.scale) &
                         tmp.Xst %in% post.states,
                         paste( tmp.Xst,"(",new.state,")",sep="" ),
                         tmp.Xst )
      all.levels <- unique( c(tmp.Cst,tmp.Xst) )
      ## put all the new levels after the old ones
      xtr.levels <- setdiff( all.levels, levels(lx$lex.Cst) )
      new.levels <- c( levels(lx$lex.Cst), xtr.levels )
      lx$lex.Cst <- factor( tmp.Cst, levels=new.levels )
      lx$lex.Xst <- factor( tmp.Xst, levels=new.levels )
      }
    ## Include the new timescale
    if( new.scale )
      {
      ## Rename the new timescale variable
      names(lx)[match("lex.new.scale",names(lx))] <- scale.name
      ## The timescales' position among columns - used to reorder columns
      tn <- c( match( attr( data, "time.scales" ), names( lx ) ),
               ncol(lx) )
      oth <- setdiff( 1:ncol(lx), tn )
      ## Reorder columns (lx will then lose attributes) and sort rows
      lx <- lx[order(lx$lex.id,lx[,timescale]),c(tn,oth)]
      ## Update the attributes
      new.br <- c( attr( data, "breaks" ), list(NULL) )
      names( new.br )[length(new.br)] <- scale.name
      attr( lx, "time.scales" ) <- c( attr( data, "time.scales" ), scale.name )
      attr( lx, "time.since" )  <- c( attr( data, "time.since" ), names(table(new.state)) )
      attr( lx, "breaks" )      <- new.br
      attr( lx, "class" )       <- attr( data, "class" )
      }
    else
      {
      # Remove the new timescale and sort rows
      lx <- lx[order(lx$lex.id,lx[,timescale]),-match("lex.new.scale",names(lx))]
      # and transfer all the other attributes
      attr( lx, "time.scales" ) <- attr( data, "time.scales" )
      attr( lx, "time.since" )  <- attr( data, "time.since" )
      attr( lx, "breaks" )      <- attr( data, "breaks" )
      attr( lx, "class" )       <- attr( data, "class" )
      }
    sortLexis(lx)
}

countLexis <- function(data, cut, timescale = 1)
{
    if (!inherits(data, "Lexis"))
      stop("data must be a Lexis object")

    if( inherits( cut, "data.frame" ) ){
      zz <- match.cut( data, cut )
      cut <- zz$cut
      new.state <- zz$new.state
      }
    else if (length(cut) == 1) {
        cut <- rep(cut, nrow(data))
      }
    else if (length(cut) != nrow(data)) {
        stop("'cut' must have length 1 or nrow(data) (=", nrow(data),
             "),\n --- but it has length ", length(cut),".")
      }

    timescale <- check.time.scale(data, timescale)
    if (length(timescale) > 1) {
        stop("Multiple time scales not meaningful")
    }

    lx <- doCutLexis(data, cut, timescale)

    ## Update status variables
    lx$lex.Xst[lx$lex.cut == 1] <- lx$lex.Cst[lx$lex.cut == 1] + 1
    lx$lex.Cst[lx$lex.cut == 2] <- lx$lex.Cst[lx$lex.cut == 2] + 1
    lx$lex.Xst[lx$lex.cut == 2] <- lx$lex.Xst[lx$lex.cut == 2] + 1
    ## Remove redundant intervals
    lx <- lx[lx$lex.dur > 0,]
    ## Remove the lex.cut column
    lx <- lx[,-match("lex.cut",names(lx))]
    ## Retain the attributes
    attr( lx, "breaks" )      <- attr( data, "breaks" )
    attr( lx, "time.scales" ) <- attr( data, "time.scales" )
    attr( lx, "class" )       <- attr( data, "class" )

    return(lx[order(lx$lex.id,lx[,timescale]),])
}

tsNA20 <- function(x, all.scales = FALSE)
    {
    # names of timescales defined by state entry
    tsc <- timeSince(x)
    if (!all.scales) tsc <- tsc[tsc != ""]
    if (length(tsc) > 0)
       for (ts in names(tsc))
           {
           x[,ts] <- ifelse(is.na(x[,ts]),
                            0,
                            x[,ts])
           }
    return(x)
    }

plotCIF <- 
function (x,
      event = 1,
       xlab = "Time",
       ylab = "Cumulative incidence", 
       ylim = c(0, 1),
        lty = 1,
        col = "black",
        ... ) 
{
    ng <- length(x$n)
    co <- rep(col, ng)[1:ng]
    lt <- rep(lty, ng)[1:ng]
    ne <- dim(x$pstate)[2] - 1
    if (event %in% 1:ne) {
        time <- x$time
        gr <- rep(1, length(time))
        if (ng > 1) 
            for (g in 2:ng) for (t in (cumsum(x$strata)[g - 1] + 
                1):cumsum(x$strata)[g]) gr[t] <- g
        CI <- x$pstate[, 2]
        for (e in 1:ne) if (event == e) 
            CI <- x$pstate[, e+1]
        plot(c(0, time[gr == 1], max(time[gr == 1])), c(0, CI[gr == 
            1], max(CI[gr == 1])), type = "s", ylim = ylim, xlab = xlab, 
            ylab = ylab, col = co[1], lty = lt[1], ...)
        if (ng > 1) 
            for (g in 2:ng) lines(c(0, time[gr == g], max(time[gr == 
                g])), c(0, CI[gr == g], max(CI[gr == g])), type = "s", 
                lty = lt[g], col = co[g], ...)
    }
    else print(paste("Error: event must be an integer from 1 to", 
        ne))
}

splitMatrix <- function (x, f, drop=FALSE) {
    lapply(split(seq_len(nrow(x)), f, drop = drop),
           function(ind) x[ind, , drop = FALSE])
}

fixEvent <- function(event)
{
    ### Convert outcome in clogit model to 0/1 binary coding
    
    if (any(is.na(event)))
        stop("Event contains missing values")

    if (is.logical(event)) {
        status <- is.numeric(event)
    }
    else if (is.numeric(event)) {
        status <- if (max(event) == 2) event - 1  else event
        temp <- (status == 0 | status == 1)
        if (!all(temp)) {
            warning("If outcome is numeric then it must be coded 0/1 or 1/2")
        }
    }
    else if (is.factor(event)) {
        if (nlevels(event) != 2)
            stop("If outcome is a factor then it must have 2 levels")
        status <- event == levels(event)[2]
    }
    return(as.integer(status))
}  

isInformative <- function(Xsplit, ysplit, strata)
{
    ## Identify which observations are informative in a conditional
    ## logistic regression.
    
    is.homogeneous <- function(x) { all(x==x[1]) }
    y.bad <- sapply(ysplit, is.homogeneous)
    X.bad <- sapply(Xsplit, function(x) { all(apply(x, 2, is.homogeneous)) })

    is.informative <- vector("list", length(ysplit))
    for (i in seq(along=is.informative)) {
        canuse <- (!y.bad[i]) && (!X.bad[i])
        is.informative[[i]] <- rep(canuse, length(ysplit[[i]]))
    }
    return(unsplit(is.informative, strata))
}
        
fitClogit <- function(X, y, offset, strata, init, iter.max, eps, toler.chol)
{
    ## Safe wrapper around the C function "clogit" that ensures all
    ## arguments have the correct type and storage mode.
    
    y <- fixEvent(y)
    if (!is.matrix(X)) {
        X <- as.matrix(X)
    }
    if (!is.double(X)) {
        X <- matrix(as.double(X), nrow(X), ncol(X))
    }
    if (is.null(offset)) {
        offset <- rep(0, nrow(X))
    }
    offset <- as.double(offset);
    
    ## Split into strata
    Xsplit <- splitMatrix(X, strata, drop=TRUE)
    ysplit <- split(y, strata, drop=TRUE)
    osplit <- split(offset, strata, drop=TRUE)

    info <- isInformative(Xsplit, ysplit, strata)
    if (!any(info)) {
        stop("There are no informative observations")
    }
    
    ans <- .Call(C_clogit, Xsplit, ysplit, osplit, as.double(init),
                 as.integer(iter.max), as.double(eps), as.double(toler.chol))
    ans$informative <- info
    return(ans)
}

clogistic <- function (formula, strata, data, subset, na.action,
                       init, model = TRUE, x = FALSE, y = TRUE,
                       contrasts = NULL, iter.max=20, eps=1e-6,
                       toler.chol = sqrt(.Machine$double.eps)) 
{
    ## User interface, edited version of glm
    
    call <- match.call()
    if (missing(data)) 
        data <- environment(formula)
    mf <- match.call(expand.dots = FALSE)
    m <- match(c("formula", "data", "subset",
                 "na.action", "offset", "strata"), names(mf), 0L)
    mf <- mf[c(1, m)]
    mf$drop.unused.levels <- TRUE
    mf[[1L]] <- as.name("model.frame")
    mf <- eval(mf, parent.frame())

    mt <- attr(mf, "terms")
    Y <- model.response(mf, "any")
    if (is.null(Y)) stop("missing outcome")
    if (length(dim(Y)) == 1L) {
        nm <- rownames(Y)
        dim(Y) <- NULL
        if (!is.null(nm)) 
            names(Y) <- nm
    }
    X <- if (!is.empty.model(mt)) 
        model.matrix(mt, mf, contrasts)
    else stop("Invalid model matrix")
    offset <- as.vector(model.offset(mf))
    if (!is.null(offset)) {
        if (length(offset) != NROW(Y)) 
            stop(gettextf("number of offsets is %d should equal %d (number of observations)", 
                length(offset), NROW(Y)), domain = NA)
    }

    strata <- model.extract(mf, "strata")
    if (is.null(strata)) stop("argument 'strata' missing")
    
    contrasts <- attr(X, "contrasts")
    if (attr(mt, "intercept") > 0) {
        X <- X[,-1, drop=FALSE]
    }
    if (missing(init))
        init <- rep(0, ncol(X))

    if (iter.max < 0)
        stop("'iter.max' must be non-negative")
    if (eps <= 0)
        stop("'eps' must be positive")
    if (toler.chol <= 0)
        stop("'toler.chol' must be positive")
    if (eps < toler.chol)
        stop("'toler.chol' must be smaller than 'eps'")
    
    fit <- fitClogit(X = X, y = Y, offset = offset, strata=strata, init=init,
                     toler.chol=toler.chol, eps=eps, iter.max=iter.max)
    if (fit$flag <= 0) {
        stop("Information matrix is not positive definite")
    }
    else if (fit$flag == 1000) {
        warning("Iteration limit exceeded")
    }

    nvar <- length(init)
    which.sing <- if (fit$flag < nvar) {
        diag(fit$var)==0
    } else {
        rep(FALSE, nvar)
    }
    fit$coefficients[which.sing] <- NA
    fit$flag <- NULL
    
    ## Add back in parameter names
    cfnames <- colnames(X)
    names(fit$coefficients) <- cfnames
    dimnames(fit$var) <- list(cfnames, cfnames)

    fit$n <- sum(fit$informative)
    if (model) {
        fit$model <- mf
    }
    else {
        ## Without model frame this cannot be interpreted
        fit$informative <- NULL
    }
    fit$na.action <- attr(mf, "na.action")
    if (x) 
        fit$x <- X
    if (!y) 
        fit$y <- NULL
    fit <- c(fit, list(call = call, formula = formula, terms = mt, 
                       contrasts = contrasts, xlevels = .getXlevels(mt, mf)))
    class(fit) <- c("clogistic")
    fit
}

coef.clogistic <- function(object,...) { object$coefficients }

vcov.clogistic <- function(object, ...) { object$var }

print.clogistic <- function (x, digits = max(options()$digits - 4, 3), ...) 
{
    ## Print method for clogistic objects, edited from print.coxph
    
    cat("\nCall: ", deparse(x$call), "\n\n", sep="\n")

    savedig <- options(digits = digits)
    on.exit(options(savedig))
    coef <- coef.clogistic(x)
    se <- sqrt(diag(vcov.clogistic(x)))
    if (is.null(coef) | is.null(se)) 
        stop("Input is not valid")

    coefmat <- cbind(coef, exp(coef), se, coef/se,
                     signif(1 - pchisq((coef/se)^2, 1), digits - 1))
    dimnames(coefmat) <- list(names(coef),
                              c("coef", "exp(coef)", "se(coef)", "z", "p"))
    cat("\n")
    prmatrix(coefmat)
    logtest <- -2 * (x$loglik[1] - x$loglik[2])
    if (is.null(x$df)) 
        df <- sum(!is.na(coef))
    else df <- round(sum(x$df), 2)
    cat("\n")
    cat("Likelihood ratio test=", format(round(logtest, 2)), 
        "  on ", df, " df,", " p=", format(1 - pchisq(logtest, df)),
        ", n=", x$n, sep = "")
    cat("\n")
    invisible()
}

ccwc <-
function(entry=0, exit, fail, origin=0, controls=1, match=list(),
         include=list(), data=NULL, silent=FALSE)
{
# Check arguments
  entry <- eval(substitute(entry), data)
  exit <- eval(substitute(exit), data)
  fail <- eval(substitute(fail), data)
  origin <- eval(substitute(origin), data)

  n <- length(fail)
  if (length(exit)!=n)
    stop("All vectors must have same length")
  if (length(entry)!=1 && length(entry)!=n)
    stop("All vectors must have same length")
  if (length(origin)==1) {
    origin <- rep(origin, n)
  }
  else {
    if (length(origin)!=n)
      stop("All vectors must have same length")
  }
# Transform times to correct scale
  t.entry <- as.numeric(entry - origin)
  t.exit <- as.numeric(exit - origin)
# match= argument
  marg <- substitute(match)
  if (mode(marg)=="name") {
    match <- list(eval(marg, data))
    names(match) <- as.character(marg)
  }
  else if (mode(marg)=="call" && marg[[1]]=="list") {
    mnames <- names(marg)
    nm <- length(marg)
    if (nm>1) {
      if (!is.null(mnames)) {
        for (i in 2:nm) {
          if (mode(marg[[i]])=="name")
            mnames[i] <- as.character(marg[[i]])
          else
            stop("illegal argument (match)")
        }
      }
      else {
        for (i in 2:nm) {
          if (mode(marg[[i]])=="name")
            mnames[i] <- as.character(marg[[i]])
          else
            stop("illegal argument (match)")
        }
        mnames[1] <= ""
      }
    }
    names(marg) <- mnames
    match <- eval(marg, data)
  }
  else {
    stop("illegal argument (match)")
  }
  m <- length(match)
  mnames <- names(match)
  if (m>0) {
    for (i in 1:m) {
      if (length(match[[i]])!=n) {
        stop("incorrect length for matching variable")
      }
    }
  }
                                        # include= argument
  iarg <- substitute(include)
  if (mode(iarg)=="name") {
    include <- list(eval(iarg, data))
    names(include) <- as.character(iarg)
  }
  else if (mode(iarg)=="call" && iarg[[1]]=="list") {
    ni <- length(iarg)
    inames <- names(iarg)
    if (ni>1) {
      if (!is.null(inames)) {
        for (i in 2:ni) {
          if (mode(iarg[[i]])=="name")
            inames[i] <- as.character(iarg[[i]])
          else
            stop("illegal argument (include)")
        }
      }
      else {
        for (i in 2:ni) {
          if (mode(iarg[[i]])=="name")
            inames[i] <- as.character(iarg[[i]])
          else
            stop("illegal argument (include)")
        }
        inames[1] <= ""
      }
    }
    names(iarg) <- inames
    include <- eval(iarg, data)
  }
  else {
    stop("illegal argument (include)")
  }
  ni <- length(include)
  inames <- names(include)
  if (ni>0) {
    for (i in 1:ni) {
      if (length(include[[i]])!=n) {
        stop("incorrect length for included variable")
      }
    }
  }
                                        # create group codes using matching variables
  grp <- rep(1,n)
  pd <- 1
  if (m>0) {
    for (im in 1:m) {
      v <- match[[im]]
      if (length(v)!=n)
        stop("All vectors must have same length")
      if (!is.factor(v))
        v <- factor(v)
      grp <- grp + pd*(as.numeric(v) - 1)
      pd <- pd*length(levels(v))
    }
  }
                                        # Create vectors long enough to hold results
  nn <- (1+controls)*sum(fail!=0)
  pr <- numeric(nn)
  sr <- numeric(nn)
  tr <- vector("numeric", nn)
  fr <- numeric(nn)
  nn <- 0
                                        # Sample each group
  if (!silent) {
    cat("\nSampling risk sets: ")
  }
  set <- 0
  nomatch <- 0
  incomplete <- 0
  ties <- FALSE
  fg <- unique(grp[fail!=0])
  for (g in fg) {
                                        # Failure times
    ft <- unique( t.exit[(grp==g) & (fail!=0)] )
                                        # Create case-control sets
    for (tf in ft) {
      if (!silent) {
        cat(".")
      }
      set <- set+1
      case <- (grp==g) & (t.exit==tf) & (fail!=0)
      ncase <- sum(case)
      if (ncase>1)
        ties <- TRUE
      noncase <- (grp==g) & (t.entry<=tf) &
      (t.exit>=tf) & !case
      ncont <- controls*ncase
      if (ncont>sum(noncase)) {
        ncont <- sum(noncase)
        if (ncont>0) incomplete <- incomplete + 1
      }
      if (ncont>0) {
        newnn <- nn+ncase+ncont
        sr[(nn+1):newnn] <- set
        tr[(nn+1):newnn] <- tf
        fr[(nn+1):(nn+ncase)] <- 1
        fr[(nn+ncase+1):newnn] <- 0
        pr[(nn+1):(nn+ncase)] <- (1:n)[case]
        ## Work around bad behaviour of sample for vectors of length 1
        noncase.id <- (1:n)[noncase]
        pr[(nn+ncase+1):(newnn)] <- noncase.id[sample.int(length(noncase.id),
                                                          size=ncont)]
        nn <- newnn
      }
      else {
        nomatch <- nomatch + ncase
      }
    }
  }
  if (!silent) {
    cat("\n")
  }
  res <- vector("list", 4+m+ni)
  if (nn>0) {
    res[[1]] <- sr[1:nn]
    res[[2]] <- map <- pr[1:nn]
    res[[3]] <- tr[1:nn] + origin[map]
    res[[4]] <- fr[1:nn]
  }
  if (m>0) {
    for (i in 1:m) {
      res[[4+i]] <- match[[i]][map]
    }
  }
  if (ni>0) {
    for (i in 1:ni) {
      res[[4+m+i]] <- include[[i]][map]
    }
  }
  names(res) <- c("Set", "Map", "Time", "Fail", mnames, inames)
  if (incomplete>0)
    warning(paste(incomplete, "case-control sets are incomplete"))
  if (nomatch>0)
    warning(paste(nomatch, "cases could not be matched"))
  if (ties)
    warning("there were tied failure times")
  data.frame(res)
}

crr.Lexis <-
function( obj, mod, quiet=FALSE, ... )
{
# Model formula to be transmitted
md <- mod
# Outcome variable
fc <- as.character( mod[2] )
# Censoring levels
cn <- unique( as.character(obj$lex.Xst)[obj$lex.Xst==obj$lex.Cst] )
# The competing causes (the remaining levels)
cc <- setdiff( levels(obj$lex.Xst), union(fc,cn) )
# No l.h.s. side of formula when deriving model matrix
mod[2] <- NULL
# Remember no intercept term
cv <- model.matrix( mod, data=obj )[,-1]
# Then do it
M <- crr( ftime = obj$lex.dur,
        fstatus = obj$lex.Xst,
       failcode = fc,
        cencode = cn,
           cov1 = cv,
            ... )
# A table of the no. of transitions
N <- with( obj, table( Relevel( lex.Xst, c(fc,cc,cn) ) ) )
names( N ) <- paste( rep( c("Event:"," comp:"," cens:"),
                          c(length(fc),length(cc),length(cn)) ),
                      names(N) )
# add model and table to the resulting object
M <- c( M, list( model.Lexis = md,
                 transitions = cbind(N) ) )
# remember the class attribute (lost by doing "c")
class( M )  <- "crr"
# print overview if desired
if( !quiet )
cat(   "crr analysis of event", paste('"',fc,'"',sep=''),
     "\n               versus", paste('"',cc,'"',sep=''),
     "\n                 with", paste('"',cn,'"',sep=''),
             "as censoring.\n" )
M
}

Icens <- function(first.well, last.well, first.ill, formula,
                  model.type=c("MRR","AER"), breaks,
                  boot=FALSE, alpha=0.05, keep.sample=FALSE, data)
{
  ## Create follow-up matrix containing three event times
  fu.expression <- substitute(cbind(first.well, last.well, first.ill))
  fu <- if (missing(data)) {
    eval(fu.expression)
  }
  else {
    eval(fu.expression, data)
  }
  ## Check consistency of arguments
  missing.f1 <- is.na(fu[,1])
  missing.f2 <- is.na(fu[,2])
  if (any(missing.f1 & missing.f2)) {
    stop("You must supply at least one of \"first.well\" and \"last.well\"")
  }
  if (any(fu[,1] > fu[,2], na.rm=TRUE) | any(fu[,2] > fu[,3], na.rm=TRUE)) {
    stop("Some units do not meet: first.well < last.well < first.ill" )
  }
  ## Fill in any gaps
  fu[,1][missing.f1] <- fu[,2][missing.f1]
  fu[,2][missing.f2] <- fu[,1][missing.f2]
  ## Recensor cases that fall after the last break point
  is.censored <- fu[,3] > max(breaks)
  is.censored[is.na(is.censored)] <- FALSE
  fu[is.censored,3] <- NA
  
  exp.dat <- expand.data(fu, formula, breaks, data)

  model.type <- match.arg(model.type)
  if (missing(formula)) {
    fit.out <- with(exp.dat, fit.baseline(y, rates.frame))
    lambda <- coef(fit.out)
  }
  else {
    fit.out <- switch(model.type,
                      "MRR"=with(exp.dat, fit.mult(y, rates.frame, cov.frame)),
                      "AER"=with(exp.dat, fit.add(y, rates.frame, cov.frame)))
    lambda <- coef(fit.out$rates)
  }
  
  beta <- if (is.null(fit.out$cov)) numeric(0) else coef(fit.out$cov)
  params <- c(lambda,beta)
  if (boot) {
    nboot <- ifelse (is.numeric(boot), boot, 100)
    boot.coef <- matrix(NA, nrow=nboot, ncol=length(lambda) + length(beta))
    colnames(boot.coef) <- names(params)
    
    for (i in 1:nboot) {
      subsample <- sample(nrow(fu), replace=TRUE)
      exp.dat <- expand.data(fu[subsample,], formula, breaks, data[subsample,])
      if (missing(formula)) {
        sim.out <- with(exp.dat, fit.baseline(y, rates.frame, params))
        boot.coef[i,] <- coef(sim.out)
      }
      else {
        sim.out <- switch(model.type,
                          "MRR"=with(exp.dat,
                            fit.mult(y, rates.frame, cov.frame, params)),
                          "AER"=with(exp.dat,
                            fit.add(y, rates.frame, cov.frame, params)))
        boot.coef[i,] <- switch(model.type,
                                "MRR"=c(coef(sim.out[[1]]),
                                  coef(sim.out[[2]])),
                                "AER"=coef(sim.out[[1]]))
      }
    }
    ci.quantiles=c(0.5, alpha/2, 1 - alpha/2)
    boot.ci <- t(apply(boot.coef,2,quantile,ci.quantiles))

    lower.ci.lab <-
      paste("lower ", formatC(100 * alpha/2, format="f", digits=1),"%", sep="")
    upper.ci.lab <- 
      paste("upper ", formatC(100 * (1-alpha/2), format="f", digits=1),"%",
            sep="")
    
    colnames(boot.ci) <- c("median", lower.ci.lab, upper.ci.lab)
    fit.out$boot.ci <- boot.ci
    if (keep.sample) {
      fit.out$sample <- boot.coef
    }
  }
  
  class( fit.out ) <- "Icens"
  attr( fit.out, "model" ) <- model.type
  return( fit.out )
}

pctab <- function( TT, margin=length( dim( TT ) ), dec=1 )
  {
  nd <- length( dim( TT ) )
  sw <- (1:nd)[-margin[1]]
  rt <-
  sweep( addmargins( TT,
                     margin,
                     list( list( All=sum,
                                   N=function( x ) sum( x )^2/100 ) ) ),
         sw,
         apply( TT,
                sw,
                sum )/100,
         "/" )
  if( dec ) print( round( rt, dec ) )
  invisible( rt )
  }

matshade <-
function( x, y,
          lty = 1,
          col = 1:(ncol(y)/3),
    col.shade = col,
        alpha = 0.15,
         plot = dev.cur()==1,
          ... )
    {
# check sanity of x and y
if( !is.vector(x) ) stop( "x must be a vector\n")        
if( ncol(y)%%3 != 0 ) warning( "number of columns in y, ", ncol(y),
                               " is not divisible by 3\n")
# create a new plot?
if( plot ) matplot( x, y, col="transparent", ... )
# number of curves to draw        
ncrv <- ncol(y) %/% 3
# The recycling rule for colors and alpha:
col       <- rep(col      ,ncrv)[1:ncrv]
col.shade <- rep(col.shade,ncrv)[1:ncrv]
alpha     <- rep(alpha    ,ncrv)[1:ncrv]
# First shaded areas - allowing NA to create holes
draw.shade <-
function( xx, yu, yl, col )
{
# Find NAs and plot shades for non-missing sequences
wh <- 1:length(xx)
# NAs where at least one is missing
wh[apply( cbind(xx,yl,yu), 1, function(x) any(is.na(x)) )] <- NA
idx <- 1 + cumsum( is.na( wh ) )
not.na <- !is.na( wh )
chunks <- split( wh[not.na], idx[not.na] )
for( ch in chunks )
if( length(ch) > 1 ) polygon( c( xx[ch], rev(xx[ch]) ),    
                              c( yu[ch], rev(yl[ch]) ),    
                              col = col,           
                              pch = NULL,          
                           border = "transparent" )
}
# Then the shades for each set of 3 columns
for( i in 1:ncrv ) draw.shade( x, y[,i*3-1],
                                  y[,i*3-0], 
                               col = adjustcolor( col.shade[i],
                                              alpha.f=alpha[i] ) )
# then curves on top of these
matlines( x, y[,(1:ncrv)*3-2], col=col, lty=lty, ... )
    }
       

"print.floated" <-
  function(x, digits=max(3, getOption("digits") - 3), level = 0.95, ...)
{
  K <- qnorm((1+level)/2)
    
  n <- length(x$coef)
  mat <- matrix("", n, 4)
  ci.mat <- matrix(0, n, 2)

  cm <- x$coefmat
  cat("Floating treatment contrasts for factor ", x$factor, "\n\n")
  mat[,1] <- names(x$coef)
  se <- sqrt(x$var)
  ci.mat[, 1] <- x$coef - K * se
  ci.mat[, 2] <- x$coef + K * se
    
  mat[,2] <- format(x$coef, digits=digits)
  mat[,3]   <- format(se, digits=digits)
  ci.mat <- format(ci.mat, digits=digits)
  mat[,4] <- paste("(", ci.mat[,1], ",", ci.mat[,2], ")", sep="")

  dimnames(mat) <- list(rep("", n), c("Level", "Coefficient",
                                      "Std. Error", "95% Floating CI"))
  print(mat, quote=FALSE)
  cat("\nError limits over all contrasts: ", 
      paste(format(c(0.99, x$limits),
                   digits=2)[-1], collapse=","),"\n")
}

Wald <-
function( obj, H0=0, ... )
{
rl <- ci.lin( obj, ..., vcov=TRUE )
beta <- rl$coef
vcov <- rl$vcov
if( missing( H0 ) ) H0 <- beta*0
if( length(H0) != length(beta) ) stop( "H0 has length ", length(H0),
          " but the set of selected parameters has length ",
          length(beta), ":\n",
          paste(round(beta,options()[["digits"]]),collapse=" ") )
chi <- t( beta-H0 ) %*% solve( vcov, beta-H0 )
 df <- length(beta)
  p <- 1 - pchisq( chi, df )
c( "Chisq"=chi, "d.f."=df, "P"=p )
}

# The msdata method
msdata <- function (obj, ...) UseMethod("msdata")

msdata.Lexis <-
function(obj, time.scale = timeScales(obj)[1], ...)
{
tr.mat <- tmat(obj)
# Essentially a msdata object is a stacked Lexis object with
# other variable names and a few attributes
tmp <- stack.Lexis(factorize.Lexis(obj))
lexvars <- c(match(timeScales(obj), names(tmp)),
             grep("lex\\.", names(tmp)))
# The transitions that we refer to are extracted from lex.Tr:
ss <- strsplit(as.character(tmp$lex.Tr), "->")
st <- levels(obj)
# The resulting dataframe is created by renaming columns in the
# stacked Lexis object and naming states by integers(!).
msd <- data.frame(id = tmp$lex.id,
                from = factor(sapply(ss, FUN = function(x) x[1]),
                              levels = st,
                              labels = 1:length(st)),
                  to = factor(sapply(ss, FUN = function(x) x[2]),
                              levels = st,
                              labels = 1:length(st)),
               trans = as.integer(tmp$lex.Tr),
              Tstart = tmp[,time.scale],
               Tstop = tmp[,time.scale] + tmp$lex.dur,
                time = tmp$lex.dur,
              status = as.integer(tmp$lex.Fail),
                       tmp[,-lexvars])
class(msd) <- c("msdata", "data.frame")
attr(msd, "trans") <- tr.mat
msd
}

# The etm method
etm <- function (data, ...) UseMethod("etm")

etm.Lexis <-
function( data,
    time.scale = timeScales(data)[1],
     cens.name = "cens",
             s = 0,
             t = "last",
    covariance = TRUE,
      delta.na = TRUE,
           ...
          )
{
dfr <- data.frame( id = data$lex.id,
                 from = as.character(data$lex.Cst),
                   to = as.character(data$lex.Xst),
                entry = data[,time.scale],
                 exit = data[,time.scale] + data$lex.dur,
     stringsAsFactors = FALSE )
dfr$to <- with( dfr, ifelse( from==to, cens.name, to ) )
etm::etm( data = dfr,
   state.names = levels(data$lex.Cst),
           tra = tmat(data, mode = "logical"),
     cens.name = cens.name,
             s = s,
             t = t,
    covariance = covariance,
      delta.na = delta.na )
}

apc.frame <-
function( a.lab,
         cp.lab,
          r.lab,
         rr.lab = r.lab / rr.ref,
         rr.ref = r.lab[length(r.lab)/2],
          a.tic = a.lab,
         cp.tic = cp.lab,
          r.tic = r.lab,
         rr.tic = r.tic / rr.ref,
        tic.fac = 1.3,
          a.txt = "Age",
         cp.txt = "Calendar time",
          r.txt = "Rate per 100,000 person-years",
         rr.txt = "Rate ratio",
       ref.line = TRUE,
            gap = diff(range(c(a.lab,a.tic)))/10,
       col.grid = gray( 0.85 ),
          sides = c(1,2,4) )
  {
cp  <- min( c(cp.tic,cp.lab) ) - max( c(a.lab,a.tic) ) - gap
xl  <- c(min( c(a.lab,a.tic) ),
         max( c(a.lab,a.tic) ) + gap + diff( range( c(cp.lab,cp.tic) ) ) )
yl  <- range( c(r.lab,r.tic) )
rrtck <- outer( c(0.5,1,1.5,2:9), 10^(-5:5), "*" )
# Empty plot frame
plot( NA,
      xlab="", xlim=xl, xaxt="n", xaxs="i",
      ylab="", ylim=yl, yaxt="n", yaxs="i", log="y" )
# Grid lines
abline( h=c(r.tic,outer( c(0.5,1,1.5,2:9), 10^(-5:5), "*" )),
        v=c(a.tic,cp.tic - cp), col=col.grid )
# Reference line for the RR=1
if ( ref.line )
segments( min(c(cp.lab,cp.tic))-cp, rr.ref,
          max(c(cp.lab,cp.tic))-cp, rr.ref )
# Close it nicely off:
box()
# Axis construction (tickmarks, labels and annotation)
if ( 1 %in% sides )
   {
   axis( side=1, at=a.lab )
   axis( side=1, at=a.tic, labels=NA, tcl=par("tcl")/tic.fac )
   axis( side=1, at=cp.lab - cp, labels=cp.lab )
   axis( side=1, at=cp.tic - cp, labels=NA, tcl=par("tcl")/tic.fac )
   axis( side=1, at=mean( a.lab ), labels=a.txt, line=1, tcl=0 )
   axis( side=1, at=mean( cp.lab - cp ), labels=cp.txt, line=1, tcl=0 )
   }
if ( 2 %in% sides )
   {
   axis( side=2, at=r.lab, labels=paste( r.lab ) )
   axis( side=2, at=r.tic, labels=NA, tcl=par("tcl")/tic.fac )
   mtext( side=2, r.txt, line=2.5, las=0 )
   }
if ( 3 %in% sides )
   {
   axis( side=3, at=a.lab )
   axis( side=3, at=a.tic, labels=NA, tcl=par("tcl")/tic.fac )
   axis( side=3, at=cp.lab - cp, labels=cp.lab )
   axis( side=3, at=cp.tic - cp, labels=NA, tcl=par("tcl")/tic.fac )
   axis( side=3, at=mean( a.lab ), labels=a.txt, line=1, tcl=0 )
   axis( side=3, at=mean( cp.lab - cp ), labels=cp.txt, line=1, tcl=0 )
   }
if ( 4 %in% sides )
   {
   axis( side=4, at=c(rr.ref,rr.lab*rr.ref), labels=paste( c(1,rr.lab) ) )
   axis( side=4, at=c(rr.ref,rr.tic*rr.ref), labels=NA, tcl=par("tcl")/tic.fac )
   mtext( side=4, rr.txt, line=2.5, las=0 )
   }
# Return the offset for the cohort/period and the RR-factor.
options( apc.frame.par = c("cp.offset"=cp,"RR.fac"=rr.ref) )
invisible( c("cp.offset"=cp,"RR.fac"=rr.ref) )
  }

expand.data <-
function(fu, formula, breaks, data)
{
  Nsubject <- nrow(fu)

  ## The model matrix to be merged up to the expanded data
  if (!missing(formula)) {
    covariate.frame <- data.frame(model.matrix(formula,data)[,-1])
    cov.names <- names(covariate.frame)
    covariate.frame$id <- 1:Nsubject
  }
  else {
    covariate.frame <- NULL
    cov.names <- character(0)
  }
    
  ## 1-responses for the survival intervals
  Nbreak <- length(breaks)
  ILENGTH <- function(x) {
    pmax(pmin(x[2], breaks[-1]) - pmax(x[1], breaks[-Nbreak]), 0)
  }
  well.mat <- -matrix(apply(fu[,c(1,2), drop=FALSE], 1, ILENGTH),
                      nrow=Nbreak-1)
  
  ## For the sake of the stability of the fitting procedure, each record
  ## with a 1-response is further split into separate records for each
  ## follow-up interval:  
  id.vec <- c(diag(Nbreak-1))
  well.mat <- matrix(apply(well.mat, 2, "*", id.vec), nrow=Nbreak-1)
  well.id <- rep(1:Nsubject, each=Nbreak-1)
  valid.cols <- apply(well.mat!=0, 2, any)
  well.mat <- well.mat[,valid.cols, drop=FALSE] #Remove cols that are all zero
  well.id <- well.id[valid.cols]
  
  ## 0-responses for the event intervals
  is.case <- !is.na(fu[,3])             # Observed to become ill
  fu.cases <- subset(fu, is.case)
  ill.mat <- -matrix(apply(fu.cases[,c(2,3), drop=FALSE], 1, ILENGTH),
                     nrow=Nbreak-1)
  ill.id <- which(is.case)
  
  ## The dataframe for analysis is one observation per survival interval
  ## (well.mat) and one per event interval (ill.mat):
  rates.frame <- as.data.frame(t(cbind(well.mat, ill.mat)))
  rates.names <- paste("(", breaks[-Nbreak],",",breaks[-1],")", sep="")
  names(rates.frame) <- rates.names
  rates.frame$y <- rep(c(1,0), c(ncol(well.mat), ncol(ill.mat)))
  rates.frame$id <- c(well.id, ill.id)
  
  ## Merge the covariates on to the model matrix for the baseline
  if (!is.null(covariate.frame)) {
    model.frame <- merge(rates.frame, covariate.frame, by="id")
  }
  else {
    model.frame <- rates.frame
  }
  
  model.frame[["id"]] <- NULL #Remove id variable
  return( list( rates.frame = model.frame[,rates.names, drop=FALSE],
                  cov.frame = model.frame[,  cov.names, drop=FALSE],
                          y = model.frame[,        "y", drop=TRUE ] ) )
}

poisreg <- function (link = "log")
{
    linktemp <- substitute(link)
    if (!is.character(linktemp)) linktemp <- deparse(linktemp)
    okLinks <- c("log", "identity", "sqrt")
    if (linktemp %in% okLinks)
	stats <- make.link(linktemp)
    else if (is.character(link)) {
        stats <- make.link(link)
        linktemp <- link
    } else {
        ## what else shall we allow?  At least objects of class link-glm.
        if(inherits(link, "link-glm")) {
            stats <- link
            if(!is.null(stats$name)) linktemp <- stats$name
        } else {
            stop(gettextf('link "%s" not available for poisreg family; available links are %s',
			  linktemp, paste(sQuote(okLinks), collapse =", ")),
		 domain = NA)
        }
    }
    variance <- function(mu) mu
    validmu <- function(mu) all(is.finite(mu)) && all(mu>0)
    dev.resids <- function(y, mu, wt)
    { ## faster than  2 * wt * (y * log(ifelse(y == 0, 1, y/mu)) - (y - mu))
	r <- mu*wt
	p <- which(y > 0)
	r[p] <- (wt * (y*log(y/mu) - (y - mu)))[p]
	2*r
    }
    aic <- function(y, n, mu, wt, dev) {
	-2*sum(ifelse(n > 0, (wt/n), 0)*dpois(round(y*n), mu*n, log=TRUE))
    }
    initialize <- expression({
        if (NCOL(y) == 1) {
            n <- rep.int(1, nobs)
            y[weights == 0] <- 0
            if (any(y < 0)) {
                stop("y values must be >= 0")
            }
            m <- weights * y
            if (any(abs(m - round(m)) > 0.001)) {
                warning("non-integer #successes in poisreg glm!")
            }
            mustart <- m + 0.1
        }
        else if (NCOL(y) == 2) {
            if (any(y[,1] < 0)) {
                stop("negative values not allowed for the 'poisreg' family")
            }
            if (any(y[,2] < 0)) {
                stop("negative time not allowed for the 'poisreg' family")
            }
            if(any(y[,1] > 0 & y[,2] == 0)) {
                stop("non-zero counts in zero time in a poisreg glm!")
            }
            if(any(abs(y[,1] - round(y[,1])) > 1e-3)) {
                warning("non-integer counts in a poisreg glm!")
            }

            n <- y[,2]
            y <- ifelse(n == 0, 0, y[, 1]/n)
            
            weights <- weights * n
            mustart <- y + 0.1
        }
        else {
            stop("for the 'poisreg' family, y must be a 2 column matrix where col 1 is no. events and col 2 is time")
        }
    })
    simfun <- function(object, nsim) {
        wts <- object$prior.weights
        ftd <- fitted(object)
        rpois(nsim*length(ftd), ftd*wts)
    }
    structure(list(family = "poisson", ##Fool summary.glm
		   link = linktemp,
		   linkfun = stats$linkfun,
		   linkinv = stats$linkinv,
		   variance = variance,
		   dev.resids = dev.resids,
		   aic = aic,
		   mu.eta = stats$mu.eta,
		   initialize = initialize,
		   validmu = validmu,
		   valideta = stats$valideta,
                   simulate = simfun),
	      class = "family")
}

Lexis.diagram <- function( age = c( 0, 60),
                          alab = "Age",
                          date = c( 1940, 2000 ),
                          dlab = "Calendar time",
                           int = 5,
                       lab.int = 2*int,
                      col.life = "black",
                      lwd.life = 2,
                      age.grid = TRUE,
                     date.grid = TRUE,
                      coh.grid = FALSE,
                      col.grid = gray(0.7),
                      lwd.grid = 1,
                           las = 1,
                    entry.date = NA,
                     entry.age = NA,
                     exit.date = NA,
                      exit.age = NA,
                     risk.time = NA,
                    birth.date = NA,
                          fail = NA,
                      cex.fail = 1.1,
                      pch.fail = c(NA,16),
                      col.fail = rep( col.life, 2 ),
                          data = NULL,
                           ... )
{
# Function to plot a Lexis-diagram
#
# BxC, 2002, revsions in 2005

    ## Get variables from data argument, if supplied, or from parent
    ## frame if not.
    entry.date <- eval(substitute(entry.date), data)
    entry.age  <- eval(substitute(entry.age ), data)
    exit.date  <- eval(substitute(exit.date ), data)
    exit.age   <- eval(substitute(exit.age  ), data)
    risk.time  <- eval(substitute(birth.date), data)
    birth.date <- eval(substitute(birth.date), data)
    fail       <- eval(substitute(fail      ), data)

# First expand intervals to both dimensions
#
    int[1:2] <- c(    int,    int)[1:2]
lab.int[1:2] <- c(lab.int,lab.int)[1:2]

# Plot the diagram
#
plot( NA,
      xlim=date, xaxt="n", xaxs="i", xlab=dlab,
      ylim=age,  yaxt="n", yaxs="i", ylab=alab, ... )
axis( side=1, at=seq( date[1], date[2], lab.int[2] ), las=las )
axis( side=2, at=seq(  age[1],  age[2], lab.int[1] ), las=las )
box( col="white" ) # par("fg") )

# Then the required grids
#
if (  age.grid ) { abline( h = seq(  age[1],  age[2], int[1] ),
                           col=col.grid, lwd=lwd.grid ) }
if ( date.grid ) { abline( v = seq( date[1], date[2], int[2] ),
                           col=col.grid, lwd=lwd.grid ) }
ages  <- seq(  age[1],  age[2], min( int ) )
dates <- seq( date[1], date[2], min( int ) )
if ( coh.grid ) { segments( rep( date[1], length( ages ) ),
                            ages,
                            pmin( date[1] + ( age[2] - ages ), date[2] ),
                            pmin( ages + ( date[2] - date[1] ), age[2] ),
                            col=col.grid, lwd=lwd.grid )
                  segments( dates,
                            rep( age[1], length( dates ) ),
                            pmin( dates + ( age[2] - age[1] ), date[2] ),
                            pmin( age[1] + ( date[2] - dates ), age[2] ),
                            col=col.grid, lwd=lwd.grid )
                 }

# Check if data for lifelines is supplied and plot lifelines if so
#
if( sum( !is.na( list( entry.date,
  	                entry.age,
  	                 exit.date,
                         exit.age,
  	                birth.date,
                         risk.time ) ) ) > 2 )
  {
  LL <- Lexis.lines( entry.date = entry.date,
                      exit.date = exit.date,
                     birth.date = birth.date,
                      entry.age = entry.age,
                       exit.age = exit.age,
                      risk.time = risk.time,
                       col.life = col.life,
                       lwd.life = lwd.life,
                           fail = fail,
                       cex.fail = cex.fail,
                       pch.fail = pch.fail,
                       col.fail = col.fail,
                           data = data )
  invisible( LL )
  }
}

unLexis <-
function(Lx)
{
if (!inherits(Lx, "Lexis")) stop("Not a Lexis object")
attr(Lx, "time.scales") <- NULL
attr(Lx, "time.since") <- NULL
attr(Lx, "breaks") <- NULL
attr(Lx, "class") <- setdiff(attr(Lx, "class"), "Lexis")
Lx
}

adtte2Lexis_usub <-
function(dat, start.state = "rand", Tr.ev, Term.st = NULL)
{
   # the crap to pass the check
   ADT <- ADT2 <- AGE <- CNSR <- Count <- EVNTDESC <- STARTDT <-
   USUBJID <- end.date <- end.state <- startdat <- NULL

  `%>%` <- magrittr::`%>%`

  dat <- as.data.frame(dat)
  dat2 <- dat %>%
    dplyr::group_by(USUBJID) %>%
    dplyr::mutate(startdat = min(STARTDT),
                      ADT2 = Epi::cal.yr(as.Date(ADT, format="%Y-%m-%d"))) %>%
    dplyr::select(USUBJID, ADT2, startdat, AGE, EVNTDESC, CNSR)

  exitstat <- dat2 %>%
    dplyr::group_by(USUBJID) %>%
    dplyr::arrange(ADT2) %>%
    dplyr::slice(dplyr::n())

  
  HH <- dat2[dat2$EVNTDESC %in% Tr.ev & dat2$CNSR!=1,]
  HH <- HH %>%
    dplyr::group_by(USUBJID, EVNTDESC) %>%
    dplyr::mutate( Count = dplyr::row_number(),
                   statecount = paste(EVNTDESC,  dplyr::row_number(), sep="_"),
                   lex.id = USUBJID)

  HHlast <- HH %>%
    dplyr::group_by(USUBJID) %>%
    dplyr::arrange(ADT2) %>%
    dplyr::slice(dplyr::n())

  exitstat$end.state <- HHlast$statecount
  exitstat$end.date <- exitstat$ADT2

  Lx <- Epi::Lexis(data = exitstat,
              entry = list(per = Epi::cal.yr(startdat, format="%Y-%m-%d"),
                           t.sE = 0,
                           age = AGE),
              exit  = list(per = end.date),
              id    = USUBJID,
              entry.status = factor(start.state),
              exit.status = end.state,
              notes = FALSE)

  HHc <- dat2[dat2$EVNTDESC %in% Tr.ev & dat2$CNSR==1,]
  HHc <- HHc %>%
    dplyr::group_by(USUBJID, CNSR) %>%
    dplyr::filter(dplyr::row_number(CNSR) == 1) %>%
    dplyr::mutate(Count = dplyr::row_number(),
                  statecount = "Censored",
                  lex.id = USUBJID)

  HH <- rbind(HH, HHc)

  HH1 <- subset(HH, Count == 1)
  HH2 <- subset(HH, Count != 1)

  if(nrow(HH1)!=0 & nrow(HH2)==0){
    HH1 <- HH1[, c("lex.id", "ADT2", "statecount")]
    colnames(HH1) <- c("lex.id", "cut", "new.state")

    }
  if(nrow(HH1)!=0 & nrow(HH2)!=0){
  HH1$stat <- HH1$statecount#
  HH1 <- HH1[, c("lex.id", "ADT2", "statecount")]
  HH2 <- HH2[, c("lex.id", "ADT2", "statecount")]

  colnames(HH1) <- colnames(HH2) <- c("lex.id", "cut", "new.state")
  }

  if(nrow(HH1)!=0 & nrow(HH2)==0){
   for(i in 1:length(unique(HH1$new.state))){
      Lx <- Epi::cutLexis(Lx,
                  cut= HH1[HH1$new.state == unique(HH1$new.state)[i],],
                  timescale = "per",
                  new.scale = paste(paste0("t.s", unique(HH1$new.state)[i])))
   }
  }
  if(nrow(HH1)!=0 & nrow(HH2)!=0){
     HH3 <- rbind(HH1, HH2)
     HH3 <- HH3[order(HH3$cut),]
     for(i in 1:length(unique(HH3$new.state))){
       Lx <- Epi::cutLexis(Lx,
                      cut= HH3[HH3$new.state == unique(HH3$new.state)[i],],
                      timescale = "per",
                      new.scale = ifelse(substr(unique(HH3$new.state)[i],
                                                nchar(unique(HH3$new.state)[i]),
                                                      nchar(unique(HH3$new.state)[i]))=="1",
                                         paste0("t.s", unique(HH1$new.state)[i]) ,FALSE))

     }
  }

 Lx2 <- subset(Lx, select = -c(EVNTDESC, AGE, CNSR, ADT2, startdat, end.state, end.date))
 if(!is.null(Term.st)){
 tsT <- noquote(paste("t.s" ,paste(Term.st, "_1", sep=""), sep=""))
 Lx2[,!(colnames(Lx2) %in% tsT)]
 }

  return(Lx2)
}

adtte2Lexis <- function(dat, start.state = "Rand", Tr.ev, Term.st = NULL)
{
  # the crap to pass the check
  USUBJID <- NULL
  `%>%` <- magrittr::`%>%`
    lst <- dat %>% dplyr::group_by(USUBJID) %>%
    dplyr::group_map(~.x, .keep=TRUE)
 do.call("rbind", lapply(lst, function(x)adtte2Lexis_usub(x,start.state, Tr.ev,
                                                          Term.st)))
}

stat.table <- function(index, contents=count(), data, margins=FALSE)
{
  ## Get names of indices, using either user-supplied names, or by
  ## deparsing. Some of this code is stolen from the list.names()
  ## function in table()
  index.sub <- substitute(index)
  index <- if (missing(data))
    eval(index)
  else
    eval(index.sub, data)

  deparse.name <- function(x) if(is.symbol(x)) as.character(x) else ""
  if (is.list(index)) {
    if (is.call(index.sub)) {
      ## List constructed in the call to stat.table
      index.names <- names(index.sub)
      fixup <- if (is.null(index.names))
        seq(along = index.sub)
      else index.names == ""
      dep <- sapply(index.sub[fixup], deparse.name)
      if (is.null(index.names))
        index.labels <- dep
      else {
        index.labels <- index.names
        index.labels[fixup] <- dep
      }
      index.labels <- index.labels[-1]
    }
    else {
      ## List constructed outside the call to stat.table
      index.labels <- if (!is.null(names(index))) {
        names(index)
      }
      else {
        rep("", length(index))
      }
    }
  }
  else {
    ## A single vector
    index.labels <- deparse.name(index.sub)
  }

  ## Coerce index to list of factors
  if (!is.list(index))
    index <- list(index)
  index <- lapply(index, as.factor)

  ## Coerce contents to an expression representing a list of function calls
  contents <- substitute(contents)
  if (!identical(deparse(contents[[1]]), "list")) {
    contents <- call("list", contents)
  }

  ## Check that functions in the contents list are valid
  valid.functions <- c("count","mean","weighted.mean","sum","quantile",
                       "median","IQR","max","min","ratio","percent","sd")
  table.fun <- character(length(contents) - 1)
  for (i in 2:length(contents)) {
    if (!is.call(contents[[i]]))
      stop("contents must be a list of function calls")
    FUN <- deparse(contents[[i]][[1]])
    if (!FUN %in% valid.functions)
      stop(paste("Function",FUN,"not permitted in stat.table"))
    else
      table.fun[i-1] <- FUN
  }

  ## Label the contents by default with the function call
  ## But if user has supplied a label use that instead.
  stat.labels <- sapply(contents, deparse)[-1]
  content.names <- names(contents)
  if (!is.null(content.names)) {
    for (i in 2:length(content.names)) {
      if (nchar(content.names[i]) > 0)
        stat.labels[i-1] <- content.names[i]
    }
  }

  ##Define the allowed tabulation functions

  count <- function(id){
      if (missing(id)) {
          id <- seq(along=index[[1]])
      }
      y <- tapply(id, INDEX=subindex, FUN=function(x) length(unique(x)))
      y[is.na(y)] <- 0
      return(y)
  }
  mean <- function(x, trim=0, na.rm=TRUE) {
    tapply(x, INDEX=subindex, FUN = base::mean, trim=trim, na.rm=na.rm)
  }
  weighted.mean <- function(x,w,na.rm=TRUE) {
    tapply(x, INDEX=subindex, FUN=stats::weighted.mean, w=w, na.rm=na.rm)
  }
  sum <- function(...,na.rm=TRUE) {
    tapply(..., INDEX=subindex, FUN = base::sum, na.rm=na.rm)
  }
  quantile <- function(x, probs, na.rm=TRUE,names=TRUE,type=7,...) {
    if (length(probs) > 1)
      stop("The quantile function only accepts scalar prob values within stat.table")
    tapply(x, INDEX=subindex, FUN = stats::quantile, probs=probs,
           na.rm=na.rm,names=names,type=type,...)
  }
  median <- function(x, na.rm=TRUE) {
    tapply(x, INDEX=subindex, FUN = stats::median, na.rm=na.rm)
  }
  IQR <- function(x, na.rm=TRUE) {
    tapply(x, INDEX=subindex, FUN= stats::IQR, na.rm=na.rm)
  }
  max <- function(..., na.rm=TRUE) {
    tapply(..., INDEX=subindex, FUN = base::max, na.rm=na.rm)
  }
  min <- function(..., na.rm=TRUE) {
    tapply(..., INDEX=subindex, FUN = base::min, na.rm=na.rm)
  }
  ratio <- function(d,y,scale=1, na.rm=TRUE) {
    if (length(scale) != 1)
      stop("Scale parameter must be a scalar")
    if (na.rm) {
      w <- (!is.na(d) & !is.na(y))
      tab1 <- tapply(d*w, INDEX=subindex, FUN=base::sum, na.rm=TRUE)
      tab2 <- tapply(y*w, INDEX=subindex, FUN=base::sum, na.rm=TRUE)
    }
    else {
      tab1 <- tapply(d, INDEX=subindex, FUN=base::sum, na.rm=FALSE)
      tab2 <- tapply(y, INDEX=subindex, FUN=base::sum, na.rm=FALSE)
    }
    return(scale*tab1/tab2)
  }
  percent <- function(...) {
    x <- list(...)
    if (length(x) == 0)
      stop("No variables to calculate percent")
    x <- lapply(x, as.factor)
    n <- count()
    ## Work out which indices to sweep out
    sweep.index <- logical(length(subindex))
    for (i in seq(along=subindex)) {
      sweep.index[i] <- !any(sapply(x,identical,subindex[[i]]))
    }
    if (!any(sweep.index)) {
      return(100*n/base::sum(n, na.rm=TRUE))
    }
    else {
      margin <- apply(n,which(sweep.index),base::sum, na.rm=TRUE)
      margin[margin==0] <- NA
      return(100*sweep(n, which(sweep.index), margin,"/"))
    }
  }
  sd <- function (..., na.rm = TRUE)
  {
      tapply(..., INDEX=subindex, FUN = stats::sd, na.rm=na.rm)
  }

  ##Calculate dimension of the main table, excluding margins
  n.dim <- length(index)
  tab.dim <- sapply(index, nlevels)

  ##Sort out margins
  if (length(margins) == 1)
    margins <- rep(margins, n.dim)
  else if(length(margins) != n.dim)
    stop("Incorrect length for margins argument")

  ##Create grid of all possible subtables.
  fac.list <- vector("list", n.dim)
  for (i in 1:n.dim) {
    fac.list[[i]] <- if (margins[i]) c(0,1) else 1
  }
  subtable.grid <- as.matrix(expand.grid(fac.list))

  ##Fill in the subtables
  ans.dim <- c(length(contents)-1, tab.dim + margins)
  ans <- numeric(prod(ans.dim))
  for (i in 1:nrow(subtable.grid)) {
    ##in.subtable is a logical vector indicating which dimensions are
    ##in the subtable (i.e. which have not been marginalized out)
    in.subtable <- as.logical(subtable.grid[i,])
    llim <- rep(1,n.dim) + ifelse(in.subtable,rep(0,n.dim),tab.dim)
    ulim <- tab.dim + ifelse(in.subtable,rep(0,n.dim),rep(1, n.dim))
    subindex <- index[in.subtable]
    if (length(subindex) == 0) {
        ## Marginalizing out all dimensions
        subindex <- list(rep(1, length(index[[1]])))
    }
    subtable.list <- if(missing(data))
        ##eval(contents, parent.frame())
        eval(contents)
    else
      eval(as.expression(contents), data)
    for (j in 1:length(subtable.list)) {
      ans[array.subset(ans.dim,c(j,llim),c(j,ulim))] <- subtable.list[[j]]
    }
  }

  ans <- array(ans, dim=ans.dim)
  ans.dimnames <- lapply(index, levels)
  names(ans.dimnames) <- index.labels
  for (i in 1:length(index)) {
    if (margins[i])
      ans.dimnames[[i]] <- c(ans.dimnames[[i]], "Total")
  }
  dimnames(ans) <- c(list("contents"=stat.labels), ans.dimnames)
  attr(ans, "table.fun") <- table.fun
  class(ans) <- c("stat.table", class(ans))
  return(ans)
}


array.subset <- function(dim,lower,upper)
{
  ##Returns a logical array of dimension dim for which elements in the range
  ##[lower[1]:upper[1], lower[2]:upper[2],...] are TRUE and others FALSE

  ##Check validity of arguments (but assume everything is an integer)
  ndim <- length(dim)
  if (length(lower) != ndim || length(upper) != ndim) {
    stop("Length of lower and upper limits must match dimension")
  }
  if (any(lower > upper) || any(lower < 1) || any(upper > dim)) {
    stop("Invalid limits")
  }

  ##The math is easier if we index arrays from 0 rather than 1
  lower <- lower - 1
  upper <- upper - 1

  N <- prod(dim)
  ans <- rep(TRUE, N)
  for (i in 1:N) {
    l <- i - 1
    for (d in 1:ndim) {
      k <- l %% dim[d] #k is the index of the ith element in dimension d
      if (k < lower[d] || k > upper[d]) {
        ans[i] <- FALSE
        break
      }
      l <- l %/% dim[d]
    }
  }
  return(array(ans, dim))
}

split2.width <- function(x,width)
{
  ## Splits a string into a vector so that each element has at most width
  ## characters.  If width is smaller than the length of the shortest word
  ## then the latter is used instead

  x.split <- strsplit(x,split=" ")[[1]]
  width <- max(c(width,nchar(x.split)))
  y <- character(0)
  imin <- 1
  n <- length(x.split)
  for (i in 1:n) {
    cum.width <- if(i==n) {
      Inf
    }
    else {
      sum(nchar(x.split[imin:(i+1)])) + (i - imin + 1)
    }
    if (cum.width > width) {
      y <- c(y,paste(x.split[imin:i], collapse=" "))
      imin <- i + 1
    }
  }
  return(y)
}

prettyPrint.stattable.1d <- function(x, width, digits)
{
  ##Pretty printing of 1-D stat.table

  if (length(dim(x)) != 2)
    stop("Cannot print stat.table")
  ncol <- nrow(x)

  col.width <- numeric(ncol+1)
  col.header <- vector("list",ncol+1)
  n.header <- integer(ncol+1)
  print.list <- vector("list",ncol+1)

  ##First column
  col.header[[1]] <- split2.width(names(dimnames(x))[2], width)
  n.header[1] <- length(col.header[[1]])
  col1 <- format(c(col.header[[1]],dimnames(x)[[2]]), justify="left")
  col.header[[1]] <- col1[1:n.header[1]]
  print.list[[1]] <- col1[-(1:n.header[1])]
  col.width[1] <- nchar(col.header[[1]][1])

  ##Other columns
  for (i in 2:(ncol+1)) {
    col.header[[i]] <- split2.width(dimnames(x)[[1]][i-1], width)
    n.header[i] <- length(col.header[[i]])
    this.col  <- formatC(x[i-1,],width=width,
                         digits=digits[attr(x,"table.fun")[i-1]], "f")
    this.col <- format(c(col.header[[i]],this.col),justify="right")
    col.width[i] <- nchar(this.col[1])
    col.header[[i]] <- this.col[1:n.header[i]]
    print.list[[i]] <- this.col[-(1:n.header[i])]
  }

  ##
  table.width <- sum(col.width) + ncol + 3
  max.n.header <- max(n.header)
  cat(" ",rep("-",table.width)," \n",sep="")
  for(i in 1:max.n.header) {
    cat(" ")
    for(j in 1:length(print.list)) {
      if (i <= n.header[j]) {
        cat(col.header[[j]][i])
      }
      else {
        cat(rep(" ", col.width[[j]]),sep="")
      }
      if (j==1)
        cat("   ")
      else
        cat(" ")
    }
    cat(" \n")
  }
  cat(" ",rep("-",table.width)," \n",sep="")

  for (i in 1:length(print.list[[1]])) {
    cat(" ")
    if (pmatch("Total",print.list[[1]][i],nomatch=0)) {
      ##Add a blank line before the total
      cat(rep(" ",col.width[1]+1),"  ",rep(" ",sum(col.width[-1])+ncol),
          " \n ",sep="")
    }
    for (j in 1:length(print.list)) {
      cat(print.list[[j]][i])
      if (j == 1) {
        cat("   " )
      }
      else {
        cat(" ")
      }
    }
    cat(" \n")
  }
  cat(" ",rep("-",table.width)," \n",sep="")
  return(invisible(x))
}

prettyPrint.stattable.2d <- function(x, width, digits)
{
  ##Pretty printing of 2-Dimensional stat.table

  if (length(dim(x)) != 3)
    stop("Cannot print stat.table")
  nstat <- dim(x)[1]
  ncol <- dim(x)[3]
  nrow <- dim(x)[2]

  col.width <- numeric(ncol+1)
  col.header <- vector("list",ncol+1)
  n.header <- integer(ncol+1)
  print.list <- vector("list",ncol+1)

  ##First column
  col.header[[1]] <- split2.width(names(dimnames(x))[2], width)
  n.header[1] <- length(col.header[[1]])
  col1 <- format(c(col.header[[1]],dimnames(x)[[2]]), justify="left")
  col.header[[1]] <- col1[1:n.header[1]]
  print.list[[1]] <- col1[-(1:n.header[1])]
  col.width[1] <- nchar(col.header[[1]][1])

  ##Other columns
  for (i in 2:(ncol+1)) {
    col.header[[i]] <- split2.width(dimnames(x)[[3]][i-1], width)
    n.header[i] <- length(col.header[[i]])
    this.col <- matrix("", nrow=nstat,ncol=nrow)
    for (j in 1:nstat) {
      z <- x[j,,i-1]
      this.col[j,]  <- formatC(z, width=width, format="f",
                               digits=digits[attr(x,"table.fun")[j]])
      ##      this.col[j,] <- formatC(z, width=width, digits=digits,
      ##                       format=ifelse(identical(round(z),z),"d","f"))
    }
    this.col <- format(c(col.header[[i]],this.col),justify="right")
    col.width[i] <- nchar(this.col[1])
    col.header[[i]] <- this.col[1:n.header[i]]
    print.list[[i]] <- this.col[-(1:n.header[i])]
  }

  ##Correct first column for multiple stats
  if (nstat > 1) {
    pl1 <- print.list[[1]]
    print.list[[1]] <- rep(paste(rep(" ",col.width[1]),collapse=""),nstat*nrow)
    print.list[[1]][1 + nstat*((1:nrow)-1)] <- pl1
  }

  table.width <- sum(col.width) + ncol + 3
  max.n.header <- max(n.header)
  cat(" ",rep("-",table.width)," \n",sep="")
  ## Supercolumn header
  super.header <- names(dimnames(x))[3]
  npad <- sum(col.width[-1]) + ncol + 1 - nchar(super.header)
  if (npad >= 0) {
    cat(" ",rep(" ",col.width[1]),"  ",sep="")
    cat(rep("-",floor(npad/2)),sep="")
    cat(super.header)
    cat(rep("-",ceiling(npad/2))," \n",sep="")
  }
  ## Headers
  for(i in 1:max.n.header) {
    cat(" ")
    for(j in 1:length(print.list)) {
      if (i <= n.header[j]) {
        cat(col.header[[j]][i])
      }
      else {
        cat(rep(" ", col.width[[j]]),sep="")
      }
      if (j==1)
        cat("   ")
      else
        cat(" ")
    }
    cat(" \n")
  }
  cat(" ",rep("-",table.width)," \n",sep="")
  ## Body of table
  blank.line <- function() {
    cat(" ",rep(" ",col.width[1]+1),"  ",rep(" ",sum(col.width[-1])+ncol),
        " \n",sep="")
  }
  for (i in 1:length(print.list[[1]])) {
    if (pmatch("Total",print.list[[1]][i],nomatch=0)) {
      ##Add a blank line before the total
      blank.line()
    }
    cat(" ")
    for (j in 1:length(print.list)) {
      cat(print.list[[j]][i])
      if (j == 1) {
        cat("   " )
      }
      else {
        cat(" ")
      }
    }
    cat(" \n")
    if (nstat > 1 && i %% nstat == 0 && i != length(print.list[[1]])) {
      ##Separate interleaved stats
      blank.line()
    }
  }
  cat(" ",rep("-",table.width)," \n",sep="")
  return(invisible(x))
}

print.stat.table <- function(x, width=7,digits,...)
{
  fun.digits <- c("count"=0,"mean"=2,"weighted.mean"=2,"sum"=2,"quantile"=2,
                  "median"=2,"IQR"=2,"max"=2,"min"=2,"ratio"=2,"percent"=1,
                  "sd"=2)
  if (!missing(digits)) {
    if (is.null(names(digits))) {
      if (length(digits) > 1)
        stop("digits must be a scalar or named vector")
      else
        fun.digits[1:length(fun.digits)] <- digits
    }
    else {
      fun.digits[names(digits)] <- digits
    }
  }
  if (length(dim(x)) == 2)
    prettyPrint.stattable.1d(x, width, fun.digits)
  else if (length(dim(x)) == 3)
    prettyPrint.stattable.2d(x, width, fun.digits)
  else
    NextMethod("print",...)
}

## Satisfy QA checks by defining  these functions. But if we never
## export them they can't be used directly.

count <- function(id)
{
}

ratio <- function(d, y, scale=1, na.rm=TRUE)
{
}

percent <- function(...)
{
}

Pplot <-
function( rates,
          age = as.numeric( dimnames( rates )[[1]] ),
          per = as.numeric( dimnames( rates )[[2]] ),
         grid = FALSE,
       p.grid = grid,
        ygrid = grid,
     col.grid = gray( 0.9 ),
        p.lim = range( per, na.rm=TRUE ) + c(0,diff(range(per))/30),
         ylim = range( rates[rates>0], na.rm=TRUE ),
        p.lab = names( dimnames( rates ) )[2],
         ylab = deparse( substitute( rates ) ),
           at = NULL,
       labels = paste( at ),
         type = "l",
          lwd = 2,
          lty = 1,
          col = par( "fg" ),
       log.ax = "y",
          las = 1,
          ann = FALSE,
      cex.ann = 0.8,
        xannx = 1/20,
       a.thin = seq( 1, length( age ), 2 ),
          ...
          )
{
# Plot the frame
if( ann ) p.lim <- p.lim + c(0,diff( range( age ) ) * xannx )
matplot( per, t(rates), type="n",
         xlim=p.lim, ylim=ylim, xlab=p.lab, ylab=ylab,
         log=log.ax, las=las, yaxt=if( !is.null( at ) ) "n" else "s" )
if( !is.null( at ) ) axis( side=2, at=at, labels=labels, yaxt="s", las=las )
# and the grid if required
if( !missing( p.grid ) | !missing( grid ) )
  {
  if( is.logical( p.grid ) & p.grid[1] ) p.grid <- nice( per, log=par("xlog") )
  abline( v=p.grid, col=col.grid )
  }
if( !missing( ygrid ) | !missing( ygrid ) )
  {
  if( is.logical( ygrid ) & ygrid[1] ) ygrid <- nice( rates[!is.na(rates)], log=par("ylog") )
  abline( h=ygrid, col=col.grid )
  }
box()
# then the curves
matlines( per, t(rates), lwd=lwd, lty=lty, col=col, type=type, ... )
# annotate them if required (every second by default )
if( ann )
  {
   nr <- nrow( rates )
   nc <- ncol( rates )
   text( rep( per[nc], nr )[a.thin], rates[,nc][a.thin],
         paste( "", age[a.thin] ), adj=c(0,0.5),
         cex=cex.ann, col=if( length(col)==1 ) col else col[a.thin] ) 
  }
}

"ftrend" <-
function(object, ...)
{
  if(length(object$xlevels) == 0) {
    stop("No factors in model")
  }
  
  xname <- names(object$xlevels)[1]
  if (!identical(object$contrasts[[1]], "contr.treatment")) {
    stop(paste("Treatment contrasts must be used for variable", xname))
  }
  xlevels <- object$xlevels[[1]]
  nlevels <- length(xlevels)
  coefnames <- paste(xname, xlevels, sep="")
  ncoef <- length(coef(object))
  
  if (!all(coefnames %in% names(coef(object)))) {
    stop("The model must not have an intercept term")
  }
  
  index1 <- match(coefnames, names(coef(object)))
  index2 <- (1:ncoef)[-index1]
  m <- length(index1)
  ncov <- length(index2)

  ## Centre the covariates according to Greenland et al (weighted mean = 0)
  X0 <- model.matrix(object)
  if (!is.null(object$weights)) {
    mu <- -apply(X0, 2, weighted.mean, object$weights )[index2]
  }
  else {
    mu <- -apply(X0, 2, mean)[index2]
  }
  mu.full <- rep(0, ncoef)
  mu.full[index2] <- mu
  X <- sweep(X0, 2, mu.full, "+")

  ## Information matrix with centred covariates
  if (!is.null(object$weights)) {
    J <- crossprod(X, sweep(X, 1, object$weights, "*"))
  }
  else {
    ## linear models
    J <- crossprod(X,X)
  }

  J11 <- J[index1, index1]
  J12 <- J[index1, index2]
  J21 <- J[index2, index1]
  J22 <- J[index2, index2]

  ## Variance matrix
  V <- solve(J)
  V11 <- V[index1, index1]
  V12 <- V[index1, index2]
  V21 <- V[index2, index1]
  V22 <- V[index2, index2]

  cal.V <- function(mu) {
    one <- as.matrix(rep(1,m))
    mu <- as.matrix(mu)
    return(V11 - one %*% t(mu) %*% V21 - V12 %*% mu %*% t(one) +
           matrix(1, m, m) * (t(mu) %*% V22 %*% mu)[1,1])
  }

  f <- function(mu) {
    V.mu <- cal.V(mu)
    # lambda is current vector of floating variances
    D <- sum(diag(V.mu)/lambda) - c(determinant(V.mu)$modulus) +
      sum(log(lambda)) - m

    S <- (1/sum(diag(J11)) + t(mu) %*% solve(J22) %*% mu)[1,1]
    
    grad1 <- - t(1/lambda) %*% V12
    grad2 <- + sum(1/lambda) * t(mu) %*% V22
    grad3 <- - t(mu) %*% solve(J22)/S
    attr(D,"gradient") <- 2 * (grad1 + grad2 +  grad3)

    H1 <- V22 * sum(1/lambda)
    H2 <- -solve(J22)/S
    b <- solve(J22) %*% mu/S
    H3 <- 2 * b %*% t(b)
    attr(D, "hessian") <- 2 * (H1 + H2 + H3)
    return(D)
  }

  ## Initial value of lambda
  lambda <- diag(V[index1,index1])

  ## Do the minimization
  nlm.out <- nlm(f, rep(0,ncov), check.analyticals=TRUE, ...)
  mu2 <- nlm.out$estimate

  ## Calculate parameter values and their covariance matrix
  ## if the covariates are appropriately centred
  coef <- coef(object)[index1] - c(crossprod(mu + mu2, coef(object)[index2]))
  return(list(coef=coef, vcov=cal.V(mu2)))
}

stackedCIF <- 
    function( x,
          group = 1,
            col = "black",
           fill = "white",
           ylim = c(0,1), 
           xlab = "Time",
           ylab = "Cumulative incidence", ...) 
{
    ne <- ncol(x$pstate) - 1
    ng <- length(x$n)
    if (group %in% 1:ng) {
        r1 <- ifelse(group == 1 | ng == 1, 1, cumsum(x$strata)[group - 
            1] + 1)
        r2 <- ifelse(ng == 1, length(x$time), cumsum(x$strata)[group])
        pSt0 <- matrix(0, nrow = nrow(x$pstate[r1:r2, ]), ncol = ne + 
            2)
        pSt0[, 1] <- x$pstate[r1:r2, 2]
        for (c in 2:ne) pSt0[, c] <- pSt0[, c - 1] + x$pstate[r1:r2, 
            c+1]
        pSt0[, ne + 1] <- 1
        pSt0[, ne + 2] <- x$time[r1:r2]
        pSt <- cbind(0, pSt0)
        pSt2 <- matrix(0, nrow = 2 * (dim(pSt)[1] - 1), ncol = dim(pSt)[2])
        pSt2[1, ne + 3] <- pSt[1, ne + 3]
        pSt2[nrow(pSt2), ne + 3] <- pSt[nrow(pSt), ne + 3]
        for (j in 1:(dim(pSt)[1] - 1)) {
            pSt2[2 * j - 1, ne + 3] <- pSt[j, ne + 3]
            pSt2[2 * j, ne + 3] <- pSt[j + 1, ne + 3]
            pSt2[2 * j - 1, 1:(ne + 2)] <- pSt[j, 1:(ne + 2)]
            pSt2[2 * j, 1:(ne + 2)] <- pSt[j, 1:(ne + 2)]
        }
        plot(as.numeric(pSt0[, ne + 2]), pSt0[, 2], type = "n", 
            ylim = ylim, yaxs = "i", xlab = xlab, ylab = ylab, 
            ...)
        for (i in 2:(ne + 1)) {
            polygon(c(pSt2[, ne + 3], rev(pSt2[, ne + 3])), c(pSt2[, 
                i], rev(pSt2[, i - 1])), col = fill[i - 1], border = NULL, 
                ...)
        }
        matlines( as.numeric(pSt0[, ne + 2]), pSt0[, 
            1:ne], type = "s", col = col)
        abline(v = max(x$time[r1:r2]), lwd = 2, col = "white")
        box()
    }
    else print(paste("Error: group indicator must be an integer from 1 to", 
        ng))
}

# Acronyms:
# dop: date of purchase
# dpt: dose per time
# amt: amount
# dur: duration
# dof: date of follow-up

######################################################################
# A function to compute cumulative doses etc. at the purchase dates
# *and* derive intervals of drug coverage resp no coverage from the
# purchase date, amounts and dosage per time
use.amt.dpt <-
function( purchase, # data frame with dop, amt and dpt
          push.max,
          rm.dose )
{
do.call( "rbind",
lapply( split( purchase, purchase$id ),
        function(set)
        {
        np <- nrow(set)
        if( np==1 ) # return data frame of 2 rows
          {
          dfR <- data.frame( id = set$id,
                            dof = set$dop + c(0,set$amt/set$dpt),
                            amt = c(set$amt,0),
                            dpt = c(set$dpt,0),
                            off = c(FALSE,TRUE),
                        cum.amt = c(0,set$amt),
                        cum.tim = c(0,set$amt/set$dpt) )
          return( dfR )
          }
        set <- set[order(set$dop),]
        # Compute length of exposure periods
        drug.dur  <- set$amt / set$dpt
        # Put the exposed period head to foot
        new.start <- min( set$dop ) + c(0,cumsum(drug.dur[-np]))
        # Move them out so that the start of a period is never earlier than
        # the dop
        exp.start <- new.start + cummax( pmax(set$dop-new.start,0) )
        # Compute the pushes
        push.one  <- exp.start - set$dop
        # Revise them to the maximally acceptable
        push.adj  <- pmin( push.one, push.max )
        # Revise the starting dates of exposure
        exp.start <- exp.start - push.one + push.adj
        # Revise the durations to be at most equal to differences between the
        # revised starting dates
        drug.dur  <- pmin( drug.dur, c(diff(exp.start),Inf) )
        # Compute the end of the intervals
        exp.end   <- exp.start + drug.dur
        # Find out which exposure intervals that are followed by a gap:
        followed.by.gap <- c( exp.start[-1]-exp.end[-np] > 0, TRUE )
        # To facilitate further calculations we add records for the gaps
        # and also a record for the date of drug expiration
        dfR <- rbind( data.frame( id = set$id[1],
                                 dof = exp.start,
                                 amt = set$amt,
                                 dpt = set$dpt ),
                      data.frame( id = set$id[1],
                                 dof = exp.end[followed.by.gap],
                                 amt = 0,
                                 dpt = 0 ) )
        dfR <- dfR[order(dfR$dof),]
        # Logical indicator of intervals off drug
        dfR$off <- (dfR$dpt==0) | (dfR$dof < min(set$dop))
        # Finally compute the cumulative dose and time on drug at the end of
        # the interval using interval length and dpt:
        if( rm.dose ) {
        dfR$cum.amt <- with( dfR, cumsum( c(0, diff(dof)* dpt[-length(dpt)]) ) )
        } else dfR$cum.amt <- cumsum( c( 0, dfR$amt[-nrow(dfR)] ) )
        dfR$cum.tim <- with( dfR, cumsum( c(0, diff(dof)*!off[-length(dpt)]) ) )
        return( dfR )
        } ) )
}

######################################################################
# Compute the cumulative dose at all purcase dates and at the last
# (unknown) future expiry date, computed based on previous
# consumption.  The resulting data frame has one more line per person
# than no. of purchases.
use.only.amt <-
function( purchase,
          pred.win )
{
do.call( "rbind",
lapply( split( purchase, purchase$id ),
        function(set)
        {
        np <- nrow(set)
        if( np==1 ) return( NULL )
        set <- set[order(set$dop),]
        # The points to include in the calculation:
        # All dates after pred.win before last purchase,
        # but at least the last two purchase dates,
        wp <- ( set$dop > pmin( max(set$dop)-pred.win,
                               sort(set$dop,decreasing=TRUE)[2] ) )
        # Cumulative amount consumed at each dop since first
        cum.amt <- cumsum(c(0,set$amt))
        # Average slope to use to project the duration of last purchase
        avg.slp <- diff(range(cum.amt[c(wp,FALSE)]))/
                   diff(range(set$dop[wp]))
        # Purchase dates and the date of last consumption
        dof <- c( set$dop, set$dop[np]+set$amt[np]/avg.slp )
        # Cumulative time since first
        cum.tim <- dof - set$dop[1]
        # The only time without drug is after last dispense expired
        off <- rep( c(FALSE,TRUE), c(length(dof)-1,1) )
        return( data.frame( id = set$id[1],
                           dof = dof,
                       cum.amt = cum.amt,
                       cum.tim = cum.tim,
                           off = off ) ) # will be used in the subsequent code
        } ) )
}

######################################################################
# Function to interpolate the drug exposures to the dates of FU.
dex.int <-
function( set, breaks, lags, lag.dec )
{
  dof <- NULL
  # If only one record return null
  if( nrow(set) < 2 ) return( NULL )
  # All values of these are identical within each set (=person)
  doe <- set$doe[1]
  dox <- set$dox[1]
  # The first date of drug exposure according to the assumption
  doi <- min(set$dof)
  # Breakpoints and the entry end exit dates are the dates we care for
  # - but only within breaks
  doin <- max( min(breaks), doe )
  doex <- min( max(breaks), dox )
  breaks <- sort( unique( c(breaks,doin,doex) ) )
  # only the breaks inside the follow-up interval
  xval <- breaks[breaks>=doin & breaks<=doex]
  if( length(xval) == 0 ) return( NULL )
  # Merge a) the purchase dates (set$dof) and
  #    indicator being off drug (set$off) and
  # the date a person goes off drug, doff (set to NA for remaining records)
  # with b) the break dates (which is where we want things computed)
  # Note we merge on the variable dof and have the data frame with
  # dof=xval as the >first< so that values of xval will be in the
  # resulting dfr$dof even if dof-values in set are almost equal til
  # an xval value. Ensures that sum(dfr$dof %in% xval)==length(xval)   
  dfr <-  merge( data.frame( id = set$id[1],
                            dof = xval ),
                 data.frame( set[,c("id","dof","off")],
                             doff = ifelse(set$off,set$dof,NA) ),
                 all=TRUE )
  # carry the off drug indicator forward to all break dates
  dfr$off <- zoo::na.locf( dfr$off, na.rm=FALSE )
  # possible NAs are before any drug purchase hence off-drug
  dfr$off <- ifelse( is.na(dfr$off), TRUE, dfr$off )
  # carry date of going off drug forward, but only for the off period
  dfr$doff <- zoo::na.locf( dfr$doff, na.rm=FALSE ) * ifelse(dfr$off,1,NA)
  # time from cessation can now be computed
  dfr$tfc <- dfr$dof - dfr$doff
  # time from initiation of drug
  dfr$tfi  <- pmax( 0, dfr$dof-doi )
  # restrict to the desired timepoints
  dfr <- subset( dfr, dof %in% xval )
  dfr <- dfr[!duplicated(dfr$dof),]
  # linear interpolation of the cumulative dose and time from the
  # purchase data (set)
  dfr$cdos <- approx( set$dof, set$cum.amt, xout=xval, rule=2 )$y
  dfr$ctim <- approx( set$dof, set$cum.tim, xout=xval, rule=2 )$y
  # the same for the desired lags
  for( lg in lags )
     dfr[,paste( "lag.pre",
                 formatC(lg,format="f",digits=lag.dec),
                 sep="" )] <-
     approx( set$dof, set$cum.amt, xout=xval-lg, rule=2 )$y
  dfr$doff <- zoo::na.locf( dfr$doff, na.rm=FALSE )
  dfr$tfc[is.na(dfr$tfc)] <- 0
  dfr$dur <- c( diff(dfr$dof), NA )
  return( dfr[-nrow(dfr),] )
  }

######################################################################
# The function that ties it all together
gen.exp <-
function( purchase,  id="id" , dop="dop", amt="amt", dpt="dpt",
                fu, doe="doe", dox="dox",
            breaks,
           use.dpt = ( dpt %in% names(purchase) ),
          push.max = Inf,
           rm.dose = FALSE,
              lags = NULL,
           lag.dec = 1,
           lag.pre = "lag.",
          pred.win = Inf )
{
# to aviod a NOTE from R CMD check
dof <- NULL
# Make sure that the data frames have the right column names
wh <- match( c(id,dop,amt), names(purchase) )
if( any( is.na(wh) ) ) stop("Wrong column names for the purchase data frame")
names( purchase )[wh] <- c("id","dop","amt")

# Allow dpt to be entered as numerical scalar common for all records
if( use.dpt )
  if( is.numeric(dpt) )
    {
    if( length(dpt) > 1 ) stop("If dpt is numeric it must have length 1\n",
        "otherwise it must be the name of the dpt variable in the dataset")
    purchase$dpt <- dpt
    }
  else names( purchase )[match(dpt,names(purchase))] <- "dpt"

wh <- match( c(id,doe,dox), names(fu) )
if( any( is.na(wh) ) ) stop("Wrong column names for the follow-up data frame")
names( fu )[wh] <- c("id","doe","dox")

if( use.dpt ) { tmp.dfr <- use.amt.dpt( purchase,
                                        push.max = push.max,
                                        rm.dose = rm.dose )
       } else { tmp.dfr <- use.only.amt( purchase,
                                         pred.win = pred.win ) }

# Having done what needs to be done about the purchase records we turn
# to the follow-up records, but we first need to accommodate the fact
# that there might be more fu records per person:

# Generate record no within id in fu (ordering in fu is immaterial)
fu$no <- ave( fu$id, fu$id, FUN=function(x) 1:length(x) )
# Set up the object to collect the resulting follow-up
res <- NULL

# loop through subsets of fu with different ids
for( ni in 1:max(fu$no) )
   {
# Merge the follow-up period for the persons to the correponding
# exposure records (hence the all.y=T)
tm.dfr <- merge( tmp.dfr, fu[fu$no==ni,], by="id", all.y=TRUE )

# Interpolate to find the cumulative doses at the dates in the vector breaks
res.dfr <-
do.call( "rbind",
         lapply( split( tm.dfr, tm.dfr$id ),
                 dex.int,
                 breaks, lags, lag.dec ) ) # end of do.call
res <- rbind( res, res.dfr ) # append to result from other fu$no
   } # end of for( ni in 1:max(fu$no) )
    
var.order <- c("id","dof","dur","off","doff","tfc","tfi","ctim","cdos",
               names(res)[grep("lag.prefix",names(res))] )
res <- res[order(res$id,res$dof),var.order]
names(res) <- gsub( "lag.prefix", lag.pre, names(res) )
res
}
# end of gen.ex

twoby2 <-
  function( exposure,
             outcome,
               alpha = 0.05,
               print = TRUE,
                 dec = 4,
          conf.level = 1-alpha,
               F.lim = 10000 ) # What is the limit for trying the
                               # Fisher.test 
{
    if( !missing( conf.level ) ) alpha <- 1 - conf.level
    if( inherits( exposure, c( "table", "matrix" ) ) ) tab <- exposure
    else tab <- table( exposure, outcome )
    tab <- tab[1:2,1:2]
    a <- tab[1, 1]
    b <- tab[1, 2]
    c <- tab[2, 1]
    d <- tab[2, 2]
    bin.ci <- function( x, n )
      {
      # Confidence interval for proportion based on Taylor-expansion of
      # the log-odds --- surprisingly good coverage.
      ef <- exp( qnorm(1-alpha/2)/sqrt(x*(n-x)/n) )
      p <- x / n
      c( x/n, p/(p+(1-p)*ef), p/(p+(1-p)/ef) )
      }
    rr <- (a/(a + b))/(c/(c + d))
    se.log.rr <- sqrt((b/a)/(a + b) + (d/c)/(c + d))
    lci.rr <- exp(log(rr) - qnorm( 1-alpha/2 ) * se.log.rr)
    uci.rr <- exp(log(rr) + qnorm( 1-alpha/2 ) * se.log.rr)
    or <- (a/b)/(c/d)
    se.log.or <- sqrt(1/a + 1/b + 1/c + 1/d)
    lci.or <- exp(log(or) - qnorm( 1-alpha/2 ) * se.log.or)
    uci.or <- exp(log(or) + qnorm( 1-alpha/2 ) * se.log.or)
# Computing the c.i. for the probability difference as method
# 10 from Newcombe, Stat.Med. 1998, 17, pp.873 ff.
    pr.dif <- ci.pd( a, c, b, d, alpha=alpha, print=FALSE )[5:7]
        pd <- pr.dif[1]
    lci.pd <- pr.dif[2]                        
    uci.pd <- pr.dif[3]
    as.pval <- 1 - pchisq( log( or )^2 / sum( 1/tab[1:2,1:2] ), 1 )
# If the numers are too large we don't bother about computing Fisher's test
    Fisher <- ( sum( tab ) < F.lim )
    ft <- if( !Fisher ) NA else fisher.test( tab, conf.level=1-alpha )
# We need row and colum names for annotating the output
    if( is.null( rownames( tab ) ) ) rownames( tab ) <- paste( "Row", 1:2 ) 
    if( is.null( colnames( tab ) ) ) colnames( tab ) <- paste( "Col", 1:2 )
    tbl <- cbind( tab[1:2,1:2], rbind( bin.ci( a, a+b ), bin.ci( c, c+d ) ) )
    colnames( tbl )[3:5] <-
      c(paste( "   P(", colnames( tab )[1], ")", sep=""), 
        paste( 100*(1-alpha),"% conf.", sep="" ), "interval")
     if( print )
       cat("2 by 2 table analysis:",
           "\n------------------------------------------------------",
         "\nOutcome   :", colnames( tab )[1],
         "\nComparing :", rownames( tab )[1], "vs.", rownames( tab )[2], "\n\n" )
    if( print ) print( round( tbl, dec ) )
    if( print ) cat( "\n" )
    rmat <- rbind( c( rr, lci.rr, uci.rr ),
                   c( or, lci.or, uci.or ),
       if( Fisher) c( ft$estimate, ft$conf.int ),
                   c( pd, lci.pd, uci.pd ) )
    rownames( rmat ) <- c("             Relative Risk:", 
                          "         Sample Odds Ratio:", 
              if( Fisher) "Conditional MLE Odds Ratio:", 
                          "    Probability difference:")
    colnames( rmat ) <- c("     ", 
                          paste( 100*(1-alpha),"% conf.", sep="" ), 
                          "interval" )
    if( print ) print( round( rmat, dec ) )
    if( print ) cat(
        if( Fisher ) "\n             Exact P-value:",
        if( Fisher ) formatC( ft$p.value, format="f", digits=dec ),
                     "\n        Asymptotic P-value:",
                     formatC( as.pval,    format="f", digits=dec ),
        "\n------------------------------------------------------\n")
    invisible( list( table = tbl,
                  measures = rmat,
                   p.value = c(as.pval,if( Fisher )ft$p.value) ) )
  }

# Survfit requires a special value for censorings not just the same as
# is the convention in Lexis objects, and moreover this must be the
# first level of factor that goes into the event argument of Surv:

mkcens.Lexis <-
function(Lx, cens = "cens")
{
Rx <- Epi::sortLexis(Lx)
last <- !duplicated(Rx$lex.id, fromLast = TRUE)
Rx$lex.Xst <- ifelse(last & Rx$lex.Cst == Rx$lex.Xst,
                     cens,
                     as.character(Rx$lex.Xst))
Relevel(factorize(Rx), cens)
}

# The AaJ method
AaJ <- function(Lx, ...) UseMethod("AaJ")

# The actual function
AaJ.default <-
AaJ.Lexis <-
function(Lx,
         formula   = ~ 1,
         timeScale = 1, ...)
{
lex.id  <- NULL
lex.Cst <- NULL
if (!inherits(Lx, "Lexis"))
    stop("1st argument must be a Lexis object")
if( length(formula) != 2 )
    stop("'formula' must be a one-sided formula")

rhs <- as.character(formula[length(formula)]) # right hand side of formula
Lx <- mkcens.Lexis(Lx)

ts <- check.time.scale(Lx, timeScale)
Lx$zeit <- Lx[,ts]
cat("NOTE: Timescale is ", ts,
#   "; initial level assumed to be ", levels(Lx)[2],
    "\n", sep = "")

form <- Surv(Lx$zeit,
             Lx$zeit + Lx$lex.dur,
             Lx$lex.Xst) ~ 1
form[3] <- formula[2]
survfit(form,
          id = lex.id,
      istate = lex.Cst,
        data = Lx)
}

harm <-
function( x, ord=1, per=1, verbose=FALSE )
{
vn <- deparse(substitute(x))
if( inherits(x,"Date") )
  {
  x <- julian( x )
  if( missing(verbose) ) verbose <- TRUE
  if( missing(per) ) per <- 365.25
  if( verbose ) cat( "Note: Date variable", vn, "was converted to days.\n" )
  }
if( verbose ) cat("Note: Period taken to be", per, "\n" )
hm <- NULL
for( i in 1:ord )
   hm <- cbind( hm, sin(2*i*pi*x/per), cos(2*i*pi*x/per) )
colnames(hm) <- paste0( rep(c("sin","cos"),ord), rep(1:ord,each=2) )
return( hm )
}

fit.add <-
function(y, rates.frame, cov.frame, start)
{
  ## Modify covariate values
  rates.sum <- apply(rates.frame, 1, sum)
  cov.frame <- sweep(cov.frame, 1, rates.sum, "*")
  model.frame <- cbind(rates.frame, cov.frame)
  
  if (missing(start)) {
    glm.out <- fit.baseline(y, rates.frame)
    mu.inits <- predict(glm.out, type="response")
    glm.out <- glm(y~-1 + ., family=binomial(link=log),
                   data=model.frame, mustart=mu.inits, maxit=100)
  }
  else {
    glm.out <- glm(y~-1 + ., family=binomial(link=log),
                   data=model.frame, start=start, maxit=100)
  }
  return(list(rates=glm.out))
}

modLexis <-
function( Lx,
      nameLx,
     formula,
        from = preceding(Lx, to),
          to = absorbing(Lx),
      paired = FALSE,
        link = "log", scale = 1, verbose = TRUE,
       model,
         ... )
{
# a common wrapper for glm and gam modeling of Lexis FU
# is this a Lexis object ?
if( !inherits(Lx,"Lexis") )
    stop("The first argument must be a Lexis object.\n")

# check that events are actual levels of lex.Xst
if( is.numeric(to) ) to <- levels( Lx$lex.Xst )[to]
wh <- match(to, levels(Lx$lex.Xst))
if( any(is.na(wh)) )
    stop("'to' must be a subset of: '",
         paste(levels(Lx$lex.Xst), collapse="','", sep=""), "'\n")

# check that from are actual levels of lex.Cst
if( is.numeric(from) ) from <- levels( Lx$lex.Cst )[from]
wh <- match( from, levels(Lx$lex.Cst) )
if( any(is.na(wh)) )
    stop("'from' must be a subset of: '",
          paste(levels(Lx$lex.Cst), collapse="','", sep=""), "'\n")
# subset to these states
Lx <- Lx[Lx$lex.Cst %in% from,]

# first a small utility to show transitions as text:
trt <- function( f, t ) paste( f, "->", t, sep="" )

# work out which transitions are modeled
if( paired )
  {
if (length(from) != length(to))
    stop("If 'paired' is TRUE, from and to must have same length!\n")
if (any(from == to))
    stop("If 'paired' is TRUE, entries in from and to must be different within pairs\n")
trnam <- trt(from, to)
  } else {
tm <- tmat(Lx)[from, to, drop = FALSE]
trnam <- outer(rownames(tm), colnames(tm), trt)[tm > 0]
trnam <- trnam[!is.na(trnam)]
  }
# just for formatting the explanatory text
onetr <- length(trnam) == 1

# warn if a potentially silly model is defined
if (any((ts <- table(sapply(strsplit(trnam, "->"),
                            function(x) x[1]))) > 1)) cat(
 "NOTE:\nMultiple transitions *from* state '",
 paste(names(ts[ts>1]), collapse = "', '"),
 "' - are you sure?",
 "\nThe analysis requested is effectively merging outcome states.",
 "\nYou may want analyses using a *stacked* dataset - see ?stack.Lexis\n")

# construct the model formula - note that we already made sure that
# from and to are pairwise different
if( length(formula) != 2 ) stop("formula must be a one-sided formula")

form <- cbind( trt(Lx$lex.Cst,Lx$lex.Xst) %in% trnam,
               Lx$lex.dur ) ~ 1
form[3] <- formula[2]

from <- levels(factor(Lx$lex.Cst)) # only levels present in lex.Cst

# Scaling
Lx$lex.dur <- Lx$lex.dur / scale

# Tell what we intend to and then do it
if( verbose ){
cat(deparse(substitute(model)),
    " Poisson analysis of Lexis object ", nameLx, " with ", link, " link",
    ":\nRates for", if( onetr) " the", " transition",
                    if(!onetr) "s", ":", paste0("\n", trnam), "\n",
    if( scale!=1 ) paste(", lex.dur (person-time) scaled by", scale ), "\n", sep="" )
             }

# Fit the model
mod <- model( form, family = poisreg(link=link), data = Lx, ... )

# Add an explanatory attribute
attr( mod, "Lexis" ) <- list( data=nameLx,
                             trans=trnam,
                           formula=form[-2],
                             scale=scale )
mod
}

# Here are the actual functions of interest:
######################################################################
# the glm function
glmLexis <-
glm.Lexis <-
function( Lx,
     formula,
        from = preceding(Lx,to),
          to = absorbing(Lx),
      paired = FALSE,
        link = "log",
       scale = 1,
     verbose = TRUE,
         ... )
{
# name of the supplied object
nameLx <- deparse(substitute(Lx))

# sensible defaults if one of to and from is missing
if(  missing(from) & !missing(to) ) from <- preceding (Lx,to  )
if( !missing(from) &  missing(to) ) to   <- succeeding(Lx,from)
xx <- modLexis( Lx, nameLx,
                formula, from, to,
                paired = paired, link = link, scale = scale, verbose = verbose,
                 model = stats::glm, ... )
class( xx ) <- c( "glm.lex", class(xx) )
xx
}

######################################################################
# the gam function
gamLexis <-
gam.Lexis <-
function( Lx,
     formula,
        from = preceding(Lx, to),
          to = absorbing(Lx),
      paired = FALSE,
        link = "log",
       scale = 1,
     verbose = TRUE,
         ... )
{
# name of the supplied object
nameLx <- deparse(substitute(Lx))

# sensible defaults if one of the two is missing
if ( missing(from) & !missing(to)) from <- preceding (Lx, to  )
if (!missing(from) &  missing(to)) to   <- succeeding(Lx, from)
xx <- modLexis(Lx, nameLx, formula, from, to,
               paired = paired,
                 link = link,
                scale = scale,
              verbose = verbose,
                model = mgcv::gam, ...)
class(xx) <- c("gam.lex", class(xx))
xx
}

######################################################################
# And here is the coxph counterpart:
coxphLexis <-
coxph.Lexis <-
function( Lx, # Lexis object
     formula, # timescale ~ model
        from = preceding(Lx, to), # Exposure ('from' states)
          to = absorbing(Lx)    , # Events ('to' states)
      paired = FALSE,
     verbose = TRUE,
          ... )
{
# the usual crap to pass the check
lex.id <- NULL

# Lexis object ?
if( !inherits(Lx,"Lexis") )
    stop( "The first argument must be a Lexis object.\n")

# Convert numbers to state names
if (is.numeric(from)) from <- levels(Lx$lex.Cst)[from]
if (is.numeric(to))   to   <- levels(Lx$lex.Xst)[to]

# sensible defaults if only one of to and from is missing
if ( missing(from) & !missing(to)) from <- preceding (Lx, to  )
if (!missing(from) &  missing(to)) to   <- succeeding(Lx, from)

# name of the dataset
nameLx <- deparse(substitute(Lx))
# subset to the states we shall use
Lx <- Lx[Lx$lex.Cst %in% from,]

# work out which transitions are modeled
# first a small utility for annotation
trt <- function( f, t ) paste( f, "->", t, sep="" )
if( paired )
  {
if( length(from) != length(to) )
    stop("If 'paired' is TRUE, from and to must have same length!\n")
if( any(from==to) )
    stop("If 'paired' is TRUE, entries in 'from' and 'to'",
         " must be pairwise different\n")
trnam <- trt( from, to )
  } else {
tm <- tmat( Lx )[from,to,drop=FALSE]
trnam <- outer( rownames(tm), colnames(tm), trt )[tm>0]
trnam <- trnam[!is.na(trnam)]
  }
# just for formatting explanatory text
onetr <- length(trnam) == 1

# warn if a potentially silly model is defined
if( any( ts<-table(sapply( strsplit(trnam,"->"),
                           function(x) x[1] ))>1 ) ) cat(
 "NOTE:\nMultiple transitions *from* state '",names(ts[ts>1]),
 "' - are you sure?",
 "\nThe analysis requested is effectively merging outcome states.",
 "\nYou may want analyses using a *stacked* dataset - see ?stack.Lexis\n" )

# Correct formula?
if( length(formula) != 3 )
    stop("'formula' must be a 2-sided formula, with the l.h.s. the timescale")

# Is the l.h.s. a timescale?
ts <- as.character( formula[2] )
if( !(ts %in% (tms<-timeScales(Lx))) )
    stop( "l.h.s. of formula must be a timescale; one of:\n", tms, "\n" )

# What are the 'from' states actually present?
from <- levels( factor(Lx$lex.Cst) )

# construct a Surv response object, and note that we want the possibility
# of transitions to transient states, hence the lex.Xst != lex.Cst
Sobj <- Surv(Lx[,ts],
             Lx[,ts]+Lx$lex.dur,
             trt( Lx$lex.Cst, Lx$lex.Xst ) %in% trnam )

# Tell what we intend to and then do it
if( verbose ){
cat("survival::coxph analysis of Lexis object ", nameLx,
    ":\nRates for", if( onetr) " the", " transition",
                    if(!onetr) "s", ":", paste0("\n", trnam),
    "\nBaseline timescale: ", ts, "\n", sep="")
             }

mod <- coxph(as.formula(paste("Sobj",
                              as.character(formula[3]),
                              sep="~")),
             data = Lx,
               id = lex.id,
             ...)

# Add an explanatory attribute
attr( mod, "Lexis" ) <- list( data=nameLx,
                             trans=trnam,
                           formula=formula )
class( mod ) <- c( "coxph.lex", class(mod) )
mod
}

legendbox <-
 function(x, y,
      state = "State",
         py = "Person-time",
      begin = "no. begin",
        end = "no. end",
      trans = "Transitions",
      rates = "\n(Rate)",
       font = 1,
      right = !left,
       left = !right,
        ...)
{
btxt <- paste0(state, "\n",
                  py, "\n",
               begin, "          ", end)
ww <- strwidth (btxt) * 1.2
hh <- strheight(btxt) * 1.3
zz <- tbox(btxt,
           x = x, y = y,
           wd = ww,
           ht = hh,
         font = font,
         ...)
if (missing(right) & missing(left)) right = TRUE
if (right) text(x + ww / 1.8, y, paste0(trans, rates), adj = 0)
if (left)  text(x - ww / 1.8, y, paste0(trans, rates), adj = 1)
}

get.ests <- function( ests, ... )
{
 # If a model object is supplied, extract the parameters and the
 # standard errors
 #
if( inherits( ests, c("glm","coxph","clogistic","gnlm","survreg") ) )
    ests <- ci.exp( ests, ... )
else
if( inherits( ests, c("lm","gls","lme","nls","polr",
                      "mer","MIresult","mipo") ) )
    ests <- ci.lin( ests, ... )[,-(2:4)]
ests
}

plotEst <-
function( ests,
             y = dim(ests)[1]:1,
           txt = rownames(ests),
        txtpos = y,
          ylim = range(y)-c(0.5,0),
          xlab = "",
          xtic = nice( ests[!is.na(ests)], log=xlog ),
          xlim = range( xtic ),
          xlog = FALSE,
           pch = 16,
           cex = 1,
           lwd = 2,
           col = "black",
       col.txt = "black",
      font.txt = 1,
     col.lines = col,
    col.points = col,
          vref = NULL,
          grid = FALSE,
      col.grid = gray(0.9),
   restore.par = TRUE,
           ...
         )
{
# Function to plot estimates from a model.
# Assumes that ests is a p by 3 matrix with estimate, lo and hi as
# columns OR a model object.

 # Extract the estimates if necessary
 #
ests <- get.ests( ests, ... )

 # Is it likley that we want a log-axis for the parameters?
 #
mult <- inherits( ests, c("glm","coxph","gnlm") )
if( missing(xlog) ) xlog <- mult

 # Create an empty plot in order to access the dimension so that
 # sufficient place can be made for the text in the margin
 #
plot.new()
mx <- max( strwidth( txt, units="in" ) )
oldpar <- par( mai=par("mai") + c(0,mx,0,0) )
if( restore.par ) on.exit( par( oldpar ) )

 # Set up the coordinate system witout advancing a frame
 #
plot.window( xlim = xlim, ylim = ylim, log = ifelse( xlog, "x", "") )

 # Draw a grid if requested
 #
if( !is.logical( grid ) )           abline( v = grid, col = col.grid )
if(  is.logical( grid ) & grid[1] ) abline( v = xtic, col = col.grid )

 # Draw a vertical reference line
 #
if( !missing( vref ) ) abline( v = vref )

 # Draw the estimates with c.i. s
 #
linesEst( ests, y, pch=pch, cex=cex, lwd=lwd,
          col.points=col.points, col.lines=col.lines )

 # Finally the x-axis and the annotation of the estimates
 #
axis( side=1, at=xtic )
mtext( side=1, xlab, line=par("mgp")[1], cex=par("cex")*par("cex.lab") )
axis( side=2, at=txtpos, labels=txt, las=1, lty=0, col.axis=col.txt )
invisible( oldpar )
  }

pointsEst <-
linesEst <-
function( ests,
             y = dim(ests)[1]:1,
           pch = 16,
           cex = 1,
           lwd = 2,
           col = "black",
     col.lines = col,
    col.points = col,
           ...
         )
{
# Function to add estimates from a model to a drawing.
# Assumes that ests is a p by 3 matrix with estimate, lo and hi as
# columns.

  # Extract the estimates if necessary
  #
ests <- get.ests( ests, ... )

  # Cut the confidence interval lines to fit inside the plot
  # before drawing them.
  #
xrng <- if( par("xlog") ) 10^par("usr")[1:2] else par("usr")[1:2]
segments( pmax(ests[, 2],xrng[1]), y, pmin(ests[, 3], xrng[2]), y,
          lwd = lwd, col=col.lines )

  # Then the point estimates on top of the lines.
  #
points( ests[, 1], y, pch = pch, cex = cex, col=col.points )
invisible()
}

nice <-
function (x, log = FALSE, lpos = c(1, 2, 5), xmx = 4, ...)
{
if (log)
   {
   # not too extreme limits on log-scale
   fc <- floor(max(log10(min(x)), .1^xmx)):
       ceiling(min(log10(max(x)), 10^xmx))
   tick <- as.vector(outer(lpos, 10^fc, "*"))
   ft <- max(tick[tick < min(x)])
   lt <- min(tick[tick > max(x)])
   tick <- tick[tick >= ft & tick <= lt]
   if (length(tick) < 4 & missing(lpos))
      tick <- nice(x, log = T, lpos = c(1:9))
   if (length(tick) > 10 & missing(lpos))
      tick <- nice(x, log = T, lpos = 1)
   tick
   } else {
pretty(x, ...)
}
}

# Mantel-Haenszel estimate and test

mh <-
function(cases, denom, compare = 1, levels = c(1, 2), by = NULL,
	cohort = !is.integer(denom), confidence = 0.9)
{
	ndim <- length(dim(cases))
	edgin <- names(dimnames(cases))
	edgen <- paste("Dimension", 1:ndim)
	if (is.null(edgin))
		edges <- edgen
	else
		edges <- ifelse(edgin == "", edgen, edgin)
	if (is.null(edges)) edges <- rep("", ndim)
	if(length(dim(denom)) != ndim) {
		stop("Cases and Pyrs arrays of unequal dimension")
	}
	if(is.numeric(compare)) {
		comp <- as.integer(compare)
		if(comp < 1 || comp > ndim) {
			stop("Illegal argument: compare")
		}
	}
	else {
		comp <- (1:ndim)[edges == compare]
		if(length(comp) != 1) {
			stop("Illegal argument: compare")
		}
	}
	if(!is.null(by)) {
		if (!is.numeric(by)) {
			mtch <- match(by, edges)
			if (any(is.na(mtch))) {
				stop("Illegal argument: by")
			}
			by <- (1:ndim)[mtch]
		}
		if (any(by < 1 | by > ndim | by == comp)) {
			stop("Illegal argument: by")
		}
	}
	gtxt <- vector("character", 3)
	gtxt[1] <- edges[comp]
	gtxt[2] <- dimnames(cases)[[comp]][levels[1]]
	gtxt[3] <- dimnames(cases)[[comp]][levels[2]]
	ctxt <- edges[-c(comp, by)]
	if (length(ctxt) == 0) ctxt <- as.null()
	others <- (1:ndim)[ - comp]
	select <- function(a, el)
	{
		b <- a[el]
		ifelse(is.na(b), 0, b)
	}
	d1 <- apply(cases, others, select, el = levels[1])
	d2 <- apply(cases, others, select, el = levels[2])
	if(length(d1) == 0 || length(d2) == 0) {
		stop("Illegal argument: levels")
	}
	y1 <- apply(denom, others, select, el = levels[1])
	y2 <- apply(denom, others, select, el = levels[2])
	d <- d1 + d2
	y <- y1 + y2
	if (cohort) {
		qt <- ifelse(y>0, (d1 * y2)/y, 0)
		rt <- ifelse(y>0, (d2 * y1)/y, 0)
		ut <- ifelse(y>0, d1 - ((d * y1)/y), 0)
		vt <- ifelse(y>0, (d * y1 * y2)/(y^2), 0)
	}
	else {
		s1 <- d1 + y1
		s2 <- d2 + y2
		t <- s1 + s2
		qt <- ifelse(t>1, (d1 * y2)/t, 0)
		rt <- ifelse(t>1, (d2 * y1)/t, 0)
		ut <- ifelse(t>1, d1 - ((d * s1)/t), 0)
		vt <- ifelse(t>1, (d * y * s1 * s2)/((t - 1) * (t^2)), 0)
	}
	if(!is.null(by)) {
		if(length(by) < ndim - 1) {
			nby <- match(by, others)
			q <- apply(qt, nby, sum)
			r <- apply(rt, nby, sum)
			u <- apply(ut, nby, sum)
			v <- apply(vt, nby, sum)
		}
		else {
			q <- qt
			r <- rt
			u <- ut
			v <- vt
		}
	}
	else {
		q <- sum(qt)
		r <- sum(rt)
		u <- sum(ut)
		v <- sum(vt)
	}
	rr <- q/r
	se <- sqrt(v/(q * r))
	ch <- (u^2)/v
	ef <- exp( - qnorm((1 - confidence)/2) * se)
	if (cohort)
		ty <- "Rate ratio"
	else
		ty <- "Odds ratio"
	res <- list(groups = gtxt, control = ctxt, type=ty,
		q=q, r=r, u=u, v=v,
		ratio = rr, se.log.ratio = se, cl.lower = rr/ef,
		cl.upper = rr * ef, chisq = ch, p.value = 1 - pchisq(
		ch, 1))
	class(res) <- "mh"
	res
}

print.mh <-
    function(x, ...)
    {
	cat("\n")
	if (!is.null(x$control))
		cat("\nMantel-Haenszel comparison for: ")
	else
		cat("Comparison for: ")
	cat(x$groups[1], " (", x$groups[2], "versus", x$groups[3], ")\n")
	if (!is.null(x$control))
		cat("controlled for:", x$control, "\n")
	cols <- c(x$type, "CL (lower)", "CL (upper)",
		"Chisq (1 df)", "p-value")
	nr <- length(x$ratio)
	if (is.array(x$ratio)) {
		dnt <- dimnames(x$ratio)
		size <- dim(x$ratio)
		nw <- length(dnt)
	}
	else {
		rn <- names(x$ratio)
		if (length(rn) > 1)
			dnt <- list(names(x$ratio))
		else
			dnt <- list("")
		size <- nr
		nw <- 1
	}
	dno <- vector("list", nw+1)
	so <- vector("numeric", nw+1)
	dno[[1]] <- dnt[[1]]
	dno[[2]] <- cols
	so[1] <- size[1]
	so[2] <- 5
	if (nw > 1) for (i in 2:nw) {
		dno[[i+1]] <- dnt[[i]]
		so[i+1] <- size[i]
	}
	s1 <- size[1]
	tab <- cbind(x$ratio, x$cl.lower, x$cl.upper, x$chisq, x$p.value)
#		as.matrix(x$ratio, nrow=s1),
#		as.matrix(x$cl.lower, nrow=s1),
#		as.matrix(x$cl.upper, nrow=s1),
#		as.matrix(x$chisq, nrow=s1),
#		as.matrix(x$p.value, nrow=s1) )

	print(array(tab, dim=so, dimnames=dno))
	if (nr > 1) {
		Q <- sum(x$q)
		R <- sum(x$r)
		cat("\nOverall Mantel-Haenszel estimate of", x$type, ":",
			format(Q/R))
		h <- sum(((x$q*R-x$r*Q)^2)/x$v)/(Q*R)
		df <- sum(x$v>0)-1
		cat("\nChi-squared test of heterogeneity:", format(h),
			"(",df," df), p =", format(1-pchisq(h, df)), "\n")
	}
	cat("\n")
}

# Power calculations

mh.power <- function(mh, ratio, alpha=0.05) {
	n.se <- log(ratio)/mh$se.log.ratio
	pnorm(n.se - qnorm(1-alpha/2))
}

ncut <- function( x, breaks, type="left" )
{
# Sorting to get the opportunity to call the function recursively.
breaks <- sort( breaks )
# Get the indices, but fix the 0 indices to produce NAs:
fi <- findInterval( x, breaks )
fi[fi==0] <- length( breaks ) + 1
switch( toupper( substr( type, 1, 1 ) ),          
        "L" = breaks[fi],                         
        "R" = -ncut( -x, -breaks ),                       
        "M" = ( breaks[fi] - ncut( -x, -breaks ) ) / 2 )
}

## Program to calculate effects
## Michael Hills
## Improved by Bendix Carstensen and Martyn Plummer
## Post Tartu 2007 version June 2007
## Addition allowing a TRUE/FALSE as binary outcome
## and possibility or relative risk for binary outcomes and rate
## difference for failure outcomes, BxC, Ocitober 2012

effx<-function(response,
type="metric",
fup=NULL,
exposure,
strata=NULL,
control=NULL,
weights=NULL,
eff=NULL,
alpha=0.05,
base=1,
digits=3,
data=NULL)
{
  ## stores the variable names for response, etc.

    rname<-deparse(substitute(response))
    ename<-deparse(substitute(exposure))
    if (!missing(strata)) {
      sname<-deparse(substitute(strata))
    }

  ## The control argument is more complex, as it may be a name or
  ## list of names

  if(!missing(control)) {
    control.arg <- substitute(control)
  }

  ## Match the type argument
  type <- match.arg(type, c("metric", "failure", "count", "binary"))

  ## Check for missing arguments
  if (missing(response))
    stop("Must specify the response","\n")
  if (missing(exposure))
    stop("Must specify the exposure","\n")
  if (type == "failure" && missing(fup)) {
    stop("Must specify a follow-up variable when type is failure")
  }

  ## performs a few other checks
  if(rname==ename)stop("Same variable specified as response and exposure")
  if (!missing(strata)) {
    if(rname==sname)stop("Same variable specified as response and strata")
    if(sname==ename)stop("Same variable specified as strata and exposure")
  }


  ## If data argument is supplied, evaluate the arguments in that
  ## data frame.

  if (!missing(data)) {
    exposure <- eval(substitute(exposure), data)
    response <- eval(substitute(response), data)
    if (!missing(strata))
      strata <- eval(substitute(strata), data)
    if (!missing(control))
      control <- eval(substitute(control), data)
    if (!missing(fup))
      fup <- eval(substitute(fup), data)
    if (!missing(weights)) {
      weights <- eval(substitute(weights), data)
    }
  }

  ## Now check validity of evaluated arguments

  if(is.logical(response)) response <- as.numeric(response)
  if(!is.numeric(response))
    stop("Response must be numeric, not a factor")

  if (!missing(weights) && type != "binary") {
    stop("weights only allowed for a binary response")
  }

  if (!missing(strata) && !is.factor(strata))
    stop("Stratifying variable must be a factor")

  if(type=="binary") {
    if( is.null(eff) ) eff<-"OR"
    if( !(eff %in% c("OR","RR") ) ) stop( "Only RR and OR allowed for binary response" )
    response <- as.numeric(response)
    tmp<-(response==0 | response==1)
    if(all(tmp,na.rm=TRUE)==FALSE)
      stop("Binary response must be logical or coded 0,1 or NA")
  }

  # If a count is given we are actually using the fup
  if(type=="count") fup<-is.na(response)*1

  if(type=="failure") {
    if( is.null(eff) ) eff<-"RR"
    response <- as.numeric(response)
    tmp<-(response==0 | response==1)
    if(all(tmp,na.rm=TRUE)==FALSE)
      stop("Failure response must be logical or coded 0,1 or NA")
  }

  ## If exposure is an ordered factor, drops the order.

  if(class(exposure)[1]=="ordered") {
    exposure<-factor(exposure, ordered=F)
  }

  ## Fix up the control argument as a named list
  if (!missing(control)) {
      if (is.list(control)) {
          control.names <- sapply(control.arg, deparse)
          if (control.names[1] == "list" &&
              length(control.names) == length(control) + 1)
          {
              control.names  <- control.names[-1]
          }
          else {
              control.names <-
                  paste0(deparse(control.arg), "[", 1:length(control), "]")
          }
          names(control) <- control.names
    }
    else {
        control <- list(control)
        names(control) <- deparse(control.arg)
    }
  }

  ## prints out some information about variables
  cat("---------------------------------------------------------------------------","\n")
  cat("response      : ", rname, "\n")
  cat("type          : ", type, "\n")
  cat("exposure      : ", ename, "\n")
  if(!missing(control))cat("control vars  : ",names(control),"\n")
  if(!missing(strata)) {
    cat("stratified by : ",sname,"\n")
  }
  cat("\n")
  if(is.factor(exposure)) {
    cat(ename,"is a factor with levels: ")
    cat(paste(levels(exposure),collapse=" / "),"\n")
    exposure <- Relevel( exposure, base )
    cat( "baseline is ", levels( exposure )[1] ,"\n")
  }
  else {
    cat(ename,"is numeric","\n")
  }
  if(!missing(strata)) {
    cat(sname,"is a factor with levels: ")
    cat(paste(levels(strata),collapse="/"),"\n")
  }
  if(type=="metric")cat("effects are measured as differences in means","\n")
  if(type=="binary")
    {
    if( eff=="OR" | is.null(eff))cat("effects are measured as odds ratios","\n")
    if( eff=="RR" )cat("effects are measured as relative risk","\n")
    }
  if(type=="failure")
    {
    if( eff=="RR" | is.null(eff))cat("effects are measured as rate ratios","\n")
    if( eff=="RD" )cat("effects are measured as rate differences","\n")
    }
  cat("---------------------------------------------------------------------------","\n")
  cat("\n")

  ## translates type of response and choice of eff into family
  if ( type=="metric")                  family<-gaussian
  if ( type=="binary")                  family<-binomial(link=logit)
  if ( type=="failure" | type=="count") family<-poisson(link=log)
  if ( !is.null(eff) )
     {
  if (type=="binary"  & eff=="RR")   family<-binomial(link=log)
  if (type=="failure" & eff=="RD")   family<-poisson(link=identity)
     }
  ## gets number of levels for exposure if a factor

  if(is.factor(exposure)) {
    nlevE<-length(levels(exposure))
  }

  ## labels the output

  if(is.factor(exposure)) {
    cat("effect of",ename,"on",rname,"\n")
  }
  else {
    cat("effect of an increase of 1 unit in",ename,"on",rname,"\n")
  }
  if(!missing(control)) {
    cat("controlled for",names(control),"\n\n")
  }
  if(!missing(strata)) {
    cat("stratified by",sname,"\n\n")
  }


  ## no stratifying variable

  if(missing(strata)) {
    if(type=="metric") {
      if(missing(control)) {
        m<-glm(response~exposure,family=family)
        cat("number of observations ",m$df.null+1,"\n\n")
        mm<-glm(response~1,family=family,subset=!is.na(exposure))
      }
      else  {
        m<-glm(response~.+exposure,family=family,
               subset=!is.na(exposure),data=control)
        cat("number of observations ",m$df.null+1,"\n\n")
        mm<-glm(response~.,family=family,
                subset=!is.na(exposure),data=control)
      }
      res<-ci.lin(m,subset=c("Intercept","exposure"),alpha=alpha)
      res<-res[,c(1,5,6)]
    }
    if(type=="binary") {
      if(missing(control)) {
        m<-glm(response~exposure,family=family,weights=weights)
        cat("number of observations ",m$df.null+1,"\n\n")
        mm<-glm(response~1,family=family,subset=!is.na(exposure),weights=weights)
      }
      else  {
        m<-glm(response~.+exposure,family=family,
               subset=!is.na(exposure),data=control,weights=weights)
        cat("number of observations ",m$df.null+1,"\n\n")
        mm<-glm(response~.,family=family,
                subset=!is.na(exposure),data=control,weights=weights)
      }
      res<-ci.lin(m,subset=c("Intercept","exposure"),Exp=TRUE,alpha=alpha)
      res<-res[,c(5,6,7)]
    }
    if (type=="failure" | type=="count") {
      if (missing(control)) {
        m<-glm(response/fup~exposure,weights=fup,family=family)
        cat("number of observations ",m$df.null+1,"\n\n")
        mm<-glm(response/fup~1,weights=fup,family=family,
                subset=!is.na(exposure))
      }
      else  {
        m<-glm(response/fup~.+exposure,weights=fup,family=family, data=control)
        cat("number of observations ",m$df.null+1,"\n\n")
        mm<-glm(response/fup~.,weights=fup,family=family,
                subset=!is.na(exposure),data=control)
      }
      res<-ci.exp(m,subset=c("Intercept","exposure"),alpha=alpha,Exp=(eff=="RR"))
    }
    res<-signif(res,digits)
    colnames(res)[1]<-c("Effect")
    if(is.factor(exposure)) {
      ln <- levels(exposure)
      rownames(res)[2:nlevE]<-paste(ln[2:nlevE],"vs",ln[1])
    }
    aov <- anova(mm,m,test="Chisq")
    print( res[-1,] )
    cat("\nTest for no effects of exposure on",
        aov[2,3],"df:",
        "p-value=",format.pval(aov[2,5],digits=3),"\n")
    invisible(list(res,paste("Test for no effects of exposure on",
                             aov[2,3],"df:","p-value=",format.pval(aov[2,5],digits=3))))

  }
  ## stratifying variable
  if(!missing(strata)) {
    sn <- levels(strata)
    nlevS<-length(levels(strata))
    if(type=="metric") {
      if(missing(control)) {
        m<-glm(response~strata/exposure,family=family)
        cat("number of observations ",m$df.null+1,"\n\n")
        mm<-glm(response~strata+exposure,family=family)
      }
      else {
        m <-glm(response~strata/exposure + .,family=family,
                data=control)
        cat("number of observations ",m$df.null+1,"\n\n")
        mm <-glm(response~strata+exposure + .,family=family,
                 data=control)
      }
      res<-ci.lin(m,subset=c("strata"),alpha=alpha)[c(-1:-(nlevS-1)),c(1,5,6)]
    }
    if(type=="binary") {
      if(missing(control)) {
        m<-glm(response~strata/exposure,family=family,weights=weights)
        cat("number of observations ",m$df.null+1,"\n\n")
        mm<-glm(response~strata+exposure,family=family,weights=weights)
      }
      else {
        m <-glm(response~strata/exposure + .,family=family,
                data=control,weights=weights)
        cat("number of observations ",m$df.null+1,"\n\n")
        mm <-glm(response~strata+exposure + .,family=family,
                 data=control,weights=weights)
      }
      res<-ci.exp(m,subset=c("strata"),alpha=alpha)[c(-1:-(nlevS-1)),]
    }
    if (type=="failure" | type=="count") {
      if(missing(control)) {
        m<-glm(response/fup~strata/exposure,weights=fup,family=family)
        cat("number of observations ",m$df.null+1,"\n\n")
        mm<-glm(response~strata+exposure,weights=fup,family=family)
      }
      else {
        m <-glm(response/fup~.+strata/exposure,weights=fup,family=family,data=control)
        cat("number of observations ",m$df.null+1,"\n\n")
        mm<-glm(response/fup~.+strata+exposure,weights=fup,family=family,data=control)
      }
      res<-ci.exp(m,subset=c("strata"),alpha=alpha)[c(-1:-(nlevS-1)),]
    }

    res<-signif(res,digits)
    colnames(res)[1]<-c("Effect")
    if(is.factor(exposure)) {
      ln<-levels(exposure)
      newrownames<-NULL
      for(i in c(1:(nlevE-1))) {
        newrownames<-c(newrownames,
                       paste("strata",sn[1:nlevS],"level",ln[i+1],"vs",ln[1]))
      }
    }
    else {
      newrownames<-paste("strata",sn[1:nlevS])
    }
    rownames(res)<-newrownames
    aov<-anova(mm,m,test="Chisq")
    print( res )
    cat("\nTest for effect modification on",
        aov[2,3],"df:","p-value=",format.pval(aov[2,5],digits=3),"\n")
    invisible(list(res,paste("Test for effect modification on",
                             aov[2,3],"df:","p-value=",format.pval(aov[2,5],digits=3))))

  }
}

# Utility: which records can be merged with the next and still have
# lex.dur < lim. Assumes that variable keep.rec is in Lx
close2next <-
function(Lx, lim)
{
if (length(lim) == 1) lim <- c(lim, 3 * lim)
setdiff(
with(Lx, which(lex.Cst == lex.Xst           &
                lex.id == c(lex.id[-1], NA) &
                lex.dur < lim[1]            &
               (lex.dur + c(lex.dur[-1], NA))
                        < lim[2])),
         which(Lx$keep.rec) - 1)
}

# Utility: merge records in wh with the records in wh+1
mergeWnext <-
function(Lx, wh)
{
if (any(near(diff(wh), 1))) stop("consecutive 'wh's not allowed\n")
Lx$lex.Xst[wh] <-                  Lx$lex.Xst[wh + 1]
Lx$lex.dur[wh] <- Lx$lex.dur[wh] + Lx$lex.dur[wh + 1]
Lx[-(wh + 1),] 
}

# Utility: remove those indices that should not be used (i.e. merged
# with the next one) by removing every other among consecutive indices 
thinWh <-
function(wh)
{
dwh <- diff(c(-1, wh)) # difference between indices; any negative
                       # number will work
is1 <- near(dwh, 1)             # those with a usable successor
st1 <- near(diff(c(0, is1)), 1) # start of runs of 1
dfr <- data.frame(is1 = as.numeric(is1)) 
sq1 <- (group_by(dfr, cumsum(st1)) %>%
        mutate(z = cumsum(is1) * is1))$z
# sq1 now has runs of 1s numbered 0,1,2,3, and stand alone numbers are
# 0, so we just remove the components of wh with odd values of sq1:
wh[(sq1 %% 2) != 1]
}

# Here is the function it's all about.
coarse.Lexis <-
function(Lx, # Lexis object
        lim, # limits for proximity
       keep = FALSE) # indicator of records not combine with any previous
{
if (length(keep) != 1 &
    length(keep) != nrow(Lx)) stop("keep must have length 1 or nrow(Lx)") 
Lx$keep.rec <- keep

Lx <- sortLexis(Lx)
wh <- close2next(Lx, lim = lim)
while (length(wh > 0))
    {
wh <- thinWh(wh)        
Lx <- mergeWnext(Lx, wh)
wh <- close2next(Lx, lim = lim)
    }
Lx
}

# The Relevel method
Relevel <- function (x, ...) UseMethod("Relevel")

# Utility to group a factor from a 2-column table;
# called from Relevel if second argument is a matrix or table
tRelevel <-
function( ff, # factor to be grouped
          gg, # 2-column matrix or table with levels resp. grouping
     xlevels = TRUE,  # include also grouped levels not present in ff
     nogroup = TRUE ) # levels of ff not grouped are tranferred (if FALSE set to NA)
{
if( any( wh <- ( apply( table( gg[,1], gg[,2] )>0, 1, sum )>1 ) ) )
    stop( "\nFactor level", if(sum(wh)>1) "s", ": \n",
          paste(names(wh)[wh],collapse=" / "),
          "\nis grouped to more than one group.\n" )
if( any( is.na( match( unique(as.character(ff)),
                       unique(as.character(gg[,1])) ) ) ) )
    cat( "Note: Some values of factor not among levels grouped.\n" )
if( any( wh <- ( apply( table( gg[,1], gg[,2] ), 1, sum )>1 ) ) )
    warning( "Factor level", if(sum(wh)>1) "s", ": ",
             paste(names(wh)[wh],collapse=" / "),
             "\nappear more than once in the table of groupings." )
# indices and names of the original factor levels
ixff <- as.integer( ff )
chff <- as.character( ff )
# where they are in the translation table
ixg1 <- as.integer( factor( gg[,1], levels=levels(ff) ) )
# indices of the new levels in the translation table
ixg2 <- as.integer( g2 <- factor(gg[,2]) )
# remove unwanted NAs (levels in g[,1] not present in ff)
ixg2 <- ixg2[!is.na(ixg1)]
ixg1 <- ixg1[!is.na(ixg1)]
# where in ixg2 are the integers ixff - match(), choose those values
grff <- levels(g2)[ixg2[match(ixff,ixg1)]]
# transfer non-grouped levels of ff
if( nogroup ) grff[is.na(grff)] <- chff[is.na(grff)]
# a factor with the correct levels in the correct order
grff <- factor( grff, levels=union(levels(g2),levels(factor(grff))) )
# keep all levels from second column or not?
if( xlevels ) grff else factor(grff)
}

# The factor method is the default method
Relevel.default <-
Relevel.factor <-
function( x, ref, first = TRUE,
               collapse = "+",
                xlevels = TRUE,
                nogroup = TRUE,
                    ... )
  {
  # Function that collapses multiple sets of levels of a factor
  #
  # If ref is a 2-dim structure
  if( is.matrix(    ref) |
      is.table(     ref) |
      is.array(     ref) |
      is.data.frame(ref) )
      {
   if( length(dim(ref)) !=2 ) stop("ref must be 2-dimensional\n")
   if( dim(ref)[2] < 2 ) stop("ref must have at least 2 colums\n")
   return( tRelevel( x, ref, xlevels, nogroup ) )    
  } else {
      
  # Otherwise use the old version
  #    
  if( !is.factor(x) )
    {
    argnam <- deparse( substitute(x) )
    f <- factor( x )
    cat( "WARNING: ", argnam,
         "has been converted to a factor with levels:\n",
         levels( f ) )
    }
  else
    f <- x

  # This is a copy of the relevel function from the base package:
  #
  relev <- function (x, ref, ...)
  {
    lev <- levels(x)
    if ( is.character( ref ) )
         ref <- match(ref, lev)
    if ( any( is.na( ref ) ) )
         stop( "any values in ref must be an existing level\n" )
    nlev <- length( lev )
    if ( any( ref < 1 ) || any( ref > nlev ) )
         stop( paste( "ref=", paste( ref, collapse="," ),
                      ": All elements must be in 1:", nlev, sep="" ) )
    factor(x, levels = lev[c(ref, seq(along = lev)[-ref])])
  }

  # If called with a non-list argument assume reshuffling of levels
  #
  if( !is.list( ref ) )
    fnew <- relev( f, ref )

  # If called with a list collapse levels in each list element.
  #
  if( is.list( ref ) )
    {
      fnew <- f
      newnames <- levels( f )
      uninames <- character( length( ref ) )
      for( s in 1:length(ref) )
         if ( is.character( ref[[s]] ) ) ref[[s]] <- match( ref[[s]], levels(f) )
      # Check for replicates in levels to be grouped
      if( any( (tt<-table(unlist(ref))) > 1 ) )
        stop("Factor level(s) ", levels( f )[as.numeric(names(tt)[tt>1])],
             " has been allocated to more than one new level.")
      for( s in 1:length(ref) )
         {
         uninames[s] <- if( is.null( names( ref ) ) )
                          {
                          paste( levels( f )[ref[[s]]], collapse=collapse )
                          }
                        else if( names( ref )[s]=="" )
                          {
                          paste( levels( f )[ref[[s]]], collapse=collapse )
                          }
                        else names( ref )[s]
          newnames[ref[[s]]] <- rep( uninames[s], length( ref[[s]] ) )
          }
      levels( fnew ) <- newnames
      if( !is.null( first ) )
        {
      if( !first ) fnew <- factor( fnew, c( levels( f )[-unlist( ref )], uninames ) )
      if(  first ) fnew <- factor( fnew, c( uninames, levels( f )[-unlist( ref )] ) )
        }
    }

  # This is in order to merge levels with identical names
  #
  return( factor( fnew, levels=levels(fnew) ) )
  }
}

splitLexis.1D <-
function(lex, breaks, time.scale, tol)
{
    time.scale <- check.time.scale(lex, time.scale)

    ## Entry and exit times on the time scale that we are splitting
    time1 <- lex[,time.scale, drop=FALSE]
    time2 <- time1 + lex$lex.dur

    ## Augment break points with +/- infinity
    breaks <- sort( unique( breaks ) )
    I1 <- c(-Inf, breaks)
    I2 <- c(breaks,Inf)

    ## Arrays containing data on each interval (rows) for each subject (cols)
    en <- apply(time1, 1, pmax, I1)     # Entry time
    ex <- apply(time2, 1, pmin, I2)     # Exit time
    NR <- nrow(en)
    NC <- ncol(en)

    ## Does subject contribute follow-up time to this interval?
    ## (intervals shorter than tol are ignored)
    valid <- en < ex - tol
    dur <- ex - en; dur[!valid] <- 0    # Time spent in interval

    ## Cumulative time since entry at the start of each interval
    time.since.entry <- rbind(0, apply(dur,2,cumsum)[-NR,,drop=FALSE])

    cal.new.entry <- function(entry.time) {
        sweep(time.since.entry, 2, entry.time, "+")[valid]
    }

    old.entry <- lex[, timeScales(lex), drop=FALSE]
    new.entry <- lapply(old.entry, cal.new.entry)

    ## Status calculation
    aug.valid <- rbind(valid, rep(FALSE, NC))
    last.valid <- valid & !aug.valid[-1,]
    any.valid <- apply(valid,2,any)

    new.Xst <- matrix( lex$lex.Cst, NR, NC, byrow=TRUE)
    new.Xst[last.valid] <- lex$lex.Xst[any.valid]

    n.interval <- apply(valid, 2, sum)
    new.lex <- Lexis("entry" = new.entry,
                     "duration" = dur[valid],
                     "id" = rep(lex$lex.id, n.interval),
                     "entry.status" = rep(lex$lex.Cst, n.interval),
                     "exit.status" = new.Xst[valid])

    ## Update breaks attribute and tranfer time.since attribute
    breaks.attr <- attr(lex, "breaks")
    breaks.attr[[time.scale]] <- sort(c(breaks.attr[[time.scale]], breaks))
    attr(new.lex, "breaks") <- breaks.attr
    attr(new.lex, "time.since") <- attr(lex, "time.since")
    return(new.lex)
}


splitLexis <- function(lex, breaks, time.scale=1, tol= .Machine$double.eps^0.5)
{
  ## Advise the uninformed user...
  if( inherits(lex,"stacked.Lexis") )
    stop( "It makes no sense to time-split after stacking ---\n",
    "split your original Lexis object and stack that to get what you want.\n")


  ## Set temporary, unique, id variable
  lex$lex.tempid <- lex$lex.id
  lex$lex.id <- 1:nrow(lex)

  ## Save auxiliary data
  aux.data.names <- setdiff(names(lex), timeScales(lex))
  aux.data.names <- aux.data.names[substr(aux.data.names,1,4) != "lex."]
  aux.data <- lex[, c("lex.id","lex.tempid", aux.data.names), drop=FALSE]

  ## Check for NAs in the timescale
  ts <- check.time.scale(lex, time.scale)
  ts.miss <- any(is.na(lex[,ts]))
  if( ts.miss )
    {
    na.lex <- lex[ is.na(lex[,ts]),]
       lex <- lex[!is.na(lex[,ts]),]
    cat( "Note: NAs in the time-scale \"", ts, "\", you split on\n")
    }

  ## If states are factors convert to numeric while splitting
  factor.states <- is.factor( lex$lex.Cst )
  if( factor.states )
    {
    state.levels <- levels( lex$lex.Cst )
    nstates     <- nlevels( lex$lex.Cst )
    lex$lex.Cst <- as.integer( lex$lex.Cst )
    lex$lex.Xst <- as.integer( lex$lex.Xst )
    }

  ## Split the data
  lex <- splitLexis.1D(lex, breaks, time.scale, tol)

  ## Reinstitute the factor levels
  if( factor.states )
    {
    lex$lex.Cst <- factor(lex$lex.Cst, levels = 1:nstates, labels = state.levels)
    lex$lex.Xst <- factor(lex$lex.Xst, levels = 1:nstates, labels = state.levels)
    }

  ## Put the NA-rows back
  if( ts.miss ) lex <- rbind( lex, na.lex[,colnames(lex)] )

  ## Save attributes
  lex.attr <- attributes(lex)
  ## Merge
  lex <- merge.data.frame(lex, aux.data, by="lex.id")
  ## Restore attributes
  attr(lex,"breaks") <- lex.attr$breaks
  attr(lex,"time.scales") <- lex.attr$time.scales
  attr(lex,"time.since") <- lex.attr$time.since
  class(lex) <- c("Lexis", "data.frame")
  ## Restore id variable
  lex$lex.id <- lex$lex.tempid
  lex$lex.tempid <- NULL

  return(lex)
}

# Functions to facilitate analysis of multistate models

# The stack method is already defined (in the utils package)
# and hence imported in the NAMESPACE file

stack.Lexis <-
function(x, ...)
{
## Function to stack obervations for survival analysis
## Make sure that lex.Cst and lex.Xst are factors with identical levels
x <- factorize(x)

## Same covariates
xx <- data.frame( cbind( x, lex.Tr="", lex.Fail=FALSE ) )[NULL,]
tm <- tmat.Lexis( x )
for( fst in levels(factor(x$lex.Cst)) )
for( tst in levels(factor(x$lex.Xst)) )
   if( !is.na(tm[fst,tst]) )
     {
     tr = paste( fst, "->", tst , sep="" )
     zz <- x[x$lex.Cst==fst,]
     xx <- rbind( xx, data.frame( zz, lex.Tr=tr, lex.Fail=(zz$lex.Xst==tst) ) )
     }
xx$lex.Tr <- factor(xx$lex.Tr)
## Reshuffle the variables
wh.om <- match( "lex.Xst", names(xx) )
wh.rl <- match( c("lex.Tr","lex.Fail"), names(xx) )
xx <- xx[,c(1:wh.om,wh.rl,(wh.om+1):(wh.rl[1]-1))]
attr(xx,"breaks") <- attr(x, "breaks")
attr(xx,"time.scales") <- attr(x, "time.scales")
class( xx ) <- c("stacked.Lexis","data.frame")
xx
}

subset.stacked.Lexis <- function(x, ... )
{
  y <- subset.data.frame(x, ...)
  attr(y,"breaks") <- attr(x, "breaks")
  attr(y,"time.scales") <- attr(x, "time.scales")
  class(y) <- c("stacked.Lexis","data.frame")
  return(y)
}

transform.stacked.Lexis <- function(`_data`, ... )
{
    save.at <- attributes(`_data`)
    ## We can't use NextMethod here because of the special scoping rules
    ## used by transform.data.frame
    y <- base::transform.data.frame(`_data`, ...)
    save.at[["names"]] <- attr(y, "names")
    attributes(y) <- save.at
    y
}

# The tmat method
tmat <- function (x, ...) UseMethod("tmat")

tmat.Lexis <-
function( x, Y = FALSE, mode = "numeric", ... )
{
zz <- table(x$lex.Cst, x$lex.Xst)
class(zz) <- "matrix"
if( Y )
  {
  diag(zz) <- summary(x, simplify=FALSE )[[1]][1:nrow(zz),"Risk time:"]
  } else {
         diag(zz) <- NA
         }
zz[zz==0] <- NA
if( mode != "numeric" ) zz <- !is.na(zz)
zz
}

Life.lines <- 
function( entry.date = NA,
           exit.date = NA,
          birth.date = NA,
           entry.age = NA,
            exit.age = NA,
           risk.time = NA
         )
{
# A function allowing any three of the arguments to be specified
# and yet returns enty age and -time and exit age and -time.

# Check if any variable is supplied with class
if( conv <- any( inherits( entry.date, "Date"     ),
                 inherits(  exit.date, "Date"     ),
                 inherits( birth.date, "Date"     ),
                 inherits( entry.age , "difftime" ),
                 inherits(  exit.age , "difftime" ),
                 inherits(  risk.time, "difftime" ) ) )
  {
  # Convert "Date" and "difftime" to years
  if( inherits( entry.date, "Date"     ) )  entry.date <- as.numeric( entry.date ) / 365.35 + 1970
  if( inherits(  exit.date, "Date"     ) )   exit.date <- as.numeric(  exit.date ) / 365.35 + 1970
  if( inherits( birth.date, "Date"     ) )  birth.date <- as.numeric( birth.date ) / 365.35 + 1970
  if( inherits( entry.age , "difftime" ) )  entry.age  <- as.numeric( entry.age  ) / 365.35
  if( inherits(  exit.age , "difftime" ) )   exit.age  <- as.numeric(  exit.age  ) / 365.35
  if( inherits(  risk.time, "difftime" ) )   risk.time <- as.numeric(  risk.time ) / 365.35
  # Convert to numeric
  class( entry.date ) <- "numeric"
  class(  exit.date ) <- "numeric"
  class( birth.date ) <- "numeric"
  class( entry.age  ) <- "numeric"
  class(  exit.age  ) <- "numeric"
  class(  risk.time ) <- "numeric"
  }
  
# Find out which three items are supplied.
#
wh <- (1:6)[!is.na( list( entry.date,
  		          entry.age,
  		           exit.date,
                           exit.age,
  		   	  birth.date,
                           risk.time ) )]

# Matrix of relevant quantities.
#
LL <- rbind( entry.date,
             entry.age,
              exit.date,
              exit.age,
             birth.date,
              risk.time )

# Matrix giving the three constraints among the six quantities:
#
M <- rbind( c( -1, 1,  0,  0, 1, 0 ),
            c(  0, 0, -1,  1, 1, 0 ),
            c(  0, 1,  0, -1, 0, 1 ) )

# Now in principle we have that M %*% LL = 0.
# Partitioning M=(A1|A2), t(LL)=(t(x1),t(x2)) 
# this gives A1 %*% x1 = -A2 %*% x2

# Check if there is sufficient information  
#
if( qr( M[,-wh[1:3]] )$rank < 3 ) 
   cat( "Insufficient information to display life lines" )

# Then do the calculation
#
A1 <- M[, wh[1:3]]
A2 <- M[,-wh[1:3]]
x1 <- LL[wh[1:3],]

x2 <- -solve( A2 ) %*% A1 %*% x1
LL[-wh[1:3],] <- x2
LL <- data.frame( t(LL) )
attr( LL, "Date" ) <- conv

# Convert to dates and difftimes
if( conv )
  {
  LL[,c(1,3,5)] <- ( LL[,c(1,3,5)] - 1970 ) * 365.25
  LL[,c(2,4,6)] <-   LL[,c(2,4,6)]          * 365.25
  class( LL[,1] ) <-
  class( LL[,3] ) <-
  class( LL[,5] ) <- "Date"
  class( LL[,2] ) <-
  class( LL[,4] ) <-
  class( LL[,6] ) <- "difftime"  
  }

LL
}

ci.eta <- function(form, cf, vcv, # model formula, coef and vcov from fitted model
                         newdata, # prediction frame (or list of frames)
                      name.check = TRUE, # report if model matrix names are as those of coef
                   alpha = 0.05, df = Inf, raw = FALSE)
{
# compute c.i. from formula and coefficients for a newdata

# allow rows of missing th the result
org.op <- options(na.action = "na.pass")
on.exit(options(org.op))

# only one-sided formula to model.matrix
if (length(form) == 3) form <- form[-2]

# matrix to multiply to the parameter vector and vcov
if (is.list(newdata))
   {
   if (!(length(newdata) %in% c(2,4)))
      stop("newdata as a list must have length 2 or 4)")
   mmx <- model.matrix(form, data = newdata[[1]])
   mmr <- model.matrix(form, data = newdata[[2]])
   mm <- mmx - mmr
   if (length(newdata) == 4)
      {
      mmx <- model.matrix(form, data = newdata[[3]])
      mmr <- model.matrix(form, data = newdata[[4]])
      mm <- mm - (mmx - mmr)
      }
   } else mm <- model.matrix(form, data = newdata)

# check sanity of formula and coeffiets
if (ncol(mm) != length(cf))
   stop("mismatch of formula and no. coefficients:\n",
        "ncol(model matrix) = ", ncol(mm),
        " but length(cf) = ", length(cf), "\n")
if (any(names(cf) != colnames(vcv)) |
    any(names(cf) != rownames(vcv)))
   stop("names of cf do not match row/col names of vcv\n")
if ((any(colnames(mm) != names(cf))) & name.check)
   {
   cat("NOTE: colnames(model matrix) and names(coef) do not match:\n")
   print(cbind(model = colnames(mm),
                  cf = names(cf)))
   }

# singular models produce NAs so remove from cf and vcov
out <- which(is.na(cf))
cf  <-  cf[-out]
vcv <- vcv[-out, -out]
 mm <-      mm[, -out]

# predicted eta and its vcov or ci
ct <- mm %*% cf
vc <- mm %*% vcv %*% t(mm)
se <- sqrt(diag(vc))

# confidence intervals
ci <- cbind(ct, se) %*% ci.mat(alpha = alpha, df = df)

# return results
if (raw) list(eta = ct,
              var = vc) else ci
}

plotevent <-
function(last.well,first.ill,data)
{
subsetdata <- data[!is.na(data[,first.ill]),c(last.well,first.ill)]
subsetdata$n <- seq(1,dim(subsetdata)[1])

plot(c(subsetdata[,1], subsetdata[,2]),rep(0,2*nrow(subsetdata)), bty="n", yaxt="n",
     type="n",
     xlim=c(round(min(subsetdata[,1],subsetdata[,2],na.rm=T)),round(max(subsetdata[,1],subsetdata[,2],na.rm=T))),
     ylim=c(-2, max(subsetdata[,3])),
     xlab="Time",ylab="Conversions", 
     main=paste("Times between ",last.well," and ",first.ill,"",sep=""))
mtext(seq(0,nrow(subsetdata),5)[-1],side=2,at=seq(0,nrow(subsetdata),5)[-1],las=1,line=0)
mtext("Eq Cl",side=2,at=-2,las=1, padj=0,col="blue",font=2)
segments(subsetdata[,1],subsetdata[,3],subsetdata[,2],subsetdata[,3],lwd=1)

left <- unique(subsetdata[,1])
right <- unique(subsetdata[,2])
names(left) <- rep("0",length(left))
names(right) <- rep("1",length(right))
MM <- sort(c(left,right))
type <- as.numeric(names(MM))
type2 <- c(type[-1],0)
diff <- type-type2
int <- MM[diff<0|diff>0]
mat <- matrix(int,length(int)/2,2,byrow=TRUE)
d <- mat[,1] - mat[,2]
segments(mat[,1][d!=0],-2,mat[,2][d!=0],-2,lwd=3,col="blue")
}

thinCol <-
function( A, tol = 1e-06, col.num=FALSE )
{
# Remove linearly dependent columns from a matrix
QR <- qr(A, tol = tol, LAPACK = FALSE)
# what columns are linearly dependen on the previous
whCol <- QR$pivot[seq(length = QR$rank)]
# return only numbers of dependent columns if required
if( col.num ) return( whCol )                 
A[, whCol, drop = FALSE]
}

in.span <-
inSpan <-
function( A, x, coef=FALSE, tol=1e-8 )
{
if( is.vector(x) ) dim(x) <- c(length(x),1)
if( nrow(A)!=nrow(x) ) stop("Matrices must have same row dimension")    
# Check if x is in span(A) using regression
xcf  <- NULL
insp <- TRUE
for( i in 1:ncol(x) )
   { 
   mod  <- lm( x[,i] ~ A - 1 )
   insp <- insp & ( all( abs(mod$residuals)<tol ) )
   xcf  <- cbind( xcf, coef(mod) )
   }
xcf[is.na(xcf)] <- 0
if( coef & insp ) {
       return( round( xcf, floor(abs(log10(tol))) ) )
} else return( insp )
}

id.span <-
idSpan <-
function( A, B, tol=1e-8 ) in.span(A,B) & in.span(B,A)
    
detrend <-
function( M, t, weight=rep(1,nrow(M)) )
{
# Detrend the matrix using a weighted inner product.
# Numerically unstable if too large t, so scaled
t <- scale(t)
thinCol( projection.ip( cbind( 1, t ), M , orth = TRUE, weight = weight ) )
}

decurve <-
function( M, t, weight=rep(1,nrow(M)) )
{
# De-trend and -curve the matrix using a weighted inner product.
t <- scale(t)
thinCol( projection.ip( cbind( 1, t, t^2 ), M , orth = TRUE, weight = weight ) )
}

Lexis <-
function(entry, exit, duration, entry.status=0, exit.status=0, id, data,
         merge=TRUE, states, notes=TRUE, tol=.Machine$double.eps^0.5,
         keep.dropped=FALSE )
{
  nmissing <- missing(entry) + missing(exit) + missing(duration)
  if (nmissing > 2)
        stop("At least one of the arguments exit and duration must be supplied")
  only.exit <- missing( entry.status ) && !missing( exit.status )

  ## If data argument is supplied, use it to evaluate arguments
  if (!missing(data)) {
    if (!missing(entry)) {
      entry <- eval(substitute(entry), data, parent.frame())
    }
    if (!missing(exit)) {
      exit <- eval(substitute(exit), data, parent.frame())
    }
    if (!missing(duration)) {
      duration <- eval(substitute(duration), data, parent.frame())
    }
    entry.status <- eval(substitute(entry.status), data, parent.frame())
    exit.status <- eval(substitute(exit.status), data, parent.frame())
    if (!missing(id)) {
      id <- eval(substitute(id), data, parent.frame())
    }
    if (merge) {
      data <- as.data.frame(data)
    }
  }

  ## Check for missing values in status variables
  wh.miss <- any(is.na(entry.status)) + 2*any(is.na(exit.status))
  if ( wh.miss > 0 ) stop("Missing values in ",
                          switch( wh.miss, "entry status",
                                           "exit status",
                                           "entry AND exit status" ) )

  ## Adjust entry status mode according to exit status
  if( only.exit )
    {
    if( is.logical( exit.status ) )
        {
        entry.status <- FALSE
        if( notes ) cat("NOTE: entry.status has been set to FALSE for all.\n")
        }
    if( is.character( exit.status ) )
        {
        exit.status <- factor( exit.status )
        }
    if( is.factor( exit.status ) )
        {
        entry.status <- factor( rep( levels(exit.status)[1],
                                     length(exit.status)),
                                levels=levels(exit.status),
                                labels=levels(exit.status) )
        if( notes ) cat("NOTE: entry.status has been set to",
            paste( '"', levels(exit.status)[1], '"', sep='' ),
            "for all.\n" )
        }
    if( is.numeric( exit.status ) )
        {
        entry.status <- rep( 0, length( exit.status ) )
        if( notes ) cat("NOTE: entry.status has been set to 0 for all.\n")
        }
    }

  ## Convert character states to factors
  if( is.character(entry.status) ) entry.status <- factor(entry.status)
  if( is.character( exit.status) )  exit.status <- factor( exit.status)

  ## Check compatibility of entry and exit status
  if (is.factor(entry.status) || is.factor(exit.status)) {
      if (is.factor(entry.status) && is.factor(exit.status)) {
          if (!identical(levels(entry.status),levels(exit.status))) {
              all.levels = union(levels(entry.status),levels(exit.status))
              entry.status <- factor( entry.status, levels=all.levels )
               exit.status <- factor(  exit.status, levels=all.levels )
          if( notes ) cat("Incompatible factor levels in entry.status and exit.status:\n",
                "both lex.Cst and lex.Xst now have levels:\n", all.levels, "\n")
          }
      }
      else {
          stop("Incompatible classes for entry and exit status")
      }
  }
  else {
      if (mode(entry.status) != mode(exit.status)) {
          stop("Incompatible mode for entry and exit status")
      }
  }

  ## If entry is missing and one of the others is given as a list of length
  ## one, entry is assumed to be 0 on this only timescale.
  if( nmissing==2 )
    {
    if( !missing(exit) )
      { if( length(exit)>1 )
          stop("If 'entry' is omitted, only one timescale can be specified.")
        else
        {
        entry <- exit
        entry[[1]] <- 0*entry[[1]]
        if( notes ) cat( "NOTE: entry is assumed to be 0 on the",names(exit),"timescale.\n")
        }
      }
    else
    if( !missing(duration) )
      { if( length(duration)>1 )
          stop("If 'entry' is omitted, only one timescale can be specified")
        else
        {
        entry <- duration
        entry[[1]] <- 0*entry[[1]]
        if( notes ) cat( "NOTE: entry is assumed to be 0 on the",names(duration),"timescale.\n")
        }
      }
    else
    stop("Either exit or duration must be supplied.")
    }

  ## Coerce entry and exit lists to data frames

  if(!missing(entry)) {
    entry <- as.data.frame(entry)
    if (is.null(names(entry)))
      stop("entry times have no names")
    if (any(substr(names(entry),1,4) == "lex."))
      stop("names starting with \"lex.\" cannot be used for time scales")
  }

  if(!missing(exit)) {
    exit <- as.data.frame(exit)
    if (is.null(names(exit)))
      stop("exit times have no names")
    if (any(substr(names(exit),1,4) == "lex."))
      stop("names starting with \"lex.\" cannot be used for time scales")
  }

  if(!missing(duration)) {
    duration <- as.data.frame(duration)
    if (is.null(names(duration)))
      stop("duration have no names")
    if (any(substr(names(duration),1,4) == "lex."))
      stop("names starting with \"lex.\" cannot be used for time scales")
  }

  if (missing(entry)) {
    ## Impute entry
    entry <- exit - duration
  }

  if (missing(duration)) {
    ## Impute duration
    full.time.scales <- intersect(names(entry), names(exit))
    if (length(full.time.scales) == 0) {
      stop("Cannot calculate duration from entry and exit times")
    }
    duration <- exit[,full.time.scales[1]] - entry[,full.time.scales[1]]
  }

  if (missing(exit)) {
    all.time.scales <- names(entry)
  }
  else {
    ## We dont need the exit times but, if they are supplied, we must
    ## make sure they are consistent with the entry and duration.

    all.time.scales <- unique(c(names(entry), names(exit)))
    ## Fill in any missing entry times
    entry.missing <- setdiff(all.time.scales, names(entry))
    if (length(entry.missing) > 0) {
      entry <- cbind(entry, exit[,entry.missing, drop=FALSE] - duration)
      }
    ## Check that duration is the same on all time scales
    dura <- exit - entry[,names(exit),drop=FALSE]
    if (missing(duration)) {
      duration <- dura[,1] #BxC# apply( dura, 1, mean, na.rm=TRUE )
                           # Allows for timescales with missing values
      }
    ok <- sapply(lapply(dura, all.equal, duration), isTRUE)
#   ok <- sapply(lapply(dura, all.equal, duration),
#                function(x) identical(FALSE,x) )
    if (!all(ok)) {
      stop("Duration is not the same on all time scales")
    }
  }
#  Taken care of by the code that detects whether lex.du <= tol
#  ## Check that duration is positive
#  if (any(duration<0)) {
#    stop("Duration must be non-negative")
#  }

  ## Make sure id values - if supplied - are valid. Otherwise supply default id

  if (missing(id)) {
    id <- 1:nrow(entry)
  }
  else if (any(duplicated(id))) {
    ##Fixme: check for overlapping intervals
    ##stop("Duplicate values in id")
  }

  ## Return a data frame with the entry times, duration, and status
  ## variables Use the prefix "lex." for the names of reserved
  ## variables.
  if( is.data.frame( duration ) ) duration <- duration[,1]
  lex <- data.frame( entry, "lex.dur" = duration,
                            "lex.Cst" = entry.status,
                            "lex.Xst" = exit.status,
                            "lex.id"  = id )

  #### Addition by BxC --- support for states as factors
  # Convert states to factors if states are given
  if( !missing( states ) ) #is.character( states ) )
    {
    # This as.character-business is necessary because we cannot assume
    # that the values of states are 1,2, etc.
    st.lev <- sort( unique( as.character( c(lex$lex.Cst,lex$lex.Xst) ) ) )
    lex$lex.Cst <- factor( as.character(lex$lex.Cst), levels=st.lev, labels=states )
    lex$lex.Xst <- factor( as.character(lex$lex.Xst), levels=st.lev, labels=states )
    }

  if (!missing(data) && merge) {
    duplicate.names <- intersect(names(lex), names(data))
    if (length(duplicate.names) > 0) {
      stop( "Cannot merge data with duplicate names:",
            paste(duplicate.names,collapse=" ") )
    }
    lex <- cbind(lex, data)
  }

  ## Drop rows with short or negative duration for consistency with splitLexis
  short.dur <- lex$lex.dur <= tol
  if ( any(short.dur) ) {
      if( notes ) cat("NOTE: Dropping ", sum(short.dur),
           " rows with duration of follow up < tol\n",
      if( keep.dropped ) "  The dropped rows are in the attribute 'dropped'\n",
      if( keep.dropped ) "  To see them type attr(Obj,'dropped'),\n",
      if( keep.dropped ) "  to get rid of them type: attr(Obj,'dropped') <- NULL\n",
      if( keep.dropped ) "  - where 'Obj' is the name of your Lexis object" )
      lex <- subset(lex, !short.dur)
      if( keep.dropped ) attr(lex,"dropped") <- subset(data, short.dur)
      }

  ## Return Lexis object
  attr(lex,"time.scales") <- all.time.scales
  attr(lex,"time.since") <- rep( "", length(all.time.scales) )
  breaks <- vector("list", length(all.time.scales))
  names(breaks) <- all.time.scales
  attr(lex,"breaks") <- breaks
  class(lex) <- c("Lexis", class(lex))
  return(lex)
}

is.Lexis <- function(x)
{
  inherits(x, "Lexis")
}

check.time.scale <- function(lex, time.scale=NULL)
{

  ## Utility function, returns the names of the time scales in a Lexis object
  ## lex - a Lexis object
  ## time.scale - a numeric or character vector. The function checks that
  ##              these are valid time scales for the Lexis object.
  ## Return value is a character vector containing the  names of the requested
  ## time scales

  all.names <- timeScales(lex)
  if (is.null(time.scale))
    return(all.names)

  nscale <- length(time.scale)
  scale.names <- character(nscale)
  if (is.character(time.scale)) {
    for (i in 1:nscale) {
      if (is.null(lex[[time.scale[i]]]))
        stop(time.scale[i], " is not a valid time scale name")
    }
  }
  else if (is.numeric(time.scale)) {
    if (any(time.scale > length(all.names))  || any(time.scale < 1))
      stop(time.scale, " not valid time scale column number(s)")
    time.scale <- all.names[time.scale]
  }
  else {
    stop("invalid type for time scale")
  }
  return(time.scale)
}

valid.times <-
function(x, time.scale=1)
  {
  # A utility function that returns a data.frame / Lexis object with
  # rows with missing timescales removed
  x[complete.cases(x[,check.time.scale(x,time.scale)]),]
  }

plot.Lexis.1D <- function(x, time.scale=1, breaks="lightgray",
                          type="l", col="darkgray", xlim, ylim, xlab, ylab,
                          ...)
{
  ## x Lexis object
  ## time.scale  name of time scale to plot

  if (length(time.scale) != 1)
    stop("Only one time scale allowed")

  x <- valid.times(x,time.scale)
  time.entry <- x[,time.scale]
  time.exit <- x[,time.scale] + x$lex.dur
  id <- x$lex.id

  if (missing(xlim))
    xlim <- c(min(time.entry), max(time.exit))
  if (missing(ylim))
    ylim <- range(id)
  if (missing(xlab))
    xlab <- time.scale
  if (missing(ylab))
    ylab <- "id number"

  plot(time.entry, id, type="n", xlab=xlab, ylab=ylab, xlim=xlim, ylim=ylim,
       ...)
  if (type=="b" || type=="l") {
    segments(time.entry, id, time.exit, id, col=col, ...)
  }
  if (type=="b" || type=="p") {
    points(time.exit, id, col=col, ...)
  }
  ## Plot break points
  brk <- attr(x,"breaks")[[time.scale]]
  abline(v=brk, col=breaks, ...)
}

points.Lexis.1D <- function(x, time.scale, ...)
{
  x <- valid.times(x,time.scale)
  time.exit <- x[,time.scale] + x$lex.dur
  points(time.exit, x$lex.id, ...)
}

lines.Lexis.1D <- function(x, time.scale, type="l", col="darkgray", breaks="lightgray",
                           ...)
{
  x <- valid.times(x,time.scale)
  time.entry <- x[,time.scale]
  time.exit <- x[,time.scale] + x$lex.dur
  id <- x$lex.id
  segments(time.entry, id, time.exit, id, col=col, ...)
  ## Plot break points
  brk <- attr(x,"breaks")[[time.scale]]
  abline(v=brk, col=breaks, ...)

}

plot.Lexis.2D <- function(x, time.scale, breaks="lightgray",
                          type="l", col="darkgray",
                          xlim, ylim, xlab, ylab,
                          grid=FALSE,
                      col.grid="lightgray",
                      lty.grid=2,
                      coh.grid=FALSE,
                          ...)
{
  if (length(time.scale) != 2)
    stop("Two time scales are required")

  x <- valid.times(x,time.scale)

  time.entry <- time.exit <- vector("list",2)
  for (i in 1:2) {
    time.entry[[i]] <- x[,time.scale[i]]
    time.exit[[i]] <- x[,time.scale[i]] + x$lex.dur
  }

  if (missing(xlim) && missing(ylim)) {
    ## If no axis limits are given, set the plotting region to be
    ## square, and adjust the axis limits to cover the same time interval.
    ## All life lines will then be at 45 degrees.
    opar <- par(pty="s")
    on.exit(par(opar))
    min.times <- sapply(time.entry, min)
    max.times <- sapply(time.exit, max)
    xywidth <- max(max.times - min.times)
    xlim <- min.times[1] + c(0, xywidth)
    ylim <- min.times[2] + c(0, xywidth)
  }
  else if (missing(xlim)) {
    xlim <- c(min(time.entry[[1]]), max(time.exit[[1]]))
  }
  else if (missing(ylim)) {
    ylim <- c(min(time.entry[[2]]), max(time.exit[[2]]))
  }

  if (missing(xlab))
    xlab <- time.scale[1]
  if (missing(ylab))
    ylab <- time.scale[2]

  plot(time.entry[[1]], time.entry[[2]], type="n",
       xlab=xlab, ylab=ylab, xlim=xlim, ylim=ylim, ...)

# Set up the background grid(s):
  if (!missing(grid)) {
     if (is.logical(grid)) {
        if (grid) {
        vgrid <- pretty(xlim)
        hgrid <- pretty(ylim)
        } }
     else if (is.list(grid)) {
        vgrid <- grid[[1]]
        hgrid <- grid[[length(grid)]]
        }
     else if (is.numeric(grid)) {
       vgrid <- grid - min( grid ) + min( pretty( xlim )[pretty(xlim)>=par("usr")[1]] )
       hgrid <- grid - min( grid ) + min( pretty( ylim )[pretty(ylim)>=par("usr")[3]] )
        }
     else stop( "'grid' must be either logical, list or a numeric vector" )
   # and plot the grid:
     abline( v=vgrid, h=hgrid, col=col.grid, lty=lty.grid )
     box()
     }
  if (!missing(grid) & coh.grid) {
     # Make the 45-degree grids as fine as the finest grid on the axes
     for (yy in c(hgrid-diff(range(hgrid)),hgrid))
         abline( yy-min(vgrid), 1, col=col.grid, lty=lty.grid )
     for (yy in c(vgrid-diff(range(vgrid)),vgrid))
         abline( min(hgrid)-yy, 1, col=col.grid, lty=lty.grid )
     }
# End of explicitly requested background grid(s) (PHEW!)

  if (type=="b" || type=="l") {
    segments(time.entry[[1]], time.entry[[2]], time.exit[[1]], time.exit[[2]],
             col=col, ...)
    }
  if (type=="b" || type=="p") {
    points(time.exit[[1]], time.exit[[2]], col = col, ...)
  }
  if (type != "n") {
    ## Plot break points
    brk <- attr(x,"breaks")[time.scale]
    abline(v=brk[[1]], h=brk[[2]], col=breaks, ...)
  }
}

points.Lexis.2D <- function(x, time.scale, ...)
{
  x <- valid.times(x,time.scale)
  time.exit <- vector("list",2)
  for (i in 1:2) {
    time.exit[[i]] <- x[,time.scale[i]] + x$lex.dur
  }
  points( time.exit[[1]], time.exit[[2]], ...)
}

lines.Lexis.2D <- function(x, time.scale, col="darkgray", ...)
{
  x <- valid.times(x,time.scale)
  time.entry <- time.exit <- vector("list",2)
  for (i in 1:2) {
    time.entry[[i]] <- x[,time.scale[i]]
    time.exit[[i]] <- x[,time.scale[i]] + x$lex.dur
  }
  segments(time.entry[[1]], time.entry[[2]], time.exit[[1]], time.exit[[2]],
           col=col, ...)
}

### Plotting generic functions
plot.Lexis <-
function( x = Lexis( entry=list(Date=1900,Age=0), exit=list(Age=0) ),
          time.scale=NULL, type="l", breaks="lightgray", ...)
{
  time.scale <- check.time.scale(x, time.scale)
  if (length(time.scale) > 2)
    time.scale <- time.scale[1:2]
  # Save the timescale(s) for use in subsequent calls
  options( Lexis.time.scale = time.scale )

  if (length(time.scale) == 1)
    plot.Lexis.1D(x, time.scale=time.scale, type=type, breaks=breaks, ...)
  else if (length(time.scale) == 2)
    plot.Lexis.2D(x, time.scale=time.scale, type=type, breaks=breaks, ...)
}


lines.Lexis <- function(x, time.scale=options()[["Lexis.time.scale"]], ...)
{
  time.scale <- check.time.scale(x, time.scale)
  if (length(time.scale) > 2)
    time.scale <- time.scale[1:2]

  if (length(time.scale) == 1)
    lines.Lexis.1D(x, time.scale=time.scale, ...)
  else if (length(time.scale) == 2)
    lines.Lexis.2D(x, time.scale=time.scale, ...)
}

points.Lexis <- function(x, time.scale=options()[["Lexis.time.scale"]], ...)
{
  time.scale <- check.time.scale(x, time.scale)
  if (length(time.scale) > 2)
    time.scale <- time.scale[1:2]
  if (length(time.scale) == 1)
    points.Lexis.1D(x, time.scale=time.scale, ...)
  else if (length(time.scale) == 2)
    points.Lexis.2D(x, time.scale=time.scale, ...)
}

print.Lexis <-
function(x, ..., td = 2, nd = td, rnam = FALSE, org = FALSE)
    {
    # "intersect" is to allow subsets of variables to be printed
    # result is ordered as the first argument to intersect()
    # The time-scale variables
    tsl <- intersect(c(timeScales(x), "lex.dur"), names(x))
    # The Lexis variables
    whL <- intersect(c("lex.id", tsl, "lex.Cst", "lex.Xst"), names(x))
    # non-Lexis variables"
    oth <- setdiff(names(x), whL)
    # non-Lexis numerical variables
    nuv <- setdiff(names(which(sapply(x, is.numeric))), whL)
    # round the time-scale variables
    x[,tsl] <- round(x[,tsl], td)
    # round the non-Lexis numerical variables
    x[,nuv] <- round(x[,nuv], nd)
    if (org) print.data.frame(x, row.names = rnam, ...)
        else print.data.frame(x[,c(whL, oth)],
                                 row.names = rnam, ...)
    # return the printed ordering of variables
    invisible(c(whL, oth))
    }

PY.ann <- function (x, ...) UseMethod("PY.ann")
PY.ann.Lexis <-
function( x, time.scale=options()[["Lexis.time.scale"]], digits=1, ... )
{
  if (!inherits(x,"Lexis"))
    stop("Only meaningful for Lexis objects not for objects of class ",
         class(x))
  wh.x <- x[,time.scale[1]] + x[,"lex.dur"]/2
  if( two.scales <- length(time.scale)==2 )
     wh.y <- x[,time.scale[2]] + x[,"lex.dur"]/2
  else
     wh.y <- x[,"lex.id"]
  text( wh.x, wh.y,
        formatC(x$lex.dur,format="f",digits=digits),
        adj=c(0.5,-0.5), srt=if(two.scales) 45 else 0, ... )
}

### Generic functions

### Methods for data.frame drop Lexis attributes, so we need Lexis
### methods that retain them

subset.Lexis <- function(x, ... )
{
  y <-  subset.data.frame(x, ...)
  attr(y,"breaks") <- attr(x, "breaks")
  attr(y,"time.scales") <- attr(x, "time.scales")
  attr(y,"time.since") <- attr(x, "time.since")
  return(y)
}

`[.Lexis` <-
function(x, ...)
{
  y <- NextMethod(x)
  if (is.data.frame(y)) {
    for (a in c("class", "time.scales", "time.since", "breaks")) {
        data.table::setattr(y ,a, attr(x, a))}
  }
  y
}

merge.Lexis <- function(x, y, id, by, ...)
{
  if (!missing(id)) {
    if (!is.character(id) || length(id) != 1 || !(id %in% names(y))) {
      stop("id must be the name of a single variable in y")
    }
    if (any(duplicated(y[[id]]))) {
      stop("values of the id variable must be unique in y")
    }
    y$lex.id <- y[[id]]
  }
  else if (missing(by)) {
    by <- intersect(names(x), names(y))
    if (length(by)==0) {
      stop("x and y have no variable names in common")
    }
  }

  z <-  base::merge.data.frame(x, y, ...)
  attr(z,"breaks") <- attr(x, "breaks")
  attr(z,"time.scales") <- attr(x, "time.scales")
  attr(z,"time.since") <- attr(x, "time.since")
  class(z) <- c("Lexis", "data.frame")
  return(z)
}

cbind.Lexis <-
function( ... )
{
allargs <- list( ... )
# Check that at least one argument is Lexis
is.lex <- sapply( allargs, inherits, "Lexis" )
if( all(!is.lex) ) stop( "At least one argument nust be a Lexis object\n",
                         "and none of the given are.\n")
if( sum(is.lex)>1 ) stop( "It is meaningless to 'cbind' several Lexis objects:",
                          " arguments ", paste( which(is.lex), collapse=","),
                          " are Lexis objects.\n" )
is.lex <- which(is.lex)
res <- do.call( base::cbind.data.frame, allargs )
attr( res, "class"       ) <- attr( allargs[[is.lex]], "class"       )
attr( res, "breaks"      ) <- attr( allargs[[is.lex]], "breaks"      )
attr( res, "time.scales" ) <- attr( allargs[[is.lex]], "time.scales" )
attr( res, "time.since"  ) <- attr( allargs[[is.lex]], "time.since"  )
res
}

rbind.Lexis <-
function( ... )
{
# A list of all Lexis objects
allargs <- list( ... )
# Check if they are all Lexis
# (or possibly NULL - often rbind-ing with NULL is very useful)
is.lex <- sapply( allargs, inherits, "Lexis" )
is.nul <- sapply( allargs, is.null )
if( !all(is.lex[!is.nul]) )
    stop( "All arguments must be Lexis objects,\n",
          "arguments number ", which(!is.lex & !is.null), " are not." )
# Put them all together
allargs <- allargs[!is.nul]
res <- plyr::rbind.fill( allargs )
# Get the union of time.scale names and the corresponding time.since
tscl <- do.call( c, lapply( allargs, function(x) attr(x,"time.scales") ) )
tsin <- do.call( c, lapply( allargs, function(x) attr(x,"time.since" ) ) )
# but only one copy of each
scls <- match( unique(tscl), tscl )
tscl <- tscl[scls]
tsin <- tsin[scls]
# Fish out the breaks on timescale in turn from all input objects and
# - if all the non-NULL are identical use this
# - if not, set the corresponding break to NULL
newbrks <- list()
# all the breaks attributes in a list
brks <- lapply( allargs, function(x) attr(x,"breaks") )
# run through the timescales found
for( scl in tscl )
   {
   # breaks for this timescale in any of the objects
   brk <- lapply( brks, function(x) x[[scl]] )
   # but only the non-null ones
   brk <- brk[!sapply( brk, is.null )]
   # if more than one occurrence, all non-NULL breaks should be identical
   if( ( length(brk)>1 &
         all( sapply( brk[-1], function(x) identical(brk[[1]],x) ) ) )
       | length(brk) == 1 ) newbrks[scl] <- brk[1]
   else newbrks[scl] <- list(NULL)
   }
# define attributes of the reulting object:
attr( res, "class"       ) <- c( "Lexis", "data.frame" )
attr( res, "breaks"      ) <- newbrks
attr( res, "time.scales" ) <- tscl
attr( res, "time.since"  ) <- tsin
res
}

## Extractor functions

entry <- function(x, time.scale = NULL, by.id = FALSE )
{
    time.scale <- check.time.scale(x, time.scale)
    wh <- x[,time.scale[1]] ==
     ave( x[,time.scale[1]], x$lex.id, FUN=if( by.id ) min else I )
    if (length(time.scale) > 1) {
        res <- as.matrix(x[wh, time.scale])
        if( by.id ) rownames( res ) <- x$lex.id[wh]
        return( res )
    }
    else {
        res <- x[wh, time.scale]
        if( by.id ) names( res ) <- x$lex.id[wh]
        return( res )
    }
}

exit <- function(x, time.scale = NULL, by.id = FALSE )
{
    time.scale <- check.time.scale(x, time.scale)
    wh <- x[,time.scale[1]] ==
     ave( x[,time.scale[1]], x$lex.id, FUN=if( by.id ) max else I )
    if (length(time.scale) > 1) {
        res <- as.matrix(x[wh, time.scale]) + x$lex.dur[wh]
        if( by.id ) rownames( res ) <- x$lex.id[wh]
        return( res )
    }
    else {
        res <- x[wh, time.scale] + x$lex.dur[wh]
        if( by.id ) names( res ) <- x$lex.id[wh]
        return( res )
    }
}

dur <- function(x, by.id=FALSE)
{
  if( by.id ) return( tapply(x$lex.dur,x$lex.id,sum) )
  else        return(        x$lex.dur               )
}

status <- function(x, at="exit", by.id = FALSE)
{
  at <- match.arg(at, c("entry","exit"))
  wh <- x[,timeScales(x)[1]] ==
   ave( x[,timeScales(x)[1]], x$lex.id, FUN=if(by.id)
                                             switch(at,
                                                    "entry"=min,
                                                     "exit"=max)
                                             else I )
  res <- switch(at, "entry"=x$lex.Cst, "exit"=x$lex.Xst)[wh]
  if( by.id ) names( res ) <- x$lex.id[wh]
  res
}

transient <-
function( x )
{
if( !is.Lexis(x) ) stop( "Not a Lexis object" )
tc <- tapply( x$lex.dur, x$lex.Cst, sum )
nt <- names( tc[tc>0] )
nt[!is.na(nt)]
}

absorbing <-
function( x )
{
if( !is.Lexis(x) ) stop( "Not a Lexis object" )
tc <- table( x$lex.Xst )
setdiff( names( tc[tc>0] ), transient(x) )
}

updn <-
function(x, tt, states)
{
if (any(is.na(match(states,levels(x)))))
  stop("'states' must be one of states: ", paste(levels(x), sep=','))
tt <- tt[, states, drop=FALSE]
cc <- intersect(rownames(tt), colnames(tt))
tt[cbind(cc,cc)] <- 0
rownames(tt[apply(tt, 1, sum) > 0, , drop = FALSE])
}
before <- preceding <- function(x, states) updn(x,
                                                table(x$lex.Cst,
                                                      x$lex.Xst),
                                                states)
after <- succeeding <- function(x, states) updn(x,
                                                table(x$lex.Xst,
                                                      x$lex.Cst),
                                                states)

timeScales <- function(x)
{
  return(attr(x, "time.scales"))
}

timeSince <- function(x)
{
        tt  <- attr(x, "time.since")
  names(tt) <- attr(x, "time.scales")
  return( tt )
}

lexVars <- function(x)
{
  c("lex.id", Epi::timeScales(x), "lex.dur", "lex.Cst", "lex.Xst")
}

timeBand <- function(lex, time.scale, type="integer")
{
  time.scale <- check.time.scale(lex, time.scale)[1]
  breaks <- attr(lex, "breaks")[[time.scale]]

  time1 <- lex[[time.scale]]
  band <- findInterval(time1, breaks)

  ##Check that right hand side of interval falls in the same band
  abrk <- c(breaks, Inf)
  tol <- sqrt(.Machine$double.eps)
  if (any(time1 + lex$lex.dur > abrk[band+1] + tol)) {
    stop("Intervals spanning multiple time bands in Lexis object")
  }

  type <- match.arg(type, choices = c("integer","factor","left","middle",
                    "right"))
  if (type=="integer") {
     return(band)
  }

  I1 <- c(-Inf, breaks)
  I2 <- c(breaks, Inf)
  labels <- switch(type,
                   "factor" = paste("(", I1, ",", I2, "]", sep=""),
                   "left" = I1,
                   "right" = I2,
                   "middle" = (I1 + I2)/2)

  if(type=="factor") {
    return(factor(band, levels=0:length(breaks), labels=labels))
  }
  else {
    return(labels[band+1])
  }
}

breaks <- function(lex, time.scale)
{
  time.scale <- check.time.scale(lex, time.scale)[1]
  return(attr(lex, "breaks")[[time.scale]])
}

transform.Lexis <- function(`_data`, ... )
{
    save.at <- attributes(`_data`)
    ## We can't use NextMethod here because of the special scoping rules
    ## used by transform.data.frame
    y <- base::transform.data.frame(`_data`, ...)
    save.at[["names"]] <- attr(y, "names")
    attributes(y) <- save.at
    y
}

# Two utility functions used to sort a Lexis object by (lex.id,time)
order.Lexis <-
 orderLexis <-
function( x )
   {
   # Some time scales may not be proper from addCov.Lexis
   # So find one which is ("X" in timeSince(x) is improper)
   if (!inherits(x, "Lexis")) stop("Argument must be a Lexis object\n")
   # Fixing things if a data.table
   attr(x, "class") <- c("Lexis", "data.frame")
   a.ts <- timeScales(x)[which(timeSince(x) != "X")[1]]
   order(x$lex.id, x[,a.ts], na.last = FALSE)
   }

sortLexis <- function( x ) x[orderLexis(x),]

mat2pol <-
function( pm,
        perm = 1:ncol(pm),
           x = as.numeric(rownames(pm)),
         col = rainbow(ncol(pm)),
          yl = 0:1,
      append = FALSE,
         ... )
{
if( missing(x) ) x <- as.numeric( rownames(pm) )
if( length(x)==0 ) x <- 1:nrow(pm)
xm <- cbind( 0, pm[,perm] )
xm <- t( apply(xm,1,cumsum) )
if( !append ) plot( x, x, type="n", ylim=yl, ... )
for( j in 1:ncol(pm) )
   polygon( c(x,rev(x)),
            c(xm[,j],rev(xm[,j+1])),
            col = col[j], border="transparent" )
invisible( xm )
}

plot.apc <-
apc.plot <-
function( x, r.txt="Rate", ... )
{
if( !inherits( x, "apc" ) ) stop( "Argument must be an apc-object" )

# Determine the ranges of the horizontal axes
 a.lab = nice( x$Age[,1] )
cp.lab = nice( c(x$Per[,1],x$Coh[,1]), high=0.1 )[-1]
# The necessary range of the two vertical axes
 r.rg  <- range( x$Age[,-1] )
rr.rg  <- range( rbind( x$Per[,-1], x$Coh[,-1] ) )
# Align the RR with the rates on an integer power of 10
rr.ref <- 10^floor( log10(r.rg[2])-log10(rr.rg[2]) )
# Find the tic-marks for the two vertical axes
 r.tic <- nice(  r.rg, log=T, lpos=1:9 )
rr.tic <- nice( rr.rg, log=T, lpos=1:9 )
# Expand to cover it all
 r.tic <- sort( unique( c( r.tic, rr.tic*rr.ref ) ) )
rr.tic <- r.tic/rr.ref
# Find the places for labels
 r.lab <- nice(  r.tic, log=T, lpos=c(1,2,5) )
rr.lab <- nice( rr.tic, log=T, lpos=c(1,2,5) )
 r.lab <-  r.lab[ r.lab>min( r.tic) &  r.lab<max( r.tic)]
rr.lab <- rr.lab[rr.lab>min(rr.tic) & rr.lab<max(rr.tic)]
# Now draw the frame
fpar <-
apc.frame( a.lab=a.lab,
          cp.lab=cp.lab,
           r.lab=r.lab,
           r.tic=r.tic,
          rr.lab=rr.lab,
          rr.tic=rr.tic,
          rr.ref=rr.ref,
           r.txt=r.txt )
# - and the reference line
segments( min(cp.lab)-fpar[1], fpar[2], max(cp.lab)-fpar[1], fpar[2] )
apc.lines( x, frame.par=fpar, ... )
fpar
}

lines.apc <-
apc.lines <-
function( x,
          P,
          C,
      scale = c("log","ln","rates","inc","RR"),
  frame.par = options()[["apc.frame.par"]],
      drift = 0,
         c0 = median( C[,1] ),
         a0 = median( A[,1] ),
         p0 = c0 + a0,
         ci = rep( FALSE, 3 ),
        lwd = c(3,1,1),
        lty = 1,
        col = "black",
       type = "l",
      knots = FALSE,
      shade = FALSE,
        ...
        )
{
# To make the lines method work, the first argument must be called x
A <- x
# What scale are we using:
log.scale <- ( match.arg( scale ) %in% c("log","ln") )
# Are confidence intervals requested ?
# Expand ci to three components if only one is given
if( is.logical( ci ) ) ci <- rep( ci, 3 )[1:3]
# Allow ci to be any character string or vector with "a", "p" or "c" in it
if( is.character( ci ) )
  {
  ci <- paste( ci, collapse="" )
  ci <- unlist( strsplit( toupper( ci ), split="" ) )
  ci <- !is.na( match( c("A","P","C"), ci ) )
  }
# Check the format of the various inputs are correct
if( inherits( A, "apc" ) )
  {
 obj <- A
  p0 <- ifelse( is.na( A$Ref[1] ), median( A$Per[,1] ), A$Ref[1] )
  c0 <- ifelse( is.na( A$Ref[2] ), median( A$Coh[,1] ), A$Ref[2] )
  a0 <- p0 - c0
   C <- A$Coh
   P <- A$Per
   A <- A$Age
  log.scale <- FALSE
  }
if( ( ci[1] & dim( A )[2] < 4 ) | dim( A )[2] < 2 )
  stop( "A, ", deparse( substitute( A ) ), ") must be a ", 2+2*ci[1]," column matrix" )
if( ( ci[2] & dim( P )[2] < 4 ) | dim( P )[2] < 2 )
  stop( "P, ", deparse( substitute( P ) ), ") must be a ", 2+2*ci[2]," column matrix" )
if( ( ci[3] & dim( C )[2] < 4 ) | dim( C )[2] < 2 )
  stop( "C, ", deparse( substitute( C ) ), ") must be a ", 2+2*ci[3]," column matrix" )
if( p0 != c0+a0 )
  stop( "p0=", p0, " differs from c0 + a0 = ", c0, " + ", a0, " = ", c0+a0 )
# Expand single graphics parameters to length 3
  col  <- rep(  col, 3 )
  lty  <- rep(  lty, 3 )
  type <- rep( type, 3 )
# Transform to log-scale if input is as rates and RRs
  if( !log.scale )
    {
    if( missing( drift ) ) drift <- 1
      drift <- log( drift )
    A[,2:4] <- log( A[,2:4] )
    P[,2:4] <- log( P[,2:4] )
    C[,2:4] <- log( C[,2:4] )
    }
  A[,2:4] <- exp( A[,2:4] - drift * ( A[,1] - a0 ) )
  P[,2:4] <- exp( P[,2:4] + drift * ( P[,1] - p0 ) )
  C[,2:4] <- exp( C[,2:4] - drift * ( C[,1] - c0 ) )
  # If no frame parameters are given
  if( is.null( frame.par ) )
  frame.par <- c( min( C[,1] ) + max( A[,1] ),
                  ifelse( scale=="log",
                          exp( mean(      A[,2] ) ),
                          exp( mean( log( A[,2] ) ) ) ) )
  # Now we can plot the lines
if( !shade )
  {  
  matlines( A[,1]               , A[,ifelse( ci[1], -1, 2)],
            col=col[1], lwd=lwd, lty=lty[1], type=type[1], ... )
  matlines( P[,1] - frame.par[1], P[,ifelse( ci[2], -1, 2)] * frame.par[2],
            col=col[2], lwd=lwd, lty=lty[2], type=type[2], ... )
  matlines( C[,1] - frame.par[1], C[,ifelse( ci[3], -1, 2)] * frame.par[2],
            col=col[3], lwd=lwd, lty=lty[3], type=type[3], ... )
  } else {  
  matshade( A[,1]               , A[,-1],
            col=col[1], lwd=lwd, lty=lty[1], ... )
  matshade( P[,1] - frame.par[1], P[,-1] * frame.par[2],
            col=col[2], lwd=lwd, lty=lty[2], ... )
  matshade( C[,1] - frame.par[1], C[,-1] * frame.par[2],
            col=col[3], lwd=lwd, lty=lty[3], ... )
  }
    
  points( obj$Ref - frame.par[1], frame.par[c(2,2)], pch=16, col="white" )
  points( obj$Ref - frame.par[1], frame.par[c(2,2)], pch=1, lwd=2, col=col[2:3] )
  if( knots & inherits( obj, "apc" ) )
  {
  rug( obj$Knots$Age, side=1, col=col[1] )
  rug( obj$Knots$Per - frame.par[1], side=1, col=col[2] )
  rug( obj$Knots$Coh - frame.par[1], side=3, col=col[3] )
  }
  box()
  invisible( list( A=A, P=P, C=C ) )
}

a.points <- points
a.lines  <- lines
a.matpoints <- matpoints
a.matlines  <- matlines
# Useful functions to plot in the calendar time part of an apc frame.
cp.points <-
pc.points <-
function( x, y, ... )
{
fp <- options()[["apc.frame.par"]]
points( x - fp[1], y * fp[2], ... )
}

cp.lines <-
pc.lines <-
function( x, y, ... )
{
fp <- options()[["apc.frame.par"]]
lines( x - fp[1], y * fp[2], ... )
}

cp.matpoints <-
pc.matpoints <-
function( x, y, ... )
{
fp <- options()[["apc.frame.par"]]
matpoints( x - fp[1], y * fp[2], ... )
}

cp.matlines <-
pc.matlines <-
function( x, y, ... )
{
fp <- options()[["apc.frame.par"]]
matlines( x - fp[1], y * fp[2], ... )
}

cp.matshade <-
pc.matshade <-
function( x, y, ... )
{
fp <- options()[["apc.frame.par"]]
matshade( x - fp[1], y * fp[2], ... )
}

list.lex <-
function( Lx, dig, sh="_" )
    {
            wh <- c("lex.id",timeScales(Lx),"lex.dur","lex.Cst","lex.Xst")
           oth <- setdiff(names(Lx),wh)
         tmpLx <- Lx[,c(wh,oth)]
       wh.num  <- sapply(tmpLx,is.numeric)
tmpLx[,wh.num] <- round(tmpLx[,wh.num],dig)
  names(tmpLx) <- gsub("lex.",sh,names(tmpLx))
print( tmpLx, row.names=FALSE )
return( NULL )
    }

fit.baseline <-
function(y, rates.frame, start)
{
  if (missing(start)) {
    ## Get starting values from logistic regression model
    glm.out.init <- glm(y~., family=binomial, data=rates.frame)
    mu.init <- fitted(glm.out.init, type="response")
    glm(y~-1 + ., family=binomial(link=log), data=rates.frame, mustart=mu.init)
  }
  else {
    glm(y~-1 + ., family=binomial(link=log), data=rates.frame, start=start)
  }
}

rcutLexis <-
function(Lx, cut,
       timescale = 1,
precursor.states = transient(Lx))
{
# avoid note about no visible binding
new.state <- lex.id <- NULL

# All new states should be precursor states
# new.state may be factor, hence the as.character
pr.st <- unique(c(precursor.states,
                  unique(as.character(cut$new.state))))

# utility to select n'th element in a vector
getn <- function(x,n) x[n]
    
# make the data frame a sorted, grouped tibble
# note that the cut in arrange is the variable cut in the tibble
cut <- group_by(cut, lex.id) %>%
       arrange(cut, .by_group = TRUE)
    
# nxL is the Lexis object to return eventually 
nxL <- Lx
    
# max no transitions for any one person in cut
maxn <- max(table(cut$lex.id))
    
# loop over transition number per person
for(n in 1:maxn)
   {
   # the n'th transitions for each person
   nx.cut <- summarize(cut,
                       cut = getn(      cut, n),
                 new.state = getn(new.state, n))
   # update with cuts at these for each person
   nxL <- cutLexis(nxL,  
                   cut = nx.cut,
             timescale = timescale,
      precursor.states = pr.st )
   }
# done, return result
nxL
}

fit.mult <-
function(y, rates.frame, cov.frame, start)
{
  if (missing(start)) {
    ## Fit model without covariates to get initial rates estimates
    glm.out.rates <- fit.baseline(y, rates.frame)
    ## Initial values for iterative fitting
    lambda <- coef(glm.out.rates)
    beta <- rep(0, ncol(cov.frame))
  }
  else {
    lambda <- start[1:ncol(rates.frame)]
    beta <- start[ncol(rates.frame) + 1:ncol(cov.frame)]
  }

  niter <- 1
  cy <- 1 - y
  while(TRUE) {
    ## covariates model
    off <- log(-as.matrix(rates.frame) %*% lambda)
    glm.out.cov <- glm(cy ~ -1 + offset(off) + .,
                       family=binomial(link=cloglog),
                       data=cov.frame, start=beta, maxit=100)
    beta <- coef(glm.out.cov)
    
    ## rates model
    wgt <- exp(as.matrix(cov.frame) %*% beta)
    temp.rates.frame <- wgt * rates.frame
    glm.out.rates <- glm(y ~ -1 + ., family=binomial(link=log),
                         data=temp.rates.frame, start=lambda, maxit=100)
    lambda <- coef(glm.out.rates)

    ## Check convergence 
    ## Convergence <==> deviances are equal
    TOL <- max(glm.out.cov$control$epsilon, glm.out.rates$control$epsilon)
    dev1 <- glm.out.cov$deviance
    dev2 <- glm.out.rates$deviance
    if (abs(dev1 - dev2)/(0.1 + abs(dev1)) < TOL) 
      break
    else
      niter <- niter + 1
  }

  return(list( rates=glm.out.rates, cov=glm.out.cov, niter=niter))
}

# The overlap method
overlap <- function (Lx, ...) UseMethod("overlap")

overlap.default <-
overlap.Lexis <-
function(Lx, ...)
{
# must be a Lexis object
if (!inherits(Lx, "Lexis")) stop("Lx must be a Lexis object")
# the Lexis object should be sorted
Lx <- sortLexis(Lx)
# a timescale without missing values
whs <- names(timeSince(Lx))[timeSince(Lx) == ""][1]
tsc <- Lx[, timeScales(Lx)[1]]
# the next value on the timescale
tnx <- c(tsc[-1], NA)
# is this record overlapping the next and is it the same person?
olp <- ((tsc + Lx$lex.dur) > tnx) &
       duplicated(Lx$lex.id, fromLast = TRUE)
# the ids of persons with any overlapping records
if (any(olp))
   {
   who <- unique(Lx$lex.id[olp])
   cat("Overlapping records found in",
       length(who), "persons - returned invisibly.\n")
   return(invisible(who))
   }
else
   {
   cat("No overlapping records within persons.\n")
   return(NULL)
   }
}

Lexis.lines <-
function( entry.date = NA,
           exit.date = NA,
          birth.date = NA,
           entry.age = NA,
            exit.age = NA,
           risk.time = NA,
            col.life = "black",
            lwd.life = 2,
                fail = NA,
            cex.fail = 1,
            pch.fail = c(NA,16),
            col.fail = col.life,
                data = NULL
          )
{
    ## Get variables from data argument, if supplied, or from parent
    ## frame if not.
    entry.date <- eval(substitute(entry.date), data)
    entry.age  <- eval(substitute(entry.age ), data)
    exit.date  <- eval(substitute(exit.date ), data)
    exit.age   <- eval(substitute(exit.age  ), data)
    risk.time  <- eval(substitute(birth.date), data)
    birth.date <- eval(substitute(birth.date), data)
    fail       <- eval(substitute(fail      ), data)

# If fail is numeric make it logical
if( is.numeric( fail ) ) fail <- ( fail > 0 )

# Complete the information on lifelines
XX <- Life.lines( entry.date = entry.date,
                   entry.age = entry.age,
                   exit.date = exit.date,
                    exit.age = exit.age,
                   risk.time = risk.time,
                  birth.date = birth.date )

# Expand lwd.life/col.life/pch.fail/col.fail/cex.fail
#
Np <- nrow( XX )

if( length( col.life )==1 ) col.life <- rep( col.life, Np  ) else
if( length( col.life )!=length(fail) ) stop("col.life must have length 1 or length(fail)" )

if( length( lwd.life )==1 ) lwd.life <- rep( lwd.life, Np ) else
if( length( lwd.life )!=length(fail) ) stop("lwd.life must have length 1 or length(fail)" )

if( length( col.fail )==1 ) col.fail <- rep( col.fail, Np ) else {
if( length( col.fail )==2 ) col.fail <- col.fail[fail+1] }
if( length( col.fail )!=length(fail) ) stop("col.fail must have length 1,2 or length(fail)" )

if( length( pch.fail )==1 ) pch.fail <- rep( pch.fail, Np ) else
if( length( pch.fail )==2 ) pch.fail <- pch.fail[fail+1]
if( length( pch.fail )!=length(fail) ) stop("pch.fail must have length 1,2 or length(fail)" )

if( length( cex.fail )==1 ) cex.fail <- rep( cex.fail, Np ) else
if( length( cex.fail )==2 ) cex.fail <- cex.fail[fail+1]
if( length( cex.fail )!=length(fail) ) stop("cex.fail must have length 1,2 or length(fail)" )

# Was XX returned as a Date-object?
# If so make a numerical version i LL, otherwise just a copy.
#
if( attr( XX, "Date" ) )
  {
  LL <- data.frame( lapply( XX, unclass ) )
  LL[,c(1,3,5)] <- LL[,c(1,3,5)] / 365.25 + 1970
  LL[,c(2,4,6)] <- LL[,c(2,4,6)] / 365.25
  } else LL <- XX

# Find age and date ranges in the current plot.
#
date <- par( "usr" )[1:2]
age  <- par( "usr" )[3:4]

# Plot the lifelines
  segments( LL[,1], LL[,2], LL[,3], LL[,4],
            lwd=lwd.life, col=col.life )
# If there are any non-NAs for pch.fail then blank out the space
# where they go before plotting the symbols
  if( any( !is.na(pch.fail) ) )
  points( LL[!is.na(pch.fail),3], LL[!is.na(pch.fail),4],
          pch=16,
          col="white", #par()$bg,
          cex=cex.fail[!is.na(pch.fail)] )
  points( LL[,3], LL[,4],
          pch=pch.fail,
          col=col.fail,
          cex=cex.fail )

# Return the untouched version of the completed dataframe
#
invisible( data.frame( XX, fail=fail ) )
}

# The factorize method
factorize <- function (x, ...) UseMethod("factorize")

# Default method is just the Lexis method
factorize.default <- 
factorize.Lexis <-
function(x, ..., verbose = FALSE)
{
Cst <- Xst <- NULL
# If lex.Cst and lex.Xst are not factors, make sure they are, and
# if they are restrict levels to to actually ocurring
Cst <- factor(x$lex.Cst)
Xst <- factor(x$lex.Xst)
all.levels <- union(levels(Cst),
                    levels(Xst))
# then amend them to have the same set of levels
x$lex.Cst <- factor(Cst, levels = all.levels)
x$lex.Xst <- factor(Xst, levels = all.levels)

# Note if levels changed
if((length(setdiff(all.levels, levels(Cst))) > 0 |
    length(setdiff(all.levels, levels(Xst))) > 0)
   & verbose)
   cat( "NOTE: lex.Cst and lex.Xst now have levels:\n", all.levels, "\n")  
return(x)
}

# The Lexis version of Relevel
Relevel.Lexis <-
function(x, ref, ...)
{
# Is this really a Lexis object
if(!inherits(x, "Lexis")) stop("First argument must be a Lexis object")
    
# Make sure the states are factors with the same levels
if(!is.factor(x$lex.Cst) | !is.factor(x$lex.Xst))
   stop("lex.Cst and lex.Xst must be factors")
if(any(levels(x$lex.Xst) != levels(x$lex.Cst)))
   stop("lex.Cst and lex.Xst must have same levels\n",
        "you may want to factorize.Lexis()\n")    

# Then use the same Relevel on the factors
x$lex.Cst <- Relevel.factor(x$lex.Cst, ref, ...)
x$lex.Xst <- Relevel.factor(x$lex.Xst, ref, ...)
return(x)
}

# The levels method is already defined (in the base package)
# and hence imported in the NAMESPACE file
levels.Lexis <-
function( x )
{
union(base::levels(x$lex.Cst),
      base::levels(x$lex.Xst))
}

## # The Lexis version of levels
## "levels.Lexis<-" <-
## function(x, value)
## {
## base::levels(x$lex.Cst) <- value
## base::levels(x$lex.Xst) <- value
## x
## }

# The addDrug method
addDrug <- function (Lx, ...) UseMethod("addDrug")

#----------------------------------------------------------------------
# First a utility function to dream up 0 purchases starting each
# non-exposed period
ins0 <-
function(pdat,  # data set of purchases with names tnam, amt, lex.id
          amt = "amt", # name of amount variable in pdat
          dpt = "dpt", # name of dose per time variable in pdat
                       # to be used by metod "dos"
       method = "ext", # extrapolation from times and amounts
                       # or "dos" based on dosage, dpt and amt
                       # or "fix" using a fixed interval maxt
         maxt = NULL,  # maximal time covered by a single prescription,
        grace = 0,     # grace period after last purchase used up
         tnam = setdiff(names(pdat), c("lex.id", amt))[1],
      verbose = TRUE)
{
# pdat must include lex.id as variable
# inserts a 0 purchase record at end of exposure from each purchase
# 3 methods implemented:
# "ext" each prescription lasts as if consumed as the previous.
# "dos" each prescription lasts amt / dpt + grace.
# "fix" exch prescription lasts the same time, maxt

# bind variables
lex.id <- dop <- extime <- tfxxx <- NULL

if(length(intersect(c("lex.id", amt), names(pdat))) < 2)
  stop("lex.id and", amt, "must be columns in the data frame\n")

if(missing(tnam) & verbose)
  cat("NOTE: timescale '", tnam, "' assumed\n", sep = "")

if(method == "ext")
  if(verbose)
  cat("NOTE: end of exposure based on differences in purchase times (",
      tnam,")\n and amount purchased (", amt, ").\n", sep = "")

if(method == "dos")
  {
  if(is.numeric(dpt))
    {
    pdat$dpt <-  dpt
         dpt <- "dpt"
    }
  if(!(dpt %in% names(pdat)))
    stop('method = "dos" reqires amount per time, "', dpt, '" in data.\n')
  if(verbose)
  cat("NOTE: end of exposure based on purchase and dosage (", dpt, ").\n",
      sep = "")
  }

if(method == "fix")
  {
  if(!is.numeric(maxt))
    stop('method="fix" reqires a fixed coverage time in argument maxt.\n')
  if(verbose)
  cat("NOTE: end of exposure based on fixed coverage time of", maxt, ".\n")
  }

# order by purchase data within each person
pdat <- pdat[order(pdat$lex.id,pdat[, tnam]),]

# put the variable names in pdat for simplicity of code
                         pdat$dop <- pdat[, tnam]
                         pdat$amt <- pdat[, amt]
if(dpt %in% names(pdat)) pdat$dpt <- pdat[, dpt]

# compute the time of expiry of each purchase by method chosen
# note methods 'fix' and 'dos' only use the current purchase
# while 'ext' refers to other purchases for the person

if(method == 'fix')
  pdat$extime <- pdat$dop + maxt

if(method == 'dos')
  pdat$extime <- pdat$dop + pdat$amt / pdat$dpt + grace

if(method == 'ext')
  pdat <-
  ( group_by(pdat, lex.id)
%>% mutate(extime = dop
                  + c(NA,
                      diff(dop) * amt[-1] / amt[-length(amt)])
                  + grace)
%>% ungroup()
  )

# generate 0 purchases in dataframe pzero if coverage expire before
# new purchase
pzero <-
  ( group_by(pdat, lex.id)
%>% filter(extime < c(dop[-1], Inf))
%>% mutate(dop = extime,
           amt = 0)
%>% select(lex.id, dop, amt)
  )

# pzero has potentially become a grouped tibble so make it a data frame
pzero <- as.data.frame(pzero)

# append pzero to original data frame
pdat <- rbind(pdat[, names(pzero)], pzero)

# reinstate the time-scale name
names(pdat)[grep("dop", names(pdat))] <- tnam

# sort by lex.id and time and coerce to data frame

# old code:
oo <- order(pdat$lex.id, pdat[, tnam, drop = TRUE])
pdat <- as.data.frame(pdat[oo, ])
rownames(pdat) <- NULL
pdat

## pdat <- as.data.frame( group_by(pdat, lex.id)
##                    %>% arrange(tnam, .by_group = TRUE)
##                    %>% ungroup() )
}

#----------------------------------------------------------------------
# addDrug.Lexis

addDrug.default <-
addDrug.Lexis <-
function(Lx, # Lexis object, should be timesplit
       pdat, # (named) list of data frames of drug purchaces
        amt = "amt", # name of amount variable in pdat
        dpt = "dpt", # name of amount per time variable in pdat
                     # to be used if method = "dos"
        apt = NULL,
     method = "ext", # extrapolation from times and amounts
                     # "dos" is based on dosage, dpt and amt
                     # "fix" is using a fixed interval, maxt
       maxt = NULL,  # vector of times covered by a single prescription,
      grace = 0,     # vector of grace periods after final data
       tnam = setdiff(names(pdat[[1]]), c("lex.id", amt))[1],
     prefix = TRUE,  # character vector, if FALSE use suffix
     sepfix = ".",   # separator for pre- and suf-fixes
    verbose = TRUE,
        ... )
{
# utility functions
  na0 <- function(x) ifelse(is.na(x), 0, x)
csum0 <- function(x) c(0, cumsum(na0(x)[-length(x)]))

# handling apt / dpt
    if(!is.null(apt))
  {
  cat("Use the agument 'dpt' instead of 'apt',\n",
      "in future releases 'apt' will be deprecated.")
  if(!missing(dpt)) stop("Using both 'dpt' and 'apt' is meaningless.\n")
  dpt = apt
  }

# binding variables to avoid check troubles in CRAN
qwzrx <- exnam <- tfc <- tfxxx <-
        lex.id <- pur <-
       lex.dur <- xtime <- expos <- dospt <- NULL

# don't bother about the warnings
oldopts <- options(warn = -1)
on.exit(options(oldopts))

# Save Lexis attributes to return with the result
lex.attr <- attributes(Lx)

# time scale
if (missing(tnam) & verbose)
   cat("NOTE: timescale taken as '", tnam, "'\n", sep = "")

# construct names for the 0-expanded purchase data frames if not given
if(is.null(names(pdat))) names(pdat) <- paste0('P', 1:length(pdat))

# the renaming vector to be used later
 longnames <- c("expos","tfex","ctime","cdos")
shortnames <- substr(longnames, 1, 2)

# number of purchase files
np <- length(pdat)

# expand maxt and grace by recycling
Lm <- (!missing(maxt ) & length(maxt ) < np)
Lg <- (!missing(grace) & length(grace) < np)
if ((Lm | Lg) & verbose) cat("Values of",
                             if (Lm     ) "maxt",
                             if (Lm & Lg) "and",
                             if (     Lg) "grace",
                             if (Lm & Lg) "have" else "has",
                             "been recycled across", np, "drugs\n")
maxt  <- rep(maxt , np)[1:np]
grace <- rep(grace, np)[1:np]

# Structure to hold the 0-expanded data sets
pdat0 <- list()
pall <- NULL

# Define the 0 amount purchases at exposure end and collect all times in pall
for(i in 1:np)
   {
   pdat0[[i]] <- ins0(pdat[[i]],
                            amt = amt,
                            dpt = dpt,
                         method = method,
                           maxt = maxt[i],
                          grace = grace[i],
                           tnam = tnam,
                        verbose = i == 1 & verbose)
   pall <- rbind(pall, pdat0[[i]][, c("lex.id", tnam)])
   }

# order pall by id and time, remove duplicates and add a bogus
# variable in order to be able to use addCov.Lexis
oo <- order(pall$lex.id, pall[, tnam])
pall <- pall[oo, ]
pall <- pall[!duplicated(pall), ]
pall$qwzrx <- 0

# add the total purchase dates in order to expand to all cut dates when
# appending data from each drug. Make sure that incoming Lx could have
# tfc as time scale
Gx <- addCov.Lexis(Lx, pall, timescale = tnam, tfc = "tfxxx")
Gx <- select(Gx, -qwzrx, -exnam, -tfxxx)

# remove tfxxx as time scale and keep Lexis attributes to resinstate later
wh.tf <- match("tfxxx", attr(Gx, "time.scales"))
 attr(Gx, "time.scales") <- Gsc <- attr(Gx, "time.scales")[-wh.tf]
 attr(Gx, "time.since")  <- Gsi <- attr(Gx, "time.since" )[-wh.tf]
class(Gx)                <- Gcl <- c("Lexis", "data.frame")
# Gx is now a a Lexis object with all the purchases end expiry dates
# added as cuts

# put the expanded drug purchases in the Lexis object one drug at a time
allnam <- NULL
for(i in 1:np)
   {
   # renaming vector the variables with pre- or suffix
              rnam <- shortnames
   if(prefix) rnam <- paste(      names(pdat)[i], rnam, sep = sepfix)
         else rnam <- paste(rnam, names(pdat)[i],       sep = sepfix)
   names(rnam) <- longnames
   allnam <- c(allnam, rnam)

   # add the i'th drug exposure data file
   Gx <- addCov.Lexis(Gx, pdat0[[i]], timescale = tnam, exnam = "pur")

   # compute the exposure variables, drop unneeded variables and rename
   Gx <- ( group_by(Gx, lex.id, pur)
       %>% mutate(xtime = sum(lex.dur) + tfc[1],
                          # total exposure time for this purchase
                          # tfc[1] is the time since purchase at start
                  dospt = amt / xtime ) # dose per time in this interval
       %>% group_by(lex.id)
       %>% mutate(expos = !is.na(amt) & amt > 0,
                  tfex  = csum0(lex.dur * (cumsum(expos) > 0)),
                  ctime = csum0(lex.dur * expos * (amt > 0)) + na0(tfc[1]),
                  cdos  = csum0(lex.dur * expos * dospt)
                                            + na0(dospt[1]) * na0(tfc[1]))
       %>% select(-pur, -xtime, -dospt, -tfc, -amt)
       %>% plyr::rename(rnam)
         )

   # reinstate as Lexis object
    attr(Gx,"time.scales") <- Gsc
    attr(Gx,"time.since")  <- Gsi
   class(Gx)               <- Gcl
   }

return(Gx[,c(setdiff(names(Gx), allnam), allnam)])
}

Lexis2msm <-
function(Lx,  state = "state", verbose = FALSE)
{
# the crap to pass the check
lex.dur <- lex.Xst <- lex.Cst <- NULL
    
# what are the timescales
tscal <- timeScales(Lx)
if (state %in% tscal)
   stop("The state variable cannot have the name of a time scale\n")

# find a timescale with no missing values
ts <- tscal[match(TRUE,
                  apply(Lx[,tscal],
                        2,
                        function(x) all(!is.na(x))))]

# sort so duplicated() makes sense to select last record for each person
Lx <- sortLexis(Lx)

# find the last records among consecutive records within persons
last.rec <- which(rev(!duplicated(rev(Lx$lex.id))) |
                  c( Lx[-1, ts] !=
                    (Lx[   ,ts] + Lx$lex.dur)[-nrow(Lx)],
                    TRUE))
Ll <- Lx[last.rec,]

# timescales at end of last record interval
for (i in tscal) Ll[,i] <- Ll[,i] + Ll$lex.dur

# state at start of each interval
Lx[,state] <- Lx$lex.Cst
# --- and end of last, note we carry covariates forward
Ll[,state] <- Ll$lex.Xst

# combine the data sets and remove irrelevant lex.-variables
Lmsm <- ( rbind(Ll, Lx)
      %>% sortLexis
      %>% select(-c(lex.dur, lex.Xst, lex.Cst))
        )

# re-order the columns
whLx <- c("lex.id", tscal, state)
Lmsm <- Lmsm[,c(whLx, setdiff(names(Lmsm), whLx))]

# fix the attributes
cls <- attr(Lmsm, "class")
cls[cls == "Lexis"] <- "msmLexis"
attr(Lmsm, "class") <- cls
attr(Lmsm, "breaks") <- NULL

# tell what has been going on
if (verbose) cat("Object with state occupied at time points",
                 "on the time scales:\n", tscal, "\n")
Lmsm
}

xcutLexis <- function(Lx, cut, timescale = 1, sep = ".")
{
# the crap to pass the check
lex.Cst <- lex.Xst <-
old.Cst <- old.Xst <-
new.Cst <- new.Xst <-
 lex.id <- old.trn <- NULL
# cuts follow-up in a Lexis obejct, preserving the original states
# hence x-cut: cross-classification
Lx <- sortLexis(Lx)
Lx <- mutate(Lx, old.Cst = as.character(lex.Cst),
                 old.Xst = as.character(lex.Xst))
#
# now, cut at the new state(s), BUT
# ignoring which are absorbing states to keep new "from" states
# - absorbing states in lex.Xst are preserved in old.Xst
Lx <- cutLexis(Lx, cut,
               timescale = timescale,
               precursor.states = levels(Lx))
#
# note that old state variables have been expanded by the cutLexis, so
# if there is one or more new cuts in an initial Lx interval with a
# transition there will be multiple records with idential values of
# both old.Cst and old.Xst but old.Cst != old.Xst.
# So we need to identify such sequences of records and then change
# the value of old.Xst to that of old.Cst for these units
Lx <- mutate(Lx, new.Cst = as.character(lex.Cst),
                 new.Xst = as.character(lex.Xst),
               # different Cst and Xst and the pair identical to next
               # indicating the units to change (within person)
                 old.trn = paste0(old.Cst, old.Xst),
                 old.chg = (old.Cst != old.Xst) &
                           (lex.id  == lex.id[c(2:nrow(Lx), 1)]) &
                           (old.trn == c(old.trn[-1], "")))
Lx$old.Xst[Lx$old.chg] = Lx$old.Cst[Lx$old.chg]
#
# now for the construction of new states
Lx <- mutate(Lx, lex.Cst = ifelse(old.Cst == new.Cst,
                                  new.Cst,
                                  paste0(old.Cst, sep, new.Cst)),
                 lex.Xst = ifelse(old.Xst == new.Xst,
                                  new.Xst,
                                  paste0(old.Xst, sep, new.Xst)))
#
# clean up and beautify
todrop <- match(c("old.Cst", "new.Cst",
                  "old.Xst", "new.Xst",
                  "old.trn", "old.chg"), names(Lx))
factorize(Lx[, -todrop])
}

## Program to calculate effects for matched case-control studies
## Michael Hills
## Improved by BxC and MP
## Post Tartu 2007 version June 2007


effx.match<-function(response,
exposure,
match,
strata=NULL,
control=NULL,
base=1,
digits=3,
alpha=0.05,
data=NULL)
{

  ## stores the variable names for response, etc.

    rname<-deparse(substitute(response))
    ename<-deparse(substitute(exposure))
    if (!missing(strata))sname<-deparse(substitute(strata))

  ## The control argument is more complex, as it may be a name or
  ## list of names
  
  if(!missing(control)) {
    control.arg <- substitute(control)
    if (length(control.arg) > 1) {
      control.names <- sapply(control.arg, deparse)[-1]
    }
    else {
      control.names <- deparse(control.arg)
    }
  }
  ## If data argument is supplied, evaluate the arguments in that
  ## data frame.

  if (!missing(data)) {
    exposure <- eval(substitute(exposure), data)
    response <- eval(substitute(response), data)
    match <- eval(substitute(match),data)
    if (!missing(strata)) {
      strata <- eval(substitute(strata), data)
    }
    if (!missing(control))
      control <- eval(substitute(control), data)
  }

  ## performs a few other checks

  if(rname==ename)stop("Same variable specified as response and exposure")
  if (!missing(strata)) {
    if(rname==sname)stop("Same variable specified as response and strata")
    if(sname==ename)stop("Same variable specified as strata and exposure")
  }

  if(!is.numeric(response))stop("Response must be numeric, not a factor")
  
  if(!missing(strata)&!is.factor(strata))stop("Stratifying
    variable must be a factor")

  tmp<-(response==0 | response==1)
  if(all(tmp,na.rm=TRUE)==FALSE)
  stop("Binary response must be coded 0,1 or NA")

  if(class(exposure)[1]=="ordered") {
      exposure<-factor(exposure, ordered=F)
  }

  ## Fix up the control argument as a named list
  if (!missing(control)) {
    if (is.list(control)) {
      names(control) <- control.names
    }
    else {
      control <- list(control)
      names(control) <- control.names
    }
  }

  ## prints out some information about variables


  cat("---------------------------------------------------------------------------","\n")
  cat("response      : ", rname, "\n")
  cat("exposure      : ", ename, "\n")
  if(!missing(control))cat("control vars  : ",names(control),"\n")
  if(!missing(strata)) cat("stratified by : ",sname,"\n")
  cat("\n")
  if(is.factor(exposure)) {
    cat(ename,"is a factor with levels: ")
    cat(paste(levels(exposure),collapse=" / "),"\n")
    cat( "baseline is ", levels( exposure )[base] ,"\n")   
    exposure <- Relevel( exposure, base )
  }
  else {
    cat(ename,"is numeric","\n")
  }
  if(!missing(strata)) {
    cat(sname,"is a factor with levels: ")
    cat(paste(levels(strata),collapse="/"),"\n")
  }

  cat("effects are measured as odds ratios","\n")
  cat("---------------------------------------------------------------------------","\n")
  cat("\n")

  ## gets number of levels for exposure if a factor

  if(is.factor(exposure)) {
    nlevE<-length(levels(exposure))
  }
  else {
    nlevE<-1
  }

  ## labels the output
  
  if(is.factor(exposure)) {
    cat("effect of",ename,"on",rname,"\n")
  }
  else {
    cat("effect of an increase of 1 unit in",ename,"on",rname,"\n")
  }
  if(!missing(control)) {
    cat("controlled for",names(control),"\n\n")
  }
  if(!missing(strata)) {
    cat("stratified by",sname,"\n\n")
  }


  ## no stratifying variable

  if(missing(strata)) {
          if(missing(control)) {            
            m<-clogit(response~exposure+strata(match))
            cat("number of observations ",m$n,"\n\n")
          }
          else  {
              m<-clogit(response~.+exposure+strata(match),
                     subset=!is.na(exposure),data=control)
              cat("number of observations ",m$n,"\n\n")
              mm<-clogit(response~.+strata(match),
                      subset=!is.na(exposure),data=control)
          }
          res<-ci.lin(m,subset=c("exposure"),Exp=TRUE,alpha=alpha)[,c(5,6,7)]
          res<-signif(res,digits)
          if(nlevE<3) {
              names(res)[1]<-c("Effect")
          }
          else {
              colnames(res)[1]<-c("Effect")
              if(is.factor(exposure)) {
                ln <- levels(exposure)
                rownames(res)[1:nlevE-1]<-paste(ln[2:nlevE],"vs",ln[1])
              }
          }
          print(res)
          if(missing(control)) {
              chisq<-round(summary(m)$logtest[1],2)
              df<-round(summary(m)$logtest[2])
              p<-round(summary(m)$logtest[3],3)
              cat("\n")
              cat("Test for no effects of exposure:  ","\n")
              cat("chisq=",chisq, " df=",df, " p-value=",format.pval(p,digits=3),"\n")
              invisible(list(res,paste("Test for no effects of exposure on",
                 df,"df:","p=",format.pval(p,digits=3))))

          }
          else  {
              aov <- anova(mm,m,test="Chisq")
              cat("\nTest for no effects of exposure on",
              aov[2,3],"df:",
              "p-value=",format.pval(aov[2,5],digits=3),"\n")
              invisible(list(res,paste("Test for no effects of exposure on",
                 aov[2,3],"df:","p=",format.pval(aov[2,5],digits=3))))
          }
  }      
   ## stratifying variable

  if(!missing(strata)) {
      sn <- levels(strata)
      nlevS<-length(levels(strata))
          if(missing(control)) {
            m<-clogit(response~strata/exposure+strata(match))
            cat("number of observations ",m$n,"\n\n")
            mm<-clogit(response~strata+exposure+strata(match))
          }
          else {
            m <-clogit(response~strata/exposure + . +strata(match),
            data=control)
            cat("number of observations ",m$n,"\n\n")
            mm <-clogit(response~strata+exposure + . +strata(match),
            data=control)
          }
          res<-ci.lin(m,Exp=TRUE,alpha=alpha,subset="strata")[c(-1:-(nlevS-1)),c(5,6,7)]
          res<-signif(res,digits)
          colnames(res)[1]<-c("Effect")
          if(is.factor(exposure)) {
            ln<-levels(exposure)
            newrownames<-NULL
            for(i in c(1:(nlevE-1))) {
              newrownames<-c(newrownames,
                             paste("strata",sn[1:nlevS],"level",ln[i+1],"vs",ln[1]))
            }
          }
          else {
             newrownames<-paste("strata",sn[1:nlevS])
          }
          rownames(res)<-newrownames
          aov<-anova(mm,m,test="Chisq")
          print( res )
          cat("\nTest for effect modification on",
          aov[2,3],"df:","p-value=",format.pval(aov[2,5],digits=3),"\n")
          invisible(list(res,paste("Test for effect modification on",
          aov[2,3],"df:","p-value=",format.pval(aov[2,5],digits=3))))

  }
}

erl <-
function( int,
          muW,
          muD,
          lam = NULL,
       age.in = 0,
            A = NULL,
       immune = is.null(lam),
          yll = TRUE,
         note = TRUE )
{
# Computes expected residual life time for Well and Dis states
# respectively in an illness-death model, optionally ignoring
# the well->ill transition

# Utility to integrate a survival function from the last point where
# it is 1, assuming points are 1 apart
trsum <-
function( x )
{
x[c(diff(x)==0,TRUE)] <- NA     
sum( ( x[-length(x)] + x[-1] ) / 2, na.rm=TRUE )
}

# Check sensibility
if( !immune & is.null(lam) ) stop( "'lam' is required when immune=FALSE\n" )

# Replace NAs with 0s
if( !is.null(lam) ) {
if( any(is.na(lam)) ){ lam[is.na(lam)] <- 0 ; warning("NAs in agument 'lam' set to 0") } }
if( any(is.na(muD)) ){ muD[is.na(muD)] <- 0 ; warning("NAs in agument 'muD' set to 0") }
if( any(is.na(muW)) ){ muW[is.na(muW)] <- 0 ; warning("NAs in agument 'muW' set to 0") }
    
# Survival functions    
             sD <- surv1( int=int,      muD,      age.in = age.in, A = A )
if( immune ) sW <- surv1( int=int, muW,           age.in = age.in, A = A )    
else         sW <- surv2( int=int, muW, muD, lam, age.in = age.in, A = A )

# Area under the survival functions    
erl <- cbind( apply( sW[,-1], 2, trsum ),
              apply( sD[,-1], 2, trsum ) ) * int
colnames( erl ) <- c("Well","Dis")
rownames( erl ) <- colnames( sW )[-1]

# Should we compute years of life lost? 
if( yll ) erl <- cbind( erl, YLL = erl[,"Well"] - erl[,"Dis"] )

# Cautionary note
if( immune )
  {
  attr( erl, "NOTE" ) <- "Calculations assume that Well persons cannot get Ill (quite silly!)."
  if( note ) cat("NOTE:",  attr( erl, "NOTE" ), "\n" )
  }  
return( erl )
}

# yll is just a simple wrapper for erl, only selecting the YLL column.
yll <-
function( int,
          muW,
          muD,
          lam = NULL,
       age.in = 0,
            A = NULL,
       immune = is.null(lam),
         note = TRUE ) erl( int = int,      
                            muW = muW,      
                            muD = muD,      
                            lam = lam,      
                         age.in = age.in,   
                              A = A,        
                         immune = immune,   
                            yll = TRUE,
                           note = note )[,"YLL"]

surv1 <-
function( int, mu, age.in=0, A=NULL )
{
# Computes the survival function from age A till the end, assuming
# that mu is a vector of mortalities in intervals of length int. 
# int and mu should be in compatible units that is T and T^-1 for
# some unit T (months, years, ...) 

# impute 0s for NAs
if( any(is.na(mu)) ){ mu[is.na(mu)] <- 0 ; warning("NAs in agument 'mu' set to 0") }
    
# age-class boundaries    
age <- 0:length(mu)*int + age.in

# cumulative rates and survival at the boundaries    
Mu <- c( 0, cumsum( mu )*int )
Sv <- exp( -Mu )
surv <- data.frame( age=age, surv=Sv )
    
# if a vector of conditioning ages A is given
if( cond <- !is.null(A) )
  {
  j <- 0
  # actual conditioning ages
  cage <- NULL
  for( ia in A )
     {
     j <- j+1
     # Where is the age we condition on
     cA <- which( diff(age>ia)==1 )
     # survival up to condtioning age is set to 1 
     surv <- cbind( surv, pmin( 1, surv$surv/(surv$surv[cA]) ) )
     cage[j] <- surv$age[cA]
     }
  }
names( surv )[-1] <- paste( "A", c( age.in, if( cond ) cage else NULL ), sep="" )
rownames( surv ) <- NULL
return( surv ) 
}

erl1 <-
function( int, mu, age.in = 0 )
{
# Computes expected residual life time at all ages    
age <- 0:length(mu)*int + age.in

# Small utility: cumulative cumulative sum from the end of a vector
musmuc <- function( x ) rev( cumsum( rev(x) ) )

# The survival function with a 0 at end, and the integral from the upper end
surv <- surv1( int = int, mu = mu, age.in = age.in )[,2]
cbind( age = age,
      surv = surv,
       erl = c( musmuc( ( surv[-1]-diff(surv)/2 ) ) /
                surv[-length(surv)], 0 ) * int )
}

surv2 <-
function( int, muW, muD, lam, age.in=0, A=NULL )
{
# check the vectors
if( length(muW) != length(muD) | 
    length(muD) != length(lam) ) 
  stop( "Vectors with rates must have same length:\n",
          "length(muW)=", length(muW),
        ", length(muD)=", length(muD),
        ", length(lam)=", length(lam) )

# Replace NAs with 0s
if( !is.null(lam) ) {
if( any(is.na(lam)) ){ lam[is.na(lam)] <- 0 ; warning("NAs in agument 'lam' set to 0") } }
if( any(is.na(muD)) ){ muD[is.na(muD)] <- 0 ; warning("NAs in agument 'muD' set to 0") }
if( any(is.na(muW)) ){ muW[is.na(muW)] <- 0 ; warning("NAs in agument 'muW' set to 0") }

# First the workhorse that computes the survival function for a
# person in Well assuming that the mortality rate from this state is
# muW, disease incidence is in lam, and mortality in the diseased
# state is muD, and that all refer to constant rates intervals of
# length int starting from age.in, conditional on survival to A 
wsurv2 <-
function( int, muW, muD, lam, age.in=0, A=0 )
{
# age-class boundaries - note one longer than rate vectors as it
# refers to boundaries of intervals not midpoints  
age <- 0:length(muW)*int + age.in
    
# cumulative rates at the boundaries, given survival to A
MuW <-  cumsum( c( 0, muW ) * ( age > A ) ) * int
MuD <-  cumsum( c( 0, muD ) * ( age > A ) ) * int
Lam <-  cumsum( c( 0, lam ) * ( age > A ) ) * int
    
# probability of being well
pW <- exp( -( Lam + MuW ) )
    
# probability of diagnosis at s --- first term in the integral for
# P(DM at a). Note that we explicitly add a 0 at the start so we get a
# probability of 0 of transition at the first age point 
Dis <- c(0,lam) * ( age > A ) * exp( -(Lam+MuW) ) * int
    
# for each age (age[ia]) we compute the integral over the range
# [0,age] of the product of the probability of diagnosis and the
# probability of surviving from diagnosis till age ia
pDM <- Dis * 0
for( ia in 1:length(age) )
   pDM[ia] <- sum( Dis[1:ia] * exp( -(MuD[ia]-MuD[1:ia]) ) )
                   # 1st term as function of s (1:ia)
                               # 2nd term integral over range s:age
                   # upper integration limit is age (ia) and the lower
                   # limit is the intermediate age (at DM) (1:ia)
# Finally, we add the probabilities of being in Well resp. DM to get
# the overall survival:
surv <- data.frame( age = age, surv = pDM + pW )
return( surv )
}    

# survival from start    
surv <- wsurv2( int, muW, muD, lam, age.in=age.in, A=0 )

# add columns for conditioning ages    
if( !is.null(A) )
   { 
   for( j in 1:length(A) )
      { 
      surv <- cbind( surv, 
                     wsurv2( int, muW, muD, lam, age.in=age.in, A=A[j] )[,2] )
      }
   }
Al <- A
for( i in 1:length(A) ) Al[i] <- max( surv$age[surv$age <= A[i]] )
colnames( surv )[-1] <- paste( "A", c( age.in, Al ), sep="" )    

# done!    
return( surv )
}

summary.Icens <-
function( object, scale=1, ... )
{
  if( attr( object, "model" ) == "MRR" )
    {
      if( is.null( object$rates ) )
        {
          class( object ) <- "glm"
          emat <- ci.lin( object )
        }
      else
        {
          rate.est <- ci.lin( object$rates )
          rate.est[,-(3:4)] <- rate.est[,-(3:4)] * scale
          emat <- rbind( cbind( rate.est, RR=NA )[,c(1:4,7,5:6)],
                        ci.lin( object$cov, Exp=T ) )
        }
    }
  if( attr( object, "model" ) == "AER" )
    {
      rate.est <- ci.lin( object$rates )
      rate.est[,-(3:4)] <- rate.est[,-(3:4)] * scale
      emat <- rate.est
    }
  if( length( object ) == 4 )
    {
      b.est <- object[["boot.ci"]]
      colnames( b.est ) <- c( "Boot-med", "lo", "hi" )
      emat <- cbind( emat, b.est )
    }
  emat
}

rm.tr <-
function( obj, from, to )
    {    
    # checks
    if( from==to) stop( "Don't be silly, 'from' and 'to' are identical." )
    if( !(from %in% levels(obj) ) ) stop( "'from' not a state in the object." )
    if( !(  to %in% levels(obj) ) ) stop( "  'to' not a state in the object." )
### These things do not change over the purging iterations 
    # Sort the rows of the object and count them   
    obj <- obj[order(obj$lex.id,obj[,timeScales(obj)[1]]),]
    nrw <- nrow( obj )
    # First obs for each person    
    no1 <- !duplicated( obj$lex.id )
    wh1 <- which( no1 )
### Utility function doing the work inside the loop        
purge.one <-
function( obj, from, to, nrw, wh1 )
    {
    # Where are the illegal transitions 
    chX <- ( paste( obj$lex.Cst, obj$lex.Xst ) == paste( from, to ) )
    whX <- which( chX )
    if( length(whX)>0 )
      {  
      # Change lex.Xst in this record
      obj$lex.Xst[whX] <- obj$lex.Cst[whX]
      # and lex.Cst in next record, but only if it is not a new person   
      whX <- setdiff( whX[whX<nrw]+1, wh1 ) 
      obj$lex.Cst[whX] <- obj$lex.Cst[whX-1]
      return( obj )
      }
    else return(NULL)
    }
### Iterate till all illegal transitions are weeded out        
cont <- TRUE
while( cont )
    {
    zzz <- obj
    cont <- !is.null( obj <- purge.one( obj=zzz,
                                        from=from, to=to,
                                        nrw = nrw, wh1 = wh1 ) )
    }
### Return last actual object        
zzz
}

float.variance <- function(V, tol=1.0e-3, iter.max = 50)
{
    ## Calculate floated variances for variance matrix V, which is
    ## assumed to represent a set of treatment contrasts
    
    m <- nrow(V)
    if (!is.matrix(V) || ncol(V) != m || m == 1)
        stop ("V must be a square matrix of size 2 x 2 or more")

    evals <- eigen(V, only.values=TRUE)$values
    if(any(evals < 0))
        stop("V not positive definite")
    
    ## Starting values from Easton et al (1991)
    R <- V - diag(diag(V))
    V00 <- sum(R)/(m * (m-1))
    V10 <- apply(R, 1, sum)/(m-1)
    fv <- c(V00, V00 - 2 * V10 + diag(V))
    
    for(iter in 1:iter.max) {
        w <- 1/fv
        S <- sum(w)
        w1 <- w[-1]/S
        ##Augment data matrix
        V10 <- as.vector(V %*% w1)
        V00 <- as.vector(1/S + t(w1) %*% V %*% w1)
        ##Calculate new estimates
        fv.old <- fv
        fv <- c(V00, V00 - 2 * V10 + diag(V))
        ## Check convergence
        if(max(abs(fv.old - fv)/fv) < tol)
            break
    }
    if (iter == iter.max)
        warning("Floated variance estimates did not converge")
    
    Vmodel.inv <- S * (diag(w1) - w1 %*% t(w1))
    evals <- 1/(eigen(V %*% Vmodel.inv, only.values=TRUE)$values)
    divergence <- sum(1/evals - 1 + log(evals))/2
    return(list(variance=fv, error.limits=sqrt(range(evals)),
                divergence=divergence))
}

"float" <-
function(object, factor, iter.max = 50, ...)
{
  if (is.null(object$xlevels)) {
    stop("No factors in model")
  }
  
  if (missing(factor)) {
    i <- 1
    factor <- names(object$xlevels)[1]
  }
  else {
    i <- pmatch(factor, names(object$xlevels))
    if (is.na(i)) {
      stop(paste("Factor",i,"not found in model"))
    }
  }
    
  xcontrasts <- object$contrasts[[i]]
  xlevels <- object$xlevels[[i]]
  xname <- names(object$xlevels)[i]
  nlevels <- length(xlevels)

  ## Extract the coefficients and variance matrix for a single factor
  ## from object
        
  if (nlevels <= 2) {
    stop ("Floated variances undefined for factors with less than 3 levels")
  }
        
  ## Get contrast matrix
  C <- if (is.matrix(xcontrasts)) {
    xcontrasts
  }
  else {
    get(xcontrasts, mode="function")(xlevels)
  }
  if (qr(C)$rank < nlevels - 1) {
    stop ("Impossible to reconstruct treatment contrasts")
  }

  ## Get coefficients and variance matrix
  if(is.null(cnames <- colnames(C)))
    cnames <- 1:(nlevels-1)
  contr.names <- paste(xname, cnames, sep="")
  coef <- coef(object)[contr.names]
  V <- vcov(object, ...)[contr.names, contr.names]
        
  ## Convert to treatment contrast parameterization
  if (identical(xcontrasts, "contr.treatment")) {
    V.tc <- V
    coef.tc <- c(0, coef)
  }
  else {
    D.inv <- cbind(rep(-1,nlevels-1), diag(nlevels-1))
    S <- D.inv %*% cbind(rep(1, nlevels), C) 
    S <- S[,-1]
    ## coefficients
    coef.tc <- c(0, S %*% coef)
    ## If we find a baseline level (implicitly defined
    ## by having a row of zeros in the contrast matrix)
    ## then adjust the coefficients
    is.base <- apply(abs(C), 1, sum) == 0
    if (any(is.base))
      coef.tc <- coef.tc - coef.tc[is.base]
    ## variance matrix
    V.tc <- S %*% V %*% t(S)
  }
  names(coef.tc) <- xlevels

  float.out <- float.variance(V.tc, iter.max = iter.max)
  var <- float.out$var
  names(var) <- xlevels
  ans <- list(coef=coef.tc, var=var, limits=float.out$error.limits,
              factor=factor)
  class(ans) <- "floated"
  return(ans)
}

rateplot <-
function( rates,
          which = c("ap","ac","pa","ca"),
            age = as.numeric( dimnames( rates )[[1]] ),
            per = as.numeric( dimnames( rates )[[2]] ),
           grid = FALSE,
         a.grid = grid,
         p.grid = grid,
         c.grid = grid,
          ygrid = grid,
       col.grid = gray( 0.9 ),
          a.lim = range( age, na.rm=TRUE ) + c(0,diff( range( age ) )/30),
          p.lim = range( per, na.rm=TRUE ) + c(0,diff( range( age ) )/30),
          c.lim = NULL, 
           ylim = range( rates[rates>0], na.rm=TRUE ),
             at = NULL,
         labels = paste( at ),
          a.lab = "Age at diagnosis",
          p.lab = "Date of diagnosis",
          c.lab = "Date of birth",
           ylab = "Rates",
           type = "l",
            lwd = 2,
            lty = 1,
         log.ax = "y",
            las = 1,
            ann = FALSE,
          a.ann = ann,
          p.ann = ann,
          c.ann = ann,
          xannx = 1/20,
        cex.ann = 0.8,
         a.thin = seq( 1, length( age ), 2 ),
         p.thin = seq( 1, length( per ), 2 ),
         c.thin = seq( 2, length( age ) + length( per ) - 1, 2 ),
            col = par( "fg" ),
          a.col = col,
          p.col = col,
          c.col = col,
            ... )
{
# Remove 0 rates, in order to avoid warnings
rates[rates==0] <- NA
# then do the plots
for( i in 1:length( which ) )
    {      
    if( toupper( which[i] ) == "AP" )
       Aplot( rates,
                age = age,                                      
                per = per,                                      
             a.grid = a.grid,                                   
              ygrid = ygrid,                                    
           col.grid = col.grid,                                 
              a.lim = a.lim,                                    
               ylim = ylim,                                     
              a.lab = a.lab,                                    
               ylab = ylab,                                     
                 at = at,
             labels = labels,
               type = type,                                     
                lwd = lwd,                                      
                lty = lty,                                      
                col = col,                                      
             log.ax = log.ax,                                   
                las = las,                                      
              p.ann = p.ann,
              xannx = xannx,
              p.col = p.col,                                    
            cex.ann = cex.ann,                                  
             p.thin = p.thin,                                   
            p.lines = TRUE,                                     
            c.lines = FALSE,
                ... )
    
    if( toupper( which[i] ) == "AC" )
       Aplot( rates,
                age = age,                                          
                per = per,                                      
             a.grid = a.grid,                                   
              ygrid = ygrid,                                    
           col.grid = col.grid,                                  
              a.lim = a.lim,                                    
               ylim = ylim,                                     
              a.lab = a.lab,                                    
               ylab = ylab,                                     
                 at = at,
             labels = labels,
               type = type,                                     
                lwd = lwd,                                      
                lty = lty,                                      
                col = col,                                      
             log.ax = log.ax,                                   
                las = las,                                      
              c.ann = c.ann,                                    
              p.ann = p.ann,                                    
              xannx = xannx,
              c.col = c.col,                                    
              p.col = p.col,                                    
            cex.ann = cex.ann,                                  
             c.thin = c.thin,                                   
            p.lines = FALSE,                                     
            c.lines = TRUE,
                ... )                                    
    
    if( toupper( which[i] ) %in% c("APC","ACP") )
       Aplot( rates,            
                age = age,          
                per = per,      
             a.grid = a.grid,   
              ygrid = ygrid,    
           col.grid = col.grid,  
              a.lim = a.lim,    
               ylim = ylim,     
              a.lab = a.lab,    
               ylab = ylab,     
                 at = at,
             labels = labels,
               type = type,     
                lwd = lwd,      
                lty = lty,      
                col = col,      
             log.ax = log.ax,   
                las = las,      
              c.ann = c.ann,    
              p.ann = p.ann,    
              xannx = xannx,
              c.col = c.col,    
              p.col = p.col,    
            cex.ann = cex.ann,  
             c.thin = c.thin,   
             p.thin = p.thin,   
            p.lines = TRUE,     
            c.lines = TRUE,
                ... )               
   
    if( toupper( which[i] ) == "PA" )
       Pplot( rates,
                age = age,
                per = per,
               grid = grid,
             p.grid = p.grid,
              ygrid = ygrid,
           col.grid = col.grid,
              p.lim = p.lim,
               ylim = ylim,
              p.lab = p.lab,
               ylab = ylab,
                 at = at,
             labels = labels,
               type = type,
                lwd = lwd,
                lty = lty,
                col = col,
             log.ax = log.ax,
                las = las,
                ann = a.ann,
              xannx = xannx,
            cex.ann = cex.ann,
             a.thin = a.thin,
                ... )

    if( toupper( which[i] ) == "CA" )
       Cplot( rates,
                age = age,
                per = per,
               grid = grid,
             c.grid = c.grid,
              ygrid = ygrid,
           col.grid = col.grid,
              c.lim = c.lim,
               ylim = ylim,
              c.lab = c.lab,
               ylab = ylab,
                 at = at,
             labels = labels,
               type = type,
                lwd = lwd,
                lty = lty,
                col = col,
             log.ax = log.ax,
                las = las,
                ann = a.ann,
              xannx = xannx,
            cex.ann = cex.ann,
             a.thin = a.thin,
                ... )
    
    }
}

# The addCov method
addCov <- function (Lx, ...) UseMethod("addCov")

addCov.default <-
addCov.Lexis <-
function(Lx,
       clin,
  timescale = 1,
      exnam,
        tfc = "tfc", ...)
{
# Function to add clinically measured covariates to a Lexis object

# The point is to cut the Lexis object at the examination dates
# and subsequently add the clinical records

# to avoid notes in check
org.Cst <- NULL
org.Xst <- NULL
lex.Cst <- NULL
lex.Xst <- NULL
lex.id  <- NULL

# ...but first the usual cheking of paraphernalia

if(!inherits(Lx  ,"Lexis")) stop("Lx must be a Lexis object.\n")
if( inherits(clin,"Lexis")) stop("clin cannot be a Lexis object.\n")

# Is the timescale argument a timescale in Lx and is it a variable in clin?
ts <- if(is.numeric(timescale)) timeScales(Lx)[timescale] else timescale
if(!(ts %in% timeScales(Lx)))
    stop("timescale argument (", ts,
         ") must be among the timescales in the Lexis object ",
         deparse(substitute(Lx)),":", timeScales(Lx), ".\n" )

clin.nam <- deparse(substitute(clin))
if( !( ts %in% names(clin) & "lex.id" %in% names(clin) ) )
    stop("'lex.id' and timescale '", ts,
         "' must be variables in the clin object ",
         clin.nam, "\n" )

# order clin to get the possible construction of examination names ok
clin <- clin[order(clin$lex.id, clin[,ts]),]

# check that examination dates are unique within persons
if(any(dd <- duplicated(clin[,c("lex.id",ts)])))
  {
  warning("Examination dates must be unique within persons\n",
          sum(dd), " records with duplicate dates from clin object ",
          clin.nam, " excluded.")
  clin <- clin[!dd,]
  }

# the variable holding the name of the examination
if(missing(exnam)) exnam <- "exnam"
# and if it is not there, construct it as ex1, ex2 etc.
if (!(exnam %in% names(clin)))
   {
   clin <- group_by(clin, lex.id)              %>%
           mutate(zz = paste0("ex",
                              1:length(lex.id))) %>%
           ungroup()
   names(clin)[grep("zz",names(clin))] <- exnam
   clin <- as.data.frame(clin)
   }

# exnam cannot have values that are also states
if (length(common <- intersect(levels(Lx),
                               unique(clin[,exnam]))))
  stop("Levels of Lx and examination names in clin must be disjoint",
       "\nbut", paste(common, collapse=", "), "are in both")

#...done checking

# variables to merge by
mvar <- c("lex.id", ts)

# clinical variables to be merged in
# --- note we take examination date and name as a clinical variable too
cvar <- setdiff(names(clin), mvar)

# A data frame of cutting times of the examinations
cfr <- data.frame(lex.id = clin$lex.id,
                     cut = clin[,ts],
               new.state = clin[,exnam])

# a copy of Lx with a saved copy of the state variables in org.
Lc <- transform(Lx, org.Cst = lex.Cst,
                    org.Xst = lex.Xst)

# Now cut Lc at each new examination date, state variables will be
# changed to examination names
Lc <- rcutLexis(Lc,
               cut = cfr,
         timescale = ts)

# Lc now has the exnam in the variable lex.Cst, so we can merge the
# clinical data to this if we rename to exnam
# the lex.Cst contains the examination name, except where
# the original levels are left
Lc[,exnam] <- as.factor(ifelse(Lc$lex.Cst %in% levels(Lx),
                               NA,
                               as.character(Lc$lex.Cst)))
mvar <- c("lex.id", exnam)

# timescale is present in both Lc and clin,
# so rename in clin, it will be the date of clin
names(clin)[grep(ts, names(clin))] <- tfc

# merge with clinical measurements keeping the attributes
att.Lc <- attributes(Lc)
Lc <- left_join(Lc, clin, by = mvar)
att.Lc$names <- attributes(Lc)$names
# attributes(Lc) <- att.Lc

# compute time since last examination
Lc[,tfc] <- Lc[,ts] - Lc[,tfc]

# we have problems if a clical date falls in an interval
# ending with a transition, they need to be fixed:
wh <- with(Lc, which(c(org.Xst[-nrow(Lc)] != org.Cst[-1] &
                       lex.id [-nrow(Lc)] == lex.id [-1])))
Lc$org.Xst[wh] <- Lc$org.Cst[wh+1]

# move the original states back
Lc <- select(Lc, -lex.Cst,
                 -lex.Xst) %>%
      rename(lex.Cst = org.Cst,
             lex.Xst = org.Xst)

# Add tfc as a time.scale, time.since and breaks:
attr(Lc, "time.scales") <- c(attr(Lx, "time.scales"), tfc)
attr(Lc, "time.since" ) <- c(attr(Lx, "time.since" ), "X")
brt <- list(x = NULL)
names(brt) <- tfc
attr(Lc, "breaks") <- c(attr(Lx, "breaks"), brt)
attr(Lc, "class")  <- c("Lexis","data.frame")

# Done! - well order first
sortLexis(Lc)
}

# convenience function to compute midpoints between points in a vector
midpoint <- function(x) x[-1] - diff(x) / 2

#----------------------------------------------------------------------
# Simulate rates from a list of models, 0th simulation is the rates
# based on the model estimates
simrates <-
function(mods, # named list of models - one for each cause
           nd, # prediction data frame - common for all causes
         tnam,
           nB = 1000, # number of simulated samples
          int = mean(diff(nd[,tnam]))) # time is first in column of nd
#        seed) not implemented yet
{
# from a list of models (mods) for nC cause-specific rates and a
# common prediction frame, nd, we generate nB samples of predicted
# rates. It is assumed that the nd refer to endpoints of intervals
res <- NArray(list(time = nd[,tnam],
                    mod = names(mods),
                    sim = 1:nB))
nT <- nrow(nd) - 1 # no of interval midpoints
nC <- length(mods) # no. competing risks
for(i in 1:nC) res[,i,] <- ci.lin(mods[[i]], nd, sample = nB)
res <- exp(res)    # esimated rates at the interval endpoints
attr( res, "int" ) <- int
res
}

#----------------------------------------------------------------------
# Compute nC+1 cumulative risks from nC rates
mkprobs <-
function(rtab, int)
{
# rtab is assumed to be a matrix with rates in nC (named) columns
# at nT equidistantly placed times at 0, int, 2*int, ...
# so of dimension nT x nC
nT <- nrow(rtab)
nC <- ncol(rtab)

# intensities at interval midpoints so (nT-1) x nC
mtab <- (rtab[-1,,drop=FALSE] + rtab[-dim(rtab)[1],,drop=FALSE]) / 2

# integrated intensities at interval boundaries, so nT x nC
Rtab <- rbind(0, apply(mtab, 2, cumsum)) * int

# state probabilities at interval endpoints so nT x (1+nC)
Pr <- Rtab[,c(1,1:nC)] * NA
colnames(Pr)[1] <- "Surv" # Survival prob in first column

# survival function (also the first column in Pr)
Pr[,"Surv"] <- Sv <- exp(-apply(Rtab, 1, sum))

# convenience function to compute midpoints between points in a vector
midpoint <- function(x) x[-1] - diff(x) / 2
# cumulative risks - note we are using the midvalue of the Sv, length nT
for( i in 1:nC ) Pr[,i+1] <- c(0, cumsum(mtab[,i] * midpoint(Sv)) * int)
Pr
}

#----------------------------------------------------------------------
# return first, mean and median and ci from a simulated sample
mnqt <-
function(x, alpha = 0.05)
{
# extracts the relevant quantiles across the
# simulated samples
c(quantile(x,
           probs = c(0.5, alpha / 2, 1 - alpha / 2),
           na.rm = TRUE))
}

#----------------------------------------------------------------------
# Here is the function we need to compute cumulative risks
ci.Crisk <-
function(mods, # list of models
           nd, # prediction data frame for rates at points
               # representing interval endpoints
         tnam = names(nd)[1],
           nB = 1000,   # no of parametic bootstrap samples
         perm = length(mods):0 + 1, # order of cumulation of states
        alpha = 0.05,   # 1 minus confidence level
      sim.res = 'none')
#        seed = ) not implemented yet
{
int <- mean(diff(nd[,tnam])) # interval length in nd
if (length(table(round(diff(nd[,tnam]), 6))) > 1)
   stop("Time column column of nd must be equidistant times\n")
cat("NOTE: Times are assumed to be in the column", tnam,
    "at equal distances of", int, "\n")

# First generate the simulated probabilities T x (C+1) x nB; simrt
# contains the estimates as the first entry in the simulation dimension
simrt <- simrates(mods = mods,
                    nd = nd,
                  tnam = tnam,
                    nB = nB,
                   int = int)
#                 seed = seed) not implemented yet

# if we want the simulation results for further processing
attr(simrt, "int") <- int # the interval length needed with simrt
attr(simrt, "time") <- nd[,tnam]
if(substr(tolower(sim.res), 1, 1) == 'r') return(invisible(simrt))

# Array for the state probabilities
probs <- NArray(list(time = nd[,tnam],
                    cause = c("Surv", dimnames(simrt)[["mod"]]),
                      sim = 1:nB))
names(dimnames(probs))[1] <- tnam
# Compute the cumulative risks
  # a trick to avoid dropping dimension 2 if it has length 1
  x2 <- c(1, 1:dim(simrt)[2])
for(i in 1:nB) probs[,,i] <- mkprobs(simrt[,x2,i][,-1,drop=FALSE], int = int)
attr(probs, "int") <- int # the interval length needed with probs

if(substr(tolower(sim.res), 1, 1) == 'c') return(invisible(probs))

# otherwise we compute confidence intervals for selected statistics
  # 1. state (cumulative) probabilities
  Crisk <- sim2ci.Crisk(probs, alpha = alpha)
  # 2. stacked (cumulative) probabilities
  Srisk <- sim2ci.Srisk(probs,  perm = perm,
                               alpha = alpha)
  # 3. sojourn times in states
  Stime <- sim2ci.Stime(probs, alpha = alpha)

# finally return in a list of useful quantities
rlist <- list(Crisk = Crisk,
              Srisk = Srisk,
              Stime = Stime,
               time = nd[,tnam])
attr(rlist, "int") <- int # the interval length used
return(invisible(rlist))
}

#----------------------------------------------------------------------
# 1. cumulative probabilities
sim2ci.Crisk <-
function(probs,
         alpha = 0.05)
aperm(apply(probs, 1:2, mnqt, alpha), c(2,3,1))

#----------------------------------------------------------------------
# 2. stacked (cumulative) probabilities
sim2ci.Srisk <-
function(probs,
          perm = 1:dim(probs)[2],
         alpha = 0.05)
{
cprobs <- aperm(apply(probs[,perm,],
                      c(1, 3),
                      cumsum),
                c(2,1,3))
cnam <- dimnames(cprobs)[["cause"]]
for(i in 2:length(cnam))
    dimnames(cprobs)[["cause"]][i] <- paste(cnam[1:i],
                                            collapse = "+")
cpr <- aperm(apply(cprobs, 1:2, mnqt, alpha), c(2,3,1))
cpr[,-dim(cpr)[2],]
}

#----------------------------------------------------------------------
# 3. sojourn times in states
sim2ci.Stime <-
function(probs,
           int = attr(probs, "int"),
         alpha = 0.05)
{
sojrn <- apply(probs,
                 2:3,
               function(x) cumsum(midpoint(x))) * int
res <- aperm(apply(sojrn, 1:2, mnqt, alpha), c(2,3,1))
res <- res[c(1, 1:dim(res)[1]), , ]
res[1,,] <- 0
      dimnames(res) [1] <-       dimnames(probs) [1]
names(dimnames(res))[1] <- names(dimnames(probs))[1]
res
}

fgrep <- function( pattern, x, ... )        x [grep( pattern,        x , ... )]
ngrep <- function( pattern, x, ... )  names(x)[grep( pattern,  names(x), ... )]
lgrep <- function( pattern, x, ... ) levels(x)[grep( pattern, levels(x), ... )]

# -------------------------------------------------------------
# extension of ns() from splines, to allow for clamping.

ns.ld <- function(x, df = NULL, knots = NULL, intercept = FALSE,
                  Boundary.knots = range(x), fixsl=c(FALSE,FALSE) )
{
  nx <- names(x)
  x <- as.vector(x)
  nax <- is.na(x)
  if(nas <- any(nax))    x <- x[!nax]


  if(!missing(Boundary.knots)) {
      if( length(Boundary.knots)!=2 )
          stop( "Argument 'Boundary.knots' must, when provided, have length 2" )
    Boundary.knots <- sort(Boundary.knots)
    outside <- (ol <- x < Boundary.knots[1L]) | (or <- x > Boundary.knots[2L])
  }

  else outside <- FALSE

  if(!missing(df) && missing(knots)) {
    ## df = number(interior knots) + 1 + intercept
    nIknots <- df - 1 - intercept
    if(nIknots < 0) {
      nIknots <- 0
      warning("'df' was too small; have used ", 1 + intercept)
    }

    knots <- if(nIknots > 0) {
      knots <- seq.int(0, 1,
                       length.out = nIknots + 2L)[-c(1L, nIknots + 2L)]
      stats::quantile(x[!outside], knots)
    } ## else  NULL

  } else nIknots <- length(knots)
  Aknots <- sort(c(rep(Boundary.knots, 4L), knots))

  if(any(outside)) {
    basis <- array(0, c(length(x), nIknots + 4L))

    if(any(ol)) {
      k.pivot <- Boundary.knots[1L]
      xl <- cbind(1, x[ol] - k.pivot)
      tt <- splines::spline.des(Aknots, rep(k.pivot, 2L), 4, c(0, 1))$design
      basis[ol,  ] <- xl %*% tt
    }

    if(any(or)) {
      k.pivot <- Boundary.knots[2L]
      xr <- cbind(1, x[or] - k.pivot)
      tt <- splines::spline.des(Aknots, rep(k.pivot, 2L), 4, c(0, 1))$design
      basis[or,  ] <- xr %*% tt
    }


    if(any(inside <- !outside))
      basis[inside,  ] <- splines::spline.des(Aknots, x[inside], 4)$design
  }

  else basis <- splines::spline.des(Aknots, x, 4)$design

# Fix the clamping
if( !is.logical(fixsl) ) warning( "fixsl elements must be of mode logical" )
# Only the 4th parameter affected, should be either 1 or 2 in the two positions
const <- splines::spline.des( Aknots, Boundary.knots, 4, c(2-fixsl[1],2-fixsl[2]) )$design

  if(!intercept) {
    const <- const[, -1 , drop = FALSE]
    basis <- basis[, -1 , drop = FALSE]
  }

  qr.const <- qr(t(const))
  basis <- as.matrix((t(qr.qty(qr.const, t(basis))))[,  - (1L:2L), drop = FALSE])
  n.col <- ncol(basis)

  if(nas) {
    nmat <- matrix(NA, length(nax), n.col)
    nmat[!nax, ] <- basis
    basis <- nmat
  }

  dimnames(basis) <- list(nx, 1L:n.col)
  a <- list(degree = 3, knots = if(is.null(knots)) numeric(0) else knots,
            Boundary.knots = Boundary.knots, intercept = intercept)
  attributes(basis) <- c(attributes(basis), a)
  class(basis) <- c("cns", "basis", "matrix")
  basis
}

#-------------------------------------------------------------------------------
# An extension of ns that automatically  takes the smallest and largest knots
# as boundary knots without further ado, but also allows centering
# around a reference and detrending by means of projection, as well as
# clamping.

Ns <- function( x, ref = NULL,
                    df = NULL,
                 knots = NULL,
             intercept = FALSE,
        Boundary.knots = NULL,
                 fixsl = c(FALSE,FALSE),
               detrend = FALSE )
{
    ## Check sensibility of arguments
    if( !is.null(ref) ) {
        if( !is.vector(ref) )
            stop( "Argument 'ref' must be a scalar, but it is a ", class(ref), "." )
        if( is.vector(ref) & length(ref)>1 )
            stop( "Argument 'ref' must be a scalar, but has length ", length(ref), "." )
        if( intercept ) {
            warning( "ref= specified, hence intercept=TRUE is ignored")
            intercept <- FALSE
        }
    }
    ## Detrending required?
    if( any(detrend>0) ) { # covers both logical and vector
        if( any(detrend<0) )
            stop( "Some elements of weight are <0, e.g. no",
                 (ww <- which(detrend<0))[1:min(5,length(ww))], "." )
        if( !(length(detrend) %in% c(1,length(x))) ) {
            warning( "Weights in inner product diagonal matrix set to 1")
            weight <- rep(1,length(x))
        }
        else weight <- if( is.numeric(detrend) ) detrend else rep(1,length(x))
        detrend <- TRUE
    }
    if( detrend & intercept ) {
        warning( "detrend= specified, hence intercept=TRUE is ignored")
        intercept <- FALSE
    }
    if( detrend & any(fixsl) ) {
        warning( "detrend= specified, hence fixsl argument is ignored")
        fixsl=c(FALSE,FALSE)
    }
    ## Here is the specification of the spline basis

    ## knots= specified
    if( !is.null( knots ) )
        {
        if( is.null( Boundary.knots ) )
            {
            if( !is.null( knots ) )
                {
                knots <- sort( unique( knots ) )
                ok <- c(1,length(knots))
                Boundary.knots <- knots[ok]
                knots <- knots[-ok]
                }
            }
        MM <- ns.ld( x, knots = knots,
               Boundary.knots = Boundary.knots,
                    intercept = (intercept & is.null(ref)),
                        fixsl = fixsl )
        if( !is.null(df) ) cat("NOTE: Both knots= and df= specified, df ignored")
        }
    ## df= specified
    if( is.null( knots ) & !is.null( df ) )
        MM <- ns.ld( x, df = df,
                 intercept = (intercept & is.null(ref)),
                     fixsl = fixsl )

  ## Reference point specified ?
  if( !is.null(ref) )
  {
    MM <- MM - ns.ld( rep(ref,length(x)),
                      knots = attr(MM,"knots"),
             Boundary.knots = attr(MM,"Boundary.knots"),
                      fixsl = fixsl )
  }
  ## Detrending required ?
  if( detrend )
  {
    DD <- detrend( MM, x, weight=weight )
    ## NOTE: detrend does not preserve attributes
    for( aa in c("degree","knots","Boundary.knots","intercept","class") )
       attr( DD, aa ) <- attr( MM, aa )
    attr( DD, "detrend" ) <- TRUE
    attr( DD, "proj.wt" ) <- weight
    MM <- DD
  }
  return( MM )
}

######################################################################
### estimation method
LCa.fit <-
function( data, A, P, D, Y,
         model = "APa",    # or one of "ACa", "APaC", "APCa" or "APaCa" 
         a.ref,            # age reference for the interactions
        pi.ref = a.ref,    # age reference for the period interaction
        ci.ref = a.ref,    # age reference for the cohort interaction
         p.ref,            # period reference for the intercation
         c.ref,            # cohort reference for the interactions
          npar = c(a = 6,  # no. knots for main age-effect
                   p = 6,  # no. knots for peroid-effect
                   c = 6,  # no. knots for cohort-effect
                  pi = 6,  # no. knots for age in the period interaction
                  ci = 6), # no. knots for age in the cohort interaction
            VC = TRUE,     # numerical calculation of the Hessia?
         alpha = 0.05,     # 1 minus confidence level
           eps = 1e-6,     # convergence criterion
         maxit = 100,      # max. no iterations
         quiet = TRUE )    # cut the crap
{
# "model" must have values in c(APa/ACa/APaC/APCa/APaCa)?
if( !(model %in% c("APa","ACa","APaC","APCa","APaCa")) )
  stop( '"model" must be one of "APa","ACa","APaC","APCa","APaCa", but is', model,'\n' )

# Which main effects and interactions are in the model
 intP <- as.logical(length(grep("Pa",model)))
 intC <- as.logical(length(grep("Ca",model)))
mainP <- as.logical(length(grep("P" ,model))) # Also includes the age-period interaction
mainC <- as.logical(length(grep("C" ,model))) # Also includes the age-cohort product

# if a dataframe is supplied, fish out data and put in the function's environment
if( !missing(data) )
  {
  if (length(match(c("A", "P", "D", "Y"), names(data))) != 4)
  stop("Data frame ", deparse(substitute(data)),
       " has columns:\n", names(data),
       "\nmust have variables:\n", "A (age), P (period), D (cases) and Y (person-time)")
  data <- data[,c("A","P","D","Y")]
  data <- data[complete.cases(data),]
  A <- data$A
  P <- data$P
  D <- data$D
  Y <- data$Y
  } else { # if single vectors supplied, check they are all there
  nm <- c(missing(A),
          missing(P),
          missing(D),
          missing(Y))
  if (any(nm))
      stop("Variable", if (sum(nm) > 1)
          "s", paste(c(" A", " P", " D", " Y")[nm], collapse = ","),
          " missing from input")
  # and that they have the same length
  if( diff(range( lv <- c( length(A),
                           length(P),
                           length(D),
                           length(Y) ) )) != 0 )
      stop( "\nLengths of variables (", paste(paste(names(lv),
            lv, sep = ":"), collapse = ", "), ") are not the same." )
  } # end of data acquisition

# code-simplifier for knot calculation
eqqnt <- function(n) round( (1:n-0.5)/n, 2 )
# Define knots - we compute also the knots not needed
if( is.list(npar) ) {
  # Check if npar is a *named* list
  if( is.null(names(npar)) ) stop( "If npar= is a list, it must be a *named* list.\n" )
  # Fill in a bogus 1 if some of the items are missing
  for(ee in c('a.kn','p.kn','c.kn','pi.kn','ci.kn') ) if(is.null(npar[[ee]])) npar[[ee]] <- 1
    a.kn <- if( length(npar$a )>1 ) npar$a  else quantile( rep(  A,D), probs=eqqnt(npar$a ) )
    p.kn <- if( length(npar$p )>1 ) npar$p  else quantile( rep(P  ,D), probs=eqqnt(npar$p ) ) 
    c.kn <- if( length(npar$c )>1 ) npar$c  else quantile( rep(P-A,D), probs=eqqnt(npar$c ) )
   pi.kn <- if( length(npar$pi)>1 ) npar$pi else quantile( rep(  A,D), probs=eqqnt(npar$pi) )
   ci.kn <- if( length(npar$ci)>1 ) npar$ci else quantile( rep(  A,D), probs=eqqnt(npar$ci) )
    }
  else { # if npar is too short fill it up
  npar <- rep( npar, 5 )[1:5]
  # if not named, name it and notify
  if( is.null(names(npar)) ) names(npar) <- c("a","p","c","pi","ci")
    a.kn <- quantile( rep(  A,D), probs=eqqnt(npar["a"] ) )
    p.kn <- quantile( rep(P  ,D), probs=eqqnt(npar["p"] ) )
    c.kn <- quantile( rep(P-A,D), probs=eqqnt(npar["c"] ) )
   pi.kn <- quantile( rep(  A,D), probs=eqqnt(npar["pi"]) )
   ci.kn <- quantile( rep(  A,D), probs=eqqnt(npar["ci"]) )
  }
    
# Reference points
if( missing( p.ref) )  p.ref <- median( rep(P  ,D) )
if( missing( c.ref) )  c.ref <- median( rep(P-A,D) )
if( missing(pi.ref) ) pi.ref <- median( rep(  A,D) )    
if( missing(ci.ref) ) ci.ref <- median( rep(  A,D) )    

############################################################################
# Here starts the actual modelling
commence <- Sys.time()
    
# Matrices to extract the age-interaction terms at reference points
Ap <- Ns( rep(pi.ref,length(A)), knots=pi.kn, intercept=TRUE )
Ac <- Ns( rep(ci.ref,length(A)), knots=ci.kn, intercept=TRUE )

# Current age-effects (in the iteration these will be term predictions)
ba <- matrix( 1, length(A), 2 ) # cbind( rep(1,length(A)), 1 )
      
# set to 0 if term is not in model at all
if( !mainP ) ba[,1] <- 0
if( !mainC ) ba[,2] <- 0
# Main effects model with (at least one) age-interaction
# --- at this stage it is either 0 or 1
mat <- glm( D ~ -1 + Ns(   A, knots=a.kn, intercept=TRUE ) + 
                     Ns( P  , knots=p.kn, ref=p.ref):ba[,1] + 
                     Ns( P-A, knots=c.kn, ref=c.ref):ba[,2],
            offset = log(Y),
            family = poisson )
oldmb <- oldmat <- mat$deviance
    
# Terms prediction --- three terms here.
# No need to divide by the ba at this point, it is eiter 1 or 0
pat <- predict( mat, type="terms" )

# iteration counter and continuation indicator
nit <- 0    
one.more <- TRUE
    
# For simple formatting of the iteration output
fC <- function(x,d) formatC(x,format="f",digits=d)
    
# now, iterate till convergence
while( one.more )
  {  
nit <- nit+1

# The estimated terms from the modeling of the APC-effects to be used
# as offsets
        Aoff <- pat[,1]
Pint <- Poff <- pat[,2]
Cint <- Coff <- pat[,3]
# P or C terms with main effects should be either in interaction or
# offset, so one of these should always be 0
if( intP ) Poff <- Poff*0 else Pint <- Pint*0
if( intC ) Coff <- Coff*0 else Cint <- Cint*0
# Iteration of the age-components of the interaction
mb <- glm( D ~ -1 + Ns( A, knots=pi.kn, intercept=TRUE ):Pint +
                    Ns( A, knots=ci.kn, intercept=TRUE ):Cint,
           offset = Aoff + Poff + Coff + log(Y),
           family = poisson )

# Get the age-interaction terms only, and if one is not needed set to 0
ba <- predict( mb, type="terms" ) /
                 cbind(Pint,Cint) /
      cbind( ci.lin( mb, subset="pi.kn", ctr.mat=Ap)[,1],  # These are the values at the reference
             ci.lin( mb, subset="ci.kn", ctr.mat=Ac)[,1] ) # point for A; we want the RRs as effects 
ba[is.na(ba)] <- 0

# If no interaction only main should be fitted; if no main effect, set to 0 using mainP/C
if( !intP ) ba[,1] <- rep(1,length(A)) * mainP
if( !intC ) ba[,2] <- rep(1,length(A)) * mainC
# apc model with assumed known interactions with age    
mat <- glm( D ~ -1 + Ns(   A, knots=a.kn, intercept=TRUE ) +
                     Ns( P  , knots=p.kn, ref=p.ref):ba[,1] + 
                     Ns( P-A, knots=c.kn, ref=c.ref):ba[,2],
            offset = log(Y),
            family = poisson )

# extract age and period terms - rmoving the interactions
pat <- predict( mat, type="terms" ) / cbind( 1, ba )
pat[is.na(pat)] <- 0

# convergence? Check both that the two models give the same deviance
# and that the chnage in each is small
newmat <- mat$deviance
newmb  <-  mb$deviance
conv <- ( reldif <- max( abs(newmat-newmb )/(newmat+newmb)/2,
                         abs(oldmat-newmat)/ newmat,
                         abs(oldmb -newmb )/        newmb) ) < eps
one.more <- ( !conv & ( nit < maxit ) )
oldmat <- newmat
oldmb  <- newmb
if( !quiet & nit==1 )
    cat( "    Deviances: model(AT) model(A) Rel. diff.\n" )
if( !quiet ) cat( "Iteration", formatC( nit, width=3, flag=" "), "",
                  fC(mat$deviance,3),
                  fC( mb$deviance,3),
                  fC( reldif, 7 ), "\n" )
  } # end of iteration loop

# Deviance and d.f - there is a "+1" because the intercept is in both models
# but not explicit, (both models fitted with "-1"), hence the df.null
# is the total no. observations 
dev <- mb$deviance
df  <- mat$df.null - ( mb$df.null- mb$df.res # no. parms in mb
                    + mat$df.null-mat$df.res # no. parms in mat
                    - 1 )                    # common intercept
if(  conv ) cat( "LCa.fit convergence in ", nit,
                " iterations, deviance:", dev, "on", df, "d.f.\n")
if( !conv ) cat( "LCa.fit *not* converged in ", nit,
                 " iterations:\ndeviance (AT):", mat$deviance,
                              ", deviance (B):" , mb$deviance, "\n",
                 if( VC ) "...no variance-covariance computed.\n" )
fin <- Sys.time()
if( !quiet ) cat("...using", round(difftime(fin,commence,units="secs"),1), "seconds.\n")
    
# unique values of A, P and C in the dataset for reporting effects
a.pt <- sort(unique(  A))
p.pt <- sort(unique(P  ))
c.pt <- sort(unique(P-A))

# extract effects from final models after convergence
ax <- ci.exp( mat, subset= "a.kn", ctr.mat=Ns(a.pt,knots= a.kn,intercept=TRUE ) )
kp <- ci.exp( mat, subset= "p.kn", ctr.mat=Ns(p.pt,knots= p.kn,ref=p.ref) )
kc <- ci.exp( mat, subset= "c.kn", ctr.mat=Ns(c.pt,knots= c.kn,ref=c.ref) )
pi <- ci.exp( mb , subset="pi.kn", ctr.mat=Ns(a.pt,knots=pi.kn,intercept=TRUE), Exp=FALSE )
ci <- ci.exp( mb , subset="ci.kn", ctr.mat=Ns(a.pt,knots=ci.kn,intercept=TRUE), Exp=FALSE )

# Label the estimated effects
rownames( ax ) <-    
rownames( pi ) <-    
rownames( ci ) <- a.pt
rownames( kp ) <- p.pt
rownames( kc ) <- c.pt

# do we bother about the correct variance-covariance?
if( VC & conv ) # ...certainly not without convergence
{
commence <- Sys.time()
if( !quiet ) cat("...computing Hessian by numerical differentiation...\n")

# the number of parameters for each of the 5 effects
na  <- length( grep(  "a.kn", names(coef(mat)) ) )
np  <- length( grep(  "p.kn", names(coef(mat)) ) )
nc  <- length( grep(  "c.kn", names(coef(mat)) ) )
npi <- length( grep( "pi.kn", names(coef(mb )) ) )
nci <- length( grep( "ci.kn", names(coef(mb )) ) )

# get only the parameters for effects that are non-zero (the others
# are in the models but they are 0)
ml.cf <- c( coef(mat)[c(rep(TRUE,na), 
                        rep(mainP,np),
                        rep(mainC,nc))],
             coef(mb)[c(rep(intP,npi),
                        rep(intC,nci))] )
# and some more snappy names for the parameters: first all names
all.nam <- c( paste("ax",1:na,sep=""),
              paste("kp",1:np,sep=""),
              paste("kc",1:nc,sep=""),
              paste("pi",1:npi,sep=""),
              paste("ci",1:nci,sep="") )
# ...then those actually present in the model
names( ml.cf ) <- all.nam[c(rep( TRUE,na),
                            rep(mainP,np),
                            rep(mainC,nc),
                            rep(intP,npi),
                            rep(intC,nci))]

# We need the variance-covariance of the estimates as the 2nd
# derivative of the log-likelihood, D*log(lambda) - lambda*Y,
# or for eta=log(lambda), D*eta - exp(eta)*Y,
# assuming the sequence of parameters is ax, kp, kc, pi, ci
# (first A, P, C from model mat, then Pa, Ca from model mb)
# Note that we cannot simplify this calculation because the model is
# non-linear in pi,kp resp. ci,kc

# Matrices to use in calculation of the terms of the model for each parms
MA  <- Ns(   A, knots= a.kn, intercept=TRUE )
Mp  <- Ns( P  , knots= p.kn, ref=p.ref )     
Mc  <- Ns( P-A, knots= c.kn, ref=c.ref )     
Mpi <- Ns(   A, knots=pi.kn, intercept=TRUE )
Mci <- Ns(   A, knots=ci.kn, intercept=TRUE )

# Computing the log-likelihood for any set of parameters
llik <-
function( parms )
{
              ax <- MA  %*% parms[   1:na]  ; nn <- na
if( mainP ) { kp <- Mp  %*% parms[nn+1:np]  ; nn <- nn+np } else kp = rep(0,length(ax))
if( mainC ) { kc <- Mc  %*% parms[nn+1:nc]  ; nn <- nn+nc } else kc = rep(0,length(ax))
if( intP  ) { pi <- Mpi %*% parms[nn+1:npi] ; nn <- nn+npi} else pi = rep(1,length(ax))
if( intC  ) { ci <- Mci %*% parms[nn+1:nci]               } else ci = rep(1,length(ax))
eta <- ax + pi*kp + ci*kc
sum( D*eta - exp(eta)*Y )
}

# Numerical calculation of the Hessian for llik
ivar <- -numDeriv::hessian( llik, ml.cf )

# Sometimes not quite positive definite, fix that after inverting the Hessian
vcov <- Matrix::nearPD( solve( ivar ) )
vcov <- as.matrix( vcov$mat )
fin <- Sys.time()
if( !quiet ) cat("...done - in", round(difftime(fin,commence,units="secs"),1), "seconds.\n")
    
# Since we now have the variances of the parameters for each of the
# effects we can compute corrected c.i.s for the effects
se.eff <-
function( sub, cM, Alpha=alpha )
{
wh <- grep( sub, names(ml.cf) )
res <- cbind(             cM %*% ml.cf[wh],
              sqrt( diag( cM %*% vcov[wh,wh] %*% t(cM) ) ) ) %*% ci.mat(alpha=Alpha)
colnames(res)[1] <- paste( "Joint", colnames(res)[1] )
res
}

# Append the corrected c.i.s to the effect objects
            ax <- cbind( ax, exp( se.eff( "ax", Ns(a.pt,knots= a.kn,intercept=TRUE) ) ) )
if( mainP ) kp <- cbind( kp, exp( se.eff( "kp", Ns(p.pt,knots= p.kn,ref=p.ref) ) ) )
if( mainC ) kc <- cbind( kc, exp( se.eff( "kc", Ns(c.pt,knots= c.kn,ref=c.ref) ) ) )
if( intP  ) pi <- cbind( pi,      se.eff( "pi", Ns(a.pt,knots=pi.kn,intercept=TRUE) ) )
if( intC  ) ci <- cbind( ci,      se.eff( "ci", Ns(a.pt,knots=ci.kn,intercept=TRUE) ) )
}

# Collect refs and knots in lists
klist <- list( a.kn=a.kn, pi.kn=pi.kn, p.kn=p.kn, ci.kn=ci.kn, c.kn=c.kn )
rlist <- list( pi.ref=pi.ref, p.ref=p.ref, ci.ref=ci.ref, c.ref=c.ref )
    
# Finally output object
res <- list( model = model,
                ax = ax, 
                pi = if( intP  ) pi else NULL, 
                kp = if( mainP ) kp else NULL, 
                ci = if( intC  ) ci else NULL, 
                kc = if( mainC ) kc else NULL, 
            mod.at = mat, 
            mod.b  = mb,
              coef = if( VC & conv ) ml.cf else NULL,
              vcov = if( VC & conv ) vcov  else NULL,
             knots = klist,
              refs = rlist,  
          deviance = dev,
       df.residual = df,
              iter = nit )
# Remove redundant stuff before returning
res <- res[!sapply(res,is.null)]
class( res ) <- "LCa"
invisible( res ) 
}

######################################################################
### utility to determine the types of terms in a model
model.terms <- 
function( x ) list( intP = as.logical(length(grep("Pa",x$model))),                
                   mainP = as.logical(length(grep("P" ,x$model))),                
                    intC = as.logical(length(grep("Ca",x$model))),                
                   mainC = as.logical(length(grep("C" ,x$model))) )

######################################################################
### print method
print.LCa <-
function( x, ... )
{
# terms in the model
mt <- model.terms( x )
    
# no. of parameters in each term    
npar <- sapply(x$knots,length)-1
npar[1] <- npar[1] + 1
npar <- npar[c(TRUE,mt$intP,mt$mainP,mt$intC,mt$mainC)]
npar <- paste( paste( npar, c(rep(", ",length(npar)-2)," and ",""), sep=""), collapse=rep("") )
cat( paste( x$model, ": Lee-Carter model with natural splines:\n",
     "  log(Rate) = ax(Age)",
     if( mt$mainP ) " + ",
     if(  mt$intP ) "pi(Age)",
     if( mt$mainP ) "kp(Per)",
     if( mt$mainC ) " + ",
     if(  mt$intC ) "ci(Age)",
     if( mt$mainC ) "kc(Coh)", "\nwith ",
     npar, " parameters respectively.\n",
     "Deviance: ", round(x$deviance,3), " on ", x$df, " d.f.\n",
     sep=""),
     if( !("vcov" %in% names(x)) ) "(only conditional c.i.s for effects)\n" )
}

######################################################################
### summary method
summary.LCa <-
function( object, show.est=FALSE, ... )
{
# terms in the model
mt <- model.terms( object )

print( object )

# the number of parameters for each of the 5 effects
na  <- length( grep(  "a.kn", names(coef(object$mod.at)) ) )
np  <- length( grep(  "p.kn", names(coef(object$mod.at)) ) )
nc  <- length( grep(  "c.kn", names(coef(object$mod.at)) ) )
npi <- length( grep( "pi.kn", names(coef(object$mod.b )) ) )
nci <- length( grep( "ci.kn", names(coef(object$mod.b )) ) )

# Show knots used    
cat( "\nKnots used:\n")
for( nm in names(object$knots[c(TRUE,mt$mainP,mt$intP,mt$mainC,mt$intC)]) )
   { cat( nm,"\n" ) ; print(object$knots[[nm]] ) }
cf <- rbind( ci.lin(object$mod.at)[c(rep(TRUE    ,na),   
                                     rep(mt$mainP,np),  
                                     rep(mt$mainC,nc)),1:2],
             ci.lin(object$mod.b )[c(rep(mt$intP ,npi),  
                                     rep(mt$intC ,nci)),1:2] )
if( "vcov" %in% names(object) )
  {  
  cf <- cbind( cf, sqrt( diag(object$vcov) ) )
  colnames( cf )[-1] <- c("cond.se","joint.se")
  }
if( show.est ) print( cf )
invisible( cf )
}

######################################################################
### plot method
plot.LCa <-
function( x, ... )
{
# terms in the model
mt <- model.terms( x )

# A small plot utility to exploit the structure of the effects    
plc <- 
function( x, ... ) matplot( as.numeric( rownames(x) ), x[,ncol(x)-2:0],
                            type="l", lty=1, lwd=c(3,1,1), ... )

plc( x$ax, col="black", xlab="Age", ylab="Age-specific rates", log="y" )
rug( x$knots$a.kn, lwd=2 )

if( mt$intP ){    
plc( x$pi, col="black", xlab="Age", ylab="Relative period log-effect multiplier" )
abline(h=1,v=x$refs$pi.ref)
rug( x$knots$pi, lwd=2 )
}

if( mt$mainP ){
plc( x$kp, col="black", log="y", xlab="Date of follow-up (period)", ylab="Period effect (RR)" )
abline(h=1,v=x$refs$p.ref)
rug( x$knots$kp, lwd=2 )
}

if( mt$intC ){    
plc( x$ci, col="black", xlab="Age", ylab="Relative cohort log-effect multiplier" )
abline(h=1,v=x$refs$ci.ref)
rug( x$knots$ci, lwd=2 )
}

if( mt$mainC ){    
plc( x$kc, col="black", log="y", xlab="Date of birth (cohort)", ylab="Cohort effect (RR)" )
abline(h=1,v=x$refs$c.ref)
rug( x$knots$kc, lwd=2 )
}

}

######################################################################
### predict method
predict.LCa <-
function( object,
         newdata,
           alpha = 0.05,
           level = 1-alpha,
             sim = ( "vcov" %in% names(object) ),
             ... )
{
# What main effects and interactions are in the model    
mt <- model.terms( object )    

# is person-years suppied, otherwise use units as in the model
if( "Y" %in% names(newdata) ) Y <-            newdata$Y else
                              Y <- rep(1,nrow(newdata))

# Matrices to extract effects at newdata rows
 Ma <- Ns(           newdata$A, knots = object$knots$a.kn, intercept = TRUE)
 Mp <- Ns( newdata$P          , knots = object$knots$p.kn, ref=object$refs$p.ref )
 Mc <- Ns( newdata$P-newdata$A, knots = object$knots$c.kn, ref=object$refs$c.ref )
Mpi <- Ns(           newdata$A, knots = object$knots$pi.kn, intercept = TRUE)
Mci <- Ns(           newdata$A, knots = object$knots$ci.kn, intercept = TRUE)

# Default terms values for models without interactions
kp <- kc <- rep( 0, nrow(newdata) )
pi <- ci <- rep( 1, nrow(newdata) )

# P, C and interaction term(s) if included in the model
if( mt$intP ) {
  pi <- ci.lin( object$mod.b , subset="pi.kn", ctr.mat=Mpi )[,1]
  kp <- ci.lin( object$mod.at, subset= "p.kn", ctr.mat=Mp  )[,1]
  }
if( mt$intC ) {
  ci <- ci.lin( object$mod.b , subset="ci.kn", ctr.mat=Mci )[,1]
  kc <- ci.lin( object$mod.at, subset= "c.kn", ctr.mat=Mc  )[,1]
  }

# First fitted values from mod.at
# Note that the model object mod.at always has the same number of
# parameters, for some of the models either period or cohort parameters
# are 0, hence not used.  
pr0 <- ci.exp( object$mod.at, alpha=alpha, ctr.mat=cbind(Ma,Mp*pi,Mc*ci) )

# Then fitted values from mod.b
# But mod.b has an offset beyond log(Y), namely all the APC terms
lp.b <-                ci.lin( object$mod.b , ctr.mat=cbind(Mpi*kp,Mci*kc) )[,1:2]
lp.b[,1] <- lp.b[,1] + ci.lin( object$mod.at, ctr.mat=cbind(Ma,Mp*(!mt$intP),Mc*(!mt$intC)) )[,1]
pr0 <- cbind( pr0, exp( lp.b %*% ci.mat(alpha=alpha) ) )

# label the estimates
colnames( pr0 )[c(1,4)] <- c("at|b Est.","b|at Est.")

# The doings above gives confidence intervals based on the conditional
# models, so if we want proper intervals we should simulate instead,
# using the posterior distribuion of all parameters, albeit under the
# slightly fishy assumption that the joint posterior is normal...
if( sim ) # also renders TRUE if sim is numerical (and not 0)
  {
if( is.logical(sim) & sim ) sim <- 1000
# Check that there is a vcov component of the model
if( !( "vcov" %in% names(object) ) )
    warning(
    "No variance-covariance in LCa object, only conditional c.i.s available.\n",
    "no simulation (parametric bootstrap) is done.\n" )  
else {   
# using the parametric bootstrap based on the parameters and the
# (numerically computed) Hessian
eta <- NArray( list( pt = 1:nrow(pr0),
                     it = 1:sim ) )                      
parms <- MASS::mvrnorm( n = sim, 
                       mu = object$coef,
                    Sigma = object$vcov )
na  <- ncol( Ma  )
np  <- ncol( Mp  )
nc  <- ncol( Mc  )
npi <- ncol( Mpi )
nci <- ncol( Mci )
# Compute the linear predictor in each of the simulated samples
# period and cohort effects if not in the model
kp <- kc <- rep( 0, nrow(newdata) )
pi <- ci <- rep( 1, nrow(newdata) )
for( i in 1:sim ){
                 ax <- Ma  %*% parms[i,   1:na]  ; nn <- na
if( mt$mainP ) { kp <- Mp  %*% parms[i,nn+1:np]  ; nn <- nn+np }
if( mt$mainC ) { kc <- Mc  %*% parms[i,nn+1:nc]  ; nn <- nn+nc }
if( mt$intP  ) { pi <- Mpi %*% parms[i,nn+1:npi] ; nn <- nn+npi}
if( mt$intC  ) { ci <- Mci %*% parms[i,nn+1:nci]               }
eta[,i] <- ax + pi*kp + ci*kc
                  }
# predicted rates with bootstrap confidence limits              
pr.sim <- exp( t( apply( eta, 1, quantile,
                         probs=c(0.5,alpha/2,1-alpha/2), 
                         na.rm=TRUE ) ) )
colnames( pr.sim )[1] <- "Joint est."
return( pr.sim )
  }
}
else return( pr0 )    
}

projection.ip <-
function( X, M, orth = FALSE, weight=rep(1,nrow(X)) )
# Generate the projection of M on span(X) w.r.t the inner
# product <x|y>=sum( x*w*y).
# Avoids computing the entire projection matrix
#   X %*% inverse( X'WX ) %*% (XW)'  by first computing
#         inverse( X'WX ) %*% (XW)'M
# (which is (p x p) %*% (p x n) %*% (n x k), i.e. (p x k) )
# and then premultiplying X (n x p) hence avoiding making
# a n x n matrix underway (note that n is large, p is small).
# Note multiplication by W (diagional matrix) is done by
# vector multiplication using the recycling facility of R.
{
  if( nrow(X) != length(weight) )
      stop( "Dimension of space and length of weights differ!" )
  if( nrow(X) != nrow(M) )
      stop( "Dimension of space and rownumber of model matrix differ!" )
  Pp <- solve( crossprod( X * sqrt(weight) ), t( X * weight ) ) %*% M
  PM <- X %*% Pp
  if (orth) PM <- M - PM
  else PM
}

rem vignettes appear in alphabetical order of filename
rem this script moves .R and .pdf file to inst.doc
rem => no vignette links on CRAN package website 

call rw n01-flup
call rt n01-flup
call bl n01-flup
rem copy n01-flup.R   ..\inst\doc\
rem copy n01-flup.pdf ..\inst\doc\

call rt n02-addLexis
call rw n02-addLexis
call bl n02-addLexis
rem copy n02-addLexis.R   ..\inst\doc\
rem copy n02-addLexis.pdf ..\inst\doc\

call rt n03-crisk
call rw n03-crisk
call bl n03-crisk
rem copy n03-crisk.R   ..\inst\doc\
rem copy n03-crisk.pdf ..\inst\doc\

call rt n04-simLexis
call rw n04-simLexis
call bl n04-simLexis
rem copy n04-simLexis.R   ..\inst\doc\
rem copy n04-simLexis.pdf ..\inst\doc\

call rt n05-yll
call rw n05-yll
call bl n05-yll
rem copy n05-yll.R   ..\inst\doc\
rem copy n05-yll.pdf ..\inst\doc\

rem move to website
call 2gp n01-flup.R   Epi
call 2gp n01-flup.pdf Epi
call 2gp n02-addLexis.R   Epi
call 2gp n02-addLexis.pdf Epi
call 2gp n03-crisk.R   Epi
call 2gp n03-crisk.pdf Epi
call 2gp n04-simLexis.R   Epi
call 2gp n04-simLexis.pdf Epi
call 2gp n05-yll.R   Epi
call 2gp n05-yll.pdf Epi

call klean
rem del *.pdf

move aaflup.R   ..\inst\doc
move aaflup.pdf ..\inst\doc
move addLexis.R   ..\inst\doc
move addLexis.pdf ..\inst\doc
move crisk.R   ..\inst\doc
move crisk.pdf ..\inst\doc
move simLexis.R   ..\inst\doc
move simLexis.pdf ..\inst\doc
move yll.R   ..\inst\doc
move yll.pdf ..\inst\doc

%\VignetteIndexEntry{Simulation of multistate models with multiple timescales: simLexis}
\SweaveOpts{results=verbatim,keep.source=TRUE,include=FALSE,eps=FALSE}
\documentclass[a4paper,dvipsnames,twoside,12pt]{report}

\newcommand{\Title}{Simulation of\\ multistate models with\\ multiple
  timescales:\\ \texttt{simLexis} in the \texttt{Epi} package}
\newcommand{\Tit}{SimMult}
\newcommand{\Version}{Version 2.7}
\newcommand{\Dates}{February 2026}
\newcommand{\Where}{SDCC}
\newcommand{\Homepage}{\url{https://BendixCarstensen.com/Epi/04simLexis.pdf}}
\newcommand{\Faculty}{\begin{tabular}{rl}
Bendix Carstensen
  & Steno Diabetes Center Copenhagen, Gentofte, Denmark\\
  & {\small \& Department of Biostatistics,
               University of Copenhagen} \\
  & \texttt{b@bxc.dk}\\
  & \url{https://BendixCarstensen.com/} \\[1em]
                      \end{tabular}}

\input{topreport}
\renewcommand{\rwpre}{./04simLexis}

\chapter{Using \texttt{simLexis}}

\section{Technicalities}
First we set some graphics parameters for convenience and load the
packages needed:
<<>>=
options(width = 90,
        show.signif.stars = FALSE,
        SweaveHooks=list(fig = function()
                         par(mar = c(3, 3, 1, 1),
                             mgp = c(3, 1, 0) / 1.6,
                             las = 1,
                            lend = "butt",
                             bty = "n")))
library(Epi)
library(popEpi)
library(survival)
clear()
@
% must be after clear() because 'anfang' is used at the end
<<echo = FALSE, results = hide>>=
anfang <- Sys.time()
cat("Start time:", format(anfang, "%F, %T"), "\n")
@ %
<<echo = FALSE>>=
vers <-
data.frame(R = substr(R.version.string, 11, 15),
         Epi = as.character(packageVersion(   "Epi")),
      popEpi = as.character(packageVersion("popEpi")))
names(vers) <- paste(" ", names(vers))
print(vers, row.names = FALSE)
@ %

\section{Introduction}

This vignette explains the machinery behind simulation of life
histories through multistate models implemented in
\texttt{simLexis}. In \texttt{simLexis} transition rates are allowed
to depend on multiple time scales, including timescales defined as
time since entry to a particular state (duration). This therefore also
covers the case where time \emph{at} entry into a state is an
explanatory variable for the rates, since time at entry is merely
(current) time minus duration. Thus, the set-up here goes beyond
Markov- and semi-Markov-models, and brings simulation based estimation
of state-occupancy probabilities into the realm of realistic
multistate models.

The basic idea is to simulate a new \texttt{Lexis} object
\cite{Plummer.2011,Carstensen.2011a} as defined in the \texttt{Epi}
package for \R, based on 1) a multistate model defined by its states
and the transition rates between them and 2) an initial population of
individuals.

Thus the output will be a \texttt{Lexis} object describing the
transitions of a predefined set of persons through a multistate
model. Therefore, if persons are defined to be identical at start,
then calculation of the probability of being in a particular state at
a given time boils down to a simple enumeration of the fraction of the
persons in the particular state at the given time. Bar of course the
(binomial) simulation error, but this can be brought down by
simulation a sufficiently large number of persons.

An observed \texttt{Lexis} object with follow-up of persons through a
number of states will normally be the basis for estimation of
transition rates between states, and thus will contain all information
about covariates determining the occurrence rates, in particular the
\emph{timescales} \cite{Iacobelli.2013}. Hence, the natural input to
simulation from an estimated multistate model will typically be an
object of the same structure as the originally observed. Since
transitions and times are what is simulated, any values of
\texttt{lex.Xst} and \texttt{lex.dur} in the input object will of
course be ignored.

This first chapter of this vignette shows by an example how to use the
function \texttt{simLexis} and display the results. The second
chapter discusses in more detail how the simulation machinery is
implemented and is not needed for the practical use of \texttt{simLexis}.

\section{\texttt{simLexis} in practice}

This section is largely a commented walk-trough of the example from
the help-page of \texttt{simLexis}, with a larger number of simulated
persons in order to minimize the pure simulation variation.

When we want to simulate transition times through a multistate model
where transition rates may depend on time since entry to the current
or a previous state, it is essential that we have a machinery to keep
track of the transition time on \emph{all} time scales, as well as a
mechanism that can initiate a new time scale to 0 when a transition
occurs to a state where we shall use time since entry as determinant
of exit rates from that state. This is provided by \texttt{simLexis}.

\subsection{Input for the simulation}

Input for simulation of a single trajectory through a multistate model
requires a representation of the \emph{current status} of a person;
the starting conditions. The object that we supply to the simulation
function must contain information about all covariates and all
timescales upon which transitions depend, and in particular which
one(s) of the timescales that are defined as time since entry into a
particular state. Hence, starting conditions should be represented as
a \texttt{Lexis} object (where \texttt{lex.dur} and \texttt{lex.Xst}
are ignored, since there is no follow-up yet), where the time scale
information is in the attributes \texttt{time.scales} and
\texttt{time.since} respectively.

Note that \texttt{time.scales} attribute is a vector of names of
variables in the \texttt{Lexis} object, so all of these variables
should be present even if they are not used in the models for the
transitions, and they should be set to 0; if they are not in the
initial dataset, \texttt{simLexis} will crash, if they are
\texttt{NA}, the \texttt{simLexis} will produce an object with 0 rows.

Thus there are two main arguments to a function to simulate from a
multistate model:
\begin{enumerate}
\item A \texttt{Lexis} object representing the initial states and
  covariates of the population to be simulated. This has to have the
  same structure as the original \texttt{Lexis} object representing
  the multistate model from which transition rates in the model were
  estimated. As noted above, the values for \texttt{lex.Xst} and
  \texttt{lex.dur} are not required (since these are the quantities
  that will be simulated).
\item A transition object, representing the transition intensities
  between states, which should be a list of lists of intensity
  representations. As an intensity representation we mean a function
  that for a given \texttt{Lexis} object can be used to produce
  estimates of the transition intensities at a set of supplied time
  points.

  The names of the elements of the transition object (which are lists)
  will be names of the \emph{transient} states, that is the states
  \emph{from} which a transition can occur. The names of the elements
  of each of these lists are the names of states \emph{to} which
  transitions can occur (which may be either transient or absorbing
  states).

  Hence, if the transition object is called \texttt{Tr} then
  \verb+TR$A$B+ (or \verb+Tr[["A"]][["B"]]+) will represent the
  transition intensity from state \texttt{A} to the state \texttt{B}.

  The entries in the transition object can be either \texttt{glm}
  objects (either with \texttt{poisson} or \texttt{poisreg} family),
  representing Poisson models for the transitions, \texttt{coxph}
  objects representing an intensity model along one time scale, or
  simply a function that takes a \texttt{Lexis} object as input and
  returns an estimated intensity for each row.

\end{enumerate}

In addition to these two input items, there will be a couple of tuning
parameters.

The output of the function will simply be a \texttt{Lexis} object with
simulated transitions between states. This will be the basis for
deriving sensible statistics from the \texttt{Lexis} object --- see
next section.

\section{Setting up a \texttt{Lexis} object}

As an example we will use the \texttt{DMlate} dataset from the
\texttt{Epi} package; it is a dataset simulated to resemble a random
sample of 10,000 patients from the Danish National Diabetes Register.

We start by loading the \texttt{Epi} package:
<<start>>=
options( width=90 )
library( Epi )
print( sessionInfo(), l=F )
@ %
First we load the diabetes data and set up a simple illness-death
model:
<<Lexis>>=
data(DMlate)
dml <- Lexis( entry = list(Per=dodm, Age=dodm-dobth, DMdur=0 ),
               exit = list(Per=dox),
        exit.status = factor(!is.na(dodth),labels=c("DM","Dead")),
               data = DMlate )
@ %
This is just data for a simple survival model with states \texttt{DM} and
\texttt{Dead}. Now we cut the follow-up at insulin start, which for the
majority of patients (T2D) is a clinical indicator of deterioration of
disease regulation. We therefore also introduce a new timescale, and
split the non-precursor states, so that we can address the question of
ever having been on insulin:
<<cut>>=
dmi <- cutLexis( dml, cut = dml$doins,
                      pre = "DM",
                new.state = "Ins",
                new.scale = "t.Ins",
             split.states = TRUE )
summary( dmi, timeScales=T )
@ % $
Note that we show the time scales in the \texttt{Lexis} object, and
that it is indicated that the time scale \texttt{t.Ins} is defined as
time since entry into stat state \texttt{Ins.}

We can show how many person-years we have and show the number of
transitions and transition rates (per 1000), using the
\texttt{boxes.Lexis} function to display the states and the number of
transitions between them:
<<boxes,fig=TRUE,height=5,width=7>>=
boxes( dmi, boxpos = list(x = c(20, 20, 80, 80),
                          y = c(80, 20, 80, 20)),
            scale.R = 1000, show.BE = TRUE )
@ %
\insfig{boxes}{0.8}{Data overview for the \textrm{\tt dmi}
  dataset. Numbers in the boxes are person-years and the number of
  persons who begin, resp. end their follow-up in each state, and
  numbers on the arrows are no. of transitions and rates (transition
  intensities) per 1000 PY.}

\section{Analysis of rates}

In the \texttt{Lexis} object (which is just a data frame) each person
is represented by one record for each transient state occupied,
thus in this case either 1 or 2 records; those who have a recorded
time both without and with insulin have two records.

In order to be able to fit Poisson models with occurrence rates varying
by the different time-scales, we split the follow-up in 3-month intervals
for modeling:
<<split>>=
Si <- splitLexis( dmi, seq(0,20,1/4), "DMdur" )
summary( Si )
print( subset( Si, lex.id==97 )[,1:10], digits=6 )
@ %
Note that when we split the follow-up, each person's follow up now
consists of many records, each with the \emph{current} values of the
timescales at the start of the interval represented by the record. In
the modeling we shall assume that the rates are constant within each
6-month interval, but the \emph{size} of these rates we model as
smooth functions of the time scales (that is the values at the
beginning of each interval).

The approach often used in epidemiology where one parameter is
attached to each interval of time (or age) is not feasible when more
than one time scale is used, because intervals are not classified the
same way on all timescales.

We shall use natural splines (restricted cubic splines) for the
analysis of rates, and hence we must allocate knots for the
splines. This is done for each of the time-scales, and separately for
the transition out of states \texttt{DM} and \texttt{Ins}. For age, we
place the knots so that the number of events is the same between each
pair of knots, but only half of this beyond each of the boundary
knots, whereas for the timescales \texttt{DMdur} and \texttt{tIns}
where we have observation from a well-defined 0, we put knots at 0 and
place the remaining knots so that the number of events is the same
between each pair of knots as well as outside the boundary knots.
<<knots>>=
nk <- 5
( ai.kn <- with( subset(Si,lex.Xst=="Ins" & lex.Cst!=lex.Xst ),
                 quantile( Age+lex.dur  , probs=(1:nk-0.5)/nk ) ) )
( ad.kn <- with( subset(Si,lex.Xst=="Dead"),
                 quantile( Age+lex.dur  , probs=(1:nk-0.5)/nk ) ) )
( di.kn <- with( subset(Si,lex.Xst=="Ins" & lex.Cst!=lex.Xst ),
                 c(0,quantile( DMdur+lex.dur, probs=(1:(nk-1))/nk ) )) )
( dd.kn <- with( subset(Si,lex.Xst=="Dead"),
                 c(0,quantile( DMdur+lex.dur, probs=(1:(nk-1))/nk ) )) )
( ti.kn <- with( subset(Si,lex.Xst=="Dead(Ins)"),
                 c(0,quantile( t.Ins+lex.dur, probs=(1:(nk-1))/nk ) )) )
@ %
Note that when we tease out the event records for transition to
\emph{transient} states (in this case \texttt{Ins}, that is
\verb|lex.Xst=="Ins"|), we should add \verb|lex.Cst!=lex.Xst|, to
include only transition records and avoiding including records of
sojourn time in the transient state.

We then fit Poisson models to transition rates, using the wrapper
\texttt{Ns} from the \texttt{Epi} package to simplify the
specification of the rates:
<<Poisson>>=
library( splines )
DM.Ins <- glm( (lex.Xst=="Ins") ~ Ns( Age  , knots=ai.kn ) +
                                  Ns( DMdur, knots=di.kn ) +
                                  I(Per-2000) + sex,
               family=poisson, offset=log(lex.dur),
               data = subset(Si,lex.Cst=="DM") )
ci.exp( DM.Ins )
class( DM.Ins )
@ %
We can also fit this model with a slightly simpler syntax using the
\texttt{glm.Lexis} function:
<<>>=
DM.Ins <- glm.Lexis( Si, from = "DM", to = "Ins",
                      formula = ~ Ns( Age  , knots=ai.kn ) +
                                  Ns( DMdur, knots=di.kn ) +
                                  I(Per-2000) + sex )
ci.exp( DM.Ins )
class( DM.Ins )
@ %
So we have a slightly simpler syntax, and we get an informative
message of which transition(s) we are modeling. However we do not have
\texttt{update} method for these objects.
<<>>=
DM.Dead <- glm.Lexis( Si, from = "DM", to = "Dead",
                       formula = ~ Ns( Age  , knots=ad.kn ) +
                                   Ns( DMdur, knots=dd.kn ) +
                                   I(Per-2000) + sex )
Ins.Dead <- glm.Lexis( Si, from = "Ins",
                        formula = ~ Ns( Age  , knots=ad.kn ) +
                                    Ns( DMdur, knots=dd.kn ) +
                                    Ns( t.Ins, knots=ti.kn ) +
                                    I(Per-2000) + sex )
@ %
Note the similarity of the code used to fit the three models, is is
mainly redefining the response variable (\texttt{to} state) and the subset
of the data used (\texttt{from} state). Also note that the last model need
no specification of \texttt{to}, the default is to model all
transitions from the \texttt{from} state, and his case there is only
one.

\section{The mortality rates}

This section discusses in some detail how to extract ad display the
mortality rates from the models fitted. But it is not necessary for
understanding how to use \texttt{simLexis} in practice.

\subsection{Proportionality of mortality rates}

Note that we have fitted separate models for the three transitions,
there is no assumption of proportionality between the mortality rates
from \texttt{DM} and \texttt{Ins}.

However, there is nothing that prevents us from testing this
assumption; we can just fit a model for the mortality rates in the
entire data frame \texttt{Si}, and compare the deviance from this with
the sum of the deviances from the separate models using the \texttt{glm.Lexis}
function:
<<prop-haz>>=
All.Dead <- glm.Lexis( Si, to = c("Dead(Ins)","Dead"),
                      formula = ~ Ns( Age  , knots=ad.kn ) +
                                  Ns( DMdur, knots=dd.kn ) +
                                  lex.Cst +
                                  I(Per-2000) + sex )
round( ci.exp( All.Dead ), 3 )
@ %
Incidentally we could have dispensed with the \texttt{to=} argument
too, because the default is to take \texttt{to} to be all absorbing
states in the model.

From the parameter values we would in a simple setting just claim that
start of insulin-treatment was associated with a slightly more than
doubling of mortality.

The model \texttt{All.dead} assumes that the age- and DM-duration
effects on mortality in the \texttt{DM} and \texttt{Ins} states are the same,
and moreover that there is no effect of insulin duration, but merely a
mortality that is larger by a multiplicative constant not depending on
insulin duration. The model \texttt{DM.Dead} has 8 parameters to
describe the dependency on age and DM duration, the model
\texttt{Ins.Dead} has 12 for the same plus the insulin duration (a
natural spline with $k$ knots gives $k-1$ parameters, and we chose
$k=5$ above).

We can compare the fit of the simple proportional hazards model with
the fit of the separate models for the two mortality rates, by adding
up the deviances and d.f. from these:
<<get-dev>>=
what <- c("null.deviance","df.null","deviance","df.residual")
( rD <- unlist(  DM.Dead[what] ) )
( rI <- unlist( Ins.Dead[what] ) )
( rA <- unlist( All.Dead[what] ) )
round( c( dd <- rA-(rI+rD), "pVal"=1-pchisq(dd[3],dd[4]+1) ), 3 )
@ %
Thus we see there is a substantial non-proportionality of mortality
rates from the two states; but a test provides no clue whatsoever to
the particular \emph{shape} of the non-proportionality.

To this end, we shall explore the predicted mortalities under the two
models quantitatively in more detail. Note that the reason that there
is a difference in the null deviances (and a difference of 1 in the
null d.f.) is that the null deviance of \texttt{All.Dead} refer to a
model with a single intercept, that is a model with constant and
\emph{identical} mortality rates from the states \texttt{DM} and \texttt{Ins},
whereas the null models for \texttt{DM.Dead} and \texttt{Ins.Dead}
have constant but \emph{different} mortality rates from the states
\texttt{DM} and \texttt{Ins}.  This is however irrelevant for the comparison of
the \emph{residual} deviances.

\subsection{How the mortality rates look}

If we want to see how the mortality rates are modeled in
\texttt{DM.Dead} and \texttt{Ins.Dead} in relation to
\texttt{All.Dead}, we make a prediction of rates for say men diagnosed
in different ages and going on insulin at different times after
this. So we consider men diagnosed in ages 40, 50, 60 and 70, and who
either never enter insulin treatment or do it 0, 2 or 5 years after
diagnosis of DM.

To this end we create a prediction data frame where we have
observation times from diagnosis and 12 years on (longer would not
make sense as this is the extent of the data).

But we start by setting up an array to hold the predicted mortality
rates, classified by diabetes duration, age at diabetes onset, time of
insulin onset, and of course type of model. What we want to do is to
plot the age-specific mortality rates for persons not on insulin, and
for persons starting insulin at different times after DM. The
mortality curves start at the age where the person gets diabetes and
continues 12 years; for persons on insulin they start at the age when
they initiate insulin.
<<pr-array>>=
pr.rates <- NArray( list( DMdur = seq(0,12,0.1),
                          DMage = 4:7*10,
                          r.Ins = c(NA,0,2,5),
                          model = c("DM/Ins","All"),
                           what = c("rate","lo","hi") ) )
str( pr.rates )
@ %
For convenience the \texttt{Epi} package contains a function that computes
predicted (log-)rates with c.i. --- it is merely a wrapper for
\texttt{predict.glm}.

So we set up the prediction data frame and modify it in loops over
ages at onset and insulin onset in order to collect the predicted
rates in different scenarios:
<<mknd>>=
nd <- data.frame( DMdur = as.numeric( dimnames(pr.rates)[[1]] ),
                lex.Cst = factor( 1, levels=1:4,
                                  labels=levels(Si$lex.Cst) ),
                    sex = factor( 1, levels=1:2, labels=c("M","F")) )
@ % $
Note that we did \emph{not} insert \texttt{lex.dur} as covariate in
the prediction frame. This would be required if we used the
\texttt{poisson} family with the \texttt{glm}, but the wrapper
\texttt{glm.Lexis} uses the \texttt{poisreg} family, so
\texttt{lex.dur} is ignored and predictions always comes in the
(inverse) units of \texttt{lex.dur}. So we get rates per 1 person-year
in the predictions.
<<make-pred>>=
for( ia in dimnames(pr.rates)[[2]] )
   {
dnew <- transform( nd, Age = as.numeric(ia)+DMdur,
                       Per = 1998+DMdur )
pr.rates[,ia,1,"DM/Ins",] <- ci.pred(  DM.Dead, newdata = dnew )
pr.rates[,ia,1,"All"   ,] <- ci.pred( All.Dead, newdata = dnew )
for( ii in dimnames(pr.rates)[[3]][-1] )
   {
dnew = transform( dnew, lex.Cst = factor( 2, levels=1:4,
                                          labels=levels(Si$lex.Cst) ),
                          t.Ins = ifelse( (DMdur-as.numeric(ii)) >= 0,
                                           DMdur-as.numeric(ii), NA ) )
pr.rates[,ia, ii ,"DM/Ins",] <- ci.pred( Ins.Dead, newdata = dnew )
pr.rates[,ia, ii ,"All"   ,] <- ci.pred( All.Dead, newdata = dnew )
    }
    }
@ % $
So for each age at DM onset we make a plot of the mortality as
function of current age both for those with no insulin treatment and
those that start insulin treatment 0, 2 and 5 years after diabetes
diagnosis, thus 4 curves (with c.i.). These curves are replicated with
a different color for the simplified model.
<<mort-int,fig=TRUE,width=8>>=
par( mar=c(3,3,1,1), mgp=c(3,1,0)/1.6, las=1 )
plot( NA, xlim=c(40,82), ylim=c(5,300), bty="n",
      log="y", xlab="Age", ylab="Mortality rate per 1000 PY" )
abline( v=seq(40,80,5), h=outer(1:9,10^(0:2),"*"), col=gray(0.8) )
for( aa in 4:7*10 ) for( ii in 1:4 )
   matshade( aa+as.numeric(dimnames(pr.rates)[[1]]),
             cbind( pr.rates[,paste(aa),ii,"DM/Ins",],
                    pr.rates[,paste(aa),ii,"All"   ,] )*1000,
             type="l", lty=1, lwd=2,
             col=c("red","limegreen") )
@ %
\insfig{mort-int}{0.9}{Estimated mortality rates for male diabetes
  patients with no insulin (lower sets of curves) and insulin (upper
  curves), with DM onset in age 40, 50, 60 and 70. The red curves are
  from the models with separate age effects for persons with and
  without insulin, and a separate effect of insulin duration. The
  green curves are from the model with common age-effects and only a
  time-dependent effect of insulin, assuming no effect of insulin
  duration (the classical time-dependent variable approach). Hence the
  upper green curve is common for any time of insulin inception.}

From figure \ref{fig:mort-int} we see that there is a substantial
insulin-duration effect which is not accommodated by the simple model
with only one time-dependent variable to describe the insulin
effect. Note that the simple model (green curves) for those on insulin
does not depend in insulin duration, and hence the mortality curves
for those on insulin are just parallel to the mortality curves for
those not on insulin, regardless of diabetes duration (or age) at the
time of insulin initiation. This is the proportional hazards
assumption.  Thus the effect of insulin initiation is under-estimated
for short duration of insulin and over-estimated for long duration of
insulin.

This is the major discrepancy between the two models, and
illustrates the importance of being able to accommodate different time
scales, but there is also a declining overall insulin effect by age which
is not accommodated by the proportional hazards approach.

Finally, this plot illustrates an important feature in reporting
models with multiple timescales; all timescales must be represented in
the predicted rates, only reporting the effect of one timescale,
conditional on a fixed value of other timescales is misleading since
all timescales by definition advance at the same pace. For example,
the age-effect for a fixed value of insulin duration really is a
misnomer since it does not correspond to any real person's follow-up,
but to the mortality of persons in different ages but with the same
duration of insulin use.

\section{Input to the \texttt{simLexis} function}

We want to estimate the cumulative probability of being in each of the
4 states, so that we can assess the fraction of diabetes patients that
go on insulin

In order to simulate from the multistate model with the estimated
transition rates, and get the follow-up of a hypothetical cohort, we
must supply \emph{both} the transition rates and the structure of the
model \emph{as well as} the initial cohort status to
\texttt{simLexis}.

\subsection{The transition object}

We first put the models into an object representing the transitions;
note this is a list of lists, the latter having \texttt{glm} objects
as elements:
<<Tr>>=
Tr <- list( "DM" = list( "Ins"       = DM.Ins,
                         "Dead"      = DM.Dead  ),
           "Ins" = list( "Dead(Ins)" = Ins.Dead ) )
@ %
Now we have the description of the rates and of the structure of the
model. The \texttt{Tr} object defines the states and models for all
transitions between them; the object \verb|Tr$A$B| is the model
for the transition intensity from state \texttt{A} to state
\texttt{B}.

\subsection{The initial cohort}

We now define an initial \texttt{Lexis} object of persons with all
relevant covariates defined. Note that we use \texttt{NULL} as row
indicator in the \texttt{Lexis} object we used for modeling; this
conserves the \texttt{time.scale} and \texttt{time.since} attributes
which are needed for the simulation:
<<make-ini>>=
str( ini <- Si[NULL,1:9] )
@ %
We now have an empty \texttt{Lexis} object with attributes reflecting
the timescales in the multistate model we want to simulate from. But
we must enter some data to represent the initial state of the persons
whose follow-up we want to simulate through the model; so fill in data
for one man and one woman:
<<ini-fill>>=
ini[1:2,"lex.id"] <- 1:2
ini[1:2,"lex.Cst"] <- "DM"
ini[1:2,"Per"] <- 1995
ini[1:2,"Age"] <- 60
ini[1:2,"DMdur"] <- 5
ini[1:2,"sex"] <- c("M","F")
ini
@ %
So the persons starts in age 60 in 1995 with 5 years of diabetes
duration. Note that the \texttt{t.Ins} is \texttt{NA}, because this is
a timescale that first comes alive if a transition to \texttt{Ins} is
simulated.

\section{Simulation of the follow-up}

Now we simulate life-courses of a 1000 of each of these persons using
the estimated model. The \texttt{t.range} argument gives the times
range where the integrated intensities (cumulative rates) are
evaluated and where linear interpolation is used when simulating
transition times. Note that this must be given in the same units as
\texttt{lex.dur} in the \texttt{Lexis} object used for fitting the
models for the transitions. It is not a parameter that can be easily
determined from the \texttt{TR} object, hence it must be supplied by
the user.
<<simL>>=
set.seed(52381764)
Nsim <- 500
system.time( simL <- simLexis( Tr,
                              ini,
                          t.range = 12,
                                N = Nsim ) )
@ %
The result is a \texttt{Lexis} object --- a data frame representing
the simulated follow-up of \Sexpr{2*Nsim} persons (\Sexpr{Nsim}
identical men and \Sexpr{Nsim} identical women) according to the rates
we estimated from the original dataset.
<<sum-simL>>=
summary( simL, by="sex" )
@ %

\subsection{Using other models for simulation}

\subsubsection{Proportional hazards Poisson model}

We fitted a proportional mortality model \texttt{All.Dead} (which fitted
worse than the other two), this is a model for \emph{both} the
transition from \texttt{DM} to \texttt{Death} \emph{and} from \texttt{Ins} to
\texttt{Dead(Ins)}, assuming that they are proportional. But it can easily
be used in the simulation set-up, because the state is embedded in the
model via the term \texttt{lex.Cst}, which is updated during the simulation.

Simulation using this instead just requires that we supply a different
transition object:
<<Tr.p-simP>>=
Tr.p <- list( "DM" = list( "Ins"       = DM.Ins,
                           "Dead"      = All.Dead  ),
             "Ins" = list( "Dead(Ins)" = All.Dead ) )
system.time( simP <- simLexis( Tr.p,
                                ini,
                            t.range = 12,
                                  N = Nsim ) )
summary( simP, by="sex" )
@ %

\subsubsection{Proportional hazards Cox model}

A third possibility would be to replace the two-time scale
proportional mortality model by a one-time-scale Cox-model, using
diabetes duration as time scale, and age at diagnosis of DM as (fixed)
covariate:
<<Cox-dur>>=
library( survival )
Cox.Dead <- coxph( Surv( DMdur, DMdur+lex.dur,
                         lex.Xst %in% c("Dead(Ins)","Dead")) ~
                   Ns( Age-DMdur, knots=ad.kn ) +
                   I(lex.Cst=="Ins") +
                   I(Per-2000) + sex,
               data = Si )
round( ci.exp( Cox.Dead ), 3 )
@ %
Note that in order for this model to be usable for simulation, it is
necessary that we use the components of the \texttt{Lexis} object to
specify the survival. Each record in the data frame \texttt{Si}
represents follow up from \texttt{DMdur} to \texttt{DMdur+lex.dur}, so
the model is a Cox model with diabetes duration as underlying timescale
and age at diagnosis, \texttt{Age-DMdur}, as covariate.

Also note that we used \texttt{I(lex.Cst=="Ins")} instead of just
\texttt{lex.Cst}, because \texttt{coxph} assigns design matrix columns
to all levels of \texttt{lex.Cst}, also those not present in data,
which would give \texttt{NA}s among the parameter estimates and
\texttt{NA}s as mortality outcomes.

We see that the effect of insulin and the other covariates are pretty
much the same as in the two-time-scale model. We can simulate from this
model too; there is no restrictions on what type of model can be used
for different transitions
<<TR.c>>=
Tr.c <- list( "DM" = list( "Ins"       = Tr$DM$Ins,
                           "Dead"      = Cox.Dead  ),
             "Ins" = list( "Dead(Ins)" = Cox.Dead ) )
system.time( simC <- simLexis( Tr.c,
                                ini,
                            t.range = 12,
                                  N = Nsim ) )
summary( simC, by="sex" )
@

\section{Reporting the simulation results}

We can now tabulate the number of persons in each state at a
predefined set of times on a given time scale. Note that in order for
this to be sensible, the \texttt{from} argument would normally be
equal to the starting time for the simulated individuals.
<<nState>>=
system.time(
nSt <- nState( subset(simL,sex=="M"),
               at=seq(0,11,0.2), from=1995, time.scale="Per" ) )
nSt[1:10,]
@ %
We see that as time goes by, the 500 men slowly move away from the
starting state (\texttt{DM}).

Based on this table (\texttt{nSt} is a table) we can now compute the
fractions in each state, or, rather more relevant, the cumulative fraction
across the states in some specified order, so that a plot of the
stacked probabilities can be made, using either the default rather
colorful layout, or a more minimalist version (both in figure \ref{fig:pstate0}):
<<pstate0,fig=TRUE,width=9>>=
pM <- pState( nSt, perm=c(1,2,4,3) )
head( pM )
par( mfrow=c(1,2), mar=c(3,3,1,1), mgp=c(3,1,0)/1.6 )
plot( pM )
plot( pM, border="black", col="transparent", lwd=3 )
text( rep(as.numeric(rownames(pM)[nrow(pM)-1]),ncol(pM)),
      pM[nrow(pM),]-diff(c(0,pM[nrow(pM),]))/5,
      colnames( pM ), adj=1 )
box( col="white", lwd=3 )
box()
@ %
\insfig{pstate0}{1.0}{Default layout of the \textrm{\tt plot.pState}
  graph (left), and a version with the state probabilities as lines and
  annotation of states.}

A more useful set-up of the graph would include a more through
annotation and sensible choice of colors, as seen in figure \ref{fig:pstatex}:
<<pstatex,fig=TRUE,width=9>>=
clr <- c("limegreen","orange")
# expand with a lighter version of the two chosen colors
clx <- c( clr, rgb( t( col2rgb( clr[2:1] )*2 + rep(255,3) ) / 3, max=255 ) )
par( mfrow=c(1,2), las=1, mar=c(3,3,4,2), mgp=c(3,1,0)/1.6 )
# Men
plot( pM, col=clx, xlab="Date of FU" )
lines( as.numeric(rownames(pM)), pM[,2], lwd=3 )
mtext( "60 year old male, diagnosed 1990, aged 55", side=3, line=2.5, adj=0, col=gray(0.6) )
mtext( "Survival curve", side=3, line=1.5, adj=0 )
mtext( "DM, no insulin   DM, Insulin", side=3, line=0.5, adj=0, col=clr[2] )
mtext( "DM, no insulin", side=3, line=0.5, adj=0, col=clr[1] )
axis( side=4 )
axis( side=4, at=1:19/20, labels=FALSE )
axis( side=4, at=1:99/100, labels=FALSE, tcl=-0.3 )
# Women
pF <- pState( nState( subset(simL,sex=="F"),
                      at=seq(0,11,0.2),
                      from=1995,
                      time.scale="Per" ),
              perm=c(1,2,4,3) )
plot( pF, col=clx, xlab="Date of FU" )
lines( as.numeric(rownames(pF)), pF[,2], lwd=3 )
mtext( "60 year old female, diagnosed 1990, aged 55", side=3, line=2.5, adj=0, col=gray(0.6) )
mtext( "Survival curve", side=3, line=1.5, adj=0 )
mtext( "DM, no insulin   DM, Insulin", side=3, line=0.5, adj=0, col=clr[2] )
mtext( "DM, no insulin", side=3, line=0.5, adj=0, col=clr[1] )
axis( side=4 )
axis( side=4, at=1:19/20, labels=FALSE )
axis( side=4, at=1:99/100, labels=FALSE, tcl=-0.3 )
@ %
\insfig{pstatex}{1.0}{\textrm{\tt plot.pState} graphs where persons
  ever on insulin are given in orange and persons never on insulin in
  green, and the overall survival (dead over the line) as a black line.}

If we instead wanted to show the results on the age-scale, we would
use age as timescale when constructing the probabilities; otherwise the
code is pretty much the same as before (Figure \ref{fig:pstatey}):
<<pstatey,fig=TRUE,width=9>>=
par( mfrow=c(1,2), las=1, mar=c(3,3,4,2), mgp=c(3,1,0)/1.6 )
# Men
pM <- pState( nState( subset(simL,sex=="M"),
                      at=seq(0,11,0.2),
                      from=60,
                      time.scale="Age" ),
              perm=c(1,2,4,3) )
plot( pM, col=clx, xlab="Age" )
lines( as.numeric(rownames(pM)), pM[,2], lwd=3 )
mtext( "60 year old male, diagnosed 1990, aged 55", side=3, line=2.5, adj=0, col=gray(0.6) )
mtext( "Survival curve", side=3, line=1.5, adj=0 )
mtext( "DM, no insulin   DM, Insulin", side=3, line=0.5, adj=0, col=clr[2] )
mtext( "DM, no insulin", side=3, line=0.5, adj=0, col=clr[1] )
axis( side=4 )
axis( side=4, at=1:19/20, labels=FALSE )
axis( side=4, at=1:19/20, labels=FALSE, tcl=-0.4 )
axis( side=4, at=1:99/100, labels=FALSE, tcl=-0.3 )
# Women
pF <- pState( nState( subset(simL,sex=="F"),
                      at=seq(0,11,0.2),
                      from=60,
                      time.scale="Age" ),
              perm=c(1,2,4,3) )
plot( pF, col=clx, xlab="Age" )
lines( as.numeric(rownames(pF)), pF[,2], lwd=3 )
mtext( "60 year old female, diagnosed 1990, aged 55", side=3, line=2.5, adj=0, col=gray(0.6) )
mtext( "Survival curve", side=3, line=1.5, adj=0 )
mtext( "DM, no insulin   DM, Insulin", side=3, line=0.5, adj=0, col=clr[2] )
mtext( "DM, no insulin", side=3, line=0.5, adj=0, col=clr[1] )
axis( side=4 )
axis( side=4, at=1:9/10, labels=FALSE )
axis( side=4, at=1:19/20, labels=FALSE, tcl=-0.4 )
axis( side=4, at=1:99/100, labels=FALSE, tcl=-0.3 )
@ %
Note the several statements with \texttt{axis(side=4,...}; they are
necessary to get the fine tick-marks in the right hand side of the
plots that you will need in order to read off the probabilities at
2006 (or 71 years).

\insfig{pstatey}{1.0}{\textrm{\tt plot.pState} graphs where persons
  ever on insulin are given in orange and persons never on insulin in
  green, and the overall survival (dead over the line) as a black line.}

\subsection{Comparing predictions from different models}

We have actually fitted different models for the transitions, and we
have simulated Lexis objects from all three approaches, so we can plot
these predictions on top of each other:
<<comp-0,fig=TRUE,width=9>>=
PrM  <- pState( nState( subset(simP,sex=="M"),
                        at=seq(0,11,0.2),
                        from=60,
                        time.scale="Age" ),
                perm=c(1,2,4,3) )
PrF  <- pState( nState( subset(simP,sex=="F"),
                        at=seq(0,11,0.2),
                        from=60,
                        time.scale="Age" ),
                perm=c(1,2,4,3) )
CoxM <- pState( nState( subset(simC,sex=="M"),
                        at=seq(0,11,0.2),
                        from=60,
                        time.scale="Age" ),
                perm=c(1,2,4,3) )
CoxF <- pState( nState( subset(simC,sex=="F"),
                        at=seq(0,11,0.2),
                        from=60,
                        time.scale="Age" ),
                perm=c(1,2,4,3) )

par( mfrow=c(1,2), mar=c(3,3,1,1), mgp=c(3,1,0)/1.6 )
 plot(   pM, border="black", col="transparent", lwd=3 )
lines(  PrM, border="blue" , col="transparent", lwd=3 )
lines( CoxM, border="red"  , col="transparent", lwd=3 )
text( 60.5, 0.05, "M" )
box( lwd=5, col="white" ) ; box( lwd=2, col="black" )

 plot(   pF, border="black", col="transparent", lwd=3 )
lines(  PrF, border="blue" , col="transparent", lwd=3 )
lines( CoxF, border="red"  , col="transparent", lwd=3 )
text( 60.5, 0.05, "F" )
box( lwd=5, col="white" ) ; box( lwd=2, col="black" )
@ %
\insfig{comp-0}{1.0}{Comparison of the simulated state occupancy
  probabilities using separate Poisson models for the mortality rates
  with and without insulin (black) and using proportional hazards
  Poisson models (blue) and Cox-models with diabetes duration as
  timescale and age at diabetes diagnosis as covariate (red).}

From figure \ref{fig:comp-0} it is clear that the two proportional
hazards models (blue and red curves) produce pretty much the same
estimates of the state occupancy probabilities over time, but also that
they relative to the model with separately estimated transition
intensities overestimates the probability of being alive without
insulin and underestimates the probabilities of being dead without
insulin. However both the overall survival, and the fraction of
persons on insulin are quite well in agreement with the more elaborate
model. Thus the proportional hazards models overestimate the relative
mortality of the insulin treated diabetes patients relative to the
non-insulin treated.

Interestingly, we also see a bump in the estimated probabilities from
the Cox-model based model, but this is entirely an artifact that comes
from the estimation method for the baseline hazard of the Cox-model
that lets the (cumulative) hazard have large jumps at event times
where the risk set is small. So also here it shows up that an
assumption of continuous underlying hazards leads to more credible
estimates.

\chapter{Simulation of transitions in multistate models}

\section{Theory}

Suppose that the rate functions for the transitions out of the current
state to, say, 3 different states are $\lambda_1$, $\lambda_2$ and
$\lambda_3$, and the corresponding cumulative rates are $\Lambda_1$,
$\Lambda_2$ and $\Lambda_3$, and we want to simulate an exit time and
an exit state (that is either 1, 2 or 3). This can be done in two
slightly different ways:
\begin{enumerate}
\item First time, then state:
  \begin{enumerate}
  \item Compute the survival function, $S(t) =
    \exp\bigl(-\Lambda_1(t)-\Lambda_2(t)-\Lambda_3(t)\bigr)$
  \item Simulate a random U(0,1) variate, $u$, say.
  \item The simulated exit time is then the solution $t_u$ to
    the equation $S(t_u) = u \quad \Leftrightarrow \quad \sum_j\Lambda_j(t_u) = -\log(u)$.
  \item A simulated transition at $t_u$ is then found by simulating a
    random draw from the multinomial distribution with probabilities
    $p_i=\lambda_i(t_u) / \sum_j\lambda_j(t_u)$.
  \end{enumerate}
\item Separate cumulative incidences:
  \begin{enumerate}
  \item Simulate 3 independent U(0,1) random variates $u_1$, $u_2$ and $u_3$.
  \item Solve the equations $\Lambda_i(t_i)=-\log(u_i), i=1,2,3$ and get $(t_1,t_2,t_3)$.
  \item The simulated survival time is then $\min(t_1,t_2,t_3)$, and
    the simulated transition is to the state corresponding to this,
    that is $k \in \{1,2,3\}$, where $t_k=\min(t_1,t_2,t_3)$
  \end{enumerate}
\end{enumerate}
The intuitive argument is that with three possible transition there are
3 independent processes running, but only the first transition is observed.
The latter approach is used in the implementation in \texttt{simLexis}.

The formal argument for the equality of the two approaches goes as
follows:
\begin{enumerate}
\item Equality of the transition times:
  \begin{enumerate}
  \item In the first approach we simulate from a distribution with
    cumulative rate $\Lambda_1(t)+\Lambda_2(t)+\Lambda_3(t)$, hence
    from a distribution with survival function:
    \begin{align*}
    S(t) & = \exp\bigl(-(\Lambda_1(t)+\Lambda_2(t)+\Lambda_3(t))\bigr) \\
         & = \exp\bigl(-\Lambda_1(t)\bigr)\times
             \exp\bigl(-\Lambda_2(t)\bigr)\times
             \exp\bigl(-\Lambda_3(t)\bigr)
    \end{align*}
  \item In the second approach we choose the smallest of three
    independent survival times, with survival functions
    $\exp(-\Lambda_i), i=1,2,3$. Now, the survival function for the
    minimum of three independent survival times is:
    \begin{align*}
      S_\text{min}(t) & = \pmat{\min(t_1,t_2,t_3)>t} \\
                     & = \pmat{t_1>t} \times
                         \pmat{t_2>t} \times
                         \pmat{t_3>t} \\
                     & = \exp\bigl(-\Lambda_1(t)\bigr)\times
                         \exp\bigl(-\Lambda_2(t)\bigr)\times
                         \exp\bigl(-\Lambda_3(t)\bigr)
    \end{align*}
    which is the same survival function as derived above.
  \end{enumerate}
\item Type of transition:
  \begin{enumerate}
  \item In the first instance the probability of a transition to state
    $i$, conditional on the transition time being $t$, is as known
    from standard probability theory:
    $\lambda_i(t)/\bigl(\lambda_1(t)+\lambda_2(t)+\lambda_3(t)\bigr)$.
  \item In the second approach we choose the transition corresponding
    to the the smallest of the transition times. So when we condition
    on the event that a transition takes place at time $t$, we have to
    show that the conditional probability that the smallest of the
    three simulated transition times was actually the $i$th, is as above.

    But conditional on \emph{survival} till $t$, the probabilities
    that events of type $1,2,3$ takes place in the interval
    $(t,t+\dif t)$ are $\lambda_1(t)\dif t$, $\lambda_2(t)\dif t$ and
    $\lambda_3(t)\dif t$, respectively (assuming that the probability
    of more than one event in the interval of length $\dif t$ is
    0). Hence the conditional probability of transition of type $i$
    \emph{given a transition time} in $(t,t+\dif t)$ is:
    \[
    \frac{\lambda_i(t)\dif t}{\lambda_1(t)\dif t+
                              \lambda_2(t)\dif t+
                              \lambda_3(t)\dif t}=
    \frac{\lambda_i(t)}{\lambda_1(t)+\lambda_2(t)+\lambda_3(t)}
    \]
    --- exactly as above.
  \end{enumerate}
\end{enumerate}

<<echo = FALSE>>=
ende <- Sys.time()
cat("  Start time:", format(anfang, "%F, %T"),
  "\n    End time:", format(  ende, "%F, %T"),
  "\nElapsed time:", round(difftime(ende, anfang, units = "mins"), 2), "minutes\n")
@ %

\bibliographystyle{plain}
\begin{thebibliography}{1}

\bibitem{Carstensen.2011a}
B~Carstensen and M~Plummer.
\newblock Using {L}exis objects for multi-state models in {R}.
\newblock {\em Journal of Statistical Software}, 38(6):1--18, 1 2011.

\bibitem{Iacobelli.2013}
S~Iacobelli and B~Carstensen.
\newblock {Multiple time scales in multi-state models}.
\newblock {\em Stat Med}, 32(30):5315--5327, Dec 2013.

\bibitem{Plummer.2011}
M~Plummer and B~Carstensen.
\newblock Lexis: An {R} class for epidemiological studies with long-term
  follow-up.
\newblock {\em Journal of Statistical Software}, 38(5):1--12, 1 2011.

\end{thebibliography}

\addcontentsline{toc}{chapter}{References}

\end{document}

%\VignetteIndexEntry{Analysis of follow-up data using the Lexis functions in Epi}
\SweaveOpts{results=verbatim, keep.source=TRUE, include=FALSE, eps=FALSE}
\documentclass[a4paper, dvipsnames, twoside, 12pt]{report}
\newcommand{\Title}{Analysis of follow-up data\\ using the \texttt{Lexis} functions in
  \texttt{Epi}}
\newcommand{\Tit}{Follow-up with \texttt{Lexis}}
\newcommand{\Version}{Version 8}
\newcommand{\Dates}{March 2024}
\newcommand{\Where}{SDCC}
\newcommand{\Homepage}{\url{http://bendixcarstensen.com/} }
\newcommand{\Faculty}{\begin{tabular}{rl}
Bendix Carstensen
  & Steno Diabetes Center Copenhagen, Herlev, Denmark\\
  & {\small \& Department of Biostatistics, 
               University of Copenhagen} \\
  & \texttt{b@bxc.dk} \\
  & \url{http://BendixCarstensen.com} \\[1em]
                      \end{tabular}}

\input{topreport}
\renewcommand{\rwpre}{./flup}
<<echo=FALSE>>=
options(width = 90, 
        SweaveHooks=list(fig=function()
                         par(mar = c(3, 3, 1, 1), 
                             mgp = c(3, 1, 0) / 1.6, 
                             las = 1,
                            lend = "butt",
                             bty = "n")))
library(Epi)
@ %

<<echo = FALSE>>=
anfang <- Sys.time()
cat("Start time:", format(anfang, "%F, %T"), "\n")
@ %

\chapter*{Introduction}
\addcontentsline{toc}{chapter}{Introduction}

This is an introduction to the \texttt{Lexis} machinery in the
\texttt{Epi} package. The machinery is intended for representation and
manipulation of follow-up data (event history data) from studies where
exact dates of events are known. It accommodates follow-up through
multiple states and on multiple time scales.

This vignette uses an example from the \texttt{Epi} package to
illustrate the set-up of a simple \texttt{Lexis} object (a data frame
of follow-up intervals), as well as the subdivision of follow-up
intervals needed for multistate representation and analysis of
transition rates by flexible parametric functions.

The first chapter is exclusively on manipulation of the follow-up
representation, but it points to the subsequent chapter where analysis
is based on a \texttt{Lexis} representation with very small follow-up
intervals.

Chapter 2 uses analysis of mortality rates among Danish diabetes
patients (available in the \texttt{Epi} package) currently on insulin
treatment or not to illustrate the use of the the \texttt{Lexis}
machinery.

\section{History}

The \texttt{Lexis} machinery in the \texttt{Epi} package was first
conceived by Martyn Plummer\cite{Plummer.2011, Carstensen.2011a}, and
since its first appearance in the \texttt{Epi} package in 2008 it has
been expanded with a number of utilities. An overview of these can be
found in the last chapter of this note, \hyperlink{lexfun}{``\texttt{Lexis} functions''}. 

\chapter{Representation of follow-up data in the \texttt{Epi} package}

In the \texttt{Epi}-package, follow-up data is represented by adding
some extra variables and a few attributes to a data frame. Such a data
frame is called a \texttt{Lexis} object. The tools for handling
follow-up data then use the structure of this for special plots, 
tabulations and modeling.

Follow-up data basically consists of a time of entry, a time of exit
and an indication of the status at exit (normally either ``alive'' or
``dead'') for each person. Implicitly is also assumed a status
\emph{during} the follow-up (usually ``alive'').

\begin{figure}[htbp]
  \centering
\setlength{\unitlength}{1pt}
\begin{picture}(210, 70)(0, 75)
%\scriptsize
\thicklines
 \put(  0, 80){\makebox(0, 0)[r]{Age-scale}}
 \put( 50, 80){\line(1, 0){150}}
 \put( 50, 80){\line(0, 1){5}}
 \put(100, 80){\line(0, 1){5}}
 \put(150, 80){\line(0, 1){5}}
 \put(200, 80){\line(0, 1){5}}
 \put( 50, 77){\makebox(0, 0)[t]{35}}
 \put(100, 77){\makebox(0, 0)[t]{40}}
 \put(150, 77){\makebox(0, 0)[t]{45}}
 \put(200, 77){\makebox(0, 0)[t]{50}}

 \put(  0, 115){\makebox(0, 0)[r]{Follow-up}}

 \put( 80, 105){\makebox(0, 0)[r]{\small Two}}
 \put( 90, 105){\line(1, 0){87}}
 \put( 90, 100){\line(0, 1){10}}
 \put(100, 100){\line(0, 1){10}}
 \put(150, 100){\line(0, 1){10}}
 \put(180, 105){\circle{6}}
 \put( 95, 110){\makebox(0, 0)[b]{1}}
 \put(125, 110){\makebox(0, 0)[b]{5}}
 \put(165, 110){\makebox(0, 0)[b]{3}}

 \put( 50, 130){\makebox(0, 0)[r]{\small One}}
 \put( 60, 130){\line(1, 0){70}}
 \put( 60, 125){\line(0, 1){10}}
 \put(100, 125){\line(0, 1){10}}
 \put(130, 130){\circle*{6}}
 \put( 80, 135){\makebox(0, 0)[b]{4}}
 \put(115, 135){\makebox(0, 0)[b]{3}}
\end{picture}
  \caption{\it Follow-up of two persons}
  \label{fig:fu2}
\end{figure}

\section{Time scales}

A time scale is a variable that varies deterministically \emph{within}
each person during follow-up, \textit{e.g.}:
\begin{itemize}
  \item Age
  \item Calendar time
  \item Time since start of treatment
  \item Time since relapse
\end{itemize}
All time scales advance at the same pace, so the time followed is the
same on all time scales. Therefore, it will suffice to use only the
entry point on each of the time scales, for example:
\begin{itemize}
  \item Age at entry
  \item Date of entry
  \item Time at treatment (\emph{at} treatment this is 0)
  \item Time at relapse (\emph{at} relapse this is 0)
\end{itemize}
For illustration we need to load the \texttt{Epi} package:
<<>>= 
library(Epi)
print( sessionInfo(), l = F)
@ %
In the \texttt{Epi} package, follow-up in a cohort is represented in a
\texttt{Lexis} object. As mentioned, a \texttt{Lexis} object is a data
frame with some extra structure representing the follow-up. For the
\texttt{DMlate} data --- follow-up of diabetes patients in Denmark
recording date of birth, date of diabetes, date of insulin use, date
of first oral drug use and date of death --- we can construct a
\texttt{Lexis} object by:
<<>>= 
data(DMlate)
head(DMlate)
dmL <- Lexis(entry = list(per = dodm, 
                          age = dodm-dobth, 
                          tfD = 0), 
              exit = list(per = dox), 
       exit.status = factor(!is.na(dodth), 
                            labels = c("DM", "Dead")), 
              data = DMlate)
timeScales(dmL)
@ %
(The excluded persons are persons with date of diabetes equal to date
of death.)

The \texttt{entry} argument is a \emph{named} list with the entry
points on each of the time scales we want to use. It defines the names
of the time scales and the entry points of the follow-up of each
person. The \texttt{exit} argument gives the exit time on \emph{one}
of the time scales, so the name of the element in this list must match
one of the names of the \texttt{entry} list. This is sufficient, 
because the follow-up time on all time scales is the same, in this
case $\mathtt{dox}$-$\mathtt{dodm}$.

The \texttt{exit.status} is a categorical variable (a \emph{factor})
that indicates the exit status --- in this case whether the person
(still) is in state \texttt{DM} or exits to \texttt{Dead} at the end
of follow-up. In principle we should also indicate the
\texttt{entry.status}, but the default is to assume that all persons
enter in the \emph{first} of the mentioned \texttt{exit.state}s ---
in this case \texttt{DM}, because $\mathtt{FALSE}<\mathtt{TRUE}$.

Now take a look at the result:
<<>>= 
str(dmL)
head(dmL)[, 1:10]
@ %
The \texttt{Lexis} object \texttt{dmL} has a variable for each
time scale which is the entry point on this time scale. The follow-up
time is in the variable \texttt{lex.dur} (\texttt{dur}ation).
Note that the exit status is in the variable \texttt{lex.Xst}
(e\texttt{X}it \texttt{st}ate. The variable \texttt{lex.Cst} is the
state where the follow-up takes place (\texttt{C}urrent
\texttt{st}ate), in this case \texttt{DM} (alive with diabetes) for
all persons. This implies that \emph{censored} observations are
characterized by having $\mathtt{lex.Cst} = \mathtt{lex.Xst}$.

There is a \texttt{summary} function for \texttt{Lexis} objects that
lists the number of transitions and records as well as the total amount
of follow-up time; it also (optionally) prints information about the names of the
variables that constitute the time scales:
<<>>= 
summary(dmL, timeScales = TRUE)
@ %
It is possible to get a visualization of the follow-up along the
time scales chosen by using the \texttt{plot} method for \texttt{Lexis}
objects. \texttt{dmL} is an object of \emph{class} \texttt{Lexis}, so
using the function \texttt{plot()} on it means that \R\ will look for
the function \texttt{plot.Lexis} and use this function.
<<dmL1, fig = TRUE>>= 
plot(dmL)
@ %
The function allows quite a bit of control over the output, and a
\texttt{points.Lexis} function allows plotting of the endpoints of
follow-up:
<<dmL2, fig = TRUE>>= 
par(mar = c(3, 3, 1, 1), mgp = c(3, 1, 0) / 1.6)
plot(dmL, 1:2, lwd = 1, col = c("blue", "red")[dmL$sex], 
     grid = TRUE, lty.grid = 1, col.grid = gray(0.7), 
     xlim = 1960 + c(0, 60), xaxs = "i", 
     ylim =   40 + c(0, 60), yaxs = "i", las = 1)
points(dmL, 1:2, pch = c(NA, 3)[dmL$lex.Xst], 
       col = "lightgray", lwd = 3, cex = 0.3)
points(dmL, 1:2, pch = c(NA, 3)[dmL$lex.Xst], 
       col = c("blue", "red")[dmL$sex], lwd = 1, cex = 0.3)
box(bty = 'o')
@ %
In the above code you will note that the values of the arguments
\texttt{col} and \texttt{pch} are indexed by factors, using the
convention in \R\ that the index is taken as \emph{number of the level}
of the supplied factor. Thus \texttt{c("blue", "red")[dmL\$sex]} is
\texttt{"blue"} when \texttt{sex} is \texttt{M} (the first level).

The results of these two plotting commands are in figure
\ref{fig:Lexis-diagram}, p. \pageref{fig:Lexis-diagram}.
\begin{figure}[tb]
\centering
\includegraphics[width = 0.35\textwidth]{flup-dmL1}
\includegraphics[width = 0.63\textwidth]{flup-dmL2}
\caption{\it Lexis diagram of the \textrm{\tt DMlate} dataset; left
  panel is the default version, right panel: plot with some bells and
  whistles. The red lines are for women, blue for men, crosses
  indicate deaths.}
\label{fig:Lexis-diagram}
\end{figure}

\section{Splitting the follow-up time along a time scale}

In next chapter we shall conduct statistical analysis of mortality
rates, and a prerequisite for parametric analysis of rates is that
follow-up time is subdivided in smaller intervals, where we can
reasonably assume that rates are constant.

The follow-up time in a cohort can be subdivided (``split'') along a
time scale, for example current age. This is achieved by the
\texttt{splitLexis} (note that it is \emph{not} called
\texttt{split.Lexis}). This requires that the time scale and the
breakpoints on this time scale are supplied. Try:
<<>>= 
dmS1 <- splitLexis(dmL, "age", breaks = seq(0, 100, 5))
summary(dmL)
summary(dmS1)
@ %
We see that the number of persons and events and the amount of
follow-up is the same in the two data sets; only the number of records
differ --- the extra records all have \texttt{lex.Cst} = \texttt{DM} and
\texttt{lex.Xst} = \texttt{DM}.

To see how records are split for each individual, it is useful to list
the results for a few individuals (whom we selected with a view to the
illustrative usefulness):
<<>>= 
wh.id <- c(9, 27, 52, 484)
subset(dmL , lex.id %in% wh.id)[, 1:10]
subset(dmS1, lex.id %in% wh.id)[, 1:10]
@ %
The resulting object, \texttt{dmS1}, is again a \texttt{Lexis} object, 
and the follow-up may be split further along another time scale, for
example diabetes duration, \texttt{tfD}. Subsequently we list the
results for the chosen individuals:
<<>>= 
dmS2 <- splitLexis(dmS1, "tfD", breaks = c(0, 1, 2, 5, 10, 20, 30, 40))
subset(dmS2, lex.id %in% wh.id)[, 1:10]
@ %
A more efficient (and more intuitive) way of making this double split
is to use the \texttt{splitMulti} function from the \texttt{popEpi}
package:
<<>>= 
if (require(popEpi, quietly = TRUE)) 
   {
   options("popEpi.datatable" = FALSE)
   dmM <- splitMulti(dmL, 
                     age = seq(0, 100, 5), 
                     tfD = c(0, 1, 2, 5, 10, 20, 30, 40), 
                    drop = FALSE)
   summary(dmS2)
   summary(dmM)
   }
@ %
Note we used the argument \texttt{drop = FALSE} which will retain
follow-up also outside the window defined by the range of the
breaks. Otherwise, the default for \texttt{splitMulti} would be to drop
follow-up outside \texttt{age} [0, 100] and \texttt{tfD} [0, 40]. This
clipping behaviour is not available in \texttt{splitLexis}, 
nevertheless this may be exactly what we want in some situations.

So we see that the two ways of splitting data yields the same amount of
follow-up, but the results are not necessarily identical on all machines:
<<>>= 
if (require(popEpi, quietly = TRUE)) 
   {
   identical(dmS2, dmM)
   class(dmS2)
   class(dmM)
   }
@ %
As we see, this is because the \texttt{dmM} object also is a
\texttt{data.table} object; the \texttt{splitMulti} uses the
\texttt{data.table} machinery which makes the splitting substantially
faster --- this is of particular interest if you operate on large data
sets ($>100, 000$ records).

Thus the recommended way of splitting follow-up time is by
\texttt{splitMulti}. But you should be aware that the result is a
\texttt{data.table} object, which in some circumstances behaves
slightly different from \texttt{data.frame}s. See the manual for
\texttt{data.table}.

\section{Cutting follow up time at dates of intermediate events}

If we have a recording of the date of a specific event as for example
recovery or relapse, we may classify follow-up time as being before or
after this intermediate event, but it requires that follow-up records
that straddle the event be cut in two and placed in separate records, 
one representing follow-up \emph{before} the intermediate event, and another
representing follow-up \emph{after} the intermediate event. This is
achieved with the function \texttt{cutLexis}, which takes three
arguments: the time point of the intermediate event, the time scale
that this point refers to, and the value of the (new) state following
the date. Optionally, we may also define a new time scale with the
argument \texttt{new.scale = }.

We are interested in the time before and after inception of insulin
use, which occurs at the date \texttt{doins}:
<<>>= 
subset(dmL, lex.id %in% wh.id)
dmC <- cutLexis(data = dmL, 
                 cut = dmL$doins, 
           timescale = "per", 
           new.state = "Ins", 
           new.scale = "tfI")
subset(dmC, lex.id %in% wh.id)[, 1:10]
@ %
Note that the process of cutting time is simplified by having all
types of events referred to the calendar time scale. This is a
generally applicable advice in handling follow-up data: Get all event
times as \emph{dates}, location of events and follow-up on other time
scales can then easily be derived from this.

Note that individual 52 has had his follow-up cut at 6.55 years from
diabetes diagnosis and individual 484 at 5.70 years from diabetes
diagnosis. This dataset could then be split along the time scales as
we did before with \texttt{dmL}.

The result of this can also be achieved by cutting the split dataset
\texttt{dmS2} instead of \texttt{dmL}:
<<>>= 
dmS2C <- cutLexis(data = dmS2, 
                   cut = dmS2$doins, 
             timescale = "per", 
             new.state = "Ins", 
             new.scale = "tfI")
subset(dmS2C, lex.id %in% wh.id)
@ %
Thus it does not matter in which order we use \texttt{splitLexis} and
\texttt{cutLexis}. Mathematicians would say that \texttt{splitLexis}
and \texttt{cutLexis} are commutative.

Note in \texttt{lex.id} = 484, that follow-up subsequent to the event is
classified as being in state \texttt{Ins}, but that the final
transition to state \texttt{Dead} is preserved. This is the point of
the \texttt{precursor.states = } argument. It names the states (in this
case \texttt{DM}) that will be over-written by \texttt{new.state} (in
this case \texttt{Ins}), while the state \texttt{Dead} should not be
updated even if it is after the time where the persons moves to state
\texttt{Ins}. In other words, only state \texttt{DM} is a precursor to
state \texttt{Ins}, state \texttt{Dead} is always subsequent to state
\texttt{Ins}.

Note that we defined a new time scale, \texttt{tfI}, using the
argument \texttt{new.scale = tfI}. This has a special status relative to
the other three time scales, it is defined as time since entry into a
state, namely \texttt{Ins}, this is noted in the time scale part of
the summary of \texttt{Lexis} object --- the information sits in the
attribute \texttt{time.since} of the \texttt{Lexis} object, which can
be accessed by the function \texttt{timeSince()} or through the
\texttt{summary()}:
<<>>= 
summary(dmS2C, timeScales = TRUE)
@ %
Finally we can get a quick overview of the states and transitions by
using \texttt{boxes} --- \texttt{scale.R} scales transition rates to
rates per 1000 PY:
<<box1, fig = TRUE, height = 6, width = 8>>= 
boxes(dmC, boxpos = TRUE, scale.R = 1000, show.BE = TRUE)
@ %
\insfig{box1}{0.8}{States, person years, transitions and rates in the
  cut dataset. The numbers \emph{in} the boxes are person-years and
  the number of persons \texttt{B}eginning, resp. \texttt{E}nding
  their follow-up in each state (triggered by \textrm{\tt
    show.BE = TRUE}). The numbers at the arrows are the number of
  transitions and transition rates per 1000 (triggered by \textrm{\tt
  scale.R = 1000}).}

\chapter{Modeling rates from \texttt{Lexis} objects}

\section{Covariates}

In the dataset \texttt{dmS2C} there are three types of covariates that
can be used to describe mortality rates:
\begin{enumerate}
\item time-dependent covariates
\item time scales
\item fixed covariates
\end{enumerate}

There is only one time-dependent covariate here, namely
\texttt{lex.Cst}, the current state of the person's follow up; it
takes the values \texttt{DM} and \texttt{Ins} according to whether the
person has ever purchased insulin at a given time of follow-up.

The time-scales are obvious candidates for explanatory variables for
the rates, notably age and time from diagnosis (duration of diabetes)
and insulin.

\subsection{Time scales as covariates}

If we want to model the effect of the time scale variables on
occurrence rates, we will for each interval use either the value of
the left endpoint in each interval or the middle. There is a function
\texttt{timeBand} which returns either of these:
<<>>= 
timeBand(dmS2C, "age", "middle")[1:10]
# For nice printing and column labelling we use the data.frame() function:
data.frame(dmS2C[, c("per", "age", "tfD", "lex.dur")], 
           mid.age = timeBand(dmS2C, "age", "middle"), 
             mid.t = timeBand(dmS2C, "tfD", "middle"), 
            left.t = timeBand(dmS2C, "tfD", "left"  ), 
           right.t = timeBand(dmS2C, "tfD", "right" ), 
            fact.t = timeBand(dmS2C, "tfD", "factor"))[1:15, ]
@ %
Note that the values of these functions are characteristics of the
intervals defined by \texttt{breaks = }, \emph{not} the midpoints nor
left or right endpoints of the actual follow-up intervals (which would
be \texttt{tfD} and \texttt{tfD+lex.dur}, respectively).

These functions are intended for modeling time scale variables as
factors (categorical variables) in which case the coding must be
independent of the censoring and mortality pattern --- it should only
depend on the chosen grouping of the time scale. Modeling time scales as
\emph{quantitative} should not be based on these codings but directly
on the values of the time-scale variables, notably the left endpoints
of the intervals.

\subsection{Differences between time scales}

Apparently, the only fixed variable is \texttt{sex}, but formally the
dates of birth (\texttt{dobth}), diagnosis (\texttt{dodm}) and first
insulin purchase (\texttt{doins}) are fixed covariates too. They can
be constructed as origins of time scales referred to the calendar time
scale. Likewise, and possibly of greater interest, we can consider
these origins on the age scale, calculated as the difference between
age and another time scale.

These would then be age at birth (hardly relevant since it is the same
for all persons), age at diabetes
diagnosis and age at insulin treatment.

\subsection{Keeping the relation between time scales}

The midpoint (as well as the right interval endpoint) should be used
with caution if the variable age at diagnosis \texttt{dodm-dobth} is
modeled too; the age at diabetes is logically equal to the difference
between current age (\texttt{age}) and time since diabetes diagnosis
(\texttt{tfD}):
<<>>= 
summary((dmS2$age - dmS2$tfD) - (dmS2$dodm - dmS2$dobth))
@ %
This calculation refers to the \emph{start} of each interval --- which
are in the time scale variables in the \texttt{Lexis} object. But when
using the middle of the intervals, this relationship is not preserved:
<<>>= 
summary(timeBand(dmS2, "age", "middle") -
        timeBand(dmS2, "tfD", "middle") - 
        (dmS2$dodm - dmS2$dobth))
@ %
If all three variables are to be included in a model, we must make
sure that the \emph{substantial} relationship between the variables be
maintained. One way is to recompute age at diabetes diagnosis from the two
midpoint variables, but more straightforward would be to use the left
endpoint of the intervals, that is the time scale variables in the
\texttt{Lexis} object.

If we dissolve the relationship between the variables \texttt{age}, 
\texttt{tfD} and age at diagnosis by grouping we may obtain
identifiability of the three separate effects, but it will be at the
price of an arbitrary allocation of a linear trend between them.

For the sake of clarity, consider current age, $a$, duration of
disease, $d$ and age at diagnosis $e$, where
\[
 \text{current age} = 
 \text{age at diagnosis} +
 \text{disease duration}, 
 \quad \text{\ie} \quad a = e + d \quad
 \Leftrightarrow \quad e + d - a = 0
\]
If we model the effect of the quantitative variables $a$, $e$ and $d$
on the log-rates by three functions $f$, $g$ and $h$:
$ \log(\lambda) = f(a)+g(d)+h(e) $
then for any $\kappa$:
\begin{align*}
   \log(\lambda) & = f(a)+g(d)+h(e)+\kappa(e+d-a)\\
                 & = 
   \big(f(a)-\kappa a \big)+
   \big(g(d)+\kappa d \big)+
   \big(h(e)+\kappa e \big) \\
& = \tilde f(a)+ \tilde g(d)+ \tilde h(e)
\end{align*}
In practical modeling this will turn up as a singular model matrix
with one parameter aliased, corresponding to some arbitrarily chosen
value of $\kappa$ (depending on software conventions for singular
models). This phenomenon is well known from age-period-cohort models.

Thus we see that we can move any slope around between the three terms, 
so if we achieve identifiability by using grouping of one of the
variables we will in reality have settled for a particular value of
$\kappa$, without known why we chose just that. The solution is to
resort to predictions which are independent of the particular
parametrization or choose a particular parametrization with explicit constraints.

\section{Modeling of rates}

As mentioned, the purpose of subdividing follow-up data in smaller
intervals is to be able to model effects of time scale variables as
parametric functions. When we split along a time scale we can get
intervals that are as small as we want; if they are sufficiently
small, an assumption of constant rates in each interval becomes
reasonable.

In a model that assumes a constant occurrence rate in each of the
intervals the likelihood contribution from each interval is the same
as the likelihood contribution from a Poisson variate $D$, say, with
mean $\lambda \ell$ where $\lambda$ is the rate and $\ell$ is the
interval length, and where the value of the variate $D$ is 1 or 0
according to whether an event has occurred or not. Moreover, the
likelihood contributions from all follow-up intervals from a single
person are \emph{conditionally} independent (conditional on having
survived till the start of the interval in question). This implies
that the total contribution to the likelihood from a single person is
a product of terms, and hence the same as the likelihood of a number
of independent Poisson terms, one from each interval.

Note that variables are neither Poisson distributed (\eg they can only
ever assume values 0 or 1) nor independent --- it is only the
likelihood for the follow-up data that happens to be the same as the
likelihood from independent Poisson variates. Different models can
have the same likelihood, a model cannot be inferred from the
likelihood.

Parametric modeling of the rates is obtained by using the
\emph{values} of the time scales for each interval as \emph{quantitative}
explanatory variables, using for example splines. And of course also
the values of the fixed covariates and the time-dependent variables
for each interval. Thus the model will be one where the rate is
assumed constant in each (small) interval, but where a parametric form
of the \emph{size} of the rate in each interval is imposed by the
model, using the time scale as a quantitative covariate.

\subsection{Interval length}

In the first chapter we illustrated cutting and splitting by listing
the results for a few individuals across a number of intervals. For
illustrational purposes we used 5-year age bands to avoid excessive
listings, but since the doubling time for mortality on the age scale
is only slightly larger than 5 years, the assumption about constant
rates in each interval would be pretty far fetched if we were to use 5
year intervals.

Thus, for modeling purposes we split the follow-up in 3 month
intervals. When we use intervals of 3 months length it is superfluous
to split along multiple time scales --- the precise location of
tightly spaced splits will be irrelevant from any practical point of
view. \texttt{splitLexis} and \texttt{splitMulti} will allocate the
actual split times for all of the time scale variables, so these can be
used directly in modeling.

So we split the cut dataset in 3 months intervals along the age
scale:
<<>>= 
dmCs <- splitLexis(dmC, time.scale = "age", breaks = seq(0, 110, 1/4))
summary(dmCs, t = T)
@ %
We see that we now have 228, 748 records and 9996 persons, so about 23
records per person. The total risk time is 54, 275 years, a bit less
than 3 months on average per record as expected.

\subsection{Practicalities for splines}

In this study we want to look at how mortality depend on age
(\texttt{age}) and time since start of insulin use (\texttt{tfI}). If
we want to use splines in the description we must allocate knots for
anchoring the splines at each of the time scales, either by some
\textit{ad hoc} method or by using some sort of penalized splines as
for example by \texttt{gam}; the latter will not be treated here; it
belongs in the realm of the \texttt{mgcv} package.

Here we shall use the former approach and allocate 5 knots on each of
the time-scales. We allocate knots so that we have the events evenly
distributed between the knots. Since the insulin state starts at 0 for
all individuals we include 0 as the first knot, such that any set of natural
splines with these knots will have the value 0 at 0 on the time
scale.
<<>>= 
(a.kn <- with(subset(dmCs, lex.Xst == "Dead"), 
              quantile(age+lex.dur, (1:5-0.5)/5)))
(i.kn <- c(0, 
           with(subset(dmCs, lex.Xst == "Dead" & lex.Cst == "Ins"), 
                quantile(tfI+lex.dur, (1:4)/5))))
@ %
In the \texttt{Epi} package there is a convenience wrapper, 
\texttt{Ns}, for the \texttt{n}atural \texttt{s}pline generator
\texttt{ns}, that takes the smallest and the largest of a set of
supplied knots to be the boundary knots, so the explicit definition of
the boundary knots becomes superfluous.

Note that it is a feature of the \texttt{Ns} (via the features of
\texttt{ns}) that any generated spline function is 0 at the leftmost
knot.

\subsection{Poisson models}

A model that describes mortality rates as only a function of age would
then be:
<<>>= 
ma <- glm((lex.Xst == "Dead") ~ Ns(age, knots = a.kn), 
           family = poisson, 
           offset = log(lex.dur), 
             data = dmCs)
summary(ma)
@ %
The offset, \texttt{log(lex.dur)} comes from the fact that the
likelihood for the follow-up data during $\ell$ time is the same as
that for independent Poisson variates with mean $\lambda \ell$, and
that the default link function for the Poisson family is the log, so
that we are using a linear model for the log-mean, 
$\log(\lambda) + \log(\ell)$.  But when we want a model for the
log-rate ($\log(\lambda)$), the term $\log(\ell)$ must still be
included as a covariate, but with regression coefficient fixed to 1; a
so-called \emph{offset}. This is however a technicality; it just
exploits that the likelihood of a particular Poisson model and that of
the rates model is the same.

In the \texttt{Epi} package is a \texttt{glm} family, \texttt{poisreg}
that has a more intuitive interface to the likelihood of rates, namely
where the response is a 2-column matrix of events and person-time,
respectively. This is in concert with the fact that the outcome
variable in follow-up studies is bivariate: (event, risk time).
<<>>= 
Ma <- glm(cbind(lex.Xst == "Dead", lex.dur) ~ Ns(age, knots = a.kn), 
          family = poisreg, 
            data = dmCs)
summary(Ma)
@ %
Exploiting the multistate structure in the \texttt{Lexis} object
there is a multistate convenience wrapper for \texttt{glm} with the
\texttt{poisreg} family, that just requires specification of the
transitions in terms of \texttt{from} and \texttt{to}. Although it is
called \texttt{glm.Lexis} it is \emph{not} an S3 method for
\texttt{Lexis} objects:
<<>>= 
Xa <- glm.Lexis(dmCs, from = "DM", to = "Dead", 
                formula = ~ Ns(age, knots = a.kn))
@ %
The result is a \texttt{glm} object but with an extra attribute, \texttt{Lexis}:
<<>>= 
attr(Xa, "Lexis")
@ %
There are similar wrappers for \texttt{gam} and \texttt{coxph} models, 
\texttt{gam.Lexis} and \texttt{coxph.Lexis}, but these will not be
elaborated in detail.

The \texttt{from = } and \texttt{to = } can even be omitted, in which case
all possible transitions \emph{into} any of the absorbing states is
modeled:
<<>>= 
xa <- glm.Lexis(dmCs, formula = ~ Ns(age, knots = a.kn))
@ 
We can check if the four models fitted are the same:
<<>>= 
c(deviance(ma), deviance(Ma), deviance(Xa), deviance(xa))
@ %
Oops! the model \texttt{Xa} is apparently not the same as the other
three? This is because the explicit specification
\verb|from = "DM", to = "Dead"|, omits modeling contributions from the
$\mathtt{Ins}\rightarrow\mathtt{Dead}$ transition --- the output
actually said so --- see also figure \ref{fig:box1} on
p. \pageref{fig:box1}. The other three models all use both transitions
--- and assume that the two transition rates are the same, \ie that
start of insulin has no effect on mortality. We shall relax this
assumption later.

The parameters from the model do not have any direct interpretation
\textit{per se}, but we can compute the estimated mortality rates for
a range of ages using \texttt{ci.pred} with a suitably defined
prediction data frame. 

Note that if we use the \texttt{poisson} family of models, we must
specify all covariates in the model, including the variable in the
offset, \texttt{lex.dur} (remember that this was a covariate with
coefficient fixed at 1). We set the latter to 1000, because we want the
mortality rates per 1000 person-years. Using the \texttt{poisreg}
family, the prediction will ignore any value of \texttt{lex.dur}
specified in the prediction data frame, the returned rates will be per
unit in which \texttt{lex.dur} is recorded.
<<pr-a, fig = TRUE>>= 
nd <- data.frame(age = 40:85, lex.dur = 1000)
pr.0 <- ci.pred(ma, newdata = nd)      # mortality per 100 PY
pr.a <- ci.pred(Ma, newdata = nd)*1000 # mortality per 100 PY
summary(pr.0/pr.a)
matshade(nd$age, pr.a, plot = TRUE, 
          type = "l", lty = 1, 
          log = "y", xlab = "Age (years)", 
          ylab = "DM mortality per 1000 PY")
@ %
\insfig{pr-a}{0.8}{Mortality among Danish diabetes patients by age
  with 95\% CI as shaded area. We see that the rates increase linearly
  on the log-scale, that is exponentially by age.}

\section{Time dependent covariate}

A Poisson model approach to mortality by insulin status, would be to
assume that the rate-ratio between patients on insulin and not on
insulin is a fixed quantity, independent of time since start of insulin, 
independent of age. This is commonly termed a proportional hazards
assumption, because the rates (hazards) in the two groups are
proportional along the age (baseline time) scale.
<<>>= 
pm <- glm(cbind(lex.Xst == "Dead", lex.dur) ~ Ns(age, knots = a.kn) 
                                              + lex.Cst + sex, 
          family = poisreg, 
            data = dmCs)
round(ci.exp(pm), 3)
@ %
So we see that persons on insulin have about twice the mortality of
persons not on insulin and that women have 2/3 the mortality of men.

\subsection{Time since insulin start}

If we want to test whether the excess mortality depends on the time
since start if insulin treatment, we can add a spline terms in
\texttt{tfI}. But since \texttt{tfI} is a time scale defined as time
since entry into a new state (\texttt{Ins}), the variable \texttt{tfI}
will be missing for those in the \texttt{DM} state, so before modeling
we must set the \texttt{NA}s to 0, which we do with \texttt{tsNA20}
(acronym for \texttt{t}ime\texttt{s}cale \texttt{NA}s to zero):
<<>>= 
pm <- glm(cbind(lex.Xst == "Dead", lex.dur) ~ Ns(age, knots = a.kn) 
                                            + Ns(tfI, knots = i.kn) 
                                            + lex.Cst + sex, 
          family = poisreg, 
            data = tsNA20(dmCs))
@ %
As noted before we could do this simpler with \texttt{glm.Lexis}, even
without the \texttt{from = } and \texttt{to = } arguments, because we are
modeling all transitions \emph{into} the absorbing state
(\texttt{Dead}):
<<>>= 
Pm <- glm.Lexis(tsNA20(dmCs), 
                form = ~ Ns(age, knots = a.kn) 
                       + Ns(tfI, knots = i.kn) 
                       + lex.Cst + sex)
c(deviance(Pm), deviance(pm))
identical(model.matrix(Pm), model.matrix(pm))
@ %
The coding of the effect of \texttt{tfI} is so that the value is 0 at
0, so the meaning of the estimate of the effect of \texttt{lex.Cst} is
the RR between persons with and without insulin, immediately after
start of insulin:
<<>>= 
round(ci.exp(Pm, subset = "ex"), 3)
@ %
We see that the effect of sex is pretty much the same as before, but
the effect of \texttt{lex.Cst} is much larger, it now refers to a
different quantity, namely the RR at \texttt{tfI} = 0. If we want to see
the effect of time since insulin, it is best viewed jointly with the
effect of age:
<<ins-time, fig = TRUE>>= 
ndI <- data.frame(expand.grid(tfI = c(NA, seq(0, 15, 0.1)), 
                               ai = seq(40, 80, 10)), 
                  sex = "M", 
                  lex.Cst = "Ins")
ndI <- transform(ndI, age = ai+tfI)
head(ndI)
ndA <- data.frame(age = seq(40, 100, 0.1), tfI = 0,  lex.Cst = "DM", sex = "M")
pri <- ci.pred(Pm, ndI) * 1000
pra <- ci.pred(Pm, ndA) * 1000
matshade(ndI$age, pri, plot = TRUE, las = 1, 
         xlab = "Age (years)", ylab = "DM mortality per 1000 PY", 
         log = "y", lty = 1, col = "blue")
matshade(ndA$age, pra)
@ %
\insfig{ins-time}{0.8}{Mortality rates of persons on insulin, starting
insulin at ages 40, 50, \ldots, 80 (blue), compared with persons not on
insulin (black curve). Shaded areas are 95\% CI.}

In figure \ref{fig:ins-time}, p. \pageref{fig:ins-time}, we see that
mortality is high just after insulin start, but falls by almost a
factor 3 during the first year. Also we see that there is a tendency
that mortality in a given age is smallest for those with the longest
duration of insulin use.

\section{The Cox model}

Note that in the Cox-model the age is used as response variable, 
slightly counter-intuitive. Hence the age part of the linear predictors
is not in that model:
<<>>= 
library(survival)
cm <- coxph(Surv(age, age+lex.dur, lex.Xst == "Dead") ~
            Ns(tfI, knots = i.kn) + lex.Cst + sex, 
            data = tsNA20(dmCs))
@ %
There is also a multistate wrapper for Cox models, requiring a
l.h.s. side for the \texttt{formula = } argument:
<<>>= 
Cm <- coxph.Lexis(tsNA20(dmCs), 
                  form = age ~ Ns(tfI, knots = i.kn) + lex.Cst + sex)
cbind(ci.exp(cm), ci.exp(Cm))
@ %
We can compare the estimates from the Cox model with those from the
Poisson model --- we must add \texttt{NA}s because the Cox-model does
not give the parameters for the baseline time scale (\texttt{age}), but
also remove one of the parameters, because \texttt{coxph} parametrizes
factors (in this case \texttt{lex.Cst}) by all defined levels and not
only by the levels present in the dataset at hand (note the line of
\texttt{1.0000000}s in the print above):
<<>>= 
round(cbind(ci.exp(Pm), 
       rbind(matrix(NA, 5, 3), 
             ci.exp(cm)[-6, ])), 3)
@ %
Thus we see that the Poisson and Cox gives pretty much the same
results. You may argue that Cox requires a smaller dataset, because
there is no need to subdivide data in small intervals \emph{before}
insulin use. But certainly the time \emph{after} insulin inception need
to be if the effect of this time should be modeled. 

The drawback of the Cox-modeling is that it is not possible to show
the absolute rates as we did in figure \ref{fig:ins-time} on
\pageref{fig:ins-time}.

\section{Marginal effect of time since insulin}

When we plot the marginal effect of \texttt{tfI} from the two models
we get pretty much the same; here we plot the RR relative to
\texttt{tfI} = 2 years. Note that we are deriving the RR as the ratio of
two sets of predictions, from the data frames \texttt{nd} and
\texttt{nr} --- for further details consult the help page for
\texttt{ci.lin}, specifically the use of a list as the
\texttt{ctr.mat} argument:
<<Ieff, fig = TRUE>>= 
nd <- data.frame(tfI = seq(0, 15, , 151), lex.Cst = "Ins", sex = "M")
nr <- data.frame(tfI =     2            , lex.Cst = "Ins", sex = "M")
ppr <- ci.exp(pm, list(nd, nr), xvars = "age")
cpr <- ci.exp(cm, list(nd, nr))
par(mar = c(3, 3, 1, 1), mgp = c(3, 1, 0)/1.6, las = 1, bty = "n")
matshade(nd$tfI, cbind(ppr, cpr), plot = T, 
         lty = c(1, 2), log = "y", 
         xlab = "Time since insulin (years)", ylab = "Rate ratio")
abline(h = 1, lty = 3)
@ %
\insfig{Ieff}{0.8}{The naked duration effects relative to 2 years of
  duration, black from Poisson model, red from Cox model. The two sets
  of estimates are identical, and so are the standard errors, so the
  two shaded areas overlap almost perfectly.}

In figure \ref{fig:Ieff}, p. \pageref{fig:Ieff}, we see that the
duration effect is exactly the same from the two modeling approaches.

We will also want the RR relative to the non-insulin users --- recall these
are coded 0 on the \texttt{tfI} variable:
<<IeffR, fig = TRUE>>= 
nd <- data.frame(tfI = seq(0, 15, , 151), lex.Cst = "Ins", sex = "M")
nr <- data.frame(tfI =     0            , lex.Cst = "DM" , sex = "M")
ppr <- ci.exp(pm, list(nd, nr), xvars = "age")
cpr <- ci.exp(cm, list(nd, nr))
par(mar = c(3, 3, 1, 1), mgp = c(3, 1, 0)/1.6, las = 1, bty = "n")
matshade(nd$tfI, cbind(ppr, cpr), 
         xlab = "Time since insulin (years)", 
         ylab = "Rate ratio relative to non-Insulin", 
         lty = c(1, 2), log = "y", plot = T)
@ %
\insfig{IeffR}{0.8}{Insulin duration effect (state \textrm{\tt Ins})
  relative to no insulin (state \textrm{\tt DM}), black from Poisson
  model, red from Cox model. The \emph{shape} is the same as the
  previous figure, but the RR is now relative to non-insulin, instead
  of relative to insulin users at 2 years duration. The two sets of
  estimates are identical, and so are the standard errors, so the two
  shaded areas overlap almost perfectly.}

In figure \ref{fig:IeffR}, p. \pageref{fig:IeffR}, we see the effect
of increasing duration of insulin use \emph{for a fixed age} which is
a bit artificial, so we would like to see the \emph{joint} effects of
age and insulin duration. What we cannot see is how the duration
affects mortality relative to \texttt{current} age (at the age
attained at the same time as the attained \texttt{tfI}).

Another way of interpreting this curve is as the rate ratio relative
to a person not on insulin, so we see that the RR (or hazard ratio, HR
as some call it) is over 5 at the start of insulin (the
\texttt{lex.Cst} estimate), and decreases to about 1.5 in the long
term.

Both figure \ref{fig:Ieff} and \ref{fig:IeffR} indicate a declining RR
by insulin duration, but only from figure \ref{fig:ins-time} it is
visible that mortality actually is \emph{in}creasing by age after some
2 years after insulin start. This point would not be available if we
had only fitted a Cox model where we did not have access to the
baseline hazard as a function of age.

\section{Age$\times$duration interaction}

The model we fitted assumes that the RR is the same regardless of the
age at start of insulin --- the hazards are multiplicative. Sometimes
this is termed the proportional hazards assumption: For \emph{any}
fixed age the HR is the same as a function of time since insulin, and
vice versa.

A more correct term would be ``main effects model'' --- there is no
interaction between age (the baseline time scale) and other
covariates. So there is really no need for the term ``proportional
hazards''; well defined and precise statistical terms for it has
existed for eons.

\subsection{Age at insulin start}

In order to check the consistency of the multiplicative assumption
across the spectrum of age at insulin inception, we can fit an
interaction model. One approach to this would be using a non-linear
effect of age at insulin use (for convenience we use the same knots as
for age) --- note that the prediction data frames are the same as we
used above, because we do not compute age at insulin use as a separate
variable, but rather enter it as the difference between current age
(\texttt{age}) and insulin duration (\texttt{tfI}). 

At first glance we might think of doing:
<<>>= 
imx <- glm.Lexis(tsNA20(dmCs), 
                 formula = ~ Ns(age      , knots = a.kn) 
                           + Ns(      tfI, knots = i.kn)
                           + Ns(age - tfI, knots = a.kn)
                           + lex.Cst + sex)
@ %
But this will fit a model with a rate-ratio between persons with and
without insulin that depends only on age at insulin start for the time
\emph{after} insulin start, the RR at \texttt{tfI} = 0 will be the same
at any age, which really is not the type of interaction we wanted.

We want the \texttt{age-tfI} term to be specific for the insulin
exposed so we may use one of two other approaches, that are
conceptually alike but mathematically different:
<<>>= 
Im <- glm.Lexis(tsNA20(dmCs), 
                formula = ~ Ns(age       , knots = a.kn) 
                          + Ns(       tfI, knots = i.kn)
                          + Ns((age - tfI) * (lex.Cst == "Ins"), knots = a.kn)
                          + lex.Cst + sex)
im <- glm.Lexis(tsNA20(dmCs), 
                formula = ~ Ns(age      , knots = a.kn) 
                          + Ns(      tfI, knots = i.kn)
                  + lex.Cst:Ns(age - tfI, knots = a.kn)
                          + lex.Cst + sex)
@ %
The first model (\texttt{Im}) has a common age-effect (\texttt{Ns(age, ...}) for
persons with and without diabetes and a RR depending on insulin
duration \texttt{tfI} and age at insulin (\texttt{age-tfI}). Since the
linear effect of these two terms are in the model as well, a linear
trend in the RR by current age (\texttt{age}) is accommodated as well.

The second model allows age-effects that differ non-linearly between
person with and without insulin, because the interaction term
\texttt{lex.Cst:Ns(age-tfI...} for persons not on insulin is merely an
age term (since \texttt{tfI} is coded 0 for all follow-up not on
insulin).

We can compare the models fitted: 
<<>>= 
anova(imx, Im, im, test = 'Chisq')
@ 
so we see that the models indeed are different, and moreover that the
last model does not provide substantial further improvement, by
allowing non-linear RR along the age-scale.

We can illustrate the different estimated rates from the three models
in figure \ref{fig:dur-int}, p. \pageref{fig:dur-int}:
<<dur-int, fig = TRUE>>= 
pxi <- ci.pred(imx, ndI)
pxa <- ci.pred(imx, ndA)
pIi <- ci.pred(Im , ndI)
pIa <- ci.pred(Im , ndA)
pii <- ci.pred(im , ndI)
pia <- ci.pred(im , ndA)
par(mar = c(3, 3, 1, 1), mgp = c(3, 1, 0)/1.6, las = 1, bty = "n")
matshade(ndI$age, cbind(pxi, pIi, pii)*1000, plot = T, log = "y", 
         xlab = "Age", ylab = "Mortality per 1000 PY", 
         lty = 1, lwd = 2, col = c("blue", "forestgreen", "red"), alpha = 0.1)
matshade(ndA$age, cbind(pxa, pIa, pia)*1000, 
         lty = 1, lwd = 2, col = c("blue", "forestgreen", "red"), alpha = 0.1)
@ %
\insfig{dur-int}{0.8}{Age at insulin as interaction between age and
  duration. Blue curves are from the naive interaction model
  \textrm{\tt imx} with identical $\RR$ at \textrm{\tt tfI} = 0 at any
  age; green curves are from the interaction model with age at
  insulin, from the model \textrm{\tt Im} with only linear
  differences by age, and red lines from the full interaction model
  \textrm{\tt im}.}

We can also plot the RRs only from these models (figure
\ref{fig:dur-int-RR}, p. \pageref{fig:dur-int-RR}); for this we need
the reference frames, and the machinery from \texttt{ci.exp} allowing
a list of two data frames:
<<dur-int-RR, fig = TRUE>>= 
ndR <- transform(ndI, tfI = 0, lex.Cst = "DM")
cbind(head(ndI), head(ndR))
Rxi <- ci.exp(imx, list(ndI, ndR))
Rii <- ci.exp(im , list(ndI, ndR))
RIi <- ci.exp(Im , list(ndI, ndR))
par(mar = c(3, 3, 1, 1), mgp = c(3, 1, 0)/1.6, las = 1, bty = "n")
matshade(ndI$age, cbind(Rxi, RIi, Rii), plot = T, log = "y", 
         xlab = "Age (years)", ylab = "Rate ratio vs, non-Insulin", 
         lty = 1, lwd = 2, col = c("blue", "forestgreen", "red"), alpha = 0.1)
abline(h = 1)
abline(h = ci.exp(imx, subset = "lex.Cst")[, 1], lty = "25", col = "blue")
@ %
\insfig{dur-int-RR}{0.9}{RR from three different interaction
  models. The horizontal dotted line is at the estimated effect of
  \textrm{\tt lex.Cst}, to illustrate that the first model (blue)
  constrains this initial HR to be constant across age. The green
  curves are the extended interaction model, and the red the full
  one.}

\clearpage

\subsection{General interaction}

As a final illustration we may want to explore a different kind of
interaction, not defined from the duration --- here we simplify the
interaction by not using the second-last knot in the interaction terms
--- figure \ref{fig:splint}, p. \pageref{fig:splint}. Note again that
the prediction code is the same:
<<splint, fig = TRUE>>= 
gm <- glm.Lexis(tsNA20(dmCs), 
                formula = ~ Ns(age, knots = a.kn) 
                          + Ns(tfI, knots = i.kn)
                          + lex.Cst:Ns(age, knots = a.kn):Ns(tfI, knots = i.kn)
                          + lex.Cst + sex)
pgi <- ci.pred(gm, ndI)
pga <- ci.pred(gm, ndA)
par(mar = c(3, 3, 1, 1), mgp = c(3, 1, 0)/1.6, las = 1, bty = "n")
matshade(ndI$age, cbind(pgi, pii)*1000,  plot = T, 
         lty = c("solid", "21"), lend = "butt", lwd = 2, log = "y", 
         xlab = "Age (years)", ylab = "Mortality rates per 1000 PY", 
         alpha = c(0.2, 0.1), col = c("black", "red"))
matshade(ndA$age, cbind(pga, pia)*1000, 
         lty = c("solid", "21"), lend = "butt", lwd = 2, 
         alpha = c(0.2, 0.1), col = c("black", "red"))
@ %
\insfig{splint}{0.8}{Spline-by-spline interaction between age and
  duration (model \textrm{\tt gm}, black), and the interaction using a
  non-linear effect of age at entry (model \textrm{\tt im}, red), 
  corresponding to the red curves in figure \ref{fig:dur-int}.}
This is in figure \ref{fig:splint}, p. \pageref{fig:splint}.

\subsection{Evaluating interactions}
  
Here we see that the interaction effect is such that in the older ages
the length of insulin use has an increasing effect on mortality.

Even though there is no statistically significant interaction between
age and time since start of insulin, it would be illustrative to show
the RR as a function of age at insulin and age at follow-up:
<<RR-int, fig = TRUE>>= 
ndR <- transform(ndI, lex.Cst = "DM", tfI = 0)
iRR <- ci.exp(im, ctr.mat = list(ndI, ndR))
gRR <- ci.exp(gm, ctr.mat = list(ndI, ndR))
par(mar = c(3, 3, 1, 1), mgp = c(3, 1, 0)/1.6, las = 1, bty = "n")
matshade(ndI$age, cbind(gRR, iRR), lty = 1, log = "y", plot = TRUE, 
         xlab = "Age (years)", ylab = "Rate ratio: Ins vs. non-Ins", 
         col = c("black", "red"))
abline(h = 1)
@ % $
\insfig{RR-int}{0.8}{The effect of duration of insulin use at
  different ages of follow-up (and age at insulin start). Estimates
  are from the model with an interaction term using a non-linear
  effect of age at insulin start (model \textrm{\tt im}, red) and
  using a general spline interactions (model \textrm{\tt gm}, 
  black). It appears that the general interaction over-models a bit.}
This is in figure \ref{fig:RR-int}, p. \pageref{fig:RR-int}.

The advantage of the parametric modeling (be that with age at insulin
or general spline interaction) is that it is straight-forward to
\emph{test} whether we have an interaction.

\section{Separate models}

In the above we insisted on making a joint model for the 
\texttt{DM}$\rightarrow$\texttt{Dead} and the
\texttt{Ins}$\rightarrow$\texttt{Dead}
transitions, but with the complications demonstrated it would actually
have been more sensible to model the two transitions separately:
<<>>= 
dmd <- glm.Lexis(dmCs, 
                 from = "DM", to = "Dead", 
                 formula = ~ Ns(age, knots = a.kn) 
                           + sex)
ind <- glm.Lexis(dmCs, 
                 from = "Ins", to = "Dead", 
                 formula = ~ Ns(age      , knots = a.kn) 
                           + Ns(      tfI, knots = i.kn)
                           + Ns(age - tfI, knots = a.kn)
                           + sex)
ini <- ci.pred(ind, ndI)
dmi <- ci.pred(dmd, ndI)
dma <- ci.pred(dmd, ndA)
@ %
The estimated mortality rates are shown in figure \ref{fig:sep-mort}, 
p. \pageref{fig:sep-mort}, using:
<<sep-mort, fig = TRUE>>= 
par(mar = c(3, 3, 1, 1), mgp = c(3, 1, 0)/1.6, las = 1, bty = "n")
matshade(ndI$age, ini*1000, plot = TRUE, log = "y", 
         xlab = "Age (years)", ylab = "Mortality rates per 1000 PY", 
         lwd = 2, col = "red")
matshade(ndA$age, dma*1000, 
         lwd = 2, col = "black")
@ %
The estimated RRs are computed using that the estimates from the two
models are uncorrelated, and hence qualify for \texttt{ci.ratio} (this
and the previous graph
appear in figure \ref{fig:Ins-noIns}, p. \pageref{fig:Ins-noIns})
<<sep-HR, fig = TRUE>>= 
par(mar = c(3, 3, 1, 1), mgp = c(3, 1, 0)/1.6, las = 1, bty = "n")
matshade(ndI$age, ci.ratio(ini, dmi), plot = TRUE, log = "y", 
         xlab = "Age (years)", ylab = "RR insulin vs. no insulin", 
         lwd = 2, col = "red")
abline(h = 1)
@ %
\begin{figure}[tb]
\centering
\includegraphics[width = 0.49\textwidth]{flup-sep-mort}
\includegraphics[width = 0.49\textwidth]{flup-sep-HR}
\caption{\it Left panel: Mortality rates from separate models for the
  two mortality transitions; the \textrm{\tt
    DM}$\rightarrow$\textrm{\tt Dead} transition modeled by age alone;
  \textrm{\tt Ins}$\rightarrow$\textrm{\tt Dead} transition modeled
  with spline effects of current age, time since insulin and and age
  at insulin. \newline Right panel: Mortality HR of insulin vs. no insulin.}
\label{fig:Ins-noIns}
\end{figure}

\chapter{More states}

\section{Subdividing states}

It may be of interest to subdivide the states following the
intermediate event according to whether the event has occurred or
not. This will enable us to address the question of the fraction of
the patients that ever go on insulin.

This is done by the argument \texttt{split.states = TRUE}.
<<>>= 
dmCs <- cutLexis(data = dmS2, 
                  cut = dmS2$doins, 
            timescale = "per", 
            new.state = "Ins", 
            new.scale = "tfI", 
         split.states = TRUE)
summary(dmCs)
@ %
We can illustrate the numbers and the transitions (figure
\ref{fig:box4}, p. \pageref{fig:box4})
<<box4, fig = TRUE, height = 5, width = 7>>= 
boxes(dmCs, boxpos = list(x = c(15, 15, 85, 85), 
                          y = c(85, 15, 85, 15)), 
      scale.R = 1000, show.BE = TRUE)
@ % $
\insfig{box4}{0.7}{Transitions between 4 states: the numbers \emph{in}
  the boxes are person-years (middle), and below the number of persons
  who start, respectively end their follow-up in each of the states.}

Note that it is only the mortality rates that we have been modeling, 
namely the transitions \texttt{DM}$\rightarrow$\texttt{Dead}
and \texttt{Ins}$\rightarrow$\texttt{Dead(Ins)}.
If we were to model the cumulative risk of using insulin we would also
need a model for the DM$\rightarrow$Ins
transition. Subsequent to that we would then compute the probability
of being in each state conditional on suitable starting
conditions. With models where transition rates depend on several time
scales this is not a trivial task. This is treated in more detail in the
vignette on \texttt{simLexis}.

\section{Multiple intermediate events}

We may be interested in starting either insulin or OAD (oral
anti-diabetic drugs), thus giving rise to more states and more
time scales. This can be accomplished by the \texttt{mcutLexis}
function, that generalizes \texttt{cutLexis}:
<<>>= 
dmM <- mcutLexis(dmL, 
           timescale = "per", 
                  wh = c("doins", "dooad"), 
          new.states = c("Ins", "OAD"), 
          new.scales = c("tfI", "tfO"), 
        ties.resolve = TRUE)
summary(dmM, t = T)
@
We see that we now have two time scales defined as entry since into
states.
<<>>= 
wh <- c(subset(dmM, lex.Cst == "Ins-OAD")$lex.id[1:2], 
        subset(dmM, lex.Cst == "OAD-Ins")$lex.id[1:2])
subset(dmM, lex.id %in% wh)
@ %
We can also illustrate the transitions to the different states, as in
figure \ref{fig:mbox}:
<<mbox, fig = TRUE, height = 10, width = 10>>= 
boxes(dmM, boxpos = list(x = c(15, 80, 40, 40, 85, 85), 
                         y = c(50, 50, 90, 10, 90, 10)), 
           scale.R = 1000, show.BE = TRUE)
@ %
\insfig{mbox}{1.0}{Boxes for the \textrm{\tt dmM} object. The numbers
  \emph{in} the boxes are person-years (middle), and below the number
  of persons who start, respectively end their follow-up in each of
  the states.}  
We may not be interested in whether persons were prescribed insulin
before OAD or vice versa, in which case we would combine the levels
with both insulin and OAD to one, regardless of order (figure
\ref{fig:mboxr}):
<<mboxr, fig = TRUE, height = 10, width = 10>>= 
summary(dmMr <- Relevel(dmM, list('OAD+Ins' = 5:6), first = FALSE))
boxes(dmMr, boxpos = list(x = c(15, 50, 15, 85, 85), 
                          y = c(85, 50, 15, 85, 15)), 
            scale.R = 1000, show.BE = TRUE)
@ %
\insfig{mboxr}{1.0}{Boxes for the \textrm{\tt dmMr} object with
  collapsed states. The numbers \emph{in} the boxes are person-years
  (middle), and below the number of persons who start, respectively
  end their follow-up in each of the states.}

Diagrams as those in figures
\ref{fig:mbox} and
\ref{fig:mboxr} gives an overview of the possible transitions, 
which states it might be relevant to collapse, and which transitions
to model and how.

The actual modeling of the transition rates is straightforward;
split the data along some timescale and then use \texttt{glm.Lexis} or
\texttt{gam.Lexis}, where it is possible to select the transitions
modeled. This is also possible with the \texttt{coxph.Lexis}
function, but it requires that a single time scale be selected as the
baseline time scale, and the effect of this will not be accessible.

\chapter{\texttt{Lexis} functions}

\hypertarget{lexfun}{The \texttt{Lexis} machinery} has evolved over
  time since it was first introduced in a workable version in
  \texttt{Epi\_1.0.5} in August 2008.

Over the years there have been additions of tools for handling
multistate data. Here is a list of the current functions relating to
\texttt{Lexis} objects with a very brief description; it does not
replace the documentation. Unless otherwise stated, functions named
\texttt{something.Lexis} (with a ``\texttt{.}'')  are S3 methods for
\texttt{Lexis} objects, so you can skip the ``\texttt{.Lexis}'' in
daily use.

\setlist{noitemsep}
\begin{description}
  
\item[Define]\ \\
\begin{description}
\item[\texttt{Lexis}] defines a \texttt{Lexis} object
\end{description}

\item[Cut and split]\ \\
\begin{description}
\item[\texttt{cutLexis}] cut follow-up at intermediate event
\item[\texttt{mcutLexis}] cut follow-up at multiple intermediate
  events, keeping track of history
\item[\texttt{rcutLexis}] cut follow-up at intermediate, possibly
  recurring, events, only recording the current state
\item[\texttt{countLexis}] cut follow-up at intermediate event time and count
  the no. events so far
\item[\texttt{splitLexis}] split follow up along a time scale
\item[\texttt{splitMulti}] split follow up along a time scale --- from
  the \texttt{popEpi} package, faster and has simpler syntax than
  \texttt{splitLexis}
\item[\texttt{addCov.Lexis}] add clinical measurements at a given date to a
  \texttt{Lexis} object
\item[\texttt{addDrug.Lexis}] add drug exposures to a \texttt{Lexis} object
\end{description}

\item[Boxes and plots]\ \\
\begin{description}
\item[\texttt{boxes.Lexis}] draw a diagram of states and transitions
\item[\texttt{legendbox}] draw a box explaining the numbers output by \texttt{boxes.Lexis} 
\item[\texttt{plot.Lexis}] draw a standard Lexis diagram
\item[\texttt{points.Lexis}] add points to a Lexis diagram
\item[\texttt{lines.Lexis}] add lines to a Lexis diagram
\item[\texttt{PY.ann.Lexis}] annotate life lines in a Lexis diagram
\end{description}

\item[Summarize and query]\ \\  
\begin{description}
\item[\texttt{summary.Lexis}] overview of transitions, risk time etc.
\item[\texttt{levels.Lexis}] what are the states in the \texttt{Lexis} object
\item[\texttt{nid.Lexis}] number of persons in the \texttt{Lexis}
  object --- how many unique values of \texttt{lex.id} are present
\item[\texttt{entry}] entry time
\item[\texttt{exit}] exit time
\item[\texttt{status}] status at entry or exit
\item[\texttt{timeBand}] factor of time bands
\item[\texttt{timeScales}] what time scales are in the \texttt{Lexis} object
\item[\texttt{timeSince}] what time scales are defined as time since a given state
\item[\texttt{breaks}] what breaks are currently defined
\item[\texttt{absorbing}] what are the absorbing states
\item[\texttt{transient}] what are the transient states
\item[\texttt{preceding}, \texttt{before}] which states precede this
\item[\texttt{succeeding}, \texttt{after}] which states can follow this
\item[\texttt{tmat.Lexis}] transition matrix for the \texttt{Lexis} object
\end{description}

\item[Manipulate]\ \\
\begin{description}
\item[\texttt{subset.Lexis}, \texttt{[}] subset of a \texttt{Lexis} object
\item[\texttt{merge.Lexis}] merges a \texttt{Lexis} objects with a
  \texttt{data.frame}
\item[\texttt{cbind.Lexis}] bind a \texttt{data.frame} to a \texttt{Lexis} object
\item[\texttt{rbind.Lexis}] put two \texttt{Lexis} objects head-to-foot
\item[\texttt{transform.Lexis}] transform and add variables
\item[\texttt{tsNA20}] turn \texttt{NA}s to 0s for time scales
\item[\texttt{factorize.Lexis}] turn \texttt{lex.Cst} and
  \texttt{lex.Xst} into factors with levels equal to the actually
  occurring values in both
\item[\texttt{Relevel.Lexis}] reorder and/or combine states
\item[\texttt{rm.tr}] remove transitions from a \texttt{Lexis} object
\item[\texttt{bootLexis}] bootstrap sample of \emph{persons}
  (\texttt{lex.id}) in the \texttt{Lexis} object
\item[\texttt{unLexis}] remove \texttt{Lexis} attributes from a
  \texttt{Lexis} object
\end{description}

\item[Simulate]\ \\  
\begin{description}  
\item[\texttt{simLexis}] simulate a \texttt{Lexis} object from
  specified transition rate models
\item[\texttt{nState}, \texttt{pState}] count state occupancy from a
  simulated \texttt{Lexis} object 
\item[\texttt{plot.pState}, \texttt{lines.pState}] plot state occupancy from a
  \texttt{pState} object 
\end{description}

\item[Stack]\ \\
\begin{description}
\item[\texttt{stack.Lexis}] make a stacked object for simultaneous
  analysis of transitions --- returns a \texttt{stacked.Lexis} object
\item[\texttt{subset.stacked.Lexis}] subsets of a \texttt{stacked.Lexis} object 
\item[\texttt{transform.stacked.Lexis}] transform  a \texttt{stacked.Lexis} object
\end{description}

\item[Interface to other packages]\ \\
\begin{description}
\item[\texttt{msdata.Lexis}] interface to \texttt{mstate} package
\item[\texttt{etm.Lexis}] interface to \texttt{etm} package
\item[\texttt{crr.Lexis}] interface to \texttt{cmprsk} package 
\end{description}

\item[Statistical models] --- these are \emph{not} S3 methods
\begin{description}  
\item[\texttt{glm.Lexis}] fit a \texttt{glm} model using the
  \texttt{poisreg} family to (hopefully) time split data
\item[\texttt{gam.Lexis}] fit a \texttt{gam} model (from the
  \texttt{mgcv} package) using the \texttt{poisreg} family to
  (hopefully) time split data
\item[\texttt{coxph.Lexis}] fit a Cox model to follow-up in a
  \texttt{Lexis} object
\end{description}

\end{description}

<<echo = FALSE>>=
ende <- Sys.time()
cat("  Start time:", format(anfang, "%F, %T"), "\n")
cat("    End time:", format(  ende, "%F, %T"), "\n")
cat("Elapsed time:", round(difftime(ende, anfang, units = "mins"), 2), "minutes\n")
@ %

\renewcommand{\bibname}{References}

\bibliographystyle{plain}
\begin{thebibliography}{1}

\bibitem{Carstensen.2011a}
B~Carstensen and M~Plummer.
\newblock Using {L}exis objects for multi-state models in {R}.
\newblock {\em Journal of Statistical Software}, 38(6):1--18, 1 2011.

\bibitem{Iacobelli.2013}
S~Iacobelli and B~Carstensen.
\newblock {Multiple time scales in multi-state models}.
\newblock {\em Stat Med}, 32(30):5315--5327, Dec 2013.

\bibitem{Plummer.2011}
M~Plummer and B~Carstensen.
\newblock Lexis: An {R} class for epidemiological studies with long-term
  follow-up.
\newblock {\em Journal of Statistical Software}, 38(5):1--12, 1 2011.

\end{thebibliography}

\addcontentsline{toc}{chapter}{\bibname}

\end{document}

%\VignetteIndexEntry{Years of life lost (YLL)}
\SweaveOpts{results=verbatim, keep.source=TRUE, include=FALSE, eps=FALSE}
\documentclass[a4paper, twoside, 12pt]{report}

\newcommand{\Title}{Years of Life Lost (YLL)
  to disease:\\Diabetes in DK as example}
\newcommand{\Tit}{YLL}
\newcommand{\Version}{2}
\newcommand{\Dates}{June 2024}
\newcommand{\Where}{SDC}
\newcommand{\Homepage}{\url{https://bendixcarstensen.com/Epi/}}
\newcommand{\Faculty}{\begin{tabular}{rl}
Bendix Carstensen
  & Steno Diabetes Center Copenhagen, Herlev, Denmark\\
  & {\small \& Department of Biostatistics,
               University of Copenhagen} \\
  & \texttt{b@bxc.dk}\\
  & \url{https://BendixCarstensen.com/} \\[1em]
                      \end{tabular}}

\input{topreport}
\renewcommand{\rwpre}{./05yll}

\chapter{Technicalities and theory}

\section{Technicalities}
First we set some graphics parameters for convenience and load the
packages needed:
<<>>=
options(width = 90,
        show.signif.stars = FALSE,
        SweaveHooks=list(fig = function()
                         par(mar = c(3, 3, 1, 1),
                             mgp = c(3, 1, 0) / 1.6,
                             las = 1,
                            lend = "butt",
                             bty = "n")))
library(Epi)
library(popEpi)
library(survival)
clear()
@
% must be after clear() because 'anfang' is used at the end
<<echo = FALSE, results = hide>>=
anfang <- Sys.time()
cat("Start time:", format(anfang, "%F, %T"), "\n")
@ %
<<echo = FALSE>>=
vers <-
data.frame(R = substr(R.version.string, 11, 15),
         Epi = as.character(packageVersion(   "Epi")),
      popEpi = as.character(packageVersion("popEpi")))
names(vers) <- paste(" ", names(vers))
print(vers, row.names = FALSE)
@ %

\section{About this vignette}

This vignette for the \texttt{Epi} package describes the probabilistic
and demographic background for and technical implementation of the
\texttt{erl} and \texttt{yll} functions that computes the expected
residual life time and years of life lost in an illness-death model.

\section{Years of life lost (YLL)}

\ldots to diabetes or any other disease for that matter.

The general concept in calculation of ``years lost to\ldots'' is the
comparison of the expected lifetime between two groups of persons; one
with and one without disease (in this example DM). The expected
lifetime is the area under the survival curve, so basically the
exercise requires that two survival curves that are deemed relevant be
available.

The years of life lost is therefore just the area between the survival
curves for those ``Well'', $S_W(t)$, and for those ``Diseased'',
$S_D(t)$:
\[
  \YLL = \int_0^\infty S_W(t) - S_D(t) \dif t
\]
The time $t$ could of course be age, but it could also be ``time after
age 50'' and the survival curves compared would then be survival
curves \emph{conditional} on survival till age 50, and the YLL would
be the years of life lost for a 50 year old person with diabetes
relative to a 50 year old person without diabetes.

If we are referring to the expected lifetime we will more precisely use
the label expected residual lifetime, ERL.

\section{Constructing the survival curves}

YLL can be computed in two different ways, depending on the way the
survival curve and hence the expected lifetime of a person
\emph{without} diabetes is computed:
\begin{itemize}
\item Assume that the ``Well'' persons are \emph{immune} to disease
  --- using only the non-DM mortality rates throughout for calculation
  of expected life time.
\item Assume that the ``Well'' persons \emph{can} acquire the disease and
  thereby see an increased mortality, thus involving all three rates
  shown in figure \ref{fig:states}.
\end{itemize}
The former gives a higher YLL because the comparison is to persons
assumed immune to DM (and yet with the same mortality as non-immune
prior to diagnosis), the latter gives a more realistic picture of the
comparison of group of persons with and without diabetes at a given
age that can be interpreted in the real world.

The differences can be illustrated by figure \ref{fig:states}; the
immune approach corresponds to an assumption of $\lambda(t)=0$ in the
calculation of the survival curve for a person in the ``Well'' state.

Calculation of the survival of a diseased person already in the ``DM''
state is unaffected by assumptions about $\lambda$.

We can illustrate the states and transitions using \texttt{boxes}:
<<states, fig=TRUE, echo=TRUE>>=
library(Epi)
TM <- matrix(NA, 4, 4)
rownames(TM) <-
colnames(TM) <- c("Well", "DM", "Dead", "Dead(DM)")
TM[1, 2:3] <- TM[2, 4] <- 1
TM
zz <- boxes(TM, boxpos = list(x = c(20, 80, 20, 80),
                              y = c(80, 80, 20, 20)),
                wm = 1.5,
                hm = 4)
@ %
We can edit the output from \texttt{boxes} to get the proper
annotation of the transition rates:
<<states, fig=TRUE, height=4, width=7, echo=TRUE>>=
zz$Arrowtext <- c(expression(lambda(a)),
                  expression(mu[W](a)),
                  expression(mu[D][M](a,d)))
boxes.MS(zz)
@ %
\insfig{states}{0.7}{Illness-death model describing diabetes incidence
  and -mortality and functions of age and duration}

\subsection{Total mortality --- a shortcut?}

A practical crude shortcut could be to compare the ERL in the diabetic
population to the ERL for the \emph{entire} population (that is using
the total mortality ignoring diabetes status).

Note however that this approach also counts the mortality of persons
that acquired the disease earlier, thus making the comparison
population on average more ill than the population we aim at, namely
those well at a given time, which only then become more gradually ill.

How large these effects are can however be empirically explored, as we
shall do later.

\subsection{Disease duration}

In the exposition above there is no explicit provision for the effect of
disease duration, but if we were able to devise mortality rates for
any combination of age and duration, this could be taken into account.

There are however severe limitations in this as we in principle would
want to have duration effects as long as the age-effects --- in
principle for all $(a, d)$ where $d\leq A$, where $A$ is the age at
which we condition. So even if we were only to compute ERL from
age, say, 40 we would still need duration effects up to 60 years
(namely to age 100).

The incorporation of duration effects is in principle trivial from a
computational point of view, but we would be forced to entertain
models predicting duration effects way beyond what is actually
observed disease duration in any practical case.

\subsection{Computing integrals}

The practical calculations of survival curves, ERL and YLL involves
calculation of (cumulative) integrals of rates and functions of these
as we shall see below. This is easy if we have a closed form
expression of the function, so its value may be computed at any time
point --- this will be the case if we model rates by smooth parametric
functions.

Computing the (cumulative) integral of a function is done as follows:
\begin{itemize}
\item Compute the value of the function (mortality rate for example)
  at the midpoints of a sequence of narrow equidistant intervals ---
  for example one- or three month intervals of age, say.
\item Take the cumulative sum of these values multiplied by the
  interval length --- this will be a very close approximation to the
  cumulative integral evaluated at the end of each interval.
\item If the intervals are really small (like 1/100 year), the
  distinction between the value at the middle and at the end of each
  interval becomes irrelevant.
\end{itemize}
Note that in the above it is assumed that the rates are given in units
corresponding to the interval length --- or more precisely, as the
cumulative rates over the interval.

\section{Survival functions in the illness-death model}

The survival functions for persons in the ``Well'' state can be
computed under two fundamentally different scenarios, depending on
whether persons in the ``Well'' state are assumed to be immune to the
disease ($\lambda(a)=0$) or not.

\subsection{Immune approach}

In this case both survival functions for person in the two states are
the usual simple transformation of the cumulative mortality rates:
\[
 S_W(a) = \exp\left(-\int_0^a\!\!\mu_W(u) \dif u \right), \qquad
 S_D(a) = \exp\left(-\int_0^a\!\!\mu_D(u) \dif u \right)
\]

\subsubsection{Conditional survival functions}

If we want the \emph{conditional} survival functions given survival to
age $A$, say, they are just:
\[
 S_W(a|A) = S_W(a)/S_W(A), \qquad S_D(a|A) = S_D(a)/S_D(A)
\]

\subsection{Non-immune approach}

For a diseased person, the survival function in this states is the same
as above, but the survival function for a person without disease (at
age 0) is (see figure \ref{fig:states}):
\[
S(a) = \ptxt{Well}\!(a) + \ptxt{DM}\!(a)
\]
In the appendix of the paper \cite{Carstensen.2008c} is an indication
of how to compute the probability of being in any of the four states
shown in figure \ref{fig:states}, which I shall repeat here:

In terms of the rates, the probability of being in the ``Well'' box is
simply the probability of escaping both death (at a rate of $\mu_W(a)$)
and diabetes (at a rate of $\lambda(a)$):
\[
   \ptxt{Well}(a)  = \exp\left(\!-\int_0^a\!\!\mu_W(u)+\lambda(u) \right) \dif u
\]
The probability of being alive with diabetes at age $a$, is computed given that
 diabetes occurred at age $s$ ($s<a$) and then integrated over $s$ from $0$
 to $a$:
\begin{align*}
 \ptxt{DM}(a) = \int_0^a\!\! & \ptxt{survive to $s$, DM diagnosed at $s$} \\
                & \times \ptxt{survive with DM from $s$ to $a$} \dif s \\
              = \int_0^a\!\! & \lambda(s)
                           \exp\left(\!-\int_0^s\!\!\mu_W(u)+\lambda(u) \dif u \right) \\
                & \times \exp\left(\!-\int_s^a\!\!\mu_D(u) \dif u \right) \dif s
\end{align*}
Sometimes we will use a version where the mortality among diabetes
patients depend both on age $a$ and duration of diabetes, $d$,
$\mu_D(a, d)$, in which case we get:
\begin{align*}
 \ptxt{DM}(a) = \int_0^a \! & \lambda(s)
                \exp\left(-\int_0^s\!\mu_W(u)+\lambda(u) \dif u \right) \\
                & \times \exp\left(-\int_s^a\!\mu_D(u, u-s) \dif u \right) \dif s
\end{align*}
because the integration variable $u$ is the age-scale and the second
integral refers to mortality among persons diagnosed at age $s$, that
is, with duration $u-s$ at age $u$.

The option of using duration-dependent mortality rates among diseased
individuals is not implemented yet.

\subsubsection{Conditional survival functions}

Unlike the immune approach, the conditional survival function in the
more realistic case is not just a ratio of the unconditional to the
value at the conditioning age, $A$, say. This would amount to
conditioning on being merely \emph{alive} at age $A$, but what we want
is to condition on being in the ``Well'' state at age $A$.

The formulae for the conditional probabilities of being either in
``Well'' or ``DM'', given being in ``Well'' at age $A$ are basically
replicates of the unconditional, albeit with changes in integration
limits:
\begin{align*}
\ptxt{Well|Well at $A$}(a) &= \exp\left(-\int_A^a \! \mu_W(u)+\lambda(u) \right) \dif u \\
  \ptxt{DM|Well at $A$}(a) &= \int_A^a \! \lambda(s)
                               \exp\left(-\int_A^s\!\mu_W(u)+\lambda(u) \dif u \right) \\
                           & \qquad \times \exp\left(-\int_s^a\!\mu_D(u, u-s) \dif u \right) \dif s
\end{align*}
The calculation of these conditional survival functions is implemented
but not allowing for duration-dependence. Thus it is only implemented
assuming $\mu_D(a, d)=\mu_D(a)$.

\chapter{Analyses for DM in Denmark}

The rates we use as basis for the following calculations are derived
from the NDR, where we have omitted the blood-glucose criteria,
because there is compelling evidence that these have quite a low
specificity (particularly in the younger ages among women), and do
not substantially contribute to the sensitivity.

As noted above the calculations of YLL requires access to
(age-specific) rates of incidence of DM and mortality for persons with
and without DM.

\section{Modeling mortality and incidence data}

We read in the dataset of DM and population mortality and incidence, \texttt{DMepi}:
<<>>=
data(DMepi)
@ %
The dataset \texttt{DMepi} contains diabetes events, deaths and
person-years for persons without diabetes and deaths and person-years
for persons with diabetes, classified by age (\texttt{A}) and calendar
year (\texttt{P}):
<<>>=
str(DMepi)
head(DMepi)
@ %
For each combination of sex, age, period and date of birth in 1 year
age groups, we have the person-years in the ``Well'' (\texttt{Y.nD})
and the ``DM'' (\texttt{Y.DM}) states, as well as the number of deaths
from these (\texttt{D.nD}, \texttt{D.DM}) and the number of incident
diabetes cases from the ``Well'' state (\texttt{X}).

In order to compute the years of life lost to diabetes and how this
has changed over time, we fit models for the mortality and incidence
of both groups (and of course, separately for men and women). The
models we use will be age-period-cohort models \cite{Carstensen.2007a}
providing estimated mortality rates for ages 0--99 and dates
1.1.1996--1.1.2016.

First we transform the age and period variables to reflect the mean
age and period in each of the Lexis triangles. We also compute the
total number of deaths and amount of risk time, as we are going to
model the total mortality as well. Finally we restrict the dataset to
ages over 30 only:
<<>>=
DMepi <- transform(subset(DMepi, A > 30),
                   A = A + 0.5,
                   P = P + 0.5,
                 D.T = D.nD + D.DM,
                 Y.T = Y.nD + Y.DM)
head(DMepi)
@ %
With the correct age and period coding in the Lexis triangles, we fit
models for the mortalities and incidences. Note that we for
comparative purposes also fit a model for the \emph{total} mortality,
ignoring the
<<>>=
# Knots used in all models
(a.kn <- seq(40, 95, , 6))
(p.kn <- seq(1997, 2015, , 4))
(c.kn <- seq(1910, 1976, , 6))
# Check the number of events between knots
ae <- xtabs(cbind(D.nD, D.DM, X) ~ cut(A, c(30, a.kn, Inf)) + sex, data=DMepi)
ftable(addmargins(ae, 1), col.vars=3:2)
pe <- xtabs(cbind(D.nD, D.DM, X) ~ cut(P, c(1990, p.kn, Inf)) + sex, data=DMepi)
ftable(addmargins(pe, 1), col.vars=3:2)
ce <- xtabs(cbind(D.nD, D.DM, X) ~ cut(P-A, c(-Inf, c.kn, Inf)) + sex, data=DMepi)
ftable(addmargins(ce, 1), col.vars=3:2)
# Fit an APC-model for all transitions, separately for men and women
mW.m <- glm(cbind(D.nD, Y.nD) ~ -1 + Ns(    A, knots=a.kn, int=TRUE) +
                                     Ns(P    , knots=p.kn, ref=2005) +
                                     Ns(P - A, knots=c.kn, ref=1950),
            family = poisreg,
              data = subset(DMepi, sex=="M"))
mD.m <- update(mW.m, cbind(D.DM, Y.DM) ~ .)
mT.m <- update(mW.m, cbind(D.T , Y.T ) ~ .)
lW.m <- update(mW.m, cbind(X   , Y.nD) ~ .)
# Model for women
mW.f <- update(mW.m, data = subset(DMepi, sex == "F"))
mD.f <- update(mD.m, data = subset(DMepi, sex == "F"))
mT.f <- update(mT.m, data = subset(DMepi, sex == "F"))
lW.f <- update(lW.m, data = subset(DMepi, sex == "F"))
@ %

\section{Residual life time and years lost to DM}

We now collect the estimated years of life lost classified by method
(immunity assumption or not), sex, age and calendar time:
<<>>=
a.ref <- 30:90
p.ref <- 1996:2016
aYLL <- NArray(list(type = c("Imm", "Tot", "Sus"),
                       sex = levels(DMepi$sex),
                       age = a.ref,
                      date = p.ref))
str(aYLL)
system.time(
for(ip in p.ref)
   {
   nd <- data.frame(A = seq(30, 90, 0.2)+0.1,
                     P = ip,
                  Y.nD = 1,
                  Y.DM = 1,
                  Y.T  = 1)
   muW.m <- ci.pred(mW.m, nd)[, 1]
   muD.m <- ci.pred(mD.m, nd)[, 1]
   muT.m <- ci.pred(mT.m, nd)[, 1]
   lam.m <- ci.pred(lW.m, nd)[, 1]
   muW.f <- ci.pred(mW.f, nd)[, 1]
   muD.f <- ci.pred(mD.f, nd)[, 1]
   muT.f <- ci.pred(mT.f, nd)[, 1]
   lam.f <- ci.pred(lW.f, nd)[, 1]
   aYLL["Imm", "M", , paste(ip)] <- yll(int=0.2, muW.m, muD.m, lam=NULL,
                                      A=a.ref, age.in=30, note=FALSE)[-1]
   aYLL["Imm", "F", , paste(ip)] <- yll(int=0.2, muW.f, muD.f, lam=NULL,
                                      A=a.ref, age.in=30, note=FALSE)[-1]
   aYLL["Tot", "M", , paste(ip)] <- yll(int=0.2, muT.m, muD.m, lam=NULL,
                                      A=a.ref, age.in=30, note=FALSE)[-1]
   aYLL["Tot", "F", , paste(ip)] <- yll(int=0.2, muT.f, muD.f, lam=NULL,
                                      A=a.ref, age.in=30, note=FALSE)[-1]
   aYLL["Sus", "M", , paste(ip)] <- yll(int=0.2, muW.m, muD.m, lam=lam.m,
                                      A=a.ref, age.in=30, note=FALSE)[-1]
   aYLL["Sus", "F", , paste(ip)] <- yll(int=0.2, muW.f, muD.f, lam=lam.f,
                                      A=a.ref, age.in=30, note=FALSE)[-1]
   })
round(ftable(aYLL[, , seq(1, 61, 10), ], col.vars=c(3, 2)), 1)
@ %
We now have the relevant points for the graph showing YLL to diabetes
for men and women by age, and calendar year, both under the immunity
and susceptibility models for the calculation of YLL.
<<imm, fig=TRUE, width=8, height=5>>=
plyll <- function(wh, xtxt){
par(mfrow = c(1, 2),
      mar = c(3, 3, 1, 1),
      mgp = c(3, 1, 0) / 1.6,
      bty = "n",
      las = 1)
matplot(a.ref, aYLL[wh, "M", , ],
         type="l", lty=1, col="blue", lwd=1:2,
         ylim=c(0, 12), xlab="Age",
         ylab=paste0("Years lost to DM", xtxt),
         yaxs="i")
abline(v=50, h=1:11, col=gray(0.7))
text(90, 11.5, "Men", col="blue", adj=1)
text(40, aYLL[wh, "M", "40", "1996"], "1996", adj=c(0, 0), col="blue")
text(43, aYLL[wh, "M", "44", "2016"], "2016", adj=c(1, 1), col="blue")

matplot(a.ref, aYLL[wh, "F", , ],
         type="l", lty=1, col="red", lwd=1:2,
         ylim=c(0, 12), xlab="Age",
         ylab=paste0("Years lost to DM", xtxt),
         yaxs="i")
abline(v=50, h=1:11, col=gray(0.7))
text(90, 11.5, "Women", col="red", adj=1)
text(40, aYLL[wh, "F", "40", "1996"], "1996", adj=c(0, 0), col="red")
text(43, aYLL[wh, "F", "44", "2016"], "2016", adj=c(1, 1), col="red")
}
plyll("Imm", " - immunity assumption")
@ %
<<tot, fig=TRUE, width=8, height=5>>=
plyll("Tot", " - total mortality refernce")
@ %
<<sus, fig=TRUE, width=8, height=5>>=
plyll("Sus", " - susceptibility assumed")
@ %
\begin{figure}[h]
  \centering
  \includegraphics[width=\textwidth]{05yll-imm}
  \caption{Years of life lost to DM: the difference in expected
    residual life time at different ages between persons with and
    without diabetes, assuming the persons without diabetes at a given
    age remain free from diabetes (immunity assumption --- not
    reasonable). The lines refer to date of evaluation; the top lines
    refer to 1996-1-1 the bottom ones to 2016-1-1. Blue curves are
    men, red women.}
  \label{fig:imm}
\end{figure}

\begin{figure}[h]
  \centering
  \includegraphics[width=\textwidth]{05yll-sus}
  \caption{Years of life lost to DM: the difference in expected
    residual life time at different ages between persons with and
    without diabetes, allowing the persons without diabetes at a given
    age to contract diabetes and thus be subject to higher
    mortality. The lines refer to date of evaluation; the top lines
    refer to 1996-1-1 the bottom ones to 2016-1-1. Blue curves are
    men, red women.}
  \label{fig:sus}
\end{figure}

\begin{figure}[h]
  \centering
  \includegraphics[width=\textwidth]{05yll-tot}
  \caption{Years of life lost to DM: the difference in expected
    residual life time at different ages between persons with and
    without diabetes. Allowance for susceptibility is approximated by
    using the total population mortality instead of non-DM
    mortality. The lines refer to date of evaluation; the top lines
    refer to 1996-1-1 the bottom ones to 2016-1-1. Blue curves are
    men, red women.}
  \label{fig:tot}
\end{figure}

From figure \ref{fig:sus} we see that for men aged 50 the years lost to
diabetes has decreased from 6.5 to 4.5 years
and for women from 4 to 4 years; so a greater improvement for women.

<<echo = FALSE>>=
ende <- Sys.time()
cat("  Start time:", format(anfang, "%F, %T"),
  "\n    End time:", format(  ende, "%F, %T"),
  "\nElapsed time:", round(difftime(ende, anfang, units = "mins"), 2), "minutes\n")
@ %

\bibliographystyle{plain}

\begin{thebibliography}{1}

\bibitem{Carstensen.2007a}
B~Carstensen.
\newblock Age-{P}eriod-{C}ohort models for the {L}exis diagram.
\newblock {\em Statistics in Medicine}, 26(15):3018--3045, 2007.

\bibitem{Carstensen.2008c}
B~Carstensen, JK~Kristensen, P~Ottosen, and K~Borch-Johnsen.
\newblock The {D}anish {N}ational {D}iabetes {R}egister: {T}rends in incidence,
  prevalence and mortality.
\newblock {\em Diabetologia}, 51:2187--2196, 2008.

\end{thebibliography}

\addcontentsline{toc}{chapter}{References}

\end{document}

%\VignetteIndexEntry{Competing risks with Lexis, parametric rates and simulation based confidence intervals}
\SweaveOpts{results=verbatim,keep.source=TRUE,include=FALSE,eps=FALSE}
\documentclass[a4paper,dvipsnames,twoside,12pt]{report}

% ----------------------------------------------------------------------
% General information for the title page and the page headings
\newcommand{\Title}{Competing risks with\\ \texttt{Lexis}, parametric
  rates and\\ simulation based confidence intervals}
\newcommand{\Tit}{CmpRskParSim}
\newcommand{\Version}{Version 5}
\newcommand{\Dates}{April 2024}
\newcommand{\Where}{SDCC}
\newcommand{\Homepage}{\url{http://bendixcarstensen.com/}}
\newcommand{\Faculty}{\begin{tabular}{rl}
Bendix Carstensen
  & Steno Diabetes Center Copenhagen, Herlev, Denmark\\
  & {\small \& Department of Biostatistics,
               University of Copenhagen} \\
  & \href{mailto:b@bxc.dk}{\tt b@bxc.dk} \\
  & \url{http://BendixCarstensen.com} \\[1em]
                      \end{tabular}}

\input{topreport}
\renewcommand{\rwpre}{./crisk}
<<echo=FALSE>>=
options(width = 90,
        SweaveHooks = list(fig = function()
        par(mar = c(3,3,1,1),
            mgp = c(3,1,0)/1.6,
            las = 1,
            bty = "n",
           lend = "butt")))
@ %
<<echo = FALSE>>=
anfang <- Sys.time()
cat("Start time:", format(anfang, "%F, %T"), "\n")
@ %

\chapter{Competing risks}

\section{Concepts}

The concept of competing risks is one where persons in a given state,
``alive'', say, are subject to a number of different causes of death,
``cause1'', ``cause2'' etc. Causes of death are required to be
exhaustive and mutually exclusive. That is, you will eventually die
from one of the designated causes, and you can only die from one.

The \emph{cause-specific} rate for cause $c$ is defined as:
\[
   \lambda_c(t) = \ptxt{death from cause $c$ in $(t,t+h]\,|\,$
                        alive at $t$} / h
\]
\ldots formally, the limit of this quantity as $h\rightarrow 0$.

The observed data will be a survival time and an exit status which is
either ``censored alive'' or one of the causes. In situations where
the causes are not causes of death but other events, it is implicit
that we only consider the first occurrence of an event from the state
``alive'', and ignore whatever occurs after that.

\subsection{Cause specific rates and likelihood}

The likelihood for observations from a competing risk scenario is a
function of the cause-specific transition rates, and is a
\emph{product} of the likelihoods that would emerge if we considered
each cause as being the only possible event. Thus analysis is in
principle straight forward: estimate a model for each of the
cause-specific rates; these will together form a complete model for
the competing risks problem.

If the cause-specific rates are all we want to assess then we are done.

\subsection{Survival and cumulative risks}

In addition to the rates we might however also be interested in the
\emph{survival} probability and the \emph{cumulative risks} of each
cause of death.

The survival is the probability of still being alive at a given time
after entry---a function of time since entry. The cumulative risk of
dying from cause $c$ is the probability of having died from cause $c$
as a function of time since entry.

This means that a time of entry is required for the calculations these
quantities.

\subsection{Sojourn times}

The \emph{sojourn time} for cause $c$ is the time spent in the ``cause
$c$'' state before a given time, $t$, say. This is also called the
expected lifetime lost to cause $c$, \emph{truncated} at the time
$t$. For the state ``alive'' it will be the expected time lived before
$t$.

\subsection{The time scale}

The cause specific rates will be functions of come covariates, notably
a time scale, be that age or time since entry to the study or even
calendar time. But the cumulative risks are probabilities that refer
to time \emph{since} some origin. Thus cumulative risks (and survival)
are only meaningful in relation to a time that begins at 0. Though not
a formal mathematical requirement this implies that we should have
data starting at time 0.

If we were to use age as timescale for cumulative risk, we would want
data available since birth; if we only had observations where most
people entered between 20 and 40 years of age, we could mathematically
compute cumulative risk to some age, but it would be nonsense. Instead we
would compute the cumulative risk \emph{given} that a person attained
age 40, say. In that case the time scale would be $\text{age} - 40$.

\section{Rates and cumulative risks}

Each of the cumulative risks is a function of the survival function which
in turn depends on \emph{all} rates. Specifically, if the
cause-specific rates are $\lambda_c(t),\,c=1,2,\ldots$, then the
survival function (probability of being alive at time $t$) is:
\begin{equation}
     S(t) = \exp\left( \!-\!\int_0^t \sum_c \lambda_c(s) \dif s \right)
     =  \exp\left(\!-\! \sum_c \Lambda_c(t) \right)
          \label{eq:Sv}
\end{equation}
The quantities $\Lambda_c(t) = \int_0^t \lambda_c(s) \dif s$ are
called cumulative \emph{rates} (probabilists call them integrated
intensities), although they are not rates. Cumulative rates are
dimensionless, but they have no probability interpretation of any
kind.

The cumulative \emph{risk}, the probability of dying from cause $k$,
say, before time $t$, $R_k(t)$ is:
\begin{equation}
   R_k(t) = \int_0^t \!\! \lambda_k(u) S(u) \dif u
           = \int_0^t \!\! \lambda_k(u) \exp\left(\!-\!
            \sum_j \Lambda_j(u) \right) \! \dif u
          \label{eq:R}
\end{equation}
The rationale is that $\lambda_k(u) \dif u$ is the probability of
death from cause $k$ in the small interval $(u, u + \dif u)$,
\emph{conditional} on being alive at time $u$. If this is multiplied
with the probability of being alive at $u$, $S(u)$, Bayes rule tells
us that we get the \emph{marginal probability} of death from cause $k$
in the small interval $(u, u + \dif u)$, This function of $u$ is the
argument in the integral; so integration from $u=0$ to $u=t$, will
give the probability of death from cause $c$ anywhere in $(0,t)$---the
cumulative risk of cause $k$ at $t$.

Parametric models for the cause-specific rates can produce estimated
transition rates $\lambda_c$ at closely spaced intervals, and the
cumulative risks can then be estimated from these by simple numerical
integration; this is illustrated in the next chapter.

Note that at any one time every person is either alive or dead from one
of the causes, so the sum of the survival and the cumulative risks is
always 1:
\[
   1 = S(t) + R_1(t) + R_2(t) + \cdots, \quad \forall t
\]

\subsection{Confidence intervals by simulation}

But even if we from the modeling of the $\lambda_c$s may have standard
errors of $\log\big(\lambda_c(t)\big)$, the standard errors of the
$R_c$s will be analytically intractable from these.

In practice, the only viable way to get confidence intervals for the
cumulative risks, $R_c$, is therefore by calculation of a set of rates
$\lambda_c(t)$ by sampling from the posterior distribution of the
parameters in the models for $\log\big(\lambda_c(t)\big)$, and then
compute the integrals numerically for each simulated sample, according
to formulae \ref{eq:Sv} and \ref{eq:R}. This will produce a so-called
parametric bootstrap sample of the cumulative risks from which we can
derive confidence intervals

The simulation approach also allows calculation of confidence
intervals for sums of the cumulative risks, $R_1(t)+R_2(t)$, for
example, which will be needed if we want to show stacked cumulative
risks.

Finally, it will also allow calculation of standard errors of sojourn
times in each of the states ``alive'' and ``cause1'',
``cause2'',\ldots. While the latter two may not be of direct interest,
then \emph{differences} between such sojourn times between different
groups can be interpreted as years of life lost to each cause between
groups.

\subsection{Subdistribution hazard}

A common concept seen in competing risks is the subdistribution
hazard, and proportional hazards models for this (the Fine-Gray model).

Suppose we only consider all-cause mortality, $\lambda(t)$. Then the
cumulative risk of death is:
\[
    R(t) = 1 - S(t) = 1 - \exp\left( \!-\!\int_0^t \lambda(s) \dif s \right)
\]
Solving this for $\lambda$ will yield:
\[
  \lambda(t) = -\frac{\dif\log\big(1 - R(t)\big)}{\dif t}
\]
In the case of multiple causes of death we can define the
\emph{subdistribution hazard} for cause $c$ by using the same
transformation of the cumulative risk for cause $c$:
\[
  \tilde\lambda_c(t) = -\frac{\dif\log\big(1 - R_c(t)\big)}{\dif t}
\]
or, for the cause-specific risk:
\[
    R_c(t) = 1 - \exp\left( \!-\!\int_0^t \tilde\lambda_c(s) \dif s \right)
\]
Thus, the subdistribution hazard $\tilde\lambda_c(t)$ is a function
which, when subjected to the (hazard$\rightarrow$risk) function from
the all-cause mortality case, yields the cause-specific risk.

The Fine-Gray model is a model for the subdistribution hazard
$\tilde\lambda_c$. It is only a model for \emph{one} cause-specific
hazard. Of course it can be applied to all available causes in turn,
but the sum of the cumulative risks derived from the models may exceed
1\ldots

Unlike a cause-specific hazard, which can depend on multiple time
scales, the subdistribution hazard can only depend on one, since it
requires an origin---just like cumulative risks.

But the interpretation of a subdistribution hazard is difficult.  I
have yet to see one that goes beyond the mathematical formalism
above. Therefore the subdistribution is not included in this vignette.

\chapter{Example data}

\section{A \texttt{Lexis} object}

As an illustrative data example we use the (fake) diabetes register
data; we set up the Lexis object, an then cut the follow-up time at
dates of OAD and Ins:
<<>>=
library(Epi)
data(DMlate)
Ldm <- Lexis(entry = list(per = dodm,
                          age = dodm-dobth,
                          tfd = 0),
              exit = list(per = dox),
       exit.status = factor(!is.na(dodth), labels = c("DM", "Dead")),
              data = DMlate[sample(1:nrow(DMlate), 1000),])
summary(Ldm, t = T)
set.seed(1952)
Mdm <- mcutLexis(Ldm,
                  wh = c('dooad','doins'),
          new.states = c('OAD','Ins'),
          seq.states = FALSE,
                ties = TRUE)
summary(Mdm)
@
We initially split the FU before drug inception in intervals of
1/12 year, creating a \texttt{Lexis} object for a competing risks
situation with three possible event types:
<<>>=
Sdm <- splitLexis(factorize(subset(Mdm,
                                   lex.Cst == "DM")),
                  time.scale = "tfd",
                      breaks = seq(0, 20, 1/12))
summary(Sdm)
@ %
We can illustrate the follow-up in the full data frame and in the
restricted data frame
<<boxes5, fig = TRUE, width = 8>>=
boxes(Mdm, boxpos = list(x = c(15, 50, 15, 85, 85),
                         y = c(85, 50, 15, 85, 15)),
          scale.R = 100,
          show.BE = TRUE)
@ %
\insfig{boxes5}{0.8}{The transitions in the multistate model, where
    follow-up is extended also after beginning of first drug
    exposure. Rates in brackets are per 100 PY.}%
<<boxes4, fig = TRUE, width = 8>>=
boxes(Relevel(Sdm, c(1, 4, 2, 3)),
      boxpos  = list(x = c(15, 85, 75, 15),
                     y = c(85, 85, 30, 15)),
      scale.R = 100,
      show.BE = TRUE )
@ %
\insfig{boxes4}{0.8}{The transitions in the competing risks model,
  where follow-up is stopped at drug exposure. By that token only the
  \texttt{DM} state has person-years; a characteristic of a competing risks
  situation.}

\section{Models for rates}

Now that we have set up a dataset with three competing events, we can
model the cause-specific rates separately by time from diagnosis as
the only underlying time scale.

This is done by Poisson-regression on the time-split data set; since
the dataset is in \texttt{Lexis} format we can use the convenience
wrapper \texttt{gam.Lexis} to model rates a smooth functions of time
(\texttt{tfd}).  Note that we only need to specify the \texttt{to=}
argument because there is only one possible \texttt{from} for each
\texttt{to} (incidentally the same for all \texttt{to} states, namely
\texttt{DM}):
<<>>=
mD <- gam.Lexis(Sdm, ~ s(tfd, k = 5), to = 'Dead')
mO <- gam.Lexis(Sdm, ~ s(tfd, k = 5), to = 'OAD' )
mI <- gam.Lexis(Sdm, ~ s(tfd, k = 5), to = 'Ins' )
@ %
We see that \texttt{gam.Lexis} (just like \texttt{glm.Lexis} would)
tells us what transitions are modeled.

With these models fitted we can compute the rates, cumulative rates
and the cumulative risks and sojourn times in states using the usual
formulae. First we compute the rates in intervals of length 1/100
years. Note that these models only have time since diagnosis as
covariates, so they are the counterpart of Nelson-Aalen estimates,
albeit in a biologically more meaningful guise.

The points where we compute the predicted rates are midpoints of
intervals of length 1/100 year. These points are unrelated to the
follow-up intervals in which we split the data for analysis---they
were 1 month intervals, here we use 1/10 year (about 1 month, in real
life a small interval should be used, say 1/50 year):
<<>>=
nd <- data.frame(tfd = seq(0, 10, 1/10))
rownames(nd) <- nd$tfd
str(nd)
@ %
With this we can show the rates as a function of the time since entry
(diagnosis of diabetes):
<<rates, fig = TRUE, width = 8>>=
matshade(nd$tfd, cbind(ci.pred(mD, nd),
                       ci.pred(mI, nd),
                       ci.pred(mO, nd))*1000,
         col = c("black", "red", "blue"), log = "y", lwd = 3, plot = TRUE,
         xlab = "Time since DM diagnosis (years)",
         ylab = "Rates per 1000 PY", ylim = c(0.05,500), yaxt = "n")
axis(side = 2, at = ll<-outer(c(1,2,5),-2:3,function(x,y) x*10^y),
               labels = formatC(ll,digits = 4), las = 1)
axis(side = 2, at = ll<-outer(c(1.5,2:9),-2:3,function(x,y) x*10^y),
               labels = NA, tcl = -0.3)
text(0, 0.5*0.6^c(1,2,0),
     c("Dead","Ins","OAD"),
     col = c("black","red","blue"), adj = 0)
@ %
\insfig{rates}{0.8}{Estimated rates from the \textrm{\tt DM} state,
  estimates are from \textrm{\tt gam} models fitted to data split in 1
  month intervals (1/12 year, that is). Rates of \textrm{\tt OAD} is
  in the vicinity of 0.1/year, and mortality about half of this. Rates
  of insulin start among persons on no other drug are beginning high,
  then decreasing with a nadir at about 4 years and then increase to a
  peak at 8 years.}

Note that the graph in figure \ref{fig:rates} is not normally shown in
analyses of competing risks; the competing cause-specific \emph{rates}
are hardly ever shown. I suspect that this is frequently because they
are often modeled by a Cox model and so are buried in the model and
hard to get at.

Since we will be integrating the rates, it would be relevant to show
the rates on a linear scale instead, de-emphasizing the very small
fluctuations of the \texttt{Ins} rates:
<<rates-l, fig = TRUE, width = 8>>=
matshade(nd$tfd, cbind(ci.pred(mD, nd),
                       ci.pred(mI, nd),
                       ci.pred(mO, nd))*1000,
         col = c("black", "red", "blue"), lwd = 3, plot = TRUE,
         xlab = "Time since DM diagnosis (years)",
         ylab = "Rates per 1000 PY", ylim = c(0,500), yaxs = "i")
text(8, 500 - c(2, 3, 1) * 20,
     c("Dead","Ins","OAD"),
     col = c("black","red","blue"), adj = 0)
@ %
\insfig{rates-l}{0.8}{Estimated rates from the \textrm{\tt DM} state,
  estimates are from \textrm{\tt gam} models fitted to data split in 1
  month intervals (1/12 year, that is). Rates of \textrm{\tt OAD} is
  in the vicinity of 0.1/year, and mortality about half of this. .}

\section{Cumulative rates and risks}

For the calculation of the cumulative rates and state probabilities,
we need just the estimated rates (without CIs). The formulae
\ref{eq:Sv} and \ref{eq:R} on page \pageref{eq:R} are transformed to
\R-code; starting with the rates, $\lambda_\text{D}$ as \texttt{lD}
etc:
<<>>=
# utility function that calculates the midpoints between sucessive
# values in a vector
mp <- function(x) x[-1] - diff(x) / 2
#
int <- 1/50
# rates at midpoints of intervals
lD <- mp(ci.pred(mD, nd)[,1])
lI <- mp(ci.pred(mI, nd)[,1])
lO <- mp(ci.pred(mO, nd)[,1])
#
# cumulative rates and survival function at right border of the intervals
LD <- cumsum(lD) * int
LI <- cumsum(lI) * int
LO <- cumsum(lO) * int
# survival function, formula (1.1)
Sv <- exp(- LD - LI - LO )
#
# when integrating to get the cumulative *risks* we use the average
# of the survival function at the two endpoints
# (adding 1 as the first), formula (1.2)
Sv <- c(1, Sv)
rD <- c(0, cumsum(lD * mp(Sv)) * int)
rI <- c(0, cumsum(lI * mp(Sv)) * int)
rO <- c(0, cumsum(lO * mp(Sv)) * int)
@ %
Now we have the cumulative risks for the three causes and the
survival, computed at the end of each of the intervals. At any time
point the sum of the 3 cumulative risks and the survival should be 1:
<<>>=
summary(rD + rI + rO + Sv)
oo <- options(digits = 20)
cbind(summary(Sv + rD + rI + rO))
options(oo)
@ %
\ldots and bar a rounding error, they are.

We can then plot the 3 cumulative risk functions stacked together
using \texttt{mat2pol} (\texttt{mat}rix to \texttt{pol}ygons):
<<stack, fig = TRUE, width = 8>>=
zz <- mat2pol(cbind(rD, rI, rO, Sv), x = nd$tfd,
              xlim = c(0,10), xaxs = "i", yaxs = "i", las = 1,
              xlab = "Time since DM diagnosis (years)",
              ylab = "Probability",
               col =  c("black","red","blue","forestgreen"))
text(9, mp(zz["9", ]), c("Dead", "Ins", "OAD"," DM"), col = "white")
box(col = "white", lwd = 3)
@ % $
\insfig{stack}{0.9}{Probabilities of being in the 4 different states
  as a function of time since diagnosis. Note that \textrm{\tt OAD} means
  that OAD was initiated first, and similarly for \textrm{\tt Ins}. We are
  not concerned about what occur after these events.  \textrm{\tt Dead}
  means dead without being on any drug.}

\section{Sojourn times}

The sojourn times in each of the states is just the area of each of
the coloured parts of figure \ref{fig:stack}. Since the $y$-dimension
of the plot is probability (dimensionless) and the $x$-axis has
dimension time, the computed areas will have dimension time.

Normally we will not report the sojourn times as functions of
(truncation) time, but only report them at a few select truncation
points, such as 5 or 10 years. Calculation of the 10 year sojourn
times would be straight-forward as integrals from 0 to 10---these
calculations rely on the predicted rates from \texttt{nd} being for
the first 10 years:
<<>>=
Sj <- c(sjA = sum(Sv * int),
        sjD = sum(rD * int),
        sjI = sum(rI * int),
        sjO = sum(rO * int))
c(Sj, sum(Sj))
@ %
We see that there is a small rounding error in the calculations; the
sum should really be exactly 10.

This was a demonstration on how to compute the rates, cumulative risks
and sojourn times. But no confidence intervals.

\chapter{Confidence intervals}

Besides confidence intervals for each of the 4 cumulative risks,
we will also be interested in confidence intervals for \emph{sums} of
any subset of the cumulative risks, corresponding to the borders
between the colours in figure \ref{fig:stack}. If we only had two
competing risks (and hence three states) the latter would not be an
issue, because the sum of any two cumulative risks will be 1 minus the
cumulative risk of the remainder, so we could get away with the
confidence intervals for the single cumulative risks. This is the
reason we have chosen an example with 3 competing risks and not just
2; we then have 4 probabilities to sum in different order.

A short look at the formulae for cumulative risks will reveal that
analytic approximation to the standard error of these probabilities
(or some transform of them) is not really a viable way to
go. Particularly if we also want confidence intervals of sums of the
state probabilities as those shown in stacked plots.

So in practice, if we want confidence intervals not only for the state
probabilities, but also for any sum of subsets of them we would want a
large number of simulated copies of the cumulative risks, each copy
being of the same structure as the one we just extracted from the
models.

Confidence intervals for sojourn times (i.e. time spent) in each state
up to a given time, would come almost for free from the simulation
approach.

This means that we must devise a method to make a prediction not from
the estimated model, but where we instead of the model parameters use
a sample from the posterior distribution of the estimated parameters.
Here, the posterior distribution of the parameters will be taken to be
the multivariate normal distribution with mean equal to the vector of
parameter estimates and variance-covariance matrix equal to the
estimated variance-covariance matrix of the parameters.

Precisely this approach is implemented in \texttt{ci.lin} via the
\texttt{sample} argument; we can get a predicted value from a given
prediction data frame just as from \texttt{ci.pred}
resp. \texttt{ci.exp}; here is shown two different ways of getting
predicted values of the cause-specific rates:
<<>>=
head(cbind(ci.pred(mI, nd),
           ci.exp (mI, nd)))
@ %
And here is an illustration of the prediction with model based confidence
intervals for the rates, alongside predictions based on samples from
the posterior distribution of the parameters in the model:
<<>>=
str(ci.lin(mI, nd, sample = 4))
head(cbind(ci.pred(mI,nd), exp(ci.lin(mI, nd, sample = 4))))
@ %
Note that we use \texttt{exp(ci.lin(...}---this is because the
\texttt{sample=} argument does not work with \texttt{ci.exp}.

The simulation (parametric bootstrapping) is taking place at the
parameter level and the transformation to survival and cumulative
risks is simply by a function applied to each simulated set of rates.

\section{Common parameters across cause-specific rates}

Note that we have implicitly been assuming that the transitions are
being modeled separately. If some transitions are modeled
jointly---for example assuming that the rates of \texttt{OAD} and
\texttt{Ins} are proportional as functions of time since entry, we are
in trouble, because we then need one sample from the posterior
generating two different predictions, one for each of the transitions
modeled together. Moreover the model will have to be a model fitted to
a \texttt{stack.Lexis} object, so a little more complicated to work
with.

A simple way to program this would be to reset the seed to the same
value before simulating with different values of \texttt{nd}, this is
what is intended to be implemented, but is not yet. This is mainly the
complication of having different prediction frames for different risks
in this case.

However, this is not a very urgent need, since the situation where you
want common parameters for different rates out of a common state is
quite rare. By that token the facility is not likely to be implemented
anytime soon, if ever.

\section{Simulation based confidence intervals}

The parametric bootstrap is implemented in the function
\texttt{ci.Crisk} (\code{c}onfidence \code{i}ntervals for
\code{C}umulative \code{risk}s) in the \texttt{Epi} package:

We can now run the function using the model objects for the three
competing events, using a common prediction data frame, \texttt{nd}
for the rates. The time points in the frame must be so closely spaced
that it makes sense to assume the rates constant in each interval;
here we use intervals of length 1/10 years, approximately 1
month---too large for real applications:
<<>>=
system.time(
res <- ci.Crisk(list(OAD = mO,
                     Ins = mI,
                    Dead = mD),
                            nd = data.frame(tfd = seq(0, 10, 1/10)),
                            nB = 100,
                          perm = 4:1))
str(res)
@ %
As we see, the returned object (\texttt{res}) is a list of length 4,
the first 3 components are 3-way arrays, and the last the vector of
times of the first dimension of the arrays. The latter is mainly for
convenience in further processing---it is easier to write
\texttt{res\$ time} than
\texttt{as.numeric(dimnames(res\$ Crisk)[[1]])}.

The three first components of \texttt{res} represent:
\begin{itemize}
\item \texttt{Crisk} \texttt{C}umulative \texttt{risk}s for each state
\item \texttt{Srisk} \texttt{S}tacked cumulative \texttt{risk}s
  across states
\item \texttt{Stime} \texttt{S}ojourn
  \texttt{time}s in each state, truncated at each point of the time
  dimension.
\end{itemize}

The first dimension of each array is time corresponding to endpoints
of intervals of length \texttt{int}, (normally assumed starting at 0,
but not necessarily). The second dimension is states (or combinations
thereof). The last dimension of the arrays is the type of statistic;
\texttt{50\%} is the median of the samples, and the bootstrap
confidence intervals as indicated; taken from the \texttt{alpha}
argument to \texttt{ci.Crisk} (defaults to 0.05).

The argument \texttt{perm} governs in which order the state
probabilities are stacked in the \texttt{Srisk} element of the
returned list, the default is the states in the order given in the
list of models in the first argument to \texttt{ci.Crisk} followed by
the survival.

If we want the bootstrap samples to make other calculations we can ask
the function to return the bootstrap samples of the rates by using the
argument \texttt{sim.res = 'rates'} (defaults to \texttt{'none'}):
<<>>=
system.time(
rsm <- ci.Crisk(list(OAD = mO,
                     Ins = mI,
                    Dead = mD),
                            nd = data.frame(tfd = seq(0, 10, 1/10)),
                            nB = 100,
                       sim.res = 'rates'))
str(rsm)
@ %
This is 100 bootstrap samples of the rates evaluated at the 501
endpoints of the intervals.

Alternatively we can get the bootstrap samples of the cumulative risks
by setting \texttt{sim.res = 'crisk'}:
<<>>=
system.time(
csm <- ci.Crisk(list(OAD = mO,
                     Ins = mI,
                    Dead = mD),
                            nd = data.frame(tfd = seq(0, 10, 1/10)),
                            nB = 100,
                       sim.res = 'crisk'))
str(csm)
@ %
These are 100 simulated samples of the cumulative risks evaluated at
the 501 endpoints of the intervals, and also includes the survival
probability in the first slot of the \nth{2} dimension of
\texttt{csm}.

\section{Simulated confidence intervals for rates}

In figure \ref{fig:rates} we showed the rates with confidence
intervals from the model. But in \texttt{rsm} we have 100
parametric bootstrap samples of the occurrence rates, so we can
derive the bootstrap medians and the bootstrap c.i.s:
<<>>=
Brates <- aperm(apply(rsm,
                      1:2,
                      quantile,
                      probs = c(.5, .025, .975)),
                c(2, 3, 1))
str(Brates)
@ %
(\texttt{aperm} permutes the dimensions of the array).
Then we can plot the bootstrap estimates on top of the estimates based
on the normal approximation to distribution of the parameters. They
are---not surprisingly---in close agreement since they are both based
on an assumption of normality of the parameters on the log-rate scale:

\insfig{rates-ci}{0.9}{Estimated rates from the \textrm{\tt DM} state,
  estimates are from \textrm{\tt gam} models fitted to data split in 1
  month intervals (1/12 year, that is). The white dotted curves are
  the bootstrap medians, black dotted curves are the bootstrap 95\%
  c.i.s.}
<<rates-ci, fig = TRUE, width = 8>>=
matshade(nd$tfd, cbind(ci.pred(mD, nd),
                       ci.pred(mI, nd),
                       ci.pred(mO, nd)) * 1000,
         ylim = c(0.1,500), yaxt = "n",
         ylab = "Rates per 1000 PY",
         xlab = "Time since DM diagnosis (years)",
         col = c("black","red","blue"), log = "y", lwd = 3, plot = TRUE)
matlines(nd$tfd,
         cbind(Brates[,"Dead",],
               Brates[,"Ins" ,],
               Brates[,"OAD" ,]) * 1000,
         col = c("white", "black", "black"), lty = 3, lwd = c(3,1,1))
axis(side = 2, at = ll<-outer(c(1,2,5),-2:3,function(x,y) x*10^y),
               labels = formatC(ll,digits = 4), las = 1)
axis(side = 2, at = ll<-outer(c(1.5,2:9),-2:3,function(x,y) x*10^y),
               labels = NA, tcl = -0.3)
text(0, 0.5*0.6^c(1,2,0),
     c("Dead", "Ins", "OAD"),
     col = c("black", "red", "blue"), adj = 0)
@ %

\section{Confidence intervals for cumulative risks}

In the \texttt{Crisk} component of \texttt{res} we have the
cumulative risks as functions of of time, with bootstrap confidence
intervals, so we can easily plot the three cumulative risks:

\insfig{crates}{0.9}{Cumulative risks for the three types of events,
  with 95\% bootstrap-based confidence intervals as shades.}
<<crates, fig = TRUE, width = 8>>=
matshade(res$time,
         cbind(res$Crisk[,"Dead",],
               res$Crisk[,"Ins" ,],
               res$Crisk[,"OAD" ,]), plot = TRUE,
         xlim = c(0,10), xaxs = "i", yaxs = "i", las = 1,
         xlab = "Time since DM diagnosis (years)",
         ylab = "Cumulative probability",
          col = c("black","red","blue"))
text(8, 0.3 + c(1, 0, 2) / 25,
     c("Dead", "Ins", "OAD"),
     col = c("black", "red", "blue"), adj = 0)
@ %

\section{Confidence intervals for stacked cumulative risks}

Unlike the single cumulative risks where we have a confidence
interval for each cumulative risk, when we want to show the stacked
probabilities we must deliver the confidence intervals for the
relevant sums, they are in the \texttt{Srisk} component of \texttt{res}.
<<>>=
str(res$Crisk)
str(res$Srisk)
@ %
But we start out by plotting the stacked probabilities using
\texttt{mat2pol} (\texttt{mat}rix to \texttt{pol}ygon), the input
required is the single components from the \texttt{Crisk}
component. Then we add the confidence intervals as white shades (using
\texttt{matshade}):

\insfig{stack-ci}{0.9}{Probabilities of being in the 4 different
  states as a function of time since diagnosis. Note that \textrm{\tt OAD}
  means that OAD was initiated first, and similarly for
  \textrm{\tt Ins}. We are not concerned about what occurs \emph{after}
  these events.  \textrm{\tt Dead} means dead without being on any
  drug.\newline The white shadings around the borders between coloured
  areas represent the 95\% confidence intervals for the (sum of)
  probabilities.}

<<stack-ci, fig = TRUE, width = 8>>=
zz <- mat2pol(res$Crisk[,c("Dead", "Ins", "OAD", "Surv"),1],
              x = res$time,
           xlim = c(0,10), xaxs = "i", yaxs = "i", las = 1,
           xlab = "Time since DM diagnosis (years)",
           ylab = "Probability",
            col =  c("black","red","blue","forestgreen") )
text(9, mp(zz["9",]), c("Dead", "Ins", "OAD", "DM"), col = "white" )
matshade(res$time,
         cbind(res$Srisk[,1,],
               res$Srisk[,2,],
               res$Srisk[,3,]),
         col = 'transparent', col.shade = "white", alpha = 0.4)
@ % $

\section{Sojourn times}

From the \texttt{Stime} component of the \texttt{res} we can derive
the estimated time spent in each state during the first, say, 5 or 10 years:

When referring to the times, we use \emph{character} values---5 and 10
years are not necessarily at the \nth{5} and \nth{10} positions of the
first dimension of the \texttt{Stime} array:
<<>>=
s510 <- res$Stime[c("5", "10"),,]
dimnames(s510)[[1]] <- c(" 5 yr","10 yr")
round(ftable(s510, row.vars=1:2), 2)
@ % $
So we see that the expected life lived without pharmaceutical treatment
during the first 10 years after DM diagnosis is 4.31 years with a 95\%
CI of (4.21;4.42), and during the first 5 years 2.77 (2.72;2.82).

\chapter{A simple illustration}

The following is a terse cook-book illustration of how to use the
\texttt{ci.Crisk} function.

\section{Data}

For illustration we simulate some causes of death in the
\texttt{DMlate} data set; first we sample numbers 1, 2, 3 representing
3 different causes of death in \texttt{DMlate}:
<<>>=
data(DMlate)
set.seed(7465)
wh <- sample(1:3, nrow(DMlate), r=T, prob = c(4, 2, 6))
@ %
Those not dead are changed to 0:
<<>>=
wh[is.na(DMlate$dodth)] <- 0
@ %
Define a factor in DMlate defining exit status as either alive or one
of the three causes of death, and check by a \texttt{table} that all
dead have a cause:
<<>>=
DMlate$codth <- factor(wh, labels = c("Alive", "CVD", "Can", "Oth"))
with(DMlate, table(codth, isDead = !is.na(dodth)))
@ %
\texttt{DMlate} now looks like a typical data set with cause of death
in a separate variable; in this case we also added a state, \texttt{Alive},
for those without a recorded death:
<<>>=
str(DMlate)
head(DMlate, 12)
@ %

\section{A \texttt{Lexis} object with 3 causes of death}

With cause of death in a separate variable it is easy to set up a
\texttt{Lexis} object:
<<>>=
dmL <- Lexis(entry = list(per = dodm,
                          age = dodm - dobth,
                          tfD = 0),
              exit = list(per = dox),
       exit.status = codth,
              data = DMlate)
summary(dmL, t = T)
@ %
We can show the overall rates (the default \texttt{boxes} is
\emph{very} primitive):
<<boxes, fig = TRUE, height = 4>>=
boxes(dmL, boxpos = TRUE)
@ %
\insfig{boxes}{0.8}{Transitions from live to different causes of
  death. You probably want to explore the other arguments to
  \textrm{\tt boxes}.}

\section{Models for the rates}

In order to model the cause-specific mortality rates by sex and time from
diagnosis (\texttt{tfD}), we first split the data in 6-month intervals
<<>>=
sL <- splitLexis(dmL, time.scale="age", breaks = seq(0, 120, 1/2))
summary(sL)
@
<<>>=
mCVD <- gam.Lexis(sL, ~ s(tfD, by=sex), to = "CVD")
mCan <- gam.Lexis(sL, ~ s(tfD, by=sex), to = "Can")
mOth <- gam.Lexis(sL, ~ s(tfD, by=sex), to = "Oth")
@ %
<<eval=FALSE, echo=FALSE>>=
mCVD <- glm.Lexis(sL, ~ Ns(tfD, kn=1:6*2):sex, to = "CVD")
mCa  <- glm.Lexis(sL, ~ Ns(tfD, kn=1:6*2):sex, to = "Ca")
mOth <- glm.Lexis(sL, ~ Ns(tfD, kn=1:6*2):sex, to = "Oth")
@ %

\section{Derived measures}

With these three models for the occurrence rates we can compute the
cumulative risks of dying from each of the causes. We need a
prediction data frame that gives the rates at closely spaced times, in
this case for men. For women the code would be practically the same:
<<>>=
nm <- data.frame(tfD = seq(0, 15, 0.1), sex = "M")
@ %
Note that we can rename the states as we please by naming the entries
in the list of models we supply to \texttt{ci.Crisk}:
<<>>=
cR <- ci.Crisk(list(CVD = mCVD,
                    Can = mCan,
                  Other = mOth),
               nB = 100,
               nd = nm)
str(cR)
@ %$
Note that we get three arrays: \texttt{Crisk}, the cumulative risks;
\texttt{Srisk}, the stacked risks and \texttt{Stime}, the sojourn
times in each state. Finally, for convenience we also have the
component \texttt{time}, the times at which the cumulative risks are
computed. It is also available as the clumpy expression
\texttt{as.numeric(dimnames(cR\$Crisk)[[1]])}, but \texttt{cR\$time}
is easier.

\subsection{Cumulative risks}

We can plot the cumulative risks for death from each of the three
causes, note we use the colors from last. Note that the time points
are in the \texttt{time} component of the \texttt{Crisk} object:
<<cR, fig=TRUE, width=8>>=
clr <- c("black", "orange", "limegreen")
matshade(cR$time, cbind(cR$Crisk[, "CVD"  , ],
                        cR$Crisk[, "Can"  , ],
                        cR$Crisk[, "Other", ]),
         col = clr, lty = 1, lwd = 2,
         plot = TRUE, ylim = c(0, 1/3), yaxs = "i")
text(0, 1/3 - 1:3/30, c("CVD", "Can", "Oth"),
     col = clr, adj = 0)
@ %
\insfig{cR}{0.9}{Cumulative risks of each cause of death based on
\textrm{\tt gam} models for the cause-specific rates.}

We also have the stacked probabilities so we can show how the
population is distributed across the states at any one time:

\subsection{Stacked cumulative risks}

We also get the stacked probabilities in the order that we supplied the
models, so that if we plot these we get the probabilities of being
dead from each cause as the \emph{difference} between the curves. And
the confidence intervals are confidence intervals for the cumulative
sums of probabilities.
<<Sr1, fig = TRUE, width = 8>>=
matshade(cR$time, cbind(cR$Srisk[,1,],
                        cR$Srisk[,2,],
                        cR$Srisk[,3,]),
         col = "black", lty = 1, lwd = 2,
         plot = TRUE, ylim = c(0,1), xaxs = "i", yaxs = "i")
text(14, mp(c(0, cR$Srisk["14", , 1], 1)),
     rev(c(dimnames(cR$Crisk)[[2]])))
box(bty = "o")
@ %
\insfig{Sr1}{0.9}{Stacked cumulative risks.}
It is not a good idea to color the curves, they do not refer to
the causes of death, it is the areas \emph{between} the curves that
refer to causes.

It would be more logical to color the areas \emph{between} the
curves. which can be done by \texttt{mat2pol} (\texttt{mat}rix to
\texttt{pol}ygons) using the \texttt{Crisk} component. We can then
superpose the confidence intervals for the sum of the state
probabilities using \texttt{matshade} by adding white shades:
<<Sr2, fig = TRUE, width = 8>>=
zz <- mat2pol(cR$Crisk[, c("Other", "Can", "CVD", "Surv"), "50%"],
              x = cR$time,
           xlim = c(0,15), xaxs = "i", yaxs = "i", las = 1,
           xlab = "Time since DM diagnosis (years)",
           ylab = "Probability",
            col =  c("gray", "red", "blue", "limegreen") )
matshade(cR$time, cbind(cR$Srisk[,1,],
                        cR$Srisk[,2,],
                        cR$Srisk[,3,]),
         col = "transparent", col.shade = "white", alpha = 0.4)
text(14, mp(c(0, cR$Srisk["14", , 1], 1)),
     rev(c(dimnames(cR$Crisk)[[2]])), col = "white")
@ %
\insfig{Sr2}{0.9}{Stacked cumulative risks with coloring of states and
overlaid with confidence intervals for the probabilities shown; that
is the relevant sums.}

\subsection{Sojourn times}

The third component of the result, \texttt{Stime} is an array of
sojourn times over intervals starting at 0 and ending at the time
indicated by the first dimension:
<<>>=
ftable(round(cR$Stime[paste(1:5 * 3), , ], 1), row.vars = 1)
@ %
The sojourn times in the three dead states can be taken as the years
of life lost to each of the causes, the sum of the medians for the
three causes equals the time frame (5, 10, 15) minus the \texttt{Surv}
component.

So we see that during the first 15 years after diagnosis of diabetes,
the expected years alive is 10.9 years. The distribution of lifetime lost
between the causes is bogus in this case as the causes of death were
randomly generated.

\subsection{Comparing groups}

Finally, we may want to see the \emph{difference} (or ratio) of
survival probabilities between men and women, say. This can be derived
from two bootstrap samples using different prediction frames (the
argument \texttt{nd=} to ci.Crisk). But the two bootstrap samples must
come from the \emph{same} stream of samples from the multivariate
normal. This can be obtained by explicitly setting the seed for the
random number generator to the same value before calling
\texttt{ci.Crisk} with each of the two different prediction frames as
\texttt{nd} argument:
<<>>=
nm <- data.frame(tfD = seq(0, 15, 0.1), sex = "M")
nw <- data.frame(tfD = seq(0, 15, 0.1), sex = "F")
set.seed(1952)
mR <- ci.Crisk(list(CVD = mCVD,
                    Can = mCan,
                  Other = mOth),
               nd = nm,
               nB = 100,
          sim.res = "crisk" )
set.seed(1952)
wR <- ci.Crisk(list(CVD = mCVD,
                    Can = mCan,
                  Other = mOth),
               nd = nw,
               nB = 100,
          sim.res = "crisk" )
str(wR)
@ %$
The two samples are now from identical streams of random numbers, so
we can get differences and ratios of the survival curves between men
and women:
<<>>=
dS <- mR[,"Surv",] - wR[,"Surv",]
dS <- apply(dS, 1, quantile, probs = c(.5, .025, .975)) * 100
str(dS)
rS <- mR[,"Surv",] / wR[,"Surv",]
rS <- apply(rS, 1, quantile, probs = c(.5, .025, .975))
@ %
We can then plot the differences and the ratios of the
probabilities---note that the dimension of the function
\texttt{apply}ed becomes the first dimension of the result:
<<difrat, fig = TRUE, width = 9>>=
par(mfrow = c(1,2))
matshade(as.numeric(colnames(dS)), t(dS), plot = TRUE,
         lwd = 3, ylim = c(-5, 5),
         xlab = "Time since DM diagnosis (years)",
         ylab = "Men - Women survival difference (%)")
abline(h = 0)
matshade(as.numeric(colnames(rS)), t(rS), plot = TRUE,
         lwd = 3, ylim = c(1/1.2, 1.2), log ="y",
         xlab = "Time since DM diagnosis (years)",
         ylab = "Men - Women survival ratio")
abline(h = 1)
@ %
\insfig{difrat}{1.0}{Differences and ratios of survival between men
  and women, derived from the same set of bootstrap samples from the
  parameter vector.}

To illustrate the effect of \texttt{not} pairing the random samples we
can generate a fresh sample for women from a different stream and do
the calculations to illustrate the excess we get from not aligning
samples.
<<>>=
fR <- ci.Crisk(list(CVD = mCVD,
                    Can = mCan,
                  Other = mOth),
               nd = nw,
               nB = 100,
          sim.res = "crisk" )
dxS <- mR[,"Surv",] - fR[,"Surv",]
dxS <- apply(dxS, 1, quantile, probs = c(.5, .025, .975)) * 100
rxS <- mR[,"Surv",] / fR[,"Surv",]
rxS <- apply(rxS, 1, quantile, probs = c(.5, .025, .975))
@ %
<<difratx, fig = TRUE, width = 9>>=
par(mfrow = c(1,2))
matshade(as.numeric(colnames(dS)), t(dS), plot = TRUE,
         lwd = 3, ylim = c(-5, 5),
         xlab = "Time since DM diagnosis (years)",
         ylab = "Men - Women survival difference (%)")
matshade(as.numeric(colnames(dxS)), t(dxS), lty = 3, col = "forestgreen")
abline(h = 0)

matshade(as.numeric(colnames(rS)), t(rS), plot = TRUE,
         lwd = 3, ylim = c(1/1.2, 1.2), log ="y",
         xlab = "Time since DM diagnosis (years)",
         ylab = "Men - Women survival ratio")
matshade(as.numeric(colnames(rxS)), t(rxS), lty = 3, col = "forestgreen")
abline(h = 1)
@ %
\insfig{difratx}{1.0}{Differences and ratios of survival between men
  and women, derived from separate bootstrap samples. The outer
  confidence bands are from bootstrap samples not properly paired
  between men end women.}

<<echo = FALSE>>=
ende <- Sys.time()
cat("  Start time:", format(anfang, "%F, %T"), "\n")
cat("    End time:", format(  ende, "%F, %T"), "\n")
cat("Elapsed time:", round(difftime(ende, anfang, units = "mins"), 2), "minutes\n")
@ %

\end{document}

%\VignetteIndexEntry{Competing risks with Lexis, parametric rates and simulation based confidence intervals}
\SweaveOpts{results=verbatim,keep.source=TRUE,include=FALSE,eps=FALSE}
\documentclass[a4paper,dvipsnames,twoside,12pt]{report}
\newcommand{\Title}{Competing risks with\\ \texttt{Lexis}, parametric
  rates and\\ simulation based confidence intervals}
\newcommand{\Tit}{CmpRskParSim}
\newcommand{\Version}{Version 6}
\newcommand{\Dates}{November 2024}
\newcommand{\Where}{SDCC}
\newcommand{\Homepage}{\url{https://bendixcarstensen.com/Epi/}}
\newcommand{\Faculty}{\begin{tabular}{rl}
Bendix Carstensen
  & Steno Diabetes Center Copenhagen, Herlev, Denmark\\
  & {\small \& Department of Biostatistics,
               University of Copenhagen} \\
  & \href{mailto:b@bxc.dk}{\tt b@bxc.dk} \\
  & \url{https://BendixCarstensen.com/} \\[1em]
                      \end{tabular}}

\input{topreport}
\renewcommand{\rwpre}{./03crisk}

\chapter{Competing risks}

\section{Technicalities}
First we set some output and graphics parameters for convenience and
load the packages needed:
<<>>=
options(width = 90,
        show.signif.stars = FALSE,
        SweaveHooks=list(fig = function()
                         par(mar = c(3, 3, 1, 1),
                             mgp = c(3, 1, 0) / 1.6,
                             las = 1,
                            lend = "butt",
                             bty = "n")))
library(Epi)
library(popEpi)
library(survival)
clear()
@
% must be after clear() because 'anfang' is used at the end
<<echo = FALSE, results = hide>>=
anfang <- Sys.time()
cat("Start time:", format(anfang, "%F, %T"), "\n")
@ %
<<echo = FALSE>>=
vers <-
data.frame(R = substr(R.version.string, 11, 15),
         Epi = as.character(packageVersion(   "Epi")),
      popEpi = as.character(packageVersion("popEpi")))
names(vers) <- paste(" ", names(vers))
print(vers, row.names = FALSE)
@ %

\section{Concepts}

The concept of competing risks is one where persons in a given state,
``alive'', say, are subject to a number of different causes of death,
``cause1'', ``cause2'' etc. Causes of death are required to be
exhaustive and mutually exclusive. That is, you will eventually die
from one of the designated causes, and you can only die from one.

The \emph{cause-specific} rate for cause $c$ is defined as:
\[
   \lambda_c(t) = \ptxt{death from cause $c$ in $(t,t+h]\,|\,$
                        alive at $t$} / h
\]
\ldots formally, the limit of this quantity as $h\rightarrow 0$.

The observed data will be a survival time and an exit status which is
either ``censored alive'' or one of the causes. In situations where
the causes are not causes of death but other events, it is implicit
that we only consider the first occurrence of an event from the state
``alive'', and ignore whatever occurs after that.

\subsection{Cause specific rates and likelihood}

The likelihood for observations from a competing risk scenario is a
function of the cause-specific transition rates, and is a
\emph{product} of the likelihoods that would emerge if we considered
each cause as being the only possible event. Thus analysis is in
principle straight forward: estimate a model for each of the
cause-specific rates; these will together form a complete model for
the competing risks problem.

If the cause-specific rates are all we want to assess then we are done.

\subsection{Survival and cumulative risks}

In addition to the rates we might however also be interested in the
\emph{survival} probability and the \emph{cumulative risks} of each
cause of death.

The survival is the probability of still being alive at a given time
after entry---a function of time since entry. The cumulative risk of
dying from cause $c$ is the probability of having died from cause $c$
as a function of time since entry.

This means that a time of entry is required for the calculations these
quantities.

\subsection{Sojourn times}

The \emph{sojourn time} for cause $c$ is the time spent in the ``cause
$c$'' state before a given time, $t$, say. This is also called the
expected lifetime lost to cause $c$, \emph{truncated} at the time
$t$. For the state ``alive'' it will be the expected time lived before
$t$. This is also called the restricted mean survival time, RMST.

\subsection{The time scale}

The cause specific rates will be functions of covariates, notably
a time scale, be that age or time since entry to the study or even
calendar time. But the cumulative risks are probabilities that refer
to time \emph{since} some origin. Thus cumulative risks (and survival)
are only meaningful in relation to a time that begins at 0. Though not
a formal mathematical requirement this implies that we should have
data starting at time 0.

If we were to use age as timescale for cumulative risk, we would want
data available since birth; if we only had observations where most
people entered between 20 and 40 years of age, we could mathematically
compute cumulative risk to some age, but it would be nonsense. Instead we
would compute the cumulative risk \emph{given} that a person attained
age 40, say. In that case the time scale would be $\text{age} - 40$.

\section{Rates and cumulative risks}

Each of the cumulative risks is a function of the survival function which
in turn depends on \emph{all} rates. Specifically, if the
cause-specific rates are $\lambda_c(t),\,c=1,2,\ldots$, then the
survival function (probability of being alive at time $t$) is:
\begin{equation}
     S(t) = \exp\left( \!-\!\int_0^t \sum_c \lambda_c(s) \dif s \right)
     =  \exp\left(\!-\! \sum_c \Lambda_c(t) \right)
          \label{eq:Sv}
\end{equation}
The quantities $\Lambda_c(t) = \int_0^t \lambda_c(s) \dif s$ are
called cumulative \emph{rates} (probabilists call them integrated
intensities), although they are not rates. Cumulative rates are
dimensionless, but they have no probability interpretation of any
kind, and they are not necessarily in $[0,1]$.

The cumulative \emph{risk}, the probability of dying from cause $k$,
say, before time $t$, $R_k(t)$ is:
\begin{equation}
   R_k(t) = \int_{u=0}^{u=t} \!\! \lambda_k(u) S(u) \dif u
           = \int_{u=0}^{u=t} \!\! \lambda_k(u) \exp\left(\!-\!
            \sum_j \Lambda_j(u) \right) \! \dif u
          \label{eq:R}
\end{equation}
The rationale is that $\lambda_k(u) \dif u$ is the probability of
death from cause $k$ in the small interval $(u, u + \dif u)$,
\emph{conditional} on being alive at time $u$. If this is multiplied
with the probability of being alive at $u$, $S(u)$, Bayes rule tells
us that we get the \emph{marginal probability} of death from cause $k$
in the small interval $(u, u + \dif u)$. This function of $u$ is the
argument in the integral; so integration from $u=0$ to $u=t$, will
give the probability of death from cause $c$ anywhere in $(0,t)$---the
cumulative risk of cause $k$ at $t$.

Parametric models for the cause-specific rates can produce estimated
transition rates $\lambda_c$ at closely spaced intervals, and the
cumulative risks can then be estimated from these by simple numerical
integration; this is illustrated in the next chapter.

Note that at any one time every person is either alive or dead from one
of the causes, so the sum of the survival and the cumulative risks is
always 1:
\[
   1 = S(t) + R_1(t) + R_2(t) + \cdots, \quad \forall t
\]

\subsection{Confidence intervals by simulation}

But even if we from the modeling of the $\lambda_c$s may have standard
errors of $\log\big(\lambda_c(t)\big)$, the standard errors of the
$R_c$s will be analytically intractable from these.

In practice, the only viable way to get confidence intervals for the
cumulative risks, $R_c$, is therefore by calculation of a set of rates
$\lambda_c(t)$ by sampling from the posterior distribution of the
parameters in the models for $\log\big(\lambda_c(t)\big)$, and then
compute the integrals numerically for each simulated sample, according
to formulae \ref{eq:Sv} and \ref{eq:R}. This will produce a so-called
parametric bootstrap sample of the cumulative risks from which we can
derive confidence intervals

The simulation approach also allows calculation of confidence
intervals for sums of the cumulative risks, $R_1(t)+R_2(t)$, for
example, which will be needed if we want to show stacked cumulative
risks.

Finally, it will also allow calculation of standard errors of sojourn
times in each of the states ``alive'' and ``cause1'',
``cause2'',\ldots. While the latter two may not be of direct interest,
then \emph{differences} between such sojourn times between different
groups can be interpreted as years of life lost to each cause between
groups.

\subsection{Subdistribution hazard}

A common concept seen in competing risks is the subdistribution
hazard, and proportional hazards models for this (the Fine-Gray model).

Suppose for a moment we only consider all-cause mortality,
$\lambda(t)$. Then the cumulative risk of death is:
\[
    R(t) = 1 - S(t) = 1 - \exp\left( \!-\!\int_0^t \lambda(s) \dif s \right)
\]
Solving this for $\lambda$ will yield:
\[
  \lambda(t) = -\frac{\dif\log\big(1 - R(t)\big)}{\dif t}
\]
In the case of multiple causes of death we can define the
\emph{subdistribution hazard} for cause $c$ by using the same
transformation of the cumulative risk for cause $c$:
\[
  \tilde\lambda_c(t) = -\frac{\dif\log\big(1 - R_c(t)\big)}{\dif t}
\]
or, for the cause-specific risk:
\[
    R_c(t) = 1 - \exp\left( \!-\!\int_0^t \tilde\lambda_c(s) \dif s \right)
\]
Thus, the subdistribution hazard $\tilde\lambda_c(t)$ is a function
which, when subjected to the (hazard$\rightarrow$risk) function from
the all-cause mortality case, yields the cause-specific risk.

The Fine-Gray model is a model for the subdistribution hazard
$\tilde\lambda_c$. It is only a model for \emph{one} cause-specific
hazard. Of course it can be applied to all available causes in turn,
but the sum of the cumulative risks derived from the models may exceed
1\ldots

Unlike a cause-specific hazard, which can depend on multiple time
scales, the subdistribution hazard can only depend on one, since it
requires an origin---just like cumulative risks.

But the interpretation of a subdistribution hazard is difficult.  I
have yet to see one that goes beyond the mathematical formalism
above. Therefore the subdistribution is not included in this vignette.

\chapter{Example data}

\section{A \texttt{Lexis} object}

As an illustrative data example we use the (fake) diabetes register
data; we set up the Lexis object, an then cut the follow-up time at
dates of \texttt{OAD} and \texttt{Ins} using \texttt{mcutLexis}, for
convenience we restrict to a 10\% sample of the dataset:
<<>>=
data(DMlate)
set.seed(1952)
DMlate <- DMlate[runif(nrow(DMlate)) < 0.1, ]
Ldm <- Lexis(entry = list(per = dodm,
                          age = dodm-dobth,
                          tfd = 0),
              exit = list(per = dox),
       exit.status = factor(!is.na(dodth), labels = c("DM", "Dead")),
              data = DMlate)
summary(Ldm, t = T)
set.seed(1952)
Mdm <- mcutLexis(Ldm,
                  wh = c('dooad','doins'),
          new.states = c('OAD','Ins'),
          seq.states = FALSE,
                ties = TRUE)
summary(Mdm)
@ We initially split the follow-up in the state \texttt{DM} (that is
before drug inception) in intervals of 1/12 year, creating a
\texttt{Lexis} object for a competing risks situation with three
possible event types:
<<>>=
Sdm <- splitLexis(factorize(subset(Mdm,
                                   lex.Cst == "DM")),
                  time.scale = "tfd",
                      breaks = seq(0, 20, 1/12))
summary(Sdm)
@ %
We can illustrate the follow-up in the full data frame and in the
restricted data frame
<<boxes5, fig = TRUE, width = 8>>=
boxes(Mdm, boxpos = list(x = c(15, 50, 15, 85, 85),
                         y = c(85, 50, 15, 85, 15)),
          scale.R = 100,
          show.BE = TRUE)
@ %
\insfig{boxes5}{0.8}{The transitions in the multistate model, where
    follow-up is extended also after beginning of first drug
    exposure. Rates in brackets are per 100 PY.}%
<<boxes4, fig = TRUE, width = 8>>=
boxes(Relevel(Sdm, c(1, 4, 2, 3)),
      boxpos  = list(x = c(15, 85, 75, 15),
                     y = c(85, 85, 30, 15)),
      scale.R = 100,
      show.BE = TRUE )
@ %
\insfig{boxes4}{0.8}{The transitions in the competing risks model,
  where follow-up is stopped at first drug exposure. By that token only the
  \texttt{DM} state has person-years; a characteristic of a competing risks
  situation.}

\section{Models for rates}

Now that we have set up a dataset with three competing events, we can
model the cause-specific rates separately by time from diagnosis as
the only underlying time scale.

This is done by Poisson-regression on the time-split data set; since
the dataset is in \texttt{Lexis} format we can use the convenience
wrapper \texttt{gamLexis} to model rates as smooth functions of time
(\texttt{tfd}).  Note that we only need to specify the \texttt{to=}
argument because there is only one possible \texttt{from} for each
\texttt{to} (incidentally the same for all \texttt{to} states, namely
\texttt{DM}):
<<>>=
mD <- gamLexis(Sdm, ~ s(tfd, k = 5), to = 'Dead')
mO <- gamLexis(Sdm, ~ s(tfd, k = 5), to = 'OAD' )
mI <- gamLexis(Sdm, ~ s(tfd, k = 5), to = 'Ins' )
@ %
We see that \texttt{gamLexis} (just like \texttt{glmLexis} would)
tells us what transition rates are modeled.

With these models fitted we can compute the rates, and from these
cumulative rates and the cumulative risks and sojourn times in states
using the usual formulae.

First we compute the rates in intervals of length 1/20 years. Note
that these prediction points are unrelated to the follow-up intervals
in which we split the observed data for analysis---they were 1 month
intervals (1/12 year), here we use 1/20 year (in real
life a smaller interval should be used, say 1/50 or 1/100 year):
<<>>=
nd <- data.frame(tfd = seq(0, 10, 1/20))
rownames(nd) <- nd$tfd
str(nd)
@ %
With this we can show the rates as a function of the time since entry
(diagnosis of diabetes):
<<rates, fig = TRUE, width = 8>>=
matshade(nd$tfd, cbind(ci.pred(mD, nd),
                       ci.pred(mI, nd),
                       ci.pred(mO, nd)) * 1000,
         col = c("black", "red", "blue"), log = "y", lwd = 3, plot = TRUE,
         xlab = "Time since DM diagnosis (years)",
         ylab = "Rates per 1000 PY", ylim = c(0.05, 500), yaxt = "n")
axis(side = 2, at = ll <- outer(c(1,2,5), -2:3, function(x,y) x * 10^y),
               labels = formatC(ll, digits = 4), las = 1)
axis(side = 2, at = outer(c(1.5,2:9), -2:3, function(x,y) x * 10^y),
               labels = NA, tcl = -0.3)
text(0, 0.5*0.6^c(1,2,0),
     c("Dead","Ins","OAD"),
     col = c("black","red","blue"), adj = 0, font = 2)
@ %
\insfig{rates}{0.8}{Estimated rates from the \textrm{\tt DM} state,
  estimates are from \textrm{\tt gam} models fitted to data split in 1
  month intervals (1/12 year, that is). Rates of \textrm{\tt OAD} is
  in the vicinity of 0.1/year, and mortality about half of this. Rates
  of insulin start among persons on no other drug are beginning high,
  then decreasing with a nadir at about 4 years and then increase to a
  peak at 8 years.}

Note that the graph in figure \ref{fig:rates} is not normally shown in
analyses of competing risks; the competing cause-specific \emph{rates}
are hardly ever shown. I suspect that this is frequently because they
are often modeled by a Cox model and so are buried in the model and
hard to get at.

Since we will be integrating the rates, it would also be relevant to show
the rates on a linear scale instead, de-emphasizing the very small
fluctuations of the \texttt{Ins} rates that are over-emphasized when
using a log-scale for the $y$-axis.
<<rates-l, fig = TRUE, width = 8>>=
matshade(nd$tfd, cbind(ci.pred(mD, nd),
                       ci.pred(mI, nd),
                       ci.pred(mO, nd)) * 1000,
         col = c("black", "red", "blue"), lwd = 3, plot = TRUE,
         xlab = "Time since DM diagnosis (years)",
         ylab = "Rates per 1000 PY", ylim = c(0, 500), yaxs = "i")
text(8, 500 - c(2, 3, 1) * 20,
     c("Dead","Ins","OAD"),
     col = c("black","red","blue"), adj = 0, font = 2)
@ %
\insfig{rates-l}{0.8}{Estimated rates from the \textrm{\tt DM} state,
  estimates are from \textrm{\tt gam} models fitted to data split in 1
  month intervals (1/12 year, that is). Rates of \textrm{\tt OAD} is
  in the vicinity of 0.1/year, and mortality about half of this. .}

\section{Cumulative rates and risks}

For the calculation of the cumulative rates and state probabilities,
we need just the estimated rates (without CIs). The formulae
\ref{eq:Sv} and \ref{eq:R} on page \pageref{eq:R} are transformed to
\R-code; starting with the rates, $\lambda_\text{D}$ as \texttt{lD}
etc:
<<>>=
# utility function to compute midpoints between sucessive values in a vector
mp <- function(x) x[-1] - diff(x) / 2
#
int <- 1 / 20
# rates at midpoints of intervals
lD <- mp(ci.pred(mD, nd)[, 1])
lI <- mp(ci.pred(mI, nd)[, 1])
lO <- mp(ci.pred(mO, nd)[, 1])
#
# cumulative rates and survival function at right border of the intervals
LD <- cumsum(lD) * int
LI <- cumsum(lI) * int
LO <- cumsum(lO) * int
# survival function, formula (1.1)
Sv <- exp(- LD - LI - LO)
#
# when integrating to get the cumulative *risks* we use the average
# of the survival function at the two endpoints
# (adding 1 as the first), formula (1.2)
Sv <- c(1, Sv)
rD <- c(0, cumsum(lD * mp(Sv)) * int)
rI <- c(0, cumsum(lI * mp(Sv)) * int)
rO <- c(0, cumsum(lO * mp(Sv)) * int)
@ %
Now we have the cumulative risks for the three causes and the
survival, computed at the end of each of the intervals. At any time
point the sum of the 3 cumulative risks and the survival should be 1:
<<>>=
summary(rD + rI + rO + Sv)
oo <- options(digits = 20)
cbind(summary(Sv + rD + rI + rO))
options(oo)
@ %
\ldots and bar a small rounding error, they are.

We can then plot the 3 cumulative risk functions stacked together
using \texttt{mat2pol} (\texttt{mat}rix to \texttt{pol}ygons):
<<stack, fig = TRUE, width = 8>>=
zz <- mat2pol(cbind(rD, rI, rO, Sv), x = nd$tfd, # $
              xlim = c(0,10), xaxs = "i", yaxs = "i", las = 1,
              xlab = "Time since DM diagnosis (years)",
              ylab = "Probability",
               col =  c("black","red","blue","forestgreen"))
text(9, mp(zz["9", ]), c("Dead", "Ins", "OAD"," DM"), col = "white")
box(col = "white", lwd = 3)
@ %
\insfig{stack}{0.9}{Probabilities of being in the 4 different states
  as a function of time since diagnosis. Note that \textrm{\tt OAD} means
  that OAD was initiated first, and similarly for \textrm{\tt Ins}. We are
  not concerned about what occurs after any these events.  \textrm{\tt Dead}
  means dead without initiating any of the two drugs.}

\section{Sojourn times}

The sojourn times in each of the states is just the area of each of
the coloured parts of figure \ref{fig:stack}. Since the $y$-dimension
of the plot is probability (dimensionless) and the $x$-axis has
dimension time, the computed areas will have dimension time.

Normally we will not report the sojourn times as functions of
(truncation) time, but only report them at a few select truncation
points, such as 5 or 10 years. Calculation of the 10 year sojourn
times would be straight-forward as integrals from 0 to 10---these
calculations rely on the predicted rates from \texttt{nd} being for
the first 10 years:
<<>>=
Sj <- c(sjA = sum(Sv * int),
        sjD = sum(rD * int),
        sjI = sum(rI * int),
        sjO = sum(rO * int))
c(Sj, sum(Sj))
@ %
We see that there is a some rounding error in the calculations; the
sum should really be exactly 10.

This was a demonstration on how to compute the rates, cumulative risks
and sojourn times. But no confidence intervals.

\chapter{Confidence intervals for cumulative risks}

Besides confidence intervals for each of the 4 cumulative risks,
we will also be interested in confidence intervals for \emph{sums} of
any subset of the cumulative risks, corresponding to the borders
between the colours in figure \ref{fig:stack}. If we only had two
competing risks (and hence three states) the latter would not be an
issue, because the sum of any two cumulative risks will be 1 minus the
cumulative risk of the remainder, so we could get away with the
confidence intervals for the single cumulative risks. This is the
reason we have chosen an example with 3 competing risks and not just
2; we then have 4 probabilities to sum in different order.

A short look at the formulae for cumulative risks will reveal that
analytic approximation to the standard error of these probabilities
(or some transform of them) is not really a viable way to
go. Particularly if we also want confidence intervals of sums of the
state probabilities as those shown in stacked plots.

So in practice, if we want confidence intervals not only for the state
probabilities, but also for any sum of subsets of them we would want a
large number of simulated copies of the cumulative risks, each copy
being of the same structure as the one we just extracted from the
models.

Confidence intervals for sojourn times (i.e. time spent) in each state
up to a given time, would come almost for free from the simulation
approach, by taking the relevant quantiles of the simulated quantities.

This means that we must devise a method to make a prediction not from
the estimated model, but where we instead of the model parameters use
a sample from the posterior distribution of the estimated parameters.
Here, the posterior distribution of the parameters will be taken to be
the multivariate normal distribution with mean equal to the vector of
parameter estimates and variance-covariance matrix equal to the
estimated variance-covariance matrix of the parameters.

Precisely this approach is implemented in \texttt{ci.lin} via the
\texttt{sample} argument; we can get a predicted value from a given
prediction data frame just as from \texttt{ci.pred}
resp. \texttt{ci.exp}; here is shown two different ways of getting
predicted values of the cause-specific rates:
<<>>=
head(cbind(ci.pred(mI, nd),
           ci.exp (mI, nd)))
@ %
Here is an illustration of the prediction with model based confidence
intervals for the rates of insulin start (model \texttt{mI}),
alongside predictions based on samples from the posterior distribution
of the parameters in the model:
<<>>=
str(ci.lin(mI, nd, sample = 4))
head(cbind(ci.pred(mI, nd), exp(ci.lin(mI, nd, sample = 4))))
@ %
Note that we use \texttt{exp(ci.lin(...}---this is because the
\texttt{sample=} argument does not work with \texttt{ci.exp}.

The simulation (parametric bootstrapping) is taking place at the
parameter level and the transformation to survival and cumulative
risks is simply by a function applied to each simulated set of rates.

\section{Common parameters across cause-specific rates}

Note that we have implicitly been assuming that the transitions are
being modeled separately. If some transitions are modeled
jointly---for example assuming that the rates of \texttt{OAD} and
\texttt{Ins} are proportional as functions of time since entry, we are
in trouble, because we then need one sample from the posterior
generating two different predictions, one for each of the transitions
modeled together. Moreover the model will have to be a model fitted to
a \texttt{stack.Lexis} object, so a little more complicated to work
with.

A simple way to program this would be to reset the seed to the same
value before simulating with different values of \texttt{nd}, this is
what is intended to be implemented, but is not yet. This is mainly the
complication of having different prediction frames for different risks
in this case.

However, this is not a very urgent need, since the situation where you
want common parameters for different rates out of a common state is
quite rare. It would for example be quite odd to assume the the M/W
rate ratio were the same across different cauese of death. By that
token the facility is not likely to be implemented anytime soon, if
ever.

\section{Simulation based confidence intervals}

The parametric bootstrap is implemented in the function
\texttt{ci.Crisk} (\code{c}onfidence \code{i}ntervals for
\code{C}umulative \code{risk}s) in the \texttt{Epi} package:

We can now run the function using the model objects for the three
competing events, using a common prediction data frame, \texttt{nd}
for the rates. The time points in the frame must be so closely spaced
that it makes sense to assume the rates constant in each interval;
here we use intervals of length 1/20 years, in real applications we
would use 1/50 (about 1 week) or less:
<<>>=
res <- ci.Crisk(list(OAD = mO,
                     Ins = mI,
                    Dead = mD),
                            nd = data.frame(tfd = seq(0, 10, 1/20)),
                            nB = 500,
                          perm = 4:1)
str(res)
@ %
As we see, the returned object (\texttt{res}) is a list of length 4,
the first 3 components are 3-way arrays, and the last the vector of
times of the first dimension of the arrays. The latter is mainly for
convenience in further processing---it is easier to write
\texttt{res\$time} than
\texttt{as.numeric(dimnames(res\$Crisk)[[1]])}.

The three first components of \texttt{res} represent:
\begin{itemize}
\item \texttt{Crisk}: \texttt{C}umulative \texttt{risk}s for each state
\item \texttt{Srisk}: \texttt{S}tacked cumulative \texttt{risk}s across
  states
\item \texttt{Stime}: \texttt{S}ojourn \texttt{time}s in each state,
  truncated at each point of the time dimension.
\end{itemize}

The first dimension of each array is time corresponding to endpoints
of intervals of length \texttt{int}, (normally assumed starting at 0,
but not necessarily). The second dimension is states (or combinations
thereof). The last dimension of the arrays is the type of statistic;
\texttt{50\%} is the median of the samples, and the bootstrap
confidence intervals as indicated; taken from the \texttt{alpha}
argument to \texttt{ci.Crisk} (defaults to 0.05).

The argument \texttt{perm} governs in which order the state
probabilities are stacked in the \texttt{Srisk} element of the
returned list, the default is the states in the order given in the
list of models in the first argument to \texttt{ci.Crisk} followed by
the survival.

If we want the bootstrap samples to make other calculations we can ask
the function to return the bootstrap samples of the rates by using the
argument \texttt{sim.res = 'rates'} (defaults to \texttt{'none'}):
<<>>=
rsm <- ci.Crisk(list(OAD = mO,
                     Ins = mI,
                    Dead = mD),
                            nd = data.frame(tfd = seq(0, 10, 1/20)),
                            nB = 500,
                       sim.res = 'rates')
str(rsm)
@ %
This is 500 bootstrap samples (defined by \texttt{nB=}) of the rates
evaluated at the 201 endpoints of the intervals (defined in
\texttt{nd}).

Alternatively we can get the bootstrap samples of the cumulative risks
by setting \texttt{sim.res = 'crisk'}:
<<>>=
csm <- ci.Crisk(list(OAD = mO,
                     Ins = mI,
                    Dead = mD),
                            nd = data.frame(tfd = seq(0, 10, 1/20)),
                            nB = 500,
                       sim.res = 'crisk')
str(csm)
@ %
These are 500 simulated samples of the cumulative risks evaluated at
the 201 endpoints of the intervals, and also includes the survival
probability in the first slot of the \nth{2} dimension of
\texttt{csm}.

\section{Simulated confidence intervals for rates}

In figure \ref{fig:rates} we showed the rates with confidence
intervals from the model. But in \texttt{rsm} we have 500
parametric bootstrap samples of the occurrence rates, so we can
derive the bootstrap medians and the bootstrap c.i.s:
<<>>=
Brates <- aperm(apply(rsm,
                      1:2,
                      quantile,
                      probs = c(.5, .025, .975)),
                c(2, 3, 1))
str(Brates)
@ %
(\texttt{aperm} permutes the dimensions of the array).
Then we can plot the bootstrap estimates on top of the estimates based
on the normal approximation to distribution of the parameters. They
are---not surprisingly---in close agreement since they are both based
on an assumption of normality of the parameters on the log-rate scale:

\insfig{rates-ci}{0.9}{Estimated rates from the \textrm{\tt DM} state,
  estimates are from \textrm{\tt gam} models fitted to data split in 1
  month intervals (1/12 year, that is). The white dotted curves are
  the bootstrap medians, black dotted curves are the bootstrap 95\%
  c.i.s.}
<<rates-ci, fig = TRUE, width = 8>>=
matshade(nd$tfd, cbind(ci.pred(mD, nd),
                       ci.pred(mI, nd),
                       ci.pred(mO, nd)) * 1000,
         ylim = c(0.1,500), yaxt = "n",
         ylab = "Rates per 1000 PY",
         xlab = "Time since DM diagnosis (years)",
         col = c("black", "red", "blue"), log = "y", lwd = 3, plot = TRUE)
matlines(nd$tfd,
         cbind(Brates[, "Dead", ],
               Brates[, "Ins" , ],
               Brates[, "OAD" , ]) * 1000,
         col = c("white", "black", "black"), lty = 3, lwd = c(3,1,1))
axis(side = 2, at = ll <- outer(c(1,2,5), -2:3, function(x, y) x * 10^y),
               labels = formatC(ll, digits = 4), las = 1)
axis(side = 2, at = outer(c(1.5, 2:9), -2:3, function(x, y) x * 10^y),
               labels = NA, tcl = -0.3)
text(0, 0.5 * 0.6^c(1,2,0),
     c("Dead", "Ins", "OAD"),
     col = c("black", "red", "blue"), adj = 0, font = 2)
@ %

\section{Confidence intervals for cumulative risks}

In the \texttt{Crisk} component of \texttt{res} we have the
cumulative risks as functions of of time, with bootstrap confidence
intervals, so we can easily plot the three cumulative risks:

\insfig{crates}{0.9}{Cumulative risks for the three types of events,
  with 95\% bootstrap-based confidence intervals as shades.}
<<crates, fig = TRUE, width = 8>>=
matshade(res$time,
         cbind(res$Crisk[,"Dead",],
               res$Crisk[,"Ins" ,],
               res$Crisk[,"OAD" ,]), plot = TRUE,
         xlim = c(0,10), xaxs = "i", yaxs = "i", las = 1,
         xlab = "Time since DM diagnosis (years)",
         ylab = "Cumulative probability",
          col = c("black","red","blue"))
text(8, 0.3 + c(1, 0, 2) / 25,
     c("Dead", "Ins", "OAD"),
     col = c("black", "red", "blue"), adj = 0)
@ %

\section{Confidence intervals for stacked cumulative risks}

Unlike the single cumulative risks where we have a confidence
interval for each cumulative risk, when we want to show the stacked
probabilities we must deliver the confidence intervals for the
relevant sums, they are in the \texttt{Srisk} component of \texttt{res}.
<<>>=
str(res$Crisk)
str(res$Srisk)
@ %
But we start out by plotting the stacked probabilities using
\texttt{mat2pol} (\texttt{mat}rix to \texttt{pol}ygon), the input
required is the single components from the \texttt{Crisk}
component. Then we add the confidence intervals as white shades (using
\texttt{matshade}):

\insfig{stack-ci}{0.9}{Probabilities of being in the 4 different
  states as a function of time since diagnosis. Note that \textrm{\tt OAD}
  means that OAD was initiated first, and similarly for
  \textrm{\tt Ins}. We are not concerned about what occurs \emph{after}
  these events.  \textrm{\tt Dead} means dead without being on any
  drug.\newline The white shadings around the borders between coloured
  areas represent the 95\% confidence intervals for the (sum of)
  probabilities.}

<<stack-ci, fig = TRUE, width = 8>>=
zz <- mat2pol(res$Crisk[,c("Dead", "Ins", "OAD", "Surv"),1],
              x = res$time,
           xlim = c(0, 10), xaxs = "i", yaxs = "i", las = 1,
           xlab = "Time since DM diagnosis (years)",
           ylab = "Probability",
            col =  c("black","red","blue","forestgreen") )
text(9, mp(zz["9",]), c("Dead", "Ins", "OAD", "DM"), col = "white" )
matshade(res$time,
         cbind(res$Srisk[, 1, ],
               res$Srisk[, 2, ],
               res$Srisk[, 3, ]),
         col = 'transparent', col.shade = "white", alpha = 0.4)
@ %

\section{Sojourn times}

From the \texttt{Stime} component of the \texttt{res} we can derive
the estimated time spent in each state during the first, say, 5 or 10 years:

When referring to the times, we use \emph{character} values---5 and 10
years are not necessarily at the \nth{5} and \nth{10} positions of the
first dimension of the \texttt{Stime} array:
<<>>=
s510 <- res$Stime[c("5", "10"),,]
dimnames(s510)[[1]] <- c(" 5 yr","10 yr")
round(ftable(s510, row.vars=1:2), 2)
@ %
So we see that the expected life lived without pharmaceutical treatment
during the first 10 years after DM diagnosis is 4.31 years with a 95\%
CI of (4.21; 4.41), and during the first 5 years 2.77 (2.72; 2.82).

\chapter{A simple illustration of \texttt{ci.Crisk}}

The following is a terse cook-book illustration of how to use the
\texttt{ci.Crisk} function.

\section{Data}

For illustration we simulate some causes of death in the
\texttt{DMlate} data set; first we sample numbers 1, 2, 3 representing
3 different causes of death in \texttt{DMlate}:
<<>>=
data(DMlate)
set.seed(7465)
wh <- sample(1:3, nrow(DMlate), replace = T, prob = c(4, 2, 6))
@ %
Those not dead are changed to 0:
<<>>=
wh[is.na(DMlate$dodth)] <- 0
@ %
Define a factor in \texttt{DMlate} defining exit status as either alive or one
of the three causes of death, and check by a \texttt{table} that all
dead have a cause:
<<>>=
DMlate$codth <- factor(wh, labels = c("Alive", "CVD", "Can", "Oth"))
with(DMlate, table(codth, isDead = !is.na(dodth)))
@ %
It is important that the \texttt{"Alive"} state is the \texttt{first}
level if the factor \texttt{codth}; the \texttt{Lexis} function will
assign this the all persons at start of follow-up.

\texttt{DMlate} now looks like a typical data set with cause of death
in a separate variable; in this case we also added a state, \texttt{Alive},
for those without a recorded death:
<<>>=
str(DMlate)
head(DMlate, 12)
@ %

\section{A \texttt{Lexis} object with 3 causes of death}

With cause of death in a separate variable it is easy to set up a
\texttt{Lexis} object:
<<>>=
dmL <- Lexis(entry = list(per = dodm,
                          age = dodm - dobth,
                          tfD = 0),
              exit = list(per = dox),
       exit.status = codth,
              data = DMlate)
summary(dmL, t = T)
@ %
We can show the overall rates (the default \texttt{boxes} is
\emph{very} primitive):
<<boxes, fig = TRUE, height = 6, widtjh = 8>>=
boxes(dmL, boxpos = TRUE)
@ %
\insfig{boxes}{0.8}{Transitions from live to different causes of
  death. You probably want to explore the other arguments to
  \textrm{\tt boxes}.}

\section{Models for the rates}

In order to model the cause-specific mortality rates by sex and time from
diagnosis (\texttt{tfD}), we first split the data in 6-month intervals
<<>>=
sL <- splitLexis(dmL, time.scale = "age", breaks = seq(0, 120, 1/2))
summary(sL)
@
The code to fit the \texttt{gam} modes is (not run):
<<eval=FALSE, echo=FALSE>>=
mCVD <- gamLexis(sL, ~ s(tfD, by=sex), to = "CVD")
mCan <- gamLexis(sL, ~ s(tfD, by=sex), to = "Can")
mOth <- gamLexis(sL, ~ s(tfD, by=sex), to = "Oth")
@ %
\ldots and the code to fit \texttt{glm}s with splines is:
<<>>=
mCVD <- glmLexis(sL, ~ Ns(tfD, kn=1:6*2):sex, to = "CVD")
mCan <- glmLexis(sL, ~ Ns(tfD, kn=1:6*2):sex, to = "Can")
mOth <- glmLexis(sL, ~ Ns(tfD, kn=1:6*2):sex, to = "Oth")
@ %

\section{Derived measures}

With these three models for the occurrence rates we can compute the
cumulative risks of dying from each of the causes. We need a
prediction data frame that gives the rates at closely spaced times, in
this case for men. For women the code would be practically the same:
<<>>=
nm <- data.frame(tfD = seq(0, 15, 1/20), sex = "M")
@ %
Note that we can rename the states as we please by naming the entries
in the list of models we supply to \texttt{ci.Crisk}:
<<>>=
cR <- ci.Crisk(list(CVD = mCVD,
                    Can = mCan,
                  Other = mOth),
               nB = 500,
               nd = nm)
str(cR)
@ %
Note that we get three arrays: \texttt{Crisk}, the cumulative risks;
\texttt{Srisk}, the stacked risks and \texttt{Stime}, the sojourn
times in each state. Finally, for convenience we also have the
component \texttt{time}, the times at which the cumulative risks are
computed. It is also available as the clumpy expression
\texttt{as.numeric(dimnames(cR\$Crisk)[[1]])}, but \texttt{cR\$time}
is easier.

\subsection{Cumulative risks}

We can plot the cumulative risks for death from each of the three
causes, note we use the colors from last. Note that the time points
are in the \texttt{time} component of the \texttt{Crisk} object:
<<cR, fig=TRUE, width=8>>=
clr <- c("black", "orange", "limegreen")
matshade(cR$time, cbind(cR$Crisk[, "CVD"  , ],
                        cR$Crisk[, "Can"  , ],
                        cR$Crisk[, "Other", ]),
         col = clr, lty = 1, lwd = 2,
         plot = TRUE, ylim = c(0, 1/3), yaxs = "i")
text(0, 1/3 - c(2,3,1)/30, c("CVD", "Can", "Oth"),
     col = clr, adj = 0, font = 2)
@ %
\insfig{cR}{0.9}{Cumulative risks of each cause of death based on
\textrm{\tt gam} models for the cause-specific rates.}

We also have the stacked probabilities so we can show how the
population is distributed across the states at any one time:

\subsection{Stacked cumulative risks}

We also get the stacked probabilities in the order that we supplied the
models, so that if we plot these we get the probabilities of being
dead from each cause as the \emph{difference} between the curves. And
the confidence intervals are confidence intervals for the cumulative
sums of probabilities.
<<Sr1, fig = TRUE, width = 8>>=
matshade(cR$time, cbind(cR$Srisk[,1,],
                        cR$Srisk[,2,],
                        cR$Srisk[,3,]),
         col = "black", lty = 1, lwd = 2,
         plot = TRUE, ylim = c(0,1), xaxs = "i", yaxs = "i")
text(14, mp(c(0, cR$Srisk["14", , 1], 1)),
     rev(c(dimnames(cR$Crisk)[[2]])))
box(bty = "o")
@ %
\insfig{Sr1}{0.9}{Stacked cumulative risks.}

It is not a good idea to color these curves, they do not refer to the
causes of death, it is the areas \emph{between} the curves that refer
to causes. By the same token, since the quantity of interest is the
area between the curves and horizontal lines at 0 and 1, it is
important that the horizontal axes are placed at precisely 0 and 1 on
the vertical axis. This is what \texttt{yaxs = "i"} achieves.

It would be more logical to color the areas \emph{between} the
curves. which can be done by \texttt{mat2pol} (\texttt{mat}rix to
\texttt{pol}ygons) using the \texttt{Crisk} component. We can then
superpose the confidence intervals for the sum of the state
probabilities using \texttt{matshade} by adding white shades:
<<Sr2, fig = TRUE, width = 8>>=
zz <- mat2pol(cR$Crisk[, c("Other", "Can", "CVD", "Surv"), "50%"],
              x = cR$time,
           xlim = c(0,15), xaxs = "i", yaxs = "i", las = 1,
           xlab = "Time since DM diagnosis (years)",
           ylab = "Probability",
            col =  c("gray", "red", "blue", "limegreen") )
matshade(cR$time, cbind(cR$Srisk[,1,],
                        cR$Srisk[,2,],
                        cR$Srisk[,3,]),
         col = "transparent", col.shade = "white", alpha = 0.4)
text(14, mp(c(0, cR$Srisk["14", , 1], 1)),
     rev(c(dimnames(cR$Crisk)[[2]])), col = "white")
@ %
\insfig{Sr2}{0.9}{Stacked cumulative risks with coloring of states and
overlaid with confidence intervals for the probabilities shown; that
is the relevant sums.}

\subsection{Sojourn times}

The third component of the result, \texttt{Stime} is an array of
sojourn times over intervals starting at 0 and ending at the time
indicated by the first dimension:
<<>>=
ftable(round(cR$Stime[paste(1:5 * 3), , ], 1), row.vars = 1)
@ %
The sojourn times in the three dead states can be taken as the years
of life lost to each of the causes, the sum of the medians for the
three causes equals the time frame (5, 10, 15) minus the \texttt{Surv}
component.

So we see that during the first 15 years after diagnosis of diabetes,
the expected years alive is 10.9 years. The distribution of lifetime lost
between the causes is bogus in this case as the causes of death were
randomly generated.

\subsection{Comparing groups}

Finally, we may want to see the \emph{difference} (or ratio) of
survival probabilities between men and women, say. This can be derived
from two bootstrap samples using different prediction frames (the
argument \texttt{nd=} to \texttt{ci.Crisk}). But the two bootstrap
samples of parameters must be the same, i.e. come from the
\emph{same} stream of samples from the multivariate normal. This can
be obtained by explicitly setting the seed for the random number
generator to the same value before calling \texttt{ci.Crisk} with each
of the two different prediction frames as \texttt{nd} argument:
<<>>=
nm <- data.frame(tfD = seq(0, 15, 1/20), sex = "M")
nw <- data.frame(tfD = seq(0, 15, 1/20), sex = "F")
# set the seed
set.seed(1952)
mR <- ci.Crisk(list(CVD = mCVD,
                    Can = mCan,
                  Other = mOth),
               nd = nm,
               nB = 500,
          sim.res = "crisk" )
# REset the seed
set.seed(1952)
wR <- ci.Crisk(list(CVD = mCVD,
                    Can = mCan,
                  Other = mOth),
               nd = nw,
               nB = 500,
          sim.res = "crisk" )
str(wR)
@ %$
The two samples are now from identical streams of random numbers, so
we can get differences and ratios of the survival curves between men
and women:
<<>>=
dS <- mR[,"Surv",] - wR[,"Surv",]
dS <- apply(dS, 1, quantile, probs = c(.5, .025, .975)) * 100
str(dS)
rS <- mR[,"Surv",] / wR[,"Surv",]
rS <- apply(rS, 1, quantile, probs = c(.5, .025, .975))
@ %
We can then plot the differences and the ratios of the
probabilities---note that the dimension of the function
\texttt{apply}ed becomes the first dimension of the result:
<<difrat, fig = TRUE, width = 9>>=
par(mfrow = c(1,2))
matshade(as.numeric(colnames(dS)), t(dS), plot = TRUE,
         lwd = 3, ylim = c(-5, 5),
         xlab = "Time since DM diagnosis (years)",
         ylab = "Men - Women survival difference (%)")
abline(h = 0)
matshade(as.numeric(colnames(rS)), t(rS), plot = TRUE,
         lwd = 3, ylim = c(1/1.2, 1.2), log ="y",
         xlab = "Time since DM diagnosis (years)",
         ylab = "Men - Women survival ratio")
abline(h = 1)
@ %
\insfig{difrat}{1.0}{Differences and ratios of survival between men
  and women, derived from the same set of bootstrap samples from the
  parameter vector.}

To illustrate the effect of \emph{not} pairing the random samples we
can generate a fresh sample for women from a different stream (by
\texttt{not} setting the seed) and do the calculations to illustrate
the excess we get from not aligning samples.
<<>>=
fR <- ci.Crisk(list(CVD = mCVD,
                    Can = mCan,
                  Other = mOth),
               nd = nw,
               nB = 500,
          sim.res = "crisk" )
dxS <- mR[,"Surv",] - fR[,"Surv",]
dxS <- apply(dxS, 1, quantile, probs = c(.5, .025, .975)) * 100
rxS <- mR[,"Surv",] / fR[,"Surv",]
rxS <- apply(rxS, 1, quantile, probs = c(.5, .025, .975))
@ %
<<difratx, fig = TRUE, width = 9>>=
par(mfrow = c(1,2))
matshade(as.numeric(colnames(dS)), t(dS), plot = TRUE,
         lwd = 3, ylim = c(-5, 5),
         xlab = "Time since DM diagnosis (years)",
         ylab = "Men - Women survival difference (%)")
matshade(as.numeric(colnames(dxS)), t(dxS), lty = 3, col = "forestgreen")
abline(h = 0)

matshade(as.numeric(colnames(rS)), t(rS), plot = TRUE,
         lwd = 3, ylim = c(1/1.2, 1.2), log ="y",
         xlab = "Time since DM diagnosis (years)",
         ylab = "Men - Women survival ratio")
matshade(as.numeric(colnames(rxS)), t(rxS), lty = 3, col = "forestgreen")
abline(h = 1)
@ %
\insfig{difratx}{1.0}{Differences and ratios of survival between men
  and women, derived from separate bootstrap samples. The outer
  confidence bands are from bootstrap samples not properly paired
  between men end women.}

<<echo = FALSE>>=
ende <- Sys.time()
cat("  Start time:", format(anfang, "%F, %T"),
  "\n    End time:", format(  ende, "%F, %T"),
  "\nElapsed time:", round(difftime(ende, anfang, units = "mins"), 2), "minutes\n")
@ %

\end{document}

%\VignetteIndexEntry{Lexis functions for representation and analysis of follow-up data}
\SweaveOpts{results=verbatim, keep.source=TRUE, include=FALSE, eps=FALSE}
\documentclass[a4paper, dvipsnames, twoside, 12pt]{report}
\newcommand{\Title}{\texttt{Lexis} functions in \texttt{Epi}\\
  for representation and analysis of\\ follow-up data}
\newcommand{\Tit}{\texttt{Lexis} FU}
\newcommand{\Version}{Version 12}
\newcommand{\Dates}{February 2025}
\newcommand{\Where}{SDCC}
\newcommand{\Homepage}{\url{https://bendixcarstensen.com/Epi/} }
\newcommand{\Faculty}{\begin{tabular}{rl}
Bendix Carstensen
  & Steno Diabetes Center Copenhagen, Herlev, Denmark\\
  & {\small \& Department of Biostatistics,
               University of Copenhagen} \\
  & \texttt{b@bxc.dk} \quad \texttt{bcar0029@regionh.dk} \\
  & \url{https://BendixCarstensen.com/} \\[1em]
                      \end{tabular}}

\input{topreport}
\renewcommand{\rwpre}{./01flup}

\chapter*{Introduction}
\addcontentsline{toc}{chapter}{Introduction}

\section{Technicalities}
First we set some graphics parameters for convenience and load the
packages needed:
<<>>=
options(width = 90,
        show.signif.stars = FALSE,
        SweaveHooks=list(fig = function()
                         par(mar = c(3, 3, 1, 1),
                             mgp = c(3, 1, 0) / 1.6,
                             las = 1,
                            lend = "butt",
                             bty = "n")))
library(Epi)
library(popEpi)
library(survival)
clear()
@
% must be after clear() because 'anfang' is used at the end
<<echo = FALSE, results = hide>>=
anfang <- Sys.time()
cat("Start time:", format(anfang, "%F, %T"), "\n")
@ %
<<echo = FALSE>>=
vers <-
data.frame(R = substr(R.version.string, 11, 15),
         Epi = as.character(packageVersion(   "Epi")),
      popEpi = as.character(packageVersion("popEpi")))
names(vers) <- paste(" ", names(vers))
print(vers, row.names = FALSE)
@ %

\section{About this vignette}
This vignette is an introduction to the \texttt{Lexis} machinery in
the \texttt{Epi} package, intended for representation and manipulation
of follow-up data (``event history data'') from studies where exact
dates of events (state transitions) are known. It accommodates
follow-up through multiple states and on multiple time scales.

We use a data example from the \texttt{Epi} package to illustrate the
set-up of a simple \texttt{Lexis} object (a data frame of follow-up
intervals), as well as the subdivision of follow-up intervals needed
for multistate representation and analysis of transition rates by
flexible parametric functions.

The first chapter is exclusively on manipulation of the follow-up
representation, but it points to the subsequent chapter where analysis
of transition rates is based on a \texttt{Lexis} representation with
very small follow-up intervals.

Chapter 2 uses analysis of mortality rates among Danish diabetes
patients (available in the \texttt{Epi} package) currently on insulin
treatment or not, to illustrate the use of \texttt{Lexis} object in
the analysis of rates.

Chapter 3 discuss the use of multiple time scales,
chapter 4 discusses creation and manipulation of multistate data, and
chapter 5 is a list of all \texttt{Lexis} functions.

\section{History}

The \texttt{Lexis} machinery in the \texttt{Epi} package was first
conceived and implemented by Martyn
Plummer\cite{Plummer.2011,Carstensen.2011a}, and since its first
appearance in the \texttt{Epi} package in 2008 it has been expanded
with a number of utilities. An overview of these can be found in the
last chapter of this note, \hyperlink{lexfun}{``\texttt{Lexis}
  functions''}.

\subsection{Wilhelm Lexis}

\begin{minipage}[t]{0.6\linewidth}
  \raggedright
  The \code{Lexis} machinery is named after the German demographer
  and economist Wilhelm Lexis (full name Wilhelm Hector Richard
  Albrecht Lexis, 17 July 1837 -- 24 August 1914), who in his book
  ``Einleitung in die Theorie der Bev\"{o}lkerungsstatistik''
  (Introduction to the theory of population statistics, Strassburg,
  1875), devised a diagram showing follow-up of persons on two time
  scales, notably calendar time and age. The diagram that nowadays is
  called a Lexis diagram, is usually drawn in a slightly different
  manner than that Lexis used in his book.

  \quad The display of follow-up on two timescales naturally leads to
  representation on several time scales and statistical modeling of
  occurrence rates with two (or more) timescales as explanatory
  terms. Hence the naming of the machinery after Wilhelm Lexis.
\end{minipage}
\hfill
\begin{minipage}[t]{0.35\linewidth}
  \ \\[-1em]
  \includegraphics[width=1.0\textwidth]{Wilhelm_Lexis}
\end{minipage}

\subsection{Modeling of rates}

In 1976, Theodore R. Holford published a paper: ``Life Tables with
Concomitant Information'', \cite{Holford.1976} where he detailed
estimation in models for occurrence rates based on subdivision of
follow-up time in smaller intervals, including the handling of
time-dependent variables.  In 1980 John Whitehead published a paper:
``Fitting Cox's regression model to survival data using GLIM'',
\cite{Whitehead.1980} in which he devised the likelihood of a model
for many small time bands with constant intensity in each, and
demonstrated that Cox's partial likelihood could be seen as a Poisson
likelihood. It is essentially these two papers that underlie the
time-splitting and subsequent modeling of transition
rates in this vignette.\\[1em]
\noindent
\ldots so there is very little (if anything) new in this note.

\chapter{Representation of follow-up data in \texttt{Epi}}

In the \texttt{Epi}-package, follow-up data is represented by adding
some extra variables and a few attributes to a data frame. Such a data
frame is called a \texttt{Lexis} object. The tools for handling
follow-up data then use the structure of this for special plots,
tabulations and modeling.

Specifically, follow-up data requires a choice of time scale, a time
of entry, a time of exit and an indication of the status at exit
(normally either ``alive'' or ``dead'') for each person. Implicitly is
also assumed a status \emph{during} the follow-up (usually ``alive'').

\begin{figure}[htbp]
  \centering
\setlength{\unitlength}{1pt}
\begin{picture}(210, 70)(0, 75)
%\scriptsize
\thicklines
 \put(  0, 80){\makebox(0, 0)[r]{Age-scale}}
 \put( 50, 80){\line(1, 0){150}}
 \put( 50, 80){\line(0, 1){5}}
 \put(100, 80){\line(0, 1){5}}
 \put(150, 80){\line(0, 1){5}}
 \put(200, 80){\line(0, 1){5}}
 \put( 50, 77){\makebox(0, 0)[t]{35}}
 \put(100, 77){\makebox(0, 0)[t]{40}}
 \put(150, 77){\makebox(0, 0)[t]{45}}
 \put(200, 77){\makebox(0, 0)[t]{50}}

 \put(  0, 115){\makebox(0, 0)[r]{Follow-up}}

 \put( 80, 105){\makebox(0, 0)[r]{\small Two}}
 \put( 90, 105){\line(1, 0){87}}
 \put( 90, 100){\line(0, 1){10}}
 \put(100, 100){\line(0, 1){10}}
 \put(150, 100){\line(0, 1){10}}
 \put(180, 105){\circle{6}}
 \put( 95, 110){\makebox(0, 0)[b]{1}}
 \put(125, 110){\makebox(0, 0)[b]{5}}
 \put(165, 110){\makebox(0, 0)[b]{3}}

 \put( 50, 130){\makebox(0, 0)[r]{\small One}}
 \put( 60, 130){\line(1, 0){70}}
 \put( 60, 125){\line(0, 1){10}}
 \put(100, 125){\line(0, 1){10}}
 \put(130, 130){\circle*{6}}
 \put( 80, 135){\makebox(0, 0)[b]{4}}
 \put(115, 135){\makebox(0, 0)[b]{3}}
\end{picture}
  \caption{\it Follow-up of two persons on the age-scale}
  \label{fig:fu2}
\end{figure}

\section{Time scales}

A time scale is a variable that varies deterministically \emph{within}
each person during follow-up, \textit{e.g.}:
\begin{itemize}[noitemsep]
  \item Age
  \item Calendar time
  \item Time since start of treatment
  \item Time since relapse
  \end{itemize}
All time scales advance at the same pace, so the time followed is the
same on all time scales. Therefore, it will suffice to use only the
entry point on each of the time scales, for example:
\begin{itemize}[noitemsep]
  \item Age at entry
  \item Date of entry
  \item Time at treatment---\emph{at} treatment, time since treatment
    is 0, but persons may enter before or after treatment beginning.
  \item Time at relapse---\emph{at} relapse, time since relapse
    is 0, but persons may enter before or after relapse.
\end{itemize}
In the \texttt{Epi} package, follow-up in a cohort is represented in a
\texttt{Lexis} object. A \texttt{Lexis} object is a data frame with
some extra structure to represent the follow-up. For the
\texttt{DMlate} dataset of follow-up of diabetes patients in Denmark---
with recorded date of birth, date of diabetes, date of first insulin use,
date of first oral drug use, date of exit and date of death---we can
construct a \texttt{Lexis} object by first including follow-up from
entry at date of diabetes (\texttt{dodm}) to exit (\texttt{dox}).
The dates should \emph{not} be in \texttt{Date} format; some data
manipulations in \texttt{Lexis} will crash if they are.
<<>>=
data(DMlate)
head(DMlate)
dmL <- Lexis(entry = list(per = dodm,
                          age = dodm-dobth,
                          tfD = 0),
              exit = list(per = dox),
       exit.status = factor(!is.na(dodth),
                            labels = c("DM", "Dead")),
              data = DMlate)
timeScales(dmL)
@ %
The 4 excluded persons are persons with date of diabetes equal to date
of death.

The \texttt{entry} argument is a \emph{named} list with the entry
points on each of the time scales we want to use. The names of the
list defines the names of the time scales. The \texttt{exit} argument
gives the exit time on \emph{one} of the time scales, so the name of
the element in this list must match one of the names of the
\texttt{entry} list. This is sufficient, because the follow-up time on
all time scales is the same, in this case
$\mathtt{dox}-\mathtt{dodm}$.

The \texttt{exit.status} will normally be a categorical variable (a
\emph{factor}) that indicates the exit status --- in this case whether
the person (still) is in state \texttt{DM} or exits to \texttt{Dead}
at the end of follow-up. We could also specify an
\texttt{entry.status}; the default is to assume that all persons enter
in the \emph{first} level of the factor \texttt{exit.state}s --- in
this case \texttt{DM} (because $\mathtt{FALSE}<\mathtt{TRUE}$).

Now take a look at the result:
<<>>=
str(dmL)
head(dmL)[, 1:11]
@ %
The \texttt{Lexis} object \texttt{dmL} has a variable for each time
scale, the value of which is the entry time for each person on this time
scale. The length of the follow-up time is in the variable
\texttt{lex.dur} (\texttt{dur}ation).  Note that the exit status is in
the variable \texttt{lex.Xst} (e\texttt{X}it \texttt{st}ate). The
variable \texttt{lex.Cst} indicates the state where follow-up takes
place (\texttt{C}urrent \texttt{st}ate), in this case \texttt{DM}
(alive with diabetes) for all persons. This implies that observations
\emph{censored} in state \texttt{A}, say, are characterized by
having $\mathtt{lex.Cst} = \mathtt{lex.Xst} = \mathtt{A}$.

There is a \texttt{summary} function for \texttt{Lexis} objects that
lists the number of transitions and records as well as the total
amount of follow-up time; it also (optionally) prints information
about the names of the variables that constitute the time scales:
<<>>=
summary(dmL, timeScales = TRUE)
@ %
It is possible to get a visualization of the follow-up along the
time scales chosen by using the \texttt{plot} method for \texttt{Lexis}
objects. \texttt{dmL} is an object of \emph{class} \texttt{Lexis}, so
using the function \texttt{plot()} on it means that \R\ will look for
the function \texttt{plot.Lexis} and use this function.

To limit the size of the graphs we only use a 5\% sample of the
persons:
<<dmL1, fig = TRUE>>=
set.seed(1952)
dmS <- bootLexis(dmL, size = nid(dmL) / 20, replace = FALSE)
summary(dmL)
summary(dmS)
plot(dmS)
@ %
The function \texttt{plot.Lexis} allows quite a bit of control over
the output, and a \texttt{points.Lexis} function allows plotting of
the endpoints of follow-up:
<<dmL2, fig = TRUE>>=
par(mar = c(3, 3, 1, 1), mgp = c(3, 1, 0) / 1.6)
plot(dmS, 1:2, lwd = 1, col = c("blue", "red")[dmS$sex],
     grid = TRUE, lty.grid = 1, col.grid = gray(0.7),
     xlim = 1960 + c(0, 60), xaxs = "i",
     ylim =   40 + c(0, 60), yaxs = "i", las = 1)
points(dmS, 1:2, pch = c(NA, 3)[dmS$lex.Xst],
       col = "lightgray", lwd = 3, cex = 0.3)
points(dmS, 1:2, pch = c(NA, 3)[dmS$lex.Xst],
       col = c("blue", "red")[dmS$sex], lwd = 1, cex = 0.3)
box(bty = 'o')
@ %
In the above code you will note that the values of the arguments
\texttt{col} and \texttt{pch} are indexed by factors, using the
convention in \R\ that the index is taken as \emph{number of the
  level} of the supplied factor. Thus \texttt{c("blue",
  "red")[dmS\$sex]} is \texttt{"blue"} when \texttt{sex} is \texttt{M}
(the first level of \texttt{sex}) and.  \texttt{"red"} when
\texttt{sex} is \texttt{F} (the second level of \texttt{sex}).

The results of these two plotting commands are in figure
\ref{fig:Lexis-diagram}, p. \pageref{fig:Lexis-diagram}.
\begin{figure}[tb]
\centering
\includegraphics[width = 0.35\textwidth]{01flup-dmL1}
\includegraphics[width = 0.63\textwidth]{01flup-dmL2}
\caption{\it Lexis diagram of the \textrm{\tt DMlate} dataset; left
  panel is the default version, right panel is with some bells and
  whistles. The red lines are for women, blue for men, crosses
  indicate deaths.}
\label{fig:Lexis-diagram}
\end{figure}

\section{Splitting the follow-up time along a time scale}

In next chapter we shall conduct statistical analysis of mortality
rates, and a prerequisite for parametric analysis of rates is that
follow-up time is subdivided in smaller intervals, where we can
reasonably assume that rates are constant.

The follow-up time in a cohort can be subdivided (``split'') along a
time scale, for example current age. This is achieved by the
\texttt{splitLexis} (note that it is \emph{not} called
\texttt{split.Lexis}). This requires that the time scale and the
breakpoints on this time scale are supplied. Try:
<<>>=
dmS1 <- splitLexis(dmL, "age", breaks = seq(0, 100, 5))
summary(dmL)
summary(dmS1)
@ %
We see that the number of persons and events and the amount of
follow-up is the same in the two data sets; only the number of records
differ --- the extra records all have \texttt{lex.Cst} = \texttt{DM} and
\texttt{lex.Xst} = \texttt{DM}.

To see how records are split for each individual, it is useful to list
the results for a few individuals (whom we selected with a view to the
illustrative usefulness):
<<>>=
wh.id <- c(9, 27, 52, 484)
subset(dmL , lex.id %in% wh.id)[, 1:10]
subset(dmS1, lex.id %in% wh.id)[, 1:10]
@ %
The resulting object, \texttt{dmS1}, is again a \texttt{Lexis} object.
Note that the values of the timescales (\texttt{per}, \texttt{age},
\texttt{tfD}) are updated for each of the the resulting intervals.
The follow-up in \texttt{dmS1} may be split further along another time
scale, for example diabetes duration, \texttt{tfD}. Subsequently we
list the results for the chosen individuals:
<<>>=
dmS2 <- splitLexis(dmS1, "tfD", breaks = c(0, 1, 2, 5, 10, 20, 30, 40))
subset(dmS2, lex.id %in% wh.id)[, 1:10]
@ %
A more efficient (and more intuitive) way of making this double split
is to use the \texttt{splitMulti} function from the \texttt{popEpi}
package:
<<>>=
dmM <- splitMulti(dmL,
                  age = seq(0, 100, 5),
                  tfD = c(0, 1, 2, 5, 10, 20, 30, 40),
                 drop = FALSE)
summary(dmS2)
summary(dmM)
@ %
Note we used the argument \texttt{drop = FALSE} which will retain
follow-up also outside the window defined by the range of the
breaks. Otherwise, the default for \texttt{splitMulti} would be to drop
follow-up outside \texttt{age} [0, 100] and \texttt{tfD} [0, 40]. This
clipping behavior is not available in \texttt{splitLexis},
nevertheless this may be exactly what we want in some situations.

The recommended way of splitting follow-up time is by
\texttt{splitMulti}, because it is faster. But you should be aware
that the result is a \texttt{data.table} object unless you set the
option \texttt{"popEpi.datatable" = FALSE}.  In some circumstances
\texttt{data.table}s behaves slightly differently from
\texttt{data.frame}s. See the manual for \texttt{data.table}.

\section{Cutting follow up time at dates of intermediate events}

If we have a recording of the date of a specific event as for example
recovery or relapse, we may classify follow-up time as being before or
after this intermediate event, but it requires that follow-up records
that straddle the event be cut in two and placed in separate records,
one representing follow-up \emph{before} the intermediate event, and another
representing follow-up \emph{after} the intermediate event. This is
achieved with the function \texttt{cutLexis}, which takes three
arguments: the time point of the intermediate event, the time scale
that this point refers to, and the value of the (new) state following
the date. Optionally, we may also define a new time scale with the
argument \texttt{new.scale = }.

We are interested in the time before and after inception of insulin
use, which occurs at the date \texttt{doins}:
<<>>=
subset(dmL, lex.id %in% wh.id)[, 1:11]
dmC <- cutLexis(data = dmL,
                 cut = dmL$doins,
           timescale = "per",
           new.state = "Ins",
           new.scale = "tfI")
subset(dmC, lex.id %in% wh.id)[, 1:11]
@ %
Note that the process of cutting time is simplified by having all
types of events referred to the calendar time scale. This is a
generally applicable advice in handling follow-up data: Get all event
times as \emph{dates}, location of events and follow-up on other time
scales can then easily be derived from this.

Note that individual 52 has had his follow-up cut at 6.55 years from
diabetes diagnosis and individual 484 at 5.70 years from diabetes
diagnosis. This dataset could then be split along the time scales as
we did before with \texttt{dmL}.

The result of this can also be achieved by cutting the split dataset
\texttt{dmS2} instead of \texttt{dmL}:
<<>>=
dmS2C <- cutLexis(data = dmS2,
                   cut = dmS2$doins,
             timescale = "per",
             new.state = "Ins",
             new.scale = "tfI")
subset(dmS2C, lex.id %in% wh.id)[, 1:11]
@ % $
Thus it does not matter in which order we use \texttt{splitLexis} and
\texttt{cutLexis}. Mathematicians would say that \texttt{splitLexis}
and \texttt{cutLexis} are commutative.

Note that for \texttt{lex.id} = 484, the follow-up subsequent to the
event is classified as being in state \texttt{Ins}, but that the final
transition to state \texttt{Dead} is preserved.

Note that we defined a new time scale, \texttt{tfI}, using the
argument \texttt{new.scale = "tfI"}. This has a special status relative to
the other three time scales: it is defined as time since entry into a
state, namely \texttt{Ins}, this is noted in the time scale part of
the summary of \texttt{Lexis} object --- the information sits in the
attribute \texttt{time.since} of the \texttt{Lexis} object, which can
be accessed by the function \texttt{timeSince()} or through the
\texttt{summary()}:
<<>>=
summary(dmS2C, timeScales = TRUE)
@ %
Finally we can get a quick overview of the states and transitions by
using \texttt{boxes} --- \texttt{scale.R} scales transition rates to
rates per 1000 person-years:
<<box1, fig = TRUE, height = 6, width = 8>>=
boxes(dmC, boxpos = TRUE, scale.R = 1000, show.BE = TRUE)
legendbox(70, 95)
@ %
\insfig{box1}{0.8}{States, person years, transitions and rates in the
  cut dataset. The numbers \emph{in} the boxes are person-years and
  the number of persons \texttt{B}eginning, resp. \texttt{E}nding
  their follow-up in each state (triggered by \textrm{\tt
    show.BE = TRUE}). The numbers at the arrows are the number of
  transitions and transition rates per 1000 (triggered by \textrm{\tt
  scale.R = 1000}).}
The explanatory box in the upper right corner was generated by \texttt{legendbox}.

\chapter{Modeling rates from \texttt{Lexis} objects}

\section{Covariates}

In the \texttt{Lexis} dataset \texttt{dmS2C} there are three types of
covariates that can be used to describe mortality rates:
\begin{enumerate}[noitemsep]
\item time-dependent covariates
\item time scales
\item fixed covariates
\end{enumerate}

There is only one time-dependent covariate, namely \texttt{lex.Cst},
the current state of the person's follow up; it takes the values
\texttt{DM} and \texttt{Ins} according to whether the person has ever
purchased insulin before the beginning of a given follow-up interval.

The time-scales are obvious candidates for explanatory variables for
the rates, notably age and time from diagnosis (duration of diabetes)
and time since start of insulin.

\subsection{Time scales as covariates}

If we want to model the effect of the time scale variables on
occurrence rates, we will for each interval use either the value of
the left endpoint in each interval or the middle. There is a function
\texttt{timeBand} which returns either of these:
<<>>=
timeBand(dmS2C, "age", "middle")[1:10]
# For nice printing and column labelling we use the data.frame() function:
data.frame(dmS2C[, c("per", "age", "tfD", "lex.dur")],
           mid.age = timeBand(dmS2C, "age", "middle"),
             mid.t = timeBand(dmS2C, "tfD", "middle"),
            left.t = timeBand(dmS2C, "tfD", "left"  ),
           right.t = timeBand(dmS2C, "tfD", "right" ),
            fact.t = timeBand(dmS2C, "tfD", "factor"))[1:15, ]
@ %
Note that the values of these functions are characteristics of the
intervals defined by \texttt{breaks = }, \emph{not} the midpoints nor
left or right endpoints of the \emph{actual} follow-up intervals
(which would be \texttt{tfD} and \texttt{tfD+lex.dur}, respectively).

These functions are intended for modeling time scale variables as
factors (categorical variables) in which case the coding must be
independent of the censoring and mortality pattern --- it should only
depend on the chosen grouping of the time scale. Modeling time scales as
\emph{quantitative} should not be based on these codings but directly
on the values of the time-scale variables, i.e. the left endpoints
of the intervals.

\subsection{Differences between time scales}

Apparently, the only fixed variable is \texttt{sex}, but the dates of
birth (\texttt{dobth}), diagnosis (\texttt{dodm}) and first insulin
purchase (\texttt{doins}) are available as fixed covariates too. These
can be constructed as differences between time scales.  These would
then be age at birth (hardly relevant since it is the same for all
persons), age at diabetes diagnosis and age at insulin treatment.

\subsection{Keeping the relation between time scales}

The midpoint (as well as the right interval endpoint) should be used
with caution if the variable age at diagnosis, \texttt{dodm-dobth}, is
modeled too; the age at diabetes is logically equal to the difference
between current age (\texttt{age}) and time since diabetes diagnosis
(\texttt{tfD}):
<<>>=
summary((dmS2$age - dmS2$tfD) - (dmS2$dodm - dmS2$dobth))
@ %
This calculation refers to the value of the timescales at the
\emph{beginning} of each interval --- which are in the timescale
variables in the \texttt{Lexis} object. But when using the middle of
the intervals, this relationship is not preserved:
<<>>=
summary(timeBand(dmS2, "age", "middle") -
        timeBand(dmS2, "tfD", "middle") -
        (dmS2$dodm - dmS2$dobth))
@ %
If all three variables are to be included in a model, we must make
sure that the \emph{substantial} relationship between the variables be
maintained. One way is to recompute age at diabetes diagnosis from the two
midpoint variables, but more straightforward would be to use the left
endpoint of the intervals, that is the time scale variables in the
\texttt{Lexis} object.

If we dissolve the relationship between the variables \texttt{age},
\texttt{tfD} and age at diagnosis by grouping we may obtain
identifiability of the three separate effects, but it will be at the
expense of an arbitrary allocation of a linear trend between the three
effects..

For the sake of clarity, consider current age, $a$, age at diagnosis
$e$ and duration of disease, $d$, where
\[
 \text{current age} =
 \text{age at diagnosis} +
 \text{disease duration},
 \quad \text{\ie} \quad a = e + d \quad
 \Leftrightarrow \quad e + d - a = 0
\]
If we model the effect of the quantitative variables $a$, $e$ and $d$
on the log-rates by three functions $f$, $g$ and $h$:
$\log(\lambda) = f(a) + g(d) + h(e)$
then for any $\kappa$:
\begin{align*}
   \log(\lambda) & = f(a)+g(d)+h(e)+\kappa(e+d-a)\\
                 & =
   \big(f(a)-\kappa a \big)+
   \big(g(d)+\kappa d \big)+
   \big(h(e)+\kappa e \big) \\
& = \tilde f(a)+ \tilde g(d)+ \tilde h(e)
\end{align*}
In practical modeling this will turn up as a singular model matrix
with one parameter aliased, corresponding to some arbitrarily chosen
value of $\kappa$ (depending on software conventions for singular
models). This phenomenon is well known from age-period-cohort models
\cite{Carstensen.2007a}.

Thus we see that we can move any slope around between the three terms,
so if we achieve identifiability by using grouping of one of the
variables we will in reality have settled for a particular value of
$\kappa$, without knowing how and why we chose just that. There is
\emph{no way} to separate the three effects. The only resorts are
either to stick to predictions which are independent of the particular
parametrization or to choose a parametrization with explicitly defined
constraints clearly stated.

\section{Modeling of rates}

As mentioned, the purpose of subdividing follow-up data in smaller
intervals is to be able to model effects of time scale variables as
parametric functions. When we split along a time scale we can get
intervals that are as small as we want; if they are sufficiently
small, an assumption of constant rates in each interval becomes
reasonable.

In a model that assumes a constant occurrence rate in each of the
intervals, the likelihood contribution from each interval is the same
as the likelihood contribution from a Poisson variate $D$, say, with
mean $\lambda \ell$ where $\lambda$ is the rate and $\ell$ is the
interval length, and where the value of the variate $D$ is 1 or 0
according to whether an event has occurred or not. Moreover, the
likelihood contributions from all follow-up intervals from a single
person are \emph{conditionally} independent (conditional on having
survived till the start of the interval in question). This implies
that the total contribution to the likelihood from a single person is
a product of terms, and hence the same as the likelihood of a number
of independent Poisson terms, one from each interval.

Note that the observations are neither Poisson distributed (\eg they
can only ever assume values 0 or 1) nor independent --- it is only the
\emph{likelihood} for the follow-up data that happens to be the same
as the likelihood from independent Poisson variates because it is
a product of terms. Different models can have the same likelihood; a
model cannot be inferred from its likelihood.

Parametric modeling of the rates is obtained by using the
\emph{values} of the time scales for each interval as \emph{quantitative}
explanatory variables, using for example splines. And of course also
the values of the fixed covariates and the time-dependent variables
for each interval. Thus the model will be one where the rate is
assumed constant in each (small) interval, but where a parametric form
of the \emph{size} of the rate in each interval is imposed by the
model, using the time scale as a quantitative covariate.

\subsection{Interval length}

In the first chapter we illustrated cutting and splitting by listing
the results for a few individuals across a number of intervals. For
illustrational purposes we used 5-year age bands to avoid excessive
listings, but since the doubling time for mortality on the age scale
is only slightly larger than 5 years, the assumption about constant
rates in each interval would be pretty far fetched if we were to use 5
year intervals.

Thus, for modeling purposes we split the follow-up in 3 month
intervals. When we use intervals of 3 months length it is superfluous
to split along multiple time scales --- the precise location of
tightly spaced splits will be irrelevant from any practical point of
view. \texttt{splitLexis} and \texttt{splitMulti} will allocate the
actual split times for all of the time scale variables, so these can be
used directly in modeling.

So we split the cut dataset in 3 months intervals along the age
scale:
<<>>=
dmCs <- splitLexis(dmC, time.scale = "age", breaks = seq(0, 110, 1/4))
summary(dmCs, t = T)
@ %
We see that we now have 228,748 records and 9996 persons, so about 23
records per person. The total risk time is 54,275 years, a bit less
than 3 months on average per record as expected.

\subsection{Practicalities for splines}

In this study we want to look at how mortality depend on age
(\texttt{age}) and time since start of insulin use (\texttt{tfI}). If
we want to use splines in the description we must allocate knots for
anchoring the splines at each of the time scales, either by some
\textit{ad hoc} method or by using some sort of penalized splines as
for example by \texttt{gam}; the latter will not be treated here; it
belongs in the realm of the \texttt{mgcv} package.

Here we shall use the former approach and allocate 5 knots on each of
the time-scales. We allocate knots so that we have the events evenly
distributed between the knots. Since the insulin state starts at 0 for
all individuals we include 0 as the first knot, such that any set of natural
splines with these knots will have the value 0 at 0 on the time
scale.
<<>>=
(a.kn <- with(subset(dmCs, lex.Xst == "Dead"),
              quantile(age+lex.dur, seq(5, 95, , 5)  /100)))
(i.kn <- c(0,
           with(subset(dmCs, lex.Xst == "Dead" & lex.Cst == "Ins"),
                quantile(tfI+lex.dur, seq(20, 95, , 4) / 100))))
@ %
In the \texttt{Epi} package there is a convenience wrapper,
\texttt{Ns}, for the \texttt{n}atural \texttt{s}pline generator
\texttt{ns}, that takes the smallest and the largest of a set of
supplied knots to be the boundary knots, so the explicit definition of
the boundary knots becomes superfluous.

Note that it is a feature of the \texttt{Ns} (via the features of
\texttt{ns}) that any generated spline function is 0 at the leftmost
knot (the smaller of the boundary knots).

\subsection{Poisson models}

A model that describes mortality rates as a function of only age
(ignoring the insulin status) would then be:
<<>>=
ma <- glm((lex.Xst == "Dead") ~ Ns(age, knots = a.kn),
           family = poisson,
           offset = log(lex.dur),
             data = dmCs)
summary(ma)
@ %
The offset, \texttt{log(lex.dur)} comes from the fact that the
likelihood for the follow-up data during $\ell$ time is the same as
that for independent Poisson variates with mean $\lambda \ell$, and
that the default link function for the Poisson family is the log, so
that we are using a linear model for the log-mean,
$\log(\lambda) + \log(\ell)$.  But when we want a model for the
log-rate ($\log(\lambda)$), the term $\log(\ell)$ must still be
included as a covariate, but with regression coefficient fixed to 1; a
so-called \emph{offset}. This is however a technicality; it just
exploits that the likelihood of a particular Poisson model and that of
the rates model is the same.

In the \texttt{Epi} package is a \texttt{glm} family, \texttt{poisreg},
that has a more intuitive interface to the likelihood of rates, namely
where the response is a 2-column matrix of events and person-time,
respectively. This is in concert with the fact that the outcome
variable in follow-up studies is bivariate: (event, risk time).
<<>>=
Ma <- glm(cbind(lex.Xst == "Dead", lex.dur) ~ Ns(age, knots = a.kn),
          family = poisreg,
            data = dmCs)
summary(Ma)
@ %
There is a convenience wrapper for \texttt{glm} with the
\texttt{poisreg} family, exploiting the multistate structure in the
\texttt{Lexis} object. It just requires specification of the
transitions in terms of the arguments \texttt{from} and \texttt{to}:
<<>>=
Xa <- glmLexis(dmCs, formula = ~ Ns(age, knots = a.kn),
                     from = "DM", to = "Dead",)
@ %
The result is a \texttt{glm} object but with an extra attribute,
\texttt{Lexis} with the name of the data, transition(s) modeled and
model formula
<<>>=
attr(Xa, "Lexis")
@ %
There are similar wrappers for \texttt{gam} and \texttt{coxph} models,
\texttt{gamLexis} and \texttt{coxphLexis}, but these will not be
elaborated in detail here.

The \texttt{from=} argument can be omitted, in which case all possible
transitions \emph{into} any of the ``\texttt{to}'' states is
modeled. Similarly \texttt{to=} can be omitted, it defaults to the set
of absorbing states. There are a couple of functions that show the
absorbing and transient states:
<<>>=
transient(dmCs)
absorbing(dmCs)
preceding(dmCs, absorbing(dmCs))
@
So the default will be to model transitions from \texttt{DM} and
\texttt{Ins} to \texttt{Dead}:
<<>>=
xa <- glmLexis(dmCs, formula = ~ Ns(age, knots = a.kn))
@
We can check if the four models fitted are the same:
<<>>=
c(ma = deviance(ma),
  Ma = deviance(Ma),
  Xa = deviance(Xa),
  xa = deviance(xa))
@ %
Oops! the model \texttt{Xa} is apparently not the same as the other
three? This is because the explicit specification
\verb|from = "DM", to = "Dead"|, omits modeling contributions from the
$\mathtt{Ins}\rightarrow\mathtt{Dead}$ transition --- the output
actually said so --- see also figure \ref{fig:box1} on
p. \pageref{fig:box1}. The other three models all use both transitions
--- and assume that the two transition rates are the same, \ie that
start of insulin has no effect on mortality. We shall relax this
assumption later.

The parameters from the model do not have any direct interpretation
\textit{per se}, but we can compute the estimated mortality rates for
a range of ages using \texttt{ci.pred} with a suitably defined
prediction data frame.

Note that if we use the \texttt{poisson} family of models, we must
specify all covariates in the model, including the variable in the
offset, \texttt{lex.dur} (remember that this was a covariate with
coefficient fixed at 1). We set the latter to 1000, because we want the
mortality rates per 1000 person-years. Using the \texttt{poisreg}
family, the prediction will ignore any value of \texttt{lex.dur}
specified in the prediction data frame, the returned rates will be per
unit in which \texttt{lex.dur} is recorded when fitting the model.
<<pr-a, fig = TRUE>>=
nd <- data.frame(age = 40:85, lex.dur = 1000)
pr.0 <- ci.pred(ma, newdata = nd)      # mortality per 1000 PY
pr.a <- ci.pred(Ma, newdata = nd)*1000 # mortality per 1000 PY
summary(pr.0 / pr.a)
matshade(nd$age, pr.a, plot = TRUE,
         type = "l", lty = 1,
         log = "y", xlab = "Age (years)",
         ylab = "DM mortality per 1000 PY")
@ % $
\insfig{pr-a}{0.8}{Mortality among Danish diabetes patients by age
  with 95\% CI as shaded area. We see that the rates increase linearly
  on the log-scale, that is, exponentially by age.}

\section{Time dependent covariates}

One approach to modeling mortality rates by insulin status would be to
assume that the mortality rate-ratio between patients on insulin and
not on insulin is a fixed quantity, independent of time since start of
insulin and independent of age. This is commonly termed a proportional
hazards assumption, because the rates (hazards) in the two groups are
proportional along the age (baseline time) scale.
<<>>=
pm <- glm(cbind(lex.Xst == "Dead", lex.dur) ~ Ns(age, knots = a.kn)
                                              + lex.Cst + sex,
          family = poisreg,
            data = dmCs)
round(ci.exp(pm), 3)
@ %
Again we can simplify the code using \code{glmLexis}:
<<>>=
pm <- glmLexis(dmCs, ~ Ns(age, knots = a.kn) + lex.Cst + sex)
round(ci.exp(pm), 3)
@ %
So we see that persons on insulin have about twice the mortality of
persons not on insulin and that women have 2/3 the mortality of men.

\chapter{Multiple time scales}

\section{Time since insulin start}

If we want to assess how the excess mortality depends on the time
since start of insulin treatment, we can add a spline term in
\texttt{tfI}, \texttt{t}ime \texttt{f}rom \texttt{I}nsulin start. But
since \texttt{tfI} is a time scale defined as time since entry into a
new state (\texttt{Ins}), the variable \texttt{tfI} is missing
for those in the \texttt{DM} state, so before modeling we must set the
\texttt{NA}s to 0, which we do with \texttt{tsNA20} (acronym for
\texttt{t}ime\texttt{s}cale \texttt{NA}s to zero):
<<>>=
pm <- glm(cbind(lex.Xst == "Dead", lex.dur) ~ Ns(age, knots = a.kn)
                                            + Ns(tfI, knots = i.kn)
                                            + lex.Cst + sex,
          family = poisreg,
            data = tsNA20(dmCs))
@ %
As noted before we could do this simpler with \texttt{glmLexis}, even
without the \texttt{from=} and \texttt{to=} arguments, because we are
modeling all transitions \emph{into} the absorbing state
(\texttt{Dead}):
<<>>=
Pm <- glmLexis(tsNA20(dmCs),
               form = ~ Ns(age, knots = a.kn)
                      + Ns(tfI, knots = i.kn)
                      + lex.Cst + sex)
c(deviance(Pm), deviance(pm))
identical(model.matrix(Pm), model.matrix(pm))
@ %
The coding of the effect of \texttt{tfI} is so that the value is 0 at
0, so the meaning of the estimate of the effect of \texttt{lex.Cst} is
the RR between persons with and without insulin, immediately after
start of insulin:
<<>>=
round(ci.exp(Pm, subset = "ex"), 3)
@ %
We see that the effect of sex is pretty much the same as before, but
the effect of \texttt{lex.Cst} is much larger; it now refers to a
different quantity, namely the RR between persons just started on
insulin (i.e. at time \texttt{tfI} = 0) and persons not on insulin.
In the model \texttt{pm} above, the effect of \texttt{lex.Cst} was the
average effect of insulin exposure, assuming that it was constant over
time since start of insulin.

If we want to see the effect of time since insulin, it is best viewed
jointly with the effect of age, so we construct a prediction data
frame --- a data frame with the explanatory variables from the model
and values of these for which we want to see the predicted occurrence
rates:
<<ins-time, fig = TRUE>>=
ndI <- data.frame(expand.grid(tfI = c(NA, seq(0, 15, 0.1)),
                               ai = seq(40, 80, 10)),
                  lex.Cst = "Ins",
                      sex = "M")
ndI <- transform(ndI, age = ai + tfI)
head(ndI)
ndA <- data.frame(age = seq(40, 100, 0.1),
                  tfI = 0,
              lex.Cst = "DM",
                  sex = "M")
pri <- ci.pred(Pm, ndI) * 100
pra <- ci.pred(Pm, ndA) * 100
matshade(ndI$age, pri, plot = TRUE,
         xlab = "Attained age (years)", ylab = "DM mortality per 100 PY",
         las = 1, log = "y", lty = 1, col = "blue")
matshade(ndA$age, pra)
@ %
\begin{expl}
  \texttt{expand.grid} yields a data frame with all combinations of
  \texttt{tfI} and \texttt{ai}, the latter is age at insulin start; we
  want predictions for different values of this. But it is (current)
  \texttt{age} that is in the model, so we must construct this. The
  \texttt{NA}s are inserted in order to produce separate curve for
  each value of \texttt{ai}.

  The prediction data frame for persons not on insulin is simpler, but
  must still include the \texttt{tfI} variable, but now uniformly set
  to 0.

  \texttt{ci.pred} will give predicted rates from the \texttt{Pm}
  model, per 1 person-year (because \texttt{lex.dur} is in years), so
  we multiply by 100 to get rates per 100 PY (\% / year).

  \texttt{matshade} produces curves with shaded confidence bands.
\end{expl}
\insfig{ins-time}{0.8}{Mortality rates of persons on insulin, starting
insulin at ages 40, 50, \ldots, 80 (blue), compared with persons not on
insulin (black curve). Shaded areas are 95\% CI.}

In figure \ref{fig:ins-time}, p. \pageref{fig:ins-time}, we see that
mortality is high just after insulin start, but falls by almost a
factor 3 during the first year. Also we see that there is a tendency
that mortality in a given age is smallest for those with the longest
duration of insulin use. Or among those who started insulin first ---
the two effects cannot be separated.

\section{The Cox model}

In the implementation of the Cox-model with \texttt{age} as baseline
time scale, \texttt{age} appears as response variable, slightly
counter-intuitive since it really is a covariate. Hence the age part
of the linear predictors is not in the specification of the
covariates:
<<>>=
cm <- coxph(Surv(age, age + lex.dur, lex.Xst == "Dead") ~
            Ns(tfI, knots = i.kn) + lex.Cst + sex,
            data = tsNA20(dmCs))
@ %
There is also a multistate wrapper for Cox models, requiring a
l.h.s. side for the \texttt{formula =} argument:
<<>>=
Cm <- coxphLexis(tsNA20(dmCs),
                  formula = age ~ Ns(tfI, knots = i.kn) + lex.Cst + sex)
round(cbind(ci.exp(cm), ci.exp(Cm)), 4)
@ %
Note that this is really a model with two time scales: the baseline
time scale \texttt{age} and the time since insulin, \texttt{tfi}. The
effects of age and time since insulin are modeled differently,
\texttt{age} non-parametrically and \texttt{tfI} with a smooth
parametric spline. And only the spline effects is shown in the
parameters.

We can compare the estimates of the insulin effect from the Cox model
with those from the Poisson model --- we must add \texttt{NA}s to the
Cox-parameters for the comparison because the Cox-model does not give
any parameters for the baseline time scale (\texttt{age}), but also
remove one of the parameters, because \texttt{coxph} parametrizes
factors (in this case \texttt{lex.Cst}) by all defined levels and not
only by the levels present in the dataset at hand (note the line of
\texttt{1.0000}s in the print above):
<<>>=
round(cbind(ci.exp(Pm),
       rbind(matrix(NA, 5, 3),
             ci.exp(cm)[-6, ])), 3)
@ %
Thus we see that the Poisson and Cox gives pretty much the same
results with regards to the regression parameters, but only the
Poisson gives a parametrization of the baseline hazard. You may argue
that Cox requires a smaller dataset, because there is no need to
subdivide data in small intervals \emph{before} insulin use. But
certainly the time \emph{after} insulin inception needs to be
subdivided in smaller intervals (as the \texttt{Lexis} data frame is)
if the effect of this time should be modeled.

The drawback of the Cox-modeling is that it is not possible to show
the absolute rates as we did in figure \ref{fig:ins-time} on page
\pageref{fig:ins-time}.

\section{Marginal effect of time since insulin}

When we plot the marginal effect of \texttt{tfI} from the two models
we get pretty much the same; here we plot the RR relative to
\texttt{tfI} = 2 years. Note that we are deriving the RR as the ratio
of two sets of predictions, from the data frames \texttt{nd} and
\texttt{nr}---variables assumed to have the same values in the two
data frames need not be included in the prediction frames, but
numerical variables omitted must be indicated in the \texttt{xvars=}
argument.  For further details, consult the help page for
\texttt{ci.lin}, specifically the use of a list as the
\texttt{ctr.mat} argument:
<<Ieff, fig = TRUE>>=
nd <- data.frame(tfI = seq(0, 15, , 151), lex.Cst = "Ins", sex = "M")
nr <- data.frame(tfI =     2            , lex.Cst = "Ins", sex = "M")
# We need to use xvars="age" in ci.exp because age is in the model
# but not in the prediction frames nd and nr
ppr <- ci.exp(pm, list(nd, nr), xvars = "age")
cpr <- ci.exp(cm, list(nd, nr))
par(mar = c(3, 3, 1, 1), mgp = c(3, 1, 0)/1.6, las = 1, bty = "n")
matshade(nd$tfI, cbind(ppr, cpr), plot = T,
         lty = c(1, 2), lwd = 3, log = "y",
         xlab = "Time since insulin (years)",
         ylab = "Mortality rate ratio")
abline(h = 1, lty = 3)
@ %
\insfig{Ieff}{0.8}{The naked duration effects on mortality relative to 2 years of
  duration. Black from Poisson model, red from Cox model. The two sets
  of estimates are identical, and so are the standard errors, so the
  two shaded areas overlap almost perfectly.}

In figure \ref{fig:Ieff}, p. \pageref{fig:Ieff}, we see that the
duration effect is exactly the same from the two modeling approaches
(Cox and Poisson).

We will also want the RR relative to the non-insulin users --- recall these
are coded 0 on the \texttt{tfI} variable:
<<IeffR, fig = TRUE>>=
nd <- data.frame(tfI = seq(0, 15, , 151), lex.Cst = "Ins", sex = "M")
nr <- data.frame(tfI =     0            , lex.Cst = "DM" , sex = "M")
ppr <- ci.exp(pm, list(nd, nr), xvars = "age")
cpr <- ci.exp(cm, list(nd, nr))
par(mar = c(3, 3, 1, 1), mgp = c(3, 1, 0)/1.6, las = 1, bty = "n")
matshade(nd$tfI, cbind(ppr, cpr), lwd = 3,
         xlab = "Time since insulin (years)",
         ylab = "Rate ratio relative to non-Insulin",
         lty = c(1, 2), log = "y", plot = TRUE)
abline(h = 1, lty = 3)
@ %
\insfig{IeffR}{0.8}{Insulin duration effect (state \textrm{\tt Ins})
  relative to no insulin (state \textrm{\tt DM}), black from Poisson
  model, red from Cox model. The \emph{shape} is the same as the
  previous figure, but the RR is now relative to non-insulin, instead
  of relative to insulin users at 2 years duration. The estimates from
  the Cox model and the Poisson model are virtually identical, and so
  are the standard errors, so the two shaded areas overlap almost
  perfectly.}

In figure \ref{fig:IeffR}, p. \pageref{fig:IeffR}, we see the effect
of increasing duration of insulin use \emph{for a fixed age} which is
a bit artificial, so we would like to see the \emph{joint} effects of
age and insulin duration. What we cannot see is how the duration
affects mortality as a function of \emph{current} age (at the age
attained at the same time as the attained \texttt{tfI}).

Another way of interpreting this curve is as the rate ratio for a
person on insulin relative to a person of the same age not on insulin,
so we see that the RR (or hazard ratio, HR as some call it) is over 5
at the start of insulin (the \texttt{lex.Cst} estimate), and decreases
to about 1.5 in the long term.

Figure \ref{fig:ins-time}, \ref{fig:Ieff} and \ref{fig:IeffR} all
indicate a declining RR by insulin duration, but only from figure
\ref{fig:ins-time} it is visible that mortality actually is
\emph{in}creasing by age after some 2 years after insulin start. This
point would not be available if we had only fitted a Cox model where
we do not have access to the baseline hazard as a function of age; the
Cox model only gives the rate ratio of the blue to the black curve in
\ref{fig:ins-time}.

\section{Age$\times$duration interaction}

The model we fitted assumes that the HR (or RR) is the same regardless
of the age at start of insulin --- the hazards are
multiplicative. Sometimes this is termed the \emph{proportional
  hazards} assumption: For \emph{any} fixed age the HR is the same as
a function of time since insulin, and vice versa.

A more correct term would be ``main effects model'' --- there is no
interaction between age (the baseline time scale) and other
covariates. So there is really no need for the term ``proportional
hazards''; a well defined and precise statistical term for it has
existed for eons.

\subsection{Age at insulin start}

In order to check the consistency of the multiplicative assumption
across the spectrum of age at insulin inception, we can fit an
interaction model. One approach to this --- which is not a completely
general interaction --- would be using a non-linear effect of age at
insulin inception (for convenience we use the same knots as for age).
Note that the prediction data frames would be the same as we used
above, because we do not compute age at insulin for use as a separate
variable, but rather enter it in the model as the difference between
current age (\texttt{age}) and insulin duration (\texttt{tfI}).

At first glance we might think of doing:
<<>>=
ii <- glmLexis(tsNA20(dmCs),
                formula = ~ Ns(age      , knots = a.kn)
                          + Ns(      tfI, knots = i.kn)
                          + Ns(age - tfI, knots = a.kn)
                          + lex.Cst + sex)
@ %
But this fits a model where the rate-ratio between persons with and
without insulin at start of insulin (where \texttt{tfI} = 0) will be the
same at any age, which is a bit too restrictive for the interaction we
want.

We want the \texttt{age-tfI} term to be specific for the insulin
exposed so will use:
<<>>=
im <- glmLexis(tsNA20(dmCs),
                formula = ~ Ns(age      , knots = a.kn)
                          + Ns(      tfI, knots = i.kn)
                + lex.Cst : Ns(age - tfI, knots = a.kn)
                          + lex.Cst + sex)
ci.exp(im)
@ %
The model (\texttt{im}) allows age-effects that differ
non-linearly between persons with and without insulin, because the
interaction term \texttt{lex.Cst:Ns(age-tfI...} for persons not on
insulin is merely an age term (since \texttt{tfI} is coded 0 for all
follow-up not on insulin).

We can compare the two models fitted:
<<>>=
anova(ii, im, test = 'Chisq')
@ %
so we see that the second model (\texttt{im}, the interaction model)
provides a substantial further improvement, by allowing non-linear HR
along the age-scale.

We can illustrate the estimated rates from the \texttt{im} model
in figure \ref{fig:dur-int}, p. \pageref{fig:dur-int}:
<<dur-int, fig = TRUE>>=
pii <- ci.pred(im, ndI)
pia <- ci.pred(im, ndA)
par(mar = c(3, 3, 1, 1), mgp = c(3, 1, 0) / 1.6, las = 1, bty = "n")
matshade(ndI$age, pii * 1000, plot = T, log = "y",
         xlab = "Age", ylab = "Mortality per 1000 PY",
         lty = 1, lwd = 2, col = c("blue", "forestgreen", "red"), alpha = 0.1)
matshade(ndA$age, pia * 1000)
@ %
\insfig{dur-int}{0.8}{Age at insulin as interaction between age and
  duration, for persons starting insulin at ages 40, 50,\ldots (in
  blue) and persons not on insulin (in black).}

We can also plot the RRs from the interaction model (figure
\ref{fig:dur-int-RR}, p. \pageref{fig:dur-int-RR}); for this we need
the reference frames, and the machinery from \texttt{ci.exp} allowing
a list of two data frames:
<<dur-int-RR, fig = TRUE>>=
ndR <- transform(ndI, tfI = 0, lex.Cst = "DM")
cbind(head(ndI), head(ndR))
Rii <- ci.exp(im , list(ndI, ndR))
par(mar = c(3, 3, 1, 1), mgp = c(3, 1, 0)/1.6, las = 1, bty = "n")
matshade(ndI$age, Rii, plot = T, log = "y",
         xlab = "Age (years)", ylab = "Rate ratio vs, non-Insulin",
         lty = 1, lwd = 2, col = c("blue", "forestgreen", "red"), alpha = 0.1)
abline(h = 1)
@ %
\insfig{dur-int-RR}{0.9}{RRs from the interaction
  model.}

\section{Separate models}

In the above we insisted on making a joint model for the
\texttt{DM}$\rightarrow$\texttt{Dead} and the
\texttt{Ins}$\rightarrow$\texttt{Dead} transitions, but it would
actually have been more sensible to model the two transitions
separately:
<<>>=
dmd <- glmLexis(dmCs,
                 from = "DM", to = "Dead",
                 formula = ~ Ns(age, knots = a.kn)
                           + sex)
ind <- glmLexis(dmCs,
                 from = "Ins", to = "Dead",
                 formula = ~ Ns(age      , knots = a.kn)
                           + Ns(      tfI, knots = i.kn)
                           + Ns(age - tfI, knots = a.kn)
                           + sex)
ini <- ci.pred(ind, ndI)
dmi <- ci.pred(dmd, ndI)
dma <- ci.pred(dmd, ndA)
@ %
The estimated mortality rates are shown in figure \ref{fig:Ins-noIns},
p. \pageref{fig:Ins-noIns}, using:
<<sep-mort, fig = TRUE>>=
par(mar = c(3, 3, 1, 1), mgp = c(3, 1, 0)/1.6, las = 1, bty = "n")
matshade(ndI$age, ini * 100, plot = TRUE, log = "y",
         xlab = "Age (years)", ylab = "Mortality rates per 100 PY",
         lwd = 2, col = "red")
matshade(ndA$age, dma*100,
         lwd = 2, col = "black")
@ % $
The estimated RRs can now be computed exploiting the fact that the
estimates from the two models are uncorrelated, and hence qualify for
\texttt{ci.ratio}:
<<sep-HR, fig = TRUE>>=
par(mar = c(3, 3, 1, 1), mgp = c(3, 1, 0)/1.6, las = 1, bty = "n")
matshade(ndI$age, ci.ratio(ini, dmi), plot = TRUE, log = "y",
         xlab = "Age (years)", ylab = "RR insulin vs. no insulin",
         lwd = 2, col = "red")
abline(h = 1)
@ %
\begin{figure}[tb]
\centering
\includegraphics[width = 0.49\textwidth]{01flup-sep-mort}
\includegraphics[width = 0.49\textwidth]{01flup-sep-HR}
\caption{\it Left panel: Mortality rates from separate models for the
  two mortality transitions; the \textrm{\tt
    DM}$\rightarrow$\textrm{\tt Dead} transition modeled by age alone;
  \textrm{\tt Ins}$\rightarrow$\textrm{\tt Dead} transition modeled
  with spline effects of current age, time since insulin and and age
  at insulin. \newline Right panel: Mortality HR of insulin vs. no insulin.}
\label{fig:Ins-noIns}
\end{figure}
The only difference between the interaction model and the two separate
models is that the latter allows different \texttt{sex}-effects
between mortality rates from \texttt{DM} and \texttt{Ins}. There
actually \emph{is} a difference between the estimates but hardly visible.

\chapter{More states}

\section{Subdividing states}

It may be of interest to subdivide the state \texttt{Dead} according
to whether the event has occurred or not. This will enable us to
estimate the number of patients that ever go on insulin.

This is done with \texttt{cutLexis} by using the argument
\texttt{split.states = TRUE}.
<<>>=
dmCs <- cutLexis(data = dmS2,
                  cut = dmS2$doins,
            timescale = "per",
            new.state = "Ins",
            new.scale = "tfI",
         split.states = TRUE)
summary(dmCs)
@ %
We can illustrate the numbers and the transitions (figure
\ref{fig:box4}, p. \pageref{fig:box4})
<<box4, fig = TRUE, height = 5, width = 7>>=
boxes(dmCs, boxpos = list(x = c(15, 15, 85, 85),
                          y = c(85, 15, 85, 15)),
      scale.R = 1000, show.BE = TRUE)
legendbox(70, 50)
@ % $
\insfig{box4}{0.7}{Transitions between 4 states: the numbers \emph{in}
  the boxes are person-years (middle), and below the number of persons
  who start, respectively end their follow-up in each of the states.}

Note that it is only the mortality rates that we have been modeling,
namely the transitions
\texttt{DM}$\rightarrow$\texttt{Dead} and
\texttt{Ins}$\rightarrow$\texttt{Dead(Ins)}.  If we were to model the
cumulative risk of using insulin or currently being on insulin we would
also need a model for the DM$\rightarrow$Ins transition. Subsequent to
that we would then compute the probability of being in each state
conditional on suitable starting conditions (including time of
start). With models where transition rates depend on several time
scales this is not a trivial task. This is treated in more detail in
the vignette on \texttt{simLexis}.

\section{Multiple intermediate events}

We may be interested in initiation of either insulin or OAD (oral
anti-diabetic drugs), thus giving rise to more states and more
time scales. This can be accomplished by the \texttt{mcutLexis}
function, that generalizes \texttt{cutLexis}:
<<>>=
dmM <- mcutLexis(dmL,
           timescale = "per",
                  wh = c("doins", "dooad"),
          new.states = c("Ins", "OAD"),
          new.scales = c("tfI", "tfO"),
        ties.resolve = TRUE)
@ %
The \texttt{Lexis} machinery does not know what a reasonable order of
states is, so that will have to be fixed by hand using \texttt{Relevel}:
<<>>=
levels(dmM)
dmM <- Relevel(dmM,  c("DM", "OAD", "Ins", "OAD-Ins", "Ins-OAD", "Dead"))
summary(dmM, t = T)
@ %
We see that we now have two time scales defined as time since entry
into states.
<<>>=
wh <- c(subset(dmM, lex.Cst == "Ins-OAD")$lex.id[1:2],
        subset(dmM, lex.Cst == "OAD-Ins")$lex.id[1:2])
print(subset(dmM, lex.id %in% wh), nd = 2)
@ %
\begin{expl}
  We use subset to locate all records with \texttt{lex.Cst} equal to
  \texttt{Ins-OAD}, resp. \texttt{OAD-Ins}, and extract the ids
  (\texttt{lex.id}) from these. We the select the two first of each,
  and print all records for these persons.
\end{expl}
We can also illustrate the transitions to the different states, as in
figure \ref{fig:mbox} --- the specification of the \texttt{boxpos}
argument is facilitated by the logical ordering of the states
<<mbox, fig = TRUE, height = 7, width = 11>>=
boxes(dmM, boxpos = list(x = c(15, 40, 40, 85, 85, 80),
                         y = c(50, 90, 10, 90, 10, 50)),
           scale.R = 1000, show.BE = TRUE)
legendbox(6, 95)
@ %
\insfig{mbox}{1.0}{Boxes for the \textrm{\tt dmM} object. The numbers
  \emph{in} the boxes are person-years (middle), and below the number
  of persons who start, respectively end their follow-up in each of
  the states.}
We may not be interested in whether persons were prescribed insulin
before OAD or vice versa, in which case we would combine the levels
with both insulin and OAD to one, regardless of order (figure
\ref{fig:mboxr}):
<<mboxr, fig = TRUE, height = 7, width = 11>>=
summary(dmMr <- Relevel(dmM, list(1, 2, 3, 'OAD+Ins' = 4:5, 6)))
boxes(dmMr, boxpos = list(x = c(15, 15, 85, 85, 50),
                          y = c(85, 15, 85, 15, 50)),
            scale.R = 1000, show.BE = TRUE)
@ %
\insfig{mboxr}{1.0}{Boxes for the \textrm{\tt dmMr} object with
  collapsed states. The numbers \emph{in} the boxes are person-years
  (middle), and below the number of persons who start, respectively
  end their follow-up in each of the states.}

Diagrams as those in figures
\ref{fig:mbox} and
\ref{fig:mboxr} gives an overview of the possible transitions,
which states it might be relevant to collapse, and which transitions
to model and how.

\subsection{Modeling rates}

The modeling of the transition rates is straightforward;
split the data along some timescale and then use \texttt{glmLexis} or
\texttt{gamLexis}, where it is possible to select the transitions
modeled. This is also possible with the \texttt{coxphLexis}
function, but it requires that a single time scale be selected as the
baseline time scale, and the effect of this will not be accessible.

Here is a brief sketch of models that might be considered:
<<>>=
dmMs <- splitMulti(dmMr, age = 0:100)
summary(dmMs)
levels(dmMs)
rateIns <- gamLexis(dmMr, ~ s(age) + lex.Cst, from = 1:2   , to = 3:4)
rateOAD <- gamLexis(dmMr, ~ s(age) + lex.Cst, from = c(1,3), to = c(2, 4))
rateDth <- gamLexis(dmMr, ~ s(age) + lex.Cst)
ci.exp(rateIns, subset = "lex")
ci.exp(rateOAD, subset = "lex")
ci.exp(rateDth, subset = "lex")
@ %

\chapter{\texttt{Lexis} functions}

\hypertarget{lexfun}{The \texttt{Lexis} machinery} has evolved over
  time since it was first introduced in a workable version in
  \texttt{Epi\_1.0.5} in August 2008.

  Over the years there have been additions of tools for handling
  multistate data. Here is a list of the current functions relating to
  \texttt{Lexis} objects with a very brief description; it does not
  replace the documentation, so read that before use. Unless otherwise
  stated, functions named \texttt{something.Lexis} (with a
  ``\texttt{.}'')  are S3 methods for \texttt{Lexis} objects, so you
  can skip the ``\texttt{.Lexis}'' in daily use.

\setlist{noitemsep}
\begin{description}

% \item[Define]\ \\
%   \begin{tabular}{rp{0.8\linewidth}}
%  \texttt{cal.yr} & transforms \texttt{Date} variables (measured in
%                    days) to \texttt{cal.yr} format (measured in
%                    years)\\
%   \texttt{Lexis} & defines a \texttt{Lexis} object\\
% \texttt{unLexis} & remove \texttt{Lexis} specific attributes\\
%   \end{tabular}

\item[Define]\ \\
\begin{description}
\item[\texttt{Lexis}] defines a \texttt{Lexis} object
\item[\texttt{cal.yr}] transforms \texttt{Date} variables (measured in
  days) to \texttt{cal.yr} format (measured in years)
\end{description}

\item[Cut and split]\ \\
\begin{description}
\item[\texttt{cutLexis}] cut follow-up at intermediate event
\item[\texttt{mcutLexis}] cut follow-up at multiple intermediate
  events, keeping track of history
\item[\texttt{rcutLexis}] cut follow-up at intermediate, possibly
  recurring, events, only recording the current state
\item[\texttt{xcutLexis}] cut follow-up at another set of events,
  keeping the orignal states (cross classification)
\item[\texttt{countLexis}] cut follow-up at intermediate event time
  and coun  the no. events so far
\item[\texttt{splitLexis}] split follow up along a time scale
\item[\texttt{splitMulti}] split follow up along several time scales --- from
  the \texttt{popEpi} package, faster and has simpler syntax than
  \texttt{splitLexis}
\item[\texttt{addCov.Lexis}] add clinical measurements at a given date to a
  \texttt{Lexis} object
\item[\texttt{addDrug.Lexis}] add drug exposures to a \texttt{Lexis} object
\item[\texttt{coarse.Lexis}] combine successive records in a \texttt{Lexis} object
\end{description}

% \item[Boxes and plots]\ \\
% \begin{tabular}{rl}
% \texttt{boxes.Lexis} & draw a diagram of states and transitions\\
% \texttt{legendbox} & draw a box explaining the numbers output by \texttt{boxes.Lexis}\\
% \texttt{plot.Lexis} & draw a standard Lexis diagram\\
% \texttt{points.Lexis} & add points to a Lexis diagram\\
% \texttt{lines.Lexis} & add lines to a Lexis diagram\\
% \texttt{PY.ann.Lexis} & annotate life lines in a Lexis diagram
% \end{tabular}

\item[Boxes and plots]\ \\
\begin{description}
\item[\texttt{boxes.Lexis}] draw a diagram of states and transitions
\item[\texttt{legendbox}] draw a box explaining the numbers output by \texttt{boxes.Lexis}
\item[\texttt{plot.Lexis}] draw a standard Lexis diagram
\item[\texttt{points.Lexis}] add points to a Lexis diagram
\item[\texttt{lines.Lexis}] add lines to a Lexis diagram
\item[\texttt{PY.ann.Lexis}] annotate life lines in a Lexis diagram
\end{description}

\newpage
\item[Summarize and query]\ \\
\begin{description}
\item[\texttt{summary.Lexis}] overview of transitions, risk time etc.
\item[\texttt{levels.Lexis}] what are the states in the \texttt{Lexis} object
\item[\texttt{paths.Lexis}] what are the paths through states for persons in a \texttt{Lexis} object
\item[\texttt{nid.Lexis}] number of persons in the \texttt{Lexis}
  object---how many unique values of \texttt{lex.id} are present
\item[\texttt{entry}] entry time
\item[\texttt{exit}] exit time
\item[\texttt{status}] status at entry or exit
\item[\texttt{lexVars}] names of the \texttt{Lexis} specific variables
\item[\texttt{timeBand}] factor of time bands
\item[\texttt{timeScales}] what time scales are in the \texttt{Lexis} object
\item[\texttt{timeSince}] what time scales are defined as time since a given state
\item[\texttt{breaks}] what breaks are currently defined
\item[\texttt{absorbing}] what are the absorbing states
\item[\texttt{transient}] what are the transient states
\item[\texttt{preceding}, \texttt{before}] which states precede this
\item[\texttt{succeeding}, \texttt{after}] which states can follow this
\item[\texttt{tmat.Lexis}] transition matrix for the \texttt{Lexis} object
\end{description}

\item[Manipulate]\ \\
\begin{description}
\item[\texttt{subset.Lexis}, \texttt{[}] subset of a \texttt{Lexis} object
\item[\texttt{merge.Lexis}] merges a \texttt{Lexis} objects with a
  \texttt{data.frame}
\item[\texttt{cbind.Lexis}] bind a \texttt{data.frame} to a \texttt{Lexis} object
\item[\texttt{rbind.Lexis}] put two \texttt{Lexis} objects head-to-foot
\item[\texttt{order.Lexis}, \texttt{orderLexis}] ordering of records
  in a \texttt{Lexis} object
\item[\texttt{sortLexis}] sort records in a \texttt{Lexis} object by
  (\texttt{lex.id}, \texttt{time})
\item[\texttt{transform.Lexis}] transform and add variables
\item[\texttt{tsNA20}] turn \texttt{NA}s to 0s for time scales
\item[\texttt{factorize.Lexis}] turn \texttt{lex.Cst} and
  \texttt{lex.Xst} into factors with levels equal to the actually
  occurring values in both
\item[\texttt{Relevel.Lexis}] reorder and/or combine states
\item[\texttt{rm.tr}] remove transitions from a \texttt{Lexis} object
\item[\texttt{bootLexis}] bootstrap sample of \emph{persons}
  (\texttt{lex.id}) from a \texttt{Lexis} object
\item[\texttt{unLexis}] remove \texttt{Lexis} attributes from a
  \texttt{Lexis} object
\end{description}

\item[Simulate]\ \\
\begin{description}
\item[\texttt{simLexis}] simulate a \texttt{Lexis} object from
  specified transition rate models
\item[\texttt{nState}, \texttt{pState}] count state occupancy from a
  simulated \texttt{Lexis} object
\item[\texttt{plot.pState}, \texttt{lines.pState}] plot state occupancy from a
  \texttt{pState} object
\end{description}

\item[Stack]\ \\
\begin{description}
\item[\texttt{stack.Lexis}] make a stacked object for simultaneous
  analysis of transitions --- returns a \texttt{stacked.Lexis} object
\item[\texttt{subset.stacked.Lexis}] subsets of a \texttt{stacked.Lexis} object
\item[\texttt{transform.stacked.Lexis}] transform  a \texttt{stacked.Lexis} object
\end{description}

\newpage
\item[Interface to other packages]\ \\
\begin{description}
\item[\texttt{msdata.Lexis}] interface to \texttt{mstate} package
\item[\texttt{etm.Lexis}] interface to \texttt{etm} package
\item[\texttt{crr.Lexis}] interface to \texttt{cmprsk} package
\end{description}

\item[Statistical models]\ \\
\begin{description}
\item[\texttt{AaJ.Lexis}] compute the Aalen-Johansen estimator for a
  \texttt{Lexis} object --- wrapper for \texttt{survfit} from
  \texttt{survival}
\item[\texttt{ci.Crisk}] compute cumulative risks with CIs from model
  objects for competing rates
\item[\texttt{glmLexis}] fit a \texttt{glm} model using the
  \texttt{poisreg} family to (hopefully) time split data
\item[\texttt{gamLexis}] fit a \texttt{gam} model (from the
  \texttt{mgcv} package) using the \texttt{poisreg} family to
  (hopefully) time split data
\item[\texttt{coxphLexis}] fit a Cox model to follow-up in a
  \texttt{Lexis} object
\item In versions of Epi up to 2.56 the three modeling functions were called
  \texttt{glm.Lexis},
  \texttt{gam.Lexis} and
  \texttt{coxph.Lexis} --- but they are not S3 methods so the naming was
  illogical. The versions with the old names still exist in
  \texttt{Epi} for backward compatibility.
\end{description}
\end{description}
<<echo = FALSE>>=
ende <- Sys.time()
cat("  Start time:", format(anfang, "%F, %T"),
  "\n    End time:", format(  ende, "%F, %T"),
  "\nElapsed time:", round(difftime(ende, anfang, units = "mins"), 2), "minutes\n")
@ %

\renewcommand{\bibname}{References}

\bibliographystyle{unsrt}%{plain}
\begin{thebibliography}{1}

\bibitem{Holford.1976}
T~R Holford.
\newblock Life tables with concomitant information.
\newblock {\em Biometrics}, 32:587--597, 1976.

\bibitem{Whitehead.1980}
J~Whitehead.
\newblock Fitting {C}ox's regression model to survival data using {GLIM}.
\newblock {\em Applied Statistics}, 29(3):268--275, 1980.

\bibitem{Plummer.2011}
M~Plummer and B~Carstensen.
\newblock Lexis: An {R} class for epidemiological studies with long-term
  follow-up.
\newblock {\em Journal of Statistical Software}, 38(5):1--12, 1 2011.

\bibitem{Carstensen.2011a}
B~Carstensen and M~Plummer.
\newblock Using {L}exis objects for multi-state models in {R}.
\newblock {\em Journal of Statistical Software}, 38(6):1--18, 1 2011.

\bibitem{Carstensen.2007a}
B~Carstensen.
\newblock Age-{P}eriod-{C}ohort models for the {L}exis diagram.
\newblock {\em Statistics in Medicine}, 26(15):3018--3045, July 2007.

\bibitem{Iacobelli.2013}
S~Iacobelli and B~Carstensen.
\newblock {Multiple time scales in multi-state models}.
\newblock {\em Stat Med}, 32(30):5315--5327, Dec 2013.

\end{thebibliography}

\addcontentsline{toc}{chapter}{\bibname}

\end{document}

%\VignetteIndexEntry{Simulation of multistate models with multiple timescales: simLexis}
\SweaveOpts{results=verbatim,keep.source=TRUE,include=FALSE,eps=FALSE}
\documentclass[a4paper,dvipsnames,twoside,12pt]{report}

\newcommand{\Title}{Simulation of\\ multistate models with\\ multiple
  timescales:\\ \texttt{simLexis} in the \texttt{Epi} package}
\newcommand{\Tit}{Multistate models with multiple timescales}
\newcommand{\Version}{Version 2.6}
\newcommand{\Dates}{\today}
\newcommand{\Where}{SDCC}
\newcommand{\Homepage}{\url{http://BendixCarstensen.com/Epi/simLexis.pdf}}
\newcommand{\Faculty}{\begin{tabular}{rl}
Bendix Carstensen
  & Steno Diabetes Center Copenhagen, Gentofte, Denmark\\
  & {\small \& Department of Biostatistics,
               University of Copenhagen} \\
  & \texttt{b@bxc.dk}\\
  & \url{http://BendixCarstensen.com} \\[1em]
                      \end{tabular}}

\input{topreport}
\renewcommand{\rwpre}{./simLexis}
<<echo=FALSE>>=
options( width=90,
         SweaveHooks=list( fig=function()
         par(mar=c(3,3,1,1),mgp=c(3,1,0)/1.6,las=1,bty="n") ) )
@ %

<<echo = FALSE>>=
anfang <- Sys.time()
cat("Start time:", format(anfang, "%F, %T"), "\n")
@ %

\chapter{Using \texttt{simLexis}}

\section{Introduction}

This vignette explains the machinery behind simulation of life
histories through multistate models implemented in
\texttt{simLexis}. In \texttt{simLexis} transition rates are allowed
to depend on multiple time scales, including timescales defined as
time since entry to a particular state (duration). This therefore also
covers the case where time \emph{at} entry into a state is an
explanatory variable for the rates, since time at entry is merely
(current) time minus duration. Thus, the set-up here goes beyond
Markov- and semi-Markov-models, and brings simulation based estimation
of state-occupancy probabilities into the realm of realistic
multistate models.

The basic idea is to simulate a new \texttt{Lexis} object
\cite{Plummer.2011,Carstensen.2011a} as defined in the \texttt{Epi}
package for \R, based on 1) a multistate model defined by its states
and the transition rates between them and 2) an initial population of
individuals.

Thus the output will be a \texttt{Lexis} object describing the
transitions of a predefined set of persons through a multistate
model. Therefore, if persons are defined to be identical at start,
then calculation of the probability of being in a particular state at
a given time boils down to a simple enumeration of the fraction of the
persons in the particular state at the given time. Bar of course the
(binomial) simulation error, but this can be brought down by
simulation a sufficiently large number of persons.

An observed \texttt{Lexis} object with follow-up of persons through a
number of states will normally be the basis for estimation of
transition rates between states, and thus will contain all information
about covariates determining the occurrence rates, in particular the
\emph{timescales} \cite{Iacobelli.2013}. Hence, the natural input to
simulation from an estimated multistate model will typically be an
object of the same structure as the originally observed. Since
transitions and times are what is simulated, any values of
\texttt{lex.Xst} and \texttt{lex.dur} in the input object will of
course be ignored.

This first chapter of this vignette shows by an example how to use the
function \texttt{simLexis} and display the results. The second
chapter discusses in more detail how the simulation machinery is
implemented and is not needed for the practical use of \texttt{simLexis}.

\section{\texttt{simLexis} in practice}

This section is largely a commented walk-trough of the example from
the help-page of \texttt{simLexis}, with a larger number of simulated
persons in order to minimize the pure simulation variation.

When we want to simulate transition times through a multistate model
where transition rates may depend on time since entry to the current
or a previous state, it is essential that we have a machinery to keep
track of the transition time on \emph{all} time scales, as well as a
mechanism that can initiate a new time scale to 0 when a transition
occurs to a state where we shall use time since entry as determinant
of exit rates from that state. This is provided by \texttt{simLexis}.

\subsection{Input for the simulation}

Input for simulation of a single trajectory through a multistate model
requires a representation of the \emph{current status} of a person;
the starting conditions. The object that we supply to the simulation
function must contain information about all covariates and all
timescales upon which transitions depend, and in particular which
one(s) of the timescales that are defined as time since entry into a
particular state. Hence, starting conditions should be represented as
a \texttt{Lexis} object (where \texttt{lex.dur} and \texttt{lex.Xst}
are ignored, since there is no follow-up yet), where the time scale
information is in the attributes \texttt{time.scales} and
\texttt{time.since} respectively.

Note that \texttt{time.scales} attribute is a vector of names of
variables in the \texttt{Lexis} object, so all of these variables
should be present even if they are not used in the models for the
transitions, and they should be set to 0; if they are not in the
initial dataset, \texttt{simLexis} will crash, if they are
\texttt{NA}, the \texttt{simLexis} will produce an object with 0 rows.

Thus there are two main arguments to a function to simulate from a
multistate model:
\begin{enumerate}
\item A \texttt{Lexis} object representing the initial states and
  covariates of the population to be simulated. This has to have the
  same structure as the original \texttt{Lexis} object representing
  the multistate model from which transition rates in the model were
  estimated. As noted above, the values for \texttt{lex.Xst} and
  \texttt{lex.dur} are not required (since these are the quantities
  that will be simulated).
\item A transition object, representing the transition intensities
  between states, which should be a list of lists of intensity
  representations. As an intensity representation we mean a function
  that for a given \texttt{Lexis} object can be used to produce
  estimates of the transition intensities at a set of supplied time
  points.

  The names of the elements of the transition object (which are lists)
  will be names of the \emph{transient} states, that is the states
  \emph{from} which a transition can occur. The names of the elements
  of each of these lists are the names of states \emph{to} which
  transitions can occur (which may be either transient or absorbing
  states).

  Hence, if the transition object is called \texttt{Tr} then
  \verb+TR$A$B+ (or \verb+Tr[["A"]][["B"]]+) will represent the
  transition intensity from state \texttt{A} to the state \texttt{B}.

  The entries in the transition object can be either \texttt{glm}
  objects (either with \texttt{poisson} or \texttt{poisreg} family),
  representing Poisson models for the transitions, \texttt{coxph}
  objects representing an intensity model along one time scale, or
  simply a function that takes a \texttt{Lexis} object as input and
  returns an estimated intensity for each row.

\end{enumerate}

In addition to these two input items, there will be a couple of tuning
parameters.

The output of the function will simply be a \texttt{Lexis} object with
simulated transitions between states. This will be the basis for
deriving sensible statistics from the \texttt{Lexis} object --- see
next section.

\section{Setting up a \texttt{Lexis} object}

As an example we will use the \texttt{DMlate} dataset from the
\texttt{Epi} package; it is a dataset simulated to resemble a random
sample of 10,000 patients from the Danish National Diabetes Register.

We start by loading the \texttt{Epi} package:
<<start>>=
options( width=90 )
library( Epi )
print( sessionInfo(), l=F )
@ %
First we load the diabetes data and set up a simple illness-death
model:
<<Lexis>>=
data(DMlate)
dml <- Lexis( entry = list(Per=dodm, Age=dodm-dobth, DMdur=0 ),
               exit = list(Per=dox),
        exit.status = factor(!is.na(dodth),labels=c("DM","Dead")),
               data = DMlate )
@ %
This is just data for a simple survival model with states \texttt{DM} and
\texttt{Dead}. Now we cut the follow-up at insulin start, which for the
majority of patients (T2D) is a clinical indicator of deterioration of
disease regulation. We therefore also introduce a new timescale, and
split the non-precursor states, so that we can address the question of
ever having been on insulin:
<<cut>>=
dmi <- cutLexis( dml, cut = dml$doins,
                      pre = "DM",
                new.state = "Ins",
                new.scale = "t.Ins",
             split.states = TRUE )
summary( dmi, timeScales=T )
@ % $
Note that we show the time scales in the \texttt{Lexis} object, and
that it is indicated that the time scale \texttt{t.Ins} is defined as
time since entry into stat state \texttt{Ins.}

We can show how many person-years we have and show the number of
transitions and transition rates (per 1000), using the
\texttt{boxes.Lexis} function to display the states and the number of
transitions between them:
<<boxes,fig=TRUE,height=5,width=7>>=
boxes( dmi, boxpos = list(x=c(20,20,80,80),
                        y=c(80,20,80,20)),
            scale.R = 1000, show.BE = TRUE )
@ %
\insfig{boxes}{0.8}{Data overview for the \textrm{\tt dmi}
  dataset. Numbers in the boxes are person-years and the number of
  persons who begin, resp. end their follow-up in each state, and
  numbers on the arrows are no. of transitions and rates (transition
  intensities) per 1000 PY.}

\section{Analysis of rates}

In the \texttt{Lexis} object (which is just a data frame) each person
is represented by one record for each transient state occupied,
thus in this case either 1 or 2 records; those who have a recorded
time both without and with insulin have two records.

In order to be able to fit Poisson models with occurrence rates varying
by the different time-scales, we split the follow-up in 3-month intervals
for modeling:
<<split>>=
Si <- splitLexis( dmi, seq(0,20,1/4), "DMdur" )
summary( Si )
print( subset( Si, lex.id==97 )[,1:10], digits=6 )
@ %
Note that when we split the follow-up, each person's follow up now
consists of many records, each with the \emph{current} values of the
timescales at the start of the interval represented by the record. In
the modeling we shall assume that the rates are constant within each
6-month interval, but the \emph{size} of these rates we model as
smooth functions of the time scales (that is the values at the
beginning of each interval).

The approach often used in epidemiology where one parameter is
attached to each interval of time (or age) is not feasible when more
than one time scale is used, because intervals are not classified the
same way on all timescales.

We shall use natural splines (restricted cubic splines) for the
analysis of rates, and hence we must allocate knots for the
splines. This is done for each of the time-scales, and separately for
the transition out of states \texttt{DM} and \texttt{Ins}. For age, we
place the knots so that the number of events is the same between each
pair of knots, but only half of this beyond each of the boundary
knots, whereas for the timescales \texttt{DMdur} and \texttt{tIns}
where we have observation from a well-defined 0, we put knots at 0 and
place the remaining knots so that the number of events is the same
between each pair of knots as well as outside the boundary knots.
<<knots>>=
nk <- 5
( ai.kn <- with( subset(Si,lex.Xst=="Ins" & lex.Cst!=lex.Xst ),
                 quantile( Age+lex.dur  , probs=(1:nk-0.5)/nk ) ) )
( ad.kn <- with( subset(Si,lex.Xst=="Dead"),
                 quantile( Age+lex.dur  , probs=(1:nk-0.5)/nk ) ) )
( di.kn <- with( subset(Si,lex.Xst=="Ins" & lex.Cst!=lex.Xst ),
                 c(0,quantile( DMdur+lex.dur, probs=(1:(nk-1))/nk ) )) )
( dd.kn <- with( subset(Si,lex.Xst=="Dead"),
                 c(0,quantile( DMdur+lex.dur, probs=(1:(nk-1))/nk ) )) )
( ti.kn <- with( subset(Si,lex.Xst=="Dead(Ins)"),
                 c(0,quantile( t.Ins+lex.dur, probs=(1:(nk-1))/nk ) )) )
@ %
Note that when we tease out the event records for transition to
\emph{transient} states (in this case \texttt{Ins}, that is
\verb|lex.Xst=="Ins"|), we should add \verb|lex.Cst!=lex.Xst|, to
include only transition records and avoiding including records of
sojourn time in the transient state.

We then fit Poisson models to transition rates, using the wrapper
\texttt{Ns} from the \texttt{Epi} package to simplify the
specification of the rates:
<<Poisson>>=
library( splines )
DM.Ins <- glm( (lex.Xst=="Ins") ~ Ns( Age  , knots=ai.kn ) +
                                  Ns( DMdur, knots=di.kn ) +
                                  I(Per-2000) + sex,
               family=poisson, offset=log(lex.dur),
               data = subset(Si,lex.Cst=="DM") )
ci.exp( DM.Ins )
class( DM.Ins )
@ %
We can also fit this model with a slightly simpler syntax using the
\texttt{glm.Lexis} function:
<<>>=
DM.Ins <- glm.Lexis( Si, from = "DM", to = "Ins",
                      formula = ~ Ns( Age  , knots=ai.kn ) +
                                  Ns( DMdur, knots=di.kn ) +
                                  I(Per-2000) + sex )
ci.exp( DM.Ins )
class( DM.Ins )
@ %
So we have a slightly simpler syntax, and we get an informative
message of which transition(s) we are modeling. However we do not have
\texttt{update} method for these objects.
<<>>=
DM.Dead <- glm.Lexis( Si, from = "DM", to = "Dead",
                       formula = ~ Ns( Age  , knots=ad.kn ) +
                                   Ns( DMdur, knots=dd.kn ) +
                                   I(Per-2000) + sex )
Ins.Dead <- glm.Lexis( Si, from = "Ins",
                        formula = ~ Ns( Age  , knots=ad.kn ) +
                                    Ns( DMdur, knots=dd.kn ) +
                                    Ns( t.Ins, knots=ti.kn ) +
                                    I(Per-2000) + sex )
@ %
Note the similarity of the code used to fit the three models, is is
mainly redefining the response variable (\texttt{to} state) and the subset
of the data used (\texttt{from} state). Also note that the last model need
no specification of \texttt{to}, the default is to model all
transitions from the \texttt{from} state, and his case there is only
one.

\section{The mortality rates}

This section discusses in some detail how to extract ad display the
mortality rates from the models fitted. But it is not necessary for
understanding how to use \texttt{simLexis} in practice.

\subsection{Proportionality of mortality rates}

Note that we have fitted separate models for the three transitions,
there is no assumption of proportionality between the mortality rates
from \texttt{DM} and \texttt{Ins}.

However, there is nothing that prevents us from testing this
assumption; we can just fit a model for the mortality rates in the
entire data frame \texttt{Si}, and compare the deviance from this with
the sum of the deviances from the separate models using the \texttt{glm.Lexis}
function:
<<prop-haz>>=
All.Dead <- glm.Lexis( Si, to = c("Dead(Ins)","Dead"),
                      formula = ~ Ns( Age  , knots=ad.kn ) +
                                  Ns( DMdur, knots=dd.kn ) +
                                  lex.Cst +
                                  I(Per-2000) + sex )
round( ci.exp( All.Dead ), 3 )
@ %
Incidentally we could have dispensed with the \texttt{to=} argument
too, because the default is to take \texttt{to} to be all absorbing
states in the model.

From the parameter values we would in a simple setting just claim that
start of insulin-treatment was associated with a slightly more than
doubling of mortality.

The model \texttt{All.dead} assumes that the age- and DM-duration
effects on mortality in the \texttt{DM} and \texttt{Ins} states are the same,
and moreover that there is no effect of insulin duration, but merely a
mortality that is larger by a multiplicative constant not depending on
insulin duration. The model \texttt{DM.Dead} has 8 parameters to
describe the dependency on age and DM duration, the model
\texttt{Ins.Dead} has 12 for the same plus the insulin duration (a
natural spline with $k$ knots gives $k-1$ parameters, and we chose
$k=5$ above).

We can compare the fit of the simple proportional hazards model with
the fit of the separate models for the two mortality rates, by adding
up the deviances and d.f. from these:
<<get-dev>>=
what <- c("null.deviance","df.null","deviance","df.residual")
( rD <- unlist(  DM.Dead[what] ) )
( rI <- unlist( Ins.Dead[what] ) )
( rA <- unlist( All.Dead[what] ) )
round( c( dd <- rA-(rI+rD), "pVal"=1-pchisq(dd[3],dd[4]+1) ), 3 )
@ %
Thus we see there is a substantial non-proportionality of mortality
rates from the two states; but a test provides no clue whatsoever to
the particular \emph{shape} of the non-proportionality.

To this end, we shall explore the predicted mortalities under the two
models quantitatively in more detail. Note that the reason that there
is a difference in the null deviances (and a difference of 1 in the
null d.f.) is that the null deviance of \texttt{All.Dead} refer to a
model with a single intercept, that is a model with constant and
\emph{identical} mortality rates from the states \texttt{DM} and \texttt{Ins},
whereas the null models for \texttt{DM.Dead} and \texttt{Ins.Dead}
have constant but \emph{different} mortality rates from the states
\texttt{DM} and \texttt{Ins}.  This is however irrelevant for the comparison of
the \emph{residual} deviances.

\subsection{How the mortality rates look}

If we want to see how the mortality rates are modelled in
\texttt{DM.Dead} and \texttt{Ins.Dead} in relation to
\texttt{All.Dead}, we make a prediction of rates for say men diagnosed
in different ages and going on insulin at different times after
this. So we consider men diagnosed in ages 40, 50, 60 and 70, and who
either never enter insulin treatment or do it 0, 2 or 5 years after
diagnosis of DM.

To this end we create a prediction data frame where we have
observation times from diagnosis and 12 years on (longer would not
make sense as this is the extent of the data).

But we start by setting up an array to hold the predicted mortality
rates, classified by diabetes duration, age at diabetes onset, time of
insulin onset, and of course type of model. What we want to do is to
plot the age-specific mortality rates for persons not on insulin, and
for persons starting insulin at different times after DM. The
mortality curves start at the age where the person gets diabetes and
continues 12 years; for persons on insulin they start at the age when
they initiate insulin.
<<pr-array>>=
pr.rates <- NArray( list( DMdur = seq(0,12,0.1),
                          DMage = 4:7*10,
                          r.Ins = c(NA,0,2,5),
                          model = c("DM/Ins","All"),
                           what = c("rate","lo","hi") ) )
str( pr.rates )
@ %
For convenience the \texttt{Epi} package contains a function that computes
predicted (log-)rates with c.i. --- it is merely a wrapper for
\texttt{predict.glm}.

So we set up the prediction data frame and modify it in loops over
ages at onset and insulin onset in order to collect the predicted
rates in different scenarios:
<<mknd>>=
nd <- data.frame( DMdur = as.numeric( dimnames(pr.rates)[[1]] ),
                lex.Cst = factor( 1, levels=1:4,
                                  labels=levels(Si$lex.Cst) ),
                    sex = factor( 1, levels=1:2, labels=c("M","F")) )
@ % $
Note that we did \emph{not} insert \texttt{lex.dur} as covariate in
the prediction frame. This would be required if we used the
\texttt{poisson} family with the \texttt{glm}, but the wrapper
\texttt{glm.Lexis} uses the \texttt{poisreg} family, so
\texttt{lex.dur} is ignored and predictions always comes in the
(inverse) units of \texttt{lex.dur}. So we get rates per 1 person-year
in the predictions.
<<make-pred>>=
for( ia in dimnames(pr.rates)[[2]] )
   {
dnew <- transform( nd, Age = as.numeric(ia)+DMdur,
                       Per = 1998+DMdur )
pr.rates[,ia,1,"DM/Ins",] <- ci.pred(  DM.Dead, newdata = dnew )
pr.rates[,ia,1,"All"   ,] <- ci.pred( All.Dead, newdata = dnew )
for( ii in dimnames(pr.rates)[[3]][-1] )
   {
dnew = transform( dnew, lex.Cst = factor( 2, levels=1:4,
                                          labels=levels(Si$lex.Cst) ),
                          t.Ins = ifelse( (DMdur-as.numeric(ii)) >= 0,
                                           DMdur-as.numeric(ii), NA ) )
pr.rates[,ia, ii ,"DM/Ins",] <- ci.pred( Ins.Dead, newdata = dnew )
pr.rates[,ia, ii ,"All"   ,] <- ci.pred( All.Dead, newdata = dnew )
    }
    }
@ % $
So for each age at DM onset we make a plot of the mortality as
function of current age both for those with no insulin treatment and
those that start insulin treatment 0, 2 and 5 years after diabetes
diagnosis, thus 4 curves (with c.i.). These curves are replicated with
a different color for the simplified model.
<<mort-int,fig=TRUE,width=8>>=
par( mar=c(3,3,1,1), mgp=c(3,1,0)/1.6, las=1 )
plot( NA, xlim=c(40,82), ylim=c(5,300), bty="n",
      log="y", xlab="Age", ylab="Mortality rate per 1000 PY" )
abline( v=seq(40,80,5), h=outer(1:9,10^(0:2),"*"), col=gray(0.8) )
for( aa in 4:7*10 ) for( ii in 1:4 )
   matshade( aa+as.numeric(dimnames(pr.rates)[[1]]),
             cbind( pr.rates[,paste(aa),ii,"DM/Ins",],
                    pr.rates[,paste(aa),ii,"All"   ,] )*1000,
             type="l", lty=1, lwd=2,
             col=c("red","limegreen") )
@ %
\insfig{mort-int}{0.9}{Estimated mortality rates for male diabetes
  patients with no insulin (lower sets of curves) and insulin (upper
  curves), with DM onset in age 40, 50, 60 and 70. The red curves are
  from the models with separate age effects for persons with and
  without insulin, and a separate effect of insulin duration. The
  green curves are from the model with common age-effects and only a
  time-dependent effect of insulin, assuming no effect of insulin
  duration (the classical time-dependent variable approach). Hence the
  upper green curve is common for any time of insulin inception.}

From figure \ref{fig:mort-int} we see that there is a substantial
insulin-duration effect which is not accommodated by the simple model
with only one time-dependent variable to describe the insulin
effect. Note that the simple model (green curves) for those on insulin
does not depend in insulin duration, and hence the mortality curves
for those on insulin are just parallel to the mortality curves for
those not on insulin, regardless of diabetes duration (or age) at the
time of insulin initiation. This is the proportional hazards
assumption.  Thus the effect of insulin initiation is under-estimated
for short duration of insulin and over-estimated for long duration of
insulin.

This is the major discrepancy between the two models, and
illustrates the importance of being able to accommodate different time
scales, but there is also a declining overall insulin effect by age which
is not accommodated by the proportional hazards approach.

Finally, this plot illustrates an important feature in reporting
models with multiple timescales; all timescales must be represented in
the predicted rates, only reporting the effect of one timescale,
conditional on a fixed value of other timescales is misleading since
all timescales by definition advance at the same pace. For example,
the age-effect for a fixed value of insulin duration really is a
misnomer since it does not correspond to any real person's follow-up,
but to the mortality of persons in different ages but with the same
duration of insulin use.

\section{Input to the \texttt{simLexis} function}

We want to estimate the cumulative probability of being in each of the
4 states, so that we can assess the fraction of diabetes pateints that
go on insulin

In order to simulate from the multistate model with the estimated
transition rates, and get the follow-up of a hypothetical cohort, we
must supply \emph{both} the transition rates and the structure of the
model \emph{as well as} the initial cohort status to
\texttt{simLexis}.

\subsection{The transition object}

We first put the models into an object representing the transitions;
note this is a list of lists, the latter having \texttt{glm} objects
as elements:
<<Tr>>=
Tr <- list( "DM" = list( "Ins"       = DM.Ins,
                         "Dead"      = DM.Dead  ),
           "Ins" = list( "Dead(Ins)" = Ins.Dead ) )
@ %
Now we have the description of the rates and of the structure of the
model. The \texttt{Tr} object defines the states and models for all
transitions between them; the object \verb|Tr$A$B| is the model
for the transition intensity from state \texttt{A} to state
\texttt{B}.

\subsection{The initial cohort}

We now define an initial \texttt{Lexis} object of persons with all
relevant covariates defined. Note that we use \texttt{NULL} as row
indicator in the \texttt{Lexis} object we used for modeling; this
conserves the \texttt{time.scale} and \texttt{time.since} attributes
which are needed for the simulation:
<<make-ini>>=
str( ini <- Si[NULL,1:9] )
@ %
We now have an empty \texttt{Lexis} object with attributes reflecting
the timescales in the multistate model we want to simulate from. But
we must enter some data to represent the initial state of the persons
whose follow-up we want to simulate through the model; so fill in data
for one man and one woman:
<<ini-fill>>=
ini[1:2,"lex.id"] <- 1:2
ini[1:2,"lex.Cst"] <- "DM"
ini[1:2,"Per"] <- 1995
ini[1:2,"Age"] <- 60
ini[1:2,"DMdur"] <- 5
ini[1:2,"sex"] <- c("M","F")
ini
@ %
So the persons starts in age 60 in 1995 with 5 years of diabetes
duration. Note that the \texttt{t.Ins} is \texttt{NA}, because this is
a timescale that first comes alive if a transtion to \texttt{Ins} is
simulated.

\section{Simulation of the follow-up}

Now we simulate life-courses of a 1000 of each of these persons using
the estimated model. The \texttt{t.range} argument gives the times
range where the integrated intensities (cumulative rates) are
evaluated and where linear interpolation is used when simulating
transition times. Note that this must be given in the same units as
\texttt{lex.dur} in the \texttt{Lexis} object used for fitting the
models for the transitions. It is not a parameter that can be easily
determined from the \texttt{TR} object, hence it must be supplied by
the user.
<<simL>>=
set.seed(52381764)
Nsim <- 500
system.time( simL <- simLexis( Tr,
                              ini,
                          t.range = 12,
                                N = Nsim ) )
@ %
The result is a \texttt{Lexis} object --- a data frame representing
the simulated follow-up of \Sexpr{2*Nsim} persons (\Sexpr{Nsim}
identical men and \Sexpr{Nsim} identical women) according to the rates
we estimated from the original dataset.
<<sum-simL>>=
summary( simL, by="sex" )
@ %

\subsection{Using other models for simulation}

\subsubsection{Proportional hazards Poisson model}

We fitted a proportional mortality model \texttt{All.Dead} (which fitted
worse than the other two), this is a model for \emph{both} the
transition from \texttt{DM} to \texttt{Death} \emph{and} from \texttt{Ins} to
\texttt{Dead(Ins)}, assuming that they are proportional. But it can easily
be used in the simulation set-up, because the state is embedded in the
model via the term \texttt{lex.Cst}, which is updated during the simulation.

Simulation using this instead just requires that we supply a different
transition object:
<<Tr.p-simP>>=
Tr.p <- list( "DM" = list( "Ins"       = DM.Ins,
                           "Dead"      = All.Dead  ),
             "Ins" = list( "Dead(Ins)" = All.Dead ) )
system.time( simP <- simLexis( Tr.p,
                                ini,
                            t.range = 12,
                                  N = Nsim ) )
summary( simP, by="sex" )
@ %

\subsubsection{Proportional hazards Cox model}

A third possibility would be to replace the two-time scale
proportional mortality model by a one-time-scale Cox-model, using
diabetes duration as time scale, and age at diagnosis of DM as (fixed)
covariate:
<<Cox-dur>>=
library( survival )
Cox.Dead <- coxph( Surv( DMdur, DMdur+lex.dur,
                         lex.Xst %in% c("Dead(Ins)","Dead")) ~
                   Ns( Age-DMdur, knots=ad.kn ) +
                   I(lex.Cst=="Ins") +
                   I(Per-2000) + sex,
               data = Si )
round( ci.exp( Cox.Dead ), 3 )
@ %
Note that in order for this model to be usable for simulation, it is
necessary that we use the components of the \texttt{Lexis} object to
specify the survival. Each record in the data frame \texttt{Si}
represents follow up from \texttt{DMdur} to \texttt{DMdur+lex.dur}, so
the model is a Cox model with diabetes duration as underlying timescale
and age at diagnosis, \texttt{Age-DMdur}, as covariate.

Also note that we used \texttt{I(lex.Cst=="Ins")} instead of just
\texttt{lex.Cst}, because \texttt{coxph} assigns design matrix columns
to all levels of \texttt{lex.Cst}, also those not present in data,
which would give \texttt{NA}s among the parameter estimates and
\texttt{NA}s as mortality outcomes.

We see that the effect of insulin and the other covariates are pretty
much the same as in the two-time-scale model. We can simulate from this
model too; there is no restrictions on what type of model can be used
for different transitions
<<TR.c>>=
Tr.c <- list( "DM" = list( "Ins"       = Tr$DM$Ins,
                           "Dead"      = Cox.Dead  ),
             "Ins" = list( "Dead(Ins)" = Cox.Dead ) )
system.time( simC <- simLexis( Tr.c,
                                ini,
                            t.range = 12,
                                  N = Nsim ) )
summary( simC, by="sex" )
@

\section{Reporting the simulation results}

We can now tabulate the number of persons in each state at a
predefined set of times on a given time scale. Note that in order for
this to be sensible, the \texttt{from} argument would normally be
equal to the starting time for the simulated individuals.
<<nState>>=
system.time(
nSt <- nState( subset(simL,sex=="M"),
               at=seq(0,11,0.2), from=1995, time.scale="Per" ) )
nSt[1:10,]
@ %
We see that as time goes by, the 500 men slowly move away from the
starting state (\texttt{DM}).

Based on this table (\texttt{nSt} is a table) we can now compute the
fractions in each state, or, rather more relevant, the cumulative fraction
across the states in some specified order, so that a plot of the
stacked probabilities can be made, using either the default rather
colorful layout, or a more minimalist version (both in figure \ref{fig:pstate0}):
<<pstate0,fig=TRUE,width=9>>=
pM <- pState( nSt, perm=c(1,2,4,3) )
head( pM )
par( mfrow=c(1,2), mar=c(3,3,1,1), mgp=c(3,1,0)/1.6 )
plot( pM )
plot( pM, border="black", col="transparent", lwd=3 )
text( rep(as.numeric(rownames(pM)[nrow(pM)-1]),ncol(pM)),
      pM[nrow(pM),]-diff(c(0,pM[nrow(pM),]))/5,
      colnames( pM ), adj=1 )
box( col="white", lwd=3 )
box()
@ %
\insfig{pstate0}{1.0}{Default layout of the \textrm{\tt plot.pState}
  graph (left), and a version with the state probabilities as lines and
  annotation of states.}

A more useful set-up of the graph would include a more through
annotation and sensible choice of colors, as seen in figure \ref{fig:pstatex}:
<<pstatex,fig=TRUE,width=9>>=
clr <- c("limegreen","orange")
# expand with a lighter version of the two chosen colors
clx <- c( clr, rgb( t( col2rgb( clr[2:1] )*2 + rep(255,3) ) / 3, max=255 ) )
par( mfrow=c(1,2), las=1, mar=c(3,3,4,2), mgp=c(3,1,0)/1.6 )
# Men
plot( pM, col=clx, xlab="Date of FU" )
lines( as.numeric(rownames(pM)), pM[,2], lwd=3 )
mtext( "60 year old male, diagnosed 1990, aged 55", side=3, line=2.5, adj=0, col=gray(0.6) )
mtext( "Survival curve", side=3, line=1.5, adj=0 )
mtext( "DM, no insulin   DM, Insulin", side=3, line=0.5, adj=0, col=clr[2] )
mtext( "DM, no insulin", side=3, line=0.5, adj=0, col=clr[1] )
axis( side=4 )
axis( side=4, at=1:19/20, labels=FALSE )
axis( side=4, at=1:99/100, labels=FALSE, tcl=-0.3 )
# Women
pF <- pState( nState( subset(simL,sex=="F"),
                      at=seq(0,11,0.2),
                      from=1995,
                      time.scale="Per" ),
              perm=c(1,2,4,3) )
plot( pF, col=clx, xlab="Date of FU" )
lines( as.numeric(rownames(pF)), pF[,2], lwd=3 )
mtext( "60 year old female, diagnosed 1990, aged 55", side=3, line=2.5, adj=0, col=gray(0.6) )
mtext( "Survival curve", side=3, line=1.5, adj=0 )
mtext( "DM, no insulin   DM, Insulin", side=3, line=0.5, adj=0, col=clr[2] )
mtext( "DM, no insulin", side=3, line=0.5, adj=0, col=clr[1] )
axis( side=4 )
axis( side=4, at=1:19/20, labels=FALSE )
axis( side=4, at=1:99/100, labels=FALSE, tcl=-0.3 )
@ %
\insfig{pstatex}{1.0}{\textrm{\tt plot.pState} graphs where persons
  ever on insulin are given in orange and persons never on insulin in
  green, and the overall survival (dead over the line) as a black line.}

If we instead wanted to show the results on the age-scale, we would
use age as timescale when constructing the probabilities; otherwise the
code is pretty much the same as before (Figure \ref{fig:pstatey}):
<<pstatey,fig=TRUE,width=9>>=
par( mfrow=c(1,2), las=1, mar=c(3,3,4,2), mgp=c(3,1,0)/1.6 )
# Men
pM <- pState( nState( subset(simL,sex=="M"),
                      at=seq(0,11,0.2),
                      from=60,
                      time.scale="Age" ),
              perm=c(1,2,4,3) )
plot( pM, col=clx, xlab="Age" )
lines( as.numeric(rownames(pM)), pM[,2], lwd=3 )
mtext( "60 year old male, diagnosed 1990, aged 55", side=3, line=2.5, adj=0, col=gray(0.6) )
mtext( "Survival curve", side=3, line=1.5, adj=0 )
mtext( "DM, no insulin   DM, Insulin", side=3, line=0.5, adj=0, col=clr[2] )
mtext( "DM, no insulin", side=3, line=0.5, adj=0, col=clr[1] )
axis( side=4 )
axis( side=4, at=1:19/20, labels=FALSE )
axis( side=4, at=1:19/20, labels=FALSE, tcl=-0.4 )
axis( side=4, at=1:99/100, labels=FALSE, tcl=-0.3 )
# Women
pF <- pState( nState( subset(simL,sex=="F"),
                      at=seq(0,11,0.2),
                      from=60,
                      time.scale="Age" ),
              perm=c(1,2,4,3) )
plot( pF, col=clx, xlab="Age" )
lines( as.numeric(rownames(pF)), pF[,2], lwd=3 )
mtext( "60 year old female, diagnosed 1990, aged 55", side=3, line=2.5, adj=0, col=gray(0.6) )
mtext( "Survival curve", side=3, line=1.5, adj=0 )
mtext( "DM, no insulin   DM, Insulin", side=3, line=0.5, adj=0, col=clr[2] )
mtext( "DM, no insulin", side=3, line=0.5, adj=0, col=clr[1] )
axis( side=4 )
axis( side=4, at=1:9/10, labels=FALSE )
axis( side=4, at=1:19/20, labels=FALSE, tcl=-0.4 )
axis( side=4, at=1:99/100, labels=FALSE, tcl=-0.3 )
@ %
Note the several statements with \texttt{axis(side=4,...}; they are
necessary to get the fine tick-marks in the right hand side of the
plots that you will need in order to read off the probabilities at
2006 (or 71 years).

\insfig{pstatey}{1.0}{\textrm{\tt plot.pState} graphs where persons
  ever on insulin are given in orange and persons never on insulin in
  green, and the overall survival (dead over the line) as a black line.}

\subsection{Comparing predictions from different models}

We have actually fitted different models for the transitions, and we
have simulated Lexis objects from all three approaches, so we can plot
these predictions on top of each other:
<<comp-0,fig=TRUE,width=9>>=
PrM  <- pState( nState( subset(simP,sex=="M"),
                        at=seq(0,11,0.2),
                        from=60,
                        time.scale="Age" ),
                perm=c(1,2,4,3) )
PrF  <- pState( nState( subset(simP,sex=="F"),
                        at=seq(0,11,0.2),
                        from=60,
                        time.scale="Age" ),
                perm=c(1,2,4,3) )
CoxM <- pState( nState( subset(simC,sex=="M"),
                        at=seq(0,11,0.2),
                        from=60,
                        time.scale="Age" ),
                perm=c(1,2,4,3) )
CoxF <- pState( nState( subset(simC,sex=="F"),
                        at=seq(0,11,0.2),
                        from=60,
                        time.scale="Age" ),
                perm=c(1,2,4,3) )

par( mfrow=c(1,2), mar=c(3,3,1,1), mgp=c(3,1,0)/1.6 )
 plot(   pM, border="black", col="transparent", lwd=3 )
lines(  PrM, border="blue" , col="transparent", lwd=3 )
lines( CoxM, border="red"  , col="transparent", lwd=3 )
text( 60.5, 0.05, "M" )
box( lwd=5, col="white" ) ; box( lwd=2, col="black" )

 plot(   pF, border="black", col="transparent", lwd=3 )
lines(  PrF, border="blue" , col="transparent", lwd=3 )
lines( CoxF, border="red"  , col="transparent", lwd=3 )
text( 60.5, 0.05, "F" )
box( lwd=5, col="white" ) ; box( lwd=2, col="black" )
@ %
\insfig{comp-0}{1.0}{Comparison of the simulated state occupancy
  probabilities using separate Poisson models for the mortality rates
  with and without insulin (black) and using proportional hazards
  Poisson models (blue) and Cox-models with diabetes duration as
  timescale and age at diabetes diagnosis as covariate (red).}

From figure \ref{fig:comp-0} it is clear that the two proportional
hazards models (blue and red curves) produce pretty much the same
estimates of the state occupancy probabilities over time, but also that
they relative to the model with separately estimated transition
intensities overestimates the probability of being alive without
insulin and underestimates the probabilities of being dead without
insulin. However both the overall survival, and the fraction of
persons on insulin are quite well in agreement with the more elaborate
model. Thus the proportional hazards models overestimate the relative
mortality of the insulin treated diabetes patients relative to the
non-insulin treated.

Interestingly, we also see a bump in the estimated probabilities from
the Cox-model based model, but this is entirely an artifact that comes
from the estimation method for the baseline hazard of the Cox-model
that lets the (cumulative) hazard have large jumps at event times
where the risk set is small. So also here it shows up that an
assumption of continuous underlying hazards leads to more credible
estimates.

\chapter{Simulation of transitions in multistate models}

\section{Theory}

Suppose that the rate functions for the transitions out of the current
state to, say, 3 different states are $\lambda_1$, $\lambda_2$ and
$\lambda_3$, and the corresponding cumulative rates are $\Lambda_1$,
$\Lambda_2$ and $\Lambda_3$, and we want to simulate an exit time and
an exit state (that is either 1, 2 or 3). This can be done in two
slightly different ways:
\begin{enumerate}
\item First time, then state:
  \begin{enumerate}
  \item Compute the survival function, $S(t) =
    \exp\bigl(-\Lambda_1(t)-\Lambda_2(t)-\Lambda_3(t)\bigr)$
  \item Simulate a random U(0,1) variate, $u$, say.
  \item The simulated exit time is then the solution $t_u$ to
    the equation $S(t_u) = u \quad \Leftrightarrow \quad \sum_j\Lambda_j(t_u) = -\log(u)$.
  \item A simulated transition at $t_u$ is then found by simulating a
    random draw from the multinomial distribution with probabilities
    $p_i=\lambda_i(t_u) / \sum_j\lambda_j(t_u)$.
  \end{enumerate}
\item Separate cumulative incidences:
  \begin{enumerate}
  \item Simulate 3 independent U(0,1) random variates $u_1$, $u_2$ and $u_3$.
  \item Solve the equations $\Lambda_i(t_i)=-\log(u_i), i=1,2,3$ and get $(t_1,t_2,t_3)$.
  \item The simulated survival time is then $\min(t_1,t_2,t_3)$, and
    the simulated transition is to the state corresponding to this,
    that is $k \in \{1,2,3\}$, where $t_k=\min(t_1,t_2,t_3)$
  \end{enumerate}
\end{enumerate}
The intuitive argument is that with three possible transition there are
3 independent processes running, but only the first transition is observed.
The latter approach is used in the implementation in \texttt{simLexis}.

The formal argument for the equality of the two approaches goes as
follows:
\begin{enumerate}
\item Equality of the transition times:
  \begin{enumerate}
  \item In the first approach we simulate from a distribution with
    cumulative rate $\Lambda_1(t)+\Lambda_2(t)+\Lambda_3(t)$, hence
    from a distribution with survival function:
    \begin{align*}
    S(t) & = \exp\bigl(-(\Lambda_1(t)+\Lambda_2(t)+\Lambda_3(t))\bigr) \\
         & = \exp\bigl(-\Lambda_1(t)\bigr)\times
             \exp\bigl(-\Lambda_2(t)\bigr)\times
             \exp\bigl(-\Lambda_3(t)\bigr)
    \end{align*}
  \item In the second approach we choose the smallest of three
    independent survival times, with survival functions
    $\exp(-\Lambda_i), i=1,2,3$. Now, the survival function for the
    minimum of three independent survival times is:
    \begin{align*}
      S_\text{min}(t) & = \pmat{\min(t_1,t_2,t_3)>t} \\
                     & = \pmat{t_1>t} \times
                         \pmat{t_2>t} \times
                         \pmat{t_3>t} \\
                     & = \exp\bigl(-\Lambda_1(t)\bigr)\times
                         \exp\bigl(-\Lambda_2(t)\bigr)\times
                         \exp\bigl(-\Lambda_3(t)\bigr)
    \end{align*}
    which is the same survival function as derived above.
  \end{enumerate}
\item Type of transition:
  \begin{enumerate}
  \item In the first instance the probability of a transition to state
    $i$, conditional on the transition time being $t$, is as known
    from standard probability theory:
    $\lambda_i(t)/\bigl(\lambda_1(t)+\lambda_2(t)+\lambda_3(t)\bigr)$.
  \item In the second approach we choose the transition corresponding
    to the the smallest of the transition times. So when we condition
    on the event that a transition takes place at time $t$, we have to
    show that the conditional probability that the smallest of the
    three simulated transition times was actually the $i$th, is as above.

    But conditional on \emph{survival} till $t$, the probabilities
    that events of type $1,2,3$ takes place in the interval $(t,t+\dif
    t)$ are $\lambda_1(t)\dif t$, $\lambda_2(t)\dif t$ and
    $\lambda_1(t)\dif t$, respectively (assuming that the probability
    of more than one event in the interval of length $\dif t$ is
    0). Hence the conditional probabilities \emph{given a transition
      time} in $(t,t+\dif t)$ is:
    \[
    \frac{\lambda_i(t)\dif t}{\lambda_1(t)\dif t+
                              \lambda_2(t)\dif t+
                              \lambda_3(t)\dif t}=
    \frac{\lambda_i(t)}{\lambda_1(t)+\lambda_2(t)+\lambda_3(t)}
    \]
    --- exactly as above.
  \end{enumerate}
\end{enumerate}

<<echo = FALSE>>=
ende <- Sys.time()
cat("  Start time:", format(anfang, "%F, %T"), "\n")
cat("    End time:", format(  ende, "%F, %T"), "\n")
cat("Elapsed time:", round(difftime(ende, anfang, units = "mins"), 2), "minutes\n")
@ %

\bibliographystyle{plain}
\begin{thebibliography}{1}

\bibitem{Carstensen.2011a}
B~Carstensen and M~Plummer.
\newblock Using {L}exis objects for multi-state models in {R}.
\newblock {\em Journal of Statistical Software}, 38(6):1--18, 1 2011.

\bibitem{Iacobelli.2013}
S~Iacobelli and B~Carstensen.
\newblock {Multiple time scales in multi-state models}.
\newblock {\em Stat Med}, 32(30):5315--5327, Dec 2013.

\bibitem{Plummer.2011}
M~Plummer and B~Carstensen.
\newblock Lexis: An {R} class for epidemiological studies with long-term
  follow-up.
\newblock {\em Journal of Statistical Software}, 38(5):1--12, 1 2011.

\end{thebibliography}

\addcontentsline{toc}{chapter}{References}

\end{document}

%\VignetteIndexEntry{Time dependent covariates in Lexis objects: addCov & addDrug}
\SweaveOpts{results=verbatim,keep.source=TRUE,include=FALSE,eps=FALSE}
\documentclass[a4paper,twoside,12pt]{report}

\newcommand{\Title}{Time dependent covariates in\\ \texttt{Lexis} objects}
\newcommand{\Tit}{addLex}
\newcommand{\Version}{Version 6}
\newcommand{\Dates}{March 2024}
\newcommand{\Where}{SDCC}
\newcommand{\Homepage}{\url{https://bendixcarstensen.com/Epi/}}
\newcommand{\Faculty}{\begin{tabular}{rl}
Bendix Carstensen
  & Steno Diabetes Center Copenhagen, Herlev, Denmark\\
  & {\small \& Department of Biostatistics,
               University of Copenhagen} \\
  & \texttt{b@bxc.dk}\\
  & \url{https://BendixCarstensen.com/} \\[1em]
                      \end{tabular}}
\input{topreport}
<<echo=FALSE>>=
options(width = 90,
        SweaveHooks = list(fig = function()
        par(mar = c(3,3,1,1),
            mgp = c(3,1,0) / 1.6,
            las = 1,
           lend = "butt",
            bty = "n")))
library(Epi)
library(popEpi)
library(dplyr)
library(tidyr)
@ %
\renewcommand{\rwpre}{./02addLexis}

<<echo = FALSE>>=
anfang <- Sys.time()
@ %

\chapter{Overview and rationale}

This note describes the functions \texttt{addCov.Lexis}, designed to
add values of clinical measurements, and \texttt{addDrug.Lexis}
designed to add drug exposure to time-split \texttt{Lexis} objects.

If time-dependent variables are binary, such as for example
``occurrence of CVD diagnosis'' it may be relevant to define a new
state as, say, \texttt{CVD}; this is the business of the funtions
\texttt{cutLexis} and its cousins. The purposes of the two functions
discussed here are to append quantitative variables that in principle
can take any value.

Both functions are so-called \texttt{S3} methods for \texttt{Lexis}
objects, so in code you can omit the ``\texttt{.Lexis}''.  Note that
neither \texttt{cutLexis}, \texttt{splitLexis} or \texttt{splitMulti}
are \texttt{S3} methods (there is no ``\texttt{.}'' in the names).

\section{\texttt{addCov.Lexis}}

\ldots provides the ability to amend a \texttt{Lexis} object with
clinical measurements taken at different times, and propagate the
values as LOCF (Last Observation Carried Forward) to all subsequent
records. This means that time-splitting of a \texttt{Lexis} object
\emph{after} adding clinical measurements will be meaningful, because
both \texttt{splitLexis} and \texttt{splitMulti} will carry variables
forward across the split records. The follow-up in the resulting
\texttt{Lexis} object will be cut at dates of clinical measurement.

\texttt{addCov.Lexis} will also propagate missing values supplied as
measurements. Therefore, if you want to have LOCF \emph{across}
supplied times of measurement you must explicitly apply
\texttt{tidyr::fill} to the resulting \texttt{Lexis} object, after a
suitable \texttt{group\_by}.

\section{\texttt{addDrug.Lexis}}

As opposed to this, \texttt{addDrug.Lexis} will first use drug
information at each date of recorded drug purchase, and subsequently
\texttt{compute} cumulative exposure measures at the times in the
resulting \texttt{Lexis} object. This is essentially by linear
interpolation, so it will not be meaningful to further split an
object resulting from \texttt{addDrug.Lexis}---LOCF is not meaningful
for continuously time-varying covariates such as cumulative exposure.

If persons have very frequent drug purchases, the intervals may become
very small and the sheer number of records may present an impediment
to analysis. Therefore the function \texttt{coarse.Lexis} is provided
to collapse adjacent follow-up records. Note that
\texttt{coarse.Lexis} is \emph{not} an \texttt{S3} method.

\chapter{\texttt{addCov.Lexis}}

\section{Rationale}
The function has arisen out of a need to attach values measured at
clinical visits to a Lexis object representing follow-up for events
constituting a multistate model. Hence the data frame with
measurements at clinical visits will be called \texttt{clin} for
mnemonic reasons.

\section{Example}
For illustration we devise a small bogus cohort of 3 people, where we
convert the character dates into numerical variables (fractional
years) using \texttt{cal.yr}. Note that we are using a character
variable as \texttt{id}:
<<>>=
xcoh <- structure(list(id = c("A", "B", "C"),
                    birth = c("1952-07-14", "1954-04-01", "1987-06-10"),
                    entry = c("1965-08-04", "1972-09-08", "1991-12-23"),
                     exit = c("1997-06-27", "1995-05-23", "1998-07-24"),
                     fail = c(1, 0, 1) ),
                   .Names = c("id", "birth", "entry", "exit", "fail"),
                row.names = c("1", "2", "3"),
                    class = "data.frame" )
xcoh$dob <- cal.yr(xcoh$birth)
xcoh$doe <- cal.yr(xcoh$entry)
xcoh$dox <- cal.yr(xcoh$exit )
xcoh
@ %

\subsection{A \texttt{Lexis} object}

Define this as a \texttt{Lexis} object with timescales calendar time
(\texttt{per}, period) and age (\texttt{age}):
<<>>=
Lcoh <- Lexis(entry = list(per = doe),
               exit = list(per = dox,
                           age = dox - dob),
                 id = id,
        exit.status = factor(fail, 0:1, c("Alive","Dead")),
               data = xcoh)
str(Lcoh)
(Lx <- Lcoh[,1:6])
@ %

\subsubsection{Factor or character \texttt{lex.id}?}

Note that when the \texttt{id} argument to \texttt{Lexis} is a
character variable then the \texttt{lex.id} will be a factor. Which,
if each person has a lot of records may save time, but if you subset
may be a waste of space. Moreover merging (\ie joining in the language
of \texttt{tidyverse}) may present problems with different
levels. \texttt{merge} from the \texttt{base} \R, will coerce to
factor with union of levels as levels, where as the \texttt{\_join}
functions from \texttt{dplyr} will coerce to character.

Thus the most reasonable strategy thus seems to keep \texttt{lex.id}
as a character variable.
<<>>=
Lx$lex.id <- as.character(Lx$lex.id)
str(Lx)
Lx
@ %

\subsubsection{Clinical measurements}

Then we generate data frame with clinical examination data, that is
date of examination in \texttt{per}, some (bogus) clinical measurements
and also names of the examination rounds:
<<>>=
clin <- data.frame(lex.id = c("A", "A", "C", "B", "C"),
                      per = cal.yr(c("1977-3-17",
                                     "1973-7-29",
                                     "1996-3-1",
                                     "1990-7-14",
                                     "1989-1-31")),
                       bp = c(120, 140, 160, 157, 145),
                     chol = c(NA, 5, 8, 9, 6),
                     xnam = c("X2", "X1", "X1", "X2", "X0"),
         stringsAsFactors = FALSE)
str(clin)
clin
@ %
We set up this data frame with an \texttt{id} variable called
\texttt{lex.id} and a date of examination, \texttt{per}, that has the
same name as one of the time scales in the Lexis object \texttt{Lx}.
Note that we have chosen a measurement for person \texttt{C} from
1989---before the person's entry to the study, and have an \texttt{NA}
for \texttt{chol} for person \texttt{A}.

\subsection{Adding clinical data}

There is a slightly different behaviour according to whether the
variable with the name of the examination is given or not, and whether
the name of the (incomplete) time scale is given or not:
<<>>=
(Cx <- addCov.Lexis(Lx, clin))
@ %
Note that the clinical measurement preceding the entry of person
\textsc{C} is included, and that the \texttt{tfc} (time from clinical
measurement) is correctly rendered, we a non-zero value at date of entry.

We also see that a variable \texttt{exnam} is constructed with
consecutive numbering of examinations within each person, while the
variable \texttt{xnam} is just carried over as any other.

If we explicitly give the name of the variable holding the examination
names we do not get a constructed \texttt{exnam}. We can also
define the name of the (incomplete) timescale to hold the time
since measurement, in this case as \texttt{tfCl}:
<<>>=
(Dx <- addCov.Lexis(Lx, clin, exnam = "xnam", tfc = "tfCl"))
summary(Dx, t=T)
@ %

\section{Exchanging split and add}

As noted in the beginning of this note, \texttt{addCov.Lexis} uses LOCF,
and so it is commutative with \texttt{splitLexis}:
<<>>=
# split BEFORE add
Lb <- addCov.Lexis(splitLexis(Lx,
                      time.scale = "age",
                          breaks = seq(0, 80, 5)),
                   clin,
                   exnam = "xnam" )
Lb
#
# split AFTER add
La <- splitLexis(addCov.Lexis(Lx,
                            clin,
                           exnam = "xnam" ),
                 time.scale = "age",
                     breaks = seq(0, 80, 5))
La
@ %
We see that the results are identical, bar the sequence of variables
and attributes.

We can more explicitly verify that the resulting data frames are the same:
<<>>=
La$tfc == Lb$tfc
La$age == Lb$age
La$per == Lb$per
@ %
The same goes for \texttt{splitMulti}:
<<>>=
## split BEFORE add
Mb <- addCov.Lexis(splitMulti(Lx, age = seq(0, 80, 5)),
                   clin,
                   exnam = "xnam" )
##
## split AFTER add
Ma <- splitMulti(addCov.Lexis(Lx,
                              clin,
                              exnam = "xnam" ),
                 age = seq(0, 80, 5))
La$tfc == Mb$tfc
Ma$tfc == Mb$tfc
@ %
In summary, because both \texttt{addCov.Lexis} and
\texttt{splitLexis}/\texttt{splitMulti} use LOCF for covariates the
order of splitting and adding does not matter.

This is certainly not the case with \textrm{addDrug.Lexis} as we shall see.

\section{Filling the \texttt{NA}s}

As mentioned in the beginning, clinical measurements given as
\texttt{NA} in the \texttt{clin} data frame are carried forward. If
you want to have these replaced by 'older' clinical measurements you
can do that explicitly by \texttt{dplyr::fill} with a construction
like:
<<>>=
cov <- c("bp", "chol")
Lx <- La
Lx <- group_by(Lx, lex.id) %>%
         fill(all_of(cov)) %>%
      ungroup()
class(Lx)
@ %
We see that the \texttt{Lexis} attributes are lost by using the
\texttt{group\_by} function, so we fish out the covariates from the
\texttt{tibble} and stick them back into the \texttt{Lexis} object:
<<>>=
Lx <- La
Lx[,cov] <- as.data.frame(group_by(Lx, lex.id)
                          %>% fill(all_of(cov)))[,cov]
class(Lx)
La
Lx
@ %
The slightly convoluted code where the covariate columns are
explicitly selected, owes to the fact that the \texttt{dplyr}
functions will strip the data frames of the \texttt{Lexis}
attributes. So we needed to use \texttt{fill} to just generate the
covariates and not touch the \texttt{Lexis} object itself.

This should of course be built into \texttt{addCov.Lexis} as a
separate argument, but is not yet.

Note that the \texttt{tfc}, time from clinical measurement, is now not
a valid time scale variable any more; the 5 in \texttt{chol} is
measured at 1973.7 but \texttt{tfc} is reset to 0 at 1977.21, even if
only \texttt{bp} but not \texttt{chol} is measured at that time.  If
you want that remedied you will have to use \texttt{addCov.Lexis}
twice, one with a \texttt{clin} data frame with only \texttt{bp} and
another with a data frame with only \texttt{chol}, each generating a
differently named variable holding the time from clinical measurement.

This is a problem that comes from the structure of the supplied
\emph{data} not from the program features; in the example we had
basically measurements of different clinical variables at different
times, and so necessarily also a need for different times since last
measurement.

\chapter{\texttt{addDrug.Lexis}}

The general purpose of the function is to amend a \texttt{Lexis}
object with drug exposure data. The data base with information on a
specific drug is assumed to be a data frame with one entry per drug
purchase (or prescription), containing the date and the amount
purchased and optionally the prescribed dosage (that is how much is
supposed to be taken per time). We assume that we have such a data
base for each drug of interest, which also includes an id variable,
\texttt{lex.id}, that matches the \texttt{lex.id} variable in the
\texttt{Lexis} object.

For each type of drug the function derives 4 variables:
\begin{description}[noitemsep]
 \item[\hspace*{1em}\texttt{ex}]: logical, is the person currently
   \texttt{ex}posed
 \item[\hspace*{1em}\texttt{tf}]: numeric, \texttt{t}ime since
   \texttt{f}irst purchase
 \item[\hspace*{1em}\texttt{ct}]: numeric, \texttt{c}umulative
   \texttt{t}ime on the drug
 \item[\hspace*{1em}\texttt{cd}]: numeric, \texttt{c}umulative
   \texttt{d}ose of the drug
\end{description}
These names are pre- or suf-fixed by the drug name, so that exposures
to different drugs can be distinguished; see the examples.

The resulting \texttt{Lexis} object has extra records corresponding to
cuts at each drug purchase and at each expiry date of a purchase.  For
each purchase the coverage period is derived (different methods for
this are available), and if the end of this (the expiry date) is
earlier than the next purchase of the person, the person is considered
off the drug from the expiry date, and a cut in the follow-up is
generated with \texttt{ex} set to \texttt{FALSE}.

\section{The help example}

The following is a slight modification of the code from the example
section of the help page for \texttt{addDrug.Lexis}

First we generate follow-up of 2 persons, and split the follow-up in
intervals of length 0.6 years along the calendar time scale,
\texttt{per}:
<<>>=
fu <- data.frame(doe = c(2006, 2008),
                 dox = c(2015, 2018),
                 dob = c(1950, 1951),
                 xst = factor(c("A","D")))
Lx <- Lexis(entry = list(per = doe,
                         age = doe- dob),
             exit = list(per = dox),
      exit.status = xst,
             data = fu)
Lx <- subset(Lx, select = -c(doe, dob, dox, xst))
Sx <- splitLexis(Lx, "per", breaks = seq(1990, 2020, 0.6))
summary(Sx)
str(Sx)
@ %
Note that as opposed to the previous example, the time scales are not
of class \texttt{cal.yr}, they are just numerical.

Then we generate example drug purchases for these two persons, one
data frame for each of the drugs \texttt{F} and \texttt{G}. Note that
we generate \texttt{lex.id}$\in (1,2)$ referring to the values of
\texttt{lex.id} in the lexis object \texttt{Sx}.
<<>>=
set.seed(1952)
rf <- data.frame(per = c(2005 + runif(12, 0, 10)),
                 amt = sample(2:4, 12, replace = TRUE),
              lex.id = sample(1:2, 12, replace = TRUE)) %>%
      arrange(lex.id, per)

rg <- data.frame(per = c(2009 + runif(10, 0, 10)),
                 amt = sample(round(2:4/3,1), 10, replace = TRUE),
              lex.id = sample(1:2, 10, replace = TRUE)) %>%
      arrange(lex.id, per)
@ %
We do not need to sort the drug purchase data frames (it is done
internally by \texttt{addDrug.Lexis}), but it makes it easier to grasp
the structure. Note that we generated the drug purchase files with the
required variable names.

The way purchase data is supplied to the function is in a
\texttt{list} where each element is a data frame of purchase records
for one type of drug. The list must be named, because the names are
used as prefixes of the generated exposure variables. We can show the
resulting data in a list:
<<>>=
pdat <- list(F = rf, G = rg)
pdat
Lx
@ %
Note that we have generated data so that there are drug purchases of
drug \texttt{F} that is \emph{before} start of follow-up for person 2.

We can then expand the time-split \texttt{Lexis} object, \texttt{Sx}
with the drug information. \texttt{addDrug.Lexis} not only adds 8
\emph{variables} (4 from each drug), it also adds \emph{records}
representing cuts at the purchase dates and possible expiry dates.
<<>>=
summary(Sx) ; names(Sx)
ex1 <- addDrug.Lexis(Sx, pdat, method = "ext") # default
summary(ex1) ; names(ex1)
print(ex1, nd = 2)
ex2 <- addDrug.Lexis(Sx, pdat, method = "ext", grace = 0.5)
summary(ex2)
print(ex2, nd = 2)
dos <- addDrug.Lexis(Sx, pdat, method = "dos", dpt = 6)
summary(dos)
print(dos, nd = 2)
fix <- addDrug.Lexis(Sx, pdat, method = "fix", maxt = 1)
summary(fix)
print(fix, nd = 2)
@ %

\section{A more realistic example with run times}

\subsection{Follow-up data: \texttt{DMlate}}

As example data we use rows from the \texttt{DMlate} example data from
the \texttt{Epi} package:
<<>>=
data(DMlate) ; str(DMlate)
Lx <- Lexis(entry = list(per = dodm,
                         age = dodm - dobth,
                         tfd = 0),
             exit = list(per = dox),
      exit.status = factor(!is.na(dodth),
                           labels = c("DM", "Dead")),
             data = DMlate[sample(1:nrow(DMlate), 1000),])
summary(Lx)
@ %
We split the data along the age-scale (omitting the variables we shall
not need):
<<>>=
Sx <- splitLexis(Lx[,1:7], time.scale="age", breaks = 0:120)
summary(Sx)
str(Sx)
@ %

\subsection{Artificial prescription data}

To explore how \texttt{addDrug.Lexis} works, we need drug exposure
data, but these are unfortunately not available, so we simulate three
datasets representing \texttt{pur}chases of three types of drugs:
<<>>=
set.seed(1952)

purA <-
  ( data.frame(lex.id = rep(Lx$lex.id,
                            round(runif(nrow(Lx), 0, 20))))
%>% left_join(Lx[,c("lex.id", "dodm", "dox")])
%>% mutate(per = dodm + runif(length(dodm), -0.1, 0.99) * (dox - dodm),
           amt = sample(4:20*10, length(dodm), replace = TRUE),
           dpt = amt * round(runif(length(dodm), 3, 7)))
%>% select(-dodm, -dox)
%>% arrange(lex.id, per)
  )
addmargins(table(table(purA$lex.id)))
str(purA)

purB <-
  ( data.frame(lex.id = rep(Lx$lex.id,
                            round(pmax(runif(nrow(Lx), -10, 15), 0))))
%>% left_join(Lx[,c("lex.id", "dodm", "dox")])
%>% mutate(per = dodm + runif(length(dodm), -0.1, 0.99) * (dox - dodm),
           amt = sample(4:20*10, length(dodm), replace = TRUE),
           dpt = amt * round(runif(length(dodm), 5, 9)))
%>% select(-dodm, -dox)
%>% arrange(lex.id, per)
  ) -> purB
addmargins(table(table(purB$lex.id)))
str(purB)

purC <-
  ( data.frame(lex.id = rep(Lx$lex.id,
                            round(pmax(runif(nrow(Lx), -5, 12), 0))))
%>% left_join(Lx[,c("lex.id", "dodm", "dox")])
%>% mutate(per = dodm + runif(length(dodm), -0.1, 0.99) * (dox - dodm),
           amt = sample(4:20*10, length(dodm), replace = TRUE),
           dpt = amt * round(runif(length(dodm), 5, 7)))
%>% select(-dodm, -dox)
%>% arrange(lex.id, per)
  )
addmargins(table(table(purC$lex.id)))
str(purC)
head(purC)
@ %
Note that the time scale is in years, so the \texttt{dpt} must be in
amount per year, so that $\mathtt{dpt}/\mathtt{amt}$ is the
approximate number of annual drug purchases.

We now have three artificial drug purchase datasets so we can see how
\texttt{addDrug.Lexis} performs on larger datasets:

\subsection{Using \texttt{addDrug}}

\subsubsection{100 and 500 persons}

We start out with a small sample and a three month grace period to limit
the number of gaps:
<<>>=
Sx1 <- subset(Sx, lex.id < 100)
pur <- list(A = subset(purA, lex.id < 1000),
            B = subset(purB, lex.id < 1000),
            C = subset(purC, lex.id < 1000))
system.time(ad1 <- addDrug.Lexis(Sx1, pur, tnam = "per", grace = 1/4))
summary(Sx1)
summary(ad1)
@ %
We then cut the number of persons in half to assess how run time
depends on no. of persons in the data:
<<>>=
Sx2 <- subset(Sx, lex.id < 500)
pur <- list(A = subset(purA, lex.id < 500),
            B = subset(purB, lex.id < 500),
            C = subset(purC, lex.id < 500))
system.time(ad2 <- addDrug.Lexis(Sx2, pur, tnam = "per", grace = 1/6))
summary(Sx2)
summary(ad2)
@ %
\ldots timing is broadly proportional to the number of persons.

\subsubsection{Fewer prescription records}

We can try to cut the number of purchases in half:
<<>>=
pur <- list(A = subset(purA, lex.id < 100 & runif(nrow(purA)) < 0.5),
            B = subset(purB, lex.id < 100 & runif(nrow(purB)) < 0.5),
            C = subset(purC, lex.id < 100 & runif(nrow(purC)) < 0.5))
sapply(pur, nrow)
system.time(ad3 <- addDrug.Lexis(Sx1, pur, tnam = "per", grace = 1/6))
summary(Sx1)
summary(ad3)
@ %
It also appears that the number of purchases per person is also a
determinant of the run time too; the timing is largely proportional to
the number of drug records.

In any concrete application it is recommended to run the function on
a fairly small sample of persons, say 1000 to get a feel for the run
time. It may also be a good idea to run the function on chunks of the
persons, to make sure that you do not lose all the processed data in a
crash.

\subsubsection{Fewer prescription types}

Finally we try to cut the number of drugs, to assess how this
influences the run time:
<<>>=
pur <- list(B = subset(purB, lex.id < 100),
            C = subset(purC, lex.id < 100))
sapply(pur, nrow)
system.time(ad4 <- addDrug.Lexis(Sx1, pur, tnam = "per", grace = 1/6))
summary(Sx1)
summary(ad4)
@ %
We see that the number of drugs also influence the run time
proportionally.

\subsection{Too many records ---\texttt{coarse.Lexis}}

If we look at the length of the intervals as given in \texttt{lex.dur}
we see that some are are quite small:
<<>>=
summary(ad1$lex.dur)
@ %
Half are smaller then 0.11 years, 40 days. We could without much loss
of precision in the analysis based on the \texttt{Lexis} object merge
adjacent records that have total risk time less than 3 months.

The function \texttt{coarse.Lexis} will collapse records with short
\texttt{lex.dur} with the subsequent record. The collapsing will use
the covariates from the first record, and so the entire follow-up from
the two records will have the characteristics of the first. Therefore
it is wise to choose first records with reasonably short
\texttt{lex.dur}---the approximation will be better than if the first
record was with a larger \texttt{lex.dur}. Therefore there are two
values supplied to \texttt{coarse.Lexis}; the maximal length of the
first record's \texttt{lex.dur} and the maximal length of the
\texttt{lex.dur} in the resulting combined record. The larger these
parameters are, the more the \texttt{Lexis} object is coarsened.
<<>>=
summary(ad1)
summary(adc <- coarse.Lexis(ad1, lim = c(1/6,1/2)))
summary(adc$lex.dur)
@ %
This could cut the number of units for analysis substantially, in this
case from about 27,000 to some 13,000.

\subsubsection{Records to be kept}

When we are dealing with drug exposure data we will be interested
keeping the record that holds the start of a drug exposure. Some may
argue that it does not matter much, though.

The records (\ie beginnings of FU intervals) that should be kept must
be given in logical vector in the argument \texttt{keep}:
<<>>=
summary(Sx2)
system.time(ad4 <- addDrug.Lexis(Sx2,
                                 pur,
                                tnam = "per",
                               grace = 1/6))
summary(ad4)
#
ad5 <- coarse.Lexis(ad4,
                    lim = c(1/4, 1/2))
summary(ad5)
@
We can identify the first date of exposure to drug \texttt{B}, say, by
the exposure (\texttt{B.ex}) being true and the cumulative time on the
drug (\texttt{B.ct}) being 0:
<<>>=
ad4$keep <- with(ad4, (B.ex & B.ct == 0) |
                      (C.ex & C.ct == 0))
ad6 <- coarse.Lexis(ad4,
                    lim = c(1/4, 1/2),
                   keep = ad4$keep)
summary(ad6)
@ %
We see the expected behaviour when we use \texttt{coarse.Lexis}: we get
fewer records, but identical follow-up. And the \texttt{keep} argument
gives the possibility to keep selected records, or more precisely
beginnings. \texttt{keep} prevents a record to be collapsed with a
previous one, but not with a subsequent one.

%% \subsection{The entire example dataset}

%% The entire amount of example data consist of some 10,000 persons and
%% some 200,000 prescriptions:
%% <<eval=FALSE>>=
%% dim(Sx)
%% pur <- list(A = purA,
%%             B = purB,
%%             C = purC)
%% sapply(pur, nrow)
%% system.time(adx <- addDrug.Lexis(Sx, pur, tnam = "per", grace = 1/6))
%% system.time(adc <- coarse.Lexis(adx, lim = c(1/6, 1/2)))
%% summary(Sx)
%% summary(adx)
%% summary(adc)
%% @ %
%% We see hat the number of records is quite large because we have cut at
%% all purchase dates and integer ages. For practical purposes we might
%% therefore want to merge successive records with a total duration
%% \texttt{lex.dur} less than some limit.

<<echo = FALSE>>=
ende <- Sys.time()
cat("  Start time:", format(anfang, "%F, %T"),
  "\n    End time:", format(  ende, "%F, %T"),
  "\nElapsed time:", round(difftime(ende, anfang, units = "mins"), 2), "minutes\n")
@ %

\bibliographystyle{plain}

\begin{thebibliography}{1}

\bibitem{Carstensen.2007a}
B~Carstensen.
\newblock Age-{P}eriod-{C}ohort models for the {L}exis diagram.
\newblock {\em Statistics in Medicine}, 26(15):3018--3045, 2007.

\bibitem{Carstensen.2008c}
B~Carstensen, JK~Kristensen, P~Ottosen, and K~Borch-Johnsen.
\newblock The {D}anish {N}ational {D}iabetes {R}egister: {T}rends in incidence,
  prevalence and mortality.
\newblock {\em Diabetologia}, 51:2187--2196, 2008.

\end{thebibliography}

\addcontentsline{toc}{chapter}{References}

\end{document}

%\VignetteIndexEntry{Years of life lost (YLL)}
\SweaveOpts{results=verbatim, keep.source=TRUE, include=FALSE, eps=FALSE}
\documentclass[a4paper, twoside, 12pt]{report}

\newcommand{\Title}{Years of Life Lost (YLL)
  to disease\\Diabetes in DK as example}
\newcommand{\Tit}{YLL}
\newcommand{\Version}{February 1.2}
\newcommand{\Dates}{November 2017}
\newcommand{\Where}{SDC}
\newcommand{\Homepage}{\url{http://bendixcarstensen.com/Epi}}
\newcommand{\Faculty}{\begin{tabular}{rl}
Bendix Carstensen
  & Steno Diabetes Center Copenhagen, Herlev, Denmark\\
  & {\small \& Department of Biostatistics,
               University of Copenhagen} \\
  & \texttt{b@bxc.dk}\\
  & \url{http://BendixCarstensen.com} \\[1em]
                      \end{tabular}}
\input{topreport}
<<echo=FALSE>>=
options(width=90,
         SweaveHooks=list(fig=function()
         par(mar = c(3, 3, 1, 1),
             mgp = c(3, 1, 0) / 1.6,
             las = 1,
             bty = "n")))
@ %
\renewcommand{\rwpre}{./yll}

<<echo = FALSE>>=
anfang <- Sys.time()
cat("Start time:", format(anfang, "%F, %T"), "\n")
@ %

\chapter{Theory and technicalities}

This vignette for the \texttt{Epi} package describes the
probabilistic and demographic background for and technical implementation
of the \texttt{erl} and \texttt{yll} functions that computes the
expected residual life time and years of life lost in an illness-death
model.

\section{Years of life lost (YLL)}

\ldots to diabetes or any other disease for that matter.

The general concept in calculation of ``years lost to\ldots'' is the
comparison of the expected lifetime between two groups of persons; one
with and one without disease (in this example DM). The expected
lifetime is the area under the survival curve, so basically the
exercise requires that two survival curves that are deemed relevant be
available.

The years of life lost is therefore just the area between the survival
curves for those ``Well'', $S_W(t)$, and for those ``Diseased'',
$S_D(t)$:
\[
  \YLL = \int_0^\infty S_W(t) - S_D(t) \dif t
\]
The time $t$ could of course be age, but it could also be ``time after
age 50'' and the survival curves compared would then be survival
curves \emph{conditional} on survival till age 50, and the YLL would
be the years of life lost for a 50 year old person with diabetes
relative to a 50 year old person without diabetes.

If we are referring to the expected lifetime we will more precisely use
the label expected residual lifetime, ERL.

\section{Constructing the survival curves}

YLL can be computed in two different ways, depending on the way the
survival curve and hence the expected lifetime of a person
\emph{without} diabetes is computed:
\begin{itemize}
\item Assume that the ``Well'' persons are \emph{immune} to disease
  --- using only the non-DM mortality rates throughout for calculation
  of expected life time.
\item Assume that the ``Well'' persons \emph{can} acquire the disease and
  thereby see an increased mortality, thus involving all three rates
  shown in figure \ref{fig:states}.
\end{itemize}
The former gives a higher YLL because the comparison is to persons
assumed immune to DM (and yet with the same mortality as non-immune
prior to diagnosis), the latter gives a more realistic picture of the
comparison of group of persons with and without diabetes at a given
age that can be interpreted in the real world.

The differences can be illustrated by figure \ref{fig:states}; the
immune approach corresponds to an assumption of $\lambda(t)=0$ in the
calculation of the survival curve for a person in the ``Well'' state.

Calculation of the survival of a diseased person already in the ``DM''
state is unaffected by assumptions about $\lambda$.

<<states, fig=TRUE, height=5, width=5, echo=FALSE>>=
library(Epi)
TM <- matrix(NA, 4, 4)
rownames(TM) <-
colnames(TM) <- c("Well", "DM", "Dead", "Dead(DM)")
TM[1, 2:3] <- TM[2, 4] <- 1
zz <- boxes(TM, boxpos = list(x = c(20, 80, 20, 80),
                              y = c(80, 80, 20, 20)),
                wm = 1.5,
                hm = 4)
@
We can edit the output from \texttt{boxes}
<<states, fig=TRUE, height=4, width=5, echo=FALSE>>=
zz$Arrowtext <- c(expression(lambda),
                  expression(mu[W]),
                  expression(mu[D][M]))
boxes.MS(zz)
@ %
\insfig{states}{0.7}{Illness-death model describing diabetes incidence
  and -mortality.}

\subsection{Total mortality --- a shortcut?}

A practical crude shortcut could be to compare the ERL in the diabetic
population to the ERL for the \emph{entire} population (that is using
the total mortality ignoring diabetes status).

Note however that this approach also counts the mortality of persons
that acquired the disease earlier, thus making the comparison
population on average more ill than the population we aim at, namely
those well at a given time, which only then become more gradually ill.

How large these effects are can however be empirically explored, as we
shall do later.

\subsection{Disease duration}

In the exposition above there is no explicit provision for the effect of
disease duration, but if we were able to devise mortality rates for
any combination of age and duration, this could be taken into account.

There are however severe limitations in this as we in principle would
want to have duration effects as long as the age-effects --- in
principle for all $(a, d)$ where $d\leq A$, where $A$ is the age at
which we condition. So even if we were only to compute ERL from
age, say, 40 we would still need duration effects up to 60 years
(namely to age 100).

The incorporation of duration effects is in principle trivial from a
computational point of view, but we would be forced to entertain
models predicting duration effects way beyond what is actually
observed disease duration in any practical case.

\subsection{Computing integrals}

The practical calculations of survival curves, ERL and YLL involves
calculation of (cumulative) integrals of rates and functions of these
as we shall see below. This is easy if we have a closed form
expression of the function, so its value may be computed at any time
point --- this will be the case if we model rates by smooth parametric
functions.

Computing the (cumulative) integral of a function is done as follows:
\begin{itemize}
\item Compute the value of the function (mortality rate for example)
  at the midpoints of a sequence of narrow equidistant intervals ---
  for example one- or three month intervals of age, say.
\item Take the cumulative sum of these values multiplied by the
  interval length --- this will be a very close approximation to the
  cumulative integral evaluated at the end of each interval.
\item If the intervals are really small (like 1/100 year), the
  distinction between the value at the middle and at the end of each
  interval becomes irrelevant.
\end{itemize}
Note that in the above it is assumed that the rates are given in units
corresponding to the interval length --- or more precisely, as the
cumulative rates over the interval.

\section{Survival functions in the illness-death model}

The survival functions for persons in the ``Well'' state can be
computed under two fundamentally different scenarios, depending on
whether persons in the ``Well'' state are assumed to be immune to the
disease ($\lambda(a)=0$) or not.

\subsection{Immune approach}

In this case both survival functions for person in the two states are
the usual simple transformation of the cumulative mortality rates:
\[
 S_W(a) = \exp\left(-\int_0^a\!\!\mu_W(u) \dif u \right), \qquad
 S_D(a) = \exp\left(-\int_0^a\!\!\mu_D(u) \dif u \right)
\]

\subsubsection{Conditional survival functions}

If we want the \emph{conditional} survival functions given survival to
age $A$, say, they are just:
\[
 S_W(a|A) = S_W(a)/S_W(A), \qquad S_D(a|A) = S_D(a)/S_D(A)
\]

\subsection{Non-immune approach}

For a diseased person, the survival function in this states is the same
as above, but the survival function for a person without disease (at
age 0) is (see figure \ref{fig:states}):
\[
S(a) = \ptxt{Well}\!(a) + \ptxt{DM}\!(a)
\]
In the appendix of the paper \cite{Carstensen.2008c} is an indication
of how to compute the probability of being in any of the four states
shown in figure \ref{fig:states}, which I shall repeat here:

In terms of the rates, the probability of being in the ``Well'' box is
simply the probability of escaping both death (at a rate of $\mu_W(a)$)
and diabetes (at a rate of $\lambda(a)$):
\[
   \ptxt{Well}(a)  = \exp\left(\!-\int_0^a\!\!\mu_W(u)+\lambda(u) \right) \dif u
\]
The probability of being alive with diabetes at age $a$, is computed given that
 diabetes occurred at age $s$ ($s<a$) and then integrated over $s$ from $0$
 to $a$:
\begin{align*}
 \ptxt{DM}(a) = \int_0^a\!\! & \ptxt{survive to $s$, DM diagnosed at $s$} \\
                & \times \ptxt{survive with DM from $s$ to $a$} \dif s \\
              = \int_0^a\!\! & \lambda(s)
                           \exp\left(\!-\int_0^s\!\!\mu_W(u)+\lambda(u) \dif u \right) \\
                & \times \exp\left(\!-\int_s^a\!\!\mu_D(u) \dif u \right) \dif s
\end{align*}
Sometimes we will use a version where the mortality among diabetes
patients depend both on age $a$ and duration of diabetes, $d$,
$\mu_D(a, d)$, in which case we get:
\begin{align*}
 \ptxt{DM}(a) = \int_0^a \! & \lambda(s)
                \exp\left(-\int_0^s\!\mu_W(u)+\lambda(u) \dif u \right) \\
                & \times \exp\left(-\int_s^a\!\mu_D(u, u-s) \dif u \right) \dif s
\end{align*}
because the integration variable $u$ is the age-scale and the second
integral refers to mortality among persons diagnosed at age $s$, that
is, with duration $u-s$ at age $u$.

The option of using duration-dependent mortality rates among diseased
individuals is not implemented yet.

\subsubsection{Conditional survival functions}

Unlike the immune approach, the conditional survival function in the
more realistic case is not just a ratio of the unconditional to the
value at the conditioning age, $A$, say. This would amount to
conditioning on being merely \emph{alive} at age $A$, but what we want
is to condition on being in the ``Well'' state at age $A$.

The formulae for the conditional probabilities of being either in
``Well'' or ``DM'', given being in ``Well'' at age $A$ are basically
replicates of the unconditional, albeit with changes in integration
limits:
\begin{align*}
\ptxt{Well|Well at $A$}(a) &= \exp\left(-\int_A^a \! \mu_W(u)+\lambda(u) \right) \dif u \\
  \ptxt{DM|Well at $A$}(a) &= \int_A^a \! \lambda(s)
                               \exp\left(-\int_A^s\!\mu_W(u)+\lambda(u) \dif u \right) \\
                           & \qquad \times \exp\left(-\int_s^a\!\mu_D(u, u-s) \dif u \right) \dif s
\end{align*}
The calculation of these conditional survival functions is implemented
but not allowing for duration-dependence. Thus it is only implemented
assuming $\mu_D(a, d)=\mu_D(a)$.

\chapter{Analyses for DM in Denmark}

The rates we use as basis for the following calculations are derived
from the NDR, where we have omitted the blood-glucose criteria,
because there is compelling evidence that these have quite a low
specificity (particularly in the younger ages among women), and do
not substantially contribute to the sensitivity.

As noted above the calculations of YLL requires access to
(age-specific) rates of incidence of DM and mortality for persons with
and without DM.

\section{Modeling mortality and incidence data}

We read in the dataset of DM and population mortality and incidence, \texttt{DMepi}:
<<>>=
data(DMepi)
@ %
The dataset \texttt{DMepi} contains diabetes events, deaths and
person-years for persons without diabetes and deaths and person-years
for persons with diabetes, classified by age (\texttt{A}) and calendar
year (\texttt{P}):
<<>>=
str(DMepi)
head(DMepi)
@ %
For each combination of sex, age, period and date of birth in 1 year
age groups, we have the person-years in the ``Well'' (\texttt{Y.nD})
and the ``DM'' (\texttt{Y.DM}) states, as well as the number of deaths
from these (\texttt{D.nD}, \texttt{D.DM}) and the number of incident
diabetes cases from the ``Well'' state (\texttt{X}).

In order to compute the years of life lost to diabetes and how this
has changed over time, we fit models for the mortality and incidence
of both groups (and of course, separately for men and women). The
models we use will be age-period-cohort models \cite{Carstensen.2007a}
providing estimated mortality rates for ages 0--99 and dates
1.1.1996--1.1.2016.

First we transform the age and period variables to reflect the mean
age and period in each of the Lexis triangles. We also compute the
total number of deaths and amount of risk time, as we are going to
model the total mortality as well. Finally we restrict the dataset to
ages over 30 only:
<<>>=
DMepi <- transform(subset(DMepi, A > 30),
                   A = A + 0.5,
                   P = P + 0.5,
                 D.T = D.nD + D.DM,
                 Y.T = Y.nD + Y.DM)
head(DMepi)
@ %
With the correct age and period coding in the Lexis triangles, we fit
models for the mortalities and incidences. Note that we for
comparative purposes also fit a model for the \emph{total} mortality,
ignoring the
<<>>=
# Knots used in all models
(a.kn <- seq(40, 95, , 6))
(p.kn <- seq(1997, 2015, , 4))
(c.kn <- seq(1910, 1976, , 6))
# Check the number of events between knots
ae <- xtabs(cbind(D.nD, D.DM, X) ~ cut(A, c(30, a.kn, Inf)) + sex, data=DMepi)
ftable(addmargins(ae, 1), col.vars=3:2)
pe <- xtabs(cbind(D.nD, D.DM, X) ~ cut(P, c(1990, p.kn, Inf)) + sex, data=DMepi)
ftable(addmargins(pe, 1), col.vars=3:2)
ce <- xtabs(cbind(D.nD, D.DM, X) ~ cut(P-A, c(-Inf, c.kn, Inf)) + sex, data=DMepi)
ftable(addmargins(ce, 1), col.vars=3:2)
# Fit an APC-model for all transitions, separately for men and women
mW.m <- glm(cbind(D.nD, Y.nD) ~ -1 + Ns(    A, knots=a.kn, int=TRUE) +
                                     Ns(P    , knots=p.kn, ref=2005) +
                                     Ns(P - A, knots=c.kn, ref=1950),
            family = poisreg,
              data = subset(DMepi, sex=="M"))
mD.m <- update(mW.m, cbind(D.DM, Y.DM) ~ .)
mT.m <- update(mW.m, cbind(D.T , Y.T ) ~ .)
lW.m <- update(mW.m, cbind(X   , Y.nD) ~ .)
# Model for women
mW.f <- update(mW.m, data = subset(DMepi, sex == "F"))
mD.f <- update(mD.m, data = subset(DMepi, sex == "F"))
mT.f <- update(mT.m, data = subset(DMepi, sex == "F"))
lW.f <- update(lW.m, data = subset(DMepi, sex == "F"))
@ %

\section{Residual life time and years lost to DM}

We now collect the estimated years of life lost classified by method
(immunity assumption or not), sex, age and calendar time:
<<>>=
a.ref <- 30:90
p.ref <- 1996:2016
aYLL <- NArray(list(type = c("Imm", "Tot", "Sus"),
                       sex = levels(DMepi$sex),
                       age = a.ref,
                      date = p.ref))
str(aYLL)
system.time(
for(ip in p.ref)
   {
   nd <- data.frame(A = seq(30, 90, 0.2)+0.1,
                     P = ip,
                  Y.nD = 1,
                  Y.DM = 1,
                  Y.T  = 1)
   muW.m <- ci.pred(mW.m, nd)[, 1]
   muD.m <- ci.pred(mD.m, nd)[, 1]
   muT.m <- ci.pred(mT.m, nd)[, 1]
   lam.m <- ci.pred(lW.m, nd)[, 1]
   muW.f <- ci.pred(mW.f, nd)[, 1]
   muD.f <- ci.pred(mD.f, nd)[, 1]
   muT.f <- ci.pred(mT.f, nd)[, 1]
   lam.f <- ci.pred(lW.f, nd)[, 1]
   aYLL["Imm", "M", , paste(ip)] <- yll(int=0.2, muW.m, muD.m, lam=NULL,
                                      A=a.ref, age.in=30, note=FALSE)[-1]
   aYLL["Imm", "F", , paste(ip)] <- yll(int=0.2, muW.f, muD.f, lam=NULL,
                                      A=a.ref, age.in=30, note=FALSE)[-1]
   aYLL["Tot", "M", , paste(ip)] <- yll(int=0.2, muT.m, muD.m, lam=NULL,
                                      A=a.ref, age.in=30, note=FALSE)[-1]
   aYLL["Tot", "F", , paste(ip)] <- yll(int=0.2, muT.f, muD.f, lam=NULL,
                                      A=a.ref, age.in=30, note=FALSE)[-1]
   aYLL["Sus", "M", , paste(ip)] <- yll(int=0.2, muW.m, muD.m, lam=lam.m,
                                      A=a.ref, age.in=30, note=FALSE)[-1]
   aYLL["Sus", "F", , paste(ip)] <- yll(int=0.2, muW.f, muD.f, lam=lam.f,
                                      A=a.ref, age.in=30, note=FALSE)[-1]
   })
round(ftable(aYLL[, , seq(1, 61, 10), ], col.vars=c(3, 2)), 1)
@ %
We now have the relevant points for the graph showing YLL to diabetes
for men and women by age, and calendar year, both under the immunity
and susceptibility models for the calculation of YLL.
<<imm, fig=TRUE, width=8, height=5>>=
plyll <- function(wh, xtxt){
par(mfrow = c(1, 2),
      mar = c(3, 3, 1, 1),
      mgp = c(3, 1, 0) / 1.6,
      bty = "n",
      las = 1)
matplot(a.ref, aYLL[wh, "M", , ],
         type="l", lty=1, col="blue", lwd=1:2,
         ylim=c(0, 12), xlab="Age",
         ylab=paste0("Years lost to DM", xtxt),
         yaxs="i")
abline(v=50, h=1:11, col=gray(0.7))
text(90, 11.5, "Men", col="blue", adj=1)
text(40, aYLL[wh, "M", "40", "1996"], "1996", adj=c(0, 0), col="blue")
text(43, aYLL[wh, "M", "44", "2016"], "2016", adj=c(1, 1), col="blue")

matplot(a.ref, aYLL[wh, "F", , ],
         type="l", lty=1, col="red", lwd=1:2,
         ylim=c(0, 12), xlab="Age",
         ylab=paste0("Years lost to DM", xtxt),
         yaxs="i")
abline(v=50, h=1:11, col=gray(0.7))
text(90, 11.5, "Women", col="red", adj=1)
text(40, aYLL[wh, "F", "40", "1996"], "1996", adj=c(0, 0), col="red")
text(43, aYLL[wh, "F", "44", "2016"], "2016", adj=c(1, 1), col="red")
}
plyll("Imm", " - immunity assumption")
@ %
<<tot, fig=TRUE, width=8, height=5>>=
plyll("Tot", " - total mortality refernce")
@ %
<<sus, fig=TRUE, width=8, height=5>>=
plyll("Sus", " - susceptibility assumed")
@ %
\begin{figure}[h]
  \centering
  \includegraphics[width=\textwidth]{yll-imm}
  \caption{Years of life lost to DM: the difference in expected
    residual life time at different ages between persons with and
    without diabetes, assuming the persons without diabetes at a given
    age remain free from diabetes (immunity assumption --- not
    reasonable). The lines refer to date of evaluation; the top lines
    refer to 1996-1-1 the bottom ones to 2016-1-1. Blue curves are
    men, red women.}
  \label{fig:imm}
\end{figure}

\begin{figure}[h]
  \centering
  \includegraphics[width=\textwidth]{yll-sus}
  \caption{Years of life lost to DM: the difference in expected
    residual life time at different ages between persons with and
    without diabetes, allowing the persons without diabetes at a given
    age to contract diabetes and thus be subject to higher
    mortality. The lines refer to date of evaluation; the top lines
    refer to 1996-1-1 the bottom ones to 2016-1-1. Blue curves are
    men, red women.}
  \label{fig:sus}
\end{figure}

\begin{figure}[h]
  \centering
  \includegraphics[width=\textwidth]{yll-tot}
  \caption{Years of life lost to DM: the difference in expected
    residual life time at different ages between persons with and
    without diabetes. Allowance for susceptibility is approximated by
    using the total population mortality instead of non-DM
    mortality. The lines refer to date of evaluation; the top lines
    refer to 1996-1-1 the bottom ones to 2016-1-1. Blue curves are
    men, red women.}
  \label{fig:tot}
\end{figure}

From figure \ref{fig:sus} we see that for men aged 50 the years lost to
diabetes has decreased from a bit over 8 to a bit less than 6 years,
and for women from 8.5 to 5 years; so a greater improvement for women.

<<echo = FALSE>>=
ende <- Sys.time()
cat("  Start time:", format(anfang, "%F, %T"), "\n")
cat("    End time:", format(  ende, "%F, %T"), "\n")
cat("Elapsed time:", round(difftime(ende, anfang, units = "mins"), 2), "minutes\n")
@ %

\bibliographystyle{plain}

\begin{thebibliography}{1}

\bibitem{Carstensen.2007a}
B~Carstensen.
\newblock Age-{P}eriod-{C}ohort models for the {L}exis diagram.
\newblock {\em Statistics in Medicine}, 26(15):3018--3045, 2007.

\bibitem{Carstensen.2008c}
B~Carstensen, JK~Kristensen, P~Ottosen, and K~Borch-Johnsen.
\newblock The {D}anish {N}ational {D}iabetes {R}egister: {T}rends in incidence,
  prevalence and mortality.
\newblock {\em Diabetologia}, 51:2187--2196, 2008.

\end{thebibliography}

\addcontentsline{toc}{chapter}{References}

\end{document}

\SweaveOpts{results=verbatim,keep.source=TRUE,include=FALSE,eps=FALSE,prefix.string=sL}
%\VignetteIndexEntry{Simulation of multistate models with multiple timescales: simLexis}
\documentclass[a4paper,twoside,12pt]{report}

%----------------------------------------------------------------------
% General information
\newcommand{\Title}{Simulation of\\ multistate models with\\ multiple
  timescales:\\ \texttt{simLexis} in the \texttt{Epi} package}
\newcommand{\Tit}{Multistate models with multiple timescales}
\newcommand{\Version}{Version 2.3}
\newcommand{\Dates}{\today}
\newcommand{\Where}{SDC}
\newcommand{\Homepage}{\url{http://BendixCarstensen.com/Epi/simLexis.pdf}}
\newcommand{\Faculty}{\begin{tabular}{rl}
Bendix Carstensen
  & Steno Diabetes Center, Gentofte, Denmark\\
  & {\small \& Department of Biostatistics,
               University of Copenhagen} \\
  & \texttt{bxc@steno.dk}\\
  & \url{http://BendixCarstensen.com} \\[1em]
                      \end{tabular}}

%----------------------------------------------------------------------
% Packages
%\usepackage[inline]{showlabels}
\usepackage[utf8]{inputenc}
\usepackage[T1]{fontenc}
\usepackage[english]{babel}
\usepackage[font=it,labelfont=normalfont]{caption}
\usepackage[colorlinks,urlcolor=blue,linkcolor=red]{hyperref}
\usepackage[ae,hyper]{Rd}
\usepackage[dvipsnames]{xcolor}
\usepackage[super]{nth}
\usepackage{makeidx,Sweave,floatflt,amsmath,amsfonts,amsbsy,enumitem,dcolumn,needspace}
\usepackage{ifthen,calc,eso-pic,everyshi}
\usepackage{booktabs,longtable,rotating,graphicx}
\usepackage{pdfpages,verbatim,fancyhdr,datetime,%
afterpage}
\usepackage[abspath]{currfile}
% \usepackage{times}
\renewcommand{\textfraction}{0.0}
\renewcommand{\topfraction}{1.0}
\renewcommand{\bottomfraction}{1.0}
\renewcommand{\floatpagefraction}{0.9}
\DeclareMathOperator{\Pp}{P}
\providecommand{\pmat}[1]{\Pp\left\{#1\right\}}
\providecommand{\ptxt}[1]{\Pp\left\{\text{#1}\right\}}
\providecommand{\dif}{{\,\mathrm d}}
% \usepackage{mslapa}
\newenvironment{exercise}[0]{\refstepcounter{exno}
   \begin{quote}
   {\bf Exercise \theexno.}}
   {\end{quote}}
\definecolor{blaa}{RGB}{99,99,255}
\DeclareGraphicsExtensions{.pdf,.jpg}
% Make the Sweave output nicer
\DefineVerbatimEnvironment{Sinput}{Verbatim}{fontsize=\footnotesize,fontshape=sl,formatcom=\color{Blue}}
\DefineVerbatimEnvironment{Soutput}{Verbatim}{fontsize=\footnotesize,formatcom=\color{Maroon},xleftmargin=0em}
\DefineVerbatimEnvironment{Scode}{Verbatim}{fontsize=\footnotesize}
\fvset{listparameters={\setlength{\topsep}{0pt}}}
\renewenvironment{Schunk}%
{\renewcommand{\baselinestretch}{0.85} \vspace{\topsep}}%
{\renewcommand{\baselinestretch}{1.00} \vspace{\topsep}}
% \renewenvironment{knitrout}
% {\renewcommand{\baselinestretch}{0.85}}
% {\renewcommand{\baselinestretch}{1.00}} % redefined in topreport.tex

%----------------------------------------------------------------------
% The usual usefuls
% \input{/home/bendix/util/tex/useful.tex}

%----------------------------------------------------------------------
% Set up layout of pages
\oddsidemargin 1mm
\evensidemargin 1mm
\topmargin -5mm
\headheight 8mm
\headsep 5mm
\textheight 240mm
\textwidth 165mm
%\footheight 5mm
\footskip 15mm
\renewcommand{\topfraction}{0.9}
\renewcommand{\bottomfraction}{0.9}
\renewcommand{\textfraction}{0.1}
\renewcommand{\floatpagefraction}{0.9}
\renewcommand{\headrulewidth}{0.1pt}
\setcounter{secnumdepth}{4}
\setcounter{tocdepth}{2}

%----------------------------------------------------------------------
% How to insert a figure in a .rnw file
\newcommand{\rwpre}{sL}
\newcommand{\insfig}[3]{
\begin{figure}[h]
  \centering
  \includegraphics[width=#2\textwidth]{\rwpre-#1}
  \caption{#3}
  \label{fig:#1}
%  \afterpage{\clearpage}
\end{figure}}

%----------------------------------------------------------------------
% Here is the document starting with the titlepage
\begin{document}

%----------------------------------------------------------------------
% The title page
\setcounter{page}{1}
\pagenumbering{roman}
\pagestyle{plain}
\thispagestyle{empty}
% \vspace*{0.05\textheight}
\flushright
% The blank below here is necessary in order not to muck up the
% linespacing in title if it has more than 2 lines
{\Huge \bfseries \Title

}\ \\[-1.5ex]
\noindent\textcolor{blaa}{\rule[-1ex]{\textwidth}{5pt}}\\[2.5ex]
\large
\Where \\
\Dates \\
\Homepage \\
\Version \\[1em]
\normalsize
Compiled \today,\ \currenttime\\
% from: \texttt{\currfileabspath}\\[1em]
% \input{wordcount}
\normalsize
\vfill
\Faculty
% End of titlepage
% \newpage

%----------------------------------------------------------------------
% Table of contents
\tableofcontents

%----------------------------------------------------------------------
% General text layout
\raggedright
\parindent 1em
\parskip 0ex
\cleardoublepage

%----------------------------------------------------------------------
% General page style
\pagenumbering{arabic}
\setcounter{page}{1}
\pagestyle{fancy}
\renewcommand{\chaptermark}[1]{\markboth{\textsl{#1}}{}}
\renewcommand{\sectionmark}[1]{\markright{\thesection\ \textsl{#1}}{}}
\fancyhead[EL]{\bf \thepage \quad \rm \leftmark}
\fancyhead[ER]{\sl \Tit}
\fancyhead[OR]{\rm \rightmark \quad \bf \thepage}
\fancyfoot{}


%----------------------------------------------------------------------
% Here comes the substance

\chapter{Using \texttt{simLexis}}

\section{Introduction}

This vignette explains the machinery behind simulation of life
histories through multistate models implemented in
\texttt{simLexis}. In \texttt{simLexis} transition rates are allowed
to depend on multiple time scales, including timescales defined as
time since entry to a particular state (duration). This therefore also
covers the case where time \emph{at} entry into a state is an
explanatory variable for the rates, since time at entry is merely time
minus duration. Thus, the set-up here goes beyond Markov- and
semi-Markov-models, and brings simulation based estimation of
state-occupancy probabilities into the realm of realistic multistate
models.

The basic idea is to simulate a new \texttt{Lexis} object
\cite{Plummer.2011,Carstensen.2011a} as defined in the \texttt{Epi}
package for \R, based on 1) a multistate model defined by its states
and the transition rates between them and 2) an initial population of
individuals.

Thus the output will be a \texttt{Lexis} object describing the
transitions of a predefined set of persons through a multistate
model. Therefore, if persons are defined to be identical at start,
then calculation of the probability of being in a particular state at
a given time boils down to a simple enumeration of the fraction of the
persons in the particular state at the given time. Bar of course the
(binomial) simulation error, but this can be brought down by
simulation a sufficiently large number of persons.

An observed \texttt{Lexis} object with follow-up of persons through a
number of states will normally be the basis for estimation of
transition rates between states, and thus will contain all information
about covariates determining the occurrence rates, in particular the
\emph{timescales} \cite{Iacobelli.2013}. Hence, the natural input to
simulation from an estimated multistate model will typically be an
object of the same structure as the originally observed. Since
transitions and times are what is simulated, any values of
\texttt{lex.Xst} and \texttt{lex.dur} in the input object will of
course be ignored.

This first chapter of this vignette shows by an example how to use the
function \texttt{simLexis} and display the results. The subsequent
chapter discusses in more detail how the simulation machinery is
implemented and is not needed for the practical use of \texttt{simLexis}.

\section{\texttt{simLexis} in practice}

This section is largely a commented walk-trough of the example from
the help-page of \texttt{simLexis}, with a larger number of simulated
persons in order to minimize the pure simulation variation.

When we want to simulate transition times through a multistate model
where transition rates may depend on time since entry to the current
or a previous state, it is essential that we have a machinery to keep
track of the transition time on \emph{all} time scales, as well as a
mechanism that can initiate a new time scale to 0 when a transition
occurs to a state where we shall use time since entry as determinant
of exit rates from that state. This is provided by \texttt{simLexis}.

\subsection{Input for the simulation}

Input for simulation of a single trajectory through a multistate model
requires a representation of the \emph{current status} of a person;
the starting conditions. The object that we supply to the simulation
function must contain information about all covariates and all
timescales upon which transitions depend, and in particular which
one(s) of the timescales that are defined as time since entry into a
particular state. Hence, starting conditions should be represented as
a \texttt{Lexis} object (where \texttt{lex.dur} and \texttt{lex.Xst}
are ignored, since there is no follow-up yet), where the time scale
information is in the attributes \texttt{time.scale} and
\texttt{time.since} respectively.

Thus there are two main arguments to a function to simulate from a
multistate model:
\begin{enumerate}
\item A \texttt{Lexis} object representing the initial states and
  covariates of the population to be simulated. This has to have the
  same structure as the original \texttt{Lexis} object representing
  the multistate model from which transition rates in the model were
  estimated. As noted above, the values for \texttt{lex.Xst} and
  \texttt{lex.dur} are not required (since these are the quantities
  that will be simulated).
\item A transition object, representing the transition intensities
  between states, which should be a list of lists of intensity
  representations. As an intensity representation we mean a function
  that for given a \texttt{Lexis} object that can be used to produce
  estimates of the transition intensities at a set of supplied time
  points since the state represented in the \texttt{Lexis} object.

  The names of the elements of the transition object (which are lists)
  will be names of the \emph{transient} states, that is the states
  \emph{from} which a transition can occur. The names of the elements
  of each of these lists are the names of states \emph{to} which
  transitions can occur (which may be either transient or absorbing
  states).

  Hence, if the transition object is called \texttt{Tr} then
  \verb+TR$A$B+ (or \verb+Tr[["A"]][["B"]]+) will represent the
  transition intensity from state \texttt{A} to the state \texttt{B}.

  The entries in the transition object can be either \texttt{glm}
  objects, representing Poisson models for the transitions,
  \texttt{coxph} objects representing an intensity model along one
  time scale, or simply a function that takes a \texttt{Lexis}
  object as input returns an estimated intensity for each row.

\end{enumerate}

In addition to these two input items, there will be a couple of tuning
parameters.

The output of the function will simply be a \texttt{Lexis} object with
simulated transitions between states. This will be the basis for
deriving sensible statistics from the \texttt{Lexis} object --- see
next section.

\section{Setting up a \texttt{Lexis} object}

As an example we will use the \texttt{DMlate} dataset from the
\texttt{Epi} package; it is a dataset simulated to resemble a random
sample of 10,000 patients from the Danish National Diabetes Register.

We start by loading the \texttt{Epi} package:
<<start>>=
options( width=90 )
library( Epi )
print( sessionInfo(), l=F )
@ %
First we load the diabetes data and set up a simple illness-death
model:
<<Lexis>>=
data(DMlate)
dml <- Lexis( entry = list(Per=dodm, Age=dodm-dobth, DMdur=0 ),
               exit = list(Per=dox),
        exit.status = factor(!is.na(dodth),labels=c("DM","Dead")),
               data = DMlate )
@ %
This is just data for a simple survival model with states ``DM'' and
``Dead''. Now we cut the follow-up at insulin start, which for the
majority of patients (T2D) is a clinical indicator of deterioration of
disease regulation. We therefore also introduce a new timescale, and
split the non-precursor states, so that we can address the question of
ever having been on insulin:
<<cut>>=
dmi <- cutLexis( dml, cut = dml$doins,
                      pre = "DM",
                new.state = "Ins",
                new.scale = "t.Ins",
             split.states = TRUE )
summary( dmi )
str(dmi)
@ % $
We can show how many person-years we have and show the number of
transitions and transition rates (per 1000), using the
\texttt{boxes.Lexis} function to display the states and the number of
transitions between them:
<<boxes,fig=TRUE,height=5,width=7>>=
boxes( dmi, boxpos = list(x=c(20,20,80,80),
                        y=c(80,20,80,20)),
            scale.R = 1000, show.BE = TRUE )
@ %
\insfig{boxes}{0.8}{Data overview for the \textrm{\tt dmi}
  dataset. Numbers in the boxes are person-years and the number of
  persons who begin, resp. end their follow-up in each state, and
  numbers on the arrows are no. of transitions and rates (transition
  intensities) per 1000 PY.}

\section{Analysis of rates}

In the \texttt{Lexis} object (which is just a data frame) each person
is represented by one record for each transient state he occupies,
thus in this case either 1 or 2 records; those who have a recorded
time both without and with insulin have two records.

In order to be able to fit Poisson models with occurrence rates varying
by the different time-scales, we split the follow-up in 6-month intervals
for modeling:
<<split>>=
Si <- splitLexis( dmi, 0:30/2, "DMdur" )
dim( Si )
print( subset( Si, lex.id==97 )[,1:10], digits=6 )
@ %
Note that when we split the follow-up each person's follow up now
consists of many records, each with the \emph{current} values of the
timescales at the start of the interval represented by the record. In
the modelling we must necessarily assume that the rates are constant
within each 6-month interval, but the \emph{size} of these rates we
model as smooth functions of the time scales (that is the values at
the beginning of each interval).

The approach often used in epidemiology where one parameter is
attached to each interval of time (or age) is not feasible when more
than one time scale is used, because intervals are not classified the
same way on all timescales.

We shall use natural splines (restricted cubic splines) for the
analysis of rates, and hence we must allocate knots for the
splines. This is done for each of the time-scales, and separately for
the transition out of states ``DM'' and ``Ins''. For age, we place the
knots so that the number of events is the same between each pair of
knots, but only half of this beyond each of the boundary knots,
whereas for the timescales \texttt{DMdur} and \texttt{tIns} where we
have observation from a well-defined 0, we put knots at 0 and place the
remaining knots so that the number of events is the same between them
and beyond the last:
<<knots>>=
nk <- 5
( ai.kn <- with( subset(Si,lex.Xst=="Ins"),
                 quantile( Age+lex.dur  , probs=(1:nk-0.5)/nk ) ) )
( ad.kn <- with( subset(Si,lex.Xst=="Dead"),
                 quantile( Age+lex.dur  , probs=(1:nk-0.5)/nk ) ) )
( di.kn <- with( subset(Si,lex.Xst=="Ins"),
                 c(0,quantile( DMdur+lex.dur, probs=(1:(nk-1))/nk ) )) )
( dd.kn <- with( subset(Si,lex.Xst=="Dead"),
                 c(0,quantile( DMdur+lex.dur, probs=(1:(nk-1))/nk ) )) )
( ti.kn <- with( subset(Si,lex.Xst=="Dead(Ins)"),
                 c(0,quantile( t.Ins+lex.dur, probs=(1:(nk-1))/nk ) )) )
@ %
We then fit Poisson models to transition rates, using the wrapper
\texttt{Ns} from the \texttt{Epi} package to simplify the
specification of the rates:
<<Poisson>>=
library( splines )
DM.Ins <- glm( (lex.Xst=="Ins") ~ Ns( Age  , knots=ai.kn ) +
                                  Ns( DMdur, knots=di.kn ) +
                                  I(Per-2000) + sex,
               family=poisson, offset=log(lex.dur),
               data = subset(Si,lex.Cst=="DM") )
DM.Dead <- glm( (lex.Xst=="Dead") ~ Ns( Age  , knots=ad.kn ) +
                                    Ns( DMdur, knots=dd.kn ) +
                                    I(Per-2000) + sex,
               family=poisson, offset=log(lex.dur),
               data = subset(Si,lex.Cst=="DM") )
Ins.Dead <- glm( (lex.Xst=="Dead(Ins)") ~ Ns( Age  , knots=ad.kn ) +
                                          Ns( DMdur, knots=dd.kn ) +
                                          Ns( t.Ins, knots=ti.kn ) +
                                          I(Per-2000) + sex,
               family=poisson, offset=log(lex.dur),
               data = subset(Si,lex.Cst=="Ins") )
@ %
Note the similarity of the code used to fit the three models, is is
mainly redefining the response variable (``to'' state) and the subset
of the data used (``from'' state).

\section{The mortality rates}

This section discusses in some detail how to extract ad display the
mortality rates from the models fitted. But it is not necessary for
understanding how to use \texttt{simLexis} in practice.

\subsection{Proportionality of mortality rates}

Note that we have fitted separate models for the three transitions,
there is no assumption of proportionality between the mortality rates
from \texttt{DM} and \texttt{Ins}.

However, there is nothing that prevents us from testing this
assumption; we can just fit a model for the mortality rates in the
entire data frame \texttt{Si}, and compare the deviance from this with
the sum of the deviances from the separate models:
<<prop-haz>>=
with( Si, table(lex.Cst) )
All.Dead <- glm( (lex.Xst %in% c("Dead(Ins)","Dead")) ~
                 Ns( Age  , knots=ad.kn ) +
                 Ns( DMdur, knots=dd.kn ) +
                 lex.Cst +
                 I(Per-2000) + sex,
                 family=poisson, offset=log(lex.dur),
                 data = Si )
round( ci.exp( All.Dead ), 3 )
@
From the parameter values we would in a simple setting just claim that
start of insulin-treatment was associated with a slightly more than
doubling of mortality.

The model \texttt{All.dead} assumes that the age- and DM-duration
effects on mortality in the ``DM'' and ``Ins'' states are the same,
and moreover that there is no effect of insulin duration, but merely a
mortality that is larger by a multiplicative constant not depending on
insulin duration. The model \texttt{DM.Dead} has 8 parameters to
describe the dependency on age and DM duration, the model
\texttt{Ins.Dead} has 12 for the same plus the insulin duration (a
natural spline with $k$ knots gives $k-1$ parameters, and we chose
$k=5$ above).

We can compare the fit of the proportional hazards model with the fit
of the separate models for the two mortality rates, by adding up the
deviances and d.f. from these:
<<get-dev>>=
what <- c("null.deviance","df.null","deviance","df.residual")
( rD <- unlist(  DM.Dead[what] ) )
( rI <- unlist( Ins.Dead[what] ) )
( rA <- unlist( All.Dead[what] ) )
round( c( dd <- rA-(rI+rD), "pVal"=1-pchisq(dd[3],dd[4]+1) ), 3 )
@ %
Thus we see there is a substantial non-proportionality of mortality
rates from the two states; but a test provides no clue whatsoever to
the particular \emph{shape} of the non-proportionality.

To this end, we shall explore the predicted mortalities under the two
models quantitatively in more detail. Note that the reason that there
is a difference in the null deviances (and a difference of 1 in the
null d.f.) is that the null deviance of \texttt{All.Dead} refer to a
model with a single intercept, that is a model with constant and
\emph{identical} mortality rates from the states ``DM'' and ``Ins'',
whereas the null models for \texttt{DM.Dead} and \texttt{Ins.Dead}
have constant but \emph{different} mortality rates from the states
``DM'' and ``Ins''.  This is however irrelevant for the comparison of
the \emph{residual} deviances.

\subsection{How the mortality rates look}

If we want to see how the mortality rates are modelled in
\texttt{DM.Dead} and \texttt{Ins.Dead} in relation to
\texttt{All.Dead}, we make a prediction of rates for say men diagnosed
in different ages and going on insulin at different times after
this. So we consider men diagnosed in ages 40, 50, 60 and 70, and who
either never enter insulin treatment or do it 0, 2 or 5 years after
diagnosis of DM.

To this end we create a prediction data frame where we have
observation times from diagnosis and 12 years on (longer would not
make sense as this is the extent of the data).

But we start by setting up an array to hold the predicted mortality
rates, classified by diabetes duration, age at diabetes onset, time of
insulin onset, and of course type of model. What we want to do is to
plot the age-specific mortality rates for persons not on insulin, and
for persons starting insulin at different times after DM. The
mortality curves start at the age where the person gets diabetes and
continues 12 years; for persons on insulin they start at the age when
they initiate insulin.
<<pr-array>>=
pr.rates <- NArray( list( DMdur = seq(0,12,0.1),
                          DMage = 4:7*10,
                          r.Ins = c(NA,0,2,5),
                          model = c("DM/Ins","All"),
                           what = c("rate","lo","hi") ) )
str( pr.rates )
@ %
For convenience the \texttt{Epi} package contains a function that computes
predicted (log-)rates with c.i. --- it is merely a wrapper for
\texttt{predict.glm}:
<<>>=
ci.pred
@
So set up the prediction data frame and modify it in loops over
ages at onset and insulin onset. Note that we set \texttt{lex.dur} to
1000 in the prediction frame, so that we obtain rates in units of
events per 1000 PY.
<<make-pred>>=
nd <- data.frame( DMdur = as.numeric( dimnames(pr.rates)[[1]] ),
                lex.Cst = factor( 1, levels=1:4,
                                  labels=levels(Si$lex.Cst) ),
                    sex = factor( 1, levels=1:2, labels=c("M","F")),
                lex.dur = 1000 )
for( ia in dimnames(pr.rates)[[2]] )
   {
dnew <- transform( nd, Age = as.numeric(ia)+DMdur,
                       Per = 1998+DMdur )
pr.rates[,ia,1,"DM/Ins",] <- ci.pred(  DM.Dead, newdata = dnew )
pr.rates[,ia,1,"All"   ,] <- ci.pred( All.Dead, newdata = dnew )
for( ii in dimnames(pr.rates)[[3]][-1] )
   {
dnew = transform( dnew, lex.Cst = factor( 2, levels=1:4,
                                          labels=levels(Si$lex.Cst) ),
                          t.Ins = ifelse( (DMdur-as.numeric(ii)) >= 0,
                                           DMdur-as.numeric(ii), NA ) )
pr.rates[,ia, ii ,"DM/Ins",] <- ci.pred( Ins.Dead, newdata = dnew )
pr.rates[,ia, ii ,"All"   ,] <- ci.pred( All.Dead, newdata = dnew )
    }
    }
@ % $
So for each age at DM onset we make a plot of the mortality as function
of current age both for those with no insulin treatment at those that
start 1, 3 and 5 years after, thus 4 curves (with c.i.). These curves
are replicated with a different color for the simplified model.
<<mort-int,fig=TRUE,width=8>>=
par( mar=c(3,3,1,1), mgp=c(3,1,0)/1.6, las=1 )
plot( NA, xlim=c(40,82), ylim=c(5,300), bty="n",
      log="y", xlab="Age", ylab="Mortality rate per 1000 PY" )
abline( v=seq(40,80,5), h=outer(1:9,10^(0:2),"*"), col=gray(0.8) )
for( aa in 4:7*10 ) for( ii in 1:4 )
   matlines( aa+as.numeric(dimnames(pr.rates)[[1]]),
            cbind( pr.rates[,paste(aa),ii,"DM/Ins",],
                   pr.rates[,paste(aa),ii,"All"   ,] ),
            type="l", lty=1, lwd=c(3,1,1),
            col=rep(c("red","limegreen"),each=3) )
@ %
\insfig{mort-int}{0.9}{Estimated mortality rates for male diabetes
  patients with no insulin (lower sets of curves) and insulin (upper
  curves), with DM onset in age 40, 50, 60 and 70. The red curves are
  from the models with separate age effects for persons with and
  without insulin, and a separate effect of insulin duration. The
  green curves are from the model with common age-effects and only a
  time-dependent effect of insulin, assuming no effect of insulin
  duration (the classical time-dependent variable approach). Hence the
  upper green curve is common for any time of insulin inception.}

From figure \ref{fig:mort-int} we see that there is a substantial
insulin-duration effect which is not accommodated by the simple model
with only one time-dependent variable to describe the insulin
effect. Note that the simple model (green curves) for those on insulin
does not depend in insulin duration, and hence the mortality curves
for those on insulin are just parallel to the mortality curves for
those not on insulin, regardless of diabetes duration (or age) at the
time of insulin initiation. This is the proportional hazards
assumption.  Thus the effect of insulin initiation is under-estimated
for short duration of insulin and over-estimated for long duration of
insulin.

This is the major discrepancy between the two models, and
illustrates the importance of being able to accommodate different time
scales, but there is also a declining overall insulin effect by age which
is not accommodated by the proportional hazards approach.

Finally, this plot illustrates an important feature in reporting
models with multiple timescales; all timescales must be represented in
the predicted rates, only reporting the effect of one timescale,
conditional on a fixed value of other timescales is misleading since
all timescales by definition advance at the same pace. For example,
the age-effect for a fixed value of insulin duration really is a
misnomer since it does not correspond to any real person's follow-up,
but to the mortality of persons in different ages but with the same
duration of incuin use.

\section{Input to the \texttt{simLexis} function}

In order to simulate from the multistate model with the estimated
transition rates, and get the follow-up of a hypothetical cohort, we
must supply \emph{both} the transition rates and the structure of the
model \emph{as well as} the initial cohort status to
\texttt{simLexis}.

\subsection{The transition object}

We first put the models into an object representing the transitions;
note this is a list of lists, the latter having \texttt{glm} objects
as elements:
<<Tr>>=
Tr <- list( "DM" = list( "Ins"       = DM.Ins,
                         "Dead"      = DM.Dead  ),
           "Ins" = list( "Dead(Ins)" = Ins.Dead ) )
@ %
Now we have the description of the rates and of the structure of the
model. The \texttt{Tr} object defines the states and models for all
transitions between them; the object \verb|Tr$A$B| is the model
for the transition intensity from state \texttt{A} to state
\texttt{B}.

\subsection{The initial cohort}

We now define an initial \texttt{Lexis} object of persons with all
relevant covariates defined. Note that we use \texttt{subset} to get a
\texttt{Lexis} object, this conserves the \texttt{time.scale} and
\texttt{time.since} attributes which is needed for the simulation (the
usual ``\texttt{[}'' operator does not preserve these attributes when
you select columns):
<<make-ini>>=
str( Si[NULL,1:9] )
ini <- subset(Si,FALSE,select=1:9)
str( ini )
ini <- subset(Si,select=1:9)[NULL,]
str( ini )
@ %
We now have an empty \texttt{Lexis} object with attributes reflecting
the timescales in multistate model we want to simulate, so we must now
enter some data to represent the persons whose follow-up we want to
simulate through the model; we set up an initial dataset with one man
and one woman:
<<ini-fill>>=
ini[1:2,"lex.id"] <- 1:2
ini[1:2,"lex.Cst"] <- "DM"
ini[1:2,"Per"] <- 1995
ini[1:2,"Age"] <- 60
ini[1:2,"DMdur"] <- 5
ini[1:2,"sex"] <- c("M","F")
ini
@ %

\section{Simulation of the follow-up}

Now we simulate 1000 of each of these persons using the estimated
model. The \texttt{t.range} argument gives the times range where the
integrated intensities (cumulative rates) are evaluated and where
linear interpolation is used when simulating transition times. Note
that this must be given in the same units as \texttt{lex.dur} in the
\texttt{Lexis} object used for fitting the models for the transitions.
<<simL>>=
set.seed( 52381764 )
Nsim <- 1000
system.time( simL <- simLexis( Tr,
                              ini,
                          t.range = 12,
                                N = Nsim ) )
@ %
%% Temp 
%% <<>>=
%% source("../../../tmpstore/rbind.Lexis.R")
%% source("../R/simLexis.R")
%% lls()
%% @ 
%% Temp 
The result is a \texttt{Lexis} object --- a data frame representing
the simulated follow-up of \Sexpr{2*Nsim} persons (\Sexpr{Nsim}
identical men and \Sexpr{Nsim} identical women) according to the rates
we estimated from the original dataset.
<<sum-simL>>=
summary( simL, by="sex" )
@ %

\subsection{Using other models for simulation}

\subsubsection{Proportional hazards Poisson model}

We fitted a proportional mortality model \texttt{All.Dead} (which fitted
worse than the other two), this is a model for \emph{both} the
transition from ``DM'' to ``Death'' \emph{and} from ``Ins'' to
``Dead(Ins)'', assuming that they are proportional. But it can easily
be used in the simulation set-up, because the state is embedded in the
model via the term \texttt{lex.Cst}, which is updated during the simulation.

Simulation using this instead just requires that we supply a different
transition object:
<<Tr.p-simP>>=
Tr.p <- list( "DM" = list( "Ins"       = DM.Ins,
                           "Dead"      = All.Dead  ),
             "Ins" = list( "Dead(Ins)" = All.Dead ) )
system.time( simP <- simLexis( Tr.p,
                                ini,
                            t.range = 12,
                                  N = Nsim ) )
summary( simP, by="sex" )
@ %

\subsubsection{Proportional hazards Cox model}

A third possibility would be to replace the two-time scale
proportional mortality model by a one-time-scale Cox-model, using
diabetes duration as time scale, and age at diagnosis of DM as (fixed)
covariate:
<<Cox-dur>>=
library( survival )
Cox.Dead <- coxph( Surv( DMdur, DMdur+lex.dur,
                         lex.Xst %in% c("Dead(Ins)","Dead")) ~
                   Ns( Age-DMdur, knots=ad.kn ) +
                   I(lex.Cst=="Ins") +
                   I(Per-2000) + sex,
               data = Si )
round( ci.exp( Cox.Dead ), 3 )
round( ci.exp( All.Dead ), 3 )
@ %
Note that in order for this model to be usable for simulation, it is
necessary that we use the components of the \texttt{Lexis} object to
specify the survival. Each record in the data frame \texttt{Si}
represents follow up from \texttt{DMdur} to \texttt{DMdur+lex.dur}, so
the model is a Cox model with diabetes duration as underlying timescale
and age at diagnosis, \texttt{Age-DMdur}, as covariate.

Also note that we used \texttt{I(lex.Cst=="Ins")} instead of just
\texttt{lex.Cst}, because \texttt{coxph} assigns design matrix columns
to all levels of \texttt{lex.Cst}, also those not present in data,
which would give \texttt{NA}s among the parameter estimates and
\texttt{NA}s as mortality outcomes.

We see that the effect of insulin and the other covariates are pretty
much the same as in the two-time-scale model. We can simulate from this
model too; there is no restrictions on what type of model can be used
for different transitions
<<TR.c>>=
Tr.c <- list( "DM" = list( "Ins"       = Tr$DM$Ins,
                           "Dead"      = Cox.Dead  ),
             "Ins" = list( "Dead(Ins)" = Cox.Dead ) )
system.time( simC <- simLexis( Tr.c,
                                ini,
                            t.range = 12,
                                  N = Nsim ) )
summary( simC, by="sex" )
@

\section{Reporting the simulation results}

We can now tabulate the number of persons in each state at a
predefined set of times on a given time scale. Note that in order for
this to be sensible, the \texttt{from} argument would normally be
equal to the starting time for the simulated individuals.
<<nState>>=
system.time(
nSt <- nState( subset(simL,sex=="M"),
               at=seq(0,11,0.2), from=1995, time.scale="Per" ) )
nSt[1:10,]
@ %
We see that as time goes by, the 5000 men slowly move away from the
starting state (\texttt{DM}).

Based on this table (\texttt{nSt} is a table) we can now compute the
fractions in each state, or, rather more relevant, the cumulative fraction
across the states in some specified order, so that a plot of the
stacked probabilities can be made, using either the default rather
colorful layout, or a more minimalistic version (both in figure \ref{fig:pstate0}):
<<pstate0,fig=TRUE,width=9>>=
pM <- pState( nSt, perm=c(1,2,4,3) )
head( pM )
par( mfrow=c(1,2), mar=c(3,3,1,1), mgp=c(3,1,0)/1.6 )
plot( pM )
plot( pM, border="black", col="transparent", lwd=3 )
text( rep(as.numeric(rownames(pM)[nrow(pM)-1]),ncol(pM)),
      pM[nrow(pM),]-diff(c(0,pM[nrow(pM),]))/5,
      colnames( pM ), adj=1 )
box( col="white", lwd=3 )
box()
@ %
\insfig{pstate0}{1.0}{Default layout of the \textrm{\tt plot.pState}
  graph (left), and a version with the state probabilites as lines and
  annotation of states.}

A more useful set-up of the graph would include a more through
annotation and sensible choice of colors, as seen in figure \ref{fig:pstatex}:
<<pstatex,fig=TRUE,width=9>>=
clr <- c("limegreen","orange")
# expand with a lighter version of the two chosen colors
clx <- c( clr, rgb( t( col2rgb( clr[2:1] )*2 + rep(255,3) ) / 3, max=255 ) )
par( mfrow=c(1,2), las=1, mar=c(3,3,4,2), mgp=c(3,1,0)/1.6 )
# Men
plot( pM, col=clx )
lines( as.numeric(rownames(pM)), pM[,2], lwd=3 )
mtext( "60 year old male, diagnosed 1990, aged 55", side=3, line=2.5, adj=0, col=gray(0.6) )
mtext( "Survival curve", side=3, line=1.5, adj=0 )
mtext( "DM, no insulin   DM, Insulin", side=3, line=0.5, adj=0, col=clr[1] )
mtext( "DM, no insulin", side=3, line=0.5, adj=0, col=clr[2] )
axis( side=4 )
axis( side=4, at=1:19/20, labels=FALSE )
axis( side=4, at=1:99/100, labels=FALSE, tcl=-0.3 )
# Women
pF <- pState( nState( subset(simL,sex=="F"),
                      at=seq(0,11,0.2),
                      from=1995,
                      time.scale="Per" ),
              perm=c(1,2,4,3) )
plot( pF, col=clx )
lines( as.numeric(rownames(pF)), pF[,2], lwd=3 )
mtext( "60 year old female, diagnosed 1990, aged 55", side=3, line=2.5, adj=0, col=gray(0.6) )
mtext( "Survival curve", side=3, line=1.5, adj=0 )
mtext( "DM, no insulin   DM, Insulin", side=3, line=0.5, adj=0, col=clr[1] )
mtext( "DM, no insulin", side=3, line=0.5, adj=0, col=clr[2] )
axis( side=4 )
axis( side=4, at=1:19/20, labels=FALSE )
axis( side=4, at=1:99/100, labels=FALSE, tcl=-0.3 )
@ %
\insfig{pstatex}{1.0}{\textrm{\tt plot.pState} graphs where persons
  ever on insulin are given in orange and persons never on insulin in
  green, and the overall survival (dead over the line) as a black line.}

If we instead wanted to show the results on the age-scale, we would
use age as timescale when constructing the probabilities; otherwise the
code is pretty much the same as before (Figure \ref{fig:pstatey}):
<<pstatey,fig=TRUE,width=9>>=
par( mfrow=c(1,2), las=1, mar=c(3,3,4,2), mgp=c(3,1,0)/1.6 )
# Men
pM <- pState( nState( subset(simL,sex=="M"),
                      at=seq(0,11,0.2),
                      from=60,
                      time.scale="Age" ),
              perm=c(1,2,4,3) )
plot( pM, col=clx, xlab="Age" )
lines( as.numeric(rownames(pM)), pM[,2], lwd=3 )
mtext( "60 year old male, diagnosed 1990, aged 55", side=3, line=2.5, adj=0, col=gray(0.6) )
mtext( "Survival curve", side=3, line=1.5, adj=0 )
mtext( "DM, no insulin   DM, Insulin", side=3, line=0.5, adj=0, col=clr[1] )
mtext( "DM, no insulin", side=3, line=0.5, adj=0, col=clr[2] )
axis( side=4 )
axis( side=4, at=1:19/20, labels=FALSE )
axis( side=4, at=1:19/20, labels=FALSE, tcl=-0.4 )
axis( side=4, at=1:99/100, labels=FALSE, tcl=-0.3 )
# Women
pF <- pState( nState( subset(simL,sex=="F"),
                      at=seq(0,11,0.2),
                      from=60,
                      time.scale="Age" ),
              perm=c(1,2,4,3) )
plot( pF, col=clx, xlab="Age" )
lines( as.numeric(rownames(pF)), pF[,2], lwd=3 )
mtext( "60 year old female, diagnosed 1990, aged 55", side=3, line=2.5, adj=0, col=gray(0.6) )
mtext( "Survival curve", side=3, line=1.5, adj=0 )
mtext( "DM, no insulin   DM, Insulin", side=3, line=0.5, adj=0, col=clr[1] )
mtext( "DM, no insulin", side=3, line=0.5, adj=0, col=clr[2] )
axis( side=4 )
axis( side=4, at=1:9/10, labels=FALSE )
axis( side=4, at=1:19/20, labels=FALSE, tcl=-0.4 )
axis( side=4, at=1:99/100, labels=FALSE, tcl=-0.3 )
@ %
Note the several statements with \texttt{axis(side=4,...}; they are
nesessary to get the fine tick-marks in the right hand side of the
plots that you will need in order to read off the probabilities at
2006 (or 71 years).

\insfig{pstatey}{1.0}{\textrm{\tt plot.pState} graphs where persons
  ever on insulin are given in orange and persons never on insulin in
  green, and the overall survival (dead over the line) as a black line.}

\subsection{Comparing predictions from different models}

We have actually fitted different models for the transitions, and we
have simulated Lexis objects from all three approaches, so we can plot
these predictions on top of each other:
<<comp-0,fig=TRUE,width=9>>=
PrM  <- pState( nState( subset(simP,sex=="M"),
                        at=seq(0,11,0.2),
                        from=60,
                        time.scale="Age" ),
                perm=c(1,2,4,3) )
PrF  <- pState( nState( subset(simP,sex=="F"),
                        at=seq(0,11,0.2),
                        from=60,
                        time.scale="Age" ),
                perm=c(1,2,4,3) )
CoxM <- pState( nState( subset(simC,sex=="M"),
                        at=seq(0,11,0.2),
                        from=60,
                        time.scale="Age" ),
                perm=c(1,2,4,3) )
CoxF <- pState( nState( subset(simC,sex=="F"),
                        at=seq(0,11,0.2),
                        from=60,
                        time.scale="Age" ),
                perm=c(1,2,4,3) )

par( mfrow=c(1,2), mar=c(3,3,1,1), mgp=c(3,1,0)/1.6 )
 plot(   pM, border="black", col="transparent", lwd=3 )
lines(  PrM, border="blue" , col="transparent", lwd=3 )
lines( CoxM, border="red"  , col="transparent", lwd=3 )
text( 60.5, 0.05, "M" )
box( lwd=3 )

 plot(   pF, border="black", col="transparent", lwd=3 )
lines(  PrF, border="blue" , col="transparent", lwd=3 )
lines( CoxF, border="red"  , col="transparent", lwd=3 )
text( 60.5, 0.05, "F" )
box( lwd=3 )
@ %
\insfig{comp-0}{1.0}{Comparison of the simulated state occupancy
  probabilities using separate Poisson models for the mortality rates
  with and without insulin (black) and using proportional hazards
  Poisson models (blue) and Cox-models with diabetes duration as
  timescale and age at diabetes diagnosis as covariate (red).}

From figure \ref{fig:comp-0} it is clear that the two proportional
hazards models (blue and red curves) produce pretty much the same
estimates of the state occupancy probabilites over time, but also that
they relative to the model with separately estimated transition
intensities overestimates the probability of being alive without
insulin and underestimates the probabilities of being dead without
insulin. However both the overall survival, and the fraction of
persons on insulin are quite well in agreement with the more elaborate
model. Thus the proportional hazards models overestimate the relative
mortality of the insulin treated diabetes patients relative to the
non-insulin treated.

Interestingly, we also see a bump in th estimated probabilities from
the Cox-model based model, but this is entirely an artifact that comes
from the estimation method for the baseline hazard of the Cox-model
that lets the (cumulative) hazard have large jumps at event times
where the risk set is small. So also here it shows up that an
assumption of continuous underlying hazards leads to more credible
estimates.

\chapter{Simulation of transitions in multistate models}

\section{Theory}

Suppose that the rate functions for the transitions out of the current
state to, say, 3 different states are $\lambda_1$, $\lambda_2$ and
$\lambda_3$, and the corresponding cumulative rates are $\Lambda_1$,
$\Lambda_2$ and $\Lambda_3$, and we want to simulate an exit time and
an exit state (that is either 1, 2 or 3). This can be done in two
slightly different ways:
\begin{enumerate}
\item First time, then state:
  \begin{enumerate}
  \item Compute the survival function, $S(t) =
    \exp\bigl(-\Lambda_1(t)-\Lambda_2(t)-\Lambda_3(t)\bigr)$
  \item Simulate a random U(0,1) variate, $u$, say.
  \item The simulated exit time is then the solution $t_u$ to
    the equation $S(t_u) = u \quad \Leftrightarrow \quad \sum_j\Lambda_j(t_u) = -\log(u)$.
  \item A simulated transition at $t_u$ is then found by simulating a
    random draw from the multinomial distribution with probabilities
    $p_i=\lambda_i(t_u) / \sum_j\lambda_j(t_u)$.
  \end{enumerate}
\item Separate cumulative incidences:
  \begin{enumerate}
  \item Simulate 3 independent U(0,1) random variates $u_1$, $u_2$ and $u_3$.
  \item Solve the equations $\Lambda_i(t_i)=-\log(u_i), i=1,2,3$ and get $(t_1,t_2,t_3)$.
  \item The simulated survival time is then $\min(t_1,t_2,t_3)$, and
    the simulated transition is to the state corresponding to this,
    that is $k \in \{1,2,3\}$, where $t_k=\min(t_1,t_2,t_3)$
  \end{enumerate}
\end{enumerate}
The intuitive argument is that with three possible transition there are
3 independent processes running, but only the first transition is observed.
The latter approach is used in the implementation in \texttt{simLexis}.

The formal argument for the equality of the two approaches goes as
follows:
\begin{enumerate}
\item Equality of the transition times:
  \begin{enumerate}
  \item In the first approach we simulate from a distribution with
    cumulative rate $\Lambda_1(t)+\Lambda_2(t)+\Lambda_3(t)$, hence
    from a distribution with survival function:
    \begin{align*}
    S(t) & = \exp\bigl(-(\Lambda_1(t)+\Lambda_2(t)+\Lambda_3(t))\bigr) \\
         & = \exp\bigl(-\Lambda_1(t)\bigr)\times
             \exp\bigl(-\Lambda_2(t)\bigr)\times
             \exp\bigl(-\Lambda_3(t)\bigr)
    \end{align*}
  \item In the second approach we choose the smallest of three
    independent survival times, with survival functions
    $\exp(-\Lambda_i), i=1,2,3$. Now, the survival function for the
    minimum of three independent survival times is:
    \begin{align*}
      S_\text{min}(t) & = \pmat{\min(t_1,t_2,t_3)>t} \\
                     & = \pmat{t_1>t} \times
                         \pmat{t_2>t} \times
                         \pmat{t_3>t} \\
                     & = \exp\bigl(-\Lambda_1(t)\bigr)\times
                         \exp\bigl(-\Lambda_2(t)\bigr)\times
                         \exp\bigl(-\Lambda_3(t)\bigr)
    \end{align*}
    which is the same survival function as derived above.
  \end{enumerate}
\item Type of transition:
  \begin{enumerate}
  \item In the first instance the probability of a transition to state
    $i$, conditional on the transition time being $t$, is as known
    from standard probability theory:
    $\lambda_i(t)/\bigl(\lambda_1(t)+\lambda_2(t)+\lambda_3(t)\bigr)$.
  \item In the second approach we choose the transition corresponding
    to the the smallest of the transition times. So when we condition
    on the event that a transition takes place at time $t$, we have to
    show that the conditional probability that the smallest of the
    three simulated transition times was actually the $i$th, is as above.

    But conditional on \emph{survival} till $t$, the probabilities
    that events of type $1,2,3$ takes place in the interval $(t,t+\dif
    t)$ are $\lambda_1(t)\dif t$, $\lambda_2(t)\dif t$ and
    $\lambda_1(t)\dif t$, respectively (assuming that the probability
    of more than one event in the interval of length $\dif t$ is
    0). Hence the conditional probabilities \emph{given a transition
      time} in $(t,t+\dif t)$ is:
    \[
    \frac{\lambda_i(t)\dif t}{\lambda_1(t)\dif t+\lambda_2(t)\dif t+\lambda_3(t)\dif t}=
    \frac{\lambda_i(t)}{\lambda_1(t)+\lambda_2(t)+\lambda_3(t)}
    \]
    --- exactly as above.
  \end{enumerate}
\end{enumerate}

\section{Components of \texttt{simLexis}}

This section explains the actually existing code for
\texttt{simLexis}, as it is in the current version of \texttt{Epi}.
<<echo=FALSE,results=hide>>=
options( keep.source=TRUE )
@ %
The function \texttt{simLexis} takes a \texttt{Lexis} object as
input. This defines the initial state(s) and times of the start for a
number of persons. Since the purpose is to simulate a history through
the estimated multistate model, the input values of the outcome
variables \texttt{lex.Xst} and \texttt{lex.dur} are ignored --- the
aim is to simulate values for them.

Note however that the attribute \texttt{time.since} must be present in the
object. This is used for initializing timescales defined as time since
entry into a particular state, it is a character vector of the same
length as the \texttt{time.scales} attribute, with value equal to a
state name if the corresponding time scale is defined as time since
entry into that state. In this example the 4th timescale is time since
entry into the ``Ins'' state, and hence:
<<>>=
cbind(
attr( ini, "time.scale" ),
attr( ini, "time.since" ) )
@ %
\texttt{Lexis} objects will have this attribute set for time scales created
using \texttt{cutLexis}.

The other necessary argument is a transition object \texttt{Tr}, which
is a list of lists. The elements of the lists should be \texttt{glm}
objects derived by fitting Poisson models to a \texttt{Lexis} object
representing follow-up data in a multistate model.  It is assumed (but
not checked) that timescales enter in the model via the timescales of
the \texttt{Lexis} object. Formally, there are no assumptions about
how \texttt{lex.dur} enters in the model.

Optional arguments are \texttt{t.range}, \texttt{n.int} or
\texttt{time.pts}, specifying the times after entry at which the
cumulative rates will be computed (the maximum of which will be taken
as the censoring time), and \texttt{N} a scalar or numerical vector of
the number of persons with a given initial state each record of the
\texttt{init} object should represent.

The central part of the functions uses a \texttt{do.call} /
\texttt{lapply} / \texttt{split} construction to do simulations for
different initial states. This is the construction in the middle that
calls \texttt{simX}. \texttt{simLexis} also calls \texttt{get.next}
which is further detailed below.
<<>>=
simLexis
@ %

\subsection{\texttt{simX}}

\texttt{simX} is called by \texttt{simLexis} and simulates
transition-times and -types for a set of patients assumed to be in the
same state. It is called from \texttt{simLexis} with a data frame as
argument, uses the state in \texttt{lex.Cst} to select the relevant
component of \texttt{Tr} and compute predicted cumulative intensities
for all states reachable from this state.

Note that it is here the switch between \texttt{glm}, \texttt{coxph}
and objects of class \texttt{function} is made.

The dataset on which this prediction is done has
\texttt{length(time.pts)} rows per person.
<<>>=
Epi:::simX
@ %
As we see, \texttt{simX} calls \texttt{sim1} which simulates the
transition for one person.

\subsection{\texttt{sim1}}

The predicted cumulative intensities are fed, person by person, to
\texttt{sim1} --- again via a \texttt{do.call} / \texttt{lapply} /
\texttt{split} construction --- and the resulting time and state is
appended to the \texttt{nd} data frame. This way we have simulated
\emph{one} transition (time and state) for each person:
<<>>=
Epi:::sim1
@ %
The \texttt{sim1} function uses \texttt{lint} to do linear interpolation.

\subsection{\texttt{lint}}

We do not use \texttt{approx} to do the linear interpolation, because
this function does not do the right thing if the cumulative incidences
(\texttt{ci}) are constant across a number of times. Therefore we have
a custom made linear interpolator that does the interpolation
exploiting the fact the the \texttt{ci} is non-decreasing and
\texttt{tt} is strictly monotonously increasing:
<<>>=
Epi:::lint
@

\subsection{\texttt{get.next}}

We must repeat the simulation operation on those that have a simulated
entry to a transient state, and also make sure that any time scales
defined as time since entry to one of these states be initialized to 0
before a call to \texttt{simX} is made for these persons. This
accomplished by the function \texttt{get.next}:
<<>>=
Epi:::get.next
@

\subsection{\texttt{chop.lex}}

The operation so far has censored individuals \texttt{max(time.pts)}
after \emph{each} new entry to a transient state. In order to groom
the output data we use \texttt{chop.lex} to censor all persons at the
same designated time after \emph{initial} entry.
<<>>=
Epi:::chop.lex
@

\section{Probabilities from simulated \texttt{Lexis} objects}

Once we have simulated a Lexis object we will of course want to use it
for estimating probabilities, so basically we will want to enumerate
the number of persons in each state at a pre-specified set of time
points.

\subsection{\texttt{nState}}

Since we are dealing with multistate model with potentially multiple
time scales, it is necessary to define the timescale
(\texttt{time.scale}), the starting point on this timescale
(\texttt{from}) and the points after this where we compute the number
of occupants in each state, (\texttt{at}).
<<>>=
nState
@ %

\subsection{\texttt{pState}, \texttt{plot.pState} and \texttt{lines.pState}}

In order to plot probabilities of state-occupancy it is useful to
compute cumulative probabilities across states in any given order;
this is done by the function \texttt{pState}, which returns a matrix
of class \texttt{pState}:
<<>>=
pState
@ %
There is also a \texttt{plot} and \texttt{lines} method for the
resulting \texttt{pState} objects:
<<>>=
plot.pState
lines.pState
@ %

\bibliographystyle{plain}
  \begin{thebibliography}{1}

\bibitem{Carstensen.2011a}
B.~Carstensen and M.~Plummer.
\newblock Using {L}exis objects for multi-state models in {R}.
\newblock {\em Journal of Statistical Software}, 38(6):1--18, 1 2011.

\bibitem{Iacobelli.2013}
S.~Iacobelli and B.~Carstensen.
\newblock {{M}ultiple time scales in multi-state models}.
\newblock {\em Stat Med}, 32(30):5315--5327, Dec 2013.

\bibitem{Plummer.2011}
M.~Plummer and B.~Carstensen.
\newblock Lexis: An {R} class for epidemiological studies with long-term
  follow-up.
\newblock {\em Journal of Statistical Software}, 38(5):1--12, 1 2011.

\end{thebibliography}

\addcontentsline{toc}{chapter}{References}

\end{document}

\SweaveOpts{results=verbatim,keep.source=TRUE,include=FALSE}
%\VignetteIndexEntry{Follow-up data with the Epi package}
\documentclass[a4paper,twoside,12pt]{article}

\usepackage[english]{babel}
\usepackage{booktabs,rotating,graphicx,amsmath,verbatim,fancyhdr,Sweave}
\usepackage[colorlinks,linkcolor=red,urlcolor=blue]{hyperref}
\newcommand{\R}{\textsf{\bf R}}
\renewcommand{\topfraction}{0.95}
\renewcommand{\bottomfraction}{0.95}
\renewcommand{\textfraction}{0.1}
\renewcommand{\floatpagefraction}{0.9}
\DeclareGraphicsExtensions{.pdf,.jpg}
\setcounter{secnumdepth}{1}
\setcounter{tocdepth}{1}

\oddsidemargin 1mm
\evensidemargin 1mm
\textwidth 160mm
\textheight 230mm
\topmargin -5mm
\headheight 8mm
\headsep 5mm
\footskip 15mm

\begin{document}

\raggedleft
\pagestyle{empty}
\vspace*{0.1\textheight}
\Huge
{\bf Follow-up data with the\\ \texttt{Epi} package}
\noindent\rule[-1ex]{\textwidth}{5pt}\\[2.5ex]
\Large
Summer 2014
\vfill
\normalsize
\begin{tabular}{rl}
 Michael Hills & Retired \\
               & Highgate, London \\[1em]
Martyn Plummer & International Agency for Research on Cancer, Lyon\\
               & \texttt{plummer@iarc.fr} \\[1em]
Bendix Carstensen & Steno Diabetes Center, Gentofte, Denmark\\
                  & \small \& Department of Biostatistics,
                    University of Copenhagen\\
                  & \normalsize \texttt{bxc@steno.dk} \\
                  & \url{www.pubhealth.ku.dk/~bxc}
\end{tabular}
\normalsize
\newpage
\raggedright
\parindent 3ex
\parskip 0ex
\tableofcontents
\cleardoublepage
\setcounter{page}{1}
\pagestyle{fancy}
\renewcommand{\sectionmark}[1]{\markboth{\thesection #1}{\thesection \ #1}}
\fancyhead[OL]{\sl Follow-up data with the \texttt{Epi} package.}
\fancyhead[ER]{\sl \rightmark}
\fancyhead[EL,OR]{\bf \thepage}
\fancyfoot{}
\renewcommand{\headrulewidth}{0.1pt}

<<>>=
library(Epi)
print( sessionInfo(), l=F )
@

\section{Follow-up data in the \texttt{Epi} package}

In the \texttt{Epi}-package, follow-up data is represented by adding
some extra variables to a dataframe. Such a dataframe is called a
\texttt{Lexis} object. The tools for handling follow-up data then use
the structure of this for special plots, tabulations etc.

Follow-up data basically consists of a time of entry, a time of exit
and an indication of the status at exit (normally either ``alive'' or
``dead''). Implicitly is also assumed a status \emph{during} the
follow-up (usually ``alive'').

\begin{figure}[htbp]
  \centering
\setlength{\unitlength}{1pt}
\begin{picture}(210,70)(0,75)
%\scriptsize
\thicklines
 \put(  0,80){\makebox(0,0)[r]{Age-scale}}
 \put( 50,80){\line(1,0){150}}
 \put( 50,80){\line(0,1){5}}
 \put(100,80){\line(0,1){5}}
 \put(150,80){\line(0,1){5}}
 \put(200,80){\line(0,1){5}}
 \put( 50,77){\makebox(0,0)[t]{35}}
 \put(100,77){\makebox(0,0)[t]{40}}
 \put(150,77){\makebox(0,0)[t]{45}}
 \put(200,77){\makebox(0,0)[t]{50}}

 \put(  0,115){\makebox(0,0)[r]{Follow-up}}

 \put( 80,105){\makebox(0,0)[r]{\small Two}}
 \put( 90,105){\line(1,0){87}}
 \put( 90,100){\line(0,1){10}}
 \put(100,100){\line(0,1){10}}
 \put(150,100){\line(0,1){10}}
 \put(180,105){\circle{6}}
 \put( 95,110){\makebox(0,0)[b]{1}}
 \put(125,110){\makebox(0,0)[b]{5}}
 \put(165,110){\makebox(0,0)[b]{3}}

 \put( 50,130){\makebox(0,0)[r]{\small One}}
 \put( 60,130){\line(1,0){70}}
 \put( 60,125){\line(0,1){10}}
 \put(100,125){\line(0,1){10}}
 \put(130,130){\circle*{6}}
 \put( 80,135){\makebox(0,0)[b]{4}}
 \put(115,135){\makebox(0,0)[b]{3}}
\end{picture}
  \caption{\it Follow-up of two persons}
  \label{fig:fu2}
\end{figure}

\section{Timescales}

A timescale is a variable that varies deterministically \emph{within}
each person during follow-up, \textit{e.g.}:
\begin{itemize}
  \item Age
  \item Calendar time
  \item Time since treatment
  \item Time since relapse
\end{itemize}
All timescales advance at the same pace, so the time followed is the
same on all timescales. Therefore, it suffices to use only the entry
point on each of the time scale, for example:
\begin{itemize}
  \item Age at entry.
  \item Date of entry.
  \item Time since treatment (\emph{at} treatment this is 0).
  \item Time since relapse (\emph{at} relapse this is 0)..
\end{itemize}
In the \texttt{Epi} package, follow-up in a cohort is represented in a
\texttt{Lexis} object.  A \texttt{Lexis} object is a dataframe with a
bit of extra structure representing the follow-up. For the
\texttt{nickel} data we would construct a \texttt{Lexis} object by:
<<>>=
data( nickel )
nicL <- Lexis( entry = list( per=agein+dob,
                             age=agein,
                             tfh=agein-age1st ),
                exit = list( age=ageout ),
         exit.status = ( icd %in% c(162,163) )*1,
                data = nickel )
@
The \texttt{entry} argument is a \emph{named} list with the entry
points on each of the timescales we want to use. It defines the names
of the timescales and the entry points. The \texttt{exit} argument
gives the exit time on \emph{one} of the timescales, so the name of
the element in this list must match one of the neames of the
\texttt{entry} list. This is sufficient, because the follow-up time on
all time scales is the same, in this case \texttt{ageout - agein}. Now
take a look at the result:
<<>>=
str( nickel )
str( nicL )
head( nicL )
@
The \texttt{Lexis} object \texttt{nicL} has a variable for each
timescale which is the entry point on this timescale. The follow-up
time is in the variable \texttt{lex.dur} (\textbf{dur}ation).

There is a \texttt{summary} function for \texttt{Lexis} objects that
list the numer of transitions and records as well as the total
follow-up time:
<<>>=
summary( nicL )
@
We defined the exit status to be death from lung cancer (ICD7
162,163), i.e. this variable is 1 if follow-up ended with a death from
this cause. If follow-up ended alive or by death from another cause,
the exit status is coded 0, i.e. as a censoring.

Note that the exit status is in the variable \texttt{lex.Xst}
(e\textbf{X}it \textbf{st}atus. The variable \texttt{lex.Cst} is the
state where the follow-up takes place (\textbf{C}urrent
\textbf{st}atus), in this case 0 (alive).

It is possible to get a visualization of the follow-up along the
timescales chosen by using the \texttt{plot} method for \texttt{Lexis}
objects. \texttt{nicL} is an object of \emph{class} \texttt{Lexis}, so
using the function \texttt{plot()} on it means that \R\ will look for
the function \texttt{plot.Lexis} and use this function.
<<nicL1,fig=TRUE>>=
plot( nicL )
@
The function allows a lot of control over the output, and a
\texttt{points.Lexis} function allows plotting of the endpoints of
follow-up:
<<nicL2,fig=TRUE>>=
par( mar=c(3,3,1,1), mgp=c(3,1,0)/1.6 )
plot( nicL, 1:2, lwd=1, col=c("blue","red")[(nicL$exp>0)+1],
      grid=TRUE, lty.grid=1, col.grid=gray(0.7),
      xlim=1900+c(0,90), xaxs="i",
      ylim=  10+c(0,90), yaxs="i", las=1 )
points( nicL, 1:2, pch=c(NA,3)[nicL$lex.Xst+1],
        col="lightgray", lwd=3, cex=1.5 )
points( nicL, 1:2, pch=c(NA,3)[nicL$lex.Xst+1],
        col=c("blue","red")[(nicL$exp>0)+1], lwd=1, cex=1.5 )
@
The results of these two plotting commands are in figure \ref{fig:Lexis-diagram}.
\begin{figure}[tb]
\centering
\label{fig:Lexis-diagram}
\includegraphics[width=0.39\textwidth]{Follow-up-nicL1}
\includegraphics[width=0.59\textwidth]{Follow-up-nicL2}
\caption{\it Lexis diagram of the \texttt{nickel} dataset, left panel
  the default version, the right one with bells
  and whistles. The red lines are for persons with exposure$>0$, so it
  is pretty evident that the oldest ones are the exposed part of the
  cohort.}
\end{figure}

\section{Splitting the follow-up time along a timescale}

The follow-up time in a cohort can be subdivided by for example
current age. This is achieved by the \texttt{splitLexis} (note that it
is \emph{not} called \texttt{split.Lexis}). This requires that the
timescale and the breakpoints on this timescale are supplied. Try:
<<>>=
nicS1 <- splitLexis( nicL, "age", breaks=seq(0,100,10) )
summary( nicL )
summary( nicS1 )
@
So we see that the number of events and the amount of follow-up is the
same in the two datasets; only the number of records differ.

To see how records are split for each individual, it is useful to list
the results for a few individuals:
<<>>=
round( subset( nicS1, id %in% 8:10 ), 2 )
@
The resulting object, \texttt{nicS1}, is again a \texttt{Lexis}
object, and so follow-up may be split further along another
timescale. Try this and list the results for individuals 8, 9 and 10 again:
<<>>=
nicS2 <- splitLexis( nicS1, "tfh", breaks=c(0,1,5,10,20,30,100) )
round( subset( nicS2, id %in% 8:10 ), 2 )
@
If we want to model the effect of these timescales we will for each
interval use either the value of the left endpoint in each interval or
the middle. There is a function \texttt{timeBand} which returns these.
Try:
<<>>=
timeBand( nicS2, "age", "middle" )[1:20]
# For nice printing and column labelling use the data.frame() function:
data.frame( nicS2[,c("id","lex.id","per","age","tfh","lex.dur")],
            mid.age=timeBand( nicS2, "age", "middle" ),
            mid.tfh=timeBand( nicS2, "tfh", "middle" ) )[1:20,]
@
Note that these are the midpoints of the intervals defined by
\texttt{breaks=}, \emph{not} the midpoints of the actual follow-up
intervals. This is because the variable to be used in modelling must
be independent of the consoring and mortality pattern --- it should
only depend on the chosen grouping of the timescale.

\section{Splitting time at a specific date}

If we have a recording of the date of a specific event as for example
recovery or relapse, we may classify follow-up time as being before of
after this intermediate event. This is achieved with the function
\texttt{cutLexis}, which takes three arguments: the time point, the
timescale, and the value of the (new) state following the date.

Now we define the age for the nickel vorkers where the cumulative
exposure exceeds 50 exposure years:
<<>>=
subset( nicL, id %in% 8:10 )
agehi <- nicL$age1st + 50 / nicL$exposure
nicC <- cutLexis( data=nicL, cut=agehi, timescale="age",
                  new.state=2, precursor.states=0 )
subset( nicC, id %in% 8:10 )
@
(The \texttt{precursor.states=} argument is explained below).
Note that individual 6 has had his follow-up split at age 25 where 50
exposure-years were attained. This could also have been achieved in
the split dataset \texttt{nicS2} instead of \texttt{nicL}, try:
<<>>=
subset( nicS2, id %in% 8:10 )
agehi <- nicS2$age1st + 50 / nicS2$exposure
nicS2C <- cutLexis( data=nicS2, cut=agehi, timescale="age",
                    new.state=2, precursor.states=0 )
subset( nicS2C, id %in% 8:10 )
@
Note that follow-up subsequent to the event is classified as being
in state 2, but that the final transition to state 1 (death from lung
cancer) is preserved. This is the point of the \texttt{precursor.states=}
argument. It names the states (in this case 0, ``Alive'') that will be
over-witten by \texttt{new.state} (in this case state 2, ``High
exposure''). Clearly, state 1 (``Dead'') should not be updated even if
it is after the time where the persons moves to state 2. In other
words, only state 0 is a precursor to state 2, state 1 is always
subsequent to state 2.

Note if the intermediate event is to be used as a time-dependent
variable in a Cox-model, then \texttt{lex.Cst} should be used as the
time-dependent variable, and \texttt{lex.Xst==1} as the event.

\section{Competing risks --- multiple types of events}

If we want to consider death from lung cancer and death from other
causes as separate events we can code these as for example 1 and 2.
<<>>=
data( nickel )
nicL <- Lexis( entry = list( per=agein+dob,
                             age=agein,
                             tfh=agein-age1st ),
                exit = list( age=ageout ),
         exit.status = ( icd > 0 ) + ( icd %in% c(162,163) ),
                data = nickel )
summary( nicL )
subset( nicL, id %in% 8:10 )
@
If we want to label the states, we can enter the names of these in the
\texttt{states} parameter, try for example:
<<>>=
nicL <- Lexis( entry = list( per=agein+dob,
                             age=agein,
                             tfh=agein-age1st ),
                exit = list( age=ageout ),
         exit.status = ( icd > 0 ) + ( icd %in% c(162,163) ),
                data = nickel,
              states = c("Alive","D.oth","D.lung") )
summary( nicL )
@

Note that the \texttt{Lexis} function automatically assumes that all
persons enter in the first level (given in the \texttt{states=}
argument)

When we cut at a date as in this case, the date where cumulative
exposure exceeds 50 exposure-years, we get the follow-up \emph{after}
the date classified as being in the new state if the exit
(\texttt{lex.Xst}) was to a state we defined as one of the
\texttt{precursor.states}:
<<>>=
nicL$agehi <- nicL$age1st + 50 / nicL$exposure
nicC <- cutLexis( data = nicL,
                   cut = nicL$agehi,
             timescale = "age",
             new.state = "HiExp",
      precursor.states = "Alive" )
subset( nicC, id %in% 8:10 )
summary( nicC, scale=1000 )
@
Note that the persons-years is the same, but that the number of events
has changed. This is because events are now defined as any transition
from alive, including the transitions to \texttt{HiExp}.

Also note that (so far) it is necessary to specify the variable with
the cutpoints in full, using only \texttt{cut=agehi} would give an error.

\subsection{Subdivision of existing states}
It may be of interest to subdivide the states following the
intermediate event according to wheter the event has occurred or
not. That is done by the argument \texttt{split.states=TRUE}.

Moreover, it will also often be of interest to introduce a new
timescale indicating the time since intermediate event. This can be
done by the argument \texttt{new.scale=TRUE}, alternatively
\texttt{new.scale="tfevent"}, as illustrated here:
<<>>=
nicC <- cutLexis( data = nicL,
                   cut = nicL$agehi,
             timescale = "age",
             new.state = "Hi",
           split.states=TRUE, new.scale=TRUE,
      precursor.states = "Alive" )
subset( nicC, id %in% 8:10 )
summary( nicC, scale=1000 )
@

\section{Multiple events of the same type (recurrent events)}
Sometimes more events of the same type are recorded for each person and
one would then like to count these and put follow-up time in states accordingly.
Essentially, each set of cutpoints represents progressions from one
state to the next. Therefore the states should be numbered, and the
numbering of states subsequently occupied be increased accordingly.

This is a behaviour different from the one outlined above, and it is
achieved by the argument \texttt{count=TRUE} to
\texttt{cutLexis}. When \texttt{count} is set to \texttt{TRUE}, the
value of the arguments \texttt{new.state} and
\texttt{precursor.states} are ignored.  Actually, when using the
argument \texttt{count=TRUE}, the function \texttt{countLexis} is
called, so an alternative is to use this directly.

\end{document}

%\VignetteIndexEntry{Time dependent covariates in Lexis objects: addCov & addDrug}
\SweaveOpts{results=verbatim,keep.source=TRUE,include=FALSE,eps=FALSE}
\documentclass[a4paper,twoside,12pt]{report}

\newcommand{\Title}{Time dependent covariates in\\ \texttt{Lexis} objects}
\newcommand{\Tit}{addLex}
\newcommand{\Version}{Version 6}
\newcommand{\Dates}{March 2024}
\newcommand{\Where}{SDCC}
\newcommand{\Homepage}{\url{http://bendixcarstensen.com/Epi}}
\newcommand{\Faculty}{\begin{tabular}{rl}
Bendix Carstensen
  & Steno Diabetes Center Copenhagen, Herlev, Denmark\\
  & {\small \& Department of Biostatistics,
               University of Copenhagen} \\
  & \texttt{b@bxc.dk}\\
  & \url{http://BendixCarstensen.com} \\[1em]
                      \end{tabular}}
\input{topreport}
<<echo=FALSE>>=
options(width = 90,
        SweaveHooks = list(fig = function()
        par(mar = c(3,3,1,1),
            mgp = c(3,1,0) / 1.6,
            las = 1,
           lend = "butt",
            bty = "n")))
library(Epi)
library(popEpi)
library(dplyr)
library(tidyr)
@ %
\renewcommand{\rwpre}{./02addLexis}

<<echo = FALSE>>=
anfang <- Sys.time()
@ %

\chapter{Overview and rationale}

This note describes the functions \texttt{addCov.Lexis}, designed to
add values of clinical measurements, and \texttt{addDrug.Lexis}
designed to add drug exposure to time-split \texttt{Lexis} objects.

If time-dependent variables are binary, such as for example
``occurrence of CVD diagnosis'' it may be relevant to define a new
state as, say, \texttt{CVD}; this is the business of the funtions
\texttt{cutLexis} and its cousins. The purposes of the two functions
discussed here are to append quantitative variables that in principle
can take any value.

Both functions are so-called \texttt{S3} methods for \texttt{Lexis}
objects, so in code you can omit the ``\texttt{.Lexis}''.  Note that
neither \texttt{cutLexis}, \texttt{splitLexis} or \texttt{splitMulti}
are \texttt{S3} methods (there is no ``\texttt{.}'' in the names).

\section{\texttt{addCov.Lexis}}

\ldots provides the ability to amend a \texttt{Lexis} object with
clinical measurements taken at different times, and propagate the
values as LOCF (Last Observation Carried Forward) to all subsequent
records. This means that time-splitting of a \texttt{Lexis} object
\emph{after} adding clinical measurements will be meaningful, because
both \texttt{splitLexis} and \texttt{splitMulti} will carry variables
forward across the split records. The follow-up in the resulting
\texttt{Lexis} object will be cut at dates of clinical measurement.

\texttt{addCov.Lexis} will also propagate missing values supplied as
measurements. Therefore, if you want to have LOCF \emph{across}
supplied times of measurement you must explicitly apply
\texttt{tidyr::fill} to the resulting \texttt{Lexis} object, after a
suitable \texttt{group\_by}.

\section{\texttt{addDrug.Lexis}}

As opposed to this, \texttt{addDrug.Lexis} will first use drug
information at each date of recorded drug purchase, and subsequently
\texttt{compute} cumulative exposure measures at the times in the
resulting \texttt{Lexis} object. This is essentially by linear
interpolation, so it will not be meaningful to further split an
object resulting from \texttt{addDrug.Lexis}---LOCF is not meaningful
for continuously time-varying covariates such as cumulative exposure.

If persons have very frequent drug purchases, the intervals may become
very small and the sheer number of records may present an impediment
to analysis. Therefore the function \texttt{coarse.Lexis} is provided
to collapse adjacent follow-up records. Note that
\texttt{coarse.Lexis} is \emph{not} an \texttt{S3} method.

\chapter{\texttt{addCov.Lexis}}

\section{Rationale}
The function has arisen out of a need to attach values measured at
clinical visits to a Lexis object representing follow-up for events
constituting a multistate model. Hence the data frame with
measurements at clinical visits will be called \texttt{clin} for
mnemonic reasons.

\section{Example}
For illustration we devise a small bogus cohort of 3 people, where we
convert the character dates into numerical variables (fractional
years) using \texttt{cal.yr}. Note that we are using a character
variable as \texttt{id}:
<<>>=
xcoh <- structure(list(id = c("A", "B", "C"),
                    birth = c("1952-07-14", "1954-04-01", "1987-06-10"),
                    entry = c("1965-08-04", "1972-09-08", "1991-12-23"),
                     exit = c("1997-06-27", "1995-05-23", "1998-07-24"),
                     fail = c(1, 0, 1) ),
                   .Names = c("id", "birth", "entry", "exit", "fail"),
                row.names = c("1", "2", "3"),
                    class = "data.frame" )
xcoh$dob <- cal.yr(xcoh$birth)
xcoh$doe <- cal.yr(xcoh$entry)
xcoh$dox <- cal.yr(xcoh$exit )
xcoh
@ %

\subsection{A \texttt{Lexis} object}

Define this as a \texttt{Lexis} object with timescales calendar time
(\texttt{per}, period) and age (\texttt{age}):
<<>>=
Lcoh <- Lexis(entry = list(per = doe),
               exit = list(per = dox,
                           age = dox - dob),
                 id = id,
        exit.status = factor(fail, 0:1, c("Alive","Dead")),
               data = xcoh)
str(Lcoh)
(Lx <- Lcoh[,1:6])
@ %

\subsubsection{Factor or character \texttt{lex.id}?}

Note that when the \texttt{id} argument to \texttt{Lexis} is a
character variable then the \texttt{lex.id} will be a factor. Which,
if each person has a lot of records may save time, but if you subset
may be a waste of space. Moreover merging (\ie joining in the language
of \texttt{tidyverse}) may present problems with different
levels. \texttt{merge} from the \texttt{base} \R, will coerce to
factor with union of levels as levels, where as the \texttt{\_join}
functions from \texttt{dplyr} will coerce to character.

Thus the most reasonable strategy thus seems to keep \texttt{lex.id}
as a character variable.
<<>>=
Lx$lex.id <- as.character(Lx$lex.id)
str(Lx)
Lx
@ %

\subsubsection{Clinical measurements}

Then we generate data frame with clinical examination data, that is
date of examination in \texttt{per}, some (bogus) clinical measurements
and also names of the examination rounds:
<<>>=
clin <- data.frame(lex.id = c("A", "A", "C", "B", "C"),
                      per = cal.yr(c("1977-3-17",
                                     "1973-7-29",
                                     "1996-3-1",
                                     "1990-7-14",
                                     "1989-1-31")),
                       bp = c(120, 140, 160, 157, 145),
                     chol = c(NA, 5, 8, 9, 6),
                     xnam = c("X2", "X1", "X1", "X2", "X0"),
         stringsAsFactors = FALSE)
str(clin)
clin
@ %
We set up this data frame with an \texttt{id} variable called
\texttt{lex.id} and a date of examination, \texttt{per}, that has the
same name as one of the time scales in the Lexis object \texttt{Lx}.
Note that we have chosen a measurement for person \texttt{C} from
1989---before the person's entry to the study, and have an \texttt{NA}
for \texttt{chol} for person \texttt{A}.

\subsection{Adding clinical data}

There is a slightly different behaviour according to whether the
variable with the name of the examination is given or not, and whether
the name of the (incomplete) time scale is given or not:
<<>>=
(Cx <- addCov.Lexis(Lx, clin))
@ %
Note that the clinical measurement preceding the entry of person
\textsc{C} is included, and that the \texttt{tfc} (time from clinical
measurement) is correctly rendered, we a non-zero value at date of entry.

We also see that a variable \texttt{exnam} is constructed with
consecutive numbering of examinations within each person, while the
variable \texttt{xnam} is just carried over as any other.

If we explicitly give the name of the variable holding the examination
names we do not get a constructed \texttt{exnam}. We can also
define the name of the (incomplete) timescale to hold the time
since measurement, in this case as \texttt{tfCl}:
<<>>=
(Dx <- addCov.Lexis(Lx, clin, exnam = "xnam", tfc = "tfCl"))
summary(Dx, t=T)
@ %

\section{Exchanging split and add}

As noted in the beginning of this note, \texttt{addCov.Lexis} uses LOCF,
and so it is commutative with \texttt{splitLexis}:
<<>>=
# split BEFORE add
Lb <- addCov.Lexis(splitLexis(Lx,
                      time.scale = "age",
                          breaks = seq(0, 80, 5)),
                   clin,
                   exnam = "xnam" )
Lb
#
# split AFTER add
La <- splitLexis(addCov.Lexis(Lx,
                            clin,
                           exnam = "xnam" ),
                 time.scale = "age",
                     breaks = seq(0, 80, 5))
La
@ %
We see that the results are identical, bar the sequence of variables
and attributes.

We can more explicitly verify that the resulting data frames are the same:
<<>>=
La$tfc == Lb$tfc
La$age == Lb$age
La$per == Lb$per
@ %
The same goes for \texttt{splitMulti}:
<<>>=
## split BEFORE add
Mb <- addCov.Lexis(splitMulti(Lx, age = seq(0, 80, 5)),
                   clin,
                   exnam = "xnam" )
##
## split AFTER add
Ma <- splitMulti(addCov.Lexis(Lx,
                              clin,
                              exnam = "xnam" ),
                 age = seq(0, 80, 5))
La$tfc == Mb$tfc
Ma$tfc == Mb$tfc
@ %
In summary, because both \texttt{addCov.Lexis} and
\texttt{splitLexis}/\texttt{splitMulti} use LOCF for covariates the
order of splitting and adding does not matter.

This is certainly not the case with \textrm{addDrug.Lexis} as we shall see.

\section{Filling the \texttt{NA}s}

As mentioned in the beginning, clinical measurements given as
\texttt{NA} in the \texttt{clin} data frame are carried forward. If
you want to have these replaced by 'older' clinical measurements you
can do that explicitly by \texttt{dplyr::fill} with a construction
like:
<<>>=
cov <- c("bp", "chol")
Lx <- La
Lx <- group_by(Lx, lex.id) %>%
         fill(all_of(cov)) %>%
      ungroup()
class(Lx)
@ %
We see that the \texttt{Lexis} attributes are lost by using the
\texttt{group\_by} function, so we fish out the covariates from the
\texttt{tibble} and stick them back into the \texttt{Lexis} object:
<<>>=
Lx <- La
Lx[,cov] <- as.data.frame(group_by(Lx, lex.id)
                          %>% fill(all_of(cov)))[,cov]
class(Lx)
La
Lx
@ %
The slightly convoluted code where the covariate columns are
explicitly selected, owes to the fact that the \texttt{dplyr}
functions will strip the data frames of the \texttt{Lexis}
attributes. So we needed to use \texttt{fill} to just generate the
covariates and not touch the \texttt{Lexis} object itself.

This should of course be built into \texttt{addCov.Lexis} as a
separate argument, but is not yet.

Note that the \texttt{tfc}, time from clinical measurement, is now not
a valid time scale variable any more; the 5 in \texttt{chol} is
measured at 1973.7 but \texttt{tfc} is reset to 0 at 1977.21, even if
only \texttt{bp} but not \texttt{chol} is measured at that time.  If
you want that remedied you will have to use \texttt{addCov.Lexis}
twice, one with a \texttt{clin} data frame with only \texttt{bp} and
another with a data frame with only \texttt{chol}, each generating a
differently named variable holding the time from clinical measurement.

This is a problem that comes from the structure of the supplied
\emph{data} not from the program features; in the example we had
basically measurements of different clinical variables at different
times, and so necessarily also a need for different times since last
measurement.

\chapter{\texttt{addDrug.Lexis}}

The general purpose of the function is to amend a \texttt{Lexis}
object with drug exposure data. The data base with information on a
specific drug is assumed to be a data frame with one entry per drug
purchase (or prescription), containing the date and the amount
purchased and optionally the prescribed dosage (that is how much is
supposed to be taken per time). We assume that we have such a data
base for each drug of interest, which also includes an id variable,
\texttt{lex.id}, that matches the \texttt{lex.id} variable in the
\texttt{Lexis} object.

For each type of drug the function derives 4 variables:
\begin{description}[noitemsep]
 \item[\hspace*{1em}\texttt{ex}]: logical, is the person currently
   \texttt{ex}posed
 \item[\hspace*{1em}\texttt{tf}]: numeric, \texttt{t}ime since
   \texttt{f}irst purchase
 \item[\hspace*{1em}\texttt{ct}]: numeric, \texttt{c}umulative
   \texttt{t}ime on the drug
 \item[\hspace*{1em}\texttt{cd}]: numeric, \texttt{c}umulative
   \texttt{d}ose of the drug
\end{description}
These names are pre- or suf-fixed by the drug name, so that exposures
to different drugs can be distinguished; see the examples.

The resulting \texttt{Lexis} object has extra records corresponding to
cuts at each drug purchase and at each expiry date of a purchase.  For
each purchase the coverage period is derived (different methods for
this are available), and if the end of this (the expiry date) is
earlier than the next purchase of the person, the person is considered
off the drug from the expiry date, and a cut in the follow-up is
generated with \texttt{ex} set to \texttt{FALSE}.

\section{The help example}

The following is a slight modification of the code from the example
section of the help page for \texttt{addDrug.Lexis}

First we generate follow-up of 2 persons, and split the follow-up in
intervals of length 0.6 years along the calendar time scale,
\texttt{per}:
<<>>=
fu <- data.frame(doe = c(2006, 2008),
                 dox = c(2015, 2018),
                 dob = c(1950, 1951),
                 xst = factor(c("A","D")))
Lx <- Lexis(entry = list(per = doe,
                         age = doe- dob),
             exit = list(per = dox),
      exit.status = xst,
             data = fu)
Lx <- subset(Lx, select = -c(doe, dob, dox, xst))
Sx <- splitLexis(Lx, "per", breaks = seq(1990, 2020, 0.6))
summary(Sx)
str(Sx)
@ %
Note that as opposed to the previous example, the time scales are not
of class \texttt{cal.yr}, they are just numerical.

Then we generate example drug purchases for these two persons, one
data frame for each of the drugs \texttt{F} and \texttt{G}. Note that
we generate \texttt{lex.id}$\in (1,2)$ referring to the values of
\texttt{lex.id} in the lexis object \texttt{Sx}.
<<>>=
set.seed(1952)
rf <- data.frame(per = c(2005 + runif(12, 0, 10)),
                 amt = sample(2:4, 12, replace = TRUE),
              lex.id = sample(1:2, 12, replace = TRUE)) %>%
      arrange(lex.id, per)

rg <- data.frame(per = c(2009 + runif(10, 0, 10)),
                 amt = sample(round(2:4/3,1), 10, replace = TRUE),
              lex.id = sample(1:2, 10, replace = TRUE)) %>%
      arrange(lex.id, per)
@ %
We do not need to sort the drug purchase data frames (it is done
internally by \texttt{addDrug.Lexis}), but it makes it easier to grasp
the structure. Note that we generated the drug purchase files with the
required variable names.

The way purchase data is supplied to the function is in a
\texttt{list} where each element is a data frame of purchase records
for one type of drug. The list must be named, because the names are
used as prefixes of the generated exposure variables. We can show the
resulting data in a list:
<<>>=
pdat <- list(F = rf, G = rg)
pdat
Lx
@ %
Note that we have generated data so that there are drug purchases of
drug \texttt{F} that is \emph{before} start of follow-up for person 2.

We can then expand the time-split \texttt{Lexis} object, \texttt{Sx}
with the drug information. \texttt{addDrug.Lexis} not only adds 8
\emph{variables} (4 from each drug), it also adds \emph{records}
representing cuts at the purchase dates and possible expiry dates.
<<>>=
summary(Sx) ; names(Sx)
ex1 <- addDrug.Lexis(Sx, pdat, method = "ext") # default
summary(ex1) ; names(ex1)
print(ex1, nd = 2)
ex2 <- addDrug.Lexis(Sx, pdat, method = "ext", grace = 0.5)
summary(ex2)
print(ex2, nd = 2)
dos <- addDrug.Lexis(Sx, pdat, method = "dos", dpt = 6)
summary(dos)
print(dos, nd = 2)
fix <- addDrug.Lexis(Sx, pdat, method = "fix", maxt = 1)
summary(fix)
print(fix, nd = 2)
@ %

\section{A more realistic example with run times}

\subsection{Follow-up data: \texttt{DMlate}}

As example data we use rows from the \texttt{DMlate} example data from
the \texttt{Epi} package:
<<>>=
data(DMlate) ; str(DMlate)
Lx <- Lexis(entry = list(per = dodm,
                         age = dodm - dobth,
                         tfd = 0),
             exit = list(per = dox),
      exit.status = factor(!is.na(dodth),
                           labels = c("DM", "Dead")),
             data = DMlate[sample(1:nrow(DMlate), 1000),])
summary(Lx)
@ %
We split the data along the age-scale (omitting the variables we shall
not need):
<<>>=
Sx <- splitLexis(Lx[,1:7], time.scale="age", breaks = 0:120)
summary(Sx)
str(Sx)
@ %

\subsection{Artificial prescription data}

To explore how \texttt{addDrug.Lexis} works, we need drug exposure
data, but these are unfortunately not available, so we simulate three
datasets representing \texttt{pur}chases of three types of drugs:
<<>>=
set.seed(1952)

purA <-
  ( data.frame(lex.id = rep(Lx$lex.id,
                            round(runif(nrow(Lx), 0, 20))))
%>% left_join(Lx[,c("lex.id", "dodm", "dox")])
%>% mutate(per = dodm + runif(length(dodm), -0.1, 0.99) * (dox - dodm),
           amt = sample(4:20*10, length(dodm), replace = TRUE),
           dpt = amt * round(runif(length(dodm), 3, 7)))
%>% select(-dodm, -dox)
%>% arrange(lex.id, per)
  )
addmargins(table(table(purA$lex.id)))
str(purA)

purB <-
  ( data.frame(lex.id = rep(Lx$lex.id,
                            round(pmax(runif(nrow(Lx), -10, 15), 0))))
%>% left_join(Lx[,c("lex.id", "dodm", "dox")])
%>% mutate(per = dodm + runif(length(dodm), -0.1, 0.99) * (dox - dodm),
           amt = sample(4:20*10, length(dodm), replace = TRUE),
           dpt = amt * round(runif(length(dodm), 5, 9)))
%>% select(-dodm, -dox)
%>% arrange(lex.id, per)
  ) -> purB
addmargins(table(table(purB$lex.id)))
str(purB)

purC <-
  ( data.frame(lex.id = rep(Lx$lex.id,
                            round(pmax(runif(nrow(Lx), -5, 12), 0))))
%>% left_join(Lx[,c("lex.id", "dodm", "dox")])
%>% mutate(per = dodm + runif(length(dodm), -0.1, 0.99) * (dox - dodm),
           amt = sample(4:20*10, length(dodm), replace = TRUE),
           dpt = amt * round(runif(length(dodm), 5, 7)))
%>% select(-dodm, -dox)
%>% arrange(lex.id, per)
  )
addmargins(table(table(purC$lex.id)))
str(purC)
head(purC)
@ %
Note that the time scale is in years, so the \texttt{dpt} must be in
amount per year, so that $\mathtt{dpt}/\mathtt{amt}$ is the
approximate number of annual drug purchases.

We now have three artificial drug purchase datasets so we can see how
\texttt{addDrug.Lexis} performs on larger datasets:

\subsection{Using \texttt{addDrug}}

\subsubsection{100 and 500 persons}

We start out with a small sample and a three month grace period to limit
the number of gaps:
<<>>=
Sx1 <- subset(Sx, lex.id < 100)
pur <- list(A = subset(purA, lex.id < 1000),
            B = subset(purB, lex.id < 1000),
            C = subset(purC, lex.id < 1000))
system.time(ad1 <- addDrug.Lexis(Sx1, pur, tnam = "per", grace = 1/4))
summary(Sx1)
summary(ad1)
@ %
We then cut the number of persons in half to assess how run time
depends on no. of persons in the data:
<<>>=
Sx2 <- subset(Sx, lex.id < 500)
pur <- list(A = subset(purA, lex.id < 500),
            B = subset(purB, lex.id < 500),
            C = subset(purC, lex.id < 500))
system.time(ad2 <- addDrug.Lexis(Sx2, pur, tnam = "per", grace = 1/6))
summary(Sx2)
summary(ad2)
@ %
\ldots timing is broadly proportional to the number of persons.

\subsubsection{Fewer prescription records}

We can try to cut the number of purchases in half:
<<>>=
pur <- list(A = subset(purA, lex.id < 100 & runif(nrow(purA)) < 0.5),
            B = subset(purB, lex.id < 100 & runif(nrow(purB)) < 0.5),
            C = subset(purC, lex.id < 100 & runif(nrow(purC)) < 0.5))
sapply(pur, nrow)
system.time(ad3 <- addDrug.Lexis(Sx1, pur, tnam = "per", grace = 1/6))
summary(Sx1)
summary(ad3)
@ %
It also appears that the number of purchases per person is also a
determinant of the run time too; the timing is largely proportional to
the number of drug records.

In any concrete application it is recommended to run the function on
a fairly small sample of persons, say 1000 to get a feel for the run
time. It may also be a good idea to run the function on chunks of the
persons, to make sure that you do not lose all the processed data in a
crash.

\subsubsection{Fewer prescription types}

Finally we try to cut the number of drugs, to assess how this
influences the run time:
<<>>=
pur <- list(B = subset(purB, lex.id < 100),
            C = subset(purC, lex.id < 100))
sapply(pur, nrow)
system.time(ad4 <- addDrug.Lexis(Sx1, pur, tnam = "per", grace = 1/6))
summary(Sx1)
summary(ad4)
@ %
We see that the number of drugs also influence the run time
proportionally.

\subsection{Too many records ---\texttt{coarse.Lexis}}

If we look at the length of the intervals as given in \texttt{lex.dur}
we see that some are are quite small:
<<>>=
summary(ad1$lex.dur)
@ %
Half are smaller then 0.11 years, 40 days. We could without much loss
of precision in the analysis based on the \texttt{Lexis} object merge
adjacent records that have total risk time less than 3 months.

The function \texttt{coarse.Lexis} will collapse records with short
\texttt{lex.dur} with the subsequent record. The collapsing will use
the covariates from the first record, and so the entire follow-up from
the two records will have the characteristics of the first. Therefore
it is wise to choose first records with reasonably short
\texttt{lex.dur}---the approximation will be better than if the first
record was with a larger \texttt{lex.dur}. Therefore there are two
values supplied to \texttt{coarse.Lexis}; the maximal length of the
first record's \texttt{lex.dur} and the maximal length of the
\texttt{lex.dur} in the resulting combined record. The larger these
parameters are, the more the \texttt{Lexis} object is coarsened.
<<>>=
summary(ad1)
summary(adc <- coarse.Lexis(ad1, lim = c(1/6,1/2)))
summary(adc$lex.dur)
@ %
This could cut the number of units for analysis substantially, in this
case from about 27,000 to some 13,000.

\subsubsection{Records to be kept}

When we are dealing with drug exposure data we will be interested
keeping the record that holds the start of a drug exposure. Some may
argue that it does not matter much, though.

The records (\ie beginnings of FU intervals) that should be kept must
be given in logical vector in the argument \texttt{keep}:
<<>>=
summary(Sx2)
system.time(ad4 <- addDrug.Lexis(Sx2,
                                 pur,
                                tnam = "per",
                               grace = 1/6))
summary(ad4)
#
ad5 <- coarse.Lexis(ad4,
                    lim = c(1/4, 1/2))
summary(ad5)
@
We can identify the first date of exposure to drug \texttt{B}, say, by
the exposure (\texttt{B.ex}) being true and the cumulative time on the
drug (\texttt{B.ct}) being 0:
<<>>=
ad4$keep <- with(ad4, (B.ex & B.ct == 0) |
                      (C.ex & C.ct == 0))
ad6 <- coarse.Lexis(ad4,
                    lim = c(1/4, 1/2),
                   keep = ad4$keep)
summary(ad6)
@ %
We see the expected behaviour when we use \texttt{coarse.Lexis}: we get
fewer records, but identical follow-up. And the \texttt{keep} argument
gives the possibility to keep selected records, or more precisely
beginnings. \texttt{keep} prevents a record to be collapsed with a
previous one, but not with a subsequent one.

%% \subsection{The entire example dataset}

%% The entire amount of example data consist of some 10,000 persons and
%% some 200,000 prescriptions:
%% <<eval=FALSE>>=
%% dim(Sx)
%% pur <- list(A = purA,
%%             B = purB,
%%             C = purC)
%% sapply(pur, nrow)
%% system.time(adx <- addDrug.Lexis(Sx, pur, tnam = "per", grace = 1/6))
%% system.time(adc <- coarse.Lexis(adx, lim = c(1/6, 1/2)))
%% summary(Sx)
%% summary(adx)
%% summary(adc)
%% @ %
%% We see hat the number of records is quite large because we have cut at
%% all purchase dates and integer ages. For practical purposes we might
%% therefore want to merge successive records with a total duration
%% \texttt{lex.dur} less than some limit.

<<echo = FALSE>>=
ende <- Sys.time()
cat("  Start time:", format(anfang, "%F, %T"),
  "\n    End time:", format(  ende, "%F, %T"),
  "\nElapsed time:", round(difftime(ende, anfang, units = "mins"), 2), "minutes\n")
@ %

\bibliographystyle{plain}

\begin{thebibliography}{1}

\bibitem{Carstensen.2007a}
B~Carstensen.
\newblock Age-{P}eriod-{C}ohort models for the {L}exis diagram.
\newblock {\em Statistics in Medicine}, 26(15):3018--3045, 2007.

\bibitem{Carstensen.2008c}
B~Carstensen, JK~Kristensen, P~Ottosen, and K~Borch-Johnsen.
\newblock The {D}anish {N}ational {D}iabetes {R}egister: {T}rends in incidence,
  prevalence and mortality.
\newblock {\em Diabetologia}, 51:2187--2196, 2008.

\end{thebibliography}

\addcontentsline{toc}{chapter}{References}

\end{document}

\name{foreign.Lexis}
\Rdversion{1.1}
\alias{msdata}
\alias{msdata.Lexis}
\alias{etm}
\alias{etm.Lexis}
\title{Create a data structures suitable for use with packages \code{mstate} or \code{etm}.
  }
\description{
The \code{mstate} package requires input in the form of a stacked
dataset with specific variable names. This is provided by
\code{msdata.Lexis}. The resulting dataframe contains the same
information as the result of a call to \code{\link{stack.Lexis}}.

The \code{etm} package requires input (almost) in the form of a
\code{Lexis} object, but with specific column names etc. This is
provided by \code{etm.Lexis}.
}
\usage{
msdata(obj, ...)
\method{msdata}{Lexis}(obj,
                time.scale = timeScales(obj)[1],
                       ... )
\method{etm}{Lexis}( data,
               time.scale = timeScales(data)[1],
                cens.name = "cens",
                        s = 0,
                        t = "last",
               covariance = TRUE,
                 delta.na = TRUE,
                      ... )
}
\arguments{
  \item{obj}{A \code{\link{Lexis}} object.}
  \item{data}{A \code{\link{Lexis}} object.}
  \item{time.scale}{Name or number of timescale in the \code{Lexis}
    object.}
  \item{cens.name}{Name of the code for censoring used by \code{etm}. It
  is only necessary to change this if one of the states in the
  \code{Lexis} object has name "\code{cens}".}
  \item{s}{Passed on to \code{etm}.}
  \item{t}{Passed on to \code{etm}.}
  \item{covariance}{Passed on to \code{etm}.}
  \item{delta.na}{Passed on to \code{etm}.}
  \item{\dots}{Further arguments.}
}
\value{
  \code{msdata.Lexis} returns a dataframe with the \code{Lexis} specific
  variables stripped, and with the following added: \code{id},
  \code{Tstart}, \code{Tstop}, \code{from}, \code{to}, \code{trans},
  \code{status}, which are used in the \code{mstate} package.

  \code{etm.Lexis} transforms the \code{Lexis} object into a dataframe
  suitable for analysis by the function \code{etm} from the \code{etm}
  package, and actually calls this function, so returns an object of
  class \code{etm}.
}
\author{
Bendix Carstensen, \email{b@bxc.dk}, \url{http://bendixcarstensen.com}
}
\examples{
data(DMlate)
str(DMlate)
dml <- Lexis( entry = list(Per=dodm,Age=dodm-dobth,DMdur=0),
               exit = list(Per=dox),
        exit.status = factor(!is.na(dodth),labels=c("DM","Dead")),
               data = DMlate[1:1000,] )
dmi <- cutLexis( dml, cut=dml$doins, new.state="Ins", pre="DM" )
summary( dmi )

# Use the interface to the mstate package
if( require(mstate) )
{
ms.dmi <- msdata.Lexis( dmi )
# Check that all the transitions and person-years got across.
with( ms.dmi, rbind( table(status,trans),
                     tapply(Tstop-Tstart,trans,sum) ) )
}

# Use the etm package directly with a Lexis object
if( require(etm) )
{
dmi <- subset(dmi,lex.id<1000)
etm.D <- etm.Lexis( dmi, time.scale=3 )
str( etm.D )
plot( etm.D, col=rainbow(5), lwd=2, lty=1, xlab="DM duration" )
}
}
\seealso{
  \code{\link{stack.Lexis}},
  \code{\link[mstate:msprep]{msprep}},
  \code{\link[etm:etm]{etm}} }
\keyword{survival}

\name{S.typh}
\alias{S.typh}
\docType{data}
\title{Salmonella Typhimurium outbreak 1996 in Denmark.}
\description{
Matched case-control study of food poisoning.
}
\format{
  A data frame with 136 observations on the following 15 variables:
  \tabular{rl}{
    \code{id}: \tab Person identification \cr
    \code{set}: \tab Matched set indicator \cr
    \code{case}: \tab Case-control status (1:case, 0:control \cr
    \code{age}: \tab Age of individual \cr
    \code{sex}: \tab Sex of individual (1:male, 2:female) \cr 
    \code{abroad}: \tab Within the last two weeks visited abroad (1:yes, 0:no) \cr
    \code{beef}: \tab Within the last two weeks eaten beef \cr
    \code{pork}: \tab Within the last two weeks eaten pork \cr
    \code{veal}: \tab Within the last two weeks eaten veal \cr
    \code{poultry}: \tab Within the last two weeks eaten poultry \cr
    \code{liverp}: \tab Within the last two weeks eaten liverpaste \cr
    \code{veg}: \tab Within the last two weeks eaten vegetables \cr
    \code{fruit}: \tab Within the last two weeks eaten fruit \cr
    \code{egg}: \tab Within the last two weeks eaten eggs \cr
    \code{plant7}: \tab Within the last two weeks eaten meat from plant no. 7 \cr
  }
  }
\details{
  In the fall of 1996 an unusually large number of Salmonella
  Typhimurium cases were recorded in Fyn county in Denmark. The Danish
  Zoonosis Centre set up a matched case-control study to find the
  sources. Cases and two age-, sex- and residency-matched controls were
  telephone interviewed about their food intake during the last two
  weeks.

  The participants were asked at which retailer(s) they had purchased
  meat. Retailers were independently of this linked to meat processing
  plants, and thus participants were linked to meat processing
  plants. This way persons could be linked to (amongst other) plant no 7.} 
\source{
  Tine Hald.
}
\references{
  Molbak K and Hald T: Salmonella Typhimurium outbreak in late summer
  1996. A Case-control study. (In Danish:
  Salmonella typhimurium udbrud paa Fyn sensommeren
  1996. En case-kontrol undersogelse.) Ugeskrift for Laeger.,
  159(36):5372-7, 1997.
}
\examples{
data(S.typh)
}
\keyword{datasets}

\name{legendbox}
\alias{legendbox}
\title{Draw a box with text explaining the numbers in and between boxes
  from \code{boxes.MS} and  \code{boxes.Lexis}
}
\description{When drawing boxes describing a multistate model a legend
  explaining the numbers in the plot is required. \code{legendbox} does
  this. 
}
\usage{
legendbox(x, y,
         state = "State",
            py = "Person-time",
         begin = "no. begin",
           end = "no. end",
         trans = "Transitions",
         rates = "\n(Rate)",
          font = 1,
         right = !left,
          left = !right,
           ...)
}
\arguments{
  \item{x}{
x-coordinate of the center of the box.
}
  \item{y}{
y-coordinate of the center of the box.
}
  \item{state}{
Text describing the state
}
  \item{py}{
Text describing the risk time
}
  \item{begin}{
Text describing the no. persons starting FU in state
}
  \item{end}{
Text describing the no. persons ending FU in state
}
  \item{trans}{
Text describing the no. of transitions
}
  \item{rates}{
Text describing the rates
}
  \item{font}{
Font to use for the text 
}
  \item{right}{
Should a text describing arrow texts be on the r.h.s. of the box?
Defaults to TRUE.
}
  \item{left}{
Should a text describing arrow texts be on the l.h.s. of the box?
}
  \item{\dots}{
Arguments passed on to \code{tbox}
}
}
\value{None.
}
\details{The function is called for its side effect of adding an
  explanatory box to the plot. If \code{right} is true, an explanation
  of events and rates are added to the right of the box. Similarly for
  \code{left}. It is admissible that \code{left == right}.
}
\author{
Bendix Carstensen, \url{http://bendixcarstensen.com}
}
\seealso{
\code{\link{boxes.Lexis}}
}
\keyword{aplot}

\name{addCov}
\alias{addCov}
\alias{addCov.Lexis}
\title{
  Add covariates (typically clinical measurements) taken at known times
  to a Lexis object. 
}
\description{
  When follow-up in a multistate model is represented in a
  \code{\link{Lexis}} object we may want to add information on
  covariates, for example clinical measurements, obtained at different
  times. This function cuts the follow-up time (see
  \code{\link{cutLexis}}) at the times of measurement and carries the
  measurements forward in time to the next measurement occasion.
}
\usage{
\method{addCov}{Lexis}(Lx,
                     clin,
                timescale = 1,
                    exnam,
                      tfc = "tfc", \dots)
}
\arguments{
  \item{Lx}{
    A Lexis object with follow-up of a cohort.  
    }
  \item{clin}{
    A data frame with covariates to add (typically clinical
    measurements). Must contain a variable \code{lex.id} identifying the
    persons represented in \code{Lx}, as well as a variable with the
    same name as one of the \code{\link{timeScales}} in \code{Lx},
    identifying the time at which covariates are measured.

    The times must be unique within each person; if not records with
    duplicate times are discarded, and a warning issued. This is done
    using \code{duplicated}, so not very well-defined, it is advisable
    that you do this yourself.
    }
  \item{timescale}{
    Numerical or character. Number or name of a timescale in
    \code{Lx}. The \code{clin} data frame must have a variable of this
    name indicating the time at which the covariate measurements were
    taken.
    }
  \item{exnam}{
    Character. Name of the variable in \code{clin} with the examination
    names (such as \code{wave1}, \code{wave2} etc.). Values may not be
    repeated within person and cannot be equal to any of
    \code{levels(Lx)}. Will be carried over to the resulting 
    \code{Lexis} object. If there is no such variable in \code{clin} it
    will be constructed; if the argument is omitted, a variable called
    \code{exnam} with values \code{ex1}, \code{ex2} etc. will be
    constructed.
    }
  \item{tfc}{      
    Character (\code{t}ime \code{f}rom last \code{c}linical visit). Name
    of the variable in the result which will contain the time since the
    most recent covariate date. It will be included among the timescales
    of the resulting Lexis object.

    If the argument is omitted a variable called \code{tfc} will be
    constructed. 
    }
  \item{\dots}{Arguments passed on. Ignored.
}
}
\value{
    A \code{Lexis} object representing the same follow-up as \code{Lx}, 
    with cuts added at the times of examination, and covariate
    measurements added for all records representing follow-up after the
    most recent time of measurement.

    Also \code{tfc} is added as a time scale, it is however not a proper
    timescale since it is reset at every clinical examination. Therefor
    the value of the \code{timeSince} attribute is set to "X" in order
    to distinguish it from other proper time scales that either have an
    empty string or the name of a state.
}
\author{
  Bendix Carstensen, \email{b@bxc.dk}, \url{http://bendixcarstensen.com}
}
\seealso{
\code{\link{cutLexis}},
\code{\link{mcutLexis}},
\code{\link{splitLexis}},
\code{\link{Lexis}}
}
\examples{
# A small bogus cohort
xcoh <- structure( list( id = c("A", "B", "C"),
                      birth = c("1952-07-14", "1954-04-01", "1987-06-10"),
                      entry = c("1965-08-04", "1972-09-08", "1991-12-23"),
                       exit = c("1997-06-27", "1995-05-23", "1998-07-24"),
                       fail = c(1, 0, 1) ),
                     .Names = c("id", "birth", "entry", "exit", "fail"),
                  row.names = c("1", "2", "3"),
                      class = "data.frame" )

# Convert the character dates into numerical variables (fractional years)
xcoh$bt <- cal.yr( xcoh$birth )
xcoh$en <- cal.yr( xcoh$entry )
xcoh$ex <- cal.yr( xcoh$exit  )

# Define as Lexis object with timescales calendar time and age
Lcoh <- Lexis( entry = list( per=en ),
                exit = list( per=ex, age=ex-bt ),
         exit.status = factor( fail, 0:1, c("Alive","Dead") ),
                data = xcoh )
str( Lcoh )
Lx <- Lcoh[,1:7]

# Data frame with clinical examination data, date of examination in per
clin <- data.frame(lex.id = c(1,1,3,2),
                      per = cal.yr(c("1977-4-7",
                                     "1971-7-1",
                                     "1996-2-15",
                                     "1990-7-3")),
                       bp = c(120,140,160,157),
                     chol = c(5,7,8,9),
                     xnam = c("X2","X1","X1","X2") )
Lx
clin
str(Lx)
str(clin)

# Different behavours when using exnam explicitly
addCov.Lexis( Lx, clin[,-5] )
addCov.Lexis( Lx, clin, exnam="xnam" )

# Works with time split BEFORE
Lb <- addCov.Lexis(splitLexis(Lx,
                              time.scale="age",
                              breaks=seq(0,80,5) ),
                   clin,
                   exnam="clX" )
Lb
# and also AFTER
La <- splitLexis(addCov.Lexis( Lx,
                             clin,
                            exnam = "xnam" ),
                 breaks=seq(0,80,5),
                 time.scale="age" )
La
La$tfc == Lb$tfc
La$age == Lb$age
str(La)
str(Lb)
}
\keyword{ survival }

\name{plot.apc}
\alias{plot.apc}
\alias{apc.plot}
\title{Plot the estimates from a fitted Age-Period-Cohort model}
\description{
This function plots the estimates created by \code{\link{apc.fit}} in a single
graph. It just calls \code{\link{apc.frame}} after computing some sensible
values of the parameters, and subsequently plots the estimates using
\code{\link{apc.lines}}.
}
\usage{
\method{plot}{apc}( x, r.txt="Rate", ...)
          apc.plot( x, r.txt="Rate", ...)
}
\arguments{
  \item{x}{ An object of class \code{apc}. }
  \item{r.txt}{ The text to put on the vertical rate axis. }
  \item{\dots}{ Additional arguments passed on to \code{\link{apc.lines}}. }
}
\value{
 A numerical vector of length two, with names
  \code{c("cp.offset","RR.fac")}. The first is the offset for the cohort
  period-axis, the second the multiplication factor for the rate-ratio
  scale. Therefore, if you want to plot at \code{(x,y)} in the right panel,
  use \code{(x-res["cp.offset"],y/res["RR.fac"])}
  \code{=(x-res[1],y/res[2])}.
  This vector should be supplied for the parameter \code{frame.par} to
  \code{\link{apc.lines}} if more sets of estimates is plotted in the
  same graph, however see \code{\link{cp.points}}.
}
\details{\code{plot.apc} is just a wrapper for \code{apc.plot}.}
\author{
  Bendix Carstensen, Steno Diabetes Center,
  \url{http://bendixcarstensen.com}
}
\seealso{
  \code{\link{apc.lines}}, \code{\link{lines.apc}},
  \code{\link{apc.frame}}, \code{\link{apc.fit}}
}
\examples{
data( lungDK )
apc1 <- apc.fit( transform( lungDK,
                            A = Ax, P = Px, Y = Y/10^5 ),
                 ref.c = 1920 )
fp <- apc.plot( apc1 )
apc.lines( apc1, frame.par=fp, drift=1.01, col="red" )
for( i in 1:11 )
  apc.lines( apc1, frame.par=fp, drift=1+(i-6)/100, col=rainbow(12)[i] )
}
\keyword{hplot}

\name{poisreg}
\alias{poisreg}

\title{Family Object for Poisson Regression}
\usage{
poisreg(link = "log")
}
\arguments{
  \item{link}{a specification for the model link function.  The
    \code{poisreg} family accepts the links \code{identity},
    \code{log} and \code{inverse}.
  }
}
\description{

  The \code{poisreg} family allows Poisson regression models to be
  fitted using the \code{glm} function.
  
  In a Poisson regression model, we assume that the data arise from a
  Poisson process. We observe D disease events in follow up time Y and
  wish to estimate the incidence rate, which is assumed to be constant
  during the follow-up period for any individual. The incidence rate
  varies between individuals according to the predictor variables and
  the link function in the model specification.
                                                                     
  When using the \code{poisreg} family in the \code{glm} function, the
  response should be specified as a two-column matrix with the first
  column giving the number of events (D) and the second column giving
  the observation time (Y). This is similar to the \code{binomial}
  family for which a two-column outcome can be used representing the
  number of successes and the number of failures.

}
\note{

  When using the log link, Poisson regression can also be carried out
  using the \code{poisson} family by including the log follow-up time
  \code{log(Y)} as an offset. However this approach does not generalize
  to other link functions. The \code{poisreg} family allows more general
  link functions including additive risk models with \code{poisreg(link
    = "identity")}. 

}
\value{
  An object of class \code{"family"}.  See \code{\link[stats]{family}}
  for details.

  The family name, represented by the element \code{"family"} in the
  returned object, is \code{"poisson"} and not \code{"poisreg"}. This is
  necessary to prevent the \code{summary.glm} function from estimating
  an overdispersion parameter (which should be fixed at 1) and therefore
  giving incorrect standard errors for the estimates.

}
\examples{
  ## Estimate incidence rate of diabetes in Denmark (1996-2015) by
  ## age and sex
  data(DMepi)
  DMepi$agegrp <- cut(DMepi$A, seq(from=0, to=100, by=5))
  inc.diab <- glm(cbind(X, Y.nD) ~ -1 + agegrp + sex, family=poisreg,
                  data=DMepi)
  ## The coefficients for agegrp are log incidence rates for men in each
  ## age group. The coefficient for sex is the log of the female:male
  ## incidence rate ratio.
  summary(inc.diab)

  ## Smooth function with non-constant M/F RR:
  requireNamespace("mgcv")
  library( mgcv )
  gam.diab <- gam( cbind(X, Y.nD) ~ s(A,by=sex) + sex,
                   family=poisreg,
                   data=DMepi)

  ## There is no need/use for Y.nD in prediction data frames:
  nM <- data.frame( A=20:90, sex="M" )
  nF <- data.frame( A=20:90, sex="F" )

  ## Rates are returned in units of (1 year)^-1, so we must scale the
  ## rates by hand: 
  matshade( nM$A, cbind( ci.pred(gam.diab,     nM    )*1000,
                         ci.pred(gam.diab,        nF )*1000,
                         ci.exp( gam.diab,list(nM,nF)) ),
            plot=TRUE, col=c("blue","red","black"),
            log="y", xlab="Age", ylab="DM incidence rates per 1000     /     M vs. F RR" )
  abline(h=1)
}
\seealso{
  \code{\link[stats]{glm}}, \code{\link[stats]{family}}.
}
\keyword{models}

\name{apc.LCa}
\alias{apc.LCa}
\alias{show.apc.LCa}
\title{Fit Age-Period-Cohort models and Lee-Carter models with effects
  modeled by natural splines.
}
\description{
\code{apc.LCa} fits an Age-Period-Cohort model and sub-models (using
\code{\link{apc.fit}}) as well as Lee-Carter models (using
\code{\link{LCa.fit}}). \code{show.apc.LCa} plots the models in little
boxes with their residual deviance with arrows showing their
relationships.
}
\usage{
apc.LCa( data,
  keep.models = FALSE,
          ... )
show.apc.LCa( x,
       dev.scale = TRUE,
             top = "Ad", ... )
}
\arguments{
  \item{data}{A data frame that must have columns \code{A}, \code{P},
    \code{D} and \code{Y}, see e.g. \code{\link{apc.fit}}
  }
  \item{keep.models}{Logical. Should the \code{apc} object and the 5
    \code{LCa} objects be returned too?
  }
  \item{...}{Further parameters passed on to \code{\link{LCa.fit}} or
    \code{\link{boxes.matrix}}. 
  }
  \item{x}{The result from a call to \code{apc.LCa}.}
  \item{dev.scale}{Should the vertical position of the boxes with the
    models be scales relative to the deviance between the Age-drift
    model and the extended Lee-Carter model?}
  \item{top}{The model presented at the top of the plot of boxes
    (together with any other model with larger deviance) when
    vertical position is scaled by deviances. Only "Ad", "AP", "AC",
    "APa" or "ACa" will make sense.}
}
\details{The function \code{apc.LCa} fits all 9 models (well, 10) available as
  extension and sub-models of the APC-model and compares them by
  returning deviance and residual df.
}
\value{A 9 by 2 matrix classified by model and deviance/df; optionally
  (if \code{models=TRUE}) a list with the matrix as \code{dev}, \code{apc}, an
  \code{apc} object (from \code{\link{apc.fit}}), and \code{LCa}, a list
  with 5 \code{LCa} objects (from \code{\link{LCa.fit}}). 
}
\author{
Bendix Carstensen, \url{http://bendixcarstensen.com}
}
\seealso{
\code{ \link{apc.fit}}, \code{\link{LCa.fit} }
}
\examples{
library( Epi )
clear()
# Danish lung cancer incidence in 5x5x5 Lexis triangles
data( lungDK )
lc <- subset( lungDK, Ax>40 )[,c("Ax","Px","D","Y")]
names( lc )[1:2] <- c("A","P")
head( lc )

al <- apc.LCa( lc, npar=c(9,6,6,6,10), keep.models=TRUE, maxit=500, eps=10e-3 )
show.apc.LCa( al, dev=TRUE )

# Danish mortality data
\dontrun{
data( M.dk )
mdk <- subset( M.dk, sex==1 )[,c("A","P","D","Y")]
head( mdk )

al <- apc.LCa( mdk, npar=c(15,15,20,6,6), maxit=50, eps=10e-3,
               quiet=FALSE, VC=FALSE )
show.apc.LCa( al, dev=FALSE )
show.apc.LCa( al, dev=TRUE )
show.apc.LCa( al, top="AP" )

# Fit a reasonable model to Danish mortality data and plot results
mAPa <- LCa.fit( mdk, model="APa", npar=c(15,15,20,6,6), c.ref=1930,
                 a.ref=70, quiet=FALSE, maxit=250 )
par( mfrow=c(1,3) )
plot( mAPa ) }
}
\keyword{regression}
\keyword{models}

\name{Ns}
\alias{Ns}
\title{
  Natural splines - (cubic splines linear beyond outermost knots) with
  convenient specification of knots and possibility of centering,
  detrending and clamping.
}
\description{
  This function is partly for convenient specification of natural splines
  in practical modeling. The convention used is to take the smallest
  and the largest of the supplied knots as boundary knots. It also has
  the option of centering the effects provided at a chosen reference
  point as well as projecting the columns on the orthogonal space to
  that spanned by the intercept and the linear effect of the variable,
  and finally fixing slopes beyond boundary knots (clamping).
}
\usage{
Ns( x, ref = NULL, df = NULL,
                knots = NULL,
            intercept = FALSE,
       Boundary.knots = NULL,
                fixsl = c(FALSE,FALSE),
              detrend = FALSE )
}
\arguments{
  \item{x}{A variable.}
  \item{ref}{Scalar. Reference point on the \code{x}-scale, where the
    resulting effect will be 0.}
  \item{df}{degrees of freedom.}
  \item{knots}{knots to be used both as boundary and internal knots. If
    \code{Boundary.knots} are given, this will be taken as the set of
    internal knots.}
  \item{intercept}{Should the intercept be included in the resulting
    basis? Ignored if any of \code{ref} or \code{detrend} is given.}
  \item{Boundary.knots}{The boundary knots beyond which the spline is
    linear. Defaults to the minimum and maximum of \code{knots}.}
  \item{fixsl}{Specification of whether slopes beyond outer knots should
    be fixed to 0. \code{FALSE} correponds to no restriction; a curve with 0
    slope beyond the upper knot is obtained using
    \code{c(FALSE,TRUE)}. Ignored if \code{!(detrend==FALSE)}.}
  \item{detrend}{If \code{TRUE}, the columns of the spline basis will be
    projected to the orthogonal of \code{cbind(1,x)}. Optionally
    \code{detrend} can be given as a vector of non-negative numbers og
    length \code{length(x)}, used
    to define an inner product as \code{diag(detrend)} for projection on
    the orthogonal to \code{cbind(1,x)}. The default is projection
    w.r.t. the inner product defined by the identity matrix.}
}
\value{
  A matrix of dimension c(length(x),df) where either \code{df} was
  supplied or if \code{knots} were supplied, \code{df = length(knots) -
    1 + intercept}. \code{Ns} returns a spline basis which is centered at
  \code{ref}. \code{Ns} with the argument \code{detrend=TRUE} returns a
  spline basis which is orthogonal to \code{cbind(1,x)} with respect to
  the inner product defined by the positive definite matrix
  \code{diag(detrend)} (an assumption which is checked). Note the latter
  is data dependent and therefore making predictions
  with a \code{newdata} argument will be senseless.
}
\author{
  Bendix Carstensen \email{b@bxc.dk},
  Lars Jorge D\'iaz, Steno Diabetes Center Copenhagen.
}
\note{
  The need for this function is primarily from analysis of rates in
  epidemiology and demography, where the dataset are time-split records
  of follow-up, and the range of data therefore rarely is of any
  interest (let alone meaningful).

  In Poisson modeling of rates based on time-split records one should
  aim at having the same number of \emph{events} between knots, rather
  than the same number of observations. 
}
\examples{
require(splines)
require(stats)
require(graphics)

ns( women$height, df = 3)
Ns( women$height, knots=c(63,59,71,67) )

# Gives the same results as ns:
summary( lm(weight ~ ns(height, df = 3), data = women) )
summary( lm(weight ~ Ns(height, df = 3), data = women) )

# Get the diabetes data and set up as Lexis object
data(DMlate)
DMlate <- DMlate[sample(1:nrow(DMlate),500),]
dml <- Lexis( entry = list(Per=dodm, Age=dodm-dobth, DMdur=0 ),
               exit = list(Per=dox),
        exit.status = factor(!is.na(dodth),labels=c("DM","Dead")),
               data = DMlate )

# Split follow-up in 1-year age intervals
dms <- splitLexis( dml, time.scale="Age", breaks=0:100 )
summary( dms )

# Model  age-specific rates using Ns with 6 knots
# and period-specific RRs around 2000 with 4 knots
# with the same number of deaths between each pair of knots
n.kn <- 6
( a.kn <- with( subset(dms,lex.Xst=="Dead"),
                quantile( Age+lex.dur, probs=(1:n.kn-0.5)/n.kn ) ) )
n.kn <- 4
( p.kn <- with( subset( dms, lex.Xst=="Dead" ),
                quantile( Per+lex.dur, probs=(1:n.kn-0.5)/n.kn ) ) )
m1 <- glm( lex.Xst=="Dead" ~ Ns( Age, kn=a.kn ) +
                             Ns( Per, kn=p.kn, ref=2000 ),
           offset = log( lex.dur ),
           family = poisson,
             data = dms )

# Plot estimated age-mortality curve for the year 2005 and knots chosen:
nd <- data.frame( Age=seq(40,100,0.1), Per=2005, lex.dur=1000 )
par( mfrow=c(1,2) )
matplot( nd$Age, ci.pred( m1, newdata=nd ),
         type="l", lwd=c(3,1,1), lty=1, col="black", log="y",
         ylab="Mortality rates per 1000 PY", xlab="Age (years)", las=1, ylim=c(1,1000) )
rug( a.kn, lwd=2 )

# Clamped Age effect to the right of rightmost knot.
m1.c <- glm( lex.Xst=="Dead" ~ Ns( Age, kn=a.kn, fixsl=c(FALSE,TRUE) ) +
                               Ns( Per, kn=p.kn, ref=2000 ),
             offset = log( lex.dur ),
             family = poisson,
               data = dms )

# Plot estimated age-mortality curve for the year 2005 and knots chosen.
matplot( nd$Age, ci.pred( m1.c, newdata=nd ),
         type="l", lwd=c(3,1,1), lty=1, col="black", log="y",
         ylab="Mortality rates per 1000 PY", xlab="Age (years)", las=1, ylim=c(1,1000) )
rug( a.kn, lwd=2 )

par( mfrow=c(1,1) )

# Including a linear Age effect of 0.05 to the right of rightmost knot.
m1.l <- glm( lex.Xst=="Dead" ~ Ns( Age, kn=a.kn, fixsl=c(FALSE,TRUE) ) +
                               Ns( Per, kn=p.kn, ref=2000 ),
             offset = log( lex.dur ) + pmax( Age, max( a.kn ) ) * 0.05,
             family = poisson,
               data = dms )

# Plot estimated age-mortality curve for the year 2005 and knots chosen.
nd <- data.frame(Age=40:100,Per=2005,lex.dur=1000)
matplot( nd$Age, ci.pred( m1.l, newdata=nd ),
         type="l", lwd=c(3,1,1), lty=1, col="black", log="y",
         ylab="Mortality rates per 1000 PY", xlab="Age (years)", las=1, ylim=c(1,1000) )
rug( a.kn, lwd=2 )
}
\keyword{regression}

\name{adtte2Lexis {Epi}}
\alias{adtte2Lexis}

\title{
Create a Lexis Object Given an ADAM ADTTE Dataset.
}

\description{
Takes (a subset of) an ADTTE ADaM-format dataset along with choices
about recurrent events and outputs a corresponding
Lexis-object. Time since first entry to each state is
included as a time scale in the Lexis-object.
}

\usage{
adtte2Lexis(dat,
            start.state,
            Tr.ev,
            Term.st)
}

\arguments{
\item{dat}{ A data.frame with data on a format complying with CDISC
  ADaM BDS standards, see
  \url{https://www.cdisc.org/standards/foundational/adam/adam-basic-data-structure-bds-time-event-tte-analyses-v1-0}.
  See details for necessary columns, and Examples for an example.}
\item{start.state}{A character string, used as the name of the common starting
  state of all individuals.}
\item{Tr.ev}{A vector containing a subset of the values taken by the
  EVNTDSC variable in ADTTE indicating possible states.}
\item{Term.st}{A user-specified text string containing the name of the unique censoring state.}
}

\details{
The ADTTE data.frame, \code{dat}, should contain the columns
\code{USUBJID, STARTDT, ADT, AGE, EVNTDESC} and \code{CNSR}.  All other
columns, including baseline values and FL-variables, will be
discarded. If they are necessary in later analyses, they will need to be
added back to the resulting Lexis-object manually.
}

\value{
A Lexis-object with possible \code{lex.Cst} and \code{lex.Xst} states
specified in \code{Tr.ev} and \code{Tr.st}, and with time-scales for
calender period, \code{per}, age, \code{age}, time since entry,
\code{t.sE} and additional time-scales for time since first entry into
the states given in \code{Tr.ev}.  Baseline information is not retained.
}

\references{
\url{https://www.cdisc.org/standards/foundational/adam/adam-basic-data-structure-bds-time-event-tte-analyses-v1-0}
}

\author{
Lars Diaz, \email{larsjorgeful@gmail.com}
}

\examples{
## Creating columns in ADTTE dataset:
STUDYID <- c("ABC123", "ABC123", "ABC123", "ABC123",
             "ABC123", "ABC123", "ABC123", "ABC123", "ABC123", "ABC123",
             "ABC123") 

USUBJID <- c("SUBJ001", "SUBJ001", "SUBJ001", "SUBJ002",
             "SUBJ002", "SUBJ002", "SUBJ002", "SUBJ002", "SUBJ002",
             "SUBJ002", "SUBJ002") 

AGE <- c(65L, 65L, 65L, 67L, 67L, 67L, 67L, 67L, 67L, 67L,
         67L) 

SEX <- c("M", "M", "M", "F", "F", "F", "F", "F", "F", "F", "F") 

ARM <- c("Treatment A", "Treatment A", "Treatment A",
         "Treatment A", "Treatment A", "Treatment A", "Treatment A",
         "Treatment A", "Treatment A", "Treatment A", "Treatment A") 

PARAMCD <- c("HOSP", "RELAPSE", "RELAPSE", "HOSP", "HOSP",
             "HOSP", "HOSP", "RELAPSE", "RELAPSE", "RELAPSE", "RELAPSE") 

PARAM <- c("Time to Hospitl (days)",
           "Time to Relapse (days)",
           "Time to Relapse (days)",
           "Time to Hospitl (days)",
           "Time to Hospitl (days)",
           "Time to Hospitl (days)",
           "Time to Hospitl (days)",
           "Time to Relapse (days)",
           "Time to Relapse (days)",
           "Time to Relapse (days)",
           "Time to Relapse (days)") 

EVNTSEQ <- c(1, 1, 2, 1, 2, 3, 4, 1, 2, 3, 4) 

STARTDT <- c("2023-01-01", "2023-01-01", "2023-01-31",
             "2023-01-03", "2023-02-13", "2023-05-10", "2023-06-29",
             "2023-01-03", "2023-03-18", "2023-04-26", "2023-06-18") 

ADT <- c("2023-07-20", "2023-01-31", "2023-07-20",
         "2023-02-13", "2023-05-10", "2023-06-29", "2023-07-22",
         "2023-03-18", "2023-04-26", "2023-06-18", "2023-07-22") 

ADTF <- c(NA_character_, NA_character_, NA_character_,
          NA_character_, NA_character_, NA_character_, NA_character_,
          NA_character_, NA_character_, NA_character_, NA_character_) 

AVAL <- c(200, 30, 170, 41, 86, 50, 23, 74, 39, 53, 34) 

CNSR <- c(1, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1) 

EVNTDESC <- c("Censored", "Relapse", "Censored",
              "Hospitl", "Hospitl", "Hospitl",
              "Censored", "Relapse", "Relapse", "Relapse", "Censored") 

ANL01FL <- c("Y", "Y", "Y", "Y", "Y", "Y", "Y", "Y", "Y",
             "Y", "Y") 

##collecting into example adtte dataset:
adtte <- data.frame(STUDYID, USUBJID, AGE, SEX, ARM, PARAMCD, PARAM, EVNTSEQ,
                    STARTDT, ADT, ADTF, AVAL, CNSR, EVNTDESC, ANL01FL)
head(adtte)

## transform to Lexis format
adtte2Lexis(adtte, 
            start.state = "Rand", 
            Tr.ev = c("Relapse", "Hospitl"),
            Term.st = "Censored") -> Lx
Lx
summary(Lx, timeScales = TRUE)
}
\keyword{manip}

\name{splitLexis}
\alias{splitLexis}
\title{Split follow-up time in a Lexis object}
\description{
  The \code{splitLexis} function divides each row of a \code{Lexis}
  object into disjoint follow-up intervals according to the supplied
  break points.
}
\usage{
splitLexis(lex, breaks, time.scale, tol=.Machine$double.eps^0.5)
}
\arguments{
  \item{lex}{an object of class \code{Lexis}}
  \item{breaks}{a vector of break points}
  \item{time.scale}{the name or number of the time scale to be split}
  \item{tol}{numeric value >= 0. Intervals shorter than this value are
    dropped}
}
\value{
  An object of class \code{Lexis} with multiple rows for each row of
  the argument \code{lex}. Each row of the new \code{Lexis} object
  contains the part of the follow-up interval that falls inside one of
  the time bands defined by the break points.

  The variables representing the various time scales, are appropriately
  updated in the new \code{Lexis} object. The entry and exit status
  variables are also updated according to the rule that the entry status
  is retained until the end of follow-up. All other variables are
  considered to represent variables that are constant in time, and so
  are replicated across all rows having the same id value.
}
\note{
  The \code{splitLexis()} function divides follow-up time into intervals
  using breakpoints that are common to all rows of the \code{Lexis} object.
  To split a \code{Lexis} object by break points that are unique to each
  row, use the \code{cut.Lexis} function.
}
\author{Martyn Plummer}
\examples{
# A small bogus cohort
xcoh <- structure( list( id = c("A", "B", "C"),
                      birth = c("14/07/1952", "01/04/1954", "10/06/1987"),
                      entry = c("04/08/1965", "08/09/1972", "23/12/1991"),
                       exit = c("27/06/1997", "23/05/1995", "24/07/1998"),
                       fail = c(1, 0, 1) ),
                     .Names = c("id", "birth", "entry", "exit", "fail"),
                  row.names = c("1", "2", "3"),
                      class = "data.frame" )

# Convert the character dates into numerical variables (fractional years)
xcoh$bt <- cal.yr( xcoh$birth, format="\%d/\%m/\%Y" )
xcoh$en <- cal.yr( xcoh$entry, format="\%d/\%m/\%Y" )
xcoh$ex <- cal.yr( xcoh$exit , format="\%d/\%m/\%Y" )

# See how it looks
xcoh

# Define as Lexis object with timescales calendar time and age
Lcoh <- Lexis( entry = list( per=en ),
                exit = list( per=ex, age=ex-bt ),
         exit.status = fail,
                data = xcoh )

# Default plot of follow-up
plot( Lcoh )

# With a grid and deaths as endpoints
plot( Lcoh, grid=0:10*10, col="black" )
points( Lcoh, pch=c(NA,16)[Lcoh$lex.Xst+1] )

# With a lot of bells and whistles:
plot( Lcoh, grid=0:20*5, col="black", xaxs="i", yaxs="i",
      xlim=c(1960,2010), ylim=c(0,50), lwd=3, las=1 )
points( Lcoh, pch=c(NA,16)[Lcoh$lex.Xst+1], col="red", cex=1.5 )

# Split time along two time-axes
( x2 <- splitLexis( Lcoh, breaks = seq(1900,2000,5), time.scale="per") )
( x2 <- splitLexis( x2, breaks = seq(0,80,5), time.scale="age" ) )
str( x2 )

# Tabulate the cases and the person-years
summary( x2 )
tapply( status(x2,"exit")==1, list( timeBand(x2,"age","left"),
                                    timeBand(x2,"per","left") ), sum )
tapply( dur(x2),  list( timeBand(x2,"age","left"),
                        timeBand(x2,"per","left") ), sum )
}
\seealso{
  \code{\link{timeBand}},
  \code{\link{cutLexis}},
  \code{\link{mcutLexis}},
  \code{\link{summary.Lexis}}}
\keyword{manip}

\docType{package}
\name{Epi}
\alias{Epi}
\alias{Epi-package}
\title{Epi: Functions for manipulation and statistical analysis of epidemiological data}
\description{
\pkg{Epi} has grown out of the course 'Statistical Practise in
Epidemiology with R' \url{http://bendixcarstensen.com/SPE/}.

The major contributions from this course have been the
\code{\link{stat.table}} function for advanced tabulation and summary,
and the functions for representation and the \code{\link{Lexis}}
function(s) for manipulation of multistate data with multiple time
scales.
} 
\details{Click on the \code{Index} link below the line to access
  vignettes (tutorial documents) and an alphabetic list of the functions
in \code{Epi}.}

\name{DMconv}
\alias{DMconv}
\docType{data}
\title{Conversion to diabetes}
\description{
  Data from a randomized intervention study ("Addition") where persons with
  prediabetic conditions are followed up for conversion to diabetes (DM).
  Conversion dates are interval censored.
  Original data are not published yet, so id-numbers have been changed and
  all dates have been randomly perturbed.
  }
\usage{data(DMconv)}
 \format{
  A data frame with 1519 observations on the following 6 variables.
  \describe{
    \item{\code{id}}{Person identifier}
    \item{\code{doe}}{Date of entry, i.e. first visit.}
    \item{\code{dlw}}{Date last seen well, i.e. last visit without DM.}
    \item{\code{dfi}}{Date first seen ill, i.e. first visit with DM.}
    \item{\code{gtol}}{Glucose tolerance. Factor with levels:
        1="IFG" (impaired fasting glucose), 2="IGT" (impaired glucose tolerance).}
    \item{\code{grp}}{Randomization. Factor with levels:
        1="Intervention", 2="Control".}
  }
}
\source{
  Signe Saetre Rasmussen, Steno Diabetes Center. The Addition Study.
}
\examples{
data(DMconv)
str(DMconv)
head(DMconv)
}
\keyword{datasets}

\name{rateplot}
\alias{rateplot}
\alias{Aplot}
\alias{Pplot}
\alias{Cplot}
\title{
  Functions to plot rates from a table classified by age and
  calendar time (period)
}
\description{  
  Produces plots of rates versus age, connected within period or cohort
  (\code{Aplot}), rates versus period connected within age-groups
  (\code{Pplot}) and rates and rates versus date of birth cohort
  (\code{Cplot}). \code{rateplot} is a wrapper for these, allowing
  to produce the four classical displays with a single call. 
}
\usage{
rateplot( rates,
          which = c("ap","ac","pa","ca"),
            age = as.numeric( dimnames( rates )[[1]] ),
            per = as.numeric( dimnames( rates )[[2]] ),
           grid = FALSE,
         a.grid = grid,
         p.grid = grid,
         c.grid = grid,
          ygrid = grid,
       col.grid = gray( 0.9 ),
          a.lim = range( age, na.rm=TRUE ) + c(0,diff( range( age ) )/30),
          p.lim = range( per, na.rm=TRUE ) + c(0,diff( range( age ) )/30),
          c.lim = NULL, 
           ylim = range( rates[rates>0], na.rm=TRUE ),
             at = NULL,
         labels = paste( at ),
          a.lab = "Age at diagnosis",
          p.lab = "Date of diagnosis",
          c.lab = "Date of birth",
           ylab = "Rates",
           type = "l",
            lwd = 2,
            lty = 1,
         log.ax = "y",
            las = 1,
            ann = FALSE,
          a.ann = ann,
          p.ann = ann,
          c.ann = ann,
          xannx = 1/20,
        cex.ann = 0.8,
         a.thin = seq( 1, length( age ), 2 ),
         p.thin = seq( 1, length( per ), 2 ),
         c.thin = seq( 2, length( age ) + length( per ) - 1, 2 ),
            col = par( "fg" ),
          a.col = col,
          p.col = col,
          c.col = col,
            ... )

Aplot( rates, age = as.numeric( dimnames( rates )[[1]] ),
       per = as.numeric( dimnames( rates )[[2]] ), grid = FALSE,
       a.grid = grid, ygrid = grid, col.grid = gray( 0.9 ),
       a.lim = range( age, na.rm=TRUE ), ylim = range( rates[rates>0], na.rm=TRUE ),
       at = NULL, labels = paste( at ), a.lab = names( dimnames( rates ) )[1],
       ylab = deparse( substitute( rates ) ), type = "l", lwd = 2, lty = 1,
       col = par( "fg" ), log.ax = "y", las = 1, c.col = col, p.col = col,
       c.ann = FALSE, p.ann = FALSE, xannx = 1/20, cex.ann = 0.8,
       c.thin = seq( 2, length( age ) + length( per ) - 1, 2 ),
       p.thin = seq( 1, length( per ), 2 ), p.lines = TRUE,
       c.lines = !p.lines, ... )

Pplot( rates, age = as.numeric( dimnames( rates )[[1]] ),
       per = as.numeric( dimnames( rates )[[2]] ), grid = FALSE,
       p.grid = grid, ygrid = grid, col.grid = gray( 0.9 ),
       p.lim = range( per, na.rm=TRUE ) + c(0,diff(range(per))/30),
       ylim = range( rates[rates>0], na.rm=TRUE ), p.lab = names( dimnames( rates ) )[2],
       ylab = deparse( substitute( rates ) ), at = NULL, labels = paste( at ),
       type = "l", lwd = 2, lty = 1, col = par( "fg" ), log.ax = "y",
       las = 1, ann = FALSE, cex.ann = 0.8, xannx = 1/20,
       a.thin = seq( 1, length( age ), 2 ), ... )

Cplot( rates, age = as.numeric( rownames( rates ) ),
       per = as.numeric( colnames( rates ) ), grid = FALSE,
       c.grid = grid, ygrid = grid, col.grid = gray( 0.9 ),
       c.lim = NULL, ylim = range( rates[rates>0], na.rm=TRUE ),
       at = NULL, labels = paste( at ), c.lab = names( dimnames( rates ) )[2],
       ylab = deparse( substitute( rates ) ), type = "l", lwd = 2, lty = 1,
       col = par( "fg" ), log.ax = "y", las = 1, xannx = 1/20, ann = FALSE,
       cex.ann = 0.8, a.thin = seq( 1, length( age ), 2 ), ...  )

}
\arguments{
  \item{rates}{A two-dimensional table (or array) with rates to be plotted. It is
    assumed that the first dimension is age and the second is period.}
  \item{which}{A character vector with elements from
      \code{c("ap","ac","apc","pa","ca")}, indication which plots should
      be produced. One plot per element is produced. The first letter
      indicates the x-axis of the plot, the remaining which groups
      should be connected, i.e. \code{"pa"} will plot rates versus
      period and connect age-classes, and \code{"apc"} will plot rates
      versus age, and connect both periods and cohorts.}
  \item{age}{Numerical vector giving the means of the
    age-classes. Defaults to the rownames of \code{rates} as numeric.}
  \item{per}{Numerical vector giving the means of the periods. Defaults
    to the columnnames of \code{rates} as numeric.}
  \item{grid}{Logical indicating whether a background grid should be drawn.}
  \item{a.grid}{Logical indicating whether a background grid on the
      age-axis should be drawn. If numerical it indicates the
      age-coordinates of the grid.}
  \item{p.grid}{do. for the period.}
  \item{c.grid}{do. for the cohort.}
  \item{ygrid}{do. for the rate-dimension.}
  \item{col.grid}{The colour of the grid.}
  \item{a.lim}{Range for the age-axis.}
  \item{p.lim}{Range for the period-axis.}
  \item{c.lim}{Range for the cohort-axis.}
  \item{ylim}{Range for the y-axis (rates).}
  \item{at}{Position of labels on the y-axis (rates).}
  \item{labels}{Labels to put on the y-axis (rates).}
  \item{a.lab}{Text on the age-axis. Defaults to "Age".}
  \item{p.lab}{Text on the period-axis. Defaults to "Date of diagnosis".}
  \item{c.lab}{Text on the cohort-axis. Defaults to "Date of birth".}
  \item{ylab}{Text on the rate-axis. Defaults to the name of the rate-table.}
  \item{type}{How should the curves be plotted. Defaults to \code{"l"}.}
  \item{lwd}{Width of the lines. Defaults to 2.}
  \item{lty}{Which type of lines should be used. Defaults to 1, a solid line.}
  \item{log.ax}{Character with letters from \code{"apcyr"}, indicating
    which axes should be logarithmic. \code{"y"} and \code{"r"} both
    refer to the rate scale. Defaults to \code{"y"}.} 
  \item{las}{see \code{par}.}
  \item{ann}{Should the curves be annotated?}
  \item{a.ann}{Logical indicating whether age-curves should be annotated.}
  \item{p.ann}{do. for period-curves.}
  \item{c.ann}{do. for cohort-curves.}
  \item{xannx}{The fraction that the x-axis is expanded when curves are annotated.}
  \item{cex.ann}{Expansion factor for characters annotating curves.}
  \item{a.thin}{Vector of integers indicating which of the age-classes
    should be labelled.}
  \item{p.thin}{do. for the periods.}
  \item{c.thin}{do. for the cohorts.}
  \item{col}{Colours for the curves.}
  \item{a.col}{Colours for the age-curves.}
  \item{p.col}{do. for the period-curves.}
  \item{c.col}{do. for the cohort-curves.}
  \item{p.lines}{Should rates from the same period be connected?}
  \item{c.lines}{Should rates from the same cohort be connected?}
  \item{...}{Additional arguments pssed on to \code{matlines} when
    plotting the curves.}
}
\details{
Zero values of the rates are ignored. They are neiter in the plot nor in
the calculation of the axis ranges.
}  
\value{
  \code{NULL}. The function is used for its side-effect, the plot.
}
\author{
  Bendix Carstensen, Steno Diabetes Center,
  \url{http://bendixcarstensen.com}
}
\seealso{
  \code{\link{apc.frame}}
}
\examples{
data( blcaIT )
attach(blcaIT)

# Table of rates:
bl.rate <- tapply( D, list(age,period), sum ) /
           tapply( Y, list(age,period), sum )
bl.rate

# The four classical plots:
par( mfrow=c(2,2) )
rateplot( bl.rate*10^6 )

# The labels on the vertical axis could be nicer:
rateplot( bl.rate*10^6, at=10^(-1:3), labels=c(0.1,1,10,100,1000) ) 

# More bells an whistles
par( mfrow=c(1,3), mar=c(3,3,1,1), oma=c(0,3,0,0), mgp=c(3,1,0)/1.6 )
rateplot( bl.rate*10^6, ylab="", ann=TRUE, which=c("AC","PA","CA"),
                      at=10^(-1:3), labels=c(0.1,1,10,100,1000),
                      col=topo.colors(11), cex.ann=1.2 ) 
}
\keyword{hplot}

\name{in.span}
\alias{in.span}
\alias{inSpan}
\alias{id.span}
\alias{idSpan}
\alias{thinCol}
\title{
Is \code{x} in the column span of matrix \code{A} and what columns are
linearly dependent?
}
\description{
The function \code{in.span} checks if the vector \code{x} (or
columns of the matrix \code{x}) is in the column span of the matrix
\code{A}. If desired, it returns the coefficient matrix \code{B} so that
\code{AB=x}. The function \code{thinCol} removes linearly dependent
columns an returns a matrix of full rank.
}
\usage{
in.span( A,
         x,
      coef = FALSE,
       tol = 1e-08 )
inSpan( A, x, coef=FALSE, tol=1e-08 )
id.span( A, B, tol=1e-08 )
idSpan( A, B, tol=1e-08 )
thinCol( A, tol = 1e-06, col.num = FALSE )
}
\details{\code{\link{thinCol}} is mainly a workhorse in
  \code{\link{detrend}}, but made available because of its general
  usefulness. 

  \code{in.span} and \code{inSpan} are just different names for the same
  to accommodate different naming schools.

  \code{in.span} (\code{inSpan}) is handy in checking whether different
  parametrizations of a model are identical in the sense of spanning the
  same linear space. Equivalent to checking whether fitted values under
  different parametrizations are identical, but has the further use of
  checking if subspaces of models are equivalent. The function
  simply checks if the regression of (columns of) \code{x} on the
  columns of \code{A} produces residuals that are all 0.

  \code{id.span} (equivalent to \code{idSpan}) checks whether two
  matrices have the same column span.  
}
\arguments{
\item{A}{A matrix.}
\item{B}{A matrix.}
\item{x}{A vector or matrix. \code{length(x)} (or \code{nrow(x)}) must
  be equal to \code{nrow(A)}.} 
\item{coef}{Logical. Should the coefficient matrix (\code{k}) be
  returned, so that \code{Ak=x}?}
\item{tol}{Tolerance for identity of matrices in check
  (\code{in.span}) or QR decomposition (\code{thinCol})}
\item{col.num}{Logical. Should the positions of dependent columns be
  returned instead of the full-rank matrix?}
}
\value{\code{in.span} returns a logical: is \code{x} is in
  \code{span(A)}? If \code{coef=TRUE} it returns a matrix \code{k} so
  that \code{Ak=x}. \code{k} is not necessarily unique (A may not have
  full rank).

  \code{id.span} returns a logical: is \code{span(A)} the same as
  \code{span(B)}? 

  \code{thinCol} returns a matrix of full rank, formed from \code{A} by
  deleting columns linearly dependent on other. If \code{col.num=TRUE}
  (one possible set of) positions of columns forming a full rank basis
  for the column space of \code{A} is returned.
}
\author{Bendix Carstensen, \url{http://bendixcarstensen.com} with essential
  help from Lars Jorge Diaz and Peter Dalgaard.
}
\seealso{\code{\link{det}}
}
\examples{
# Matrices and vectors, x in span(A), z (hopefully) not
A <- matrix(round(rnorm(15)*20),5,3)
B <- matrix(round(rnorm(15)*20),5,3)
B <- cbind( B, B\%*\%c(3,4,2) )
x <- A \%*\% c(3,4,2)
z <- 5:9

# how they look
data.frame( A=A, x=x, z=z, B=B )

# vectors in span(A)?
in.span(A,x)
in.span(x,A)
in.span(A,x,coef=TRUE)

in.span(A,z)
in.span(A,z,coef=TRUE)

# Do matrices span the same space ?
in.span( A, B )
in.span( B, A )

# B is not in span of a subspace of B columns, but vice versa
( M <- matrix( rnorm(8)*7, 4, 2 ) )
in.span( B\%*\%M, B )
in.span( B, B\%*\%M )
id.span( B, B\%*\%M )

# But not unique for singular matrices:
( xx <- in.span( B, B\%*\%M, coef=TRUE ) )
cbind( B\%*\%M, B\%*\%xx )
cbind( xx, M )

# Easier for full rank matrices:
( K <- matrix( rnorm(9)*7, 3, 3 ) )
in.span( A\%*\%K, A )
in.span( A, A\%*\%K )
id.span( A, A\%*\%K )
in.span( A, A\%*\%K, coef=TRUE )
}
\keyword{math}

\name{apc.fit}
\alias{apc.fit}
\title{
  Fit an Age-Period-Cohort model to tabular data.
  }
\description{
  Fits the classical five models to tabulated rate data (cases,
  person-years) classified by two of age, period, cohort:
  Age, Age-drift, Age-Period, Age-Cohort and Age-Period-Cohort. There are no
  assumptions about the age, period or cohort classes being of the same
  length, or that tabulation should be only by two of the variables.
  Only requires that mean age and period for each tabulation unit is given.
  }
\usage{
apc.fit( data,
            A,
            P,
            D,
            Y,
        ref.c,
        ref.p,
          dist = c("poisson","binomial"),
         model = c("ns","bs","ls","factor"),
       dr.extr = "Y",
          parm = c("ACP","APC","AdCP","AdPC","Ad-P-C","Ad-C-P","AC-P","AP-C"),
          npar = c( A=5, P=5, C=5 ),
         scale = 1,
         alpha = 0.05,
     print.AOV = TRUE )
  }
\arguments{
  \item{data}{Data frame with (at least) variables, \code{A} (age),
    \code{P} (period), \code{D} (cases, deaths) and \code{Y}
    (person-years). Cohort (date of birth) is computed as \code{P-A}.
    If this argument is given the arguments \code{A}, \code{P},
    \code{D} and \code{Y} are ignored.}
  \item{A}{Age; numerical vector with mean age at diagnosis for each unit.}
  \item{P}{Period; numerical vector with mean date of diagnosis for each
            unit.}
  \item{D}{Cases, deaths; numerical vector.}
  \item{Y}{Person-years; numerical vector. Also used as denominator for binomial
           data, see the \code{dist} argument.}
  \item{ref.c}{Reference cohort, numerical. Defaults to median date of
    birth among cases. If used with \code{parm="AdCP"} or \code{parm="AdPC"},
    the residual cohort effects will be 1 at \code{ref.c}}
  \item{ref.p}{Reference period, numerical. Defaults to median date of
    diagnosis among cases.}
  \item{dist}{Distribution (or more precisely: Likelihood) used for modeling.
              if a binomial model us used, \code{Y} is assumed to be the
              denominator; \code{"binomial"} gives a binomial model with logit
              link. The Age-effects returned are converted to the
	      probability scale, Period and Cohort effects are still odds-ratios.}
  \item{model}{Type of model (covariate effects) fitted:
    \itemize{
	    \item \code{ns} fits a model with natural splines for each of
	      the terms, with \code{npar} parameters for the terms.
      \item \code{bs} fits a model with B-splines for each of
	      the terms, with \code{npar} parameters for the terms.
	    \item \code{ls} fits a model with linear splines.
      \item \code{factor} fits a factor model with one parameter
	      per value of \code{A}, \code{P} and \code{P-A}. \code{npar}
	      is ignored in this case. 
           }
      }
  \item{dr.extr}{Character or numeric.	
    How the drift parameter should be extracted from the
    age-period-cohort model. Specifies the inner product used for
    definition of orthogonality of the period / cohort effects to the
    linear effects --- in terms of a diagonal matrix.

    \code{"Y"} (default) uses the no. person-time, \code{Y},
    corresponding to the observed information about the square root of
    the rate.

    \code{"R"} or \code{"L"} uses \code{Y*Y/D} corresponding to the
    observed information about the rate (usually termed "lambda", hence
    the "\code{L}"). 

    \code{"D"} or \code{"T"} uses the no. events as the weight in the
    inner product, corresponding to the information about the log-rate
    (usually termed "theta", hence the "\code{T}").
    
    If given \code{"n"} (naive) (well, in fact any other character value) will
    induce the use of the standard inner product putting equal weight on
    all units in the dataset.

    If \code{dr.extr} is a numeric vector this is used as the diagonal
    of the matrix inducing the inner product. 

    If \code{dr.extr} is a numeric scalar, \code{D + dr.extr*Y} is used
    as the diagonal of the matrix inducing the inner product. This
    family of inner products are the only ones that meet the
    split-observation invariance criterion.
    
    The setting of this parameter has no effect on the fit of the model,
    it only influences the parametrization returned in the \code{Age},
    \code{Per} and \code{Coh} elements of the resulting list.
  }
  \item{parm}{Character. Indicates the parametrization of the effects.
    The first four refer to the ML-fit of the Age-Period-Cohort model,
    the last four give Age-effects from a smaller model and residuals
    relative to this. If one of the latter is chosen, the argument
    \code{dr.extr} is ignored. Possible values for \code{parm} are:
    \itemize{
      \item \code{"ACP"}: ML-estimates. Age-effects as rates for the
      reference cohort. Cohort effects as RR relative to the reference
      cohort. Period effects constrained to be 0 on average with 0 slope.
      \item \code{"APC"}: ML-estimates. Age-effects as rates for the
      reference period. Period effects as RR relative to the reference
      period. Cohort effects constrained to be 0 on average with 0 slope.
      \item \code{"AdCP"}: ML-estimates. Age-effects as rates for the
      reference cohort. Cohort and period effects constrained to be 0 on
      average with 0 slope. In this case returned effects do not
      multiply to the fitted rates, the drift is missing and needs to be
      included to produce the fitted values.
      \item \code{"AdPC"}: ML-estimates. Age-effects as rates for the
      reference period. Cohort and period effects constrained to be 0 on
      average with 0 slope. In this case returned effects do not
      multiply to the fitted rates, the drift is missing and needs to be
      included to produce the fitted values.
      \item \code{"Ad-C-P"}: Age effects are rates for the reference
      cohort in the Age-drift model (cohort drift). Cohort effects are from the model
      with cohort alone, using log(fitted values) from the Age-drift
      model as offset. Period effects are from the model with period
      alone using log(fitted values) from the cohort model as offset.
      \item \code{"Ad-P-C"}: Age effects are rates for the reference
      period in the Age-drift model (period drift). Period effects are from the model
      with period alone, using log(fitted values) from the Age-drift
      model as offset. Cohort effects are from the model with cohort
      alone using log(fitted values) from the period model as offset.
      \item \code{"AC-P"}: Age effects are rates for the reference
      cohort in the Age-Cohort model, cohort effects are RR relative to
      the reference cohort. Period effects are from the model
      with period alone, using log(fitted values) from the Age-Cohort
      model as offset.
      \item \code{"AP-C"}: Age effects are rates for the reference
      period in the Age-Period model, period effects are RR relative to
      the reference period. Cohort effects are from the model
      with cohort alone, using log(fitted values) from the Age-Period
      model as offset.
    } }
  \item{npar}{The number of parameters/knots to use for each of the terms in
    the model. If it is vector of length 3, the numbers are taken as the
    no. of knots for Age, Period and Cohort, respectively. Unless it has
    a names attribute with values "A", "P" and "C" in which case these
    will be used. The knots chosen are the quantiles
    \code{(1:nk-0.5)/nk} of the events (i.e. of \code{rep(A,D)} and
    similarly for \code{P} and \code{C}).

    \code{npar} may also be a named list of three numerical vectors with
    names "A", "P" and "C", in which case these taken as the knots for
    the age, period and cohort effect, the smallest and largest element in
    each vector are used as the boundary knots.}
  \item{alpha}{The significance level. Estimates are given with
    (1-\code{alpha}) confidence limits.}
  \item{scale}{numeric(1), factor multiplied to the rate estimates before output.}
  \item{print.AOV}{Should the analysis of deviance table for the models
    be printed?}
  }
\value{
  An object of class "apc" (recognized by \code{\link{apc.plot}} and
  \code{\link{apc.lines}}) --- a list with components:
  \item{Type}{Text describing the model and parametrization returned.}
  \item{Model}{The model object(s) on which the parametrization is based.}
  \item{Age}{Matrix with 4 columns: \code{A.pt} with the ages (equals
    \code{unique(A)}) and three columns giving the estimated rates with
    c.i.s.}
  \item{Per}{Matrix with 4 columns: \code{P.pt} with the dates of
    diagnosis (equals \code{unique(P)}) and three columns giving the
    estimated RRs with c.i.s.}
  \item{Coh}{Matrix with 4 columns: \code{C.pt} with the dates of birth
    (equals \code{unique(P-A)}) and three columns giving the estimated
    RRs with c.i.s.}
  \item{Drift}{A 3 column matrix with drift-estimates and c.i.s: The
    first row is the ML-estimate of the drift (as defined by
    \code{drift}), the second row is the estimate from the Age-drift
    model. The first row name indicates which type of inner product were
    used for projections. For the sequential parametrizations, only the
    latter is given.}
  \item{Ref}{Numerical vector of length 2 with reference period and cohort.
             If ref.p or ref.c was not supplied the corresponding element is NA.}
  \item{Anova}{Analysis of deviance table comparing the five classical
    models.}
  \item{Knots}{If \code{model} is one of \code{"ns"} or \code{"bs"}, a list
    with three components: \code{Age}, \code{Per}, \code{Coh}, each one a
    vector of knots. The max and the min of the vectors are the boundary knots.}
}
\details{
   Each record in the input data frame represents a subset of a Lexis
   diagram. The subsets need not be of equal length on the age and
   period axes, in fact there are no restrictions on the shape of
   these; they could be Lexis triangles for example. The requirement is
   that \code{A} and \code{P} are coded with the mean age and calendar
   time of observation in the subset. This is essential since \code{A}
   and \code{P} are used as quantitative variables in the models.

   This approach is different from to the vast majority of the uses of
   APC-models in the literature where a factor model is used for age,
   period and cohort effects. The latter can be obtained by using
   \code{model="factor"}. Note however that the cohort factor is defined
   from \code{A} and \code{P}, so that it is not possible in this
   framework to replicate the Boyle-Robertson fallacy.
   }
\references{
   The considerations behind the parametrizations used in this function
   are given in detail in:
   B. Carstensen: Age-Period-Cohort models for the Lexis diagram.
   Statistics in Medicine, 10; 26(15):3018-45, 2007.

   Various links to course material etc. is available through
   \url{http://bendixcarstensen.com/APC/} 
   }
\author{
   Bendix Carstensen, \url{http://bendixcarstensen.com}
   }
\seealso{
   \code{\link{apc.frame}},
   \code{\link{apc.lines}},
   \code{\link{apc.plot}},
   \code{\link{LCa.fit}},
   \code{\link{apc.LCa}}.
   }
\examples{
library( Epi )
data(lungDK)

# Taylor a dataframe that meets the requirements for variable names
exd <- lungDK[,c("Ax","Px","D","Y")]
names(exd)[1:2] <- c("A","P")

# Three different ways of parametrizing the APC-model, ML
ex.1 <- apc.fit( exd, npar=7, model="ns", dr.extr="1", parm="ACP", scale=10^5 )
ex.D <- apc.fit( exd, npar=7, model="ns", dr.extr="D", parm="ACP", scale=10^5 )
ex.Y <- apc.fit( exd, npar=7, model="ns", dr.extr="Y", parm="ACP", scale=10^5 )

# Sequential fit, first AC, then P given AC.
ex.S <- apc.fit( exd, npar=7, model="ns", parm="AC-P", scale=10^5 )

# Show the estimated drifts
ex.1[["Drift"]]
ex.D[["Drift"]]
ex.Y[["Drift"]]
ex.S[["Drift"]]

# Plot the effects
lt <- c("solid","22")[c(1,1,2)]
apc.plot( ex.1, lty=c(1,1,3) )
apc.lines( ex.D, col="red", lty=c(1,1,3) )
apc.lines( ex.Y, col="limegreen", lty=c(1,1,3) )
apc.lines( ex.S, col="blue", lty=c(1,1,3) )
}
\keyword{models}
\keyword{regression}

\name{plotEst}
\alias{plotEst}
\alias{pointsEst}
\alias{linesEst}
\title{
  Plot estimates with confidence limits (forest plot)
}
\description{
  Plots parameter estimates with confidence intervals, annotated with
  parameter names. A dot is plotted at the estimate and a horizontal
  line extending from the lower to the upper limit is superimposed.
}
\usage{
plotEst( ests,
            y = dim(ests)[1]:1,
          txt = rownames(ests),
       txtpos = y,
         ylim = range(y)-c(0.5,0),
         xlab = "",
         xtic = nice(ests[!is.na(ests)], log = xlog),
         xlim = range( xtic ),
         xlog = FALSE,
          pch = 16,
          cex = 1,
          lwd = 2,
          col = "black",
      col.txt = "black",
     font.txt = 1,
    col.lines = col,
   col.points = col,
         vref = NULL,
         grid = FALSE,
     col.grid = gray(0.9),
  restore.par = TRUE,
          ... )

linesEst( ests, y = dim(ests)[1]:1, pch = 16, cex = 1, lwd = 2,
          col="black", col.lines=col, col.points=col, ... )

pointsEst( ests, y = dim(ests)[1]:1, pch = 16, cex = 1, lwd = 2,
          col="black", col.lines=col, col.points=col, ... )
}
\arguments{
  \item{ests}{Matrix with three columns: Estimate, lower limit, upper
    limit. If a model object is supplied, \code{\link{ci.lin}} is
    invoked for this object first.}
  \item{y}{Vertical position of the lines.}
  \item{txt}{Annotation of the estimates. Either a character vector or
          an expression vector.}
  \item{txtpos}{Vertical position of the text. Defaults to \code{y}.}
  \item{ylim}{Extent of the vertical axis.}
  \item{xlab}{Annotation of the horizontal axis.}
  \item{xtic}{Location of tickmarks on the x-axis.}
  \item{xlim}{Extent of the x-axis.}
  \item{xlog}{Should the x-axis be logarithmic?}
  \item{pch}{What symbol should be used?}
  \item{cex}{Expansion of the symbol.}
  \item{col}{Colour of the points and lines.}
  \item{col.txt}{Colour of the text annotating the estimates.}
  \item{font.txt}{Font for the text annotating the estimates.}
  \item{col.lines}{Colour of the lines.}
  \item{col.points}{Colour of the symbol.}
  \item{lwd}{Thickness of the lines.}
  \item{vref}{Where should vertical reference line(s) be drawn?}
  \item{grid}{If TRUE, vertical gridlines are drawn at the
    tickmarks. If a numerical vector is given vertical lines are drawn
    at \code{grid}.}
  \item{col.grid}{Colour of the vertical gridlines}
  \item{restore.par}{Should the graphics parameters be restored? If set
    to \code{FALSE} the coordinate system will still be available for
    additional plotting, and \code{par("mai")} will still have the very
    large value set in order to make room for the labelling of the
    estimates.}
  \item{...}{Arguments passed on to \code{ci.lin} when a model object is
             supplied as \code{ests}.}
}
\details{
  \code{plotEst} makes a news plot, whereas \code{linesEst} and
  \code{pointsEst} (identical functions) adds to an existing plot.

  If a model object of class \code{"glm"}, \code{"coxph"}, \code{"clogistic"} or
  \code{"gnlm"} is supplied the argument \code{xlog} defaults to
  \code{TRUE}, and exponentiated estimates are extracted by default.
}
\value{
  NULL
}
\author{
  Bendix Carstensen,
  \email{b@bxc.dk},
  \url{http://bendixcarstensen.com}}
\seealso{
  ci.lin
}
\examples{
# Bogus data and a linear model
f <- factor( sample( letters[1:5], 100, replace=TRUE ) )
x <- rnorm( 100 )
y <- 5 + 2 * as.integer( f ) + 0.8 * x + rnorm(100) * 2
m1 <- lm( y ~ f )

# Produce some confidence intervals for contrast to first level
( cf <- ci.lin( m1, subset=-1 )[,-(2:4)] )

# Plots with increasing amounts of bells and whistles
par( mfcol=c(3,2), mar=c(3,3,2,1) )
plotEst( cf )
plotEst( cf, grid=TRUE, cex=2, lwd=3 )
plotEst( cf, grid=TRUE, cex=2, col.points="red", col.lines="green" )
plotEst( cf, grid=TRUE, cex=2, col.points="red", col.lines="green",
             xlog=TRUE, xtic=c(1:8), xlim=c(0.8,6) )
rownames( cf )[1] <- "Contrast to fa:\n fb"
plotEst( cf, grid=TRUE, cex=2, col.points=rainbow(4),
                                col.lines=rainbow(4), vref=1 )
#
etxt <- expression("Plain text, qouted",
                   "combined with maths:"*sqrt(a)*phi[c],
                   f^d*"  Hb"*A[1][c],
                   eff^e*"  kg/"*m^2)
plotEst( cf, txt=etxt, grid=TRUE, cex=2, col.points=rainbow(4),
                                         col.lines =rainbow(4), vref=1 )
  }
\keyword{hplot}
\keyword{models}

\name{steno2}
\alias{steno2}
\alias{st2clin}
\alias{st2alb}
\docType{data}
\title{
Clinical trial: Steno2 baseline and follow-up. 
}
\description{
  
Steno-2 was a clinical trial conducted at Steno Diabetes Center
1993-2001. Intervention was intensified treatment versus conventional
treatment of diabetes patients with micro-albuminuria.
concern the extended follow-up of the trial population till 2015. Three
files are provided: \code{steno2} with one record per person,
\code{st2clin} with one record per clinical visit and \code{st2alb} with
one record per transition between states of albuminuria.

These dataset are simulated entirely simulated, but designed to give
approximately the same results as the original.
}
\usage{data("steno2")
       data("st2clin")
       data("st2alb")
}
\format{
  \code{steno2} is a data frame with 160 observations on the following 14
  variables: 
  \describe{
    \item{\code{id}}{person id, numeric}
    \item{\code{allo}}{Original trial allocation, a factor with levels
      \code{Int} \code{Conv}} 
    \item{\code{sex}}{Sex, a factor with levels \code{F} \code{M}}
    \item{\code{baseCVD}}{0/1 indicator of preexisting CVD at baseline}
    \item{\code{deathCVD}}{0/1 indicator whether cause of death was CVD}
    \item{\code{doBth}}{Date of birth, a Date}
    \item{\code{doDM}}{Date of diabetes diagnosis, a Date}
    \item{\code{doBase}}{Date of entry to study, a Date}
    \item{\code{doCVD1}}{Date of 1st CVD event, a Date}
    \item{\code{doCVD2}}{Date of 2nd CVD event, a Date}
    \item{\code{doCVD3}}{Date of 3rd CVD event, a Date}
    \item{\code{doESRD}}{Date of end stage renal disease, a Date}
    \item{\code{doEnd}}{Date of exit from follow-up, a Date}
    \item{\code{doDth}}{Date of death, a Date}
  }
  \code{st2clin} is data frame with 750 observations on clinical
  measurements at different clinical visits:
  \describe{
    \item{\code{id}}{person id, numeric}
    \item{\code{doV}}{Date of clinical visit, a Date}
    \item{\code{a1c}}{Glycosylated hemoglobin, mmol/mol}
    \item{\code{chol}}{Total cholesterol, mg/mol}
    \item{\code{crea}}{Creatinine, mg/mol}
  }
  \code{st2alb} is data frame with 307 observations of changes in
  complication (albuminuria) state
  \describe{
    \item{\code{id}}{person id, numeric}
    \item{\code{doTr}}{Date of transition, a Date}
    \item{\code{state}}{State of albuminuria, factor with levels
                        \code{Norm}, \code{Mic}, \code{Mac}. All persons
                        begin in the state \code{Mic}ro-albuminuria.} 
  }

}
\details{
  The data are not the original; all values of measurements and dates
  have been randomly perturbed, to prevent identifiability of
  individuals. Analysis of these data will give only (very)
  approximately the same results as in the published article, and only
  some of the aspects of data are included.
}
\references{
  P. Gaede, J. Oellgaard, B. Carstensen, P. Rossing, H. Lund-Andersen,
  H. H. Parving & O. Pedersen: Years of life gained by multifactorial
  intervention in patients with type 2 diabetes mellitus and
  microalbuminuria: 21 years follow-up on the Steno-2
  randomised trial. Diabetologia (2016), 59, pp 2298-2307
}
\examples{
data(steno2)
data(st2alb)
L2 <- Lexis( entry = list(per = doBase,
                          age = doBase - doBth),
              exit = list(per = doEnd),
       exit.status = factor(deathCVD + !is.na(doDth),
                            labels=c("Mic","D(oth)","D(CVD)")),
                id = id,
              data = cal.yr(steno2) )
summary(L2)
#
# Cut at intermediate transitions
cut2 <- data.frame(lex.id = st2alb$id,
                      cut = cal.yr(st2alb$do),
                new.state = st2alb$state)
L3 <- rcutLexis(L2, cut2)
summary(L3)
#
# no direct transitions Mic <-> Mac allowed, so put a cut in between:
dd <- subset(L3, (lex.Cst == "Mac" & lex.Xst =="Norm") |
                 (lex.Cst =="Norm" & lex.Xst == "Mac"))
# artificial visits to the middle state Mic: 
cut3 <- data.frame( lex.id = dd$lex.id,
                       cut = dd$per + dd$lex.dur/2,
                 new.state = "Mic")
L4 <- rcutLexis(L3, cut3)
summary(L4)
#
# Show all transitions
boxes(L4, boxpos = list(x = c(15,15,15,85,85),
                        y = c(50,15,85,25,75)),
          show.BE = TRUE, scale.R = 1000,
          cex=0.8, pos.arr=0.7, font=1, font.arr=1)
}
\keyword{datasets}

\name{ci.pd}
\alias{ci.pd}
\title{
  Compute confidence limits for a difference of two independent proportions.
}
\description{
  The usual formula for the c.i. of at difference of proportions is
  inaccurate. Newcombe has compared 11 methods and method 10 in his
  paper looks like a winner. It is implemented here.
}
\usage{
ci.pd(aa, bb=NULL, cc=NULL, dd=NULL,
     method = "Nc",
      alpha = 0.05, conf.level=0.95,
     digits = 3,
      print = TRUE,
detail.labs = FALSE )
}
\arguments{
  \item{aa}{Numeric vector of successes in sample 1. Can also be a
    matrix or array (see details).} 
  \item{bb}{Successes in sample 2.}
  \item{cc}{Failures in sample 1.}
  \item{dd}{Failures in sample 2.}
  \item{method}{Method to use for calculation of confidence interval, see
    "Details".}
  \item{alpha}{Significance level}
  \item{conf.level}{Confidence level}
  \item{print}{Should an account of the two by two table be printed.}
  \item{digits}{How many digits should the result be rounded to if printed.}
  \item{detail.labs}{Should the computing of probability differences be
                     reported in the labels.}
}
\details{
  Implements method 10 from Newcombe(1998) (method="Nc") or from
  Agresti & Caffo(2000) (method="AC").

  \code{aa}, \code{bb}, \code{cc} and \code{dd} can be vectors.
  If \code{aa} is a matrix, the elements \code{[1:2,1:2]} are used, with
  successes \code{aa[,1:2]}. If \code{aa} is a three-way table or array,
  the elements \code{aa[1:2,1:2,]} are used.
}
\value{
  A matrix with three columns: probability difference, lower and upper
  limit. The number of rows equals the length of the vectors  \code{aa},
  \code{bb}, \code{cc} and \code{dd} or, if \code{aa} is a 3-way matrix,
  \code{dim(aa)[3]}.   
}
\references{
  RG Newcombe: Interval estimation for the difference between
  independent proportions. Comparison of eleven methods. Statistics in
  Medicine, 17, pp. 873-890, 1998.
  
  A Agresti & B Caffo: Simple and effective confidence intervals for
  proportions and differences of proportions result from adding two
  successes and two failures. The American Statistician,
  54(4), pp. 280-288, 2000.
}
\author{
  Bendix Carstensen, Esa Laara.
  \url{http://bendixcarstensen.com}
}
\seealso{
  \code{\link{twoby2}}, \code{\link{binom.test}}
}
\examples{
( a <- matrix( sample( 10:40, 4 ), 2, 2 ) )
ci.pd( a )
twoby2( t(a) )
prop.test( t(a) )
( A <- array( sample( 10:40, 20 ), dim=c(2,2,5) ) )
ci.pd( A )
ci.pd( A, detail.labs=TRUE, digits=3 )
}
\keyword{distribution}
\keyword{htest}

\name{pc.lines}
\alias{pc.points}
\alias{pc.lines}
\alias{pc.matpoints}
\alias{pc.matlines}
\alias{pc.matshade}
\alias{cp.points}
\alias{cp.lines}
\alias{cp.matpoints}
\alias{cp.matlines}
\alias{cp.matshade}
\title{
  Plot period and cohort effects in an APC-frame.
}
\description{
  When an APC-frame has been produced by \code{\link{apc.frame}}, this
  function draws curves or points in the period/cohort part of the frame.
}
\usage{
  pc.points( x, y, ... )
  pc.lines( x, y, ... )
  pc.matpoints( x, y, ... )
  pc.matlines( x, y, ... )
  pc.matshade( x, y, ... )
  cp.points( x, y, ... )
  cp.lines( x, y, ... )
  cp.matpoints( x, y, ... )
  cp.matlines( x, y, ... )
  cp.matshade( x, y, ... )
}
\arguments{
  \item{x}{vector of \code{x}-coordinates.}
  \item{y}{vector or matrix of \code{y}-coordinates.}
  \item{...}{Further parameters to be transmitted to points, lines,
    matpoints, matlines or matshade used for plotting curves in the
    calendar time realm of a graph generated by \code{\link{apc.frame}}}
}
\details{Since the Age-part of the frame is referred to by its real
  coordinates plotting in the calendar time part requires translation
  and scaling to put things correctly there, that is done by the
  functions \code{pc.points} etc.

  The functions \code{cp.points} etc. are just synonyms for these, in
  recognition of the fact that you can never remember whether it is "pc"
  or "cp".
  }
\value{
  The functions return nothing.
  }
\author{
  Bendix Carstensen, Steno Diabetes Center Copenhagen,
  \url{http://bendixcarstensen.com}
  }
\seealso{
  \code{\link{apc.frame}}, \code{\link{apc.fit}}, \code{\link{plot.apc}}, \code{\link{lines.apc}}
}
\keyword{hplot}

\name{cutLexis}
\alias{cutLexis}
\alias{countLexis}
\title{
  Cut follow-up at a specified date for each person.
}
\description{

  Follow-up intervals in a Lexis object are divided into two
  sub-intervals: one before and one after an intermediate event.  The
  intermediate event may denote a change of state, in which case the
  entry and exit status variables in the split Lexis object are
  modified.

}
\usage{
cutLexis( data, cut, timescale = 1,
                     new.state = nlevels(data$lex.Cst)+1,
                     new.scale = FALSE,
                  split.states = FALSE,
                   progressive = FALSE,
              precursor.states = transient(data),
                         count = FALSE )
countLexis( data, cut, timescale = 1 )
}
\arguments{
  \item{data}{A \code{Lexis} object.}
  \item{cut}{A numeric vector with the times of the intermediate event.
    If a time is missing (\code{NA}) then the event is assumed to occur
    at time \code{Inf}. \code{cut} can also be a dataframe, see details.}
  \item{timescale}{The timescale that \code{cut} refers to. Numeric or character.}
  \item{new.state}{The state to which a transition occur at time
    \code{cut}. It may be a single value, which is then applied to all
    rows of \code{data}, or a vector with a separate value for each row}
  \item{new.scale}{Name of the timescale defined as "time since entry to
    new.state". If \code{TRUE} a name for the new scale is constructed.
    See details.}
  \item{split.states}{Should states that are not precursor states be split
    according to whether the intermediate event has occurred.}
  \item{progressive}{a logical flag that determines the changes to exit
    status. See details.}
  \item{precursor.states}{an optional vector of states to be considered
    as "less severe" than \code{new.state}. See Details below}
  \item{count}{logical indicating whether the \code{countLexis} options should
    be used. Specifying \code{count=TRUE} amounts to calling \code{countLexis},
    in which case the arguments \code{new.state}, \code{progressive} and
    \code{precursor.states} will be ignored. }
}
\value{
  A \code{Lexis} object, for which each follow-up interval containing
  the cutpoint is split in two: one before and one after the
  cutpoint. Any record representing follow up after the cutpoint has its
  value of \code{lex.Cst} updated to the new state. An extra time-scale
  is added; the time since the event at \code{cut}. This time scale will
  be \code{NA} for any follow-up prior to the intermediate event.

  The function \code{tsNA20} will replace all missing values in
  timescales with 0. This is commonly meeded when timescales defined as
  time since entry into an intermediate state are used in modeling. But
  you do not want to do that permanently in the cut data frame.
  }
\note{
  The \code{cutLexis} function superficially resembles the
  \code{splitLexis} function. However, the \code{splitLexis} function
  splits on a vector of common cut-points for all rows of the Lexis
  object, whereas the \code{cutLexis} function splits on a single time
  point, which may be distinct for each row, modifies the status
  variables, adds a new timescale and updates the attribute
  "time.since". This attribute is a character vector of the same length
  as the "time.scales" attribute, whose value is '""' if the
  corresponding timescale is defined for any piece of follow-up, and if
  the corresponding time scale is defined by say
  \code{cutLexis(obj,new.state="A",new.scale=TRUE)}, it has the value
  "A".
  }
\details{
  The \code{cutLexis} function allows a number of different ways
  of specifying the cutpoints and of modifying the status variable.

  If the \code{cut} argument is a dataframe it must have columns \code{lex.id},
  \code{cut} and \code{new.state}. The values of \code{lex.id} must be unique.
  In this case it is assumed that each row represents a cutpoint (on the
  timescale indicated in the argument \code{timescale}). This cutpoint will
  be applied to all records in \code{data} with the corresponding \code{lex.id}.
  This makes it possible to apply \code{cutLexis} to a split \code{Lexis} object.

  If a \code{new.state} argument is supplied, the status variable is
  only modified at the time of the cut point. However, it is often
  useful to modify the status variable after the cutpoint when an
  important event occurs. There are three distinct ways of doing this.

  If the \code{progressive=TRUE} argument is given, then a "progressive"
  model is assumed, in which the status can either remain the same or
  increase during follow-up, but never decrease. This assumes that the
  state variables \code{lex.Cst} and \code{lex.Xst} are either numeric or
  ordered factors. In this case, if
  \code{new.state=X}, then any exit status with a value less than
  \code{X} is replaced with \code{X}. The Lexis object
  must already be progressive, so that there are no rows for which the
  exit status is less than the entry status. If \code{lex.Cst} and
  \code{lex.Xst} are factors they must be ordered factors if
  \code{progressive=TRUE} is given.

  As an alternative to the \code{progressive} argument, an explicit
  vector of precursor states, that are considered less severe than the
  new state, may be given. If \code{new.state=X} and
  \code{precursor.states=c(Y,Z)} then any exit status of \code{Y} or
  \code{Z} in the second interval is replaced with \code{X} and all
  other values for the exit status are retained.

  The \code{countLexis} function is a variant of \code{cutLexis} when
  the cutpoint marks a recurrent event, and the status variable is used
  to count the number of events that have occurred. Times given in \code{cut}
  represent times of new events. Splitting with
  \code{countLexis} increases the status variable by 1. If the current
  status is \code{X} and the exit status is \code{Y} before cutting,
  then after cutting the entry status is \code{X}, \code{X+1} for
  the first and second intervals, respectively, and the exit status is
  \code{X+1}, \code{Y+1} respectively. Moreover the values of the status
  is increased by 1 for all intervals for all intervals after the cut
  for the person in question. Hence, a call to \code{countLexis} is
  needed for as many times as the person with most events. But also it
  is immaterial in what order the cutpoints are entered.
}
\author{Bendix Carstensen, Steno Diabetes Center, \email{b@bxc.dk},
  Martyn Plummer, \email{martyn.plummer@r-project.org}
}
\seealso{
  \code{\link{mcutLexis}},
  \code{\link{rcutLexis}},
  \code{\link{xcutLexis}},
  \code{\link{addCov.Lexis}},
  \code{\link{addDrug.Lexis}},
  \code{\link{splitLexis}},
  \code{\link{Lexis}},
  \code{\link{summary.Lexis}},
  \code{\link{timeSince}},
  \code{\link{boxes.Lexis}}
}
\examples{
# A small artificial example
xx <- Lexis( entry=list(age=c(17,24,33,29),per=c(1920,1933,1930,1929)),
             duration=c(23,57,12,15), exit.status=c(1,2,1,2) )
xx
cut <- c(33,47,29,50)
cutLexis(xx, cut, new.state=3, precursor=1)
cutLexis(xx, cut, new.state=3, precursor=2)
cutLexis(xx, cut, new.state=3, precursor=1:2)
# The same as the last example
cutLexis(xx, cut, new.state=3)

# The same example with a factor status variable
yy <- Lexis(entry = list(age=c(17,24,33,29),per=c(1920,1933,1930,1929)),
            duration = c(23,57,12,15),
            entry.status = factor(rep("alpha",4),
            levels=c("alpha","beta","gamma")),
            exit.status = factor(c("alpha","beta","alpha","beta"),
            levels=c("alpha","beta","gamma")))

cutLexis(yy,c(33,47,29,50),precursor="alpha",new.state="gamma")
cutLexis(yy,c(33,47,29,50),precursor=c("alpha","beta"),new.state="aleph")

## Using a dataframe as cut argument
rl <- data.frame( lex.id=1:3, cut=c(19,53,26), timescale="age", new.state=3 )
rl
cutLexis(xx, rl )
cutLexis(xx, rl, precursor = 1)
cutLexis(xx, rl, precursor = 0:2)

## It is immaterial in what order splitting and cutting is done
xs <- splitLexis( xx, breaks=seq(0,100,10), time.scale="age")
xs
xsC <- cutLexis(xs, rl, precursor = 0 )

xC <- cutLexis(xx, rl, pre = 0)
xC
xCs <- splitLexis(xC, breaks=seq(0, 100, 10), time.scale = "age")
xCs
str(xCs)
}
\keyword{survival}
\concept{Lexis}
\concept{multistate models}
\concept{transition rates}
\concept{state transition}

\name{apc.lines}
\alias{lines.apc}
\alias{apc.lines}
\title{
  Plot APC-estimates in an APC-frame.
}
\description{
  When an APC-frame has been produced by \code{\link{apc.frame}}, this
  function draws a set of estimates from an APC-fit in the frame. An
  optional drift parameter can be added to the period parameters and
  subtracted from the cohort and age parameters.
}
\usage{
\method{lines}{apc}( x, P, C,
        scale = c("log","ln","rates","inc","RR"),
    frame.par = options()[["apc.frame.par"]],
        drift = 0,
           c0 = median( C[,1] ),
           a0 = median( A[,1] ),
           p0 = c0 + a0,
           ci = rep( FALSE, 3 ),
          lwd = c(3,1,1),
          lty = 1,
          col = "black",
         type = "l",
        knots = FALSE,
        shade = FALSE,
          ... )
 apc.lines( x, P, C,
        scale = c("log","ln","rates","inc","RR"),
    frame.par = options()[["apc.frame.par"]],
        drift = 0,
           c0 = median( C[,1] ),
           a0 = median( A[,1] ),
           p0 = c0 + a0,
           ci = rep( FALSE, 3 ),
          lwd = c(3,1,1),
          lty = 1,
          col = "black",
         type = "l",
        knots = FALSE,
        shade = FALSE,
          ... )
}
\arguments{
  \item{x}{If an \code{apc}-object, (see \code{\link{apc.fit}}), then
    the arguments \code{P}, \code{C}, \code{c0}, \code{a0} and \code{p0}
    are ignored, and the estimates from \code{x} plotted.

    Can also be a 4-column matrix with columns age, age-specific
    rates, lower and upper c.i., in which case period and cohort effects
    are taken from the arguments \code{P} and \code{C}.
    }
  \item{P}{Period effects. Rate-ratios. Same form as for the age-effects.}
  \item{C}{Cohort effects. Rate-ratios. Same form as for the age-effects.}
  \item{scale}{Are effects given on a log-scale? Character variable, one
    of \code{"log"}, \code{"ln"}, \code{"rates"}, \code{"inc"},
    \code{"RR"}. If \code{"log"} or \code{"ln"} it is assumed that
    effects are log(rates) and log(RRs) otherwise the actual effects are
    assumed given in \code{A}, \code{P} and \code{C}. If \code{A} is of
    class \code{apc}, it is assumed to be \code{"rates"}.}
  \item{frame.par}{2-element vector with the cohort-period offset and
    RR multiplicator. This will typically be the result from the call of
    \code{\link{apc.frame}}. See this for details.}
  \item{drift}{The drift parameter to be added to the period effect. If
    \code{scale="log"} this is assumed to be on the log-scale, otherwise
    it is assumed to be a multiplicative factor per unit of the first
    columns of  \code{A}, \code{P} and \code{C} }
  \item{c0}{The cohort where the drift is assumed to be 0; the subtracted
  drift effect is \code{drift*(C[,1]-c0)}.}
  \item{a0}{The age where the drift is assumed to be 0.}
  \item{p0}{The period where the drift is assumed to be 0.}
  \item{ci}{Should confidence interval be drawn. Logical or
    character. If character, any occurrence of \code{"a"} or \code{"A"}
    produces confidence intervals for the age-effect. Similarly for
    period and cohort.}
  \item{lwd}{Line widths for estimates, lower and upper confidence limits.}
  \item{lty}{Linetypes for the three effects.}
  \item{col}{Colours for the three effects.}
  \item{type}{What type of lines / points should be used.}
  \item{knots}{Should knots from the model be shown?}
  \item{shade}{Should confidence intervals be plotted as shaded areas?
    If true, the setting of \code{ci} is ignored.}
  \item{...}{Further parameters to be transmitted to \code{points}
    \code{lines}, \code{matpoints} or \code{matlines} used
    for plotting the three sets of curves.}
}
\details{
  There is no difference between the functions \code{apc.lines} and
    \code{lines.apc}, except the the latter is the \code{lines} method
    for \code{apc} objects.
  
  The drawing of three effects in an APC-frame is a rather trivial task,
  and the main purpose of the utility is to provide a function that
  easily adds the functionality of adding a drift so that several sets
  of lines can be easily produced in the same frame.
}
\value{
  \code{apc.lines} returns (invisibly) a list of three matrices of the
  effects plotted.
}
\author{
  Bendix Carstensen, Steno Diabetes Center,
  \url{http://bendixcarstensen.com}
}
\seealso{
  \code{\link{apc.frame}}, \code{\link{pc.lines}}, \code{\link{apc.fit}}, \code{\link{apc.plot}}
}
\keyword{hplot}

\name{Lexis.diagram}
\alias{Lexis.diagram}
\title{Plot a Lexis diagram}
\description{
   Draws a Lexis diagram, optionally with life lines from a cohort, and
   with lifelines of a cohort if supplied. Intended for presentation purposes.
}
\usage{
Lexis.diagram( age = c( 0, 60), 
              alab = "Age",
              date = c( 1940, 2000 ),
              dlab = "Calendar time",
               int = 5,
           lab.int = 2*int, 
          col.life = "black",
          lwd.life = 2,
          age.grid = TRUE,
         date.grid = TRUE,
          coh.grid = FALSE,
          col.grid = gray(0.7),
          lwd.grid = 1,     
               las = 1,
        entry.date = NA,
         entry.age = NA,
         exit.date = NA,
          exit.age = NA,
         risk.time = NA,
        birth.date = NA,
              fail = NA,
          cex.fail = 1.1, 
          pch.fail = c(NA,16),
          col.fail = rep( col.life, 2 ),
              data = NULL, ... )
}
\arguments{
 \item{age}{Numerical vector of length 2, giving the age-range for the diagram}
 \item{alab}{Label on the age-axis.}
 \item{date}{Numerical vector of length 2, giving the calendar
    time-range for the diagram} 
 \item{dlab}{label on the calendar time axis.}
 \item{int}{The interval between grid lines in the diagram. If a
vector of length two is given, the first value will be used for
spacing of age-grid and the second for spacing of the date grid.}
 \item{lab.int}{The interval between labelling of the grids.}
 \item{col.life}{Colour of the life lines.}
 \item{lwd.life}{Width of the life lines.}
 \item{age.grid}{Should grid lines be drawn for age?}
 \item{date.grid}{Should grid lines be drawn for date?}
 \item{coh.grid}{Should grid lines be drawn for birth cohorts (diagonals)?}
 \item{col.grid}{Colour of the grid lines.}
 \item{lwd.grid}{Width of the grid lines.}
 \item{las}{How are the axis labels plotted?}
 \item{entry.date, entry.age, exit.date, exit.age, risk.time,
       birth.date}{Numerical vectors defining lifelines to be plotted
       in the diagram. At least three must be given to produce lines.
       Not all subsets of three will suffice, the given subset has to 
       define life lines.
       If insufficient data is given, no life
       lines are produced.}
 \item{fail}{Logical of event status at exit for the persons whose 
       life lines are plotted.}
 \item{pch.fail}{Symbols at the end of the life lines for censorings
   (\code{fail==0}) and failures (\code{fail != 0}).}
 \item{cex.fail}{Expansion of the status marks at the end of life lines.}
 \item{col.fail}{Character vector of length 2 giving the colour of the
   failure marks for censorings and failures respectively.}
  \item{data}{Dataframe in which to interpret the arguments.}
  \item{...}{Arguments to be passed on to the initial call to plot.}
}
\value{
If sufficient information on lifelines is given, a data frame with
one row per person and columns with entry ages and dates, birth date,
risk time and status filled in.

Side effect: a plot of a Lexis diagram is produced with the life lines
in it is produced. This will be the main reason for using the
function. If the primary aim is to illustrate follow-up of a cohort, then
it is better to represent the follow-up in a \code{\link{Lexis}} object, and
use the generic \code{\link{plot.Lexis}} function.
}
\details{
The default unit for supplied variables are (calendar) years.
If any of the variables \code{entry.date}, \code{exit.date} or
\code{birth.date} are of class "\code{Date}" or if any of the variables
\code{entry.age}, \code{exit.age} or \code{risk.time} are of class
"\code{difftime}", they will be converted to calendar years, and plotted
correctly in the diagram. The returned dataframe will then have colums of
classes "\code{Date}" and  "\code{difftime}".
}
\author{
  Bendix Carstensen,
  \url{http://bendixcarstensen.com}
}
\examples{
Lexis.diagram( entry.age = c(3,30,45),
               risk.time = c(25,5,14),
              birth.date = c(1970,1931,1925.7),
                    fail = c(TRUE,TRUE,FALSE) )
LL <- Lexis.diagram( entry.age = sample( 0:50, 17, replace=TRUE ),
                     risk.time = sample( 5:40, 17, r=TRUE),
                    birth.date = sample( 1910:1980, 17, r=TRUE ),
          		  fail = sample( 0:1, 17, r=TRUE ), 
	              cex.fail = 1.1,
	              lwd.life = 2 )
# Identify the persons' entry and exits
text( LL$exit.date, LL$exit.age, paste(1:nrow(LL)), col="red", font=2, adj=c(0,1) )
text( LL$entry.date, LL$entry.age, paste(1:nrow(LL)), col="blue", font=2, adj=c(1,0) )
data( nickel )
attach( nickel )
LL <- Lexis.diagram( age=c(10,100), date=c(1900,1990), 
	             entry.age=age1st, exit.age=ageout, birth.date=dob, 
		     fail=(icd \%in\% c(162,163)), lwd.life=1,
		     cex.fail=0.8, col.fail=c("green","red") )
abline( v=1934, col="blue" )
nickel[1:10,]
LL[1:10,]
}
\keyword{hplot}
\keyword{dplot}
\seealso{
  \code{\link{Life.lines}},
  \code{\link{Lexis.lines}}
}

\name{Lexis}
\alias{Lexis}
\alias{print.Lexis}
\title{Create a Lexis object of follow-up}
\description{
  Create an object of class \code{Lexis} to represent follow-up in
  multiple states on multiple time scales.
}
\usage{
Lexis( entry,
        exit,
    duration,
entry.status = 0,
 exit.status = 0,
          id,
        data,
       merge = TRUE,
      states,
       notes = TRUE,
         tol = .Machine$double.eps^0.5,
keep.dropped = FALSE)
\method{print}{Lexis}(x, ...,
                      td = 2,
                      nd = td,
                    rnam = FALSE,
                     org = FALSE)
}
\arguments{
  \item{entry}{a named list of entry times. Each element of the list is
    a numeric variable representing the entry time on the named time
    scale. The name of the elements of the list will appear as names of
    variables designated as timescales in the resulting object. All time
    scales must have the same units (e.g. years). 
    The names of the timescales must be different from any column name in
    \code{data}.}
  \item{exit}{a named list of exit times.}
  \item{duration}{a numeric vector giving the duration of follow-up.}
  \item{entry.status}{a vector or a factor giving the status at
    entry}
  \item{exit.status}{a vector or factor giving status at exit. Any
    change in status during follow-up is assumed to take place exactly
    at the exit time.}
  \item{id}{a vector giving a unique identity value for each person
    represented in the Lexis object. Defaults to \code{1:nrow(data)}}
  \item{data}{an optional data frame, list, or environment containing
    the variables. If  not found in \code{data}, the variables are
    taken from the environment from which \code{Lexis} was called.}
  \item{merge}{a logical flag. If \code{TRUE} then the \code{data}
    argument will be coerced to a data frame and then merged with
    the resulting \code{Lexis} object.}
  \item{states}{A vector of labels for the states. If given, the state
    variables \code{lex.Cst} and \code{lex.Xst} are returned as factors with
    identical levels attributes equal to \code{states}.}
  \item{notes}{Logical. Should notes on entry states and time be given.}
  \item{tol}{Numerical tolerance for follow-up time. Rows with duration
    less than this value are automatically dropped.}
  \item{keep.dropped}{Logical. Should dropped rows from \code{data} be
    saved as an attribute with the object for inspection?}
  \item{x}{A \code{Lexis} object.}
  \item{td}{Number of digits after the decimal separator used for
    timescales and \code{lex.dur} when printing}
  \item{nd}{Number of digits after the decimal separator used for other
    numerical variables in the \code{Lexis} object.}
  \item{rnam}{Logical, should row names be printed?}
  \item{org}{Logical, should columns be printed in the original order?}
  \item{ ... }{Other parameters passed on to \code{print.data.frame}.}
}
\details{
  The analysis of long-term population-based follow-up studies typically
  requires multiple time scales to be taken into account, such as
  age, calendar time, or time since an event. A \code{Lexis} object is
  a data frame with additional attributes that allows these multiple time
  dimensions of follow-up to be managed.

  Separate variables for current end exit state allows representation of
  multistate data.

  Lexis objects are named after the German demographer Wilhelm
  Lexis (1837-1914), who is credited with the invention of the
  "Lexis diagram" for representing population dynamics simultaneously
  by several timescales in the book
  "Einleitung in die Theorie der Bevolkerungsstatistik" from 1875. 

  The \code{Lexis} function can create a minimal \code{Lexis} object
  with only those variables required to define the follow-up history in
  each row. Additional variables can be merged into the \code{Lexis}
  object using the \code{merge} method for \code{Lexis} objects. The
  latter is the default.

  The \code{print} method prints the time-scale variables and other
  numerical variables rounded, possibly differently. Reorders columns so
  the Lexis-specific variables comes first. Returns (invisibly) a character
  vector with the (re)ordering of the columns in the object, even if
  \code{org = TRUE} is set.
  
  There are also \code{merge}, \code{subset}, \code{transform} and many
  other methods for \code{Lexis} objects. They work as the corresponding
  methods for data-frames but ensures that the result is a \code{Lexis}
  object.
  }
\note{
  Only two of the three arguments \code{entry}, \code{exit} and
  \code{duration} need to be given.  If the third parameter is missing,
  it is imputed.

  \code{entry}, \code{exit} must be numeric, using  \code{\link{Date}}
  variables will cause some of the utilities to crash. Transformation by
  \code{\link{cal.yr}} is recommended.

  If only either \code{exit} or \code{duration} are supplied it is
  assumed that \code{entry} is 0. This is only meaningful (and therefore
  checked) if there is only one timescale.

  If any of \code{entry.status} or \code{exit.status} are of mode character,
  they will both be converted to factors.

  If \code{entry.status} is not given, then its class is automatically
  set to that of \code{exit.status}. If \code{exit.status} is a
  character or factor, the value of \code{entry.status} is set to the
  first level. This may be highly undesirable, and therefore noted. For
  example, if \code{exit.status} is character the first level will be
  the first in the alphabetical ordering; slightly unfortunate if values
  are \code{c("Well","Diseased")}. If \code{exit.status} is logical, the
  value of \code{entry.status} set to \code{FALSE}. If
  \code{exit.status} is numeric, the value of \code{entry.status} set to
  0.

  If \code{entry.status} or \code{exit.status} are factors or character,
  the corresponding state variables in the returned \code{Lexis} object,
  \code{lex.Cst} and \code{lex.Xst} will be (unordered) factors with
  identical set of levels, namely the union of the levels of
  \code{entry.status} and \code{exit.status}.
  }

\value{
  An object of class \code{Lexis}. This is represented as a data frame
  with a column for each time scale (with names equal to the union of
  the names of \code{entry} and \code{exit}), and additional columns with the
  following names:
  \item{lex.id}{Identification of the persons.}
  \item{lex.dur}{Duration of follow-up.}
  \item{lex.Cst}{Entry status (\code{C}urrent \code{st}ate),
                 i.e. the state in which the follow up takes place.}
  \item{lex.Xst}{Exit status (e\code{X}it \code{st}ate),
                 i.e. that state taken up after \code{dur} in \code{lex.Cst}.}
  If \code{merge=TRUE} (the default) then the \code{Lexis} object will
  also contain all variables from the \code{data} argument.
  }

\author{Martyn Plummer with contributions from Bendix Carstensen}

\examples{
# A small bogus cohort
xcoh <- structure(list( id = c("A", "B", "C"),
                     birth = c("14/07/1952", "01/04/1954", "10/06/1987"),
                     entry = c("04/08/1965", "08/09/1972", "23/12/1991"),
                      exit = c("27/06/1997", "23/05/1995", "24/07/1998"),
                      fail = c(1, 0, 1) ),
                    .Names = c("id", "birth", "entry", "exit", "fail"),
                 row.names = c("1", "2", "3"),
                     class = "data.frame")

# Convert the character dates into numerical variables (fractional years)
xcoh <- cal.yr(xcoh, format="\%d/\%m/\%Y", wh=2:4)
# xcoh <- cal.yr(xcoh, format="%d/%m/%Y", wh=2:4)

# See how it looks
xcoh
str( xcoh )

# Define a Lexis object with timescales calendar time and age
Lcoh <- Lexis(entry = list(per = entry ),
               exit = list(per = exit,
                           age = exit - birth),
        exit.status = fail,
               data = xcoh)

# Using character states may have undesired effects:
xcoh$Fail <- c("Dead","Well","Dead")
xcoh
L1 <- Lexis(entry = list(per = entry),
             exit = list(per = exit,
                         age = exit - birth),
      exit.status = Fail,
             data = xcoh)
L1
# people start being dead!

# ...unless you order the levels sensibly
xcoh$Fail <- factor(xcoh$Fail, levels = c("Well", "Dead"))
L2 <- Lexis(entry = list(per = entry),
             exit = list(per = exit,
                         age = exit - birth),
      exit.status = Fail,
             data = xcoh)
L2
# behaviour of print method:
L2[,1:6]
L2[,6:1]
print(L2[,6:1], org=TRUE)
(print(L2[,-3]))
}
\seealso{
   \code{\link{plot.Lexis}},
   \code{\link{splitLexis}},
   \code{\link{cutLexis}},
   \code{\link{mcutLexis}},
   \code{\link{rcutLexis}},
   \code{\link{addCov.Lexis}},
%   \code{\link{glm.Lexis}},
%   \code{\link{gam.Lexis}},
%   \code{\link{coxph.Lexis}},
   \code{\link{merge.Lexis}},
   \code{\link{subset.Lexis}},
   \code{\link{cbind.Lexis}},
   \code{\link{rbind.Lexis}},
   \code{\link{transform.Lexis}},
   \code{\link{summary.Lexis}},
   \code{\link{unLexis}},
   \code{\link{timeScales}},
   \code{\link{timeBand}},
   \code{\link{entry}},
   \code{\link{exit}},
   \code{\link{transient}},
   \code{\link{absorbing}},
   \code{\link{dur}}
   }
\keyword{survival}
\keyword{manip}

\name{ROC}
\alias{ROC}
%- Also NEED an `\alias' for EACH other topic documented here.
\title{Function to compute and draw ROC-curves.}
\description{
Computes sensitivity, specificity and positive and negative predictive
values for a test based on dichotomizing along the variable
\code{test}, for prediction of \code{stat}. Plots curves of these and a ROC-curve.
}
\usage{
ROC( test = NULL,
     stat = NULL,
     form = NULL,
     plot = c("sp", "ROC"),
       PS = is.null(test),
       PV = TRUE,
       MX = TRUE,
       MI = TRUE,
      AUC = TRUE,
     grid = seq(0,100,10),
 col.grid = gray( 0.9 ),
     cuts = NULL,
      lwd = 2,
     data = parent.frame(),
      ... )
}
\arguments{
  \item{test}{ Numerical variable used for prediction. }
  \item{stat}{ Logical variable of true status. }
  \item{form}{ Formula used in a logistic regression. If this is given,
               \code{test} and \code{stat} are ignored. If not given then
               both \code{test} and \code{stat} must be supplied. }
  \item{plot}{ Character variable. If "sp", the a plot of sensitivity,
specificity and predictive values against test is produced, if "ROC" a
ROC-curve is plotted. Both may be given.}
  \item{PS}{logical, if TRUE the x-axis in the
    plot "ps"-plot is the the predicted probability for
    \code{stat}==TRUE, otherwise it is the scale of \code{test} if this
    is given otherwise the scale of the linear predictor from the
    logistic regression.}
  \item{PV}{Should sensitivity, specificity and
    predictive values at the optimal cutpoint be given on the ROC plot? }
  \item{MX}{Should the ``optimal cutpoint'' (i.e. where sens+spec is
maximal) be indicated on the ROC curve?}
  \item{MI}{Should model summary from the logistic
    regression model be printed in the plot?}
  \item{AUC}{Should the area under the curve (AUC) be printed in the ROC
    plot?}
  \item{grid}{Numeric or logical. If FALSE no background grid is
    drawn. Otherwise a grid is drawn on both axes at \code{grid} percent.}
  \item{col.grid}{Colour of the grid lines drawn.}
  \item{cuts}{Points on the test-scale to be annotated on the
    ROC-curve. }
  \item{lwd}{Thickness of the curves}
  \item{data}{Data frame in which to interpret the variables.}
  \item{\dots}{Additional arguments for the plotting of the
    ROC-curve. Passed on to \code{plot}}
}
\details{
 As an alternative to a \code{test} and a \code{status} variable, a
model formula may given, in which case the the linear predictor is the
test variable and the response is taken as the true status variable.
The test used to derive  sensitivity, specificity, PV+ and PV- as a
function of \eqn{x} is \code{test}\eqn{\geq x}{>=x} as a predictor of
\code{stat}=TRUE.
}
\value{
  A list with two components:

  \item{res}{dataframe with variables \code{sens}, \code{spec},
    \code{pvp}, \code{pvn} and name of the test variable. The latter is
    the unique values of test or linear predictor from the logistic
    regression in ascending order with -Inf prepended. Since the
    sensitivity is defined as \eqn{P(test>x)|status=TRUE}, the first row
    has \code{sens} equal to 1 and \code{spec} equal to 0, corresponding
    to drawing the ROC curve from the upper right to the lower left corner.}

  \item{lr}{glm object with the logistic regression result used for
construction of the ROC curve}
  0, 1 or 2 plots are produced according to the setting of \code{plot}.
}
\author{Bendix Carstensen, Steno Diabetes Center Copenhagen,
  \url{http://bendixcarstensen.com} }
\examples{
x <- rnorm( 100 )
z <- rnorm( 100 )
w <- rnorm( 100 )
tigol <- function( x ) 1 - ( 1 + exp( x ) )^(-1)
y <- rbinom( 100, 1, tigol( 0.3 + 3*x + 5*z + 7*w ) )
ROC( form = y ~ x + z, plot="ROC" )
}
\keyword{manip}
\keyword{htest}
%\keyword{ROC-curves}
%\keyword{sensitivity}
%\keyword{specificity}
%\keyword{predictive values}

\name{merge.Lexis}
\alias{merge.Lexis}
\title{Merge a Lexis object with a data frame}
\description{
  Merge additional variables from a data frame into a Lexis object.
}
\usage{
\method{merge}{Lexis}(x, y, id, by, ...)
}
\arguments{
  \item{x}{an object of class \code{Lexis}}
  \item{y}{a data frame}
  \item{id}{the name of the variable in \code{y} to use for matching
    against the variable \code{lex.id} in \code{x}.
  }
  \item{by}{if matching is not done by id, a vector of variable names
    common to both \code{x} and \code{y}}
  \item{...}{optional arguments to be passed to \code{merge.data.frame}}
}
\details{
  A \code{Lexis} object can be considered as an augmented data frame
  in which some variables are time-dependent variables representing
  follow-up. The \code{Lexis} function produces a minimal object
  containing only these time-dependent variables.  Additional variables
  may be added to a \code{Lexis} object using the \code{merge} method.
}
\value{
  A \code{Lexis} object with additional columns taken from the
  merged data frame.
}
\author{Martyn Plummer}
\note{
  The variable given as the \code{by.y} argument must not contain
  any duplicate values in the data frame \code{y}.
  }
\seealso{\code{\link{merge.data.frame}}, \code{\link{subset.Lexis}}}
\keyword{array}
\keyword{manip}

\name{bdendo11}
\alias{bdendo11}
\docType{data}
\title{A 1:1 subset of the endometrial cancer case-control study}
\description{
The \code{bdendo11} data frame has 126 rows and 13 columns.
This is a subset of the dataset \code{\link{bdendo}} in which each case
was matched with a single control.
}
\source{
Breslow NE, and Day N, Statistical Methods in Cancer Research. Volume
I: The Analysis of Case-Control Studies. IARC Scientific
Publications, IARC:Lyon, 1980.  
}
\examples{
data(bdendo11)
}
\keyword{datasets}

\name{boxes.MS}
\Rdversion{1.1}
\alias{tbox}
\alias{dbox}
\alias{fillarr}
\alias{boxarr}
\alias{boxes}
\alias{boxes.Lexis}
\alias{boxes.matrix}
\alias{boxes.MS}
\title{
  Draw boxes and arrows for illustration of multistate models.
  }
  \description{
  Boxes can be drawn with text (\code{tbox}) or a cross (\code{dbox}),
  and arrows pointing between the boxes (\code{boxarr}) can be drawn
  automatically not overlapping the boxes. The \code{boxes} method for
  \code{\link{Lexis}} objects generates displays of states with
  person-years and transitions with events or rates.
  }
\usage{
   tbox( txt, x, y, wd, ht,
         font=2, lwd=2,
         col.txt=par("fg"),
         col.border=par("fg"),
         col.bg="transparent" )
   dbox( x, y, wd, ht=wd,
         font=2, lwd=2, cwd=5,
         col.cross=par("fg"),
         col.border=par("fg"),
         col.bg="transparent"  )
   boxarr( b1, b2, offset=FALSE, pos=0.45, ... )
\method{boxes}{Lexis}( obj,
                    boxpos = FALSE,
                     wmult = 1.20,
                     hmult = 1.20 + 0.85*(!show.Y),
                       cex = 1.40,
                    show   = inherits( obj, "Lexis" ),
                    show.Y = show,
                   scale.Y = 1,
                  digits.Y = 1,
                   show.BE = FALSE,
                    BE.sep = c("","","          ",""),
                    show.D = show,
                   scale.D = FALSE,
                  digits.D = as.numeric(as.logical(scale.D)),
                    show.R = show & is.numeric(scale.R),
                   scale.R = 1,
                  digits.R = as.numeric(as.logical(scale.R)),
                    DR.sep = if( show.D ) c("\n(",")") else c("",""),
                     eq.wd = TRUE,
                     eq.ht = TRUE,
                        wd,
                        ht,
                    subset = NULL,
                   exclude = NULL,
                      font = 1,
                       lwd = 2,
                   col.txt = par("fg"),
                col.border = col.txt,
                    col.bg = "transparent",
                   col.arr = par("fg"),
                   lwd.arr = lwd,
                  font.arr = font,
                   pos.arr = 0.45,
                   txt.arr = NULL,
               col.txt.arr = col.arr,
                offset.arr = 2,
                             ... )
\method{boxes}{matrix}( obj, ... )
\method{boxes}{MS}( obj, sub.st, sub.tr, cex=1.5, ... )
   fillarr( x1, y1, x2, y2, gap=2, fr=0.8,
            angle=17, lwd=2, length=par("pin")[1]/30, ... )
   }
\arguments{
  \item{txt}{Text to be placed inside the box.}
  \item{x}{x-coordinate of center of box.}
  \item{y}{y-coordinate of center of box.}
  \item{wd}{width of boxes in percentage of the plot width.}
  \item{ht}{height of boxes in percentage of the plot height.}
  \item{font}{Font for the text. Defaults to 2 (=bold).}
  \item{lwd}{Line width of the box borders.}
  \item{col.txt}{Color for the text in boxes.}
  \item{col.border}{Color of the box border.}
  \item{col.bg}{Background color for the interior of the box.}
  \item{\dots}{Arguments to be passed on to the call of other functions.}
  \item{cwd}{Width of the lines in the cross.}
  \item{col.cross}{Color of the cross.}
  \item{b1}{Coordinates of the "from" box. A vector with 4 components,
            \code{x}, \code{y}, \code{w}, \code{h}.}
  \item{b2}{Coordinates of the "to" box; like \code{b1}.}
  \item{offset}{Logical. Should the arrow be offset a bit to the left.}
  \item{pos}{Numerical between 0 and 1, determines the position of the point
    on the arrow which is returned.}
  \item{obj}{A \code{\link{Lexis}} object or a transition matrix; that
    is a square matrix indexed by state in both dimensions, and the
    \eqn{(i,j)}th entry different from \code{NA} if a transition \eqn{i}
    to \eqn{j} can occur. If \code{show.D=TRUE}, the arrows between
    states are annotated by these numbers. If \code{show.Y=TRUE}, the
    boxes representing states are annotated by the numbers in the
    diagonal of \code{obj}.

    For \code{boxes.matrix} \code{obj} is a matrix and for
    \code{boxes.MS}, \code{obj} is an \code{MS.boxes} object (see below).}
  \item{boxpos}{If \code{TRUE} the boxes are positioned  equidistantly on a
                circle, if \code{FALSE} (the default) you are queried to
                click on the screen for the positions. This argument can also
                be a named list with elements \code{x} and
                \code{y}, both numerical vectors, giving the centers of
                the boxes. These must be numbers between 0 and 100
    indicating percentages of the display in the two directions.}
  \item{wmult}{Multiplier for the width of the box relative to the width of the
               text in the box.}
  \item{hmult}{Multiplier for the height of the box relative to the height of the
               text in the box.}
  \item{cex}{Character expansion for text in the box.}
  \item{show}{Should person-years and transitions be put in the plot.
              Ignored if \code{obj} is not a \code{Lexis} object.}
  \item{show.Y}{If logical: Should person-years be put in the boxes.
              If numeric: Numbers to put in boxes.}
  \item{scale.Y}{What scale should be used for annotation of person-years.}
  \item{digits.Y}{How many digits after the decimal point should be used for the
                  person-years.}
  \item{show.BE}{Logical. Should number of persons beginning
    resp. ending follow up in each state be shown? If given as character
    "nz" or "noz" the numbers will be shown, but zeros omitted.}
  \item{BE.sep}{Character vector of length 4, used for annotation of the
    number of persons beginning and ending in each state: 1st element
    precedes no. beginning, 2nd trails it, 3rd precedes the no. ending
    (defaults to 8 spaces), and the 4th trails the no. ending.}
  \item{show.D}{Should no. transitions be put alongside the arrows.
                Ignored if \code{obj} is not a \code{Lexis} object.}
  \item{scale.D}{Synonymous with \code{scale.R}, retained for compatibility.}
  \item{digits.D}{Synonymous with \code{digits.R}, retained for compatibility.}
  \item{show.R}{Should the transition rates be shown on the arrows?}
  \item{scale.R}{If this a scalar, rates instead of no. transitions are printed
                 at the arrows, scaled by \code{scale.R}.}
  \item{digits.R}{How many digits after the decimal point should be used for the
                  rates.}
  \item{DR.sep}{Character vector of length 2. If rates are shown, the
    first element is inserted before and the second after the rate.}
  \item{eq.wd}{Should boxes all have the same width?}
  \item{eq.ht}{Should boxes all have the same height?}
  \item{subset}{Draw only boxes and arrows for a subset of the states.
                Can be given either as a numerical vector or character
                vector state names.}
  \item{exclude}{Exclude states from the plot. The complementary of \code{subset}.
    Ignored if \code{subset} is given.}
  \item{col.arr}{Color of the arrows between boxes.
    A vector of character strings, the arrows are referred to as the
    row-wise sequence of non-NA elements of the transition matrix.
    Thus the first ones refer to the transitions out of state 1, in
    order of states.}
  \item{lwd.arr}{Line widths of the arrows.}
  \item{font.arr}{Font of the text annotation the arrows.}
  \item{pos.arr}{Numerical between 0 and 1, determines the position on
    the arrows where the text is written.}
  \item{txt.arr}{Text put on the arrows.}
  \item{col.txt.arr}{Colors for text on the arrows.}
  \item{offset.arr}{The amount offset between arrows representing
    two-way transitions, that is where there are arrows both ways
    between two boxes.}
  \item{sub.st}{Subset of the states to be drawn.}
  \item{sub.tr}{Subset of the transitions to be drawn.}
  \item{x1}{x-coordinate of the starting point.}
  \item{y1}{y-coordinate of the starting point.}
  \item{x2}{x-coordinate of the end point.}
  \item{y2}{y-coordinate of the end point.}
  \item{gap}{Length of the gap between the box and the ends of the arrows.}
  \item{fr}{Length of the arrow as the fraction of the distance between the
            boxes. Ignored unless given explicitly, in which case any value
            given for \code{gap} is ignored.}
  \item{angle}{What angle should the arrow-head have?}
  \item{length}{Length of the arrow head in inches. Defaults to 1/30 of the
                physical width of the plot.}
  }
  \details{
  These functions are designed to facilitate the drawing of multistate
  models, mainly by automatic calculation of the arrows between boxes.

  \code{tbox} draws a box with centered text, and returns a vector of
  location, height and width of the box. This is used when drawing
  arrows between boxes. \code{dbox} draws a box with a cross,
  symbolizing a death state. \code{boxarr} draws an arrow between two
  boxes, making sure it does not intersect the boxes. Only straight
  lines are drawn.

  \code{boxes.Lexis} takes as input a Lexis object sets up an empty plot
  area (with axes 0 to 100 in both directions) and if
  \code{boxpos=FALSE} (the default) prompts you to click on the
  locations for the state boxes, and then draws arrows implied by the
  actual transitions in the \code{Lexis} object. The default is to
  annotate the transitions with the number of transitions.

  A transition matrix can also be supplied, in which case the row/column
  names are used as state names, diagonal elements taken as
  person-years, and off-diagonal elements as number of transitions.
  This also works for \code{boxes.matrix}.

  Optionally returns the R-code reproducing the plot in a file, which
  can be useful if you want to produce exactly the same plot with
  differing arrow colors etc.

  \code{boxarr} draws an arrow between two boxes, on the line connecting
  the two box centers. The \code{offset} argument is used to offset the
  arrow a bit to the left (as seen in the direction of the arrow) on order
  to accommodate arrows both ways between boxes. \code{boxarr} returns a named
  list with elements \code{x}, \code{y} and \code{d}, where the two former
  give the location of a point on the arrow used for printing (see argument
  \code{pos}) and the latter is a unit vector in the
  direction of the arrow, which is used by \code{boxes.Lexis} to
  position the annotation of arrows with the number of transitions.

  \code{boxes.MS} re-draws what \code{boxes.Lexis} has done based on the
  object of class \code{MS} produced by \code{boxes.Lexis}. The point
  being that the \code{MS} object is easily modifiable, and thus it is a
  machinery to make variations of the plot with different color
  annotations etc.

  \code{fill.arr} is just a utility drawing nicer arrows than the default
  \code{\link{arrows}} command, basically by using filled arrow-heads; called
  by \code{boxarr}.
  }
\value{The functions \code{tbox} and \code{dbox} return the location and
       dimension of the boxes, \code{c(x,y,w,h)}, which are designed to be used
       as input to the \code{boxarr} function.

       The \code{boxarr} function returns the coordinates (as a named
       list with names \code{x} and \code{y}) of a point on the
       arrow, designated to be used for annotation of the arrow.

       The function \code{boxes.Lexis} returns an \code{MS} object, a list with
       five elements: 1) \code{Boxes} - a data frame with one row
       per box and columns
       \code{xx},
       \code{yy},
       \code{wd},
       \code{ht},
       \code{font},
       \code{lwd},
       \code{col.txt},
       \code{col.border} and
       \code{col.bg},
       2) an object \code{State.names} with names of states (possibly an
       expression, hence not possible to include as a column in \code{Boxes}),
       3) a matrix \code{Tmat}, the transition matrix, 4) a data
       frame, \code{Arrows} with one row per transition and columns:
       \code{lwd.arr},
       \code{col.arr},
       \code{pos.arr},
       \code{col.txt.arr},
       \code{font.arr} and
       \code{offset.arr} and
       5) an object \code{Arrowtext} with names of states (possibly an
       expression, hence not possible to include as a column in
       \code{Arrows})

       An \code{MS} object is used as input to
       \code{boxes.MS}, the primary use is to modify selected entries in
       the \code{MS} object first, e.g. colors, or supply
       sub-setting arguments in order to produce displays that have the
       same structure, but with different colors etc.
       }
\author{Bendix Carstensen}
\examples{
par( mar=c(0,0,0,0), cex=1.5 )
plot( NA,
      bty="n",
      xlim=0:1*100, ylim=0:1*100, xaxt="n", yaxt="n", xlab="", ylab="" )
bw  <- tbox( "Well"    , 10, 60, 22, 10, col.txt="blue" )
bo  <- tbox( "other Ca", 45, 80, 22, 10, col.txt="gray" )
bc  <- tbox( "Ca"      , 45, 60, 22, 10, col.txt="red" )
bd  <- tbox( "DM"      , 45, 40, 22, 10, col.txt="blue" )
bcd <- tbox( "Ca + DM" , 80, 60, 22, 10, col.txt="gray" )
bdc <- tbox( "DM + Ca" , 80, 40, 22, 10, col.txt="red" )
      boxarr( bw, bo , col=gray(0.7), lwd=3 )
# Note the argument adj= can takes values outside (0,1)
text( boxarr( bw, bc , col="blue", lwd=3 ),
      expression( lambda[Well] ), col="blue", adj=c(1,-0.2), cex=0.8 )
      boxarr( bw, bd , col=gray(0.7) , lwd=3 )
      boxarr( bc, bcd, col=gray(0.7) , lwd=3 )
text( boxarr( bd, bdc, col="blue", lwd=3 ),
      expression( lambda[DM] ), col="blue", adj=c(1.1,-0.2), cex=0.8 )

# Set up a transition matrix allowing recovery
tm <- rbind( c(NA,1,1), c(1,NA,1), c(NA,NA,NA) )
rownames(tm) <- colnames(tm) <- c("Cancer","Recurrence","Dead")
tm
boxes.matrix( tm, boxpos=TRUE )

# Illustrate texting of arrows
boxes.Lexis( tm, boxpos=TRUE, txt.arr=c("en","to","tre","fire") )
zz <- boxes( tm, boxpos=TRUE, txt.arr=c(expression(lambda[C]),
                                        expression(mu[C]),
                                        "recovery",
                                        expression(mu[R]) ) )

# Change color of a box
zz$Boxes[3,c("col.bg","col.border")] <- "green"
boxes( zz )

# Set up a Lexis object
data(DMlate)
str(DMlate)
dml <- Lexis( entry=list(Per=dodm, Age=dodm-dobth, DMdur=0 ),
               exit=list(Per=dox),
        exit.status=factor(!is.na(dodth),labels=c("DM","Dead")),
               data=DMlate[1:1000,] )

# Cut follow-up at Insulin
dmi <- cutLexis( dml, cut=dml$doins, new.state="Ins", pre="DM" )
summary( dmi )
boxes( dmi, boxpos=TRUE )
boxes( dmi, boxpos=TRUE, show.BE=TRUE )
boxes( dmi, boxpos=TRUE, show.BE="nz" )
boxes( dmi, boxpos=TRUE, show.BE="nz", BE.sep=c("In:","      Out:","") )

# Set up a bogus recovery date just to illustrate two-way transitions
dmi$dorec <- dmi$doins + runif(nrow(dmi),0.5,10)
dmi$dorec[dmi$dorec>dmi$dox] <- NA
dmR <- cutLexis( dmi, cut=dmi$dorec, new.state="DM", pre="Ins" )
summary( dmR )
boxes( dmR, boxpos=TRUE )
boxes( dmR, boxpos=TRUE, show.D=FALSE )
boxes( dmR, boxpos=TRUE, show.D=FALSE, show.Y=FALSE )
boxes( dmR, boxpos=TRUE, scale.R=1000 )
MSobj <- boxes( dmR, boxpos=TRUE, scale.R=1000, show.D=FALSE )
MSobj <- boxes( dmR, boxpos=TRUE, scale.R=1000, DR.sep=c(" (",")") )
class( MSobj )
boxes( MSobj )
MSobj$Boxes[1,c("col.txt","col.border")] <- "red"
MSobj$Arrows[1:2,"col.arr"] <- "red"
boxes( MSobj )
    }
\seealso{
   \code{\link{tmat.Lexis}}, \code{\link{legendbox}}
   }
\keyword{survival}
\keyword{hplot}
\keyword{iplot}

\name{ccwc}
\alias{ccwc}
\title{Generate a nested case-control study}
\usage{
ccwc( entry=0, exit, fail, origin=0, controls=1, match=list(),
      include=list(), data=NULL, silent=FALSE )
}
\arguments{
  \item{entry}{
    Time of entry to follow-up
  }
  \item{exit}{
    Time of exit from follow-up
  }
  \item{fail}{
    Status on exit (1=Fail, 0=Censored)
  }
  \item{origin}{
    Origin of analysis time scale
  }
  \item{controls}{
    The number of controls to be selected for each case
  }
  \item{match}{
    List of categorical variables on which to match cases and controls
  }
  \item{include}{
    List of other variables to be carried across into the case-control
    study
  }
  \item{data}{
    Data frame in which to look for input variables
  }
  \item{silent}{
    If FALSE, echos a . to the screen for each case-control set
    created; otherwise produces no output.
  }
}
\description{
Given the basic outcome variables for a cohort study: the time of entry
to the cohort, the time of exit and the reason for exit ("failure" or
"censoring"), this function computes risk sets and generates a matched
case-control study in which each case is compared with a set of controls
randomly sampled from the appropriate risk set. Other variables may be
matched when selecting controls.
}
\value{
  The case-control study, as a  dataframe containing:
  \item{Set}{
    case-control set number
  }
  \item{Map}{
    row number of record in input dataframe
  }
  \item{Time}{
    failure time of the case in this set
  }
  \item{Fail}{
    failure status (1=case, 0=control)
  }
  These are followed by the matching variables, and finally by the
  variables in the \code{include} list
}
\references{
  Clayton and Hills, Statistical Models in Epidemiology, Oxford
  University Press, Oxford:1993.
  }
\author{
  David Clayton
  }
\seealso{
  \code{\link{Lexis}}
  }
\examples{
#
# For the diet and heart dataset, create a nested case-control study
# using the age scale and matching on job
#
data(diet)
dietcc <- ccwc( doe, dox, chd, origin=dob, controls=2, data=diet,
                include=energy, match=job)
}
\keyword{datagen}

\name{ftrend}
\alias{ftrend}
\title{Fit a floating trend to a factor in generalized linear model}
\description{
  Fits a "floating trend" model to the given factor in a glm in a generalized
  linear model by centering covariates.
}
\usage{
ftrend(object, ...)
}
\arguments{
  \item{object}{fitted \code{lm} or \code{glm} object. The model must not have an intercept term}
  \item{...}{arguments to the \code{nlm} function}
}
\details{
  \code{ftrend()} calculates "floating trend" estimates for factors in
  generalized linear models.  This is an alternative to treatment
  contrasts suggested by Greenland et al. (1999).  If a regression model
  is fitted with no intercept term, then contrasts are  not used for the
  first factor in the model. Instead, there is one parameter  for each
  level of this factor.  However, the interpretation of these
  parameters,  and their variance-covariance matrix, depends on the
  numerical coding used for the  covariates. If an arbitrary constant is
  added to the covariate values, then   the variance matrix is changed.

  The \code{ftrend()} function takes the fitted model and works out an optimal
  constant to add to the covariate values so that the covariance matrix is
  approximately diagonal.  The parameter estimates can then be treated as
  approximately independent, thus simplifying their presentation. This is
  particularly useful for graphical display of dose-response relationships
  (hence the name).

  Greenland et al. (1999) originally suggested centring the covariates so that
  their weighted mean, using the fitted weights from the model, is zero.  This
  heuristic criterion is improved upon by \code{ftrend()} which uses the same
  minimum information divergence criterion as used by Plummer (2003) for
  floating variance calculations. \code{ftrend()} calls \code{nlm()} to
  do the minimization and will pass optional arguments to control it.
}
\note{
  The "floating trend" method is an alternative to the "floating
  absolute risk" method, which is implemented in the function
  \code{float()}.
}
\value{
  A list with the following components
  \item{coef}{coefficients for model with adjusted covariates.}
  \item{vcov}{Variance-covariance matrix of adjusted coefficients.}
}
\references{
  Greenland S, Michels KB, Robins JM, Poole C and Willet WC (1999)
  Presenting statistical uncertainty in trends and dose-response relations,
  \emph{American Journal of Epidemiology}, \bold{149}, 1077-1086.
}
\author{Martyn Plummer}
\seealso{\code{\link{float}}}
\keyword{regression}

\name{mortDK}
\alias{mortDK}
\docType{data}
\title{Population mortality rates for Denmark in 1-year age-classes.}
\description{
The \code{mortDK} data frame has 1820 rows and 21 columns.
}
\format{
This data frame contains the following columns:
\tabular{rl}{
\code{age}: \tab Age class, 0--89, 90:90+. \cr
\code{per}: \tab Calendar period, 38: 1938--42, 43: 1943--47, ..., 88:1988-92. \cr
\code{sex}: \tab Sex, 1: male, 2: female. \cr
\code{risk}: \tab Number of person-years in the Danish population. \cr
\code{dt}: \tab Number of deaths. \cr
\code{rt}: \tab Overall mortality rate in cases per 1000 person-years, i.e.
\code{rt=1000*dt/risk} \cr
           \tab Cause-specific mortality rates in cases per 1000 person-years: \cr
\code{r1}: \tab Infections \cr
\code{r2}: \tab Cancer. \cr
\code{r3}: \tab Tumors, benign, unspecific nature. \cr
\code{r4}: \tab Endocrine, metabolic. \cr
\code{r5}: \tab Blood. \cr
\code{r6}: \tab Nervous system, psychiatric. \cr
\code{r7}: \tab Cerebrovascular. \cr
\code{r8}: \tab Cardiac. \cr
\code{r9}: \tab Respiratory diseases, excl. cancer. \cr
\code{r10}: \tab Liver, excl. cancer. \cr
\code{r11}: \tab Digestive, other. \cr
\code{r12}: \tab Genitourinary. \cr
\code{r13}: \tab Ill-defined symptoms. \cr
\code{r14}: \tab All other, natural. \cr
\code{r15}: \tab Violent. \cr
}
}
\source{
Statistics Denmark, National board of health provided original data. 
Michael Andersson grouped the causes of death.
}
\examples{
data(mortDK)
}
\seealso{\code{\link{thoro}}, \code{\link{gmortDK}}}
\keyword{datasets}

\name{twoby2}
\alias{twoby2}
\title{Analysis of a two by two table}
\description{
  Computes the usual measures of association in a 2 by 2 table with
  confidence intervals. Also produces asymtotic and exact tests. Assumes
  that comparison of probability of the first column level between
  levels of the row variable is of interest. Output requires that the
  input matrix has meaningful row and column labels.
}
\usage{
twoby2(exposure, outcome,
       alpha = 0.05, print = TRUE, dec = 4,
       conf.level = 1-alpha, F.lim = 10000)
}
\arguments{
  \item{exposure}{If a table the analysis is based on the first two rows
    and first two columns of this. If a variable, this variable is
    tabulated against} 
  \item{outcome}{as the second variable}
  \item{alpha}{Significance level}
  \item{print}{Should the results be printed?}
  \item{dec}{Number of decimals in the printout.}
  \item{conf.level}{1-\code{alpha}}
  \item{F.lim}{If the table total exceeds \code{F.lim}, Fisher's exact
    test is not computed}
}
\value{A list with elements:
  \item{table}{The analysed 2 x 2 table augmented with probabilities and
    confidence intervals. The confidence intervals for the probabilities
    are computed using the normal approximation to the
    log-odds. Confidence intervals for the difference of proportions are
    computed using method 10 from Newcombe, Stat.Med. 1998, 17, pp.873
    ff.}
  \item{measures}{A table of Odds-ratios and relative risk with
    confidence intervals.}
  \item{p.value}{Exact p-value for the null hypothesis of OR=1}
}
\author{
  Mark Myatt. Modified by Bendix Carstensen.
}
\examples{
Treat <- sample(c("A","B"), 50, rep=TRUE )
Resp <- c("Yes","No")[1+rbinom(50,1,0.3+0.2*(Treat=="A"))]
twoby2( Treat, Resp )                 
twoby2( table( Treat, Resp )[,2:1] ) # Comparison the other way round
}
\keyword{univar}
\keyword{htest}

\name{ hivDK }
\alias{ hivDK }
\docType{ data }
\title{ hivDK: seroconversion in a cohort of Danish men}
\description{
  Data from a survey of HIV-positivity of a cohort of Danish
  men followed by regular tests from 1983 to 1989.
  }
\usage{
  data(hivDK)
  }
\format{
  A data frame with 297 observations on the following 7 variables.
  \describe{
    \item{\code{id}}{ID of the person}
    \item{\code{entry}}{Date of entry to the study. Date variable.}
    \item{\code{well}}{Date last seen seronegative. Date variable.}
    \item{\code{ill}}{Date first seen seroconverted. Date variable.}
    \item{\code{bth}}{Year of birth minus 1950.}
    \item{\code{pyr}}{Annual number of sexual partners.}
    \item{\code{us}}{Indicator of wheter the person has visited the USA.}
  }
}
\source{
  Mads Melbye, Statens Seruminstitut.
  }
\references{
  Becker N.G. and Melbye M.: Use of a log-linear model to
  compute the empirical survival curve from interval-censored data,
  with application to data on tests for HIV-positivity, Australian
  Journal of Statistics, 33, 125--133, 1990.

  Melbye M., Biggar R.J., Ebbesen P., Sarngadharan M.G., Weiss
  S.H., Gallo R.C. and Blattner W.A.: Seroepidemiology of HTLV-III
  antibody in Danish homosexual men: prevalence, transmission and
  disease outcome. British Medical Journal, 289, 573--575, 1984.
  }
\examples{
  data(hivDK)
  str(hivDK) 
  }
\keyword{ datasets }

\name{Relevel}
\alias{Relevel}
\alias{Relevel.factor}
\title{Reorder and combine levels of a factor}
\description{
  The levels of a factor are re-ordered so that the levels specified by
  \code{ref} appear first and remaining levels are moved down. This is
  useful for \code{contr.treatment} contrasts which take the first level
  as the reference. Factor levels may also be combined; two possibilities for
  specifying this are supported: hard coding or table look-up.
}
\usage{
\method{Relevel}{factor}( x, ref, first = TRUE, collapse="+",
                 xlevels=TRUE, nogroup=TRUE, \dots )
}
\arguments{
  \item{x}{A(n unordered) factor}
  \item{ref}{Vector, list or data frame, array, matrix or table.

    If \code{ref} is a vector (integer or character), it is assumed it
    contains the names or numbers of levels to be the first ones; non
    mentioned levels are kept.

    If \code{ref} is a list (but not a data frame), factor levels
    mentioned in each list element are combined. If the list is named
    the names are used as new factor levels, otherwise new level names
    are constructed from the old.

    If \code{ref} is a data frame or 2-dimensional array, matrix or
    table, the first column is assumed to have unique levels of \code{x}
    and the second to have groupings of this, respectively.

    }
  \item{first}{Should the levels mentioned in \code{ref} (if it is a
    list) come before those not?} 
  \item{collapse}{String used when constructing names for combined
  factor levels.} 
  \item{xlevels}{Logical. Should all levels in the 2nd column of
    \code{ref} be maintained as levels of the result, or (if
    \code{FALSE}) only the actually occurring.}
  \item{nogroup}{Logical. Should levels present in the input but not in
    the 1st column of \code{ref} be maintained as levels after the
    grouping? If \code{FALSE}, NAs will be returned for such elements.}
  \item{\dots}{Arguments passed on to other methods.}
}
\value{
  An unordered factor, where levels of \code{x} have been reordered
  and/or collapsed.
}
\details{
  The facility where \code{ref} is a two-column matrix mimics the
  SAS-facility of formats where a dataset can be used to construct a
  format --- SAS format is the grouping tool for variable
  values.

  If \code{ref} is a two-column object and \code{ref[,2]} is a factor
  \code{Relevel} will preserve the order of levels from \code{ref[,2]}.  
}
\author{Bendix Carstensen \url{http://bendixcarstensen.com}, Lars Jorge Diaz}
\seealso{\code{\link{Relevel.Lexis}}}
\examples{
# Grouping using a list (hard coding)
#
ff <- factor(sample(letters[1:5], 100, replace = TRUE))
table( ff, Relevel(ff, list( AB = 1:2, "Dee" = 4, c(3,5))))
table( ff, Relevel(ff,
                   list( 5:4, Z = c("c", "a") ),
                   coll = "-und-",
                   first = FALSE ) )

## Grouping using a two-column matrix as input:
## A factor with levels to be grouped together
ff <- factor(c("Bear","Bear","Crocodile","Snake","Crocodile","Bear"))
ff
## A grouping table
(gg <- data.frame(Animal = c("Bear","Whale","Crocodile","Snake","Eagle"),
                   Class = c("Mammal","Mammal","Reptile","Reptile","Bird")))
str(gg)
Relevel(ff, gg, xlevels = FALSE)
Relevel(ff, gg )
Relevel(ff, gg[c(1:5,5:1),])

## This crashes with an error
(GG <- rbind( gg, c("Bear","Reptile")))
try(Relevel(ff, GG))

ff <- factor(c(as.character(ff), "Jellyfish", "Spider"))
Relevel(ff, gg)
# excludes non-occupied levels
Relevel(ff, gg, xlevels = FALSE)
# If you do not want unknown animals classified, this returns NAs:
Relevel(ff, gg, nogroup = FALSE)
# Both
Relevel(ff, gg, nogroup = FALSE, xlevels = FALSE) 
}
\keyword{manip}

\name{Lexis2msm}
\alias{Lexis2msm}
\title{Convert a Lexis obejct to a data set suitable for input to the
  \code{msm:msm} function.} 
\description{The number of records in the resulting dataset will have a
  number of records that is normally \code{nrec(Lx) + \link{nid}(Lx)}, that is
  one extra record for each person. If there are 'holes' in persons'
  follow-up, each hole will also generate an extra record in the result. 
}
\usage{
Lexis2msm(Lx,
       state = "state",
     verbose = FALSE)
}
\arguments{
  \item{Lx}{
A \code{\link{Lexis}} object.
}
  \item{state}{
Character; the name of the state variable in the result.
}
  \item{verbose}{
If true, you will be reminded what the function did.
}
}
\value{
A data frame of class \code{msmLexis} with the timescales preserved and \code{lex.id}
preserved but with other \code{lex.} variables removed. 

Has more records than the original \code{Lexis} object
}
\author{
Bendix Carstensen, \url{http://bendixcarstensen.com}
}
\seealso{
\code{\link{Lexis}}
}
\examples{
example(mcutLexis)
# we now have the Lexis object L3:
summary(L3)

# data frame for use with msm
msm3 <- Lexis2msm(L3)

# see the difference 
subset(  L3, lex.id \%in\% 1:3)
subset(msm3, lex.id \%in\% 1:3)
timeScales(msm3)
}
\keyword{survival}
\keyword{manip}

\name{occup}
\alias{occup}
\docType{data}
\title{ A small occupational cohort }
\description{This is the data that is behind the illustrative Lexis
diagram in Breslow & Day's book on case-control studies.}
\usage{data(occup)}
\format{
  A data frame with 13 observations on the following 4 variables.
  \describe{
    \item{\code{AoE}}{a numeric vector, Age at Entry}
    \item{\code{DoE}}{a numeric vector, Date of entry}
    \item{\code{DoX}}{a numeric vector, Date of eXit}
    \item{\code{Xst}}{eXit status \code{D}-event, \code{W}-withdrawal,
                     \code{X}-censoring}
  }
}
\references{
Breslow & Day: Statistical Methods in Cancer Research, vol 1:
The analysis of case-control studies, figure 2.2, p. 48.}
\examples{
data(occup)
lx <- Lexis( entry = list( per=DoE, age=AoE ),
              exit = list( per=DoX ),
      entry.status = "W",
       exit.status = Xst,
              data = occup )
plot( lx )
# Split follow-up in 5-year classes
sx <- splitLexis( lx, seq(1940,1960,5), "per" )
sx <- splitLexis( sx, seq(  40,  60,5), "age" )
plot( sx )

# Plot with a bit more paraphernalia and a device to get
# the years on the same physical scale on both axes
ypi <- 2.5 # Years per inch
dev.new( height=15/ypi+1, width=20/ypi+1 ) # add an inch in each direction for
par( mai=c(3,3,1,1)/4, mgp=c(3,1,0)/1.6 )  # the margins set in inches by mai=
plot(sx,las=1,col="black",lty.grid=1,lwd=2,type="l",
     xlim=c(1940,1960),ylim=c(40,55),xaxs="i",yaxs="i",yaxt="n",
     xlab="Calendar year", ylab="Age (years)")
axis( side=2, at=seq(40,55,5), las=1 )
points(sx,pch=c(NA,16)[(sx$lex.Xst=="D")+1] )
box()
# Annotation with the person-years
PY.ann.Lexis( sx, cex=0.8 )
}
\keyword{datasets}

\name{cbind.Lexis}
\alias{cbind.Lexis}
\alias{rbind.Lexis}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{Combining a Lexis objects with data frames or other Lexis objects 
}
\description{A Lexis object may be combined side-by-side with
  data frames. Or several Lexis objects may stacked, possibly increasing
  the number of states and time scales.
}
\usage{
\method{cbind}{Lexis}(...)
\method{rbind}{Lexis}(...)
}
\arguments{
  \item{\dots}{For \code{cbind} a sequence of data frames or vectors of
    which exactly one has class \code{Lexis}. For \code{rbind} a sequence
    of Lexis objects, supposedly representing follow-up in the same
    population.}
  }
\details{
  Arguments to \code{rbind.Lexis} must all be \code{\link{Lexis}}
  objects; except for possible NULL objects. The timescales in the
  resulting object will be the union of all timescales present in all
  arguments. Values of timescales not present in a contributing Lexis
  object will be set to \code{NA}. The \code{breaks} for a given time
  scale will be \code{NULL} if the \code{breaks} of the same time scale
  from two contributing Lexis objects are different.

  The arguments to \code{cbind.Lexis} must consist of at most one Lexis
  object, so the method is intended for amending a Lexis object with
  extra columns without losing the Lexis-specific attributes.
}
\value{
  A \code{\link{Lexis}} object. \code{rbind} renders a \code{Lexis}
  object with timescales equal to the union of timescales in the
  arguments supplied. Values of a given timescale are set to \code{NA}
  for rows corresponding to supplied objects. \code{cbind} basically
  just adds columns to an existing Lexis object.
}
\author{
  Bendix Carstensen, \url{http://bendixcarstensen.com}
}
\seealso{
  \code{\link{subset.Lexis}}
}
\examples{
# A small bogus cohort
xcoh <- structure( list( id = c("A", "B", "C"),
                      birth = c("14/07/1952", "01/04/1954", "10/06/1987"),
                      entry = c("04/08/1965", "08/09/1972", "23/12/1991"),
                       exit = c("27/06/1997", "23/05/1995", "24/07/1998"),
                       fail = c(1, 0, 1) ),
                     .Names = c("id", "birth", "entry", "exit", "fail"),
                  row.names = c("1", "2", "3"),
                      class = "data.frame" )

# Convert the character dates into numerical variables (fractional years)
xcoh <- cal.yr( xcoh, format="\%d/\%m/\%Y", wh=2:4 )
# See how it looks
xcoh
str( xcoh )

# Define as Lexis object with timescales calendar time and age
Lcoh <- Lexis( entry = list( per=entry ),
                exit = list( per=exit, age=exit-birth ),
         exit.status = fail,
                data = xcoh )
Lcoh
cbind( Lcoh, zz=3:5 )

# Lexis object wit time since entry time scale
Dcoh <- Lexis( entry = list( per=entry, tfe=0 ),
                exit = list( per=exit ),
         exit.status = fail,
                data = xcoh )
# A bit meningless to combie these two, really...
rbind( Dcoh, Lcoh )

# Split different places
sL <- splitLexis( Lcoh, time.scale="age", breaks=0:20*5 )
sD <- splitLexis( Dcoh, time.scale="tfe", breaks=0:50*2 )
sDL <- rbind( sD, sL )
}

\keyword{survival}
\keyword{manip}

\name{lgrep}
\alias{fgrep}
\alias{ngrep}
\alias{lgrep}
\title{
Convenience versions of grep
}
\description{
Often you want the elements of a vector (or its names or levels) that
meet a certain pattern. But \code{grep} only gives you the position, so
these functions are designed to give you that. 
}
\usage{
fgrep( pattern, x, ... )
ngrep( pattern, x, ... )
lgrep( pattern, x, ... )
}
\arguments{
  \item{pattern}{Pattern searched for.}
  \item{x}{Object where \code{pattern} is searched. Or in whose \code{names}
    or \code{levels} attributes \code{pattern} is sought.}
  \item{...}{Arguments passed on to \code{\link[base]{grep}}.}
}
\value{Elements of the input \code{x} (\code{fgrep}) or its names
  attribute (\code{ngrep}) or levels attribute (\code{lgrep}).
}
\author{Bendix Carstensen, \email{b@bxc.dk}, \url{http://bendixcarstensen.com} }
\seealso{\code{\link{grep}}}
\examples{
ff <- factor( ll <- paste( sample( letters[1:3], 20, replace=TRUE ),
                           sample( letters[1:3], 20, replace=TRUE ), sep="" ) )
ff
fgrep( "a", ff )
fgrep( "a", ll )
ngrep( "a", ff )
lgrep( "a", ff )
lgrep( "a", ff, invert=TRUE )
}
\keyword{ manip }

\name{cal.yr}
\alias{cal.yr}
\alias{as.Date.cal.yr}
\title{
  Functions to convert character, factor and various date objects into a number,
  and vice versa.
  }
\description{
  Dates are converted to a numerical value, giving the calendar year as
  a fractional number. 1 January 1970 is converted to 1970.0, and other
  dates are converted by assuming that years are all 365.25 days long,
  so inaccuracies may arise, for example, 1 Jan 2000 is converted to
  1999.999. Differences between converted values will be 1/365.25 of the
  difference between corresponding \code{\link{Date}} objects.
  }
\usage{
  cal.yr( x, format="\%Y-\%m-\%d", wh=NULL )
  \method{as.Date}{cal.yr}( x, ... )
}
\arguments{
  \item{x}{A factor or character vector, representing a date in format
    \code{format}, or an object of class
    \code{\link{Date}},
    \code{\link{POSIXlt}},
    \code{\link{POSIXct}},
    \code{\link{date}},
    \code{dates} or
    \code{chron} (the latter two requires the \code{chron} package).
    If \code{x} is a data frame, all variables in the data-frame
    which are of one the classes mentioned are converted to class \code{cal.yr}.
    See arguemt \code{wh}, though.}
  \item{format}{Format of the date values if \code{x} is factor or character.
                If this argument is supplied and \code{x} is a datafame, all
                character variables are converted to class \code{cal.yr}.
                Factors in the dataframe will be ignored.}
  \item{wh}{Indices of the variables to convert if \code{x} is a data frame.
            Can be either a numerical or character vector.}
  \item{...}{Arguments passed on from other methods.}
  }
\value{
  \code{cal.yr} returns a numerical vector of the same length as
  \code{x}, of class \code{c("cal.yr","numeric")}. If \code{x} is a data frame
  a dataframe with some of the columns converted to class \code{"cal.yr"} is
  returned.

  \code{as.Date.cal.yr} returns a \code{\link{Date}} object.
}
\author{
  Bendix Carstensen, Steno Diabetes Center Copenhagen, \email{b@bxc.dk},
  \url{http://bendixcarstensen.com}
}
\seealso{
  \code{\link{DateTimeClasses}},
  \code{\link{Date}}
}
\examples{
 # Character vector of dates:
 birth <- c("14/07/1852","01/04/1954","10/06/1987","16/05/1990",
            "12/11/1980","01/01/1997","01/01/1998","01/01/1999")
 # Proper conversion to class "Date":
 birth.dat <- as.Date( birth, format="\%d/\%m/\%Y" )
 # Converson of character to class "cal.yr"
 bt.yr <- cal.yr( birth, format="\%d/\%m/\%Y" )
 # Back to class "Date":
 bt.dat <- as.Date( bt.yr )
 # Numerical calculation of days since 1.1.1970:
 days <- Days <- (bt.yr-1970)*365.25
 # Blunt assignment of class:
 class( Days ) <- "Date"
 # Then data.frame() to get readable output of results:
 data.frame( birth, birth.dat, bt.yr, bt.dat, days, Days, round(Days) )
}
\keyword{manip}
\keyword{chron}

\name{expand.data}
\alias{expand.data}
\title{
  Function to expand data for regression analysis of interval censored
  data.
  }
\description{
  This is a utility function.
  
  The original records with \code{first.well}, \code{last.well} and
  \code{first.ill} are
  expanded to multiple records; several for each interval where the
  person is known to be well and one where the person is known to fail.
  At the same time columns for the covariates needed to estimate rates
  and the response variable are generated.
  }
\usage{
expand.data(fu, formula, breaks, data)
}
\arguments{
  \item{fu}{A 3-column matrix with \code{first.well}, \code{last.well} and
  \code{first.ill} in each row.}
  \item{formula}{Model fromula, used to derive the model matrix.}
  \item{breaks}{Defines the intervals in which the baseline rate is
    assumed constant. All follow-up before the first and after the
    last break is discarded.}
  \item{data}{Datafrem in which \code{fu} and \code{formula} is interpreted.}
}
\value{
  Returns a list with three components
  \item{rates.frame}{Dataframe of covariates for estimation of the
    baseline rates --- one per interval defined by \code{breaks}.}
  \item{cov.frame}{Dataframe for estimation of the covariate effects. A
    data-framed version of the designmatrix from \code{formula}.}
  \item{y}{Response vector.}
}
\references{
  B Carstensen: Regression models for interval censored
  survival data: application to HIV infection in Danish homosexual
  men. Statistics in Medicine, 15(20):2177-2189, 1996.
  }
\author{
  Martyn Plummer, \email{plummer@iarc.fr}
  }
\seealso{
  \code{\link{Icens}}
  \code{\link{fit.mult}}
  \code{\link{fit.add}}
  }
\keyword{ models }
\keyword{ regression }
\keyword{ survival }

\name{projection.ip}
\alias{projection.ip}
\title{ Projection of columns of a matrix. }
\description{
  Projects the columns of the matrix \code{M} on the space spanned by the
  columns of the matrix \code{X}, with respect to the inner product
  defined by \code{weight}: \code{<x|y>=sum(x*w*y)}.
  }
\usage{
projection.ip(X, M, orth = FALSE, weight = rep(1, nrow(X)))
}
\arguments{
  \item{X}{ Matrix defining the space to project onto. }
  \item{M}{ Matrix of columns to be projected. Must have the same number
    of rows as \code{X}. }
  \item{orth}{ Should the projection be on the orthogonal complement to
    \code{span(X)}? }
  \item{weight}{ Weights defining the inner product. Numerical vector of
  length \code{nrow(X)}. }
}
\value{
  A matrix of full rank with columns in \code{span(X)}
}
\author{
  Bendix Carstensen, Steno Diabetes Center,
  \url{http://bendixcarstensen.com}, with help from Peter Dalgaard.
}
\seealso{ \code{\link{detrend}} }
\keyword{array}

\name{entry.Lexis}
\alias{entry}
\alias{exit}
\alias{status}
\alias{dur}
\alias{transient}
\alias{absorbing}
\alias{before}
\alias{preceding}
\alias{after}
\alias{succeeding}
\title{Time series and other methods for Lexis objects}
\description{
  Extract the entry time, exit time, status or duration of follow-up from a
  \code{Lexis} object. Classify states.
}
\usage{
     entry(x, time.scale = NULL, by.id=FALSE)
      exit(x, time.scale = NULL, by.id=FALSE)
    status(x, at="exit"        , by.id=FALSE)
       dur(x,                    by.id=FALSE)
 transient(x)
 absorbing(x)
 preceding(x, states)
    before(x, states)
succeeding(x, states)
     after(x, states)
}
\arguments{
  \item{x}{an object of class \code{Lexis}.}
  \item{time.scale}{a string or integer indicating the time scale. If
    omitted, all times scales are used.}
  \item{by.id}{Logical, if \code{TRUE}, only one record per unique value
  of \code{lex.id} is returned; either the first, the last, or for
  \code{dur}, the sum of \code{lex.dur}. If \code{TRUE}, the returned
  object have the \code{lex.id} as (row)names attribute.}
  \item{at}{string indicating the time point(s) at which status is to be
    measured. Possible values are "exit" or "entry".}
  \item{states}{Character vector of states.}
}
\value{
  The \code{entry} and \code{exit} functions return a vector of
  entry times and exit times, respectively, on the requested time
  scale. If multiple time scales are requested, a matrix is
  returned.

  The \code{status} function returns a vector giving the status at
  "\code{at}" (either '\code{entry}' or '\code{exit}') and \code{dur}
  returns a vector with the lengths of the follow-up intervals.

  \code{entry}, \code{exit}, \code{status} and \code{dur} return vectors
  of length \code{nrow(x)} if \code{by.id=FALSE}; if \code{by.id=TRUE} a
  vector of length \code{length(unique(lex.id))}.
  
  The functions \code{transient} and \code{absorbing} return character
  vectors of the transient, resp. absorbing states in \code{x}. These
  are necessarily disjoint but the union may be a proper subset of
  \code{levels(x)}, since the latter may have levels that are never
  assumed by either \code{lex.Cst} or \code{lex.Xst}.

  \code{preceding} returns a character vector with names of the states
  of the Lexis object \code{x} from which one of the states in
  \code{states} can be reached directly - the preceding
  states. \code{before} is just a synonym for \code{preceding}.

  \code{succeeding} returns a character vector with names of the states
  of the Lexis object \code{x} that can be reached directly from one of
  the states in \code{states}. \code{after} is just a synonym for
  \code{succeeding}.
}
\author{Martyn Plummer & Bendix Carstensen}
\seealso{\code{\link{Lexis}}}
\keyword{survival}
\keyword{ts}

\name{Lexis.lines}
\alias{Lexis.lines}
\title{Draw life lines in a Lexis diagram.}
\description{
  Add life lines to a Lexis diagram.
}
\usage{
Lexis.lines( entry.date = NA,
              exit.date = NA,
             birth.date = NA,
              entry.age = NA,
               exit.age = NA,
              risk.time = NA,
               col.life = "black",
               lwd.life = 2,
                   fail = NA,
               cex.fail = 1,
               pch.fail = c(NA, 16),
               col.fail = col.life,
                   data = NULL )
}
\arguments{
 \item{entry.date, entry.age, exit.date, exit.age, risk.time,
       birth.date}{Numerical vectors defining lifelines to be plotted
       in the diagram. At least three must be given to produce lines.
       Not all subsets of three will suffice, the given subset has to 
       define life lines. If insufficient data is given, no life
       lines are produced.}
 \item{col.life}{Colour of the life lines.}
 \item{lwd.life}{Width of the life lines.}
 \item{fail}{Logical of event status at exit for the persons whose 
       life lines are plotted.}
 \item{cex.fail}{The size of the status marks at the end of life lines.}
 \item{pch.fail}{The status marks at the end of the life lines.}
 \item{col.fail}{Colour of the marks for censorings and failures
   respectively.} 
 \item{data}{Data frame in which to interpret values.}
}
\value{
  If sufficient information on lifelines is given, a data frame with one
  row per person and columns with entry ages and dates, birth date, risk
  time and status filled in.

  Side effect: Life lines are added to an existing Lexis
  diagram. Lexis.lines adds life lines to an existing plot. 
}
\author{
  Bendix Carstensen, Steno Diabetes Center,
  \url{http://bendixcarstensen.com}
}
\examples{
Lexis.diagram( entry.age = c(3,30,45),
               risk.time = c(25,5,14),
              birth.date = c(1970,1931,1925.7),
                    fail = c(TRUE,TRUE,FALSE) )
Lexis.lines( entry.age = sample( 0:50, 100, replace=TRUE ),
             risk.time = sample( 5:40, 100, r=TRUE),
            birth.date = sample( 1910:1980, 100, r=TRUE ),
                  fail = sample(0:1,100,r=TRUE),
              cex.fail = 0.5,
              lwd.life = 1 )
}
\keyword{ hplot }
\keyword{ dplot }
\seealso{
  \code{\link{Lexis.diagram}},
  \code{\link{Life.lines}}
}

\name{rm.tr}
\alias{rm.tr}
\title{
Remove transitions from a Lexis object.
}
\description{
Sometimes certain transitions are not of interest. This function removes
these and assigns the risk time in the target state of the transitions
to the originating state.
}
\usage{
rm.tr(obj, from, to)
}
\arguments{
  \item{obj}{
A \code{Lexis} object.
}
  \item{from}{
Character; name of the state from which the transition to be purged
originates. Must be a valid state name for \code{obj}.
}
  \item{to}{
Character; name of the state to which the transition to be purged
targets. Must be a valid state name for \code{obj}.
}
}
\details{
The function removes all transitions from \code{from} to \code{to}, and
assigns all risk time in the \code{to} state after the transition
(\code{lex.dur}) to the \code{from} state. This is only done for risk
time in \code{to} occurring directly after \code{from}. Risk time in
\code{to} occurring after a transition from states different from
\code{from} is not affected. Transitions from \code{to} to another
state, \code{other}, say, will be changed to transitions from
\code{from} to \code{other}. 
}
\value{
A \code{\link{Lexis}} object with the indicated transition removed.
}
\author{
Bendix Carstensen, \url{http://bendixcarstensen.com}.
}
\seealso{
\code{\link{Relevel}}
}
\examples{
data(DMlate)
dml <- Lexis( entry = list(Per=dodm, Age=dodm-dobth, DMdur=0 ),
               exit = list(Per=dox),
        exit.status = factor(!is.na(dodth),labels=c("DM","Dead")),
               data = DMlate )

# A small subset for illustration
dml <- subset( dml, lex.id \%in\% c(13,15,20,28,40) )

# Cut the follow-up at start of insulin therapy
dmi <- cutLexis( dml, cut = dml$doins,
                      pre = "DM",
                new.state = "Ins" )[,1:10]

# How does it look?
dmi

# Remove all transitions DM -> Ins
rm.tr( dmi, "DM", "Ins" )
}
\keyword{manip}

\name{plotevent}
\alias{plotevent}
\title{ Plot Equivalence Classes }
\description{
  For interval censored data, segments of times between last.well and first.ill are plotted for each conversion in the data. It also plots the equivalence classes.
}
\usage{
plotevent(last.well, first.ill, data)
}
\arguments{
  \item{last.well}{ Time at which the individuals are
    last seen negative for the event }
  \item{first.ill}{ Time at which the individuals are
    first seen positive for the event }
  \item{data}{ Data with a transversal shape }
}
\details{
  last.well and first.ill should be written as character in the function.
}
\value{
  Graph
}
\references{ 
Carstensen B. Regression models for interval censored survival data:
application to HIV infection in Danish homosexual men.Stat Med. 1996 Oct
30;15(20):2177-89. 

Lindsey JC, Ryan LM. Tutorial in biostatistics methods for
interval-censored data.Stat Med. 1998 Jan 30;17(2):219-38. 
  }
\author{
  Delphine Maucort-Boulch, Bendix Carstensen, Martyn Plummer
  }
\seealso{
  \code{\link{Icens}}
  }
% \examples{
% }
\keyword{ models }
\keyword{ regression }
\keyword{ survival }

\name{summary.Lexis}
\alias{summary.Lexis}
\alias{print.summary.Lexis}
\title{
  Summarize transitions and risk time from a Lexis object
  }
\description{
  A two-way table of records and transitions classified by states
  (\code{lex.Cst} and \code{lex.Xst}), as well the risk time in each state.
  }
\usage{
  \method{summary}{Lexis}( object, simplify=TRUE, scale=1, by=NULL, Rates=FALSE, ... )
  \method{print}{summary.Lexis}( x, ..., digits=2 )
  }
\arguments{
  \item{object}{A Lexis object.}
  \item{simplify}{Should rows with 0 follow-up time be dropped?}
  \item{scale}{Scaling factor for the rates. The calculated rates are
    multiplied by this number.}
  \item{by}{Character vector of name(s) of variable(s) in
    \code{object}. Used to give a separate summaries for subsets of
    \code{object}. If longer than than 1, the interaction between that
    variables is used to stratify the summary. It is also possible to
    supply a vector of length \code{nrow(object)}, and the distinct
    values of this will be used to stratify the summary.}
\item{Rates}{Should a component with transition rates be returned (and
  printed) too?}
  \item{x}{A \code{summary.Lexis} object.}
  \item{digits}{How many digits should be used for printing?}
  \item{ ... }{Other parameters - ignored}
}
\value{
  An object of class \code{summary.Lexis}, a list with two components,
  \code{Transitions} and \code{Rates}, each one a matrix with rows
  classified by states where persons spent time, and columns classified
  by states to which persons transit. The \code{Transitions} contains
  number of transitions and has 4 extra columns with number of records,
  total number of events, total risk time and number of person
  contributing attached.  The \code{Rates} contains the transitions
  rates.

  If the argument \code{Rates} is FALSE (the default), then only the
  first component of the list is returned.
}
\author{Bendix Carstensen, \email{bxc@steno.dk}}
\examples{
data( nickel )
# Lung cancer deaths and other deaths are coded 1 and 2
nic <- Lexis( data=nickel,
             entry=list(age=agein),
              exit=list(age=ageout,cal=ageout+dob,tfh=ageout-age1st),
       exit.status=factor( (icd > 0) + (icd \%in\% c(162,163)),
                           labels=c("Alive","Other","Lung") ) )
str( nic )
head( nic )
summary( nic )
# More detailed summary, by exposure level
summary( nic, by=nic$exposure>5, Rates=TRUE, scale=100 )
}
\keyword{survival}

\name{lungDK}
\alias{lungDK}
\docType{data}
\title{Male lung cancer incidence in Denmark}
\description{
  Male lung cancer cases and population riks time in Denmark, for the
  period 1943--1992 in ages 40--89.
}
\usage{data(lungDK)}
\format{
  A data frame with 220 observations on the following 9 variables.
  \tabular{rl}{
    \code{A5}: \tab Left end point of the age interval, a numeric vector. \cr
    \code{P5}: \tab Left enpoint of the period interval, a numeric vector. \cr
    \code{C5}: \tab Left enpoint of the birth cohort interval, a numeric vector. \cr
    \code{up}: \tab Indicator of upper trianges of each age by period
      rectangle in the Lexis diagram. (\code{up=(P5-A5-C5)/5}). \cr
    \code{Ax}: \tab The mean age of diagnois (at risk) in the triangle. \cr
    \code{Px}: \tab The mean date of diagnosis (at risk) in the triangle. \cr
    \code{Cx}: \tab The mean date of birth in the triangle, a numeric vector. \cr
    \code{D}: \tab Number of diagnosed cases of male lung cancer. \cr
    \code{Y}: \tab Risk time in the male population, person-years. \cr
  }
}
\details{ Cases and person-years are tabulated by age and date of
  diagnosis (period) as well as date of birth (cohort) in 5-year
  classes. Each observation in the dataframe correponds to a triangle in
  a Lexis diagram. Triangles are classified by age and date of
  diagnosis, period of diagnosis and date of birth, all in 5-year
  groupings. 
}
\source{The Danish Cancer Registry and Statistics Denmark.
}
\references{
 For a more thorough exposition of statistical inference in the Lexis
 diagram, see: B. Carstensen: Age-Period-Cohort models for the Lexis
 diagram. Statistics in Medicine, 26: 3018-3045, 2007.  
}
\examples{
data( lungDK )
# Draw a Lexis diagram and show the number of cases in it.
attach( lungDK )
Lexis.diagram( age=c(40,90), date=c(1943,1993), coh.grid=TRUE )
text( Px, Ax, paste( D ), cex=0.7 )
}
\keyword{datasets}

\name{unLexis}
\alias{unLexis}
\title{Remove Lexis attributes from a \code{\link{Lexis}} object.}
\description{Removes the Lexis attributes, including the class
  \code{Lexis} from a Lexis object.}
\usage{unLexis(Lx)}
\arguments{\item{Lx}{A Lexis object}}
\value{The input object with "\code{Lexis}" removed from the class attribute.}
\author{Bendix Carstensen}
\seealso{\code{\link{Lexis}}}
\examples{
# A small bogus cohort
xcoh <- structure(list( id = c("A", "B", "C"),
                     birth = c("14/07/1952", "01/04/1954", "10/06/1987"),
                     entry = c("04/08/1965", "08/09/1972", "23/12/1991"),
                      exit = c("27/06/1997", "23/05/1995", "24/07/1998"),
                      fail = c(1, 0, 1) ),
                    .Names = c("id", "birth", "entry", "exit", "fail"),
                 row.names = c("1", "2", "3"),
                     class = "data.frame")

# Convert the character dates into numerical variables (fractional years)
  xcoh <- cal.yr(xcoh, format="\%d/\%m/\%Y", wh=2:4)
# xcoh <- cal.yr(xcoh, format="%d/%m/%Y", wh=2:4)

# Define a Lexis object with timescales calendar time and age
Lcoh <- Lexis(entry = list(per = entry ),
               exit = list(per = exit,
                           age = exit - birth),
        exit.status = fail,
               data = xcoh)
summary(Lcoh)
try(summary(unLexis(Lcoh)))
}
\concept{data manipulation}

\name{Icens}
\alias{Icens}
\alias{print.Icens}
\alias{summary.Icens}
\title{
  Fits a regression model to interval censored data.
  }
\description{   
  The models fitted assumes a piecewise constant baseline rate in
  intervals specified by the argument \code{breaks}, and for the
  covariates either a multiplicative relative risk function (default) or
  an additive excess risk function.
  }
\usage{
  Icens(first.well, last.well, first.ill,
        formula, model.type = c("MRR", "AER"), breaks,
        boot = FALSE, alpha = 0.05, keep.sample = FALSE,
        data)
\method{summary}{Icens}(object, scale = 1, ...)
\method{print}{Icens}(x, scale = 1, digits = 4, ...)
  }
\arguments{
  \item{first.well}{Time of entry to the study, i.e. the time first seen
    without event. Numerical vector.}
  \item{last.well}{Time last seen without event. Numerical vector.}
  \item{first.ill}{Time first seen with event. Numerical vector.}
  \item{formula}{Model formula for the log rate ratios / excess rates.}
  \item{model.type}{Which model should be fitted.}
  \item{breaks}{Breakpoints between intervals in which the underlying
    timescale effect is assumed constant. Any observation outside the range of
    \code{breaks} is discarded.}
  \item{boot}{Should bootstrap be performed to produce confidence
    intervals for parameters. If a number is given this will be the
    number of bootstrap samples. The default is 1000.}
  \item{alpha}{1 minus the confidence level.}
  \item{keep.sample}{Should the bootstrap sample of the parameter values
    be returned?} 
  \item{data}{Data frame in which the times and formula are
    interpreted.}
  \item{object}{an \code{Icens} object.}
  \item{x}{an \code{Icens} object.}
  \item{scale}{scaling factor for rates.}
  \item{digits}{how many digits is used for printing results.}
  \item{ ... }{Other parameters passed on.}  
  }
\details{
  The model is fitted by calling either \code{\link{fit.mult}} or
  \code{\link{fit.add}}. 
  }
\value{
  An object of class \code{"Icens"}: a list with two or three components:
  \item{rates}{A glm object from a binomial model with log-link,
    estimating the baseline rates, and the excess rates if \code{"AER"}
    is specified. The parameters in the linear predictor are rates,
    \emph{not} log-rates. They are in units corresponding to
    (the inverse of) the units
    in which the arguments \code{first.well},  \code{last.well},
    \code{first.ill} and  \code{breaks} are given.}
  \item{cov}{A glm object from a binomial model with complementary
    log-log link, estimating the log-rate-ratios. Only if \code{"MRR"}
    is specified.}
  \item{niter}{Number of iterations, a scalar}
  \item{boot.ci}{If \code{boot=TRUE}, a 3-column matrix with estimates
    and 1-\code{alpha} confidence intervals for the parameters in the model.}
  \item{sample}{A matrix of the parameter estimates from the
    bootstrapping. Rows refer to parameters, columns to bootstrap samples.}
  }
\references{
  B Carstensen: Regression models for interval censored
  survival data: application to HIV infection in Danish homosexual
  men. Statistics in Medicine, 15(20):2177-2189, 1996.

  CP Farrington: Interval censored survival data: a generalized linear
  modelling approach. Statistics in Medicine, 15(3):283-292, 1996.
  }
\author{
  Martyn Plummer, \email{martyn.plummer@r-project.org},
  Bendix Carstensen, \email{b@bxc.dk} }
\seealso{
  \code{\link{fit.add}}
  \code{\link{fit.mult}}
  }
\examples{
data(hivDK)
# Convert the dates to fractional years so that rates are
# expressed in cases per year and not per day
for(i in 2:4) hivDK[,i] <- cal.yr(hivDK[,i])

# multiplicative rate ratio model, baseline rates and RRs
m.RR <- Icens(entry, well, ill,
              model = "MRR",
            formula = ~ pyr + us,
             breaks = seq(1980, 1990, 5),
               data = hivDK)
# the MRR model returns a list with 2 glm objects.
round(ci.lin(m.RR$rates), 4)         # rates per 1 year
round(ci.lin(m.RR$cov, Exp=TRUE), 4) # RRs
# There is a print method that does the job:
print(m.RR)
# and scales only the rate parameters, here per 10 years
print(m.RR, scale = 10)

# additive excess rate model, baseline rates and excess rates
m.ER <- Icens(entry, well, ill,
              model = "AER",
            formula = ~ pyr + us,
             breaks = seq(1980, 1990, 5),
               data = hivDK)
# The print method returns rates per 1 year:
print(m.ER)
# or per 10 years:
print(m.ER, scale = 10)
  }
\keyword{models}
\keyword{regression}
\keyword{survival}

\name{diet}
\alias{diet}
\docType{data}
\title{Diet and heart data}
\description{
The \code{diet} data frame has 337 rows and 14 columns.
The data concern a subsample of subjects drawn from larger cohort
studies of the incidence of coronary heart disease (CHD). These subjects
had all completed a 7-day weighed dietary survey while taking part in
validation studies of dietary questionnaire methods. Upon the closure of
the MRC Social Medicine Unit, from where these studies were directed, it
was found that 46 CHD events had occurred in this group, thus allowing a
serendipitous study of the relationship between diet and the incidence
of CHD. 
}
\format{
This data frame contains the following columns:
\tabular{rl}{
\code{id}: \tab subject identifier, a numeric vector. \cr
\code{doe}: \tab date of entry into follow-up study, a
                 \code{\link{Date}} variable. \cr
\code{dox}: \tab date of exit from the follow-up study, a
                 \code{\link{Date}} variable. \cr
\code{dob}: \tab date of birth, a
                 \code{\link{Date}} variable. \cr
\code{y}: \tab number of years at risk, a numeric vector. \cr
\code{fail}: \tab status on exit, a numeric vector (codes 1, 3 and
13 represent CHD events) \cr
\code{job}: \tab occupation, a factor with levels
                 \code{Driver} 
                 \code{Conductor} 
                 \code{Bank worker} \cr
\code{month}: \tab month of dietary survey, a numeric vector \cr
\code{energy}: \tab total energy intake (kCal per day/100), a numeric
vector \cr
\code{height}: \tab (cm), a numeric vector \cr
\code{weight}: \tab (kg), a numeric vector \cr
\code{fat}: \tab   fat intake (10 g/day), a numeric vector \cr
\code{fibre}: \tab dietary fibre intake (10 g/day), a numeric vector \cr
\code{energy.grp}: \tab high daily energy intake, a factor with levels
\code{<=2750 KCal}
\code{>2750 KCal} \cr
\code{chd}: \tab CHD event, a numeric vector (1=CHD event, 0=no event) \cr
}
}
\source{
The data are described and used extensively by Clayton and Hills,
Statistical Models in Epidemiology, Oxford University Press,
Oxford:1993. They were rescued from destruction by David Clayton and
reentered from paper printouts.
}
\examples{
data(diet)
# Illustrate the follow-up in a Lexis diagram
Lexis.diagram( age=c(30,75), date=c(1965,1990),
               entry.date=cal.yr(doe), exit.date=cal.yr(dox), birth.date=cal.yr(dob), 
               fail=(fail>0), pch.fail=c(NA,16), col.fail=c(NA,"red"), cex.fail=1.0,
               data=diet )

}
\keyword{datasets}

\name{M.dk}
\alias{M.dk}
\docType{data}
\title{Mortality in Denmark 1974 ff.}
\description{
  Mortality in one-year classes of age (0-98,99+) and period (1974 ff.) in Denmark.
}
\usage{data(M.dk)}
\format{
  A data frame with 6400 observations on the following 6 variables.
  \describe{
    \item{\code{A}}{Age-class, 0-98, 99:99+}
    \item{\code{sex}}{Sex. 1:males, 2:females}
    \item{\code{P}}{Period (year) of death}
    \item{\code{D}}{Number of deaths}
    \item{\code{Y}}{Number of person-years}
    \item{\code{rate}}{Mortality rate per 1000 person-years}
  }
}
\details{
  Deaths in ages over 100 are in the class labelled 99. Risk time is
  computed by tabulation of the risk time in \code{\link{Y.dk}}, except
  for the class 99+ where the average of the population size in ages
  99+ at the first and last date of the year is used.}
\source{
  \url{http://www.statistikbanken.dk/statbank5a/SelectTable/omrade0.asp?SubjectCode=02&PLanguage=1&ShowNews=OFF}
}
\examples{
data(M.dk)
str(M.dk)

zz <- xtabs( rate ~ sex+A+P, data=M.dk )
zz[zz==0] <- NA # 0s makes log-scale plots crash
par(mfrow=c(1,2), mar=c(0,0,0,0), oma=c(3,3,1,1), mgp=c(3,1,0)/1.6 )
for( i in 1:2 )
{
matplot( dimnames(zz)[[2]], zz[i,,],
         lty=1, lwd=1, col=rev(heat.colors(37)),
         log="y", type="l", ylim=range(zz,na.rm=TRUE),
         ylab="", xlab="", yaxt="n" )
text( 0, max(zz,na.rm=TRUE), c("M","F")[i], font=2, adj=0:1, cex=2, col="gray" )
if( i==1 ) axis( side=2, las=1 )
}
mtext( side=1, "Age", line=2, outer=TRUE )
mtext( side=2, "Mortality rate", line=2, outer=TRUE )
}
\keyword{datasets}

\name{LexisScarf}
\alias{LexisScarf}
\title{
Generate a knitting pattern for a tube-shaped scarf with Lexis lines
}
\description{
The function generates a dotted pattern that refers to stitches of a
knitting pattern. Since there are several colors involved the back side
of the knit becomes quite messy, hence the pattern is for a tube-shaped
scarf, that will have all the messiness on the inside. So use a circular
needle for the work. And send the author a picture of your finished scarf.
}
\usage{
LexisScarf(nw = 4,   # fields around (width of scarf)           
           nl = 20,  # fields along  (length)           
           nm = 23,  # no. stitches per field   
           np = 30,  # no. persons              
        scale = 1.5, # maximal FU in units of nw
          clr = c("red", "blue", "limegreen"))
}
\arguments{
  \item{nw}{number of fields around the tube}
  \item{nl}{number of fields along the tube}
  \item{nm}{no. stitches per field}
  \item{np}{no. lifelines (persons)}
  \item{scale}{maximal length of life lines in units of \code{nw}}
  \item{clr}{vector of colors to be used for life lines}
}
\details{The function produces a graph; ask for a plotting area with
  aspect ratio of \code{nl} to \code{nw}.
}
\value{NULL. The function is used for its side-effect of
  producing a graph of the knitting pattern.
}
\author{Bendix Carstensen,
  \url{http://bendixcarstensen.com},
  \email{b@bxc.dk}}
\examples{
# pdf("scarf.pdf", height = 10, width = 50)
LexisScarf()
# dev.off()
}
\keyword{manip}

\name{crr.Lexis}
\alias{crr.Lexis}
\title{Fit a competing risks regression model (Fine-Gray model) using a
  Lexis object)
}

\description{
Fits a competing risks regression model using a \code{\link{Lexis}}
object assuming that every person enters at time 0 and exits at time
\code{lex.dur}. Thus is only meaningful for Lexis objects with one record
per person, (so far).
}

\usage{
crr.Lexis( obj, mod, quiet=FALSE, ...)
}

\arguments{
  \item{obj}{A Lexis object; variables in \code{mod} are taken from this.}
  \item{mod}{Formula, with the l.h.s. a character constant equal to a
    level of \code{obj$lex.Xst}, and the r.h.s. a model formula
    interpreted in \code{obj}.}
  \item{quiet}{Logical indicating whether a brief summary should be printed.}
  \item{\dots}{Further arguments passed on to \code{\link[cmprsk:crr]{crr}}.}
}

\details{
This function is a simple wrapper for \code{crr}, allowing a
formula-specification of the model (which allows specifications of
covariates on the fly), and utilizing the structure of Lexis
objects to simplify specification of the outcome. Prints a summary of
the levels used as event, competing events and censoring.

By the structure of the \code{\link{Lexis}} object it is not necessary
to indicate what the censoring code or competing events are, that is
automatically derived from the \code{Lexis} object.

Currently only one state is allowed as l.h.s. (response) in \code{mod}.
}

\value{
A \code{\link[cmprsk:crr]{crr}} object (which is a list), with two extra
elements in the list, \code{model.Lexis} - the model formula supplied,
and \code{transitions} - a table of transitions and censorings showing
which transition was analysed and which were taken as competing events.
}

\author{Bendix Carstensen, \url{http://bendixcarstensen.com} }

\seealso{
  \code{\link[cmprsk:crr]{crr}}, \code{\link{Lexis}}
}

\examples{
# Thorotrats patients, different histological types of liver cancer
# Load thorotrast data, and restrict to exposed
data(thoro)
tht <- thoro[thoro$contrast==1,]
# Define exitdate as the date of livercancer
tht$dox <- pmin( tht$liverdat, tht$exitdat, na.rm=TRUE )
tht <- subset( tht, dox > injecdat )
# Convert to calendar years in dates
tht <- cal.yr( tht )

# Set up a Lexis object with three subtypes of liver cancer and death
tht.L <- Lexis( entry = list( per = injecdat,
                              tfi = 0 ),
                 exit = list( per = dox ),
          exit.status = factor( 1*hepcc+2*chola+3*hmang+
                                4*(hepcc+chola+hmang==0 & exitstat==1),
                                labels=c("No cancer","hepcc","chola","hmang","Dead") ),
                 data = tht )
summary( tht.L )

# Show the transitions
boxes( tht.L, boxpos=list(x=c(20,rep(80,3),30),
                          y=c(60,90,60,30,10) ),
              show.BE="nz", scale.R=1000 )

# Fit a model for the Hepatocellular Carcinoma as outcome
# - note that you can create a variable on the fly:
library( cmprsk )
hepcc <- crr.Lexis( tht.L, "hepcc" ~ volume + I(injecdat-1940) )
hepcc$model.Lexis
hepcc$transitions

# Models for the three other outcomes:
chola <- crr.Lexis( tht.L, "chola" ~ volume + I(injecdat-1940) )
hmang <- crr.Lexis( tht.L, "hmang" ~ volume + I(injecdat-1940) )
dead  <- crr.Lexis( tht.L, "Dead"  ~ volume + I(injecdat-1940) )

# Compare the effects
# NOTE: This is not necessarily a joint model for all transitions.
zz <- rbind( ci.exp(hepcc),
             ci.exp(chola),
             ci.exp(hmang),
             ci.exp(dead) )
zz <- cbind( zz[c(1,3,5,7)  ,],
             zz[c(1,3,5,7)+1,] )
rownames( zz ) <- c("hepcc","chola","hmang","dead")
colnames( zz )[c(1,4)] <- rownames( ci.exp(chola) )
round( zz, 3 )
}

\keyword{survival}

\name{expand.data}
\alias{expand.data}
\title{
  Function to expand data for regression analysis of interval censored
  data.
  }
\description{
  This is a utility function.
  
  The original records with \code{first.well}, \code{last.well} and
  \code{first.ill} are
  expanded to multiple records; several for each interval where the
  person is known to be well and one where the person is known to fail.
  At the same time columns for the covariates needed to estimate rates
  and the response variable are generated.
  }
\usage{
expand.data(fu, formula, breaks, data)
}
\arguments{
  \item{fu}{A 3-column matrix with \code{first.well}, \code{last.well} and
  \code{first.ill} in each row.}
  \item{formula}{Model fromula, used to derive the model matrix.}
  \item{breaks}{Defines the intervals in which the baseline rate is
    assumed constant. All follow-up before the first and after the
    last break is discarded.}
  \item{data}{Datafrem in which \code{fu} and \code{formula} is interpreted.}
}
\value{
  Returns a list with three components
  \item{rates.frame}{Dataframe of covariates for estimation of the
    baseline rates --- one per interval defined by \code{breaks}.}
  \item{cov.frame}{Dataframe for estimation of the covariate effects. A
    data-framed version of the designmatrix from \code{formula}.}
  \item{y}{Response vector.}
}
\references{
  B Carstensen: Regression models for interval censored
  survival data: application to HIV infection in Danish homosexual
  men. Statistics in Medicine, 15(20):2177-2189, 1996.
  }
\author{
  Martyn Plummer, \email{martyn.plummer@r-project.org}
  }
\seealso{
  \code{\link{Icens}}
  \code{\link{fit.mult}}
  \code{\link{fit.add}}
  }
\keyword{ models }
\keyword{ regression }
\keyword{ survival }

\name{effx.match}
\alias{effx.match}
\title{Function to calculate effects for individually matched case-control studies}
\description{
The function calculates the effects of an exposure on a response,
possibly stratified by a stratifying variable, and/or controlled for one
or more confounding variables. 
}
\usage{
effx.match(response,
exposure,
match,
strata=NULL,
control=NULL,
base=1,
digits=3,
alpha=0.05,
data=NULL) 
}
%- maybe also 'usage' for other objects documented here.
\arguments{
\item{response}{The \code{response} variable - must be numeric}
\item{exposure}{The \code{exposure} variable can be numeric or a factor}
\item{match}{The variable which identifies the matched sets}
\item{strata}{The \code{strata} stratifying variable - must be a factor}
\item{control}{ The \code{control} variable(s). These are passed as a
  list if there are more than one of them.} 
\item{base}{Baseline for the effects of a categorical exposure, default 1}
\item{digits}{Number of significant digits for the effects, default 3}
\item{alpha}{1 - confidence level}
\item{data}{\code{data} refers to the data used to evaluate the function}
}
\details{Effects are calculated odds ratios.
The function is a wrapper for clogit, from the survival package.
The k-1 effects for a categorical  exposure with k levels are relative 
to a baseline which, by default, is the first level. The effect of a metric (quantitative) 
exposure is calculated per unit of exposure.
The exposure variable can be numeric or a factor, but if it is an ordered factor the order will be ignored.
}
\value{
%  ~Describe the value returned
 % If it is a LIST, use
  \item{comp1 }{Effects of exposure}
  \item{comp2 }{Tests of significance}
% ...
}
\references{ www.mhills.pwp.blueyonder.co.uk }
\author{Michael Hills}
%\note{ ~~further notes~~ }

%\seealso{ ~~objects to See Also as \code{\link{~~fun~~}}, ~~~ }
\examples{
library(Epi)
library(survival)
data(bdendo)

# d is the case-control variable, set is the matching variable.
# The variable est is a factor and refers to estrogen use (no,yes)
# The variable hyp is a factor with 2 levels and refers to hypertension (no, yes)
# effect of est on the odds of being a case
effx.match(d,exposure=est,match=set,data=bdendo)
# effect of est on the odds of being a case, stratified by hyp
effx.match(d,exposure=est,match=set,strata=hyp,data=bdendo)
# effect of est on the odds of being a case, controlled for hyp
effx.match(d,exposure=est,match=set,control=hyp,data=bdendo)
}
\keyword{ models }
\keyword{ regression }

\name{fit.mult}
\alias{fit.mult}
\title{
  Fits a multiplicative relative risk model to interval censored data.
  }
\description{
  Utility function.
  
  The model fitted assumes a piecewise constant baseline rate in
  intervals specified by the argument \code{breaks}, and a
  multiplicative relative risk function.  
}
\usage{
  fit.mult( y, rates.frame, cov.frame, start )
  }
\arguments{
  \item{y}{Binary vector of outcomes}
  \item{rates.frame}{Dataframe expanded from the original data by
    \code{\link{expand.data}}, cooresponding to covariates for the rate
    parameters.}
  \item{cov.frame}{ do., but covariates corresponding to the
    \code{formula} argument of \code{\link{Icens}}} 
  \item{start}{Starting values for the rate parameters. If not supplied,
       then starting values are generated.}
  }
\details{
  The model is fitted by alternating between two generalized linear
  models where one estimates the underlying rates in the intervals, and
  the other estimates the log-relative risks.
  }
\value{
  A list with three components:
  \item{rates}{A glm object from a binomial model with log-link,
    estimating the baseline rates.}
  \item{cov}{A glm object from a binomial model with complementary
    log-log link, estimating the log-rate-ratios}
  \item{niter}{Nuber of iterations, a scalar}
  }
\references{
  B Carstensen: Regression models for interval censored
  survival data: application to HIV infection in Danish homosexual
  men. Statistics in Medicine, 15(20):2177-2189, 1996.

  CP Farrington: Interval censored survival data: a generalized linear
  modelling approach. Statistics in Medicine, 15(3):283-292, 1996.
  }
\author{
  Martyn Plummer, \email{martyn.plummer@r-project.org},
  Bendix Carstensen, \email{b@bxc.dk} }
\seealso{
  \code{\link{Icens}}
  \code{\link{fit.add}}
  }
\examples{
  data( HIV.dk ) 
  }
\keyword{ models }
\keyword{ regression }
\keyword{ survival }

\name{plot.Lexis}
\alias{plot.Lexis}
\alias{points.Lexis}
\alias{lines.Lexis}
\alias{PY.ann}
\alias{PY.ann.Lexis}
\title{Lexis diagrams}
\description{
  The follow-up histories represented by a Lexis object can be plotted
  using one or two dimensions.  The two dimensional plot is a Lexis
  diagram showing follow-up time simultaneously on two time scales.
}
\usage{
\method{plot}{Lexis}(x=Lexis( entry=list(Date=1900,Age=0), exit=list(Age=0) ),
                     time.scale = NULL, type="l", breaks="lightgray", ...)
\method{points}{Lexis}(x, time.scale = options()[["Lexis.time.scale"]] , ...)
\method{lines}{Lexis}(x, time.scale = options()[["Lexis.time.scale"]], ...)
\method{PY.ann}{Lexis}(x, time.scale = options()[["Lexis.time.scale"]], digits=1, ...)
}
\arguments{
  \item{x}{An object of class \code{Lexis}. The default is a bogus \code{Lexis}
           object, so that \code{plot.Lexis} can be called without the first
           argument and still produce a(n empty) Lexis diagram. Unless arguments
           \code{xlim} and \code{ylim} are given in this case the diagram is
           looking pretty daft.}
  \item{time.scale}{A vector of length 1 or 2 giving the time scales to
    be plotted either by name or numerical order}
  \item{type}{Character indication what to draw: "n" nothing (just set up the
    diagram), "l" - liefelines, "p" - endpoints of follow-up, "b" - both
    lifelines and endpoints.}
  \item{breaks}{a string giving the colour of grid lines to be drawn
    when plotting a split Lexis object. Grid lines can be suppressed by
    supplying the value \code{NULL} to the \code{breaks} argument}
  \item{digits}{Numerical. How many digits after the demimal points should be
       when plotting the person-years.}
  \item{\dots}{Further graphical parameters to be passed to the plotting
    methods.

    Grids can be drawn (behind the life lines) using the following
    parameters in \code{plot}: 
    \itemize{
      \item \code{grid} If logical, a background grid is set up
      using the axis ticks. If a list, the first component is used as
      positions for the vertical lines and the last as positions for the
      horizontal. If a nunerical vector, grids on both axes are set up
      using the distance between the numbers.
      \item \code{col.grid="lightgray"} Color of the background grid.
      \item \code{lty.grid=2} Line type for the grid.
      \item \code{coh.grid=FALSE} Should a 45 degree grid be plotted?}
} }
\details{
  The plot method for \code{Lexis} objects traces ``life lines'' from
  the start to the end of follow-up.  The \code{points} method plots
  points at the end of the life lines.

  If \code{time.scale} is of length 1, the life lines are drawn
  horizontally, with the time scale on the X axis and the id value on the Y
  axis. If \code{time.scale} is of length 2, a Lexis diagram is
  produced, with diagonal life lines plotted against both time scales
  simultaneously.

  If \code{lex} has been split along one of the time axes by a call to
  \code{splitLexis}, then vertical or horizontal grid lines are plotted
  (on top of the life lines) at the break points.
  
  \code{PY.ann} writes the length of each (segment of) life line at the middle
  of the line. Not advisable to use with large cohorts. Another example is
  in the example file for \code{\link{occup}}.
}
\author{Martyn Plummer}
\examples{
# A small bogus cohort
xcoh <- structure( list( id = c("A", "B", "C"),
                      birth = c("14/07/1952", "01/04/1957", "10/06/1987"),
                      entry = c("04/08/1965", "08/09/1972", "23/12/1991"),
                       exit = c("27/06/1997", "23/05/1995", "24/07/1998"),
                       fail = c(1, 0, 1) ),
                     .Names = c("id", "birth", "entry", "exit", "fail"),
                  row.names = c("1", "2", "3"),
                      class = "data.frame" )

# Convert the character dates into numerical variables (fractional years)
xcoh$bt <- cal.yr( xcoh$birth, format="\%d/\%m/\%Y" )
xcoh$en <- cal.yr( xcoh$entry, format="\%d/\%m/\%Y" )
xcoh$ex <- cal.yr( xcoh$exit , format="\%d/\%m/\%Y" )

# See how it looks
xcoh

# Define as Lexis object with timescales calendar time and age
Lcoh <- Lexis( entry = list( per=en ),
                exit = list( per=ex, age=ex-bt ),
         exit.status = fail,
                data = xcoh )

# Default plot of follow-up
plot( Lcoh )
# Show follow-up time
PY.ann( Lcoh )

# Show exit status
plot( Lcoh, type="b" )
# Same but failures only
plot( Lcoh, type="b", pch=c(NA,16)[Lcoh$fail+1] )

# With a grid and deaths as endpoints
plot( Lcoh, grid=0:10*10, col="black" )
points( Lcoh, pch=c(NA,16)[Lcoh$lex.Xst+1] )
# With a lot of bells and whistles:
plot( Lcoh, grid=0:20*5, col="black", xaxs="i", yaxs="i",
      xlim=c(1960,2010), ylim=c(0,50), lwd=3, las=1 )
points( Lcoh, pch=c(NA,16)[Lcoh$lex.Xst+1], col="red", cex=1.5 )
}
\seealso{\code{\link{Lexis}}, \code{\link{splitLexis}}}
\keyword{hplot}
\keyword{aplot}

\name{stat.table}
\alias{stat.table}
\alias{print.stat.table}
\title{Tables of summary statistics}
\description{
  \code{stat.table} creates tabular summaries of the data, using a
  limited set of functions. A list of index variables is used
  to cross-classify summary statistics. It does NOT work inside \code{with()}!
}
\usage{
stat.table(index, contents = count(), data, margins = FALSE)
\method{print}{stat.table}(x, width=7, digits,...)
}
\arguments{
  \item{index}{A factor, or list of factors, used for cross-classification.
    If the list is named, then the names will be used when printing the
    table. This feature can be used to give informative labels to the
    variables.}
  \item{contents}{A function call, or list of function calls. Only a
    limited set of functions may be called (See Details below).  If the
    list is named, then the names will be used when printing the table.}
  \item{data}{an optional data frame containing the variables to be
    tabulated. If this is omitted, the variables will be searched for in the
    calling environment.}
  \item{margins}{a logical scalar or vector indicating which  marginal
    tables are to be calculated.  If a vector, it should be the same
    length as the \code{index} argument: values corresponding to
    \code{TRUE} will be retained in marginal tables.}
  \item{x}{an object of class \code{stat.table}.}
  \item{width}{a scalar giving the minimum column width when printing.}
  \item{digits}{a scalar, or named vector, giving the number
    of digits to print after the decimal point. If a named vector is used,
    the names should correspond to one of the permitted functions (See
    Details below) and all results obtained with that function will be
    printed with the same precision.}
  \item{...}{further arguments passed to other print methods.}
}
\details{
  This function is similar to \code{tapply}, with some enhancements:
  multiple summaries of multiple variables may be mixed in the
  same table; marginal tables may be calculated; columns and rows may
  be given informative labels; pretty printing may be controlled by the
  associated print method.

  This function is not a replacement for \code{tapply} as it also has
  some limitations.  The only functions that may be used in the
  \code{contents} argument  are: \code{\link{count}},
  \code{\link{mean}}, \code{\link{weighted.mean}}, \code{\link{sum}},
  \code{\link{quantile}}, \code{\link{median}}, \code{\link{IQR}},
  \code{\link{max}}, \code{\link{min}}, \code{\link{ratio}}, 
  \code{\link{percent}}, and \code{\link{sd}}.
  
  The \code{count()} function, which is the default, simply creates a
  contingency table of counts.  The other functions are applied to
  each cell created by combinations of the \code{index} variables.
}
\value{
  An object of class \code{stat.table}, which is a multi-dimensional
  array. A print method is available to create formatted one-way and
  two-way tables.
}
\author{Martyn Plummer}
\note{
  The permitted functions in the contents list %are overloaded functions that
  are defined inside \code{stat.table}.  They have the same interface as
  the functions callable from the command line, except for two
  differences. If there is an argument \code{na.rm} then its default
  value is always \code{TRUE}. A second difference is that the
  \code{quantile} function can only produce a single quantile in each call.
}

\seealso{\code{\link{table}}, \code{\link{tapply}},
\code{\link{mean}}, \code{\link{weighted.mean}},
\code{\link{sum}}, \code{\link{quantile}},
\code{\link{median}}, \code{\link{IQR}},
\code{\link{max}}, \code{\link{min}}, \code{\link{ratio}},
\code{\link{percent}}, \code{\link{count}}, \code{\link{sd}}.}
\examples{
data(warpbreaks)
# A one-way table
stat.table(tension,list(count(),mean(breaks)),data=warpbreaks)
# The same table with informative labels
stat.table(index=list("Tension level"=tension),list(N=count(),
           "mean number of breaks"=mean(breaks)),data=warpbreaks)

# A two-way table
stat.table(index=list(tension,wool),mean(breaks),data=warpbreaks)  
# The same table with margins over tension, but not wool
stat.table(index=list(tension,wool),mean(breaks),data=warpbreaks,
           margins=c(TRUE, FALSE))

# A table of column percentages
stat.table(list(tension,wool), percent(tension), data=warpbreaks)
# Cell percentages, with margins
stat.table(list(tension,wool),percent(tension,wool), margin=TRUE,
           data=warpbreaks)

# A table with multiple statistics
# Note how each statistic has its own default precision
a <- stat.table(index=list(wool,tension),
                contents=list(count(),mean(breaks),percent (wool)),
                data=warpbreaks)
print(a)
# Print the percentages rounded to the nearest integer
print(a, digits=c(percent=0))

}
\keyword{iteration}
\keyword{category}

\name{mod.Lexis}
\alias{glmLexis}
\alias{gamLexis}
\alias{coxphLexis}
\alias{glm.Lexis}
\alias{gam.Lexis}
\alias{coxph.Lexis}
\title{Fit intensity models to follow-up data in Lexis objects  
}
\description{
Modeling intensities based on Lexis objects, exploiting the structure of the
Lexis objects where the events and risk time have predefined
representations. This allows a simpler syntax than the
traditional explicit modeling using \code{\link{glm}}, \code{\link[mgcv]{gam}}
and \code{\link[survival]{coxph}}. Requires that \code{lex.Cst} and \code{lex.Xst}
are defined as factors.

But it is just a set of wrappers for
\code{glm}, \code{gam} and \code{coxph}.
}
\usage{
glmLexis(Lx, formula,
         from = preceding(Lx, to), to = absorbing(Lx),
         paired = FALSE, link = "log", scale = 1, verbose = TRUE, \dots )     
gamLexis(Lx, formula,
         from = preceding(Lx, to), to = absorbing(Lx),
         paired = FALSE, link = "log", scale = 1, verbose = TRUE, \dots )     
coxphLexis(Lx, formula,
           from = preceding(Lx, to), to = absorbing(Lx),
           paired = FALSE, verbose = TRUE, \dots )     
  glm.Lexis( Lx,         # Lexis object	
        formula,         # ~ model	
           from = preceding(Lx, to), # 'from' states
             to = absorbing(Lx)    , # 'to' states
         paired = FALSE, # only the pairwise
           link = "log", # link function
          scale = 1,     # scaling of PY
        verbose = TRUE,  # report what is done?
            \dots )        # further arguments to glm
  gam.Lexis( Lx,         # Lexis object	
        formula,         # ~ model	
           from = preceding(Lx, to), # 'from' states
             to = absorbing(Lx)    , # 'to' states	
         paired = FALSE, # only the pairwise
           link = "log", # link function
          scale = 1,     # scaling of PY
        verbose = TRUE,  # report what is done?
            \dots )        # further arguments to gam
coxph.Lexis( Lx,         # Lexis object	
        formula,         # timescale ~ model	
           from = preceding(Lx, to), # 'from' states
             to = absorbing(Lx)    , # 'to' states	
         paired = FALSE, # only the pairwise
        verbose = TRUE,  # report what is done?
            \dots )        # further arguments to coxph
}
\arguments{
  \item{Lx}{A \code{\link{Lexis}} object representing cohort follow-up.
}
  \item{formula}{Model formula describing the model for the
    intensity(-ies). For \code{glm} and \code{gam}, the formula should be
    one-sided; for \code{coxph} the formula should be two-sided and have
    the name of the time-scale used for baseline hazard as the l.h.s.
}
  \item{from}{Character vector of states \bold{from} which transitions
    are considered. May also be an integer vector in which case the
    reference will be to the position of levels of
    \code{lex.Cst}. Defaults to the collection of transient states
    immediately preceding the absorbing states. 
}
  \item{to}{Character vector of states \bold{to} which a transition is
    considered an event. May also be an integer vector in which case the
    reference will be to the position of levels of \code{lex.Xst}.
    Defaults to the set of absorbing states.  
}
  \item{paired}{Logical. Should the states mentioned in \code{to},
    rep. \code{from} be taken as pairs, indicating the only transitions
    modeled. If \code{FALSE} all transitions from any of the states in
    \code{from} to any states in \code{to} are modeled.
}
  \item{link}{Character; name of the link function used, allowed values
    are \code{'log'} (the default), \code{'identity'} and \code{'sqrt'},
    see the family \code{\link{poisreg}}. 
}
  \item{scale}{Scalar. \code{lex.dur} is divided by this number before 
    analysis, so that you can get resulting rates on a scale of your wish. 
}
  \item{verbose}{Print information on the states modeled?
}
  \item{\dots}{Further arguments passed on to \code{glm}, \code{glm} or
    \code{coxph} 
}
}

\details{
  The functions with and without dots in the name are identical
  
  The \code{glm} and \code{gam} models are fitted using the family
  \code{\link{poisreg}} which is a bit faster than the traditional
  \code{poisson} family. The response variable for this family is a
  two-column vector of events and person-time respectively, so the
  predictions, for example using \code{\link{ci.pred}} does not require
  \code{lex.dur} (and would ignore this) as variable in the
  \code{newdata}. \code{ci.pred} will return the estimated rates in
  units of the \code{lex.dur} in the \code{Lexis} object, scaled by
  \code{scale}, which has a default value of 1.

  The default is to model all transitions into any absorbing state by
  the same model (how wise is that??). If only \code{from} is given,
  \code{to} is set to all states reachable from \code{from}, which may
  be a really goofy model and if so a warning is issued. If only
  \code{to} is given, \code{from} is set to the collection of states
  from which \code{to} can be reached directly --- see
  \code{\link{preceding}} and its cousins. This convention means that if
  you have a \code{\link{Lexis}} object representing a simple survival
  analysis, with states, say, "alive" and "dead", you can dispense with
  the \code{from} and \code{to} arguments.

  Occasionally you only want to model a subset of the possible
  transitions from states in \code{from} to states in \code{to}, in
  which case you specify \code{from} and \code{to} as character vectors
  of the same length and set \code{paired=TRUE}. Then only transitions
  \code{from[i]} to \code{to[i]}, \code{i}=1,2,... will be modeled.
  
  There is no working \code{update} functions for these objects (yet). 
  
  Strictly speaking, it is a bit counter-intuitive to have the time-scale
  on the l.h.s. of the formula for the \code{coxph} since the time scale
  is also a predictor of the occurrence rate. On the other hand, calling
  \code{coxph} directly would also entail having the name of the time
  scale in the \code{Surv} object on the l.h.s. of the formula. So the
  inconsistency is merely carried over from \code{coxph}.
}

\value{\code{glmLexis} returns a \code{\link{glm}} object, which is
  also of class \code{glm.lex},
  \code{gamLexis} returns a \code{\link[mgcv]{gam}} object, which is
  also of class \code{gam.lex}, and
  \code{coxphLexis} returns a \code{\link[survival]{coxph}} object, which is
  also of class \code{coxph.lex}. These extra class attributes are meant
  to facilitate the (still pending) implementation of an \code{update} function.

  The returned objects all have an extra attribute, \code{Lexis} which
  is a list with entries
  \code{data}, the name of the \code{Lexis} object modeled (note that it
  is \emph{not} the object, only the name of it, which may not be portable);
  \code{trans}, a character vector of transitions modeled;
  \code{formula}, the model formula; and
  \code{scale}, the scaling applied to \code{lex.dur} before modeling. 
  
  Only the \code{glm} and \code{gam} objects have the \code{scale} element
  in the list; a scalar indicating the scaling of \code{lex.dur} before
  modeling. Note that the formula component of the \code{Lexis}
  attribute of a \code{coxph} object is a
  two-sided formula with the baseline time scale as the l.h.s.
}

\author{
Bendix Carstensen, \url{http://bendixcarstensen.com}.
}

\seealso{
  \code{\link{Lexis}},
  \code{\link{cutLexis}},
  \code{\link{mcutLexis}},
  \code{\link{addCov.Lexis}},
  \code{\link{absorbing}},
  \code{\link{transient}}
}

\examples{
library( Epi )
library( survival )
data( DMlate )

# Lexis object of total follow-up
mL <- Lexis( entry = list(age=dodm-dobth,per=dodm),
              exit = list(per=dox),
       exit.status = factor(!is.na(dodth),labels=c("Alive","Dead")),
              data = DMlate )

# Cut follow-up at start of insulin use
cL <- cutLexis( mL, cut = mL$doins,
              timescale = "per",
              new.state = "Ins",
       precursor.states = "Alive" )

# Split follow-up on age-axis
system.time( sL <- splitLexis( cL, breaks=0:25*4, time.scale="age") )
# ( consider splitMulti from the popEpi package )
summary( sL )

# glm models for rates based on the time-split dataset by insulin and sex

# Proportional hazards model with insulin as time-dependent variable
# - uses the defaul of modeling all transitions from both transient
# states ("Alive" and "Ins") to the absorbing state ("Dead"). 
mt <- glmLexis( sL, ~ sex + lex.Cst + Ns(age,knots=c(15,3:8*10)) )

# prediction of mortality rates from "Alive" with and without PH assumption
nA <- data.frame( age=40:70, sex="M", lex.Cst="Alive" )
nI <- data.frame( age=40:70, sex="M", lex.Cst="Ins" )
matshade( nA$age, cbind( ci.pred(mt,nA),
                         ci.pred(mt,nI) )*1000, plot=TRUE,
          lwd=3, lty=1, log="y", col=c("black","blue","red"),
          xlab="Age", ylab="Mortality per 1000 PY" )
 
# gam models may take some time to run so we leave it out
\dontrun{
mt.gam <- gamLexis( sL, ~ sex + lex.Cst + s(age), to="Dead",
                     scale=1000 )
        }

# Fit a Cox model for mortality with age as baseline time scale and
# insulin (lex.Cst) as time-dependent covariate 
mt.cox <- coxphLexis( sL, age ~ sex + lex.Cst, c("Alive","Ins"), "Dead" )

# Pretty much the same results for regression paramters as the glm:
  ci.exp( mt    , subset="ex" )
# ci.exp( mt.gam, subset="ex" )
  ci.exp( mt.cox, subset="ex" )
}
\keyword{models}

\name{rcutLexis}
\alias{rcutLexis}
\title{
A function to cut follow-up at intermediate event times.
}
\description{
Cuts follow-up at intermediate event times, multiple events per person
are allowed, as well as recurrences of the sme type of event. The
resulting states only refer to the last assumed state, unlike the result
from \code{\link{mcutLexis}}.
}
\usage{
rcutLexis( Lx, cut,
    timescale = 1,
    precursor.states = transient(Lx))
}
\arguments{
  \item{Lx}{
  A \code{\link{Lexis}} object to be amended,.
}
  \item{cut}{
  A data frame with columns \code{lex.id}, \code{cut} (event times) and
  \code{new.state} (event type, character)
}
  \item{timescale}{
  What time scale do values in \code{cut$cut} refer to. Numeric or character.
}
  \item{precursor.states}{an optional vector of states to be considered
    as "less severe" than \code{new.state}. See Details in the
    documentation of \code{\link{cutLexis}}
}
}
\value{A \code{\link{Lexis}} object with follow-up cut at the event
  times supplied in \code{cut}}
\author{
Bendix Carstensen, \url{http://bendixcarstensen.com}
}
\seealso{
  \code{\link{cutLexis}},
  \code{\link{mcutLexis}},
  \code{\link{addCov.Lexis}},
  \code{\link{Lexis}},
  \code{\link{splitLexis}}
}
\examples{
df <- data.frame(lex.id = rep(c(3, 7), c(3, 5)))
df$new.state <- sample(LETTERS[2:4], 8, r = TRUE) 
df$cut <- round(runif(8) * 100) + 1
df

Lx <- Lexis( exit = list(time=c(89, 97)),
               id = c(3, 7),
      exit.status = factor(c("A", "X")) )
Lx 
rcutLexis(Lx, df, pre = "A")
        }
\keyword{survival}

\name{ci.eta}
\alias{ci.eta}
\title{
  Linear predictor (\code{eta}) from a formula, coefficients, vcov and a
  prediction frame.
}
\description{
  Computes the linear predictor with its confidence limits from the
  model formula and the estimated parameters with the vcov.
}
\usage{
ci.eta(form, cf, vcv, newdata,
       name.check = TRUE,
       alpha = 0.05, df = Inf, raw = FALSE)
}
\arguments{
  \item{form}{A model formula. A one-sided formula will suffice; left
    side will be ignored if two-sided.
}
  \item{cf}{Coefficients from a model using \code{formula}.
}
  \item{vcv}{variance-covariance matrix from a model using \code{formula}.
}
  \item{newdata}{Prediction data frame with variables used in
    \code{formula}. Can also be a list of 2 or 4 prediction frames, for
    details see \code{\link{ci.lin}}.
}
  \item{name.check}{Logical. Check if the column names of the genereated
    model matrix are identical to the names of the supplied \code{coef} vector.
}
  \item{alpha}{Significance level for calculation of c.i.
}
  \item{df}{Integer. Number of degrees of freedom in the t-distribution
          used to compute the quantiles used to construct the
          confidence intervals.
}
  \item{raw}{Logical. Should predictions and their vcov be returned
    instead of predictions and confidence limits?
}
}
\details{  
  Does pretty much the same as \code{\link{ci.lin}}, but requires only a
  formula and coefficients with vcov and not a full model
  object. Designed to avoid saving entire (homongously large) model
  objects and still be able to compute predictions. But only the linear
  predictor is returned, if there is a link in your model function it is
  your own responsibility to back-transform. If the model formula
  contains reference to vectors of spline knots or similar these must be
  in the global environment.

  There is no guarantee that this function works for models that do not
  inherit from \code{lm}. But there is a guarantee that it will not work
  for \code{gam} objects with \code{s()} terms.
}
\value{
  The linear predictor for the \code{newdata} with a confidence interval
  as a \code{nrow(newdata)} by 3 matrix. If \code{raw=TRUE}, a list the linear
  predictor (\code{eta}) and its variance-covariance matrix (\code{var}).
}
\author{
  Bendix Carstensen, \url{http://bendixcarstensen.com}
}
\seealso{\code{\link{ci.lin}}
}
\keyword{models}
\keyword{regression}
\concept{prediction}
\concept{prediction frame}
\concept{linear predictor}

\name{NArray}
\alias{NArray}
\alias{ZArray}
\title{Set up an array of NAs, solely from the list of dimnames
}
\description{Defines an array of NAs, solely from the list of dimnames
}
\usage{
NArray( x, cells=NA )
ZArray( x, cells=0 )
}
\arguments{
  \item{x}{A (possibly named) list to be used as dimnames for the
    resulting array}
  \item{cells}{Value(s) to fill the array}
}
\details{This is a simple useful way of defining arrays to be used for
  collection of results. The point is that everything is defined from
  the named list, so in the process of defining what you want to
  collect, there is only one place in the program to edit. It's just a
  wrapper for \code{array}. \code{ZArray} is just a wrapper for
  \code{NArray} with a different default.
}
\value{An array with \code{dimnames} attribute \code{x}, and all values
  equal to \code{cells}.
}
\author{Bendix Carstensen
}
\examples{
ftable(
NArray( list(Aye = c("Yes", "Si", "Oui"),
             Bee = c("Hum", "Buzz"),
             Sea = c("White", "Black", "Red", "Dead") ) ) )
}

\name{summary.Lexis}
\alias{summary.Lexis}
\alias{print.summary.Lexis}
\title{
  Summarize transitions and risk time from a Lexis object
  }
\description{
  A two-way table of records and transitions classified by states
  (\code{lex.Cst} and \code{lex.Xst}), as well the risk time in each state.
  }
\usage{
  \method{summary}{Lexis}(object,
                        simplify = TRUE,
                           scale = 1,
                              by = NULL,
                           Rates = FALSE,
                      timeScales = FALSE, ...)
  \method{print}{summary.Lexis}(x, ..., digits = 2)
  }
\arguments{
  \item{object}{A Lexis object.}
  \item{simplify}{Should rows with 0 follow-up time be dropped?}
  \item{scale}{Scaling factor for the rates. The calculated rates are
    multiplied by this number.}
  \item{by}{Character vector of name(s) of variable(s) in
    \code{object}. Used to give a separate summaries for subsets of
    \code{object}. If longer than than 1, the interaction between the
    variables is used to stratify the summary. It is also possible to
    supply a vector of length \code{nrow(object)}, in which case the distinct
    values of this will be used to stratify the summary.}
  \item{Rates}{Should a component with transition rates be returned (and
    printed) too?}
  \item{timeScales}{Should the names of the timescales and the indication
    of since which entry also be returned?}
  \item{x}{A \code{Lexis} or \code{summary.Lexis} object.}
  \item{digits}{Number of digits after the decimal separator used when
    printing the summary.}
  \item{...}{Ignored.}
}
\value{
  An object of class \code{summary.Lexis}, a list with two components,
  \code{Transitions} and \code{Rates}, each one a matrix with rows
  classified by states where persons spent time, and columns classified
  by states to which persons transit. The \code{Transitions} contains
  number of transitions and has 4 extra columns with number of records,
  total number of events, total risk time and number of person
  contributing attached.  The \code{Rates} contains the transitions
  rates.

  If the argument \code{Rates} is \code{FALSE} (the default), then only the
  first component of the list is returned.

  If \code{mat} is \code{FALSE} a list is returned, possibly of length 1.
}
\author{
   Bendix Carstensen, \url{http://bendixcarstensen.com}
   }
\examples{
data( nickel )
# Lung cancer deaths and other deaths are coded 1 and 2
nic <- Lexis(data = nickel,
            entry = list(age = agein),
             exit = list(age = ageout,
                         cal = ageout + dob,
                         tfh = ageout - age1st),
       exit.status = factor((icd > 0) + (icd \%in\% c(162,163)),
                            labels = c("Alive", "Other", "Lung") ) )
str( nic )
head( nic )
summary( nic )
# More detailed summary, by exposure level, and rates per 100 PY
summary(nic, by = nic$exposure > 5,
          Rates = TRUE,
          scale = 100)
}
\keyword{survival}

\name{N2Y}
\alias{N2Y}
\title{
Create risk time ("Person-Years") in Lexis triangles from population count data.
}
\description{
Data on population size at equidistant dates and age-classes are
used to estimate person-time at risk in Lexis-triangles, i.e. classes
classified by age, period AND cohort (date of birth). Only works for
data where age-classes have the same width as the period-intervals.
}
\usage{
N2Y( A, P, N,
     data = NULL,
     return.dfr = TRUE)
}
\arguments{
  \item{A}{Name of the age-variable, which should be numeric,
    corresponding to the left endpoints of the age intervals.}
  \item{P}{Name of the period-variable, which should be numeric,
    corresponding to the date of population count.}
  \item{N}{The population size at date \code{P} in age class \code{A}.}
  \item{data}{A data frame in which arguments are interpreted.}
  \item{return.dfr}{Logical. Should the results be returned as a data frame
    (default \code{TRUE}) or as a table.}
}
\value{A data frame with variables \code{A}, \code{P} and \code{Y},
  representing the mean age and period in the Lexis triangles and the
  person-time in them, respectively. The person-time is in units of the
  distance between population count dates.

  If \code{return.dfr=FALSE} a three-way table classified by the left end
  point of the age-classes and the periods and a factor \code{wh} taking
  the values \code{up} and \code{lo} corresponding to upper (early
  cohort) and lower (late cohort) Lexis triangles.
}
\details{The calculation of the risk time from the population figures is
  done as described in: B. Carstensen: Age-Period-Cohort models for the
  Lexis diagram. Statistics in Medicine, 26: 3018-3045, 2007.

  The number of periods in the result is one less than the number
  of dates (\code{nP=length(table(P))}) in the input, so the number of
  distinct values is \code{2*(nP-1)}, because the \code{P} in the output
  is coded differently for upper and lower Lexis triangles.

  The number of age-classes is the same as in the input
  (\code{nA=length(table(A))}), so the number of distinct values is
  \code{2*nA}, because the \code{A} in the output is coded differently
  for upper and lower Lexis triangles.

  In the paper "Age-Period-Cohort models for the Lexis diagram" I
  suggest that the risk time in the lower triangles in the first
  age-class and in the upper triangles in the last age-class are
  computed so that the total risk time in the age-class corresponds to
  the average of the two population figures for the age-class at either
  end of a period multiplied with the period length. This is the method
  used.
}
\references{
  B. Carstensen: Age-Period-Cohort models for the
  Lexis diagram. Statistics in Medicine, 26: 3018-3045, 2007.
}
\author{
  Bendix Carstensen, \url{http://bendixcarstensen.com}
       }
\seealso{
\code{\link{splitLexis}}, \code{\link{apc.fit}}
}
\examples{
# Danish population at 1 Jan each year by sex and age
data( N.dk )
# An illustrative subset
( Nx <- subset( N.dk, sex==1 & A<5 & P<1975 ) )
# Show the data in tabular form
xtabs( N ~ A + P, data=Nx )
# Lexis triangles as data frame
Nt <- N2Y( data=Nx, return.dfr=TRUE )
xtabs( Y ~ round(A,2) + round(P,2), data=Nt )
# Lexis triangles as a 3-dim array
ftable( N2Y( data=Nx, return.dfr=FALSE ) )

# Calculation of PY for persons born 1970 in 1972
( N.1.1972 <- subset( Nx, A==1 & P==1972)$N )
( N.2.1973 <- subset( Nx, A==2 & P==1973)$N )
N.1.1972/3 + N.2.1973/6
N.1.1972/6 + N.2.1973/3
# These numbers can be found in the following plot:

# Blue numbers are population size at 1 January
# Red numbers are the computed person-years in Lexis triangles:
Lexis.diagram(age=c(0,5), date=c(1970,1975), int=1, coh.grid=TRUE )
with( Nx, text(P,A+0.5,paste(N),srt=90,col="blue") )
with( Nt, text(P,A,formatC(Y,format="f",digits=1),col="red") )
text( 1970.5, 2, "Population count 1 January", srt=90, col="blue")
text( 1974.5, 2, "Person-\nyears", col="red")
}
\keyword{Data}

\name{stattable.funs}
\alias{count}
\alias{percent}
\alias{ratio}
\title{Special functions for use in stat.table}
\description{
  These functions may be used as \code{contents} arguments to the
  function \code{stat.table}. They are defined internally in
  \code{stat.table} and have no independent existence.
}
\usage{
count(id)
ratio(d,y,scale=1, na.rm=TRUE)
percent(...)
}
\arguments{
  \item{id}{numeric vector in which identical values identify the
    same individual.}
  \item{d, y}{numeric vectors of equal length (\code{d} for Deaths,
    \code{y} for person-Years)}
  \item{scale}{a scalar giving a value by which the ratio should be
    multiplied}
  \item{na.rm}{a logical value indicating whether \code{NA} values should
    be stripped before computation proceeds.}
  \item{...}{a list of variables taken from the \code{index} argument
    to \code{\link{stat.table}}}
}
\value{
  When used as a \code{contents} argument to \code{stat.table}, these
  functions create the following tables:
  \item{\code{count}}{If given without argument (\code{count()}) it
    returns a contingency table of counts. If given an \code{id}
    argument it returns a table of the number of different values of
    \code{id} in each cell, i.e. how many persons contribute in each
    cell.}
  \item{\code{ratio}}{returns a table of values
    \code{scale * sum(d)/sum(y)}}
  \item{\code{percent}}{returns a table of percentages of the
    classifying variables. Variables that are in the \code{index}
    argument to \code{stat.table} but not in the call to
    \code{percent} are used to define strata, within which the
    percentages add up to 100.}
}
\author{Martyn Plummer}
\seealso{\code{\link{stat.table}}}
\keyword{iteration}
\keyword{category}

\name{overlap.Lexis}
\alias{overlap}
\alias{overlap.Lexis}
\title{Check if records in a Lexis object represent overlapping epochs}
\description{
When constructing a \code{\link{Lexis}} object from a data frame with
multiple records there is no guarantee that the records from a given
person will represent disjoint time intervals (epochs), \code{overlap}
checks this.
}
\usage{
\method{overlap}{Lexis}(Lx, \dots)
}
\arguments{
  \item{Lx}{A \code{Lexis} object}
  \item{\dots}{ignored}
}
\value{
If no overlapping epochs are found, a message is printed and \code{NULL}
is returned. Otherwise a vector of unique ids (from \code{Lx$lex.id})
for the persons with overlapping records is returned invisibly
}
\author{Bendix Carstensen,
  \url{http://bendixcarstensen.com},
  \email{b@bxc.dk}}
\seealso{\code{\link{Lexis}}}
\examples{
# run the DMlate example to get the Lexis object dmi with
# some persons with multiple (2 that is) records:
example(DMlate)
overlap.Lexis(dmi)
#
# now find 4 persons with multiple records
tt <- table(dmi$lex.id)
has2 <- dmi$lex.id \%in\% names(tt[tt > 1])[1:4]
table(has2)
#
# add time (0.3 y) to lex.dur for these persons to produce overlapping intervals
dmi$lex.dur[has2] <- dmi$lex.dur[has2] + 0.3
(overlap.Lexis(dmi))
}
\keyword{manip}
\concept{Lexis checks}

\name{LCa.fit}
\alias{LCa.fit}
\alias{print.LCa}
\alias{summary.LCa}
\alias{plot.LCa}
\alias{predict.LCa}
\title{
  Fit Lee-Carter-type models for rates to arbitrarily shaped observations
  of rates in a Lexis diagram. 
}
\description{
  The Lee-Carter model is originally defined as a model for rates
  observed in A-sets (age by period) of a Lexis diagram, as
  log(rate(x,t)) = a(x) + b(x)k(t), using one parameter per age(x) and
  period(t). This function uses natural splines for a(), b() and k(),
  placing knots for each effect such that the number of events is the
  same between knots. 
}
\usage{
LCa.fit( data, A, P, D, Y,
         model = "APa",    # or one of "ACa", "APaC", "APCa" or "APaCa" 
         a.ref,            # age reference for the interactions
        pi.ref = a.ref,    # age reference for the period interaction
        ci.ref = a.ref,    # age reference for the cohort interaction
         p.ref,            # period reference for the interaction
         c.ref,            # cohort reference for the interactions
          npar = c(a = 6,  # no. knots for main age-effect
                   p = 6,  # no. knots for period-effect
                   c = 6,  # no. knots for cohort-effect
                  pi = 6,  # no. knots for age in the period interaction
                  ci = 6), # no. knots for age in the cohort interaction
            VC = TRUE,     # numerical calculation of the Hessian?
         alpha = 0.05,     # 1 minus confidence level
           eps = 1e-6,     # convergence criterion
         maxit = 100,      # max. no iterations
         quiet = TRUE )    # cut the crap
\method{print}{LCa}( x, ... )
\method{summary}{LCa}( object, show.est=FALSE, ... )
\method{plot}{LCa}( x, ... )
\method{predict}{LCa}( object, newdata,
                        alpha = 0.05,
                        level = 1-alpha,
                          sim = ( "vcov" \%in\% names(object) ),
                          ... )
      }
\arguments{
  \item{data}{
  A data frame. Must have columns \code{A}(age), \code{P}(period, that is
  calendar time), \code{D}(no. of events) and \code{Y}(person-time,
  exposure). Alternatively these four quantities can be given as
  separate vectors:
  }
  \item{A}{
  Vector of ages (midpoint of observation).
  }
  \item{P}{
  Vector of period (midpoint of observation).
  }
  \item{D}{
  Vector of no. of events.
  }
  \item{Y}{
  Vector of person-time. Demographers would say "exposure", bewildering epidemiologists.
  }
  \item{a.ref}{
  Reference age for the age-interaction term(s) \code{pi(x)} and/or
  \code{pi(x)}, where \code{pi(a.ref)=1} and \code{ci(a.ref)=1}.
  }
  \item{pi.ref}{
  Same, but specifically for the interaction with period.
  }
  \item{ci.ref}{
  Same, but specifically for the interaction with cohort.
  }
  \item{p.ref}{
  Reference period for the time-interaction term \code{kp(t)} where \code{kp(p.ref)=0}.
  }
  \item{c.ref}{
  Reference period for the time-interaction term \code{kp(t)} where \code{kc(c.ref)=0}.
  }
  \item{model}{
  Character, either \code{"APa"} which is the classical Lee-Carter model
  for log-rates, other possibilities are \code{"ACa"}, \code{"APCa"},
  \code{"APaC"} or \code{"APaCa"}, see details.
  }
  \item{npar}{
  A (possibly named) vector or list, with either the number of knots or
  the actual vectors of knots for each term. If unnamed, components are
  taken to be in the order (a,b,t), if the model is "APaCa" in the order
  (a,p,c,pi,ci).  If a vector, the three integers indicate the number of
  knots for each term; these will be placed so that there is an equal
  number of events (\code{D}) between each, and half as many below the
  first and above the last knot. If \code{npar} is a list of scalars the
  behavior is the same. If \code{npar} is a list of vectors, these are
  taken as the knots for the natural splines. See details for naming
  convention. 
  }
  \item{VC}{
  Logical. Should the variance-covariance matrix of the parameters be
  computed by numerical differentiation? See details.
  }
  \item{alpha}{
  1 minus the confidence level used when calculating
  confidence intervals for estimates in \code{LCa.fit} and for
  predictions by \code{predict.LCa}.
  }
  \item{eps}{
  Convergence criterion for the deviance, we use the the relative
  difference between deviance from the two models fitted.
  }
  \item{maxit}{
  Maximal number of iterations.
  }
  \item{quiet}{
  Shall I shut up or talk extensively to you about iteration progression etc.?
  }
  \item{object}{An \code{LCa} object, see under "Value".}
  \item{show.est}{Logical. Should the estimates be printed?}
  \item{x}{An \code{LCa} object, see under "Value".}
  \item{newdata}{Prediction data frame, must have columns \code{A} and
                 \code{P}. Any \code{Y} column is ignored, predictions are given in
  units of the \code{Y} supplied for the call that generated the
  \code{LCa} object.}
  \item{level}{Confidence level.}
  \item{sim}{Logical or numeric. If \code{TRUE}, prediction c.i.s will be
  based on 1000 simulations from the posterior parameters. If numeric,
  it will be based on that number of simulations.}
  \item{...}{Additional parameters passed on to the method.}
}
\details{
  The Lee-Carter model is non-linear in age and time so does not fit
  in the classical glm-Poisson framework. But for fixed \code{b(x)} it
  is a glm, and also for fixed \code{a(x)}, \code{k(t)}. The function
  alternately fits the two versions until the same fit is produced (same
  deviance).

  The multiplicative age by period term could equally well have been a
  multiplicative age by cohort or even both. Thus the most extensive
  model has 5 continuous functions:

  \deqn{\log(\lambda(a,p)) = f(a) + b_p(a)k_p(p) + b_c(a)k_c(p-a)}{%
         log( lambda(a,p)) = f(a) + b_p(a)k_p(p) + b_c(a)k_c(p-a)}

  Each of these is fitted by a natural spline, with knots placed at the
  quantiles of the events along the age (a), calendar time (p) respective
  cohort (p-a) scales. Alternatively the knots can be specified explicitly
  in the argument \code{npar} as a named list, where
  \code{a} refers to \eqn{f(a)}{f(a)},
  \code{p} refers to \eqn{k_p(p)}{k_p(p)},
  \code{c} refers to \eqn{k_c(p-a)}{k_c(p-a)},
  \code{pi} (\code{p}eriod \code{i}nteraction) refers to \eqn{b_p(a)}{b_p(a)}
  and
  \code{ci} (\code{c}ohort \code{i}nteraction) refers to \eqn{b_c(p-a)}{b_c(p-a)}.

  The naming convention for the models is a capital \code{P} and/or
  \code{C} if the effect is in the model followed by a lower case
  \code{a} if there is an interaction with age. Thus there are 5 different
  models that can be fitted: \code{APa}, \code{ACa}, \code{APaC}  \code{APCa}
  and \code{APaCa}.
	  
  The standard errors of the parameters from the two separate model fits
  in the iterations are however wrong; they are conditional on a subset
  of the parameters having a fixed value. However, analytic calculation
  of the Hessian is a bit of a nightmare, so this is done numerically
  using the \code{hessian} function from the \code{numDeriv} package if
  \code{VC=TRUE}.
  
  The coefficients and the variance-covariance matrix of these are used
  in \code{predict.LCa} for a parametric bootstrap (that is, a
  simulation from a multivariate normal with mean equal to the parameter
  estimates and variance as the estimated variance-covariance) to get
  confidence intervals for the predictions if \code{sim} is \code{TRUE}
  --- which it is by default if they are part of the object.

  The \code{plot} for \code{LCa} objects merely produces between 3 and 5
  panels showing each of the terms in the model. These are mainly for
  preliminary inspection; real reporting of the effects should use
  proper relative scaling of the effects.}

\value{
  \code{LCa.fit} returns an object of class \code{LCa} (smooth
  effects \code{L}ee-\code{Ca}rter model); it is a list with the
  following components:
\item{model}{Character, either \code{APa}, \code{ACa}, \code{APaC},
  \code{APCa} or \code{APaCa}, indicating the variable(s) interacting
  with age.} 
\item{ax}{3-column matrix of age-effects, c.i. from the age-time
  model. Row names are the unique occurring ages in the
  dataset. Estimates are rates.}  
\item{pi}{3-column matrix of age-period interaction effects, c.i. from the age
  model. Row names are the actually occurring ages in the
  dataset. Estimates are multipliers of the log-RRs in \code{kp},
  centered at 1 at \code{pi.ref}.}
\item{kp}{3-column matrix of period-effects, with c.i.s from the
  age-time model. Row names are the actually occurring times in the 
  dataset. Estimates are rate-ratios centered at 1 at \code{p.ref}.}
\item{ci}{3-column matrix of age-cohort interaction effects, c.i. from the age
  model. Row names are the actually occurring ages in the
  dataset. Estimates are multipliers of the log-RRs in \code{kc},
  centered at 1 at \code{ci.ref}.}
\item{kc}{3-column matrix of cohort-effects, with c.i.s from the age-time
  model. Row names are the actually occurring times in the
  dataset. Estimates are rate-ratios centered at 1 at \code{c.ref}.}
\item{mod.at}{\code{glm} object with the final age-time model --- estimates
  the terms \code{ax}, \code{kp}, \code{kc}. Gives
  the same fit as the \code{mod.b} model after convergence.}
\item{mod.b}{\code{glm} object with the final age model --- estimates
  the terms \code{pi}, \code{ci}. Gives
  the same fit as the \code{mod.at} model after convergence.}
\item{coef}{All coefficients from both models, in the order \code{ax},
  \code{kp}, \code{kc}, \code{pi}, \code{ci}. Only present if
  \code{LCa.fit} were called with \code{VC=TRUE} (the default).} 
\item{vcov}{Variance-covariance matrix of coefficients from both
  models, in the same order as in the \code{coef}. Only present if
  \code{LCa.fit} were called with \code{VC=TRUE}.} 
\item{knots}{List of vectors of knots used in for the age, period and
  cohort effects.}
\item{refs}{List of reference points used for the age, period and
  cohort terms in the interactions.}
\item{deviance}{Deviance of the model}
\item{df.residual}{Residual degrees of freedom}
\item{iter}{Number of iterations used to reach convergence.}

  \code{plot.LCa} plots the estimated effects in separate panels,
  using a log-scale for the baseline rates (\code{ax}) and the time-RR
  (\code{kt}). For the \code{APaCa} model 5 panels are plotted.
  
  \code{summary.LCa} returns (invisibly) a matrix with the parameters
  from the models and a column of the conditional se.s and additionally
  of the se.s derived from the numerically computed Hessian (if
  \code{LCa.fit} were called with \code{VC=TRUE}.)

  \code{predict.LCa} returns a matrix with one row per row in
  \code{newdata}. If \code{LCa.fit} were called with \code{VC=TRUE}
  there will be 3 columns, namely prediction (1st column) and c.i.s
  based on a simulation of parameters from a multivariate normal with
  mean \code{coef} and variance \code{vcov} using the median and
  \code{alpha}/2 quantiles from the \code{sim} simulations.  If
  \code{LCa.fit} were called with \code{VC=FALSE} there will be 6
  columns, namely estimates and c.i.s from age-time model
  (\code{mod.at}), and from the age-interaction model (\code{mod.b}),
  both using conditional variances, and therefore likely with too narrow
  confidence limits.
  }
\author{
  Bendix Carstensen, \url{http://bendixcarstensen.com}

  This function was conceived while teaching a course on APC models at
  the Max Planck Institute of Demographic Research (MPIDR,
  \url{https://www.demogr.mpg.de/en/}) in Rostock in May 2016
  (\url{http://bendixcarstensen.com/APC/MPIDR-2016/}), and finished
  during a week long research stay there, kindly sponsored by the MPIDR.
  }
\seealso{
 \code{\link{apc.fit}},
 \code{\link{apc.LCa}},
 \code{\link[demography]{lca}} }
\examples{
library( Epi )
# Load the testis cancer data by Lexis triangles
data( testisDK )
tc <- subset( testisDK, A>14 & A<60 )
head( tc )

# We want to see rates per 100,000 PY
tc$Y <- tc$Y / 10^5

# Fit the Lee-Carter model with age-period interaction (default)
LCa.tc <- LCa.fit( tc, model="ACa", a.ref=30, p.ref=1980, quiet=FALSE, eps=10e-4, maxit=50 )

LCa.tc
summary( LCa.tc )

# Inspect what we got
names( LCa.tc )

# show the estimated effects
par( mfrow=c(1,3) )
plot( LCa.tc )

# Prediction data frame for ages 15 to 60 for two time points: 
nd <- data.frame( A=15:60 )
# LCa predictions
p70 <- predict.LCa( LCa.tc, newdata=cbind(nd,P=1970), sim=1000 )
p90 <- predict.LCa( LCa.tc, newdata=cbind(nd,P=1990), sim=1000 )

# Inspect the curves from the parametric bootstrap (simulation):
par( mfrow=c(1,1) )
head( cbind(p70,p90) )
matplot( nd$A, cbind(p70,p90), type="l", lwd=c(6,3,3), lty=c(1,3,3),
         col=rep( 2:3, each=3 ), log="y",
         ylab="Testis cancer incidence per 100,000 PY in 1970 resp. 1990", xlab="Age" )
}
\keyword{models}
\keyword{regression}

\name{Life.lines}
\alias{Life.lines}
\title{
  Compute dates/ages for life lines in a Lexis diagram
}
\description{
  Fills out the missing information for follow up of persons in a Lexis
  diagram if sufficient information is given.
}
\usage{
Life.lines( entry.date = NA,
             exit.date = NA,
            birth.date = NA,
             entry.age = NA,
              exit.age = NA,
             risk.time = NA )
}
\arguments{
 \item{entry.date, exit.date,birth.date, entry.age, exit.age,
       risk.time}{Vectors defining lifelines to be plotted 
       in the diagram. At least three must be given to produce a result.
       Not all subsets of three will suffice, the given subset has to 
       define life lines. If insufficient data is given, nothing is
       returned and a warning is given.}
}
\value{
  Data frame with variables \code{entry.date}, \code{entry.age},
  \code{exit.date}, \code{exit.age}, \code{risk.time},
  \code{birth.date}, with all entries computed for each person. If any
  of \code{entry.date}, \code{exit.date} or \code{birth.date} are of
  class \code{Date} or if any of \code{entry.age}, \code{exit.age} or
  \code{risk.time} are of class \code{difftime} the date variables will
  be of class  \code{Date} and the other three of class
  \code{difftime}. 
}
\examples{
( Life.lines( entry.age = c(3,30,45),
              risk.time = c(25,5,14),
             birth.date = c(1970,1931,1925.7) ) )

# Draw a Lexis diagram
Lexis.diagram()

# Compute entry and exit age and date.
( LL <-  Life.lines( entry.age = c(3,30,45),
                     risk.time = c(25,5,14),
                    birth.date = c(1970,1931,1925.7) ) )
segments( LL[,1], LL[,2], LL[,3], LL[,4] ) # Plot the life lines.

# Compute entry and exit age and date, supplying a date variable
bd <- ( c(1970,1931,1925.7) - 1970 ) * 365.25
class( bd ) <- "Date"
( Life.lines( entry.age = c(3,30,45),
              risk.time = c(25,5,14),
             birth.date = bd ) )
}
\seealso{
  \code{\link{Lexis.diagram}},
  \code{\link{Lexis.lines}}
}
\keyword{ manip }
\keyword{ dplot }

\name{ci.lin}
\alias{ci.lin}
\alias{ci.mat}
\alias{ci.exp}
\alias{ci.pred}
\alias{ci.ratio}
\alias{Wald}
\title{
  Compute linear functions of parameters with standard errors and
  confidence limits, optionally transforming to a different scale.
  }
\description{
  For a given model object the function computes a linear function of
  the parameters and the corresponding standard errors, p-values and
  confidence intervals.
  }
\usage{
ci.lin( obj,
    ctr.mat = NULL,
     subset = NULL,
     subint = NULL,
      xvars = NULL,
      diffs = FALSE,
       fnam = !diffs,
       vcov = FALSE,
      alpha = 0.05,
         df = Inf,
        Exp = FALSE,
     sample = FALSE )
ci.exp( ..., Exp = TRUE, pval = FALSE )
Wald( obj, H0=0, ... )
ci.mat( alpha = 0.05, df = Inf )
ci.pred( obj,
     newdata,
         Exp = NULL,
       alpha = 0.05 )
ci.ratio( r1, r2,
         se1 = NULL,
         se2 = NULL,
      log.tr = !is.null(se1) & !is.null(se2),
       alpha = 0.05,
        pval = FALSE )
}
\arguments{
  \item{obj}{A model object (in general of class \code{glm}, but for
    \code{ci.lin} and \code{ci.exp} it may also be of class
    \code{lm},
    \code{coxph},
    \code{survreg},
    \code{clogistic},
    \code{cch},
    \code{lme},
    \code{mer},
    \code{lmerMod},
    \code{glmerMod},
    \code{gls},
    \code{nls},
    \code{gnlm},
    \code{MIresult},
    \code{mipo},
    \code{polr},
 or \code{rq}).
  }
  \item{ctr.mat}{Matrix, data frame or list (of two or four data frames).

    If \code{ctr.mat} is a matrix, it should be a contrast matrix to be
    multiplied to the parameter vector, i.e. the desired linear function
    of the parameters.

    If it is a data frame it should have columns corresponding to a
    prediction frame, see details.
    
    If it is a list, it must contain two or four data frames that are
    (possibly partial) prediction frames for \code{obj}, see argument
    \code{xvars} and details.}
  \item{xvars}{Character vector. If quantitative variables in the model
    are omitted from data frames supplied in a list to \code{ctr.mat},
    they should be listed here. Omitted factors need not be mentioned
    here.} 
  \item{subset}{The subset of the parameters to be used. If given as a
    character vector, the elements are in turn matched against the
    parameter names (using \code{grep}) to find the subset. Repeat
    parameters may result from using a character vector. This is
    considered a facility.}
  \item{subint}{Character. \code{sub}set selection, but where each
    element of the character vector is used to
    select a subset of parameters and only the \code{int}ersection
    of these is returned.}
  \item{diffs}{If TRUE, all differences between parameters
    in the subset are computed, and the \code{subset} argument is
    required. The argument \code{ctr.mat} is ignored. If \code{obj}
    inherits from \code{lm}, and \code{subset} is given as a string
    \code{subset} is used to search among the factors in the model and
    differences of all factor levels for the first match are shown.
    If \code{subset} does not match any of the factors in the model, all
    pairwise differences between parameters matching are returned.}
  \item{fnam}{Should the common part of the parameter names be included
    with the annotation of contrasts? Ignored if \code{diffs==T}. If a
    string is supplied this will be prefixed to the labels.}
  \item{vcov}{Should the covariance matrix of the set of parameters be
    returned? If this is set, \code{Exp} is ignored. See details.}
  \item{alpha}{Significance level for the confidence intervals.}
  \item{df}{Integer. Number of degrees of freedom in the t-distribution used
    to compute the quantiles used to construct the confidence intervals.}
  \item{Exp}{For \code{ci.lin}, if \code{TRUE} columns 5:6 are replaced
    with exp( columns 1,5,6 ). For \code{ci.exp}, if \code{FALSE}, the
    untransformed parameters are returned. For \code{ci.pred} it
    indicates whether the predictions should be exponentiated - the
    default (\code{Exp=NULL}) is to make a prediction with a Wald CI on
    the scale of the linear predictor and back-transform it by the
    inverse link function; if \code{FALSE}, the prediction on the link
    scale is returned.}
  \item{sample}{Logical or numerical. If \code{TRUE} or numerical a
    sample of size \code{as.numeric(sample)} is drawn from the
    multivariate normal with mean equal to the (\code{subset} defined)
    coefficients and variance equal to the estimated variance-covariance
    of these. These are then transformed by \code{ctr.mat} and
    returned.}
  \item{pval}{Logical. Should a column of P-values be included with the
    estimates and confidence intervals output by \code{ci.exp}.}
  \item{H0}{Numeric. The null values for the selected/transformed
    parameters to be tested by a Wald test. Must have the same length as
    the selected parameter vector.}
  \item{\ldots}{Parameters passed on to \code{ci.lin}.}
  \item{newdata}{Data frame of covariates where prediction is made.}
  \item{r1,r2}{Estimates of rates in two independent groups, with
    confidence limits. Can be either 3-column matrices or data frames
    with estimates and confidence intervals or 2 two column structures
    with confidence limits only. Only the confidence limits are used.}
  \item{se1,se2}{Standard errors of log-rates in the two groups. If
    given, it is assumed that \code{r1} and \code{r2} represent
    log-rates.}
  \item{log.tr}{Logical, if true, it is assumed that \code{r1} and
    \code{r2} represent log-rates with confidence intervals.}
  }
\value{
    
  \code{ci.lin} returns a matrix with number of rows and row names as
  \code{ctr.mat}. The columns are Estimate, Std.Err, z, P, 2.5\% and
  97.5\% (or according to the value of \code{alpha}).  If
  \code{vcov=TRUE}, instead a list of length 2 with components \code{coef} (a
  vector), the desired functional of the parameters and \code{vcov} (a
  square matrix), the variance covariance matrix of this, is returned
  but not printed. If \code{Exp==TRUE} the confidence intervals for the
  parameters are replaced with three columns: exp(estimate,c.i.).

  \code{ci.exp} returns only the exponentiated parameter estimates with
  confidence intervals. It is merely a wrapper for \code{ci.lin},
  fishing out the last 3 columns from \code{ci.lin(...,Exp=TRUE)}. If
  you just want the estimates and confidence limits, but not
  exponentiated, use \code{ci.exp(...,Exp=FALSE)}.
  
  If \code{ctr.mat} is a data frame, the model matrix corresponding to
  this is constructed and supplied. This is only supported for objects
  of class \code{lm}, \code{glm}, \code{gam} and \code{coxph}.

  So the default behaviour will be to produce the same as
  \code{ci.pred}, apparently superfluous. The purpose of this is to
  allow the use of the arguments \code{vcov} that produces the
  variance-covariance matrix of the predictions, and \code{sample} that
  produces a sample of predictions using sampling from the multivariate
  normal with mean equal to parameters and variance equal to the
  hessian.

  If \code{ctr.mat} is a list of two data frames, the difference of the
  predictions from using the first versus the last as newdata arguments
  to predict is computed. Columns that would be identical in the two
  data frames can be omitted (see below), but names of numerical
  variables omitted must be supplied in a character vector
  \code{xvars}. Factors omitted need not be named.

  If the second data frame has only one row, this is replicated to match
  the number of rows in the first. This facility is primarily aimed at
  teasing out RRs that are non-linear functions of a quantitative
  variable without setting up contrast matrices using the same code as
  in the model. Note however if splines are used with computed knots
  stored in a vector such as \code{Ns(x,knots=kk)} then the \code{kk}
  must be available in the (global) environment; it will not be found
  inside the model object. In practical terms it means that if you save
  model objects for later prediction you should save the knots used in
  the spline setups too.

  If \code{ctr.mat} is a list of four data frames, the difference of the
  difference of predictions from using the first and second versus
  difference of predictions from using the third and fourth is computed.
  Simply \code{(pr1-pr2) - (pr3-pr4)} with obvious notation. Useful to
  derive esoteric interaction effects.
  
  Finally, only arguments \code{Exp}, \code{vcov}, \code{alpha} and
  \code{sample} from \code{ci.lin} are honored when \code{ctr.mat} is a
  data frame or a list of two data frames.

  You can leave out variables (columns) from the two data frames that
  would be identical, basically variables not relevant for the
  calculation of the contrast. In many cases \code{ci.lin} (really
  \code{Epi:::ci.dfr}) can figure out the names of the omitted columns,
  but occasionally you will have to supply the names of the omitted
  variables in the \code{xvars} argument. Factors omitted need not be
  listed in \code{xvars}, although no harm is done doing so.

  \code{Wald} computes a Wald test for a subset of (possibly linear
  combinations of) parameters being equal to the vector of null
  values as given by \code{H0}. The selection of the subset of
  parameters is the same as for \code{ci.lin}. Using the \code{ctr.mat}
  argument makes it possible to do a Wald test for equality of
  parameters. \code{Wald} returns a named numerical vector of length 3,
  with names \code{Chisq}, \code{d.f.} and \code{P}.

  \code{ci.mat} returns a 2 by 3 matrix with rows \code{c(1,0,0)} and
  \code{c(0,-1,1)*1.96}, devised to post-multiply to a p by 2 matrix with
  columns of estimates and standard errors, so as to produce a p by 3 matrix
  of estimates and confidence limits. Used internally in \code{ci.lin} and
  \code{ci.cum}.
  The 1.96 is replaced by the appropriate quantile from the normal or
  t-distribution when arguments \code{alpha} and/or \code{df} are given.

  \code{ci.pred} returns a 3-column matrix with estimates and upper and
  lower confidence intervals as columns. This is just a convenience
  wrapper for \code{predict.glm(obj,se.fit=TRUE)} which returns a rather
  unhandy structure. The prediction with c.i. is made in the \code{link}
  scale, and by default transformed by the inverse link, since the most
  common use for this is for multiplicative Poisson or binomial models
  with either log or logit link.

  \code{ci.ratio} returns the rate-ratio of two independent sets of
  rates given with confidence intervals or s.e.s. If \code{se1} and
  \code{se2} are given and \code{log.tr=FALSE} it is assumed that
  \code{r1} and \code{r2} are rates and \code{se1} and \code{se2} are
  standard errors of the log-rates.
  }
\author{
  Bendix Carstensen,
  \url{http://bendixcarstensen.com} & Michael Hills
}

\seealso{See \code{\link{ci.eta}} for a simple version only needing
  coefficients and variance-covariance matrix. See also \code{\link{ci.cum}}
  for a function computing cumulative sums of (functions of) parameter
  estimates, and \code{\link{ci.surv}} for a function computing
  confidence intervals for survival functions based on smoothed
  rates. The example code for \code{\link{matshade}} has an application
  of predicting a rate-ratio using a list of two prediction frames in the
  \code{ctr.mat} argument.}

\examples{
# Bogus data:
f <- factor( sample( letters[1:5], 200, replace=TRUE ) )
g <- factor( sample( letters[1:3], 200, replace=TRUE ) )
x <- rnorm( 200 )
y <- 7 + as.integer(f) * 3 + 2 * x + 1.7 * rnorm(200)

# Fit a simple model:
mm <- lm(y ~ x + f + g)
ci.lin( mm )
ci.lin( mm, subset = 3:6, diff = TRUE, fnam = FALSE )
ci.lin( mm, subset = 3:6, diff = TRUE, fnam = TRUE )
ci.lin( mm, subset = "f", diff = TRUE, fnam = "f levels:" )
print( ci.lin( mm, subset = "g", diff = TRUE, fnam = "gee!:", vcov = TRUE ) )

# Use character defined subset to get ALL contrasts:
ci.lin( mm, subset = "f", diff = TRUE )

# Suppose the x-effect differs across levels of g:
mi <- update( mm, . ~ . + g:x )
ci.lin( mi )
# RR a vs. b by x:
nda <- data.frame( x = -3:3, g = "a", f = "b" )
ndb <- data.frame( x = -3:3, g = "b", f = "b" )
# 
ci.lin( mi, list(nda,ndb) )
# Same result if f column is omitted because "f" columns are identical
ci.lin( mi, list(nda[,-3],ndb[,-3]) )
# However, crashes if knots in spline is supplied, and non-factor omitted
xk <- -1:1
xi <- c(-0.5,0.5)
ww <- rnorm(200)
mi <- update( mm, . ~ . -x + ww + Ns(x,knots=xk) + g:Ns(x,knots=xi) )
# Will crash 
try( cbind( nda$x, ci.lin( mi, list(nda,ndb) ) ) )
# Must specify num vars (not factors) omitted from nda, ndb
cbind( nda$x, ci.lin( mi, list(nda,ndb), xvars="ww" ) )

# A Wald test of whether the g-parameters are 0
Wald( mm, subset="g" )
# Wald test of whether the three first f-parameters are equal:
( CM <- rbind( c(1,-1,0,0), c(1,0,-1,0)) )
Wald( mm, subset="f", ctr.mat=CM )
# or alternatively
( CM <- rbind( c(1,-1,0,0), c(0,1,-1,0)) )
Wald( mm, subset="f", ctr.mat=CM )

# Confidence intervals for ratio of rates
# Rates and ci supplied, but only the range (lower and upper ci) is used
ci.ratio( cbind(10,8,12.5), cbind(5,4,6.25) )
ci.ratio( cbind(8,12.5), cbind(4,6.25) )

# Beware of the offset when making predictions with ci.pred and ci.exp
\dontrun{
library( mgcv )
data( mortDK )
m.arg  <- glm( dt ~ age , offset=log(risk) , family=poisson, data=mortDK )
m.form <- glm( dt ~ age + offset(log(risk)), family=poisson, data=mortDK )
a.arg  <- gam( dt ~ age , offset=log(risk) , family=poisson, data=mortDK )
a.form <- gam( dt ~ age + offset(log(risk)), family=poisson, data=mortDK )

nd <- data.frame( age=60:65, risk=100 )
round( ci.pred( m.arg , nd ), 4 )
round( ci.pred( m.form, nd ), 4 )
round( ci.exp ( m.arg , nd ), 4 )
round( ci.exp ( m.form, nd ), 4 )
round( ci.pred( a.arg , nd ), 4 )
round( ci.pred( a.form, nd ), 4 )
round( ci.exp ( a.arg , nd ), 4 )
round( ci.exp ( a.form, nd ), 4 )

nd <- data.frame( age=60:65 )
try( ci.pred( m.arg , nd ) )
try( ci.pred( m.form, nd ) )
try( ci.exp ( m.arg , nd ) )
try( ci.exp ( m.form, nd ) )
try( ci.pred( a.arg , nd ) )
try( ci.pred( a.form, nd ) )
try( ci.exp ( a.arg , nd ) )
try( ci.exp ( a.form, nd ) )
}
}
\keyword{models}
\keyword{regression}

\name{bdendo}
\alias{bdendo}
\alias{bdendo11}
\docType{data}
\title{A case-control study of endometrial cancer}
\description{
The \code{bdendo} data frame has 315 rows and 13 columns,
\code{bdendo11} 126 rows.  These data concern a study in which each case
of endometrial cancer was matched with 4 controls. \code{bdendo11} is a
1:1 mathed subset of \code{bdendo}. Matching was by date of birth
(within one year), marital status, and residence.
}
\format{
These data frames have the following columns:
\tabular{rl}{
\code{set}: \tab Case-control set: a numeric vector  \cr
\code{d}: \tab Case or control: a numeric vector (1=case, 0=control) \cr
\code{gall}: \tab Gall bladder disease: a factor with levels
                  \code{No} 
                  \code{Yes}. \cr
\code{hyp}: \tab Hypertension: a factor with levels
                 \code{No} 
                 \code{Yes}. \cr
\code{ob}: \tab Obesity: a factor with levels
                \code{No} 
                \code{Yes}. \cr
\code{est}: \tab A factor with levels
                 \code{No} 
                 \code{Yes}. \cr
\code{dur}: \tab Duration of conjugated oestrogen therapy: a factor with levels
                 \code{0}, \code{1}, \code{2}, \code{3}, \code{4}. \cr
\code{non}: \tab Use of non oestrogen drugs: a factor with levels
                 \code{No} 
                 \code{Yes}. \cr
\code{duration}: \tab Months of oestrogen therapy: a numeric vector.  \cr
\code{age}: \tab A numeric vector. \cr
\code{cest}: \tab Conjugated oestrogen dose: a factor with levels
\code{0}, \code{1}, \code{2}, \code{3}. \cr
\code{agegrp}: \tab A factor with levels
                    \code{55-59} 
                    \code{60-64} 
                    \code{65-69} 
                    \code{70-74} 
                    \code{75-79} 
                    \code{80-84} \cr
\code{age3}: \tab a factor with levels
                  \code{<64} 
                  \code{65-74} 
                  \code{75+}  \cr
}
}		
\source{
Breslow NE, and Day N, Statistical Methods in Cancer Research. Volume
I: The Analysis of Case-Control Studies. IARC Scientific
Publications, IARC:Lyon, 1980.  
}
\examples{
data(bdendo)
str(bdendo)
}
\keyword{datasets}

\name{thoro}
\alias{thoro}
\docType{data}
\title{Thorotrast Study}
\description{
The \code{thoro} data frame has 2470 rows and 14 columns.
Each row represents one patient that have had cerebral angiography (X-ray of
the brain) with an injected contrast medium, either Thorotrast or
another one (the controls).
}
\format{
This data frame contains the following columns:
\describe{
\item{\code{id}}{Identification of person.}
\item{\code{sex}}{Sex, 1: male / 2: female.}
\item{\code{birthdat}}{Date of birth, \code{Date} variable.}
\item{\code{contrast}}{Group, 1: Thorotrast / 2: Control.}
\item{\code{injecdat}}{Date of contrast injection, \code{Date} variable.}
\item{\code{volume}}{Injected volume of Thorotrast in ml. Control
                  patients have a 0 in this variable.}
\item{\code{exitdat}}{Date of exit from the study, \code{Date} variable.}
\item{\code{exitstat}}{Status at exit, 1: dead / 2: alive,
                  censored at closing of study, 20 February 1992 / 3:
                  censored alive at some earlier date.}
\item{\code{cause}}{Cause of death. See causes in the helpfile for
                  \code{\link{gmortDK}}.}
\item{\code{liverdat}}{Date of liver cancer diagnosis, \code{Date} variable.}
\item{\code{liver}}{Indicator of liver cancer diagnosis. Not all livercancers
                  are histologically verified, hence
                  \code{liver >= hepcc + chola + hmang}}
\item{\code{hepcc}}{Hepatocellular carcinoma at \code{liverdat}.}
\item{\code{chola}}{Cholangiocellular carcinoma at \code{liverdat}.}
\item{\code{hmang}}{Haemangisarcoma carcinoma at \code{liverdat}.}
}
}
\source{
M Andersson, M Vyberg, J Visfeldt, B Carstensen & HH Storm:
Primary liver tumours among Danish patients exposed to Thorotrast.
Radiation Research, 137, pp. 262--273, 1994.

M Andersson, B Carstensen HH Storm: Mortality and cancer
incidence after cerebral angiography. Radiation Research, 142,
pp. 305--320, 1995.
}
\examples{
data(thoro)
str(thoro)
}
\seealso{\code{\link{mortDK}}, \code{\link{gmortDK}}}
\keyword{datasets}

\name{fit.baseline}
\alias{fit.baseline}
\title{
  Fit a piecewise contsnt intesity model for interval censored data.
  }
\description{
  Utility function
  
  Fits a binomial model with logaritmic link, with \code{y} as outcome
  and covariates in \code{rates.frame} to estimate rates in the
  inttervals between \code{breaks}.
  }
\usage{
fit.baseline( y, rates.frame, start )
}
\arguments{
  \item{y}{Binary vector of outcomes}
  \item{rates.frame}{Dataframe expanded from the original data by
    \code{\link{expand.data}}} 
  \item{start}{Starting values for the rate parameters. If not supplied,
       then starting values are generated.}
}
\value{
  A \code{\link{glm}} object, with binomial error and logaritmic link.
  }
\author{
  Martyn Plummer, \email{plummer@iarc.fr}
  }
\seealso{
  \code{\link{fit.add}}
  \code{\link{fit.mult}}
  }
\keyword{ models }
\keyword{ regression }
\keyword{ survival }

\name{B.dk}
\alias{B.dk}
\docType{data}
\title{Births in Denmark by year and month of birth and sex}
\description{
  The number of live births as entered from printed publications from
  Statistics Denmark.
}
\usage{data(B.dk)}
\format{
  A data frame with 1248 observations on the following 4 variables.
  \describe{
    \item{\code{year}}{Year of birth}
    \item{\code{month}}{Month of birth}
    \item{\code{m}}{Number of male live births}
    \item{\code{f}}{Number of female live births}
  }
}
\details{
  Division of births by month and sex is only available for the years 1957--69
  and 2002ff. For the remaining period, the total no. births in each month is
  divided between the sexes so that the fraction of boys is equal to the overall
  fraction for the years where the sex information is available.
  
  There is a break in the series at 1920, when Sonderjylland was joined to Denmark.
}
\source{
  Statistiske Undersogelser nr. 19: Befolkningsudvikling og sundhedsforhold
                                    1901-60, Copenhagen 1966.
  Befolkningens bevaegelser 1957
  Befolkningens bevaegelser 1958
  ...
  Befolkningens bevaegelser 2003
  Befolkningens bevaegelser 2004
  Vital Statistics 2005
  Vital Statistics 2006
}
\examples{
data(B.dk)
str(B.dk)
attach(B.dk)
# Plot the no of births and the M/F-ratio
par(las = 1, mar = c(4,4,2,4))
matplot(year + (month - 0.5) / 12, cbind(m, f),
        bty = "n", col = c("blue", "red"), lty = 1, lwd = 1, type = "l",
        ylim = c(0, 5000), xlab = "Date of birth", ylab = "" )
usr <- par()$usr
mtext("Monthly no. births in Denmark", side = 3,
      at = usr[1],  adj = 0.25, line = 1/1.6)
text(usr[1:2] \%*\% cbind(c(19,1), c(19,1)) / 20,
     usr[3:4] \%*\% cbind(c(1,19), c(2,18)) / 20,
     c("Boys","Girls"), col = c("blue","red"), adj = 0 ) 
lines(year + (month - 0.5) / 12, (m / (m + f) - 0.5) * 30000, lwd = 1)
axis(side = 4, at = (seq(0.505, 0.525, 0.005)-0.5) * 30000, labels = NA, tcl = -0.3 )
axis(side = 4, at = (50:53 / 100 - 0.5) * 30000, labels = 50:53, tcl = -0.5 )
axis(side = 4, at = (0.54 - 0.5) * 30000, labels = "\% boys",
     tick = FALSE, mgp = c(3,0.1,0))
abline(v = 1920, col = gray(0.8))
}
\keyword{datasets}

\name{effx}
\alias{effx}
\title{Function to calculate effects}
\description{
The function calculates the effects of an exposure on a response,
possibly stratified by a stratifying variable, and/or controlled for one
or more confounding variables.
}
\usage{
effx( response, type = "metric",
	         fup = NULL,
	    exposure,
	      strata = NULL,
	     control = NULL,
             weights = NULL,
                 eff = NULL,
	       alpha = 0.05,
	        base = 1,
	      digits = 3,
	        data = NULL )
}
\arguments{
\item{response}{The \code{response} variable - must be numeric or
  logical. If logical, \code{TRUE} is considered the outcome.}
\item{type}{The type of response\code{type} - must be one of "metric",
      "binary", "failure", or "count"}
\item{fup}{The \code{fup} variable contains the follow-up time for a
      failure response. This must be numeric.}
\item{exposure}{The \code{exposure} variable can be numeric or a factor}
\item{strata}{The \code{strata} stratifying variable - must be a factor}
\item{control}{The \code{control} variable(s) (confounders) - these are
  passed as a list if there are more than one.}
\item{weights}{Frequency weights for binary response only}
\item{eff}{How should effects be measured. If \code{response} is
  binomial, the default is "OR" (odds-ratio) with "RR" (relative risk)
  as an option. If \code{response} is failure, the default is "RR"
  (rate-ratio) with "RD" (rate difference) as an option.}
\item{base}{Baseline for the effects of a categorical exposure, either a
  number or a name of the level. Defaults to 1}
\item{digits}{Number of significant digits for the effects, default 3}
\item{alpha}{1 - confidence level}
\item{data}{\code{data} refers to the data used to evaluate the function}
}
\details{The function is a wrapper for glm. Effects are calculated as
  differences in means for a metric response, odds ratios/relative risks
  for a binary response, and rate ratios/rate differences for a failure
  or count response.

  The k-1 effects for a categorical exposure with k levels are relative
  to a baseline which, by default, is the first level. The effect of a
  metric (quantitative) exposure is calculated per unit of exposure.

  The exposure variable can be numeric or a factor, but if it is an
  ordered factor the order will be ignored.}
\value{
%  ~Describe the value returned
 % If it is a LIST, use
  \item{comp1 }{Effects of exposure}
  \item{comp2 }{Tests of significance}
% ...
}
\author{Michael Hills (*1934-Jun-07, +2021-Jan-07)}
%\note{ ~~further notes~~ }

%\seealso{ ~~objects to See Also as \code{\link{~~fun~~}}, ~~~ }
\examples{
library(Epi)
data(births)
births$hyp <- factor(births$hyp,labels=c("normal","hyper"))
births$sex <- factor(births$sex,labels=c("M","F"))

# bweight is the birth weight of the baby in gms, and is a metric
# response (the default)

# effect of hypertension on birth weight
effx(bweight,exposure=hyp,data=births)
# effect of hypertension on birth weight stratified by sex
effx(bweight,exposure=hyp,strata=sex,data=births)
# effect of hypertension on birth weight controlled for sex
effx(bweight,exposure=hyp,control=sex,data=births)

print( options('na.action') )
# effect of gestation time on birth weight
effx(bweight,exposure=gestwks,data=births)
# effect of gestation time on birth weight stratified by sex
effx(bweight,exposure=gestwks,strata=sex,data=births)
# effect of gestation time on birth weight controlled for sex
effx(bweight,exposure=gestwks,control=sex,data=births)

# lowbw is a binary response coded 1 for low birth weight and 0 otherwise
# effect of hypertension on low birth weight
effx(lowbw,type="binary",exposure=hyp,data=births)
effx(lowbw,type="binary",exposure=hyp,eff="RR",data=births)
}
\keyword{ models }
\keyword{ regression }

\name{merge.data.frame}
\alias{merge.data.frame}
\title{Merge data frame with a Lexis object}
\description{
  Merge two data frames, or a data frame with a \code{Lexis} object.
}
\usage{
\method{merge}{data.frame}(x, y, ...)
}
\arguments{
  \item{x, y}{data frames, or objects to be coerced into one}
  \item{...}{optional arguments for the merge method}
}
\details{
  This version of \code{merge.default} masks the one in the \code{base}.
  It ensures that, if either \code{x} or \code{y} is a \code{Lexis}
  object, then \code{merge.Lexis} is called.
}
\value{
  A merged \code{Lexis} object or data frame.
}
\author{Martyn Plummer}
\seealso{\code{\link{Lexis}}}
\keyword{ts}

\name{transform.Lexis}
\alias{transform.Lexis}
\alias{Relevel.Lexis}
\alias{transform.stacked.Lexis}
\alias{factorize}
\alias{factorize.Lexis}
\alias{levels.Lexis}
\alias{order.Lexis}
\alias{orderLexis}
\alias{sortLexis}
\title{Transform a Lexis (or stacked.Lexis) object}
\description{
  Modify a Lexis object.
}
\usage{
\method{factorize}{Lexis}(x, ..., verbose = FALSE)
\method{Relevel}{Lexis}(x, ref, \dots)
\method{levels}{Lexis}(x)
\method{transform}{Lexis}(`_data`, \dots)
\method{transform}{stacked.Lexis}(`_data`, \dots)
 order.Lexis(x)
  orderLexis(x)
   sortLexis(x)
}
\arguments{
  \item{_data}{an object of class \code{Lexis}.}
  \item{x}{an object of class \code{Lexis}.}
  \item{ref}{New names (or order) of the factor levels (states) for
    \code{lex.Cst} 
    and \code{lex.Xst}. Can be a list, in which case some levels are
    collapsed, see the documentation for \code{\link{Relevel}}. No
    sanity check for the latter type of operation is undertaken.}
  \item{\dots}{Additional arguments to be passed to
    \code{\link{transform.data.frame}}, \code{\link{Relevel.factor}}.}
  \item{verbose}{Logical. Should a list of new levels be printed?}
%   \code{order} or \code{sort}.}
}
\details{
  The transform method for \code{Lexis} objects works exactly as the
  method for data frames, but keeps the \code{Lexis} attributes.

  \code{factorize} transforms the variables
  \code{lex.Cst} and \code{lex.Xst} to factors with identical sets of
  levels.
  
  \code{Relevel} does the same as \code{\link{Relevel.factor}}, but for
  both the factors \code{lex.Cst} and \code{lex.Xst} in \code{x}.
  \code{lex.Cst} and \code{lex.Xst} must be factors with the same
  levels. They can be made so by \code{factorize}.

  If \code{ref} is an integer or character vector, the levels of
  \code{lex.Cst} and \code{lex.Xst} are permuted to match the order of
  \code{ref}. 
  
  If \code{ref} is \code{NULL}, as when for example the argument is
  not passed to the function, the returned object have levels of
  \code{lex.Cst}, \code{lex.Xst} (and for \code{stacked.Lexis} objects
  \code{lex.Tr}) shaved down to the actually occurring values; that is,
  empty levels are discarded.

  \code{order.Lexis} returns the order of the rows in a Lexis object to sort
  it by ()\code{lex.id},\code{ts}), where \code{ts} is a timescale in
  the Lexis object with no \code{NA}s. \code{orderLexis} is just a synonym.

  \code{sortLexis} returns the Lexis object sorted by
  (\code{lex.id}, \code{ts}) where \code{ts} is one of the
  \code{\link{timeScales}} with no \code{NA}s.   
}
\value{
  A transformed \code{Lexis} object.

  The function \code{levels} returns the names of the states (levels of
  the factors \code{lex.Cst} and \code{lex.Xst}.
  }
\author{Martyn Plummer, Bendix Carstensen}
\seealso{\code{\link{Lexis}},
         \code{\link{merge.Lexis}},
         \code{\link{subset.Lexis}},
         \code{\link{subset.stacked.Lexis}},
         \code{\link{Relevel}},
         \code{\link{transient}},
         \code{\link{absorbing}}}
\examples{
data( nickel )
nic <- Lexis( data = nickel,
                id = id,
             entry = list(age = agein),
              exit = list(age = ageout,
                          cal = ageout+dob,
                          tfh = ageout-age1st),
# Lung cancer deaths end as 2 and other deaths as 1
       exit.status = factor((icd > 0) + (icd \%in\% c(162,163)),
                            labels = c("Alive","Dead","Lung") ) )
str( nic )
levels( nic )
nit <- transform( nic, cumex = exposure * (agein - age1st) )
str( nit )

# It is still a Lexis object!
summary(nic)

# change order of levels
nix <- Relevel(nic, c("Alive", "Lung", "Dead"))
summary(nix)

# change names of levels 
niw <- Relevel(nix, list("Alive" = 1, "Pulm" = "Lung", "Mort" = "Dead"))
summary(niw)
boxes(niw, boxpos = TRUE)

# combine levels
niz <- Relevel(niw, list("Alive", c("Pulm", "Mort")), coll=" \n& ")
summary(niz)
par( new = TRUE )
boxes(niz, boxpos = TRUE)

#stack Lexis object
siw <- stack(niw)
str(siw)
}
\keyword{manip}

\name{nice}
\alias{nice}
\title{Nice breakpoints for axes on plots}
\description{The function calls \code{\link{pretty}}
  for linear scale. For a log-scale nice are computed using a set of
  specified number in each decade.
}
\usage{
nice(x, log = FALSE, lpos = c(1, 2, 5), xmx = 4, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{x}{Numerical vector to}
  \item{log}{Logical. Is the scale logartimic?}
  \item{lpos}{Numeric. Numbers between 1 and 10 giving the desired
    breakpoints in this interval.}
  \item{xmx}{Numeric. The maximal (absolute) power of 10 to be used for
    a log-scale.}
  \item{\dots}{Arguments passed on to \code{pretty} if \code{log}=FALSE}
}
\value{A vector of breakpoints.}
\author{Bendix Carstensen, \email{b@bxc.dk},
  \url{http://bendixcarstensen.com}}
\seealso{pretty}
\examples{
nice( exp( rnorm( 100 ) ), log=TRUE )
}
\keyword{manip}

\name{stack.Lexis}
\Rdversion{1.1}
\alias{stack.Lexis}
\alias{tmat}
\alias{tmat.Lexis}
\title{
Functions to facilitate analysis of multistate models.
}
\description{
\code{stack.Lexis} produces a stacked object suited for analysis of
several transition intensities simultaneously.
}
\usage{
\method{stack}{Lexis}(x, ...)
tmat( x, ... )
\method{tmat}{Lexis}(x, Y=FALSE, mode = "numeric", ...)
}
\arguments{
  \item{x}{A \code{\link{Lexis}} object.}
  \item{Y}{Logical. Should the risk time be put in the diagonal? This is
    a facility which is used by \code{\link{boxes.Lexis}}.}
  \item{mode}{Should the matrix be returned as a numeric matrix with
    \code{NA}s at unused places or (\code{mode="logical"}) as a logical
    matrix with \code{FALSE} on the diagonal.}
  \item{\dots}{Not used.}
}
\value{
\code{tmat.Lexis} returns a square transition matrix, classified by the
levels of \code{lex.Cst} and \code{lex.Xst}, for every transition
occurring the entry is the number of transitions occurring and \code{NA}
in all oter entries. If \code{Y=TRUE}, the diagonal will contain the
risk time in each of the states.

\code{stack.Lexis} returns a dataframe to be used for analysis of
multistate data when all transitions are modelled together, for example
if some parameters are required to be the same for different transitions.
The dataframe has class \code{stacked.Lexis}, and inherits the
attributes \code{time.scales} and \code{breaks} from the \code{Lexis}
object, and so function \code{\link{timeBand}} applies to a
\code{stacked.Lexis} object too.

The dataframe has same variables as the original \code{Lexis} object,
but with each record duplicated as many times as there are possible
exits from the current state, \code{lex.Cst}. Two variables are added:
\code{lex.Fail}, an indicator of wheter an event for the transition
named in the factor \code{lex.Tr} has occurred or not. \code{lex.Tr} is
a factor with levels made up of combinations of the levels of
\code{lex.Cst} and \code{lex.Xst} that do occur together in \code{x},
joined by a "\code{->}".}

\author{
Bendix Carstensen, \email{b@bxc.dk}, \url{http://bendixcarstensen.com}
}
\examples{
data(DMlate)
str(DMlate)
dml <- Lexis( entry=list(Per=dodm, Age=dodm-dobth, DMdur=0 ),
               exit=list(Per=dox),
        exit.status=factor(!is.na(dodth),labels=c("DM","Dead")),
               data=DMlate )
dmi <- cutLexis( dml, cut=dml$doins, new.state="Ins", pre="DM" )
summary( dmi )
ls.dmi <- stack( dmi )
str( ls.dmi )
# Check that all the transitions and person-years got across.
with( ls.dmi, rbind( table(lex.Fail,lex.Tr),
                     tapply(lex.dur,lex.Tr,sum) ) )
}
\seealso{
\code{\link{splitLexis}}
\code{\link{cutLexis}}
\code{\link{Lexis}}
}
\keyword{survival}

\name{float}
\alias{float}
\alias{print.floated}
\title{Calculate floated variances}
\description{
  Given a fitted  model object, the \code{float()} function calculates
  floating variances (a.k.a. quasi-variances) for a given factor in the model.
}
\usage{
float(object, factor, iter.max=50, ...)
}
\arguments{
  \item{object}{a fitted model object.}
  \item{factor}{character string giving the name of the factor of
    interest. If this is not given, the first factor in the model is used.}
  \item{iter.max}{Maximum number of iterations for EM algorithm.}
  \item{...}{Optional arguments passed to the \code{vcov()} method for
  the fitted model object.}
}
\details{

  The \code{float()} function implements the "floating absolute risk"
  proposal of Easton, Peto and Babiker (1992). This is an alternative way
  of presenting parameter estimates for factors in regression models,
  which avoids some of the difficulties of treatment contrasts. It was
  originally designed for epidemiological studies of relative risk, but
  the idea is widely applicable.

  Treatment contrasts are not orthogonal. Consequently, the variances of
  treatment contrast estimates may be inflated by a poor choice of
  reference level, and the correlations between them may also be high.
  The \code{float()} function associates each level of the factor with a
  "floating" variance (or quasi-variance), including the reference
  level.  Floating variances are not real variances, but they can be
  used to calculate the variance error of contrast by treating each
  level as independent.

  Plummer (2003) showed that floating variances can be derived from a
  covariance structure model applied to the variance-covariance matrix
  of the contrast estimates. This model can be fitted by minimizing the
  Kullback-Leibler information divergence between the true distribution
  of the parameter estimates and the simplified distribution given by
  the covariance structure model. Fitting is done using the EM
  algorithm.

  In order to check the goodness-of-fit of the floating variance model,
  the \code{float()} function compares the standard errors predicted by
  the model with the standard errors derived from the true
  variance-covariance matrix of the parameter contrasts. The maximum and
  minimum ratios between true and model-based standard errors are
  calculated over all possible contrasts. These should be within 5
  percent, or the use of the floating variances may lead to invalid
  confidence intervals.

}
\value{
  An object of class \code{floated}. This is a list with the following
  components
  \item{coef}{A vector of coefficients. These are the same as the
    treatment contrasts but the reference level is present with
    coefficient 0.}
  \item{var}{A vector of floating (or quasi-) variances}
  \item{limits}{The bounds on the accuracy of standard errors over all
    possible contrasts}
}
\note{
  Menezes(1999) and Firth and Menezes (2004) take a slightly different
  approach to this problem, using a pseudo-likelihood approach to fit
  the quasi-variance model. Their work is implemented in the package
  qvcalc.
}
\references{
  Easton DF, Peto J and Babiker GAG (1991) Floating absolute risk: An
  alternative to relative risk in survival and case control analysis
  avoiding an arbitrary reference group. \emph{Statistics in Medicine},
  \bold{10}, 1025-1035.

  Firth D and Mezezes RX (2004)  Quasi-variances.
  \emph{Biometrika} \bold{91}, 65-80.

  Menezes RX(1999)  More useful standard errors for group and factor
  effects in generalized linear models.  \emph{D.Phil. Thesis},
  Department of Statistics, University of Oxford.

  Plummer M (2003) Improved estimates of floating absolute risk,
  \emph{Statistics in Medicine}, \bold{23}, 93-104.
}
\author{Martyn Plummer}
\seealso{\code{\link{ftrend}}, \code{qvcalc}}
\keyword{regression}

\name{lls}
\alias{lls}
\alias{clear}
\title{Functions to manage and explore the workspace
}
\description{These functions help you to find out what has gone wrong and to
start afresh if needed.
}
\usage{
lls(pos = 1, pat = "", all=FALSE, print=TRUE )
clear()
}
\arguments{
  \item{pos}{Numeric. What position in the search path do you want listed.}
  \item{pat}{Character. List only objects that have this string in their name.}
  \item{all}{Logical. Should invisible objects be printed too -
             see \code{\link{ls}} to which this argument is passed.}
  \item{print}{Logical. Should the result be printed?}
  }
\details{\code{lls} is designed to give a quick overview of the name, mode, class
  and dimension of the object in your workspace. They may not always be what you
  think they are.

  \code{clear} clears all your objects from workspace, and all attached objects
  too --- it only leaves the loaded packages in the search path; thus allowing a
  fresh start without closing and restarting R.
  }
\value{
\code{lls} returns a data frame with four character variables:
\code{name},
\code{mode},
\code{class} and
\code{size} and one row per object in the workspace (if \code{pos=1}).
\code{size} is either the length or the dimension of the object.
The data frame is by default printed with left-justified columns.
}
\author{\code{lls}: Unknown. Modified by Bendix Carstensen from a long
        forgotten snatch.
        
        \code{clear}: Michael Hills / David Clayton.}
\examples{
x <- 1:10
y <- rbinom(10, 1, 0.5)
m1 <- glm( y ~ x, family=binomial )
M <- matrix( 1:20, 4, 5 )
.M <- M
dfr <- data.frame(x,y)
attach( dfr )
lls()
search()
clear()
search()
lls()
lls(all=TRUE)
}
\keyword{attributes}

\name{gmortDK}
\alias{gmortDK}
\docType{data}
\title{Population mortality rates for Denmark in 5-years age groups.}
\description{
The \code{gmortDK} data frame has 418 rows and 21 columns.
}
\format{
This data frame contains the following columns:
\tabular{rl}{
\code{agr}: \tab Age group, 0:0--4, 5:5--9,..., 90:90+. \cr
\code{per}: \tab Calendar period, 38: 1938--42, 43: 1943--47, ..., 88:1988-92. \cr
\code{sex}: \tab Sex, 1: male, 2: female. \cr
\code{risk}: \tab Number of person-years in the Danish population. \cr
\code{dt}: \tab Number of deaths. \cr
\code{rt}: \tab Overall mortality rate in cases per 1000 person-years, i.e.
\code{rt=1000*dt/risk} \cr
           \tab Cause-specific mortality rates in cases per 1000 person-years: \cr
\code{r1}: \tab Infections \cr
\code{r2}: \tab Cancer. \cr
\code{r3}: \tab Tumors, benign, unspecific nature. \cr
\code{r4}: \tab Endocrine, metabolic. \cr
\code{r5}: \tab Blood. \cr
\code{r6}: \tab Nervous system, psychiatric. \cr
\code{r7}: \tab Cerebrovascular. \cr
\code{r8}: \tab Cardiac. \cr
\code{r9}: \tab Respiratory diseases, excl. cancer. \cr
\code{r10}: \tab Liver, excl. cancer. \cr
\code{r11}: \tab Digestive, other. \cr
\code{r12}: \tab Genitourinary. \cr
\code{r13}: \tab Ill-defined symptoms. \cr
\code{r14}: \tab All other, natural. \cr
\code{r15}: \tab Violent. \cr
}
}
\source{
Statistics Denmark, National board of health provided original data. 
Michael Andersson grouped the causes of death.
}
\examples{
data(gmortDK)
}
\seealso{\code{\link{thoro}}, \code{\link{mortDK}}}
\keyword{datasets}

\name{lep}
\alias{lep}
\docType{data}
\title{An unmatched case-control study of leprosy incidence}
\description{
The \code{lep} data frame has 1370 rows and 7 columns. This was an
unmatched case-control study in which incident cases of leprosy in a
region of N. Malawi were compared with population controls. 
}
\format{
This data frame contains the following columns:
\tabular{rl}{
\code{id}: \tab subject identifier: a numeric vector \cr
\code{d}: \tab case/control status: a numeric vector (1=case, 0=control) \cr

\code{age}: \tab a factor with levels
\code{5-9} 
\code{10-14} 
\code{15-19} 
\code{20-24} 
\code{25-29} 
\code{30-44} 
\code{45+}  \cr

\code{sex}: \tab a factor with levels
\code{male},  
\code{female}  \cr 

\code{bcg}: \tab presence of vaccine scar, a factor with levels
\code{no} 
\code{yes}  \cr

\code{school}: \tab schooling, a factor with levels
\code{none} 
\code{1-5yrs} 
\code{6-8yrs} 
\code{sec/tert} \cr

\code{house}: \tab housing, a factor with levels
\code{brick} 
\code{sunbrick} 
\code{wattle} 
\code{temp}  \cr
}
}
\source{
The study is described in more detail in Clayton and Hills, Statistical
Models in Epidemiology, Oxford University Press, Oxford:1993.
}
\examples{
data(lep)
}
\keyword{datasets}

\name{brv}
\alias{brv}
\docType{data}
\title{Bereavement in an elderly cohort}
\description{
The \code{brv} data frame has 399 rows and 11 columns.
The data concern the possible effect of marital bereavement on
subsequent mortality. They arose from a survey of the physical and
mental health of a cohort of 75-year-olds in one large general
practice. These data concern mortality up to 1 January, 1990 (although
further follow-up has now taken place).

Subjects included all lived with a living spouse when they entered the
study. There are three distinct groups of such subjects: (1) those in
which both members of the couple were over 75 and therefore included in
the cohort, (2) those whose spouse was below 75 (and was not, therefore,
part of the main cohort study), and (3) those living in larger
households (that is, not just with their spouse).
}
\format{
  This data frame contains the following columns:
\describe{  
\item{\code{id}}{subject identifier, a numeric vector}
\item{\code{couple}}{couple identifier, a numeric vector}
\item{\code{dob}}{date of birth, a date}
\item{\code{doe}}{date of entry into follow-up study, a date}
\item{\code{dox}}{date of exit from follow-up study, a date}
\item{\code{dosp}}{date of death of spouse, a date (if the spouse was still alive
                  at the end of follow-up,this was coded to January 1, 2000)}
\item{\code{fail}}{status at end of follow-up,
                  a numeric vector (0=alive,1=dead)}
\item{\code{group}}{see Description, a numeric vector}
\item{\code{disab}}{disability score, a numeric vector}
\item{\code{health}}{perceived health status score, a numeric vector}
\item{\code{sex}}{a factor with levels
                 \code{Male} and
                 \code{Female} }
	     }
 }
\source{
Jagger C, and Sutton CJ, Death after Marital Bereavement. Statistics in
Medicine, 10:395-404, 1991. (Data supplied by Carol Jagger).
}
\examples{
data(brv)
}
\keyword{datasets}

\name{xcutLexis}
\alias{xcutLexis}
\title{
Cut follow-up in a \code{\link{Lexis}} object by event date(s) while
preserving the original states.
}

\description{
With a multistate \code{\link{Lexis}} object we might want a
classification of the follow-up according to a set of events independent
of the states in the Lexis object, while keeping the original states.
This is essentially expanding the states with a cross-classification of
the original states and a set of new ones, hence the "\code{x}". Limited
to 1 event time per person.
}
\usage{
xcutLexis(Lx, cut, timescale = 1, sep = ".")
}
\arguments{
  \item{Lx}{
A \code{Lexis} object
           }
  \item{cut}{
A data frame with at most one row per \code{lex.id}, with columns:
    \itemize{
    \item \code{lex.id}: person-id referring to \code{lex.id} in \code{Lx}
    \item \code{cut}, a numeric; times of event om the time scale
          \code{timescale} 
    \item \code{new.state}, character or factor; name of the new state
    occurring at time \code{cut} 
            }
            }
  \item{timescale}{
The timescale that \code{cut} refers to. Numeric or character.
                  }
  \item{sep}{
Character; string used in forming new state names by joining original
state names in \code{Lx} and state names in \code{cut$new.states} 
            }
}
\details{
The function was motivated by a follow-up through states of
multimorbidity with state names \code{0morb},  \code{1morb},
\code{2morb}, etc. where we wanted a subdivision of each state by the
presence of T2 diabetes, resulting in states
\code{0morb},  \code{1morb}, \code{2morb},
\code{0morb.T2},  \code{1morb.T2}, \code{2morb.T2} etc.

At most 1 transition per person is allowed in \code{cut}.
}
\value{
A \code{Lexis} object with states as in \code{Lx} plus states named by
concatenating \code{Lx} state names with names in \code{cut$new.state}
}
\author{Bendix Carstensen, Steno Diabetes Center Copenhagen,
  \email{b@bxc.dk}, 
\url{http://bendixcarstensen.com}
}
\seealso{
  \code{\link{cutLexis}},
  \code{\link{mcutLexis}},
  \code{\link{rcutLexis}},
  \code{\link{addCov.Lexis}},
  \code{\link{addDrug.Lexis}},
  \code{\link{splitLexis}},
  \code{\link{Lexis}},
  \code{\link{summary.Lexis}},
  \code{\link{timeSince}},
  \code{\link{boxes.Lexis}}
}
\examples{
# construct a Lexis object and show states and transitions
example(DMlate)
levels(dmi)
#
# show transitions between states in dmi
boxes(dmi, boxpos = list(x = rep(50, 4),
                         y = c(65, 35, 95, 5)),
           scale.R = 1000,
           show.BE = TRUE,
           cex = 1)
#
# randomly generated intermediate events X and Y
set.seed(1952)
cutXY <- data.frame(lex.id = unique(dmi$lex.id),          # one row per id
                       cut = runif(nid(dmi), 1995, 2008), # event dates
                 new.state = sample(c("X","Y"),           # event types
                                    nid(dmi),    # only 4 in 10 has an event
                                    repl = TRUE))[runif(nid(dmi)) < 0.4, ]
#
# cut at these event dates but also keep original states
Lxy <- xcutLexis(dmi, cutXY)
levels(Lxy)
#
# reorder the levels for easier specification of box places
Lxy <- Relevel(Lxy, as.vector(t(outer(levels(dmi),
                                      c("", ".X", ".Y"),
                                      paste0))))
cbind(levels(Lxy))
summary(Lxy)
#
# resulting transitions between boxes
boxes(Lxy, boxpos = list(x = rep(c(50, 15, 85), 4),
                         y = rep(c(65,35,95,5), each = 3)),
           scale.R = 1000,
           show.BE = TRUE,
           cex = 1)
}
\keyword{survival}
\concept{Lexis}
\concept{multistate models}
\concept{transition rates}
\concept{state transition}

\name{ncut}
\alias{ncut}
\title{
Function to group a variable in intervals.}
\description{
  Cuts a continuous variable in intervals. As opposed to \code{cut}
  which returns a factor, \code{ncut} returns a numeric variable.
}
\usage{
ncut(x, breaks, type="left" )
}
\arguments{
  \item{x}{A numerical vector.}
  \item{breaks}{Vector of breakpoints. \code{NA} will results for values
    below \code{min(breaks)} if \code{type="left"}, for values
    above \code{max(breaks)} if \code{type="right"} and for values
    outside \code{range(breaks)} if \code{type="mid"}}
  \item{type}{Character: one of \code{c("left","right","mid")},
    indicating whether the left, right or midpoint of the intervals
    defined in breaks is returned.}
}
\details{
The function uses the base function \code{findInterval}.
}
\value{
A numerical vector of the same length as \code{x}.
}
\author{
  Bendix Carstensen, Steno Diabetes Center, \email{b@bxc.dk},
  \url{http://bendixcarstensen.com}, with essential input
  from Martyn Plummer, \email{martyn.plummer@r-project.org}
}
\seealso{
  \code{\link{cut}}, \code{\link{findInterval}}
}
\examples{
br <- c(-2,0,1,2.5)
x <- c( rnorm( 10 ), br, -3, 3 )
cbind( x, l=ncut( x, breaks=br, type="l" ),
          m=ncut( x, breaks=br, type="m" ),
          r=ncut( x, breaks=br, type="r" ) )[order(x),]
x <- rnorm( 200 )
plot( x, ncut( x, breaks=br, type="l" ), pch=16, col="blue", ylim=range(x) )
abline( 0, 1 )
abline( v=br )
points( x, ncut( x, breaks=br, type="r" ), pch=16, col="red" )
points( x, ncut( x, breaks=br, type="m" ), pch=16, col="green" )
}
\keyword{manip}

\name{timeBand}
\alias{timeBand}
\alias{breaks}
\title{Extract time band data from a split Lexis object}
\description{
  The break points of a \code{Lexis} object (created by a call to
  \code{splitLexis}) divide the follow-up intervals into time bands
  along a given time scale. The \code{breaks} function returns
  the break points, for a given time scale, and the \code{timeBand}
  classifies each row (=follow-up interval) into one of the time bands.
}
\usage{
timeBand(lex, time.scale, type="integer")
breaks(lex, time.scale)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{lex}{an object of class \code{Lexis}}
  \item{time.scale}{a character or integer vector of length 1
    identifying the time scale of interest}
  \item{type}{a string that determines how the time bands are labelled.
    See Details below}
}
\details{
  Time bands may be labelled in various ways according to the
  \code{type} argument. The permitted values of the \code{type}
  argument, and the corresponding return values are:
  \describe{
  \item{"integer"}{a numeric vector with integer codes starting from 0.}
  \item{"factor"}{a factor (unordered) with labels "(left,right]"}
  \item{"left"}{the left-hand limit of the time band}
  \item{"middle"}{the midpoint of the time band}
  \item{"right"}{the right-hand limit of the time band}
  }
}    
\value{
  The \code{breaks} function returns a vector of break points
  for the \code{Lexis} object, or NULL if no break points have been
  defined by a call to \code{splitLexis}.  The \code{timeBand}
  function returns a numeric vector or factor, depending on the value
  of the \code{type} argument.
}
\author{Martyn Plummer}
\note{
  A newly created \code{Lexis} object has no break points defined.
  In this case, \code{breaks} will return NULL, and
  \code{timeBand} will a vector of zeros.
}  
\examples{
data(diet)
diet <- cal.yr(diet)
diet.lex <- Lexis(entry=list(period=doe),
                   exit=list(period=dox, age=dox-dob),
            exit.status=chd,
                   data=diet)
diet.split <- splitLexis(diet.lex, breaks=seq(40,70,5), "age" )
age.left <- timeBand(diet.split, "age", "left")
table(age.left)
age.fact <- timeBand(diet.split, "age", "factor")
table(age.fact)
age.mid <- timeBand(diet.split, "age", "mid")
table(age.mid)
}
\seealso{\code{\link{Lexis}}}
\keyword{attribute}

\name{pr}
\alias{pr}
\docType{data}
\title{Diabetes prevance as of 2010-01-01 in Denmark
}
\description{
Diabetes prevalence as of 2010-01-01 in Denmark in 1-year age classes by sex.
}
\usage{data("pr")}
\format{
  A data frame with 200 observations on the following 4 variables.
  \describe{
    \item{\code{A}}{Numeric, age, 0-99}
    \item{\code{sex}}{Sex, a factor with levels \code{M} \code{F}}
    \item{\code{X}}{Number of diabetes patients}
    \item{\code{N}}{Population size}
  }
}
\examples{
data(pr)
str(pr)
}
\keyword{datasets}

\name{gen.exp}
\alias{gen.exp}
\title{
Generate covariates for drug-exposure follow-up from drug purchase records.
}
\description{
From records of drug purchase and possibly known treatment intensity,
the time since first drug use and cumulative dose at prespecified times
is computed. Optionally, lagged exposures are computed too,
i.e. cumulative exposure a prespecified time ago.
}
\usage{
gen.exp( purchase, id="id", dop="dop", amt="amt", dpt="dpt",
               fu, doe="doe", dox="dox",
           breaks,
          use.dpt = ( dpt \%in\% names(purchase) ),
         push.max = Inf,
          rm.dose = FALSE,
             lags = NULL,
          lag.dec = 1,
          lag.pre = "lag.",
         pred.win = Inf )
}
\arguments{
  \item{purchase}{Data frame with columns \code{id}-person id,
    \code{dop} - \code{d}ate \code{o}f \code{p}urchase, \code{amt} -
    \code{am}oun\code{t} purchased, and optionally \code{dpt} -
    (\code{d}ose \code{p}er \code{t}ime) ("defined daily dose", DDD,
    that is), how much is assumed to be ingested per unit time. The
    units used for \code{dpt} is assumed to be units of \code{amt} per
    units of \code{dop}.} 
  \item{id}{Character. Name of the id variable in the data frame.}
  \item{dop}{Character. Name of the \code{d}ate \code{o}f
    \code{p}urchase variable in the data frame.} 
  \item{amt}{Character. Name of the \code{am}oun\code{t} purchased
    variable in the data frame.} 
  \item{dpt}{Character. Name of the \code{d}ose-\code{p}er-\code{t}ime
    variable in the data frame.} 
  \item{fu}{Data frame with \code{f}ollow-\code{u}p period for each
    person, the person id variable must have the same name as in the
    \code{purchase} data frame.} 
  \item{doe}{Character. Name of the \code{d}ate \code{o}f \code{e}ntry
    variable.} 
  \item{dox}{Character. Name of the \code{d}ate \code{o}f e\code{x}it
    variable.} 
  \item{breaks}{Numerical vector of dates at which the time since first
    exposure, cumulative dose etc. are computed.} 
  \item{use.dpt}{Logical: should we use information on dose per time.} 
  \item{push.max}{Numerical. How much can purchases maximally be pushed
    forward in time. See details.}
  \item{rm.dose}{Logical. Should the dose from omitted period of
    exposure (due to the setting of \code{push.max}) be ignored. If
    \code{FALSE}, the cumulative dose will be the cumulation of the
    actually purchased amounts, regardless of how far the inception
    dates have been pushed.}
  \item{lags}{Numerical vector of lag-times used in computing lagged
    cumulative doses.}
  \item{lag.dec}{How many decimals to use in the construction of names
    for the lagged exposure variables} 
  \item{lag.pre}{Character string used for prefixing names of lagged
    exposure variables. Aimed to facilitate the use of \code{gen.exp}
    for different drugs with the aim of merging information.}
  \item{pred.win}{The length of the window used for constructing the
    average dose per time used to compute the duration of the last
    purchase. Only used when \code{use.dpt=FALSE}. The default value
    \code{Inf} corresponds to using the time between first and last
    purchase of drug as the interval for computing average consumption
    per time, and thus the termination of use.}
}

\details{
  The intention of this function is to generate covariates for a
  particular drug for the entire follow-up of each person. The reason
  that the follow-up prior to first drug purchase and post-exposure is
  included is that the covariates must be defined for all follow-up for
  each person in order to be useful for analysis of disease outcomes.

  The functionality is described in terms of calendar time as underlying
  time scale, because this will normally be the time scale for drug
  purchases and for entry and exit for persons. In principle the
  variables termed as dates might equally well refer to say the age
  scale, but this would then have to be true \emph{both} for the
  purchase data, the follow-up data and the \code{breaks} argument.  

  Drug purchase records (in \code{purchase}) are used to construct
  measures of drug exposure at prespecified timepoints (in
  \code{breaks}) in follow-up intervals (in \code{fu}). Each person may
  have more than one follow-up interval. They should be disjoint, but
  this is not checked.

  If \code{use.dpt} is \code{TRUE} then the dose per time information is
  used to compute the exposure interval associated with each purchase.
  Exposure intervals are stacked, that is each interval is put after any
  previous. This means that the start of exposure to a given purchase
  can be pushed into the future. The parameter \code{push.max} indicates
  the maximally tolerated push. If this is reached by a person, the
  assumption is that some of the purchased drug may not be counted in
  the exposure calculations --- see \code{rm.dose}.
  
  The \code{dpt} can either be a constant, basically translating each
  purchased amount into exposure time the same way for all persons, or
  it can be a vector with different treatment intensities for each
  purchase. In any case the cumulative dose is computed taking
  \code{dpt} into account, unless \code{rm.dose} is \code{FALSE} in
  which case the actual purchased amount is cumulated. The latter is
  slightly counter-intuitive because we are using the \code{dpt} to push
  the intervals, and then disregard it when computing the cumulative
  dose. The counter argument is that if the limit \code{push.max} is
  reached, the actual dosage may be larger than indicated the
  \code{dpt}, and is essentially what this allows for.

  If \code{use.dpt} is \code{FALSE} then the exposure from one purchase
  is assumed to stretch over the time to the next purchase, so we are
  effectively allowing different dosing rates (dose per time) between
  purchases. Formally this approach conditions on the future, because
  the rate of consumption (the accumulation of cumulative exposure) is
  computed based on knowledge of when next purchase is made. Moreover,
  with this approach, periods of non-exposure does not exist, except
  after the last purchase where the future consumption rate is taken to
  be the average over the period of use (or a period of length
  \code{pred.win}), and hence defines a date of cessation of drug.

  Finally, if \code{use.dpt} is \code{FALSE}, at least two purchase
  records are required to compute the measures. Therefore persons with
  only one drug purchase record are ignored in calculations.
}

\value{A data frame with one record per person and follow-up date
  (\code{breaks}). Date of entry and date of exit are included too; but
  only follow-up in the intersetion of \code{range(breaks)} and
  \code{range(fu$doe,fu$dox)} is output.
\describe{
    \item{\code{id}}{person id.}
    \item{\code{dof}}{date of follow up, i.e. start of interval. Apart
  from possibly the first interval for each person, this will assume
  values in the set of the values in \code{breaks}. All other variables
  refer to status as of this date.}
    \item{\code{dur}}{the length (\code{dur}ation) of interval.}
    \item{\code{tfi}}{\code{t}ime \code{f}rom first \code{i}nitiation of drug.}
    \item{\code{off}}{Logical, indicating whether the person is
  \code{off} drug. So it is \code{FALSE} if the person is exposed at \code{dof}.} 
    \item{\code{doff}}{\code{d}ate of latest transition to \code{off}
  drug. Note that tis defined also at dates after drug exposure has been
  resumed.}
    \item{\code{tfc}}{\code{t}ime \code{f}rom latest \code{c}essation of drug.}
    \item{\code{ctim}}{\code{c}umulative \code{tim}e on the drug.}
    \item{\code{cdos}}{\code{c}umulative \code{dos}e.}
    \item{\code{ldos}}{suffixed with one value per element in
  \code{lags}, the latter giving the cumulative doses \code{lags} before
  \code{dof}. }
}
}

\author{Bendix Carstensen, \email{b@bxc.dk}.  The development of
this function was supported partly through a grant from the EFSD
(European Foundation for the Study of Diabetes)}

\seealso{
  \code{\link{Lexis}}, 
  \code{\link{cutLexis}},
  \code{\link{mcutLexis}},
  \code{\link{addCov.Lexis}}}
\examples{
# Example data for drug purchases in 3 persons --- dates (dop) are
# measured in years, amount purchased (amt) in no. pills and dose per
# time (dpt) consequently given in units of pills/year. Note we also
# include a person (id=4) with one purchase record only.
n <- c( 10, 18, 8, 1 )
hole <- rep(0,n[2])
hole[10] <- 2 # to create a hole of 2 years in purchase dates
# dates of drug purchase
dop <- c( 1995.278+cumsum(sample(1:4/10,n[1],replace=TRUE)),
          1992.351+cumsum(sample(1:4/10,n[2],replace=TRUE)+hole),
          1997.320+cumsum(sample(1:4/10,n[3],replace=TRUE)),
          1996.470 )
# purchased amounts mesured in no. pills
amt <- sample( 1:3*50 , sum(n), replace=TRUE )
# prescribed dosage therefore necessarily as pills per year 
dpt <- sample( 4:1*365, sum(n), replace=TRUE )
# collect to purchase data frame
dfr <- data.frame( id = rep(1:4,n),
                  dop,
                  amt = amt,
                  dpt = dpt )
head( dfr, 3 )

# a simple dataframe for follow-up periods for these 4 persons
fu <- data.frame( id = 1:4,
                 doe = c(1995,1992,1996,1997)+1:4/4,
                 dox = c(2001,2003,2002,2010)+1:4/5 )
fu

# Note that the following use of gen.exp relies on the fact that the
# purchase dataframe dfr has variable names "id", "dop", "amt" and
# "dpt"" and the follow-up data frame fu has variable names "id",
# "doe" and "dox"

# 1: using the dosage information
dposx <- gen.exp( dfr,
                   fu = fu,
              use.dpt = TRUE,
               breaks = seq(1990,2015,0.5),
                 lags = 2:4/4,
              lag.pre = "l_" )
format( dposx, digits=5 )

# 2: ignoring the dosage information,
#    hence person 4 with only one purchase is omitted
xposx <- gen.exp( dfr,
                   fu = fu,
              use.dpt = FALSE,
               breaks = seq(1990,2015,0.5),
                 lags = 2:3/5 )
format( xposx, digits=5 )

# It is possible to have disjoint follow-up periods for the same person:
fu <- fu[c(1,2,2,3),]
fu$dox[2] <- 1996.2
fu$doe[3] <- 1998.3
fu

# Note that drug purchase information for the period not at risk *is* used
dposx <- gen.exp( dfr,
                   fu = fu,
              use.dpt = TRUE,
               breaks = seq(1990,2015,0.1),
                 lags = 2:4/4 )
format( dposx, digits=5 )
}
\keyword{data manipulation}

\name{DMlate}
\Rdversion{1.1}
\alias{DMlate}
\alias{DMrand}
\docType{data}
\title{
The Danish National Diabetes Register.
}
\description{
These two datasets each contain a random sample of 10,000 persons from
the Danish National Diabetes Register. \code{DMrand} is a random sample
from the register, whereas \code{DMlate} is a random sample among those
with date of diagnosis after 1.1.1995. All dates are radomly jittered by
adding a U(-7,7) (days).
}
\usage{data(DMrand)
       data(DMlate)}
\format{
  A data frame with 10000 observations on the following 7 variables.
  \describe{
    \item{\code{sex}}{Sex, a factor with levels \code{M} \code{F}}
    \item{\code{dobth}}{Date of birth}
    \item{\code{dodm}}{Date of inclusion in the register}
    \item{\code{dodth}}{Date of death}
    \item{\code{dooad}}{Date of 2nd prescription of OAD}
    \item{\code{doins}}{Date of 2nd insulin prescription}
    \item{\code{dox}}{Date of exit from follow-up.}
  }
}
\details{All dates are given in fractions of years, so 1998.000
  corresponds to 1 January 1998 and 1998.997 to 31 December 1998.

  All dates are randomly perturbed by a small amount, so no real
  persons have any of the combinations of the 6 dates in the
  dataset. But results derived from the data will be quite close to
  those that would be obtained if the entire 'real' diabetes register
  were used.
}
\source{
Danish National Board of Health.
}
\references{
B Carstensen, JK Kristensen, P Ottosen and K Borch-Johnsen:
The Danish National Diabetes Register: Trends in incidence, prevalence and
mortality, Diabetologia, 51, pp 2187--2196, 2008.

In partucular see the appendix at the end of the paper.
}
\examples{
data(DMlate)
str(DMlate)
dml <- Lexis( entry=list(Per=dodm, Age=dodm-dobth, DMdur=0 ),
               exit=list(Per=dox),
        exit.status=factor(!is.na(dodth),labels=c("DM","Dead")),
               data=DMlate )

# Cut the follow-up at insulin start, and introduce a new timescale,
# and split non-precursor states
system.time(
dmi <- cutLexis( dml, cut = dml$doins,
                      pre = "DM",
                new.state = "Ins",
                new.scale = "t.Ins",
             split.states = TRUE ) )
summary( dmi )
}
\keyword{datasets}

\name{addDrug}
\alias{addDrug}
\alias{addDrug.Lexis}
\alias{coarse.Lexis}
\title{
Expand a Lexis object with information of drug exposure based on
purchase dates and -amounts
}
\description{  
A \code{\link{Lexis}} object will contain information on follow-up for a
cohort of persons through time, each record containing information of
one time interval, including the time at the beginning of each
interval. If information on drug purchase is known for the persons via
\code{lex.id} in a list of data frames, \code{addDrug.Lexis} will expand
the \code{Lexis} object by cutting at all drug purchase dates, and
compute the exposure status for any number of drugs, and add these as
variables.

In some circumstances the result is a Lexis object with a very large
number of very small follow-up intervals. The function
\code{coarse.Lexis} combines consecutive follow-up intervals using the
covariates from the first of the intervals.
}
\usage{
\method{addDrug}{Lexis}(Lx,  # Lexis object
            pdat,  # list of data frames with drug purchase information
             amt = "amt", # name of the variable with purchased amount
             dpt = "dpt", # name of the variable with amount consumed per time
             apt = NULL,  # old name for dpt
          method = "ext", # method use to compute exposure 
            maxt = NULL,  # max duration for a purchase when using "fix"
           grace = 0,     # grace  period to be added
            tnam = setdiff(names(pdat[[1]]), c("lex.id", amt))[1],
                          # name of the time variable from Lx
          prefix = TRUE,  # should drug names prefix variable names 
          sepfix = ".",   # what should the separator be when forming prefix/suffix
         verbose = TRUE,
           \dots)
coarse.Lexis(Lx, lim, keep = FALSE)
}

\arguments{
  \item{Lx}{A \code{Lexis} object.
}
  \item{pdat}{Named list of data frames with drug \code{p}urchase \code{dat}a.
}
  \item{amt}{Name of the variable in the data frames in \code{pdat} with
    the purchased \code{am}oun\code{t}.
}
  \item{dpt}{Name of the variable in the data frames in \code{pdat} with
    the consumed \code{d}ose \code{p}er \code{t}ime. Must be given in
    units of units of \code{amt} per units of \code{lex.dur} in \code{Lx}.
}
  \item{apt}{Name previously used for \code{dpt}. Will disappear in next
    version.
}
  \item{method}{Character. One of \code{"ext"} (default), \code{"dos"}
    or  \code{"fix"}, for a description, see details.
}
  \item{maxt}{Numerical. Maximal duration for a purchase when using
    \code{method="fix"}, same units as \code{lex.dur}.
}
  \item{grace}{Numeric. Grace period to be added after last time of
    computed drug coverage to define end of exposure, same units as
    \code{lex.dur}. 
}
  \item{tnam}{Character. Name of the timescale used in the data frames
  in \code{pdat}. 
}
  \item{prefix}{Logical. Should the names of \code{pdat} be used as
  prefix for the 4 generated exposure variables for each drug. If false
  the names of \code{pdat} will be used as suffix.
}
  \item{sepfix}{Character, used to separate the \code{prefix} and the
    name of the generated type of variable.
}
  \item{verbose}{Logical. Should the function tell you about the choices
  you made?
}
 \item{lim}{Numeric vector of length 2. Consecutive follow-up intervals
  are combined if the first has \code{lex.dur} < \code{lim[1]}, and the
  sum of \code{lex.dur} in the two intervals is smaller than
  \code{lim[2]}. If a scalar i given, \code{c(lim,3*lim)} is used.
}
  \item{keep}{Logical of length 1 or \code{nrow(Lx)} that points to
  records that cannot be combined with preceding records.
}
  \item{\dots}{Arguments passed on. Ignored.
}
}

\details{
This function internally uses \code{\link{addCov.Lexis}} to attach
exposure status for several drugs (dispensed medicine) to follow-up in a
\code{Lexis} object. Once that is done, the exposure measures are
calculated at each time.

There is one input data frame per type of drug, each with variables
\code{lex.id}, \code{amt}, a timescale variable and possibly a variable
\code{dpt}.

Three different methods for computing drug exposures from dates and
amounts of purchases are supported via the argument \code{method}.

\itemize{ 
 \item \code{"ext"}: Extrapolation: the first drug purchase is assumed
 consumed over the interval to the second purchase. Exposure
 for subsequent purchases are assumed to last as long as it would have if
 consumed at a speed corresponding to the previous purchase being
 consumed over the time span between the previous and current purchase,
 plus a period of length \code{grace}.
 
 \item \code{"dos"}: Dosage: assumes that each purchase lasts
 \code{amt}/\code{dpt} plus \code{grace}.
 
 \item \code{"fix"}: Fixed time: assumes that each purchase lasts
 \code{maxt}.
} 

So for each purchase we have defined an end of coverage (expiry
date). If next purchase is before this, we assume that the amount
purchased is consumed over the period between the two purchases,
otherwise over the period to the end of coverage. So the only difference
between the methods is the determination of the coverage for each
purchase.

Based on this, for each date in the resulting \code{\link{Lexis}} four
exposure variables are computed, see next section.
}

\value{  
A \code{\link{Lexis}} object with the same risk time, states and events
as \code{Lx}. The follow-up for each person has been cut at the purchase
times of each of the drugs, as well as at the expiry times for each drug
coverage. Further, for each drug (i.e. the data frame in the \code{pdat}
list) the name of the \code{pdat} component determines the prefix for
the 4 variables that will be added. Supposing this is \code{AA} for a
given drug, then 4 new variables will be:
\itemize{
 \item \code{AA.ex}: logical; is the person exposed in this interval
 \item \code{AA.tf}: numeric: time since first purchase, same units as
 \code{lex.dur} 
 \item \code{AA.ct}: numeric: cumulative time on the drug, same units
 as \code{lex.dur} 
 \item \code{AA.cd}: numeric: cumulative dose of the drug, same units
 as \code{amt} 
 }
So if \code{pdat} is a list of length 3 with names \code{c("a","b","c")}
the function will add variables
\code{a.ex, a.tf, a.ct, a.cd,
      b.ex, b.tf, b.ct, b.cd,
      c.ex, c.tf, c.ct, c.cd}
}
\author{
Bendix Carstensen, \url{http://bendixcarstensen.com}
}
\seealso{
\code{\link{gen.exp}},
\code{\link{addCov.Lexis}},
\code{\link{cutLexis}},
\code{\link{rcutLexis}},
\code{\link{mcutLexis}}
}
\examples{
# Follow-up of 2 persons
clear()
fu <- data.frame(doe = c(2006, 2008),
                 dox = c(2015, 2018),
                 dob = c(1950, 1951),
                 xst = factor(c("A","D")))
Lx <- Lexis(entry = list(per = doe,
                         age = doe- dob),
             exit = list(per = dox),
      exit.status = xst,
             data = fu)
Lx <- subset(Lx, select = -c(doe, dob, dox, xst))

# split FU in 1 year intervals
Sx <- splitLexis(Lx, "per", breaks = seq(1990, 2020, 1.0))

# drug purchases, one data frame for each drug 
ra <- data.frame(per = c(2007 + runif(12,0,10)),
                 amt = sample(2:4, 12, r = TRUE),
              lex.id = sample(1:2, 12, r = TRUE))
ra <- ra[order(ra$lex.id, ra$per),]

rb <- data.frame(per = c(2009 + runif(10, 0, 10)),
                 amt = sample(round(2:4/3,1), 10, r = TRUE),
              lex.id = sample(1:2, 10, r = TRUE))
rb <- rb[order(rb$lex.id, rb$per),]

# put in a named list
pdat <- list(A = ra, B = rb)
pdat

ex1 <- addDrug.Lexis(Sx, pdat, method = "ext") # default
summary(ex1)
# collapsing some of the smaller intervals with the next
summary(coarse.Lexis(ex1, c(0.2,0.5)))

ex2 <- addDrug.Lexis(Sx, pdat, method = "ext", grace = 0.2)
dos <- addDrug.Lexis(Sx, pdat, method = "dos", dpt = 6)
fix <- addDrug.Lexis(Sx, pdat, method = "fix", maxt = 1)
}
\keyword{survival}
\keyword{manip}

\name{Y.dk}
\alias{Y.dk}
\docType{data}
\title{Population risk time in Denmark}
\description{
  Risk time (person-years) in the Danish population, classified by sex,
  age, period and date of birth in 1-year classes. This corresponds to
  triangles in a Lexis diagram. 
}
\usage{data(Y.dk)}
\format{
  A data frame with 13860 observations on the following 6 variables.
  \describe{
    \item{\code{sex}}{Sex. 1:males, 2:females}
    \item{\code{A}}{One-year age class}
    \item{\code{P}}{Period}
    \item{\code{C}}{Birth cohort}
    \item{\code{Y}}{Person-years}
    \item{\code{upper}}{Indicator of upper triangle in the Lexis diagram}
  }
}
\details{
  The risk time is computed from the population size figures in
  \code{\link{N.dk}}, using the formulae devised in:
   B. Carstensen: Age-period-cohort models for the Lexis diagram.
   Statistics in Medicine, 10; 26(15):3018-45, 2007.
}
\source{
  \url{http://www.statistikbanken.dk/statbank5a/SelectTable/omrade0.asp?SubjectCode=02&PLanguage=1&ShowNews=OFF}
}
\examples{
data(Y.dk)
str(Y.dk)
# Compute mean age, period for the triangles
attach( Y.dk )
age <- A + (1+upper)/3
per <- P + (2-upper)/3
# Plot a Lexis diagram
library( Epi )
Lexis.diagram( age=c(0,10), date=c(1990,2000), coh.grid=TRUE, int=1 )
box()
# Print the person-years for males there
text( per[sex==1], age[sex==1],
      formatC( Y[sex==1]/1000, format="f", digits=1 ) )
}
\keyword{datasets}

\name{clogistic}
\alias{clogistic}
\title{Conditional logistic regression}
\description{
  Estimates a logistic regression model by maximizing the conditional
  likelihood.  The conditional likelihood calculations are exact, and
  scale efficiently to strata with large numbers of cases.
}
\usage{
clogistic(formula, strata, data, subset, na.action, init,
model = TRUE, x = FALSE, y = TRUE, contrasts = NULL,
iter.max=20, eps=1e-6, toler.chol = sqrt(.Machine$double.eps)) 
}
\arguments{
  \item{formula}{Model formula}
  \item{strata}{Factor describing membership of strata for conditioning}
  \item{data}{data frame containing the variables in the formula and
    strata arguments}
  \item{subset}{subset of records to use}
  \item{na.action}{missing value handling}
  \item{init}{initial values}
  \item{model}{
    a logical value indicating whether \emph{model frame}
    should be included as a component of the returned value}
  \item{x,y}{
    logical values indicating whether the response vector and model
    matrix used in the fitting process should be returned as components
    of the returned value.
  }
  \item{contrasts}{
    an optional list. See the \code{contrasts.arg} of
    \code{model.matrix.default}
  }
  \item{iter.max}{maximum number of iterations}
  \item{eps}{
    Convergence tolerance.  Iteration continues until the relative
    change in the conditional log likelihood is less than \code{eps}.
    Must be positive.
  }
  \item{toler.chol}{
    Tolerance used for detection of a singularity during a Cholesky
    decomposition of the variance matrix. This is used to detect
    redundant predictor variables. Must be less than \code{eps}.
  }
}
\value{
  An object of class \code{"clogistic"}. This is a list containing
  the following components:
  \item{coefficients}{
    the estimates of the log-odds ratio parameters.  If the model is
    over-determined there will be missing values in the vector corresponding
    to the redundant columns in the model matrix.
  }
  \item{var}{
    the variance matrix of the coefficients.  Rows and columns corresponding to 
    any missing coefficients are set to zero. 
  }
  \item{loglik}{
    a vector of length 2 containing the log-likelihood with the initial
    values and with the final values of the coefficients. 
  }
  \item{iter}{
    number of iterations used. 
  }
  \item{n}{
    number of observations used. Observations may be dropped
    either because they are missing, or because they belong to a
    homogeneous stratum. For more details on which observations were
    used, see \code{informative} below.
  }
  \item{informative}{
    if \code{model=TRUE}, a logical vector of length equal to the number
    of rows in the model frame.  This indicates whether an observation
    is informative. Strata that are homogeneous with respect to either
    the outcome variable or the predictor variables are uninformative,
    in the sense that they can be removed without modifying the
    estimates or standard errors. If \code{model=FALSE}, this is NULL.
  }
  
  The output will also contain the following, for documentation see the
  \code{glm} object: \code{terms}, \code{formula},
  \code{call},  \code{contrasts}, \code{xlevels}, and, optionally,
  \code{x}, \code{y}, and/or \code{frame}. 
}
\examples{
  data(bdendo)
  clogistic(d ~ cest + dur, strata=set, data=bdendo)
}
\author{Martyn Plummer}
\seealso{\code{\link{glm}}}
\keyword{models}

\name{timeScales}
\alias{timeScales}
\alias{timeSince}
\alias{tsNA20}
\alias{lexVars}
\title{The time scales of a Lexis object}
\description{
  Functions to get the names and type of the time scales of a
  \code{Lexis} object. 
}
\usage{
timeScales(x)
timeSince(x)
tsNA20(x, all.scales = FALSE)
lexVars(x)
}
\arguments{
  \item{x}{an object of class \code{Lexis}.}
  \item{all.scales}{Should NAs in all timescales be replaced by 0? If
    \code{FALSE} (the default) only timescales defined as time since
    entry to a state get \code{NA}s replaced by 0s}
}
\value{
  \code{timeScales} returns a character vector containing the names of
  the variables in \code{x} that represent the time scales. Extracted
  from the \code{time.scales} attribute of the object.
  
  \code{timeSince} returns a named character vector, the names being the
  names of the timescales and the content being the names of the states
  to which the corresponding timescale is defined as time since
  entry. For those time scales that are not defined as such an empty
  string is used. Hence, if none of the timescales are defined as time
  since entry to a state \code{timeSince} will return a vector of empty
  strings.

  \code{lexVars} returns a character vector containing the names if the
  \code{Lexsi}-specific variables of \code{x}; those starting with
  \code{lex.} and the names of the timescales. 
}
\author{Martyn Plummer, Bendix Carstensen}
\seealso{\code{\link{Lexis}}, \code{\link{splitLexis}}}
\keyword{attribute}

\name{BrCa}
\alias{BrCa}
\docType{data}
\title{Clinical status,
relapse, metastasis and death in 2982 women with breast cancer.
}
\description{
This dataset is a transformation of the example dataset used by Crowther
and Lambert in their multistate paper.}
\usage{data(BrCa)}
\format{
  A data frame with 2982 observations on the following 17 variables:
  \describe{
    \item{\code{pid}}{Person-id; numeric}
    \item{\code{year}}{Calendar year of diagnosis}
    \item{\code{age}}{Age at diagnosis}
    \item{\code{meno}}{Menopausal status; a factor with levels \code{pre} \code{post}}
    \item{\code{size}}{Tumour size; a factor with levels \code{<=20 mm} \code{>20-50 mm} \code{>50 mm}}
    \item{\code{grade}}{Tumour grade; a factor with levels \code{2} \code{3}}
    \item{\code{nodes}}{Number of positive lymph nodes, a numeric vector}
    \item{\code{pr}}{Progesteron receptor level}
    \item{\code{pr.tr}}{Transformed progesteron level}
    \item{\code{er}}{Estrogen receptor level}
    \item{\code{hormon}}{Hormon therapy at diagnosis; a factor with levels \code{no} \code{yes}}
    \item{\code{chemo}}{Chemotherapy treatment; a factor with levels \code{no} \code{yes}}
    \item{\code{tor}}{Time of relapse, years since diagnosis}
    \item{\code{tom}}{Time of metastasis, years since diagnosis}
    \item{\code{tod}}{Time of death, years since diagnosis}
    \item{\code{tox}}{Time of exit from study, years since diagnosis}
    \item{\code{xst}}{Vital status at exit; a factor with levels \code{Alive} \code{Dead}}
  }
}
\details{
The dataset has been modified to contain the times (since diagnosis) of the events of
interest, to comply with the usual structure of data.
}
\source{
  The original data were extracted from:
  \url{http://fmwww.bc.edu/repec/bocode/m/multistate_example.dta}, this
  is modified representation of the same amount of information.
}
\references{
  The data were used as example in the paper by Crowther and
  Lambert: Parametric multistate survival models: Flexible
  modelling allowing transition-specific distributions with application
  to estimating clinically useful measures of effect differences; Stat
  Med 36 (29), pp 4719-4742, 2017. (No, it is not the paper, just the
  title.) 

  A parallel analysis using the \code{\link{Lexis}} machinery is available
  as: \url{http://bendixcarstensen.com/AdvCoh/papers/bcMS.pdf}
		}
\examples{
data(BrCa)
}
\keyword{datasets}

\name{apc.frame}
\alias{apc.frame}
\title{
  Produce an empty frame for display of parameter-estimates from
  Age-Period-Cohort-models.
}
\description{

  A plot is generated where both the age-scale and the cohort/period
  scale is on the x-axis. The left vertical axis will be a logarithmic
  rate scale referring to age-effects and the right a logarithmic
  rate-ratio scale of the same relative extent as the left referring to
  the cohort and period effects (rate ratios).

  Only an empty plot frame is generated. Curves or points must be added
  with \code{points}, \code{lines} or the special utility function
  \code{\link{apc.lines}}.
}
\usage{
  apc.frame( a.lab,
            cp.lab,
             r.lab,
            rr.lab = r.lab / rr.ref,
            rr.ref = r.lab[length(r.lab)/2],
             a.tic = a.lab,
            cp.tic = cp.lab,
             r.tic = r.lab,
            rr.tic = r.tic / rr.ref,
           tic.fac = 1.3,
             a.txt = "Age",
            cp.txt = "Calendar time",
             r.txt = "Rate per 100,000 person-years",
            rr.txt = "Rate ratio",
          ref.line = TRUE,
               gap = diff(range(c(a.lab, a.tic)))/10,
          col.grid = gray(0.85),
             sides = c(1,2,4) )
}
\arguments{
  \item{a.lab}{Numerical vector of labels for the age-axis.}
  \item{cp.lab}{Numerical vector of labels for the cohort-period axis.}
  \item{r.lab}{Numerical vector of labels for the rate-axis (left vertical)}
  \item{rr.lab}{Numerical vector of labels for the RR-axis (right vertical)}
  \item{rr.ref}{At what level of the rate scale is the RR=1 to be.}
  \item{a.tic}{Location of additional tick marks on the age-scale}
  \item{cp.tic}{Location of additional tick marks on the cohort-period-scale}
  \item{r.tic}{Location of additional tick marks on the rate-scale}
  \item{rr.tic}{Location of additional tick marks on the RR-axis.}
  \item{tic.fac}{Factor with which to diminish intermediate tick marks}
  \item{a.txt}{Text for the age-axis (left part of horizontal axis).}
  \item{cp.txt}{Text for the cohort/period axis (right part of
    horizontal axis).}
  \item{r.txt}{Text for the rate axis (left vertical axis).}
  \item{rr.txt}{Text for the rate-ratio axis (right vertical axis)}
  \item{ref.line}{Logical. Should a reference line at RR=1 be drawn at the
    calendar time part of the plot?}
  \item{gap}{Gap between the age-scale and the cohort-period scale}
  \item{col.grid}{Colour of the grid put in the plot.}
  \item{sides}{Numerical vector indicating on which sides axes should
    be drawn and annotated. This option is aimed for multi-panel
    displays where axes only are put on the outer plots.}
}
\details{
  The function produces an empty plot frame for display of results
  from an age-period-cohort model, with age-specific rates in the left
  side of the frame and cohort and period rate-ratio parameters in the
  right side of the frame. There is a gap of \code{gap} between the
  age-axis and the calendar time axis, vertical grid lines at
  \code{c(a.lab,a.tic,cp.lab,cp.tic)}, and horizontal grid lines at
  \code{c(r.lab,r.tic)}.

  The function returns a numerical vector of
  length 2, with names \code{c("cp.offset","RR.fac")}. The y-axis for
  the plot will be a rate scale for the age-effects, and the x-axis will
  be the age-scale. The cohort and period effects are plotted by
  subtracting the first element (named \code{"cp.offset"}) of the returned result
  form the cohort/period, and multiplying  the rate-ratios by the second
  element of the returned result (named \code{"RR.fac"}).
}
\value{
  A numerical vector of length two, with names
  \code{c("cp.offset","RR.fac")}. The first is the offset for the cohort
  period-axis, the second the multiplication factor for the rate-ratio
  scale.

  Side-effect: A plot with axes and grid lines but no points or curves.
  Moreover, the option \code{apc.frame.par} is given the value
  \code{c("cp.offset","RR.fac")}, which is recognized by \code{\link{apc.plot}}
  and \code{\link{apc.lines}}.
}
\references{
  B. Carstensen: Age-Period-Cohort models for the Lexis
  diagram. Statistics in Medicine, 26: 3018-3045, 2007.
}
\author{
  Bendix Carstensen, Steno Diabetes Center,
  \url{http://bendixcarstensen.com}
}
\examples{
par( mar=c(4,4,1,4) )
res <-
apc.frame( a.lab=seq(30,90,20), cp.lab=seq(1880,2000,30), r.lab=c(1,2,5,10,20,50),
           a.tic=seq(30,90,10), cp.tic=seq(1880,2000,10), r.tic=c(1:10,1:5*10),
           gap=27 )
res
# What are the axes actually?
par(c("usr","xlog","ylog"))
# How to plot in the age-part: a point at (50,10)
points( 50, 10, pch=16, cex=2, col="blue" )
# How to plot in the cohort-period-part: a point at (1960,0.3)
points( 1960-res[1], 0.3*res[2], pch=16, cex=2, col="red" )
# or referring to the period-cohort part of the plot 
pc.points( 1960, 0.3, pch=16, cex=1, col="green" )
}
\seealso{
  \code{\link{apc.lines},\link{apc.fit}}
}
\keyword{hplot}

\name{ewrates}
\alias{ewrates}
\docType{data}
\title{Rates of lung and nasal cancer mortality, and total mortality.}
\description{
England and Wales mortality rates from lung cancer, nasal cancer,
and all causes 1936 - 1980. The 1936 rates are repeated as 1931 rates in
order to accommodate follow up for the \code{\link{nickel}} study.
}
\usage{data(ewrates)}
\format{
  A data frame with 150 observations on the following 5 variables:
\tabular{rl}{
\code{id}: \tab Subject identifier (numeric) \cr
    \code{year}  \tab Calendar period, 1931: 1931--35, 1936: 1936--40,
    \ldots \cr
    \code{age}   \tab Age class: 10: 10--14, 15:15--19, \ldots  \cr
    \code{lung}  \tab Lung cancer mortality rate per 1,000,000 py. \cr
    \code{nasal} \tab Nasal cancer mortality rate per 1,000,000 py. \cr
    \code{other} \tab All cause mortality rate per 1,000,000 py.
}
}
\source{
From Breslow and Day, Vol II, Appendix IX.
}
\examples{
data(ewrates)
str(ewrates)
}
\keyword{datasets}

\name{fit.baseline}
\alias{fit.baseline}
\title{
  Fit a piecewise contsnt intesity model for interval censored data.
  }
\description{
  Utility function
  
  Fits a binomial model with logaritmic link, with \code{y} as outcome
  and covariates in \code{rates.frame} to estimate rates in the
  inttervals between \code{breaks}.
  }
\usage{
fit.baseline( y, rates.frame, start )
}
\arguments{
  \item{y}{Binary vector of outcomes}
  \item{rates.frame}{Dataframe expanded from the original data by
    \code{\link{expand.data}}} 
  \item{start}{Starting values for the rate parameters. If not supplied,
       then starting values are generated.}
}
\value{
  A \code{\link{glm}} object, with binomial error and logaritmic link.
  }
\author{
  Martyn Plummer, \email{martyn.plummer@r-project.org}
  }
\seealso{
  \code{\link{fit.add}}
  \code{\link{fit.mult}}
  }
\keyword{ models }
\keyword{ regression }
\keyword{ survival }

\name{subset.Lexis}
\alias{subset.Lexis}
\alias{[.Lexis}
\alias{subset.stacked.Lexis}
\title{Subsetting Lexis (and stacked.Lexis) objects}
\description{
  Return subsets of Lexis objects which meet conditions
}
\usage{
\method{subset}{Lexis}(x, ...)
\method{[}{Lexis}(x, ...)
\method{subset}{stacked.Lexis}(x, ...)
}

\arguments{
  \item{x}{an object of class \code{Lexis}}
  \item{\dots}{additional arguments to be passed to
    \code{subset.data.frame}. This will normally be some logical
    expression selecting a subset of the rows. For details see
    \code{\link{subset.data.frame}}.}
  }
\details{
  The subset method for \code{Lexis} objects works exactly as the method
  for data frames. So does the "[" method. The special methods are needed in
  order to propagate the Lexis-specific attributes.

  The method for \code{stacked.Lexis} objects also shrinks the set of
  levels for \code{lex.Cst} and \code{lex.Xst} to those actually
  occurring in the resulting data frame.
  }
\value{
  A \code{Lexis} object with selected rows and columns.
}
\author{Martyn Plummer}
\seealso{\code{\link{Lexis}},
         \code{\link{merge.Lexis}},
         \code{\link{bootLexis}}}
\keyword{manip}

\name{erl}
\alias{surv1}
\alias{surv2}
\alias{erl1}
\alias{erl}
\alias{yll}
\title{Compute survival functions from rates and expected residual
  lifetime in an illness-death model as well as years of life lost to disease.
}
\description{
  These functions compute survival functions from a set of mortality and
  disease incidence rates in an illness-death model. Expected residual
  life time can be computed under various scenarios by the \code{erl}
  function, and areas between survival functions can be computed under
  various scenarios by the \code{yll} function. Rates are assumed
  supplied for equidistant intervals of length \code{int}.
  }
\usage{
  surv1( int, mu ,                age.in = 0, A = NULL )
   erl1( int, mu ,                age.in = 0 ) 
  surv2( int, muW, muD, lam,      age.in = 0, A = NULL )
    erl( int, muW, muD, lam=NULL, age.in = 0, A = NULL,
         immune = is.null(lam), yll=TRUE, note=TRUE )
    yll( int, muW, muD, lam=NULL, age.in = 0, A = NULL,
         immune = is.null(lam), note=TRUE )
  }
\arguments{
  \item{int}{
  Scalar. Length of intervals that rates refer to.    
  }
  \item{mu}{
  Numeric vector of mortality rates at midpoints of intervals of length \code{int}
  }
  \item{muW}{
  Numeric vector of mortality rates among persons in the "Well" state  at
  midpoints of intervals of length \code{int}. Left endpoint of first
  interval is \code{age.in}. 
  }
  \item{muD}{
  Numeric vector of mortality rates among persons in the "Diseased" state
  at midpoints of intervals of length \code{int}.  Left endpoint of first
  interval is \code{age.in}. 
  }
  \item{lam}{
  Numeric vector of disease incidence rates among persons in the "Well" state
  at midpoints of intervals of length \code{int}. Left endpoint of first
  interval is \code{age.in}. 
  }
  \item{age.in}{
  Scalar indicating the age at the left endpoint of the first interval. 
  }
  \item{A}{
  Numeric vector of conditioning ages for calculation of survival
  functions.   
  }
  \item{immune}{
  Logical. Should the years of life lost to the disease be computed
  using assumptions that non-diseased individuals are immune to the
  disease (\code{lam}=0) and that their mortality is yet still \code{muW}.
  }
  \item{note}{
  Logical. Should a warning of silly assumptions be printed?
  }
  \item{yll}{
  Logical. Should years of life lost be included in the result?
  }
}
\details{
  The mortality rates given are supposed to refer to the ages
  \code{age.in+(i-1/2)*int}, \code{i=1,2,3,...}.
  
  The units in which \code{int} is given must correspond to the units in
  which the rates \code{mu}, \code{muW}, \code{muD} and \code{lam} are
  given. Thus if \code{int} is given in years, the rates must be given
  in the unit of events per year.

  The ages in which the survival curves are computed are from
  \code{age.in} and then at the end of \code{length(muW)}
  (\code{length(mu)}) intervals each of length \code{int}.
  
  The \code{age.in} argument is merely a device to account for rates
  only available from a given age. It has two effects, one is that
  labeling of the interval endpoint is offset by this quantity, thus
  starting at \code{age.in}, and the other that the conditioning ages
  given in the argument \code{A} will refer to the ages defined by this.
  
  The \code{immune} argument is \code{FALSE} whenever the disease
  incidence rates are supplied. If set to \code{TRUE}, the years of life
  lost is computed under the assumption that individuals without the
  disease at a given age are immune to the disease in the sense that the
  disease incidence rate is 0, so transitions to the diseased state
  (with presumably higher mortality rates) are assumed not to
  occur. This is a slightly peculiar assumption (but presumably the most
  used in the epidemiological literature) and the resulting object is
  therefore given an attribute, \code{NOTE}, that point this out.

  If however \code{muW} is the total mortality in the population
  (including the diseased) the result is a good approximation to the
  correct YLL.
  
  The default of the \code{surv2} function is to take the possibility of
  disease into account.}

\value{\code{surv1} and \code{surv2} return a matrix whose first column
  is the ages at the ends of the 
  intervals, thus with \code{length(mu)+1} rows. The following columns
  are the survival functions (since \code{age.in}), and conditional on
  survival till ages as indicated in \code{A}, thus a matrix with
  \code{length(A)+2} columns. Columns are labeled with the actual
  conditioning ages; if \code{A} contains values that are not among the
  endpoints of the intervals used, the nearest smaller interval border
  is used as conditioning age, and columns are named accordingly.

  \code{surv1} returns the survival function for a simple model with one
  type of death, occurring at intensity \code{mu}.
  
  \code{surv2} returns the survival function for a person in the "Well"
  state of an illness-death model, taking into account that the person
  may move to the "Diseased" state, thus requiring all three transition
  rates to be specified. The conditional survival functions are
  conditional on being in the "Well" state at ages given in \code{A}.

  \code{erl1} returns a three column matrix with columns \code{age},
  \code{surv} (survival function) and \code{erl} (expected residual life
  time) with \code{length(mu)+1} rows. 
  
  \code{erl} returns a two column matrix, columns labeled "Well" and
  "Dis", and with row-labels \code{A}. The entries are the expected
  residual life times given survival to \code{A}. If \code{yll=TRUE} the
  difference between the columns is added as a
  third column, labeled "YLL".
  }
\author{Bendix Carstensen, \email{b@bxc.dk}
  }
\seealso{
\code{\link{ci.cum}}
}
\examples{
library( Epi )
data( DMlate )
# Naive Lexis object
Lx <- Lexis( entry = list( age = dodm-dobth ),
              exit = list( age = dox -dobth ),
       exit.status = factor( !is.na(dodth), labels=c("DM","Dead") ),
              data = DMlate )
# Cut follow-up at insulin inception
Lc <- cutLexis( Lx, cut = Lx$doins-Lx$dob,
              new.state = "DM/ins",
       precursor.states = "DM" )
summary( Lc )
# Split in small age intervals
Sc <- splitLexis( Lc, breaks=seq(0,120,2) )
summary( Sc )

# Overview of object
boxes( Sc, boxpos=TRUE, show.BE=TRUE, scale.R=100 )

# Knots for splines
a.kn <- 2:9*10

# Mortality among DM
mW <- glm( lex.Xst=="Dead" ~ Ns( age, knots=a.kn ),
           offset = log(lex.dur),
           family = poisson,
             data = subset(Sc,lex.Cst=="DM") )

# Mortality among insulin treated
mI <- update( mW, data = subset(Sc,lex.Cst=="DM/ins") )

# Total motality
mT <- update( mW, data = Sc )

# Incidence of insulin inception
lI <- update( mW, lex.Xst=="DM/ins" ~ . )

# From these we can now derive the fitted rates in intervals of 1 year's
# length. In real applications you would use much smaller interval like
# 1 month:
# int <- 1/12 
int <- 1

# Prediction frame to return rates in units of cases per 1 year
# - we start at age 40 since rates of insulin inception are largely
# indeterminate before age 40
nd <- data.frame( age = seq( 40+int, 110, int ) - int/2,
              lex.dur = 1 )
muW <- predict( mW, newdata = nd, type = "response" )
muD <- predict( mI, newdata = nd, type = "response" )
lam <- predict( lI, newdata = nd, type = "response" )

# Compute the survival function, and the conditional from ages 50 resp. 70
s1 <- surv1( int, muD, age.in=40, A=c(50,70) )
round( s1, 3 )

s2 <- surv2( int, muW, muD, lam, age.in=40, A=c(50,70) )
round( s2, 3 )

# How much is YLL overrated by ignoring insulin incidence?
round( YLL <- cbind(
yll( int, muW, muD, lam, A = 41:90, age.in = 40 ),
yll( int, muW, muD, lam, A = 41:90, age.in = 40, immune=TRUE ) ), 2 )[seq(1,51,10),]

par( mar=c(3,3,1,1), mgp=c(3,1,0)/1.6, bty="n", las=1 )
matplot( 40:90, YLL,
         type="l", lty=1, lwd=3,
         ylim=c(0,10), yaxs="i", xlab="Age" )
}
\keyword{survival}

\name{AaJ}
\alias{AaJ}
\alias{AaJ.Lexis}
\title{The Aalen-Johansen estimator of state probabilities from a
  multistate \code{Lexis} object.
}
\description{
The Aalen-Johansen estimator is computed on the basis of a
\code{\link{Lexis}} multistate object along a given time scale. The
function is a wrapper for the \code{\link[survival]{survfit}}.
}
\usage{
\method{AaJ}{Lexis}(Lx,
               formula = ~ 1,
             timeScale = 1, \dots)
}
\arguments{
  \item{Lx}{A \code{Lexis} object. The starting state must be the first
    among \code{levels(Lx)}.
}
  \item{formula}{A one-sided formula passed on to \code{survfit}.
}
  \item{timeScale}{Character or integer, selecting one of the timescales
    of the \code{Lexis} object.
}
  \item{\dots}{Arguments passed on. Ignored.
}
}
\value{An object of class \code{survfitms} --- see
  \code{\link[survival]{survfit}}. 
}
\author{
Bendix Carstensen, \url{http://bendixcarstensen.com}
}
\seealso{
\code{\link[survival]{survfit}}
\code{\link{ci.Crisk}}
}
\examples{
data(DMlate)
str(DMlate)
dml <- Lexis(entry = list(Per = dodm,
                          Age = dodm-dobth,
                        DMdur = 0 ),
              exit = list(Per = dox),
       exit.status = factor(!is.na(dodth),
                            labels = c("DM","Dead")),
              data = DMlate )

# Cut the follow-up at insulin start
dmi <- cutLexis(dml,
                cut = dml$doins,
          new.state = "Ins",
        split.state = TRUE)
summary( dmi )
ms <- AaJ.Lexis(dmi, timeScale = "DMdur")
class(ms)
ms$states
head(ms$pstate)
}
\keyword{models}

\name{detrend}
\alias{detrend}
\alias{decurve}
\title{ Projection of a model matrix on the orthogonal
  complement of a trend or curvature.}
\description{
  The columns of a model matrix \code{M} is projected on the
  orthogonal complement to the matrix \code{(1,t)},
  resp. \code{(1,t,t^2)}.
  
  Orthogonality is w.r.t. an inner product defined by the positive
  definite matrix matrix \code{diag(weight)}. Non-diagonal matrices
  defining the inner product is not supported.
}
\usage{
  detrend( M, t, weight = rep(1, nrow(M)) )
  decurve( M, t, weight = rep(1, nrow(M)) )
}
\arguments{
  \item{M}{A model matrix.}
  \item{t}{The trend defining a subspace. A numerical vector of length
    \code{nrow(M)}.}
  \item{weight}{ Weights defining the inner product of vectors \code{x}
    and \code{y} as \code{sum(x*w*y)}.
    A numerical vector of length \code{nrow(M)}, defaults to a vector of
    \code{1}s. Must be all non-negative.}
}
\details{
  The functions are intended to be used in construction of particular
  parametrizations of age-period-cohort models. 
}
\value{
  \code{detrend} returns full-rank matrix with columns orthogonal to
  \code{(1,t)};
  \code{decurve} returns full-rank matrix with columns orthogonal to
  \code{(1,t,t^2)}.
}
\author{
  Bendix Carstensen, Steno Diabetes Center Copenhagen,
  \url{http://bendixcarstensen.com}, with essential help from Peter Dalgaard.
}
\seealso{ \code{\link{projection.ip}} }
\keyword{array}

\name{list.lex}
\alias{list.lex}
\title{Compact printing of Lexis object
}
\description{Often the number of digits after the decimal point ant not
  the sugnificant digits is relevant in printing timescales.
}
\usage{
list.lex( Lx, dig, sh="_" )
}
\arguments{
  \item{Lx}{A \code{\link{Lexis}} object.}
  \item{dig}{Scalar. How many digits should be printed after the decimal
    separator for the numerical variables.}
  \item{sh}{Replacement string for \code{lex.} in names when printing.}
}
\details{The function is designed to make a compact listing of a Lexis
  obejct. Variables are ordered with \code{lex.id}, time scales,
  \code{lex.dur} and state variables first. In order to avoid save
  between columns the \code{lex.} prefix is replaced by an underscore to
  shorten the names.
}
\value{NULL, the functions is used for its compact printing of a Lexis object. 
}
\author{Bendix Carstensen
}
\examples{
# A small bogus cohort
xcoh <- structure( list( id = c("A", "B", "C"),
                      birth = c("14/07/1952", "01/04/1954", "10/06/1987"),
                      entry = c("04/08/1965", "08/09/1972", "23/12/1991"),
                       exit = c("27/06/1997", "23/05/1995", "24/07/1998"),
                       fail = c(1, 0, 1) ),
                     .Names = c("id", "birth", "entry", "exit", "fail"),
                  row.names = c("1", "2", "3"),
                      class = "data.frame" )

# Convert the character dates into numerical variables (fractional years)
xcoh <- cal.yr( xcoh, format="\%d/\%m/\%Y", wh=2:4 )
# Make a Lexis obejct
Lcoh <- Lexis( entry = list( per=entry ),
                exit = list( per=exit,
                             age=exit-birth ),
         exit.status = fail,
                data = xcoh )
Lcoh
list.lex( Lcoh, 2 )
}
\keyword{manip}

\name{simLexis}
\alias{simLexis}
\alias{nState}
\alias{pState}
\alias{plot.pState}
\alias{lines.pState}
\title{Simulate a Lexis object representing follow-up in a multistate
  model.}

\description{Based on a (pre-)\code{Lexis} object representing persons
  at given states and times, and full specification of transition
  intensities between states in the form of models for the transition
  rates, this function simulates transition times and -types for persons
  and returns a \code{Lexis} object representing the simulated
  cohort. The simulation scheme accommodates multiple timescales,
  including time since entry into an intermediate state, and accepts
  fitted Poisson models, Cox-models or just a function as specification
  of rates.}

\usage{
simLexis( Tr, init,
           N = 1,
      lex.id,
     t.range = 20,
       n.int = 101,
    time.pts = seq(0,t.range,length.out=n.int) )
nState( obj, at, from, time.scale = 1 )
pState( nSt, perm = 1:ncol(nSt) )
\method{plot}{pState}( x,
                     col = rainbow(ncol(x)),
                  border = "transparent",
                    xlab = "Time",
                    ylim = 0:1,
                    ylab = "Probability", ... )
\method{lines}{pState}( x,
                      col = rainbow(ncol(x)),
                   border = "transparent", ... )
   }
\arguments{
  \item{Tr}{A named list of named lists. The names of the list are names
    of the transient states in the model. Each list element is again a
    named list. The names of the elements of this inner list are the
    names of the states reachable from the state with name equal to the
    list. Elements of the intter lists represent transitions. See details.}
  \item{init}{A (pre-)\code{\link{Lexis}} object representing the initial
    state of the persons whose trajectories through the multiple states
    we want to simulate. Must have attributes "time.scales" and "time.since" --- see
    details. Duplicate values of \code{lex.id} are not sensible and not
    accepted.}
  \item{N}{Numeric. How many persons should be simulated. \code{N}
    persons with covariate configuration of each row in \code{init}
    will be simulated. Either a scalar or a vector of length \code{nrow(init)}.}
  \item{lex.id}{Vector of ids of the simulated persons. Useful when
    simulating in chunks.}
  \item{t.range}{Numerical scalar. The range of time over which to
    compute the cumulative rates when simulating. Simulted times
    beyond this will result in an obervation censored at \code{t.range}
    after entry.}
  \item{n.int}{Number of intervals to use when computing (cumulative) rates.}
  \item{time.pts}{Numerical vector of times since start. Cumulative
    rates for transitions are computed at these times after stater and
    entry state. Simulation is only done till time \code{max(time.pts)}
    after start, where persons are censored. Must start with 0.}
  \item{obj}{A \code{Lexis} object.}
  \item{from}{The point on the time scale \code{time.scale} from which
    we start counting.}
  \item{time.scale}{The timescale to which \code{from} refer.}
  \item{at}{Time points (after \code{from}) where the number of persons
    in each state is to be computed.}
  \item{nSt}{A table obtained by \code{nState}.}
  \item{perm}{A permutation of columns used before cumulating row-wise
    and taking percentages.}
  \item{x}{An object of class \code{pState}, e.g. created by \code{pState}.}
  \item{col}{Colors for filling the areas between curves.}
  \item{border}{Colors for outline of the areas between curves.}
  \item{xlab}{Label on x-axis}
  \item{ylim}{Limits on y-axis}
  \item{ylab}{Label on y-axis}
  \item{...}{Further arguments passed on to \code{plot}.}
}
\details{The simulation command \code{simLexis} is not defined as a
  method for \code{Lexis} objects, because the input is not a
  \code{Lexis} object, the \code{Lexis}-like object is merely
  representing a prevalent population and a specification of which
  variables that are timescales. The variables \code{lex.dur} and
  \code{lex.Xst} are ignored (and overwritten) if present. The core
  input is the list \code{Tr} giving the transitions.

  The components of \code{Tr} represents the transition intensities
  between states. The transition from state \code{A} to \code{B}, say,
  is assumed stored in \code{Tr$A$B}. Thus names of the elements of
  \code{Tr} are names of transient states, and the names of the elements
  of each these are the names of states reachable from the corresponding
  transient state.

  The transition intensities are assumed modelled by either a glm with
  Poisson family or a Cox-model. In both cases the timescale(s) in the
  model must be using the names fo the timescales in a Lexis object
  representng the follow-up in a cohort, and the risk time must be taken
  from the variable \code{lex.dur} --- see the example.

  Alternatively, an element in \code{Tr} could be a function
  that from a data frame produces transition rates, or specifically
  cumulative transition rates over intervals of length \code{lex.dur}.

  The pre-\code{Lexis} object \code{init} must contain values of all
  variables used in any of the objects in \code{Tr}, as well as all
  timescales - even those not used in the models. Moreover, the
  attributes \code{time.scales} and \code{time.since} must be
  present. The attribute \code{time.since} is a character vector of the
  same length as \code{time.scales} and an element has value \code{"A"}
  if the corresponding time scale is defined as
  "time since entry into state \code{A}", otherwise the value is
  \code{""}. If not present it will be set to a vector of \code{""}s,
  which is only OK if no time scales are defined as time since entry to
  a state.

  Note that the variables pre-\code{Lexis} object \code{init} must have
  the same mode and class as in the dataset used for fitting the models
  --- hence the indexing of rows by brackets in the assignment of values used in
  the example below - this way the variables have their attributes
  preserved; using \code{init[,"var"] <-} or \code{init$var <-} replaces
  the variable, whereas  \code{init[1:4,"var"] <-} or
  \code{init$var[1:4] <-} replaces values only and prevents you from
  entering non-existing factor levels etc.
  
  The function \code{\link{Lexis}} automatically generates an attribute
  \code{time.since}, and \code{\link{cutLexis}} updates it when new time
  scales are defined. Hence, the simplest way of defining a initial
  pre-\code{Lexis} object representing a current state of a (set of) persons
  to be followed through a multistate model is to take \code{NULL} rows
  of an existing Lexis object (normally the one used for estimation),
  and so ensuring that all relevant attributes and state levels are
  properly defined. See the example code.

  The prevalence function \code{nState} computes the distribution of
  individuals in different states at prespecified times. Only sensible
  for a simulated \code{Lexis} object. The function \code{pState} takes
  a matrix as output by \code{nState} and computes the row-wise
  cumulative probabilities across states, and leaves an object of class
  \code{pState}, suitable for plotting.
}

\value{\code{simLexis} returns a \code{\link{Lexis}} object representing
  the experience of a population starting as \code{init} followed
  through the states according to the transitions in \code{Tr}.

  The function \code{nState} returns a table of persons classified by
  states at each of the times in \code{at}. Note that this function can
  easily produce meaningless results, for example if applied to a
  \code{Lexis} object not created by simulation. If you apply it to a
  \code{Lexis} object generated by \code{simLexis}, you must make sure
  that you start (\code{from}) the point where you started the
  simulation on the correct timescale, and you will get funny results if
  you try to tabulate beyond the censoring time for the simulation.
  The resulting object has class \code{"table"}.

  The result from using \code{pState} on the result from \code{nState}
  has class \code{c("pState","matrix")}.
}

\author{Bendix Carstensen, \url{http://bendixcarstensen.com}.}
\seealso{
  \code{\link{Lexis}},
  \code{\link{cutLexis}},
  \code{\link{splitLexis}}
}
\examples{
data(DMlate)
dml <- Lexis( entry = list(Per=dodm, Age=dodm-dobth, DMdur=0 ),
               exit = list(Per=dox),
        exit.status = factor(!is.na(dodth),labels=c("DM","Dead")),
               data = DMlate[runif(nrow(DMlate))<0.1,] )
# Split follow-up at insulin, introduce a new timescale,
# and split non-precursor states
dmi <- cutLexis( dml, cut = dml$doins,
                      pre = "DM",
                new.state = "Ins",
                new.scale = "t.Ins",
             split.states = TRUE )
# Split the follow in 1-year intervals for modelling
Si <- splitLexis( dmi, 0:30/2, "DMdur" )
# Define knots
nk <- 4
( ai.kn <- with( subset(Si,lex.Xst=="Ins"),
                 quantile( Age+lex.dur, probs=(1:nk-0.5)/nk ) ) )
( ad.kn <- with( subset(Si,lex.Xst=="Dead"),
                 quantile( Age+lex.dur, probs=(1:nk-0.5)/nk ) ) )
( di.kn <- with( subset(Si,lex.Xst=="Ins"),
                 quantile( DMdur+lex.dur, probs=(1:nk-0.5)/nk ) ) )
( dd.kn <- with( subset(Si,lex.Xst=="Dead"),
                 quantile( DMdur+lex.dur, probs=(1:nk-0.5)/nk ) ) )
( td.kn <- with( subset(Si,lex.Xst=="Dead(Ins)"),
                 quantile( t.Ins+lex.dur, probs=(1:nk-0.5)/nk ) ) )

# Fit Poisson models to transition rates
library( splines )
DM.Ins <- glm( (lex.Xst=="Ins") ~ Ns( Age  , knots=ai.kn ) +
                                  Ns( DMdur, knots=di.kn ) +
                                  I(Per-2000) + sex,
               family=poisson, offset=log(lex.dur),
               data = subset(Si,lex.Cst=="DM") )
DM.Dead <- glm( (lex.Xst=="Dead") ~ Ns( Age  , knots=ad.kn ) +
                                    Ns( DMdur, knots=dd.kn ) +
                                    I(Per-2000) + sex,
               family=poisson, offset=log(lex.dur),
               data = subset(Si,lex.Cst=="DM") )
Ins.Dead <- glm( (lex.Xst=="Dead(Ins)") ~ Ns( Age  , knots=ad.kn ) +
                                          Ns( DMdur, knots=dd.kn ) +
                                          Ns( t.Ins, knots=td.kn ) +
                                          I(Per-2000) + sex,
               family=poisson, offset=log(lex.dur),
               data = subset(Si,lex.Cst=="Ins") )

# Stuff the models into an object representing the transitions
Tr <- list( "DM" = list( "Ins"       = DM.Ins,
                         "Dead"      = DM.Dead  ),
           "Ins" = list( "Dead(Ins)" = Ins.Dead ) )
lapply( Tr, names )

# Define an initial object - note the subsetting that ensures that
# all attributes are carried over
ini <- Si[1,1:9][-1,]
ini[1:2,"lex.Cst"] <- "DM"
ini[1:2,"Per"] <- 1995
ini[1:2,"Age"] <- 60
ini[1:2,"DMdur"] <- 5
ini[1:2,"sex"] <- c("M","F")
str(ini)

# Simulate 200 of each sex using the estimated models in Tr
simL <- simLexis( Tr, ini, time.pts=seq(0,11,0.5), N=200 )
summary( simL )

# Find the number of persons in each state at a set of times.
# Note that the times are shirter than the time-span simulated.
nSt <- nState( subset(simL,sex=="M"),
               at=seq(0,10,0.1), from=1995, time.scale="Per" )
nSt

# Show the cumulative prevalences in a different order than that of the
# state-level ordering and plot them using all defaults
pp <- pState( nSt, perm=c(1,2,4,3) )
head( pp )
plot( pp )

# A more useful set-up of the graph
clr <- c("orange2","forestgreen")
par( las=1 )
plot( pp, col=clr[c(2,1,1,2)] )
lines( as.numeric(rownames(pp)), pp[,2], lwd=2 )
mtext( "60 year old male, diagnosed 1995", side=3, line=2.5, adj=0 )
mtext( "Survival curve", side=3, line=1.5, adj=0 )
mtext( "DM, no insulin   DM, Insulin", side=3, line=0.5, adj=0, col=clr[1] )
mtext( "DM, no insulin", side=3, line=0.5, adj=0, col=clr[2] )
axis( side=4 )

# Using a Cox-model for the mortality rates assuming the two mortality
# rates to be proportional:
# When we fit a Cox-model, lex.dur must be used in the Surv() function,
# and the I() constrction must be used when specifying intermediate
# states as covariates, since factors with levels not present in the
# data will create NAs in the parameter vector returned by coxph, which
# in return will crash the simulation machinery.
library( survival )
Cox.Dead <- coxph( Surv( DMdur, DMdur+lex.dur,
                         lex.Xst \%in\% c("Dead(Ins)","Dead")) ~
                   Ns( Age-DMdur, knots=ad.kn ) +
                   I(lex.Cst=="Ins") +
                   I(Per-2000) + sex,
               data = Si )
Cr <- list( "DM" = list( "Ins"       = DM.Ins,
                         "Dead"      = Cox.Dead  ),
           "Ins" = list( "Dead(Ins)" = Cox.Dead ) )
simL <- simLexis( Cr, ini, time.pts=seq(0,11,0.2), N=200 )
summary( simL )
nSt <- nState( subset(simL,sex=="M"),
               at=seq(0,10,0.2), from=1995, time.scale="Per" )
pp <- pState( nSt, perm=c(1,2,4,3) )
plot( pp )
}
\keyword{survival}

\name{blcaIT}
\alias{blcaIT}
\docType{data}
\title{Bladder cancer mortality in Italian males}
\description{
Number of deaths from bladder cancer and person-years in the Italian
male population 1955--1979, in ages 25--79.
}
% \usage{data(blcaIT)}
\format{
A data frame with 55 observations on the following 4 variables:
\tabular{rl}{
  \code{age}:    \tab Age at death. Left endpoint of age class \cr
  \code{period}: \tab Period of death. Left endpoint of period \cr
  \code{D}:      \tab Number of deaths \cr
  \code{Y}:      \tab Number of person-years.
}
}
% \source{
%   Reference to a source...
% }
% \references{
%   Reference to a publication...
% }
\examples{
data(blcaIT)
}
\keyword{datasets}

\name{nickel}
\alias{nickel}
\docType{data}
\title{A Cohort of Nickel Smelters in South Wales}
\description{
The \code{nickel} data frame has 679 rows and 7 columns.
The data concern a cohort of nickel smelting workers in South Wales and
are taken from Breslow and Day, Volume 2. For comparison purposes,
England and Wales mortality rates (per 1,000,000 per annum)
from lung cancer (ICDs 162 and 163),
nasal cancer (ICD 160), and all causes, by age group and calendar period, are
supplied in the dataset \code{\link{ewrates}}.
}
\format{
This data frame contains the following columns:
\tabular{rl}{
\code{id}: \tab Subject identifier (numeric) \cr
\code{icd}: \tab ICD cause of death if dead, 0 otherwise (numeric) \cr
\code{exposure}: \tab Exposure index for workplace (numeric) \cr
\code{dob}: \tab Date of birth (numeric) \cr
\code{age1st}: \tab Age at first exposure (numeric) \cr
\code{agein}: \tab Age at start of follow-up (numeric) \cr
\code{ageout}: \tab Age at end of follow-up (numeric) \cr
}
}
\source{
Breslow NE, and Day N, Statistical Methods in Cancer Research. Volume
II: The Design and Analysis of Cohort Studies. IARC Scientific
Publications, IARC:Lyon, 1987.  
}
\examples{
data(nickel)
str(nickel)
}
\keyword{datasets}

\name{paths.Lexis}
\alias{paths}
\alias{paths.Lexis}
\title{
Generate paths traveled through a Lexis multistate model data frame. 
}
\description{
For each person the path traveled through states in a \code{\link{Lexis}}
multistate object is derived, 
}
\usage{
\method{paths}{Lexis}(Lx, dfr = FALSE, \dots)
}
\arguments{
  \item{Lx}{A \code{\link{Lexis}} object}
  \item{dfr}{Logical. Should results be returned as a data frame with
    columns \code{lex.id} and \code{path}?}
  \item{\dots}{Arguments passed on. Ignored}
}
\value{A factor with levels describing each person's path through states. It is
  of length \code{length(\link{nid}(Lx))}, named by the (character)
  values of \code{Lx$lex.id}. It will usually be wise to use
  \code{table} on the output.

  If \code{dfr} is \code{TRUE} a two-column data frame is returned. 
}
\author{
  Bendix Carstensen, \email{b@bxc.dk}, \url{http://bendixcarstensen.com}
}
\seealso{
\code{\link{cutLexis}},
\code{\link{mcutLexis}},
\code{\link{rcutLexis}},
\code{\link{nid}},
\code{\link{Lexis}}
}
\examples{
# a simple example from the package
example(DMlate)
str(paths.Lexis(dmi, dfr = TRUE))
str(pathD <- paths.Lexis(dmi))
cbind(addmargins(table(pathD)))
#
# an example with recurring events
example(steno2)
summary(L4)
str(pathS <- paths.Lexis(L4))
cbind(addmargins(table(pathS)))
  }
\concept{Lexis}
\concept{Multistate model}
\concept{Multistate paths}

\name{pctab}
\alias{pctab}
\title{Create percentages in a table}
\description{
Computes percentages and a margin of totals along a given margin of a table.
}
\usage{
pctab(TT, margin = length(dim(TT)), dec=1)
}
\arguments{
  \item{TT}{A table or array object}
  \item{margin}{Which margin should be the the total?}
  \item{dec}{How many decimals should be printed? If 0 or \code{FALSE}
    nothing is printed}
}
\value{
  A table of percentages, where all dimensions except the one specified \code{margin}
  has two extra levels named "All" (where all entries are 100) and "N".
  The function prints the table with \code{dec} decimals.
}
\author{
  Bendix Carstensen, Steno Diabetes Center,
  \url{http://bendixcarstensen.com}.
  }
\seealso{
  \code{\link{addmargins}}
}
\examples{
Aye <- sample( c("Yes","Si","Oui"), 177, replace=TRUE )
Bee <- sample( c("Hum","Buzz"), 177, replace=TRUE )
Sea <- sample( c("White","Black","Red","Dead"), 177, replace=TRUE )
A <- table( Aye, Bee, Sea )
A
ftable( pctab( A ) )
ftable( pctab( addmargins( A, 1 ), 3 ) )
round( ftable( pctab( addmargins( A, 1 ), 3 ), row.vars=3 ), 1)
}
\keyword{ manip }
\keyword{ methods }
\keyword{ array }

\name{N.dk}
\alias{N.dk}
\docType{data}
\title{Population size in Denmark}
\description{
  The population size at 1st January in ages 0-99.
}
\usage{data(N.dk)}
\format{
  A data frame with 7200 observations on the following 4 variables.
  \describe{
    \item{\code{sex}}{Sex, 1:males, 2:females}
    \item{\code{A}}{Age. 0:0, 1:1, ..., 98:98, 99:99+}
    \item{\code{P}}{Year}
    \item{\code{N}}{Number of persons alive at 1st January year \code{P}}
  }
}
\source{
  \url{http://www.statistikbanken.dk/statbank5a/SelectTable/omrade0.asp?SubjectCode=02&PLanguage=1&ShowNews=OFF}
}
\examples{
data(N.dk)
str(N.dk)
with(N.dk,addmargins(tapply(N,list(P,sex),sum),2))
with(subset(N.dk,P==max(P)),addmargins(tapply(N,list(A,sex),sum)))
}
\keyword{datasets}

\name{mh}
\alias{mh}
\alias{print.mh}
\title{
Mantel-Haenszel analyses of cohort and case-control studies
}
\description{
This function carries out Mantel-Haenszel comparisons in tabulated data 
derived from both cohort and case-control studies. 
}
\usage{
mh(cases, denom, compare=1, levels=c(1, 2), by=NULL,
     cohort=!is.integer(denom), confidence=0.9)
\method{print}{mh}(x, ...)
}
\arguments{
\item{cases}{
the table of case frequencies (a multiway array).
}
\item{denom}{
the denominator table. For cohort studies this should be a table of 
person-years observation, while for case-control studies it should be a 
table of control frequencies. 
}
\item{compare}{
the dimension of the table which defines the comparison groups (can be 
referred to either by number or by name). The default is the first 
dimension of the table.
}
\item{levels}{
a vector identifying (either by number or by name) the two groups to be 
compared. The default is the first two levels of the selected dimension.
}
\item{by}{
the dimensions not to be collapsed in the Mantel-Haenszel computations. 
Thus, this argument defines the structure of the resulting tables of
estimates and tests.  
}
\item{cohort}{
an indicator  whether the data derive from a cohort or a case-control 
study. If the denominator table is stored as an integer, a case-control
study is assumed.
}
\item{confidence}{
the approximate coverage probability for the confidence intervals to be 
computed.
}
\item{x}{a \code{mh} object
}
\item{...}{arguments passed on to \code{print}
}}
\value{
A list of class \code{mh} giving tables of rate (odds) ratio estimates,
their standard errors (on a log scale), lower and upper confidence
limits, chi-squared tests (1 degree of freedom) and the corresponding
p-values. The result list also includes numerator and denominator of the
Mantel-Haenszel estimates (q, r), and score test statistics and score
variance (u, v).
}
\section{Side Effects}{
None
}
\details{
Multiway tables of data
are accepted and any two levels of any dimension can be chosen as defining
the comparison groups. The rate (odds) ratio estimates and the associated
significance tests may be collapsed over all the remaining dimensions of the 
table, or over selected dimensions only, so that tables of estimates and 
tests are computed.
}
\references{
Clayton, D. and Hills, M. : Statistical Models in Epidemiology, Oxford 
University Press (1993). 
}
\seealso{
\code{\link{Lexis}}
}
\examples{
# If d and y are 3-way tables of cases and person-years 
# observation formed by tabulation by two confounders 
# (named "C1" and "C2") an exposure of interest ("E"), 
# the following command will calculate an overall 
# Mantel-Haenszel comparison of the first two exposure 
# groups.
#
# Generate some bogus data
dnam <- list( E=c("low","medium","high"), C1=letters[1:2], C2=LETTERS[1:4] )
d <- array( sample( 2:80, 24),
            dimnames=dnam, dim=sapply( dnam, length ) )
y <- array( abs( rnorm( 24, 227, 50 ) ),
            dimnames=dnam, dim=sapply( dnam, length ) )
mh(d, y, compare="E")
#
# Or, if exposure levels named "low" and "high" are to be 
# compared and these are not the first two levels of E :
#
mh(d, y, compare="E", levels=c("low", "high"))
#
# If we wish to carry out an analysis which controls for C1, 
# but examines the results at each level of C2:
#
mh(d, y, compare="E", by="C2")
#
# It is also possible to look at rate ratios for every 
# combination of C1 and C2 :
#
mh(d, y, compare="E", by=c("C1", "C2"))
#
# If dimensions and levels of the table are unnamed, they must 
# be referred to by number.
#
}
\keyword{htest}

\name{matshade}
\alias{matshade}
\title{
  Plot confidence intervals as shaded areas around lines.
}
\description{
  Uses an x-vector and a matrix of 3*N columns with estimates and ci.s
  to produce the lines of estimates and confidence intervals as shaded
  areas in transparent colours around the lines of the estimates.
}
\usage{
matshade( x, y, lty = 1,
          col = 1:(ncol(y)/3), col.shade=col, alpha=0.15,
         plot = dev.cur()==1,
          ... )
}
\arguments{
  \item{x}{Numerical vector. Unlike \code{\link{matplot}} this can only be a vector.
}
  \item{y}{A matrix with 3*N columns --- representing estimates and
    confidence bounds for N curves. Order of columns are assumed to be
    (est,lo,hi,est,lo,hi...) (or (est,hi,lo...))
}
  \item{lty}{Line types for the curves.
}
  \item{col}{Color(s) of the estimated curves.
}
  \item{col.shade}{Color(s) of the shaded areas. These are the colors
    that are made transparent by the \code{alpha} factor. Defaults to
    the same colors as the lines.
}
  \item{alpha}{Number in [0,1] indicating the transparency of the colors for
    the confidence intervals. Larger values makes the shades
    darker. Can be a vector which then applies to the curves in turn.
}
  \item{plot}{Logical. Should a new plot frame be started? If no device
    is active, the default is to start one, and plot all \code{y}s versus
    x in transparent color. On the rare occasion a device is open, but
    no plot have been called you will get an error telling that plot.new
    has not been called yet, in which case you should explicitly set
    \code{plot} to \code{TRUE}. 
}  
  \item{\dots}{Arguments passed on to \code{\link{matplot}} (if
    \code{plot=TRUE}) and \code{\link{matlines}} for use when plotting
    the lines. Note that \code{lwd=0} will cause lines to be omitted and
    only the shades be plotted. 
}
}

\details{All shaded areas are plotted first, the curves added
  afterwards, so that lines are not 'overshadowed'.

  If there are NAs in \code{x} or \code{y} there will be separate shaded
  areas for each non-\code{NA} sequence. Applies separately to each set
  of confidence bands in \code{y}. 
  
  Note that if you repeat the same command, you will get the curves
  and the shaded areas overplotted in the same frame, so the effect is to
  have the shades darker, because the transparent colors are plotted on
  top of those from the first command.
}
\value{NULL. Used for its side effects.
}
\author{Bendix Carstensen, \url{http://bendixcarstensen.com}
}
\seealso{\code{\link{pc.matshade}}
}
\examples{
# Follow-up data of Danish DM patients
data( DMlate )
mL <- Lexis( entry=list(age=dodm-dobth,per=dodm),
              exit=list(per=dox),
       exit.status=factor(!is.na(dodth),labels=c("Alive","Dead")),
              data=DMlate )
# Split follow-up and model by splines
sL <- splitLexis( mL, breaks=0:100, time.scale="age")
\dontrun{
# the same thing with popEpi
sL <- splitMulti( mL, age=0:100 )    
        }
# Mortality rates separately for M and F:
mort <- glm( (lex.Xst=="Dead") ~ sex*Ns(age,knots=c(15,3:8*10)),
             offset = log(lex.dur),
             family = poisson,
               data = sL )
\dontrun{
# The counterpart with gam
library( mgcv )
mort <- gam( (lex.Xst=="Dead") ~ s(age,by=sex) + sex,
             offset = log(lex.dur),
             family = poisson,
               data = sL )
       }
# predict rates (per 1000 PY) for men and women
ndM <- data.frame( age=10:90, sex="M", lex.dur=1 )
ndF <- data.frame( age=10:90, sex="F", lex.dur=1 )
# gam objects ignores the offset in prediction so
# lex.dur=1000 in prediction frame wll not work.
prM <- ci.pred( mort, ndM )*1000
prF <- ci.pred( mort, ndF )*1000
# predict rate-ratio
MFr <- ci.exp( mort, ctr.mat=list(ndM,ndF) )
# plot lines with shaded confidence limits
# for illustration we make a holes for the RRs:
MFr[40:45,2] <- NA
MFr[44:49,1] <- NA
matshade( ndM$age, cbind( MFr, prF, prM ), col=c(1,2,4), lwd=3,
          log="y", xlab="Age", ylab="Mortality per 1000 PY (and RR)" )
abline( h=1 )
}
\keyword{color}

\name{DMepi}
\alias{DMepi}
\docType{data}
\title{Epidemiological rates for diabetes in Denmark 1996--2015}
\description{Register based counts and person-years for incidence of
  diabetes and mortality with and without diabetes.
}
\usage{data("DMepi")}
\format{
  A data frame with 4200 observations on the following 8 variables.
  \describe{
    \item{\code{sex}}{a factor with levels \code{M}, \code{F}}
    \item{\code{A}}{Age class, 0--99}
    \item{\code{P}}{Calendar year, 1996--2016}
    \item{\code{X}}{Number of new diagnoses of diabetes among persons without diabetes}
    \item{\code{D.nD}}{Number of deaths among persons without diabetes}	   
    \item{\code{Y.nD}}{Person-years among persons without diabetes}
    \item{\code{D.DM}}{Number of deaths among persons with diabetes}	   
    \item{\code{Y.DM}}{Person-years among persons with diabetes}
  }
}
\details{Based on registers of the Danish population. Only included for
  illustrative purposes. Cannot be used as scientifically validated
  data, since small numbers are randomly permuted between units.
}
\examples{
data(DMepi)
# Total deaths and person-years in the Danish population
ftable( addmargins( xtabs( cbind( Deaths=D.nD+D.DM,
                                    PYrs=Y.nD+Y.DM ) ~ P + sex,
                           data=DMepi ),
                    2 ),
        row.vars = 1 )
# Deaths and person-years in the population of diabetes patients
round(
ftable( addmargins( xtabs( cbind( Deaths=D.DM,
                                    PYrs=Y.DM ) ~ P + sex,
                           data=DMepi ),
                    2 ),
        row.vars = 1 ) )

# Model for age-specific incidence rates;
inc <- glm( X ~ sex + Ns( A, knots=seq(30,80,10) ) + P,
                offset = log(Y.nD),
                family = poisson,
                  data = DMepi )

# Predict for men and women separately in 2010:
ndm <- data.frame( sex="M", A=20:90, P=2010, Y.nD=1000 )
ndf <- data.frame( sex="F", A=20:90, P=2010, Y.nD=1000 )
prM <- ci.pred( inc, ndm )
prF <- ci.pred( inc, ndf )
matplot( ndm$A, cbind(prM,prF),
         type="l", lty=1, lwd=c(3,1,1),
         col=rep(c("blue","red"),each=3),
         log="y", xlab="Age", ylab="DM incidence per 1000 PY" )

# This is a proportional hazards model - add sex-age interaction
int <- update( inc, . ~ . + sex:Ns( A, knots=seq(30,80,10) ) )
prM <- ci.pred( int, ndm )
prF <- ci.pred( int, ndf )
matplot( ndm$A, cbind(prM,prF),
         type="l", lty=1, lwd=c(3,1,1),
         col=rep(c("blue","red"),each=3),
         log="y", xlab="Age", ylab="DM incidence per 1000 PY" )

# The rate-ratio is teased out using the ci.exp:
RRp <- ci.exp( inc, list(ndm,ndf) )
RRi <- ci.exp( int, list(ndm,ndf) )

# and added to the plot
matlines( ndm$A, cbind(RRi,RRp),
          type="l", lty=1, lwd=c(3,1,1), col=gray(rep(c(0.3,0.7),each=3)) )
abline(h=1)
axis(side=4)
mtext( "Male/Female IRR", side=4, line=2 )
}
\keyword{datasets}

\name{mcutLexis}
\alias{mcutLexis}
\title{
Cut follow-up at multiple event dates and keep track of order of events 
}
\description{
A generalization of \code{\link{cutLexis}} to the case where different
events may occur in any order (but at most once for each).
}
\usage{
mcutLexis( L0, timescale = 1, wh,
              new.states = NULL,
        precursor.states = transient(L0),
              seq.states = TRUE,
              new.scales = NULL,
            ties.resolve = FALSE )
}
\arguments{
  \item{L0}{A Lexis object.}
  \item{timescale}{Which time scale do the variables in \code{L0[,wh]}
    refer to. Can be character or integer.}
  \item{wh}{Which variables contain the event dates. Character or
    integer vector}
  \item{new.states}{Names of the events forming new states. If
    \code{NULL} equal to the variable names from \code{wh}.}
  \item{precursor.states}{Which states are precursor states. See
    \code{\link{cutLexis}} for definition of precursor states.}
  \item{seq.states}{Should the sequence of events be kept track of? That
    is, should A-B be considered different from B-A. If \code{FALSE},
    the state with combined preceding events A and B will be called
    A+B (alphabetically sorted).
    
    May also be supplied as character: \code{s} - sequence, keep
    track of sequence of states occupied (same as \code{TRUE}), \code{u}
    - unordered, keep track only of states visited (same as
    \code{FALSE}) or \code{l}, \code{c} - last or current state, only
    record the latest state visited. If given as character, only the
    first letter converted to lower case is used.}
  \item{new.scales}{Should we construct new time scales indicating the
    time since each of the event occurrences.}
  \item{ties.resolve}{Should tied event times be resolved by adding
    random noise to tied event dates. If \code{FALSE} the function will
    not accept that two events occur at the same time for a person
    (ties). If \code{TRUE} a random quantity in the range
    \code{c(-1,1)/100} will be added to all event times in all records
    with at least one tie. If \code{ties.resolve} is numeric a random
    quantity in the range \code{c(-1,1)*ties.resolve} will be added to
    all event times in all records with at least one tie.}
}
\value{A \code{\link{Lexis}} object with extra states created by
  occurrence of a number of intermediate events.
}
\author{
Bendix Carstensen, \url{http://bendixcarstensen.com}
}
\seealso{
  \code{\link{cutLexis}},
  \code{\link{rcutLexis}},
  \code{\link{xcutLexis}},
  \code{\link{addCov.Lexis}},
  \code{\link{addDrug.Lexis}},
  \code{\link{Lexis}},
  \code{\link{splitLexis}}
}
\examples{
# A dataframe of times
set.seed(563248)
dd <- data.frame( id = 1:30,
                 doN = round(runif(30,-30, 0),1),
                 doE = round(runif(30,  0,20),1),
                 doX = round(runif(30, 50,60),1),
                 doD = round(runif(30, 50,60),1),
                 # these are the event times
                 doA = c(NA,21,NA,27,35,NA,52, 5,43,80,
                         NA,22,56,28,53,NA,51, 5,43,80,
                         NA,23,NA,33,51,NA,55, 5,43,80),
                 doB = c(NA,20,NA,53,27,NA, 5,52,34,83,
                         NA,20,23,37,35,NA,52, 8,33,NA,
                         25,NA,37,40,NA,NA,15,23,36,61) )

# set up a Lexis object with time from entry to death/exit
Lx <- Lexis( entry = list(time=doE,
                           age=doE-doN),
              exit = list(time=pmin(doX,doD)),
       exit.status = factor(doD<doX,labels=c("OK","D")),
              data = dd )
summary( Lx )

# cut the follow-up at dates doA and doB
L2 <- mcutLexis( Lx, "time", wh=c("doA","doB"),
                 new.states = c("A","B"),
           precursor.states = "OK",
                 seq.states = TRUE,
                 new.scales = c("tfA","tfB") )
summary( L2 )
L2

# show the states
boxes( L2, boxpos=list(x=c(10,60,50,90,50,90),
                       y=c(50,50,90,90,10,10)),
           scale.R=100, show.BE=TRUE, DR.sep=c(" (",")"))


L3 <- mcutLexis( Lx, "time", wh=c("doA","doB"),
                 new.states = c("A","B"),
           precursor.states = "OK",
                 seq.states = FALSE,
                 new.scales = c("tfA","tfB") )
summary( L3 )
boxes( L3, boxpos=list(x=c(10,50,50,90,50),
                       y=c(50,50,90,50,10)),
           show.R=FALSE, show.BE=TRUE )
}
\keyword{survival}

\name{harm}
\alias{harm}
\title{Create a basis of harmonic functions.
}
\description{Returns a matrix of harmonic functions usable for modeling
periodic effects
}
\usage{
harm(x, ord=1, per=1, verbose=FALSE )
}
\arguments{
  \item{x}{A numeric variable.
}
  \item{ord}{Integer, the order of the harmonic.
}
  \item{per}{Numeric, the length of the period on the \code{x} scale.
}
  \item{verbose}{Logical: shall I tell what I do with dates?
}
}
\details{
  Columns are constructed under the assumption that the periodic function
  has period \code{per} on the \code{x} scale. Thus, the first columns
  is defined as \code{sin(2*pi*x/per)}, \code{cos(2*pi*x/per)},
  \code{sin(4*pi*x/per)} etc.

  Since \code{sin} and \code{cos} are periodic functions there is no
  requirement that \code{x} be in any particular range.
}
\value{A matrix with \code{nrow(x)} rows and
  2*\code{deg} columns and columnnames
  \code{sin1}, \code{cos1},
  \code{sin2}, \code{cos2} etc.
}
\author{
  Bendix Carstensen, \url{http://bendixcarstensen.com}
}
\examples{
x <- seq(-1,1,0.01)
head( harm(x,ord=2) )
matplot( x, harm(x,ord=2), type="l", lty=1, lwd=3 )
}
\keyword{models}
\keyword{regression}

\name{mat2pol}
\alias{mat2pol}
\title{Plot columns of a matrix as stacked areas.
}
\description{\code{mat}rix to \code{pol}ygon: Plot columns of a matrix
  as stacked areas.
}
\usage{
mat2pol( pm,
       perm = 1:ncol(pm),
          x = as.numeric(rownames(pm)),
        col = rainbow(ncol(pm)),
         yl = 0:1,
     append = FALSE,
        ... )
}
\arguments{
  \item{pm}{Numerical matrix.}
  \item{perm}{integer vector of length \code{ncol(pm)}, used to permute
    the columns of \code{pm}.}
  \item{x}{Numeric. The x-axis of the plot.}
  \item{col}{Colors of the areas.}
  \item{yl}{y-axis limits.}
  \item{append}{Logical. Should the polygons be added to an exiating plot}
  \item{\dots}{Further parameters passed to \code{plot}.}
}
\details{The function is originally intended to plot stacked
  probabilities, hence the default of \code{0:1} for the y-axis.
}
\value{A matrix of \code{ncol(pm)+1} columns with the first equal to 0,
  and the remaining the cumulative sum of the columns of
  \code{pm[perm]}.  

  The function is called for its side effect - the stacked polygons.
}
\author{Bendix Carstensen
}
\examples{
M <- cbind( sort(runif(10)), sort(runif(10)), sort(runif(10)) )
pm <- sweep( M, 1, apply(M,1,sum), "/" )
mat2pol( pm )
}
\keyword{manip}

\name{testisDK}
\alias{testisDK}
\docType{data}
\title{Testis cancer incidence in Denmark, 1943--1996
}
\description{Number of testiscancer cases and male person-years in the
  Danish population 1943--1996}
\usage{data(testisDK)}
\format{
  A data frame with 4860 observations on the following 4 variables.
  \describe{
    \item{\code{A}}{Age class, 0,1,2,...,89}
    \item{\code{P}}{Year, 1943,...,1996}
    \item{\code{D}}{Number of testis cancer cases}
    \item{\code{Y}}{Person years}
  }
}
\source{The Danish Cancer Registry
}
\examples{
data(testisDK)
head(testisDK)
}
\keyword{datasets}

\name{ci.cum}
\alias{ci.cum}
\alias{ci.surv}
\title{ Compute cumulative sum of estimates. }
\description{
Computes the cumulative sum of parameter functions and the
standard error of it. Used for computing the cumulative rate, or the
survival function based on a \code{glm} with parametric baseline.
}
\usage{
ci.cum( obj,
    ctr.mat = NULL,
     subset = NULL,
       intl = NULL,
      alpha = 0.05,
        Exp = TRUE,
     ci.Exp = FALSE,
     sample = FALSE )
ci.surv( obj,
    ctr.mat = NULL,
     subset = NULL,
       intl = NULL,
      alpha = 0.05,
        Exp = TRUE,
     sample = FALSE )
}
\arguments{

\item{obj}{A model object (of class \code{lm}, \code{glm}. }

\item{ctr.mat}{Matrix or data frame.

If \code{ctr.mat} is a matrix, it should be a contrast matrix to be
multiplied to the parameter vector, i.e. the desired linear function of
the parameters.

If it is a data frame it should have columns corresponding to a
prediction data frame for \code{obj}, see details for
\code{\link{ci.lin}}.}

\item{subset}{ Subset of the parameters of the model to which a matrix
   \code{ctr.mat} should be applied. }

\item{intl}{ Interval length for the cumulation. Either a constant or a
    numerical vector of length \code{nrow(ctr.mat)}. If omitted taken as
    the difference between the two first elments if the first column in
    \code{ctr.mat}, assuming that that holds the time scale. A note is
    issued in this case.} 

\item{alpha}{Significance level used when computing confidence limits.}

\item{Exp}{Should the parameter function be exponentiated before it is
    cumulated?}

\item{ci.Exp}{Should the confidence limits for the cumulative rate be
  computed on the log-scale, thus ensuring that exp(-cum.rate) is always
  in [0,1]?}

\item{sample}{Should a sample of the original parameters be used to
    compute a cumulative rate?}  } \details{
  
The purpose of \code{ci.cum} is to the compute cumulative rate
(integrated intensity) at a set of points based on a model for the
rates. \code{ctr.mat} is a matrix which, when premultiplied to the
parameters of the model returns the (log)rates at a set of equidistant
time points. If log-rates are returned from the prediction method for
the model, the they should be exponentiated before cumulated, and the
variances computed accordingly. Since the primary use is for log-linear
Poisson models the \code{Exp} parameter defaults to TRUE.

Each row in the object supplied via \code{ctr.mat} is assumed to
represent a midpoint of an interval. \code{ci.cum} will then return the
cumulative rates at the \emph{end} of these intervals. \code{ci.surv}
will return the survival probability at the \emph{start} of each of
these intervals, assuming the the first interval starts at 0 - the first
row of the result is \code{c(1,1,1)}.

The \code{ci.Exp} argument ensures that the confidence intervals for the
cumulative rates are always positive, so that exp(-cum.rate) is always
in [0,1].
}
\value{
  A matrix with 3 columns: Estimate, lower and upper c.i.  If
  \code{sample} is TRUE, a single sampled vector is returned, if
  \code{sample} is numeric a matrix with \code{sample} columns is
  returned, each column a cumulative rate based on a random sample from
  the distribution of the parameter estimates.

  \code{ci.surv} returns a 3 column matrix with estimate, lower and
  upper confidence bounds for the survival function.
}
\author{
  Bendix Carstensen,
  \url{http://bendixcarstensen.com}
}
\seealso{ See also \code{\link{ci.lin}}, \code{\link{ci.pred}} }
\examples{
# Packages required for this example
library( splines )
library( survival )
data( lung )
par( mfrow=c(1,2) )

# Plot the Kaplan-meier-estimator
plot( survfit( Surv( time, status==2 ) ~ 1, data=lung ) )

# Declare data as Lexis
lungL <- Lexis(exit = list(tfd=time),
               exit.status = (status == 2) * 1,
               data = lung)
summary(lungL)

# Split the follow-up every 10 days
sL <- splitLexis(lungL, "tfd", breaks=seq(0,1100,10))
summary(sL)

# Fit a Poisson model with a natural spline for the effect of time (left
# end points of intervals are used as covariate)
mp <- glm(cbind(lex.Xst == 1, lex.dur)
          ~ Ns(tfd,knots = c(0, 50, 100, 200, 400, 700)),
          family = poisreg,
            data = sL)

# mp is now a model for the rates along the time scale tfd
# prediction data frame for select time points on this time scale
nd <- data.frame(tfd = seq(5,995,10)) # *midpoints* of intervals
Lambda <- ci.cum ( mp, nd, intl=10 )
surv   <- ci.surv( mp, nd, intl=10 )

# Put the estimated survival function on top of the KM-estimator
# recall the ci.surv return the survival at *start* of intervals
matshade(nd$tfd - 5, surv, col = "Red", alpha = 0.15)

# Extract and plot the fitted intensity function
lambda <- ci.pred(mp, nd) * 365.25 # mortality 
matshade(nd$tfd, lambda, log = "y", ylim = c(0.2, 5), plot = TRUE,
          xlab = "Time since diagnosis",
          ylab = "Mortality per year")

# same thing works with gam from mgcv
library(mgcv)
mg <- gam(cbind(lex.Xst == 1, lex.dur) ~ s(tfd), family = poisreg, data=sL )
matshade(nd$tfd - 5, ci.surv(mg, nd, intl=10), plot=TRUE,
         xlab = "Days since diagnosis", ylab="P(survival)")
matshade(nd$tfd  , ci.pred(mg, nd) * 365.25, plot=TRUE, log="y",
         xlab = "Days since diagnosis", ylab="Mortality per 1 py")
}
\keyword{models}
\keyword{regression}

\name{plotCIF}
\alias{plotCIF}
\alias{stackedCIF}

\title{Plotting Aalen-Johansen curves for competing events
}

\description{Function \code{plotCIF} plots, for one or more groups, the
cumulative incidence curves for a selected event out of two or more
competing events. Function \code{stackedCIF} plots, for one group or
population, the cumulative incidence curves for two or more competing
events such that the cumulative incidences are stacked upon each
other. The CIFs are are estimated by the Aalen-Johansen method.
}
\usage{
## S3 method for class 'survfit'
 plotCIF( x, event = 1,
              xlab = "Time",
              ylab = "Cumulative incidence",
              ylim = c(0, 1),
               lty = 1,
               col = "black", ... )

## S3 method for class 'survfit'
stackedCIF( x, group = 1,
                 col = "black",
                fill = "white",
                ylim = c(0,1),
                xlab = "Time",
                ylab = "Cumulative incidence", ... )
}
\arguments{
  
\item{x}{An object of class \code{\link[survival]{survfit}}, the \code{type} of
  \code{event} in \code{Surv()} being "\code{mstate}"; the first level
  of the event factor represents censoring and the remaining ones the
  alternative competing events. 
}
\item{event}{Determines the event for which the cumulative incidence
  curve is plotted by \code{plotCIF}.
}
\item{group}{An integer showing the selected level of a possible
  grouping factor appearing in the model formula in \code{survfit} when
  plotting by \code{stackedCIF} 
}
\item{col}{A vector specifying the plotting color(s) of the curve(s) for
  the different groups in \code{\link{plotCIF}}-- default: all "black". 
}
\item{fill}{A vector indicating the colours to be used for shading the
  areas pertinent to the separate outcomes in \code{\link{stackedCIF}}
  - default: all \code{"white"}.
}
\item{xlab}{Label for the $x$-axis.
}
\item{ylab}{Label for the $y$-axis.
}
\item{ylim}{Limits of the $y$-axis.
}
\item{lty}{A vector specifying the line type(s) of the curve(s) for
    the different groups - default: all 1 (=solid). 
}
\item{\dots}{Further graphical parameters to be passed.
}
}
\details{
The order in which the curves with \code{\link{stackedCIF}} are piled
upon each other is the same as the ordering of the values or levels of
the competing events in the pertinent event variable. The ordering can
be changed by permuting the levels as desired using function
\code{Relevel}, after which \code{survfit} is called with the relevelled
\code{event} variable in \code{Surv()}
}
\value{No value is returned but a plot is produced as a side-effect.
}
\references{Putter, H., Fiocco, M., Geskus, R.B. (2007). 
Tutorial in biostatistics: competing risks and multi-state models. 
Statistics in Medicine, 26: 2389--2430.
}
\author{Esa Laara, \email{esa.laara@oulu.fi}
}
\note{
Aalen-Johansen curves for competing events in several groups can also
be plotted by function \code{\link[survival]{plot.survfit}} of the survival
library as well as by some functions in other packages covering analysis
of time-to-event data.}
\seealso{
  \code{\link[survival]{survfit}},
  \code{\link{plot}},
  \code{\link[survival]{plot.survfit}}. 
}
\examples{
library(survival)   #  requires version 2.39-4 or later
head(mgus1)
#  Aalen-Johansen estimates of CIF are plotted by sex for two 
#  competing events: (1) progression (pcm), and (2) death, in 
#  a cohort of patients with monoclonal gammopathy.

#  The data are actually covering transitions from pcm to death, too,
#  for those entering the state of pcm. Such patients have two rows
#  in the data frame, and in their 2nd row the 'start' time is 
#  the time to pcm (in days). 

#  In our analysis we shall only include those time intervals with value 0
#  for variable 'start'. Thus, the relevant follow-up time is represented 
#  by variable 'stop' (days). For convenience, days are converted to years.

fitCI <- survfit(Surv(stop/365.25, event, type="mstate") ~ sex,
              data= subset(mgus1, start==0) )
par(mfrow=c(1,2))
plotCIF(fitCI, event = 1, col = c("red", "blue"),
  main = "Progression", xlab="Time (years)" )
text( 38, 0.15, "Men", pos = 2)
text( 38, 0.4, "Women", pos = 2)
plotCIF(fitCI, event = 2, col = c("red", "blue"), 
  main = "Death", xlab="Time (years)" )
text( 38, 0.8, "Men", pos = 2)
text( 38, 0.5, "Women", pos = 2)

par(mfrow=c(1,2))
stackedCIF(fitCI, group = 1, fill = c("gray80", "gray90"),
  main = "Women", xlab="Time (years)" )	
text( 36, 0.15, "PCM", pos = 2)
text( 36, 0.6, "Death", pos = 2)
stackedCIF(fitCI, group = 2, fill = c("gray80", "gray90"), 
  main = "Men", xlab="Time (years)" )
text( 39, 0.10, "PCM", pos = 2)
text( 39, 0.6, "Death", pos = 2)	
}

\name{contr.cum}
\alias{contr.cum}
\alias{contr.2nd}
\alias{contr.diff}
\alias{contr.orth}
\title{
  Contrast matrices
  }
\description{
  Return a matrix of contrasts for factor coding.
  }
\usage{
  contr.cum(n)
  contr.diff(n)
  contr.2nd(n)
  contr.orth(n) 
  }
\arguments{
  \item{n}{A vector of levels for a factor, or the number of levels.}
  }
\details{
  These functions are used for creating contrast matrices for use in
  fitting regression models. The columns of the
  resulting matrices contain contrasts which can be used for coding a
  factor with \code{n} levels.

  \code{contr.cum} gives a coding corresponding to successive differences
  between factor levels.

  \code{contr.diff} gives a coding that correspond to the cumulative sum
  of the value for each level. This is not meaningful in a model where the
  intercept is included, therefore \code{n} columns ia always returned.

  \code{contr.2nd} gives contrasts corresponding to 2nd order differences
  between factor levels. Returns a matrix with \code{n-2} columns.

  \code{contr.orth} gives a matrix with \code{n-2} columns, which are
  mutually orthogonal and orthogonal to the matrix \code{cbind(1,1:n)}
  }
\value{
  A matrix with \code{n} rows and \code{k} columns, with
  \code{k}=\code{n} for \code{contr.diff}
  \code{k}=\code{n-1} for \code{contr.cum}
  \code{k}=\code{n-2} for \code{contr.2nd} and  \code{contr.orth}.
  }
\author{Bendix Carstensen}
\seealso{
  \code{\link{contr.treatment}}
  }
\examples{
contr.cum(6)
contr.2nd(6)
contr.diff(6)
contr.orth(6)
  }
\keyword{design}
\keyword{models}

\name{ci.Crisk}
\alias{ci.Crisk}
\title{
Compute cumulative risks and expected sojourn times from models for
cause-specific rates.  
}
\description{
  
Consider a list of parametric models for rates of competing events, such
as different causes of death, A, B, C, say. From estimates of the
cause-specific rates we can compute 1) the cumulative risk of being in
each state ('Surv' (=no event) and A, B and C) at different times, 2)
the stacked cumulative rates such as A, A+C, A+C+Surv and 3) the
expected (truncated) sojourn times in each state up to each time point.

This can be done by simple numerical integration using estimates from
models for the cause specific rates. But the standard errors of the
results are analytically intractable.

The function \code{ci.Crisk} computes estimates with confidence
intervals using simulated samples from the parameter vectors of supplied
model objects. Some call this a parametric bootstrap.

The times and other covariates determining the cause-specific rates must
be supplied in a data frame which will be used for predicting rates for
all transitions.
}
\usage{
ci.Crisk(mods,
           nd,
         tnam = names(nd)[1],
           nB = 1000,
         perm = length(mods):0 + 1,
        alpha = 0.05, 
      sim.res = 'none')
}
\arguments{
  \item{mods}{A named list of \code{glm}/\code{gam} model objects
    representing the cause-specific rates. If the list is not named the
    function will crash. The names will be used as names for the states
    (competing risks), while the state without any event will be called
    "\code{Surv}".
  }
  \item{nd}{A data frame of prediction points and covariates to be used
    on all models supplied in \code{mods}. 
  }
  \item{tnam}{Name of the column in \code{nd} which is the time scale.It
    must represent endpoints of equidistant intervals.
  }
  \item{nB}{Scalar. The number of simulations from the (posterior)
    distribution of the model parameters to be used in computing
    confidence limits.  
  }
  \item{perm}{Numerical vector of length \code{length(mods)+1} indicating
    the order in which states are to be stacked. The \code{'Surv'} state
    is taken to be the first, the remaining in the reverse order supplied
    in the \code{mods} argument. The default is therefore to stack with
    the survival as the first, which may not be what you normally want.   
}
  \item{alpha}{numeric. 1 minus the confidence level used in calculating
    the c.i.s 
}
  \item{sim.res}{Character. What simulation samples should be
    returned. If \code{'none'} (the default) the function returns a list
    of 3 arrays (see under 'value'). If \code{'rates'} it returns an
    array of dimension \code{nrow(nd)} x \code{length(mod)} x \code{nB}
    of bootstrap samples of the rates. If \code{'crisk'} it returns an
    array of dimension \code{nrow(nd)} x \code{length(mod)+1} x
    \code{nB} of bootstrap samples of the cumulative rates.  Only the
    first letter matters, regardless of whether it is in upper lower
    case.
}  
}

\value{If \code{sim.res='none'} a named list with 4 components, the first 3
  are 3-way arrays classified by time, state and estimate/confidence
  interval:  
   
  \itemize{
    \item \code{Crisk} Cumulative risks for the \code{length(mods)}
    events \emph{and} the survival
    \item \code{Srisk} Stacked versions of the cumulative risks
    \item \code{Stime} Sojourn times in each states
    \item \code{time} Endpoints of intervals. It is just the numerical
    version of the names of the first dimension of the three arrays
  }
  All three arrays have (almost) the same dimensions:
  \itemize{
    \item time, named as \code{tnam}; endpoints of intervals. Length
    \code{nrow(nd)}.
    
    \item \code{cause}. The arrays \code{Crisk} and \code{Stime} have
    values "\code{Surv}" plus the names of the list \code{mods} (first
    argument). \code{Srisk} has length \code{length(mod)}, with each
    level representing a cumulative sum of cumulative risks, in order
    indicated by the \code{perm} argument.

    \item Unnamed, \code{ci.50\%}, \code{ci.2.5\%}, \code{ci.97.5\%}
    representing quantiles of the quantities derived from the bootstrap
    samples. If \code{alpha} is different from 0.05, names are of course
    different.
  }
  If \code{sim.res='rates'} the function returns bootstrap samples of
    rates for each cause as an array
    classified by time, cause and bootstrap sample.
  
  If \code{sim.res='crisk'} the function returns bootstrap samples of
    cumulative risks for each cause (including survival) as an array
    classified by time, state (= causes + surv) and bootstrap sample.
}

\author{
  Bendix Carstensen, \url{http://bendixcarstensen.com}
}
\seealso{
  \code{\link{mat2pol}}
  \code{\link{simLexis}}
  \code{\link{plotCIF}}
  \code{\link{ci.surv}}
}
\examples{
library(Epi)
data(DMlate)

# A Lexis object for survival
Ldm <- Lexis(entry = list( per = dodm,
                           age = dodm-dobth, 
                           tfd = 0 ),
              exit = list( per = dox ),
       exit.status = factor( !is.na(dodth), labels = c("DM","Dead") ),
              data = DMlate[sample(1:nrow(DMlate),1000),] )
summary(Ldm, timeScales = TRUE)

# Cut at OAD and Ins times
Mdm <- mcutLexis(Ldm,
                  wh = c('dooad','doins'),
          new.states = c('OAD','Ins'),
          seq.states = FALSE,
                ties = TRUE)
summary(Mdm$lex.dur)

# restrict to DM state and split
Sdm <- splitLexis(factorize(subset(Mdm, lex.Cst == "DM")),
                  time.scale = "tfd",
                  breaks = seq(0,20,1/12))
summary(Sdm)
summary(Relevel(Sdm, c(1, 4, 2, 3)))

boxes(Relevel(Sdm, c(1, 4, 2, 3)), 
      boxpos = list(x = c(15, 85, 80, 15),
                    y = c(85, 85, 20, 15)),
      scale.R = 100)

# glm models for the cause-specific rates
system.time(
mD <- glm.Lexis(Sdm, ~ Ns(tfd, knots=0:6*2), to = 'Dead') )
system.time(
mO <- glm.Lexis(Sdm, ~ Ns(tfd, knots=0:6*2), to = 'OAD' ) )
system.time(
mI <- glm.Lexis(Sdm, ~ Ns(tfd, knots=0:6*2), to = 'Ins' ) )

# intervals for calculation of predicted rates
int <- 1 / 100
nd <- data.frame(tfd = seq(0, 10, int)) # not the same as the split, 
                                        # and totally unrelated to it

# cumulaive risks with confidence intervals
# (too few timepoints, too few simluations)
system.time(
res <- ci.Crisk(list(OAD = mO, 
                     Ins = mI, 
                    Dead = mD),
                            nd = data.frame(tfd = 0:100 / 10),
                            nB = 100,
                          perm = 4:1))
str(res)
}
\keyword{regression}

\name{Termplot}
\alias{Termplot}
\title{
  A wrapper for \code{termplot} that optionally (but by default)
  exponentiates terms, and plot them on a common log-scale. Also scales
  x-axes to the same physical scale.
}
\description{
  The function uses \code{\link{termplot}} to extract terms from a model
  with, say, spline, terms, including the standard errors, computes
  confidence intervals and transform these to the rate / rate-ratio
  scale. Thus the default use is for models on the log-scale such as
  Poisson-regression models. The function produces a plot with panels
  side-by-side, one panel per term, and returns the 
}
\usage{
  Termplot( obj,
           plot = TRUE,
           xlab = NULL,
           ylab = NULL,
            xeq = TRUE,
           yshr = 1,
          alpha = 0.05,
          terms = NULL,
         max.pt = NULL )
}
\arguments{
  \item{obj}{An object with a \code{terms}-method --- for details the
    the documentation for \code{\link{termplot}}. }
  \item{plot}{Should a plot be produced?}
  \item{xlab}{Labels for the \code{x}-axes. Defaults to the names of the terms.}
  \item{ylab}{Labels for the \code{x}-axes. Defaults to blank.}
  \item{xeq}{Should the units all all plots have the same physical scale
    for the \code{x}-axes).}
  \item{yshr}{Shrinking of \code{y}-axis. By default, the \code{y}-axes
    have an extent that accommodates the entire range of confidence
    intervals. This is a shrinking parameter for the \code{y}-axes,
    setting it to less than 1 will lose a bit of the confidence limits
    on some of the panels.}
  \item{alpha}{1 minus the confidence level for computing confidence
    intervals}
  \item{terms}{Which terms should be reported. Passed on to
    \code{\link{termplot}} and eventually \code{\link{predict}}.}
  \item{max.pt}{The maximal number of points in which to report the
    terms. If \code{NULL} all unique points from the analysis dataset
    are reported for each term (this is a feature of \code{\link{termplot}}).}
}
\value{
  A list with one component per term in the model object \code{obj},
  each component is a 4-column matrix with $x$ as the first column, and
  3 columns with estimae and lower and upper confidence limit.
}
\author{
  Bendix Cartensen
}
\seealso{
 \code{\link{Ns}}, \code{termplot}
}
\examples{
# Get the diabetes data and set up as Lexis object
data(DMlate)
DMlate <- DMlate[sample(1:nrow(DMlate),500),]
dml <- Lexis( entry = list(Per=dodm, Age=dodm-dobth, DMdur=0 ),
               exit = list(Per=dox),
        exit.status = factor(!is.na(dodth),labels=c("DM","Dead")),
               data = DMlate )

# Split in 1-year age intervals
dms <- splitLexis( dml, time.scale="Age", breaks=0:100 )

# Model with 6 knots for both age and period
n.kn <- 6
# Model age-specific rates with period referenced to 2004
( a.kn <- with( subset(dms,lex.Xst=="Dead"),
                quantile( Age+lex.dur, probs=(1:n.kn-0.5)/n.kn ) ) )
( p.kn <- with( subset(dms,lex.Xst=="Dead"),
                quantile( Per+lex.dur, probs=(1:n.kn-0.5)/n.kn ) ) )
m2 <- glm( lex.Xst=="Dead" ~ -1 +
                             Ns( Age, kn=a.kn, intercept=TRUE ) +
                             Ns( Per, kn=p.kn, ref=2004 ),
           offset = log( lex.dur ), family=poisson, data=dms )

# Finally we can plot the two effects:
Termplot( m2, yshr=0.9 )
}
\keyword{hplot}

\name{fit.add}
\alias{fit.add}
\title{
  Fit an additive excess risk model to interval censored data.
  }
\description{
  Utility function.
    
  The model fitted assumes a piecewise constant intensity for the
  baseline, and that the covariates act additively on the rate scale.
  }
\usage{
  fit.add( y, rates.frame, cov.frame, start )
  } 
\arguments{
  \item{y}{Binary vector of outcomes}
  \item{rates.frame}{Dataframe expanded from the original data by
    \code{\link{expand.data}}, cooresponding to covariates for the rate
    parameters.}
  \item{cov.frame}{ do., but covariates corresponding to the
    \code{formula} argument of \code{\link{Icens}}} 
  \item{start}{Starting values for the rate parameters. If not supplied,
       then starting values are generated.}
  }
\value{
  A list with one component:
  \item{rates}{A glm object from a binomial model with log-link function.}
}
\references{
  B Carstensen: Regression models for interval censored
  survival data: application to HIV infection in Danish homosexual
  men. Statistics in Medicine, 15(20):2177-2189, 1996.

  CP Farrington: Interval censored survival data: a generalized linear
  modelling approach. Statistics in Medicine, 15(3):283-292, 1996.
  }
\author{
  Martyn Plummer, \email{martyn.plummer@r-project.org}
  }
\seealso{
  \code{\link{Icens}}
  \code{\link{fit.mult}}
  }
\examples{
  data( HIV.dk ) 
  }
\keyword{ models }
\keyword{ regression }
\keyword{ survival }

\name{births}
\alias{births}
\docType{data}
\title{Births in a London Hospital}
\description{
Data from 500 singleton births in a London Hospital
}
\usage{data(births)}
\format{
  A data frame with 500 observations on the following 8 variables.
  \tabular{rl}{
    \code{id}: \tab Identity number for mother and baby. \cr
    \code{bweight}: \tab Birth weight of baby. \cr
    \code{lowbw}: \tab Indicator for birth weight less than 2500 g. \cr
    \code{gestwks}: \tab Gestation period. \cr
    \code{preterm}: \tab Indicator for gestation period less than 37 weeks. \cr
    \code{matage}: \tab Maternal age. \cr
    \code{hyp}: \tab Indicator for maternal hypertension. \cr
    \code{sex}: \tab Sex of baby: 1:Male, 2:Female. \cr
  }
}
\source{
Anonymous
}
\references{
Michael Hills and Bianca De Stavola (2002). A Short Introduction to
Stata 8 for Biostatistics, Timberlake Consultants Ltd
}
\examples{
data(births)
}
\keyword{datasets}

\name{bootLexis}
\alias{nid}
\alias{nid.Lexis}
\alias{bootLexis}
\title{
Create a bootstrap sample of persons (as identified by \code{lex.id}) from a Lexis object 
}
\description{
\code{lex.id} is the person identifier in a \code{\link{Lexis}}
object. This is used to sample persons from a Lexis object. If a person
is sampled, all records from this persons is transported to the
bootstrap sample. 
}
\usage{
nid( Lx, \dots )
\method{nid}{Lexis}( Lx, by=NULL, \dots )
bootLexis( Lx, size = NULL, by = NULL, replace=TRUE )
}
\arguments{
  \item{Lx}{A \code{Lexis} object.}
  \item{\dots}{Parameters passed on to other methods.}
  \item{size}{Numeric. How many persons should be sampled from the
    \code{Lexis} object. Defaults to the number of persons in the
    \code{Lx}, or, if \code{by} is given, to the number of persons in
    each level of \code{by}. If \code{by} is given, \code{size} can have length
    \code{length(unique(by))}, to indicate how many are sampled from
    each level of \code{by}.}  
  \item{by}{Character. Name of a variable (converted to factor) in the
    \code{Lexis} object.

    Bootstrap sampling is done within each level of by.

    Calculation of the number of persons (\code{lex.id}) is done within
    each level of \code{by}, and a vector returned.
  }
  \item{replace}{Should persons be sampled by replacement? Default is
    \code{TRUE}. Setting \code{replace} to \code{FALSE} enables
    selecting a random subset of persons from the Lexis object.}
}
\value{  
  \code{bootLexis} returns a Lexis object of the same structure as the
    input, with \emph{persons} bootstrapped. The variable \code{lex.id}
    in the resulting \code{Lexis} object has values 1,2,... The original
    values of \code{lex.id} from \code{Lx} are stored in the variable
    \code{old.id}.

  \code{nid} counts the number of persons in a Lexis object, possibly by
  \code{by}. If \code{by} is given, a named vector is returned.
}
\author{Bendix Carstensen, \url{http://bendixcarstensen.com}.}
\seealso{\code{\link{Relevel.Lexis}},\code{\link{subset.Lexis}}}
\examples{
# A small bogus cohort
xcoh <- data.frame( id = c("A", "B", "C"),
                 birth = c("1952-07-14", "1954-04-01", "1987-06-10"),
                 entry = c("1965-08-04", "1972-09-08", "1991-12-23"),
                  exit = c("1997-06-27", "1995-05-23", "1998-07-24"),
                  fail = c(1, 0, 1),
                   sex = c("M","F","M") )

# Convert to calendar years
for( i in 2:4 ) xcoh[,i] <- cal.yr(xcoh[,i])
xcoh <- xcoh[sample(1:3, 10, replace = TRUE),]
xcoh$entry <- xcoh$entry + runif(10, 0, 10)
xcoh$exit  <- xcoh$entry + runif(10, 0, 10)

Lcoh <- Lexis(entry = list(per = entry),
               exit = list(per = exit,
                           age = exit - birth),
        exit.status = fail,
               data = xcoh)
Lcoh

Lx <- splitLexis(Lcoh, breaks = 0:10 * 10, "age")
Lx
nid(Lx)
nid(Lx, by="sex")
Lb <- bootLexis(Lx)
head(Lb)
nid(bootLexis(Lx, size = 7))
Li <- bootLexis(Lx, by = "id") # superfluous
summary(Lx)
summary(Li)
L2 <- bootLexis(Lx, by = "sex", size = c(2, 5))
nid(L2, by = "sex")
summary(L2, by = "sex")
}
\keyword{manip}
